//=============================================================================
// RPG Maker MZ - 武器精通、护甲精通
//=============================================================================

/*:
 * @target MZ
 * @plugindesc v1.1.0 飞猫云工作室-<武器精通、护甲精通>
 * @author 飞猫云工作室（Fly_Cat/Fly_Roc）
 * 
 * @help
 * ==============================插件帮助==================================
 * 本插件功能是如果佩戴了精通武器或者护甲，则赋予角色一个状态来达到能力的提升。
 * 注意：本插件仅支持一个武器槽，但是支持单个角色多种武器精通。
 * 注意：本插件仅支持单个角色一个护甲类精通。
 * ==========================武器精通：备注命令==================================
 * 角色栏备注：<武器精通:2,19>  2为系统设置里武器类型编号 19是状态Id
 * 角色栏备注：<武器精通:2,19,3,20>  一个角色设置多个武器精通 
 * 2为系统设置里武器类型编号 19是状态Id 
 * 3为系统设置里武器类型编号 20是状态Id
 * ==========================护甲精通：备注命令==================================
 * 角色栏备注：<护甲精通:2,19,4>  
 * 1. 2为系统设置里护甲类型编号 
 * 2. 19是最大数量获得的状态Id  
 * 3. 4为佩戴此类装备的最大数量
 * 4. 满足佩戴数量才能激活精通效果
 * ==============================使用条款==================================
 * 1.定制MZ、MV插件请联系（QQ:903516931）
 * ==============================更新日志==================================
 * 2020.12.22 <1>修复升级后恢复所有状态引起的BUG
 */
'use strict';
var Imported = Imported || {};
Imported.FlyCat_WeaponArmorMastery = true;

var FlyCat = FlyCat || {};
FlyCat.WeaponArmorMastery = {};
FlyCat.WeaponArmorMastery.parameters = PluginManager.parameters('FlyCat_WeaponArmorMastery');


/////////////////////Window//////////////////////
FlyCat.WeaponArmorMastery.Window_EquipStatus_drawAllParams = Window_EquipStatus.prototype.drawAllParams;
Window_EquipStatus.prototype.drawAllParams = function () {
    FlyCat.WeaponArmorMastery.Window_EquipStatus_drawAllParams.call(this)
    var x = this.itemPadding();
    var y = this.paramY(6);
    if (this._actor._equipWeaponMastery && this._actor._equipWeaponMastery == true) {
        const name = this._actor._weaponMastery.name;
        // const iconIndex = this._actor._weaponMastery.iconIndex;
        this.drawTextEx('\\C[14][已装备精通武器]\\C[10]<' + name + '>\\C[0]', x, y, this.width, 'left')
    }
    else {
        this.drawTextEx('\\C[8][未装备精通武器]\\C[0]', x, y, this.width, 'left')
    }
    var y = this.paramY(7);
    if (this._actor._equipArmorMastery && this._actor._equipArmorMastery == true) {
        // const name = this._actor._armorMastery.name;
        // this.drawTextEx('\\C[14][已装备精通护甲]\\C[10]<' + name + '>\\C[0]', x, y, this.width, 'left')
        this.drawTextEx('\\C[14][已装备精通护甲]\\C[0]', x, y, this.width, 'left')
    }
    else {
        this.drawTextEx('\\C[8][未装备精通护甲]\\C[0]', x, y, this.width, 'left')
    }
};
Window_EquipStatus.prototype.resetFontSettings = function () {
    this.contents.fontFace = $gameSystem.mainFontFace();
    this.contents.fontSize = 18;
    this.resetTextColor();
};
///////////////////////////Game_Actor///////////////////////////
FlyCat.WeaponArmorMastery.Game_Actor_changeEquip = Game_Actor.prototype.changeEquip;
Game_Actor.prototype.changeEquip = function (slotId, item) {
    FlyCat.WeaponArmorMastery.Game_Actor_changeEquip.call(this, slotId, item)
    this.addWeaponMastery();
    this.addArmorMastery();
};
FlyCat.WeaponArmorMastery.Game_Actor_clearStates = Game_Actor.prototype.clearStates;
Game_Actor.prototype.clearStates = function () {
    FlyCat.WeaponArmorMastery.Game_Actor_clearStates.call(this)
    if (!$gameParty.inBattle()) {
        this.addWeaponMastery();
        this.addArmorMastery();
    }
};
FlyCat.WeaponArmorMastery.Scene_Battle_terminate = Scene_Battle.prototype.terminate;
Scene_Battle.prototype.terminate = function () {
    FlyCat.WeaponArmorMastery.Scene_Battle_terminate.call(this);
    Game_Actor.prototype.addWeaponMastery();
    Game_Actor.prototype.addArmorMastery();
};
Game_Actor.prototype.addWeaponMastery = function () {
    const actor = this.actor()
    if (actor && actor.meta.武器精通) {
        const metaList = actor.meta.武器精通.split(',');
        const equipList = this.equips();
        for (let i = 0; i < equipList.length; i++) {
            if (equipList[i] && equipList[i].wtypeId) {
                for (let w = 0; w < metaList.length; w += 2) {
                    if (equipList[i].wtypeId == metaList[w]) {
                        this.addState(Number(metaList[w + 1]));
                        this._weaponIndex = i;
                        this._equipWeaponMastery = true;
                        this._weaponMastery = equipList[i];
                    }
                    else {
                        this.removeState(Number(metaList[w + 1]))
                        this._weaponIndex = null;
                        this._equipWeaponMastery = false;
                        this._weaponMastery = null;
                    }
                }
            }
            else if (equipList[this._weaponIndex] == null) {
                this.removeState(Number(metaList[i + 1]))
                this._equipWeaponMastery = false;
                this._weaponMastery = null;
            }
        }
    }
}
Game_Actor.prototype.addArmorMastery = function () {
    this._armorNumber = 0;
    const actor = this.actor()
    if (actor && actor.meta.护甲精通) {
        const metaList = actor.meta.护甲精通.split(',');
        const equipList = this.equips();
        for (let i = 0; i < equipList.length; i++) {
            if (equipList[i] && equipList[i].atypeId) {
                if (equipList[i].atypeId == metaList[0]) {
                    this._armorNumber++;
                    if (this._armorNumber >= metaList[2]) {
                        this.addState(Number(metaList[1]));
                        this._armorIndex = i;
                        this._equipArmorMastery = true;
                        this._armorMastery = equipList[i];
                    }
                    else {
                        this.removeState(Number(metaList[1]))
                        this._armorIndex = null;
                        this._equipArmorMastery = false;
                        this._armorMastery = null;
                    }
                }
                // else {
                //     this.removeState(Number(metaList[1]))
                //     this._armorIndex = null;
                //     this._equipArmorMastery = false;
                //     this._armorMastery = null;

                // }
            }
            else if (equipList[this._armorIndex] == null) {
                this.removeState(Number(metaList[1]))
                this._equipArmorMastery = false;
                this._armorMastery = null;
            }
        }
    }
}