//================================================================================
// Grover Plugins - Online Leaderboard
// Grover_OnlineLeaderboard.js
//================================================================================

//================================================================================
/*:
 * @plugindesc v1.0.5 排行榜插件,需要自定义排行榜请关闭此插件
 * @author Grover
 *
 * @param 背景设置
 *
 * @param backgroundType
 * @parent 背景设置
 * @default 1
 * @type select
 * @option 渐变色背景
 * @value 1
 * @option 图片背景
 * @value 2
 * @text 背景图类型
 * @desc 排行榜背景图类型
 *
 * @param gradient
 * @parent 背景设置
 * @text 渐变色背景
 * @desc 使用渐变色作为排行榜背景图
 *
 * @param left
 * @parent gradient
 * @text 渐变色起点颜色
 * @type string
 * @default #bdc3c7
 * @desc 请填写 CSS 颜色,透明可填 transparent
 *
 * @param right
 * @parent gradient
 * @text 渐变色终点颜色
 * @type string
 * @default #2c3e50
 * @desc 请填写 CSS 颜色,透明可填 transparent
 *
 * @param background
 * @parent 背景设置
 * @text 图片背景
 * @desc 使用指定图片作为排行榜背景图
 *
 * @param background1
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 1号排行榜图片背景
 * @desc 使用图片作为1号排行榜背景图
 *
 * @param background2
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 2号排行榜图片背景
 * @desc 使用图片作为2号排行榜背景图
 *
 * @param background3
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 3号排行榜图片背景
 * @desc 使用图片作为3号排行榜背景图
 *
 * @param background4
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 4号排行榜图片背景
 * @desc 使用图片作为4号排行榜背景图
 *
 * @param background5
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 5号排行榜图片背景
 * @desc 使用图片作为5号排行榜背景图
 *
 * @param background6
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 6号排行榜图片背景
 * @desc 使用图片作为6号排行榜背景图
 *
 * @param background7
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 7号排行榜图片背景
 * @desc 使用图片作为7号排行榜背景图
 *
 * @param background8
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 8号排行榜图片背景
 * @desc 使用图片作为8号排行榜背景图
 *
 * @param background9
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 9号排行榜图片背景
 * @desc 使用图片作为9号排行榜背景图
 *
 * @param background10
 * @parent background
 * @require 1
 * @default
 * @dir img/pictures/
 * @type file
 * @text 10号排行榜图片背景
 * @desc 使用图片作为10号排行榜背景图
 *
 * @param 窗口皮肤设置
 * @parent 背景设置
 *
 * @param backSpriteVisible
 * @parent 窗口皮肤设置
 * @type boolean
 * @on 显示
 * @off 不显示
 * @default false
 * @text 窗口皮肤背景显示
 * @desc 显示窗口皮肤自带背景的开关
 *
 * @param frameSpriteVisible
 * @parent 窗口皮肤设置
 * @type boolean
 * @on 显示
 * @off 不显示
 * @default false
 * @text 窗口皮肤边框显示
 * @desc 显示窗口皮肤自带边框的开关
 *
 * @param 排行榜设置
 *
 * @param leagueTables
 * @parent 排行榜设置
 * @type text[]
 * @default ["战力榜","财富榜"]
 * @text 排行榜名称
 * @desc 设置各排行榜的名称
 *
 * @help
 * 排行榜插件
 *
 * 更新排行榜:事件中用脚本指令(非插件指令)运行下列参考代码即可
 * // 更新1号排行榜数据:以5号变量的值作为1号排行榜的降序排序依据,5号变量的值由你来决定
 * HWGOE.game.updateRankData(1, $gameVariables.value(5));
 *
 * // 更新2号排行榜数据:以游戏金币的值作为2号排行榜的降序排序依据,没错支持到5号
 * HWGOE.game.updateRankData(2, $gameParty.gold());
 *
 * // 假如是更新战力榜,参考如下(示例采用1号榜作为战力榜)
 * let power = 0;
 * for (let actor of $gameParty.battleMembers()) {
 *     // 最大生命值 最大魔力值 攻击力 魔法攻击力 防御力 魔法防御力 敏捷 幸运 命中率 暴击率
 *     // 暴击回避率 魔法回避率 魔法反射率 反击率 生命值恢复率 魔法恢复率 TP补充率 防御效果 恢复效果 与其对应的战斗力点数
 *     let param = {"mhp" : 1, "mmp": 2, "atk": 12, "mat": 10, "def" : 40, "mdf": 35, "agi":6, "luk":6, "hit":400, "cri":600,
 *                 "cev":400, "mev":600, "mrf":600, "cnt":600, "hrg":180, "mrg":360, "tgr":500, "grd":300, "rec":400 };
 *     for (let p in param) { power += Math.floor(actor[p] * param[p]) }
 * }
 * HWGOE.game.updateRankData(1, power);
 *
 * 打开排行榜:事件中用脚本指令(非插件指令)运行下列参考代码即可
 * // 打开排行榜
 * SceneManager.push(Scene_Leaderboard);
 *
 * // 查询自己当前1号榜今日当前排名(配合服务器时间可以做日常奖励)
 * // 返回结果 0 未上榜或数据未准备好 其它数字为当前排名
 * Leaderboard.dailyRanking(1);
 *
 * // 查询自己当前1号榜总榜排名(配合服务器时间可以做日常奖励)
 * // 返回结果 0 未上榜或数据未准备好 其它数字为当前排名
 * Leaderboard.totalRanking(1);
 *
 * // 查询自己1号排行榜的数据
 * // 返回结果 自己对应的排行榜数据
 * Leaderboard.player(1);
 *
 * // 查询自己1号排行榜,第3名的数据
 * // 返回结果 1号排行榜第3名的排行榜数据
 * Leaderboard.playerData(1, 3);
 *
 * 注意:
 * 排行榜最多支持10种类型,即1~10号排行榜
 * 为避免你自定义的排行榜插件被更新覆盖
 * 比如你改名插件叫Grover_OnlineLeaderboard_QingYi.js
 * 对应的插件代码里的
 * PluginManager.parameters("Grover_OnlineLeaderboard");
 * 也要变成
 * PluginManager.parameters("Grover_OnlineLeaderboard_QingYi");
 *
 * 更新日志:
 * v1.0.5
 * 增加获取排行榜数据函数,排行榜支持10种
 * v1.0.4
 * 优化体验,增加窗口皮肤显示开关
 * v1.0.3
 * 增加可复写接口,小部分函数名改变,有意义的一次变化,对于自定义拓展更方便了
 * v1.0.2
 * 重新设计界面,支持查看多个榜单,支持切换今日排名或历史排名
 * v1.0.1
 * 增加获取排行榜排名函数
 * v1.0.0:
 * 完成插件.
 */

"use strict";
function Scene_Leaderboard() {
  this.initialize.apply(this, arguments);
}

function Window_LeaderboardType() {
  this.initialize.apply(this, arguments);
}

function Window_Leaderboards() {
  this.initialize.apply(this, arguments);
}

var Leaderboard = Leaderboard || {};

(function () {
  const {
    backgroundType,
    left: gradientLeft,
    right: gradientRight,
    background1: backgroundFile1,
    background2: backgroundFile2,
    background3: backgroundFile3,
    background4: backgroundFile4,
    background5: backgroundFile5,
    background6: backgroundFile6,
    background7: backgroundFile7,
    background8: backgroundFile8,
    background9: backgroundFile9,
    background10: backgroundFile10,
    backSpriteVisible,
    frameSpriteVisible,
    leagueTables
  } = PluginManager.parameters("Grover_OnlineLeaderboard");

  const BackgroundFile = {
    1: backgroundFile1,
    2: backgroundFile2,
    3: backgroundFile3,
    4: backgroundFile4,
    5: backgroundFile5,
    6: backgroundFile6,
    7: backgroundFile7,
    8: backgroundFile8,
    9: backgroundFile9,
    10: backgroundFile10
  };

  const BackSpriteVisible = backSpriteVisible === "true";
  const FrameSpriteVisible = frameSpriteVisible === "true";

  const LeagueTables = JSON.parse(leagueTables).filter((t) => !!t);

  const MarginTop = 72;

  // 重写接收排行榜数据处理
  HWGOE.override.game.rankList = function (data) {
    $gameTemp._leaderboard[data.type] = data.result;
  };

  const _Game_Temp_initialize = Game_Temp.prototype.initialize;
  Game_Temp.prototype.initialize = function () {
    _Game_Temp_initialize.apply(this, arguments);
    this._leaderboard = {
      1: [],
      2: [],
      3: [],
      4: [],
      5: [],
      6: [],
      7: [],
      8: [],
      9: [],
      10: []
    };
  };

  Leaderboard.totalRanking = function (type) {
    type = Math.min(type || 1, 10);
    HWGOE.game.getRankList(type);
    let index = $gameTemp._leaderboard[type].findIndex(function (player) {
      return player.userId === HWGOE.player.id();
    });
    index += 1;
    return index <= 0 ? 0 : index;
  };

  Leaderboard.dailyRanking = function (type) {
    type = Math.min(type || 1, 10);
    HWGOE.game.getRankList(type);
    let timestamp = HWGOE.server.timestamp();
    let today = new Date(new Date(timestamp).toDateString()).getTime();
    let leaderboard = $gameTemp._leaderboard[type].filter(function (player) {
      return player.updateTime >= today;
    });
    let index = leaderboard.findIndex(function (player) {
      return player.userId === HWGOE.player.id();
    });
    index += 1;
    return index <= 0 ? 0 : index;
  };

  Leaderboard.player = function (type) {
    type = Math.min(type || 1, 10);
    let player = $gameTemp._leaderboard[type].find(function (player) {
      return player.userId === HWGOE.player.id();
    });
    return player;
  };

  Leaderboard.playerData = function (type, index) {
    type = Math.min(type || 1, 10);
    index = Math.max((index || 0) - 1, 0);
    return $gameTemp._leaderboard[type][index];
  };

  function isMV() {
    return Utils.RPGMAKER_NAME === "MV";
  }

  function isMZ() {
    return Utils.RPGMAKER_NAME === "MZ";
  }
  // ==========================================================================
  // 排行榜界面部分
  // ==========================================================================

  Scene_Leaderboard.prototype = Object.create(Scene_MenuBase.prototype);
  Scene_Leaderboard.prototype.constructor = Scene_Leaderboard;

  Scene_Leaderboard.prototype.initialize = function () {
    Scene_MenuBase.prototype.initialize.call(this);
    this._type = 1;
    HWGOE.game.getRankList(this._type);
    this._leaderboard = $gameTemp._leaderboard[1];
  };

  Scene_Leaderboard.prototype.create = function () {
    Scene_MenuBase.prototype.create.call(this);
    this.createTypeWindow();
    this.createLeaderboardsWindow();
    this.createStatusWindow();
  };

  Scene_Leaderboard.prototype.createBackground = function () {
    this._backSprite = new Sprite();
    if (backgroundType === "1") {
      let { width, height } = Graphics;
      let glc = gradientLeft;
      let grc = gradientRight;
      this._backSprite.bitmap = new Bitmap(width, height);
      this._backSprite.bitmap.gradientFillRect(0, 0, width, height, glc, grc);
    } else {
      let filename = BackgroundFile[this._type];
      this._backSprite.bitmap = ImageManager.loadPicture(filename);
    }
    this.addChild(this._backSprite);
  };

  Scene_Leaderboard.prototype.createTypeWindow = function () {
    this._lbTypeWindow = new Window_LeaderboardType(
      0,
      MarginTop,
      this._type,
      this._leaderboard
    );
    this._lbTypeWindow.setHandler("ok", this.onTypeOk.bind(this));
    this._lbTypeWindow.setHandler("cancel", this.popScene.bind(this));
    this._backSprite.addChild(this._lbTypeWindow);
  };

  Scene_Leaderboard.prototype.createLeaderboardsWindow = function () {
    var ww = this._lbTypeWindow.width;
    var wh = this._lbTypeWindow.y;
    this._lbWindow = new Window_Leaderboards(
      ww,
      wh,
      Graphics.width / 2,
      this._type,
      this._leaderboard
    );
    this._lbWindow.setHandler("ok", this.onSelectPlayer.bind(this));
    this._lbWindow.setHandler("cancel", this.onLeaderboardsCancel.bind(this));
    this._lbTypeWindow.setLeaderboardsWindow(this._lbWindow);
    this._backSprite.addChild(this._lbWindow);
  };

  Scene_Leaderboard.prototype.createStatusWindow = function () {
    var wx = this._lbWindow.x + this._lbWindow.width;
    var wy = this._lbTypeWindow.y;
    var ww = Graphics.width - wx;
    var wh = this._lbWindow.height;
    if (isMZ()) {
      let rect = new Rectangle(wx, wy, ww, wh);
      this._statusWindow = new Window_Base(rect);
      this._statusWindow._backSprite.visible = BackSpriteVisible;
      this._statusWindow._frameSprite.visible = FrameSpriteVisible;
      !FrameSpriteVisible && this._statusWindow._frameSprite.removeChildren();
    } else {
      this._statusWindow = new Window_Base(wx, wy, ww, wh);
      this._statusWindow._windowBackSprite.visible = BackSpriteVisible;
      this._statusWindow._windowFrameSprite.visible = FrameSpriteVisible;
    }
    this._backSprite.addChild(this._statusWindow);
  };

  Scene_Leaderboard.prototype.onTypeOk = function () {
    this._lbWindow.activate();
    this._lbWindow.select(0);
    this.onTypeOkFlow();
  };

  // 当排行榜种类按下确认时
  Scene_Leaderboard.prototype.onTypeOkFlow = function () {};

  Scene_Leaderboard.prototype.onSelectPlayer = function () {
    // 这里暂时留着后续拓展 发起组队 加好友等功能, 如果没有需求请勿重写此函数
    this._lbWindow.activate();
  };

  Scene_Leaderboard.prototype.onLeaderboardsCancel = function () {
    this._lbTypeWindow.activate();
    this._lbWindow.deselect();
    this._lbWindow.activate();
    this._lbWindow.fixLastIndex();
    this.onLeaderboardsCancelFlow();
  };

  // 从排行榜玩家列表窗口返回上一级时
  Scene_Leaderboard.prototype.onLeaderboardsCancelFlow = function () {
    this._statusWindow.contents.clear();
  };

  Scene_Leaderboard.prototype.update = function () {
    Scene_MenuBase.prototype.update.call(this);
    this.updateTypeChange();
    this.updateLBSelectChange();
  };

  Scene_Leaderboard.prototype.updateTypeChange = function () {
    if (this._lbTypeWindow && this._lbTypeWindow.active) {
      if (this._type !== this._lbTypeWindow.type()) {
        HWGOE.game.getRankList(this._lbTypeWindow.type());
        this.updateBackGround();
        this.updateTypeChangeFlow();
      }
      this._type = this._lbTypeWindow.type();
    }
  };

  // 当排行榜类型窗口的种类发生变化时
  Scene_Leaderboard.prototype.updateTypeChangeFlow = function () {};

  Scene_Leaderboard.prototype.updateBackGround = function () {
    if (backgroundType === "2") {
      let filename = BackgroundFile[this._lbTypeWindow.type()];
      this._backSprite.bitmap = ImageManager.loadPicture(filename);
    }
  };

  Scene_Leaderboard.prototype.updateLBSelectChange = function () {
    if (this._lbWindow && this._lbWindow.active) {
      if (this._lbWindow._lastIndex !== this._lbWindow.index()) {
        this.updateLBSelectChangeFlow();
      }
      this._lbWindow.fixLastIndex();
    }
  };

  // 当排行榜玩家列表窗口光标发生变化时
  Scene_Leaderboard.prototype.updateLBSelectChangeFlow = function () {
    // 处理你想描绘的内容,为避免你修改后的插件内容被覆盖,请新建一个插件于排行榜插件下方,
    // 并重写本函数,当然其他函数也支持重写
    // this._type为当前排行榜种类
    this._statusWindow.contents.clear();
    let player = this._lbWindow.currentPlayer();
    if (!player) return;
    let { username } = player;
    let text = "\\c[10]" + username + "\\}\\c[0] 角色信息未公开";
    this._statusWindow.drawTextEx(text, 4, 0);
  };

  // ==========================================================================
  // 排行榜'周榜'和'总榜'选择窗口部分
  // ==========================================================================
  Window_LeaderboardType.prototype = Object.create(Window_Selectable.prototype);
  Window_LeaderboardType.prototype.constructor = Window_LeaderboardType;

  Window_LeaderboardType.lastTopRow = 0;
  Window_LeaderboardType.lastIndex = 0;

  Window_LeaderboardType.prototype.initialize = function (
    x,
    y,
    type,
    leaderboard
  ) {
    var width = this.windowWidth();
    var height = this.windowHeight();
    if (isMZ()) {
      let rect = new Rectangle(x, y, width, height);
      Window_Selectable.prototype.initialize.call(this, rect);
    } else {
      Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    }
    this._type = type;
    this._leaderboard = leaderboard;
    this.createPageupSprite();
    this.createPagedownSprite();
    this.createModeSprite();
    this.refresh();
    this.setTopRow(Window_LeaderboardType.lastTopRow);
    this.select(Window_LeaderboardType.lastIndex);
    this.activate();
  };

  Window_LeaderboardType.prototype.spritePosition = function (index) {
    index = 3 - (index || 0);
    let offset = isMZ() ? this.colSpacing() / 2 : 0;
    let x = this.padding + offset;
    let y = isMZ()
      ? this.itemPadding() + $gameSystem.windowPadding() / 2
      : this.padding;
    let padding = this.padding || 0;
    let contentsHeight = Graphics.height - MarginTop - padding;
    let row = Math.floor(contentsHeight / this.itemHeight()) - index;
    y += this.itemHeight() * row;
    return { x, y };
  };

  Window_LeaderboardType.prototype.initializeSprite = function (
    spriteName,
    handler
  ) {
    let colSpacing = isMZ() ? this.colSpacing() : 0;
    let width = this.itemWidth() - colSpacing;
    if (isMZ()) {
      this[spriteName] = new Sprite_Clickable();
      this[spriteName].onClick = handler.bind(this);
    } else {
      this[spriteName] = new Sprite_Button();
      this[spriteName].setClickHandler(handler.bind(this));
    }
    this[spriteName].bitmap = new Bitmap(width, this.itemHeight());
    this[spriteName].bitmap.fontBold = true;
  };

  Window_LeaderboardType.prototype.createPageupSprite = function () {
    this.initializeSprite("_pageupSprite", this.clickPageupHandler);
    let { x, y } = this.spritePosition();
    this._pageupSprite.move(x, y);
    this.drawPageup();
    this.addChild(this._pageupSprite);
  };

  Window_LeaderboardType.prototype.clickPageupHandler = function () {
    if (!this._lbWindow.active) return SoundManager.playBuzzer();
    this._lbWindow.cursorPageup();
    if (this._lbWindow.index() !== this._lbWindow._lastIndex) {
      SoundManager.playCursor();
    } else {
      SoundManager.playBuzzer();
    }
  };

  Window_LeaderboardType.prototype.drawPageup = function () {
    let { width, height } = this._pageupSprite;
    this._pageupSprite.bitmap.clear();
    this._pageupSprite.bitmap.drawText("上一页", 0, 0, width, height, "center");
  };

  Window_LeaderboardType.prototype.createPagedownSprite = function () {
    this.initializeSprite("_pagedownSprite", this.clickPagedownHandler);
    let { x, y } = this.spritePosition(1);
    this._pagedownSprite.move(x, y);
    this.drawPagedown();
    this.addChild(this._pagedownSprite);
  };

  Window_LeaderboardType.prototype.clickPagedownHandler = function () {
    if (!this._lbWindow.active) return SoundManager.playBuzzer();
    this._lbWindow.cursorPagedown();
    if (this._lbWindow.index() !== this._lbWindow._lastIndex) {
      SoundManager.playCursor();
    } else {
      SoundManager.playBuzzer();
    }
  };

  Window_LeaderboardType.prototype.drawPagedown = function () {
    let { width, height } = this._pagedownSprite;
    this._pagedownSprite.bitmap.clear();
    this._pagedownSprite.bitmap.drawText(
      "下一页",
      0,
      0,
      width,
      height,
      "center"
    );
  };

  Window_LeaderboardType.prototype.createModeSprite = function () {
    this.initializeSprite("_modeSprite", this.clickModeHandler);
    let { x, y } = this.spritePosition(2);
    this._modeSprite.move(x, y);
    this._modeSprite._mode = 1;
    this.drawMode();
    this.addChild(this._modeSprite);
  };

  Window_LeaderboardType.prototype.clickModeHandler = function () {
    this._modeSprite._mode++;
    SoundManager.playCursor();
    if (this._modeSprite._mode > 1) {
      this._modeSprite._mode = 0;
    }
    this.drawMode();
    this._lbWindow.setMode(this._modeSprite._mode);
    if (this._lbWindow.active && this._lbWindow.index() !== -1) {
      this._lbWindow.select(0);
    }
  };

  Window_LeaderboardType.prototype.drawMode = function () {
    let text = ["今日榜单", "总榜单"][this._modeSprite._mode];
    let { width, height } = this._modeSprite;
    this._modeSprite.bitmap.clear();
    let tips = ["▶", "◀"][this._modeSprite._mode];
    let align = ["right", "left"][this._modeSprite._mode];
    this._modeSprite.bitmap.drawText(tips, 0, 0, width, height, align);
    this._modeSprite.bitmap.drawText(text, 0, 0, width, height, "center");
  };

  Window_LeaderboardType.prototype.windowWidth = function () {
    return Math.round(Graphics.width / 6);
  };

  Window_LeaderboardType.prototype.windowHeight = function () {
    return Graphics.height - MarginTop;
  };

  Window_LeaderboardType.prototype.maxItems = function () {
    return Math.min(LeagueTables.length, 10);
  };

  Window_LeaderboardType.prototype.maxPageItems = function () {
    return this.maxPageRows() * this.maxCols();
  };

  Window_LeaderboardType.prototype.maxPageRows = function () {
    return Math.min(LeagueTables.length, 10);
  };

  Window_LeaderboardType.prototype.maxVisibleItems = function () {
    return this.maxPageRows() * this.maxCols();
  };

  Window_LeaderboardType.prototype.maxCols = function () {
    return 1;
  };

  Window_LeaderboardType.prototype._refreshBack = function () {
    if (BackSpriteVisible) {
      Window_Selectable.prototype._refreshBack.apply(this, arguments);
    }
  };

  Window_LeaderboardType.prototype._refreshFrame = function () {
    if (FrameSpriteVisible) {
      Window_Selectable.prototype._refreshFrame.apply(this, arguments);
    }
  };

  Window_LeaderboardType.prototype.update = function () {
    Window_Selectable.prototype.update.call(this);
    if (this._lbWindow) {
      this._lbWindow.setType(this.type());
    }
    this._type = this.type();
    if (this.active && $gameTemp._leaderboard) {
      let currentData = JSON.stringify(this._leaderboard);
      let newData = JSON.stringify($gameTemp._leaderboard[this._type] || []);
      if (currentData !== newData) {
        this._leaderboard = $gameTemp._leaderboard[this._type];
        this._lbWindow._leaderboard = this._leaderboard;
        this._lbWindow.refresh();
      }
    }
  };

  Window_LeaderboardType.prototype.type = function () {
    return this.index() + 1;
  };

  Window_LeaderboardType.prototype.drawItem = function (index) {
    let fun = isMZ() ? "itemLineRect" : "itemRectForText";
    var rect = this[fun](index);
    var text = LeagueTables[index];
    this.drawText(text, rect.x, rect.y, rect.width, "center");
  };

  Window_LeaderboardType.prototype.isCancelTriggered = function () {
    return Window_Selectable.prototype.isCancelTriggered();
  };

  Window_LeaderboardType.prototype.processCancel = function () {
    Window_Selectable.prototype.processCancel.call(this);
    Window_LeaderboardType.lastTopRow = this.topRow();
    Window_LeaderboardType.lastIndex = this.index();
  };

  Window_LeaderboardType.prototype.setLeaderboardsWindow = function (
    leaderboardsWindow
  ) {
    this._lbWindow = leaderboardsWindow;
  };

  // ==========================================================================
  // 排行榜玩家列表窗口部分
  // ==========================================================================
  Window_Leaderboards.prototype = Object.create(Window_Selectable.prototype);
  Window_Leaderboards.prototype.constructor = Window_Leaderboards;

  Window_Leaderboards.prototype.initialize = function (
    x,
    y,
    width,
    type,
    leaderboard
  ) {
    var height = this.windowHeight(); //fittingHeight(this.maxPageRows() + 1);
    this._mode = 1;
    this._data = [];
    this._type = type;
    this._lastIndex = -1;
    this._leaderboard = leaderboard;
    this._skinCursorSprite = new Bitmap(width, Graphics.boxHeight);
    if (isMZ()) {
      let rect = new Rectangle(x, y, width, height);
      Window_Selectable.prototype.initialize.call(this, rect);
      Object.defineProperty(this, "innerHeight", {
        get: function () {
          return Math.max(
            0,
            this._height - this._padding * 3 - this.itemHeight()
          );
        },
        configurable: true
      });
    } else {
      Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    }
    this.createMyRankSprite();
    this.refresh();
  };

  Window_Leaderboards.prototype.createMyRankSprite = function () {
    let colSpacing = isMZ() ? this.colSpacing() : 0;
    this._myRankSprite = new Sprite(
      new Bitmap(this.itemWidth() - colSpacing, this.itemHeight())
    );
    this.addChild(this._myRankSprite);
  };

  Window_Leaderboards.prototype.maxCols = function () {
    return 1;
  };

  Window_Leaderboards.prototype.maxItems = function () {
    if (!this._leaderboard.length) return 0;
    return this._data.length;
  };

  Window_Leaderboards.prototype.maxPageRows = function () {
    let padding = this.padding || 0;
    let contentsHeight = Graphics.height - MarginTop - padding;
    let maxPageRows = Math.floor(contentsHeight / this.itemHeight());
    return maxPageRows - 1;
  };

  Window_Leaderboards.prototype.maxPageItems = function () {
    return this.maxPageRows() * this.maxCols();
  };

  Window_Leaderboards.prototype.maxVisibleItems = function () {
    return this.maxPageRows() * this.maxCols();
  };

  Window_Leaderboards.prototype.windowHeight = function () {
    return Graphics.height - MarginTop;
  };

  Window_Leaderboards.prototype._refreshBack = function () {
    if (BackSpriteVisible) {
      Window_Selectable.prototype._refreshBack.apply(this, arguments);
    }
  };

  Window_Leaderboards.prototype._refreshFrame = function () {
    if (FrameSpriteVisible) {
      Window_Selectable.prototype._refreshFrame.apply(this, arguments);
    }
  };

  Window_Leaderboards.prototype.setData = function (data) {
    this._data = [];
    for (let userData of data) {
      this._data.push(userData);
    }
  };

  Window_Leaderboards.prototype.refresh = function () {
    this.refreshData();
    Window_Selectable.prototype.refresh.call(this);
    this.drawMyRank();
  };

  Window_Leaderboards.prototype.refreshData = function () {
    let data = this._leaderboard;
    if (!this._mode) {
      let timestamp = HWGOE.server.timestamp();
      let today = new Date(new Date(timestamp).toDateString()).getTime();
      data = this._leaderboard.filter(function (player) {
        return player.updateTime >= today;
      });
    }
    this.setData(data);
  };

  Window_Leaderboards.prototype.drawMyRank = function () {
    let offset = isMZ() ? this.colSpacing() / 2 : 0;
    let x = this.padding + offset;
    let y = isMZ()
      ? this.itemPadding() + $gameSystem.windowPadding() / 2
      : this.padding;
    this._myRankSprite.move(x, y + this.itemHeight() * this.maxPageRows());
    let bitmap = this._myRankSprite.bitmap;
    bitmap.clear();
    const { width, height } = bitmap;
    bitmap.gradientFillRect(0, 0, width, height, "#536976", "#292E49");
    bitmap.fontBold = true;
    const myData = this.myData();
    if (myData && myData.index !== 0) {
      let index = myData.index;
      bitmap.drawText("第 " + index + " 名", 0, 0, width, height, "center");
    } else {
      bitmap.drawText("未上榜", 0, 0, width, height, "center");
    }
  };

  Window_Leaderboards.prototype.drawItem = function (index) {
    var idText = String(index + 1);
    while (idText.length < 3) {
      idText = " " + idText;
    }
    var idWidth = this.textWidth(idText + "    ");
    var statusWidth = this.textWidth("-0000000000");
    var name = this.userName(index);
    var status = this.value(index);
    let fun = isMZ() ? "itemLineRect" : "itemRectForText";
    var rect = this[fun](index);
    this.resetTextColor();
    this.contents.fontBold = true;
    this.setTextColor(index, rect.x, idWidth, rect.height, true);
    this.drawText(idText, rect.x, rect.y, rect.width);
    rect.x += idWidth;
    rect.width -= idWidth + statusWidth;
    var nameWidth = this.textWidth(name);
    this.setTextColor(index, rect.x, nameWidth, rect.height);
    this.drawText(name, rect.x, rect.y, rect.width);
    this.resetTextColor();
    this.setTextColor(index, rect.x + rect.width, statusWidth, rect.height);
    this.drawText(status, rect.x + rect.width, rect.y, statusWidth, "right");
  };

  Window_Leaderboards.prototype.myData = function () {
    let data = this._data.find(function (player) {
      return player.userId === HWGOE.player.id();
    });
    if (!data) return;
    data.index = this._data.findIndex(function (player) {
      return player.userId === HWGOE.player.id();
    });
    data.index += 1;
    return data;
  };

  Window_Leaderboards.prototype.userName = function (index) {
    if (!this._data[index]) return "";
    return this._data[index].username;
  };

  Window_Leaderboards.prototype.setTextColor = function (
    index,
    x,
    w,
    h,
    vertical
  ) {
    let context = this.contents._context;
    const colorPool = function (i) {
      if (i < 5) {
        let color1 = ["#f12711", "#fe8c00", "#ee0979", "#7f00ff", "#076585"];
        let color2 = ["#f5af19", "#f83600", "#ff6a00", "#e100ff", "#ffffff"];
        var gradient;
        if (vertical) {
          gradient = context.createLinearGradient(0, 0, 0, h);
        } else {
          gradient = context.createLinearGradient(x, 0, x + w, 0);
        }
        gradient.addColorStop("0", color1[i]);
        gradient.addColorStop("1", color2[i]);
        return gradient;
      }
      return;
    };
    let textColor = colorPool(index);
    if (
      !textColor &&
      this._data[index] &&
      this._data[index].userId === HWGOE.player.id()
    ) {
      let gradient = context.createLinearGradient(x, 0, x + w, 0);
      gradient.addColorStop("0", "#FDFC47");
      gradient.addColorStop("1", "#24FE41");
      textColor = gradient;
    }
    if (textColor) this.contents.textColor = textColor;
  };

  if (isMV()) {
    Window_Leaderboards.prototype._refreshCursor = function () {
      var pad = this._padding;
      var x = this._cursorRect.x + pad - this.origin.x;
      var y = this._cursorRect.y + pad - this.origin.y;
      var w = this._cursorRect.width;
      var h = this._cursorRect.height;
      var x2 = Math.max(x, pad);
      var y2 = Math.max(y, pad);
      var w2 = Math.min(w, this._width - pad - x2);
      var h2 = Math.min(h, this._height - pad - y2);
      var bitmap = new Bitmap(w2, h2);

      this._windowCursorSprite.bitmap = bitmap;
      this._windowCursorSprite.setFrame(0, 0, w2, h2);
      this._windowCursorSprite.move(x2, y2);

      if (w > 0 && h > 0) {
        const width = this.width;
        const height = this.height;
        const colorPool = [
          { color1: "#f12711", color2: "#f5af19" },
          { color1: "#fe8c00", color2: "#f83600" },
          { color1: "#ee0979", color2: "#ff6a00" },
          { color1: "#7f00ff", color2: "#e100ff" },
          { color1: "#076585", color2: "#ffffff" },
          { color1: "rgba(6, 82, 121, 1)", color2: "rgba(158, 217, 0, 1)" }
        ];
        let index = this.index();
        let textColor = colorPool[index < 5 ? index : 5];
        this._skinCursorSprite.clear();
        this._skinCursorSprite.gradientFillRect(
          0,
          0,
          width,
          height,
          textColor.color1,
          textColor.color2
        );
        bitmap.blt(this._skinCursorSprite, 0, 0, width, height, 0, 0);
      }
    };
  }

  Window_Leaderboards.prototype.value = function (index) {
    if (!this._data[index]) return "";
    return this._data[index].value;
  };

  Window_Leaderboards.prototype.fixLastIndex = function () {
    this._lastIndex = this._index;
  };

  Window_Leaderboards.prototype.setType = function (type) {
    if (this._type !== type) {
      this._type = type;
      this.refresh();
    }
  };

  Window_Leaderboards.prototype.setMode = function (mode) {
    if (this._mode !== mode) {
      this._mode = mode;
      this.refresh();
    }
  };

  Window_Leaderboards.prototype.currentPlayer = function () {
    let player = this._data[this.index()];
    if (player) player.index = this.index() + 1;
    return player;
  };

  Window_Leaderboards.prototype.update = function () {
    Window_Selectable.prototype.update.call(this);
    if (this.active && $gameTemp._leaderboard) {
      let currentData = JSON.stringify(this._leaderboard);
      let newData = JSON.stringify($gameTemp._leaderboard[this._type] || []);
      if (currentData !== newData) {
        this._leaderboard = $gameTemp._leaderboard[this._type];
        this.refresh();
      }
    }
  };
})();
