//=============================================================================
//
//=============================================================================
/*:
 * @plugindesc 安卓端针对性优化[22-12-13]
 * @author Qiu Jiu
 *
 * @param tip1
 * @type boolean
 * @text 优化点1
 * @desc 优化点1
 * @default true
 *
 * @param tip2
 * @type boolean
 * @text 优化点2
 * @desc 优化点2
 * @default true
 *
 * @param tip3
 * @type boolean
 * @text 优化点3
 * @desc 优化点3
 * @default true
 *
 * @param tip4
 * @type boolean
 * @text 优化点4
 * @desc 优化点4
 * @default true
 *
 * @param tip5
 * @type boolean
 * @text 优化点5
 * @desc 优化点5
 * @default true
 *
 * @param tip6
 * @type boolean
 * @text 优化点6
 * @desc 优化点6
 * @default true
 *
 * @param tip7
 * @type boolean
 * @text 优化点7
 * @desc 优化点7
 * @default true
 *
 * @param tip8
 * @type boolean
 * @text 优化点8
 * @desc 优化点8
 * @default true
 *
 * @param tip9
 * @type boolean
 * @text 优化点9
 * @desc 优化点9
 * @default true
 *
 *
 * @help QJ_AndroidOptimize.js
 *
 * 前言：
 * 1.所有优化点均有开关，可根据实际情况决定是否启用某个优化点。
 * 2.此优化插件只适配于本工程，非通用插件。
 * 3.优化的插件不一定是其本身有问题，而是用于本工程时可能有稍微不足。
 * 4.此插件正常来说不会带来新的Bug，若的确有时可以先根据下面的描述来看一看，关闭对应
 *   的优化点。
 * 5.因此插件带来的Bug可免费修复。
 * 6.此插件需放在所有插件的最下面。
 *
 * 具体说明【对应不同的优化点】：
 * 1.XdRs_HangUp.js挂机插件。此插件会在离开地图时对地图进行截图，以在设置挂机移动范围
 *   时进行显示。但是在【此工程】中，所截的图没有正常显示，且完全不影响挂机设置中的效果，
 *   具体来说是在1挂机范围设置页面中显示当前地图。
 *   但因为此游戏画面分辨率很大，所以截图很耗时（即使在NWJS中也有15~25ms）故而此点将此截
 *   图功能屏蔽，可减少进入其他界面的时间。
 *   如果需要的话也可以直接修复这个功能，但这个功能实际上一直没有起效，作用也不大，直
 *   接优化掉更好。
 *   原生的截图也得耗这么久（素材太大加上readPixels截断），但因为有个进入战斗的动画，所
 *   以这个过程被缓冲了，没有那么明显。
 * 2.Yep插件修改了Bitmap.prototype.blt，但此函数中一次性读取8次全局函数Math，当工
 *   程中使用插件，特别是一些重写函数时直接将旧址记录在全局中的插件时，会造成单次读取
 *   的时间延长，对于需要多次调用进行绘制时影响累计，最后的影响很大。
 * 3.游戏分辨率很大，而滚动窗口创建时会创建很大的画布来绘制背景，此点将滚动窗口的大小
 *   设置为一半，可使游戏返回地图时减少多余消耗。这样若以后用到滚动窗口，窗口也不会再
 *   影响到屏幕下半部分的按键。
 * 4.Utils.isMobileDevice()函数优化。此函数本身不算什么，但在部分插件中多次调用，累计耗
 *   时较久，在NWJS环境中都有3.5~4.5ms。
 * 5.GT_ObjectInfoWindow.js在创建说明窗口后，会在正真显示的时候再修改。但最初初始化时
 *   创建的窗口和画面大小一样，而该工程游戏分辨率很大，这样就在移动端导致了卡顿。
 *   此点将说明窗口创建时的默认设置大小变小，这样就不会有这样的浪费，也不影响实际功能。
 * 6.LiuYue_EquipTime.js在创建倒计时列表时，直接创建一个与屏幕等大的窗口，然后在此
 *   窗口内添加精灵。但此窗口不是必要的，且还因为工程分辨率大而导致创建窗口时创建了很大的
 *   画布，即使将窗口的opacity设置为0，相关的数据也需创建。此点将此插件窗口调小，省去
 *   非必要的资源消耗。
 * 7.系统设置中有PIXI.tilemap.TileRenderer.DO_CLEAR = true;这条语句只在很少见的十年前
 *   的低机子中会有影响，取消掉这条指令大部分时候不影响原版程序运行也可减少不必要消耗。
 * 8.彻底去除进入战斗前的动画。
 * 9.即时清除地图和战斗场景数据，提高CG效率。
 *
 *
 *
 *
*/
//=============================================================================
//
//=============================================================================
var JWJMX = JWJMX || {};
JWJMX.AO = {reWrite:{}};
//=============================================================================
//
//=============================================================================
(($ = {})=>{
//=============================================================================
//
//=============================================================================
const pluginName = "JWJMX_AndroidOptimize";
const parameters = PluginManager.parameters(pluginName);
const aoList = [false];
for (let i=1;i<=9;i++) {
    aoList.push(eval(parameters["tip"+i]));
}
//=============================================================================
//1.XdRs_HangUp.js
//=============================================================================
if (aoList[1]&&XdRsData&&XdRsData.hangUp) {
//==================================================
XdRsData.hangUp.rangeBitmap = function() {
    //注释掉此行
    //if (!SceneManager.hangUpBitmap()) return null;
    var bitmap = new Bitmap(Graphics.boxWidth, Graphics.boxHeight);
    this.drawLattice(bitmap);
    return bitmap;
};
Window_HuRange.prototype.drawRange = function() {
    this.contents.clear();
    this._rangeData = $gameParty.hangUpData().range;
    var rw = this.contents.width, rh = this.contents.height;
    var rg = XdRsData.hangUp.rangeBitmap();
    this.contents.blt(rg,0,0,rg.width,rg.height,0,0,rw,rh);
};
SceneManager.snapForHangUp = function() {
    this._hangUpBitmap = null;
};
SceneManager.hangUpBitmap = function() {
    return this._hangUpBitmap;
};
//==================================================
}
//=============================================================================
//2.Yep插件修改了Bitmap.prototype.blt
//=============================================================================
if (aoList[2]&&Yanfly&&Yanfly.Core) {
Bitmap.prototype.blt = function(source, sx, sy, sw, sh, dx, dy, dw, dh) {
    dw = dw || sw;
    dh = dh || sh;
    let floor = Math.floor;
    sx = floor(sx);
    sy = floor(sy);
    sw = floor(sw);
    sh = floor(sh);
    dx = floor(dx);
    dy = floor(dy);
    dw = floor(dw);
    dh = floor(dh);
    //Yanfly.Core.Bitmap_blt.call(this, source, sx, sy, sw, sh, dx, dy, dw, dh);
    if (sx >= 0 && sy >= 0 && sw > 0 && sh > 0 && dw > 0 && dh > 0 &&
            sx + sw <= source.width && sy + sh <= source.height) {
        this._context.globalCompositeOperation = 'source-over';
        this._context.drawImage(source._canvas, sx, sy, sw, sh, dx, dy, dw, dh);
        this._setDirty();
    }
};
}
//=============================================================================
//3.滚动窗口
//=============================================================================
if (aoList[3]) {
Window_ScrollText.prototype.initialize = function() {
    var width = Graphics.boxWidth;
    var height = Graphics.boxHeight/2;
    Window_Base.prototype.initialize.call(this, 0, 0, width, height);
    this.opacity = 0;
    this.hide();
    this._text = '';
    this._allTextHeight = 0;
};
}
//=============================================================================
//4.Utils.isMobileDevice
//=============================================================================
if (aoList[4]) {
Utils.isMobileDevice_Old = Utils.isMobileDevice;
Utils.isMobileDevice = function() {
    let rem = Utils._remIsMobileDevice;
    if (rem!==undefined) {
        return rem;
    } else {
        rem = Utils.isMobileDevice_Old();
        Utils._remIsMobileDevice = rem;
        return rem;
    }
};
}
//=============================================================================
//5.GT_ObjectInfoWindow.js
//=============================================================================
if (aoList[5]&&Window_ObjectInfo) {
Window_ObjectInfo.prototype.initialize = function(windowSet) {
    this._windowSet = windowSet;
    this._item = null;
    this._actor = null;
    this._category = '';
    this._data = [];
    this._textTop = [];
    this._textBottom = [];
    this._damageData = [];
    this._costData = [];
    this._requireData = [];
    this._lastData = [];
    this._showInfo = GT.Param.OIWAutoShowWindow;
    //Window_Base.prototype.initialize.call(this, 0, 0, Graphics.boxWidth, Graphics.boxHeight);
    Window_Base.prototype.initialize.call(this, 0, 0, 100, 100);
    this.openness = 0;
    this.opacity = Number(this._windowSet.WindowOpacity);
};
}
//=============================================================================
//6.LiuYue_EquipTime.js
//=============================================================================
if (aoList[6]&&Window_ZzyETFShow) {
Window_ZzyETFShow.prototype.initialize = function() {
    var x = 0;
    var y = 0;
    //var width = Graphics.boxWidth;
    //var height = Graphics.boxHeight;
    Window_Base.prototype.initialize.call(this, x, y, 10, 10);
    this.opacity = 0;//透明度
    this.sprArr = [];//对象池
    this.sprIndex = [];//标记ID数组
    this.CreateTimeList();//创造队列
};
}
//=============================================================================
//PIXI.tilemap.TileRenderer.DO_CLEAR = true;
//=============================================================================
if (aoList[7]) {
PIXI.tilemap.TileRenderer.DO_CLEAR = false;
}
//=============================================================================
//去掉战斗前动画
//=============================================================================
if (aoList[8]) {
Scene_Map.prototype.startFlashForEncounter=function() {};
Scene_Map.prototype.startEncounterEffect = function() {
    this.snapForBattleBackground();
};
}
//=============================================================================
//深度删除数据。提早进行回收
//=============================================================================
if (aoList[9]) {
$.Scene_Battle_terminate = Scene_Battle.prototype.terminate;
Scene_Battle.prototype.terminate = function() {
    $.Scene_Battle_terminate.apply(this,arguments);
    this.deepDestroyScene_JWJMX();
};
Scene_Battle.prototype.deepDestroyScene_JWJMX = function() {
    BattleManager.initMembers();
    try{
        this.iterationDeepDestroyScene_JWJMX(this,[]);
    } catch(e) {

    }
};
Scene_Battle.prototype.iterationDeepDestroyScene_JWJMX = function(_scene,list) {
    try{
        if (this && this.destroy && typeof this.destroy === "function") {
            this.destroy({children:true,texture:true});
        }
    } catch(e) {

    }
    list.push(this);
    let keyList = Object.keys(this);
    for (let i of keyList) {
        if (this[i] !== this && !list.includes(this[i]) && list.length<100) {
            if (this[i] && typeof this[i] === "object") {
                _scene.iterationDeepDestroyScene_JWJMX.call(this[i],_scene,list);
            }
            this[i] = null;
        }
    }
};
$.Scene_Map_terminate = Scene_Map.prototype.terminate;
Scene_Map.prototype.terminate = function() {
    $.Scene_Map_terminate.apply(this,arguments);
    this.deepDestroyScene_JWJMX(this);
};
Scene_Map.prototype.deepDestroyScene_JWJMX = function(_scene) {
    this.destroy({children:true,texture:true});
    let keyList = Object.keys(this);
    for (let i of keyList) {
        this[i] = null;
    }
};
Sprite.prototype.destroy = function() {
    const options = { children: true, texture: true };
    PIXI.Sprite.prototype.destroy.call(this, options);
};
Tilemap.prototype.destroy = function() {
    const options = { children: true, texture: true };
    PIXI.Container.prototype.destroy.call(this, options);
};
TilingSprite.prototype.destroy = function() {
    const options = { children: true, texture: true };
    PIXI.TilingSprite.prototype.destroy.call(this, options);
};
Window.prototype.destroy = function() {
    if (this.filters) this.filters[0] = null;
    this.filters = null;
    const options = { children: true, texture: true };
    PIXI.Container.prototype.destroy.call(this, options);
};
}
//=============================================================================
//
//=============================================================================
//=============================================================================
//
//=============================================================================
//=============================================================================
//
//=============================================================================
//=============================================================================
//
//=============================================================================
//=============================================================================
//
//=============================================================================
})(JWJMX.AO.reWrite);
//=============================================================================
//
//=============================================================================