//=============================================================================
// RPG Maker MZ - JiuJiu-ActorQuality.js
//=============================================================================
/*:
 * @plugindesc  Actor Quality [22-4-15]
 * @author Qiu Jiu
 *
 * @help JiuJiu-ActorQuality.js
 *
 *
 * 用法：
 * 1.在数据库中设置品质的数据。
 * 2.在角色的备注栏中写基础的经验值（由最低级到下一级）和增长的比率：
 *   <qualityExp:基础值,增长率>
 * 3.在物品的备注栏中写 <actorQualityExp:增加的经验值> 可以让玩家在使用此物品后增加
 *   一定的品质经验。
 *
 *
 * 例如：
 * 1.在数据库中设置5个品质： 低级 较低级 中级 较高级 高级
 * 2.在某个角色的备注中写<qualityExp:5,10>
 * 则此角色的默认品质为低级，由低级到较低级需要5点经验，由较低级到中级需要15点经验，
 * 又中级到较高级需要25点经验,由较高级到高级需要35点经验。
 *
 *
 *
 * @param data
 * @type struct<dataList>[]
 * @text 品质设置
 * @desc 品质设置
 * @default ["{\"name\":\"低级\",\"icon\":\"160\",\"mhp\":\"100\",\"mmp\":\"100\",\"atk\":\"100\",\"def\":\"100\",\"mat\":\"100\",\"mdf\":\"100\",\"agi\":\"100\",\"luk\":\"100\"}","{\"name\":\"中低级\",\"icon\":\"161\",\"mhp\":\"105\",\"mmp\":\"105\",\"atk\":\"105\",\"def\":\"105\",\"mat\":\"105\",\"mdf\":\"105\",\"agi\":\"105\",\"luk\":\"105\"}","{\"name\":\"中级\",\"icon\":\"162\",\"mhp\":\"110\",\"mmp\":\"110\",\"atk\":\"110\",\"def\":\"110\",\"mat\":\"110\",\"mdf\":\"110\",\"agi\":\"110\",\"luk\":\"110\"}","{\"name\":\"中高级\",\"icon\":\"163\",\"mhp\":\"115\",\"mmp\":\"115\",\"atk\":\"115\",\"def\":\"115\",\"mat\":\"115\",\"mdf\":\"115\",\"agi\":\"115\",\"luk\":\"115\"}","{\"name\":\"高级\",\"icon\":\"164\",\"mhp\":\"120\",\"mmp\":\"120\",\"atk\":\"120\",\"def\":\"120\",\"mat\":\"120\",\"mdf\":\"120\",\"agi\":\"120\",\"luk\":\"120\"}"]
 *
 * @param text
 * @type text
 * @text 满级品质文字
 * @desc 满级品质文字
 * @default 最高品质
 *
 * @param baseDefault1
 * @type number
 * @text 默认基础经验
 * @desc 默认基础经验
 * @default 5
 *
 * @param baseDefault2
 * @type number
 * @text 默认增长比率
 * @desc 默认增长比率
 * @default 10
 *
 * @param color1
 * @type number
 * @text 品质经验条背景颜色1
 * @desc 品质经验条背景颜色1
 * @default #ffffff
 *
 * @param color2
 * @type number
 * @text 品质经验条背景颜色2
 * @desc 品质经验条背景颜色2
 * @default #999999
 *
 * @param color3
 * @type number
 * @text 品质经验条背景颜色3
 * @desc 品质经验条背景颜色3
 * @default #b459e8
 *
 * @param color4
 * @type number
 * @text 品质经验条背景颜色4
 * @desc 品质经验条背景颜色4
 * @default #ffc7fb
 *
*/
/*~struct~dataList:
 *
 * @param name
 * @type text
 * @text 品质名
 * @desc 品质名
 * @default 初级
 *
 * @param icon
 * @type icon
 * @text 图标编号
 * @desc 图标编号
 * @default 0
 *
 * @param mhp
 * @type number
 * @text 最大生命值增长比率
 * @desc 最大生命值增长比率（%）
 * @default 100
 *
 * @param mmp
 * @type number
 * @text 最大法力增长比率
 * @desc 最大法力增长比率（%）
 * @default 100
 *
 * @param atk
 * @type number
 * @text 最大攻击力增长比率
 * @desc 最大攻击力增长比率（%）
 * @default 100
 *
 * @param def
 * @type number
 * @text 最大防御力增长比率
 * @desc 最大防御力增长比率（%）
 * @default 100
 *
 * @param mat
 * @type number
 * @text 最大魔法攻击增长比率
 * @desc 最大魔法攻击增长比率（%）
 * @default 100
 *
 * @param mdf
 * @type number
 * @text 最大魔法防御增长比率
 * @desc 最大魔法防御增长比率（%）
 * @default 100
 *
 * @param agi
 * @type number
 * @text 最大敏捷增长比率
 * @desc 最大敏捷增长比率（%）
 * @default 100
 *
 * @param luk
 * @type number
 * @text 最大幸运值增长比率
 * @desc 最大幸运值增长比率（%）
 * @default 100
 *
 *
*/
//=============================================================================
//
//=============================================================================
(($ = {}) => {
//=============================================================================
//
//=============================================================================
const pluginName = "JiuJiu-ActorQuality";
const parameters = PluginManager.parameters(pluginName);
const actorQualityData = ((value)=>{
    let list = [];
    for (let i of eval(value)) {
        list.push(JsonEx.parse(i));
    }
    return list;
})(parameters.data);
const actorQualityName = parameters.text;
const defaultData = [Number(parameters.baseDefault1),Number(parameters.baseDefault2)];
const paramIndex = ['mhp','mmp','atk','def','mat','mdf','agi','luk'];
//=============================================================================
//
//=============================================================================
$.Game_Actor_initMembers = Game_Actor.prototype.initMembers;
Game_Actor.prototype.initMembers = function() {
    $.Game_Actor_initMembers.apply(this,arguments);
    this.initActorQuality();
};
$.Game_Actor_setup = Game_Actor.prototype.setup;
Game_Actor.prototype.setup = function(actorId) {
    $.Game_Actor_setup.apply(this,arguments);
    this.initActorQuality(actorId);
};
Game_Actor.prototype.initActorQuality = function(actorId) {
    this._qualityLevel = 0;
    this._qualityExp = 0;
    if (actorId) {
        const actor = $dataActors[actorId];
        if (actor.meta.qualityExp) {
            let data = actor.meta.qualityExp.split(',');
            this._qualityData = [Number(data[0]),Number(data[1])];
        } else {
            this._qualityData = defaultData;
        }
    } else {
        this._qualityData = defaultData;
    }
};
Game_Actor.prototype.addActorQuality = function(value) {
    this._baseParamCache = [];
    this._qualityExp=Math.max(0,this._qualityExp+value);
    if (this._qualityExp==0) {
        this._qualityLevel = 0;
        return;
    }
    let a = (this._qualityData[0]+this._qualityData[1])/2;
    let b = (this._qualityData[0]-this._qualityData[1])/2;
    let c = -this._qualityExp;
    if (b*b-4*a*c<0) {
        //impossible
    } else {
        let result = Math.min(Math.floor((-b+Math.sqrt(b*b-4*a*c))/(2*a)),actorQualityData.length-1);
        this._qualityLevel = result;
    }
};
Game_Actor.prototype.actorQualityLevelExp = function(level) {
    level = Math.min(level,actorQualityData.length-1);
    let a = (this._qualityData[0]+this._qualityData[1])/2;
    let b = (this._qualityData[0]-this._qualityData[1])/2;
    return a*level*level + b*level;
};
Game_Actor.prototype.actorQualityPer = function() {
    this.addActorQuality(0);
    let x1 = this.actorQualityLevelExp(this._qualityLevel);
    let x2 = this.actorQualityLevelExp(this._qualityLevel+1);
    let x3 = this._qualityExp;
    return (x2==x1)?1:(x3-x1)/(x2-x1);
};
Game_Actor.prototype.actorQualityExpText = function() {
    if (this._qualityLevel==actorQualityData.length-1) {
        return actorQualityName;
    } else {
        return this._qualityExp+'/'+this.actorQualityLevelExp(this._qualityLevel+1);
    }
};
//=============================================================================
//
//=============================================================================
$.Window_MenuStatus_drawItemImage = Window_MenuStatus.prototype.drawItemImage;
Window_MenuStatus.prototype.drawItemImage = function(index) {
    $.Window_MenuStatus_drawItemImage.apply(this,arguments);
    this.drawActorQuality(index);
};
Window_MenuStatus.prototype.drawActorQuality = function(index) {
    const actor = $gameParty.members()[index];
    const rect = this.itemRect(index);
    rect.y += (rect.height - 144) / 2;
    const width = 144;
    const height = 144;
    const lineHeight = this.lineHeight();
    this.resetFontSettings();
    if (actorQualityData[actor._qualityLevel]) {
        this.drawIcon(actorQualityData[actor._qualityLevel].icon,rect.x + 1,rect.y + 6 + height - lineHeight);
        this.contents.fontSize = 16;
        this.drawText(actorQualityData[actor._qualityLevel].name,rect.x + 1 + 32,rect.y + height - lineHeight,
            width - 32,'left');
        this.drawText(actor.actorQualityExpText(),rect.x + 1 + 32,rect.y + height - lineHeight,
            width - 32,'right');
        this.contents.gradientFillRect(rect.x + 1 + 32, rect.y + height - lineHeight + 28, width - 32, lineHeight - 28,
            parameters.color1,parameters.color2);
        this.contents.gradientFillRect(rect.x + 1 + 32, rect.y + height - lineHeight + 28, 
            (width - 32)*actor.actorQualityPer(), lineHeight - 28,parameters.color3,parameters.color4);
    } else {
        //==========
    }
    this.resetFontSettings();
};
//=============================================================================
//
//=============================================================================
$.Game_Action_apply = Game_Action.prototype.apply;
Game_Action.prototype.apply = function(target) {
    $.Game_Action_apply.apply(this,arguments);
    if (target.constructor.name!='Game_Actor') return;
    let item = this.item();
    if (item.meta.actorQualityExp) {
        let num = Number(item.meta.actorQualityExp);
        if (!isNaN(num)) {
            target.addActorQuality(num);
        }
    }
};
$.Game_Action_testApply = Game_Action.prototype.testApply;
Game_Action.prototype.testApply = function(target) {
    let result = $.Game_Action_testApply.apply(this,arguments);
    if (target.constructor.name!='Game_Actor') return result;
    if (!result) {
        if (target._qualityLevel&&target._qualityLevel==actorQualityData.length-1) {
            return false;
        }
        let item = this.item();
        if (item.meta.actorQualityExp) {
            let num = Number(item.meta.actorQualityExp);
            if (!isNaN(num)) {
                return true;
            } else {
                return false; 
            }
        } else {
            return false;
        }
    } else {
        return true;
    }
};
//=============================================================================
//
//=============================================================================
$.Game_BattlerBase_traitsWithId = Game_BattlerBase.prototype.traitsWithId;
Game_BattlerBase.prototype.traitsWithId = function(code, id) {
    let data = $.Game_BattlerBase_traitsWithId.apply(this,arguments);
    if (this.constructor.name!='Game_Actor') return data;
    if (code==Game_BattlerBase.TRAIT_PARAM) {
        let list = actorQualityData[this._qualityLevel];
        data.push({code: code, dataId: id, value: Math.floor(Number(list[paramIndex[id]])*100)/10000});
    }
    return data;
};
//=============================================================================
//
//=============================================================================
//=============================================================================
//
//=============================================================================
//=============================================================================
//
//=============================================================================
})(/*暴露在外部的可能名*/);
//=============================================================================
//
//=============================================================================