/*:
 * @plugindesc 为RPG Maker MV新增任务系统。版本：V12.0.3
 * @author Lagomoro【66RPG：rpg-sheep】【百度贴吧：优加星爱兔子】
 *
 * @param windowmode
 * @desc 决定任务窗口位置（1=左置，2=右置）
 * @default 1
 *
 * @param windowcolor
 * @desc 决定任务窗口颜色（rgba）
 * @default rgba(0, 0, 0, 0.4)
 * 
 * @param toast
 * @desc 是否推送任务信息（1=是，2=否）
 * @default 1
 * 
 * @param windowopen
 * @desc 是否开启地图窗口初始值（1=是，2=否）将会记入存档
 * 可使用脚本$gameSystem._WindowOpen_Mission = true/false更改
 * @default 1
 * 
 * @param xlssheet
 * @desc 读取xlsx的第几页Sheet（1或2）
 * @default 1
 * 
 * @param usexlsx
 * @desc 是否开启xlsx功能（1=是，2=否）
 * 请确保你加载了xlsx运行库再选择是！
 * @default 2
 *
 * @help 
 * ======================================================================
 * Lagomoro的任务系统采用全新逻辑方案，与传统任务系统相比，你可以做出任何效果！
 * ----------------------------------------------------------------------
 * 详情请参阅帮助文档！
 * ======================================================================
 * 本插件须遵守的规约：
 * ----------------------------------------------------------------------
 * 允许用于商业发布的游戏。
 * 禁止二次修改后单独发布。您可以二次修改，但只限于在您的游戏中使用它。
 * 如果您的游戏中使用了此插件，该游戏感谢名单中的醒目位置必须包含Lagomoro，这是对作者最起码的尊重。
 * 作者Lagomoro非常愿意参与您的游戏测试，不妨让我也玩一玩您的游戏QAQ？
 * BUG反馈，联系作者请通过QQ：2669708373 加好友请注明来意，不写好友申请或者理由奇怪的话我不会理哦！
 * ======================================================================
 */
 
//
// ======================================================================
// * Window_Lagomoro_Mission
// ======================================================================
function Window_Lagomoro_Mission() {this.initialize.apply(this, arguments);}
Window_Lagomoro_Mission.prototype = Object.create(Window_Base.prototype);
Window_Lagomoro_Mission.prototype.constructor = Window_Lagomoro_Mission;
Window_Lagomoro_Mission.prototype.initialize = function() {
    Window_Base.prototype.initialize.call(this, (Number(PluginManager.parameters('Lagomoro_Mission')['windowmode'] || 2) === 2 ? Graphics.boxWidth - this.windowWidth() : 0), Graphics.boxHeight/2, this.windowWidth(), this.windowHeight());
    this.opacity = 0;
	this.line = 0;
	this.padd = 1;
    this.refresh();
};
Window_Lagomoro_Mission.prototype.standardFontSize = function() {return 18;};
Window_Lagomoro_Mission.prototype.standardPadding = function() {return 0;};
Window_Lagomoro_Mission.prototype.textPadding = function() {return 10;};
Window_Lagomoro_Mission.prototype.windowWidth = function() {return 300;};
Window_Lagomoro_Mission.prototype.windowHeight = function() {return 300};
Window_Lagomoro_Mission.prototype.update = function() {
	Window_Base.prototype.update.call(this);
	this.refresh();
};
Window_Lagomoro_Mission.prototype.refresh = function() {
    this.contents.clear();
    var width = this.contentsWidth();
	var missionlist = $gameParty.getucmission().reverse();
	var height = this.standardFontSize()*$gameParty.getmissiondrawline() + (1 + missionlist.length)*this.textPadding();
	this.line = 0;
	this.padd = 0;
	if(missionlist.length > 0){this.drawBackground(width, height);}
	for(var i = 0;i < missionlist.length;i++){
		var childinfos = [];
		for(var b = 0;b < missionlist[i].childs.length;b++){
		    childinfos.push([missionlist[i].childs[b][1],missionlist[i].getchildratebyindex(b),missionlist[i].childs[b][5]]);	
	    }
		this.drawMission(width,missionlist[i].name,missionlist[i].description,childinfos,missionlist[i].color);
		this.padd ++;
	}
};
Window_Lagomoro_Mission.prototype.drawBackground = function(width, height) {
	this.contents.context.fillStyle = (PluginManager.parameters('Lagomoro_Mission')['windowcolor']||'rgba(0, 0, 0, 0.4)');
    this.contents.context.fillRect(0, 0, width, height);
};
Window_Lagomoro_Mission.prototype.drawMission = function(width,name,description,childinfos,color) {
	this.contents.textColor = color;
    this.drawText(name, this.textPadding(), this.standardFontSize()*this.line + this.textPadding()*this.padd, width ,'left');
	this.contents.textColor = '#ffffff';
	this.line ++;
	this.contents.fontSize -= 2;
	for(var i = 0;i < childinfos.length;i++){
		this.contents.textColor = (childinfos[i][2] ? '#999999' : '#ffffff');
		this.drawText('▪ ' + childinfos[i][0], this.textPadding()*3, this.standardFontSize()*this.line + this.textPadding()*this.padd, width, 'left');
        this.drawText(childinfos[i][1], 0, this.standardFontSize()*this.line + this.textPadding()*this.padd, width - this.textPadding(), 'right');
		this.contents.textColor = '#ffffff';
		this.line ++;
	}
	this.contents.fontSize += 2;
};
// ======================================================================
// * Scene_Map
// ======================================================================
Scene_Map.prototype.Lagomoro_Mission_old_createDisplayObjects = Scene_Map.prototype.createDisplayObjects;
Scene_Map.prototype.createDisplayObjects = function() {
	this.Lagomoro_Mission_old_createDisplayObjects();
	this.Lagomoro_createMissionWindow();
};
Scene_Map.prototype.Lagomoro_createMissionWindow = function() {
    this._Lagomoro_MissionWindow = new Window_Lagomoro_Mission();
    this.addWindow(this._Lagomoro_MissionWindow);
};
Scene_Map.prototype.Lagomoro_Mission_old_update = Scene_Map.prototype.update;
Scene_Map.prototype.update = function() {
    this.Lagomoro_Mission_old_update();
    if($gameMap._interpreter.isRunning()||!$gameSystem._WindowOpen_Mission){
        this._Lagomoro_MissionWindow.close();
    }else{
        this._Lagomoro_MissionWindow.open();  
    }
};
// ======================================================================
// * Lagomoro_Mission*/
// ======================================================================
function Lagomoro_Mission() {this.initialize.apply(this, arguments);}
Lagomoro_Mission.prototype.initialize = function(id,name,description,childs,reward,color,autocomplete) {
    this.id = id;
    this.name = name;
    this.description = description;
	//Child[id,name,maxnumber,readnumber,autocomplete,completed]
    this.childs = childs;
    this.reward = reward;
	this.color = (color ? color : '#FFFFFF');
    this.autocomplete = (autocomplete ? autocomplete : false);
    this.completed = false;
    if(Number(PluginManager.parameters('Lagomoro_Mission')['toast'] || 1) === 1){
        $gameTemp.toast('你接取了任务 “' + this.name + '”', this.color);
    }
};
//添加任务要点
Lagomoro_Mission.prototype.addchild = function(child){
    this.childs.push(child);
};
//根据id寻找任务要点
Lagomoro_Mission.prototype.findchildindexbyid = function(id){
    for(var i = 0;i < this.childs.length;i++){
        if(this.childs[i][0] === id){
            return i;
        }
    }
	return null;
};
//获取任务要点完成度（仅绘图用）
Lagomoro_Mission.prototype.getchildratebyid = function(id){
	if(this.findchildindexbyid(id)!==null){
	    return this.getchildratebyindex(this.findchildindexbyid(id));
    }
	return false;
};
Lagomoro_Mission.prototype.getchildratebyindex = function(index){
    if(index >= 0 && index < this.childs.length){
		return ((this.childs[index][3]<=0||this.childs[index][3]>0) ? this.childs[index][3] : eval(this.childs[index][3])) + "/" + this.childs[index][2];
    }
	return false;
};
//增加一点任务要点完成度
Lagomoro_Mission.prototype.upchildratebyid = function(id){
 	if(this.findchildindexbyid(id)!==null){
		this.upchildratebyindex(this.findchildindexbyid(id));
    }
};
Lagomoro_Mission.prototype.upchildratebyindex = function(index){
    if(index >= 0 && index < this.childs.length){
		if(this.childs[index][3]<=0||this.childs[index][3]>0){
			this.childs[index][3]++;
			if(Number(PluginManager.parameters('Lagomoro_Mission')['toast'] || 1) === 1){
                $gameTemp.toast(this.childs[index][1] + '（' + this.getchildratebyindex(index) + '）', this.color);
            }
        }
    }
};
//设置任务要点完成度
Lagomoro_Mission.prototype.setchildratebyid = function(id,num){
 	if(this.findchildindexbyid(id)!==null){
	    this.setchildratebyindex(this.findchildindexbyid(id),num);
    }
};
Lagomoro_Mission.prototype.setchildratebyindex = function(index,num){
    if(index >= 0 && index < this.childs.length){
		if(this.childs[index][3]<=0||this.childs[index][3]>0){
			this.childs[index][3] = num;
			if(Number(PluginManager.parameters('Lagomoro_Mission')['toast'] || 1) === 1){
                $gameTemp.toast(this.childs[index][1] + '（' + this.getchildratebyindex(index) + '）', this.color);
            }
        }
    }
};
//移除任务要点
Lagomoro_Mission.prototype.removechildbyid = function(id){
 	if(this.findchildindexbyid(id)!==null){
	    this.removechildbyindex(this.findchildindexbyid(id));
    }
};
Lagomoro_Mission.prototype.removechildbyindex = function(index){
    if(index >= 0 && index < this.childs.length){
	    this.childs.splice(index,1);
    }
};
//强制完成任务要点
Lagomoro_Mission.prototype.completechildbyid = function(id){
 	if(this.findchildindexbyid(id)!==null){
	    this.completechildbyindex(this.findchildindexbyid(id));
    }
};
Lagomoro_Mission.prototype.completechildbyindex = function(index){
    if(index >= 0 && index < this.childs.length && !this.childs[index][5]){
		if(Number(PluginManager.parameters('Lagomoro_Mission')['toast'] || 1) === 1){
            $gameTemp.toast('完成任务要点 “' + this.childs[index][1] + '” ！', this.color);
        }
        this.childs[index][5] = true;
    }
};
//获取任务要点是否达成
Lagomoro_Mission.prototype.ischildcompletebyid = function(id){
 	if(this.findchildindexbyid(id)!==null){
	    return this.ischildcompletebyindex(this.findchildindexbyid(id));
    }
	return false;
};
Lagomoro_Mission.prototype.ischildcompletebyindex = function(index){
    if(index >= 0 && index < this.childs.length){
		return (((this.childs[index][3]<=0||this.childs[index][3]>0) ? this.childs[index][3] : eval(this.childs[index][3])) >= this.childs[index][2]);
    }
	return false;
};
//获取任务要点是否完成
Lagomoro_Mission.prototype.ischildcompletedbyid = function(id){
 	if(this.findchildindexbyid(id)!==null){
	    return this.ischildcompletedbyindex(this.findchildindexbyid(id));
    }
	return false;
};
Lagomoro_Mission.prototype.ischildcompletedbyindex = function(index){
    if(index >= 0 && index < this.childs.length){
		return this.childs[index][5];
    }
	return false;
};
//手动完成任务要点
Lagomoro_Mission.prototype.donechildbyid = function(id){
 	if(this.findchildindexbyid(id)!==null){
	    return this.donechildbyindex(this.findchildindexbyid(id));
    }
	return false;
};
Lagomoro_Mission.prototype.donechildbyindex = function(index){
    if(index >= 0 && index < this.childs.length){
	    if(this.ischildcompletebyindex(index)){this.completechildbyindex(index);}
	    return this.ischildcompletedbyindex(index);
    }
	return false;
};
//自动完成任务要点
Lagomoro_Mission.prototype.testchildbyid = function(id){
 	if(this.findchildindexbyid(id)!==null){
	    this.testchildbyindex(this.findchildindexbyid(id));
    }
};
Lagomoro_Mission.prototype.testchildbyindex = function(index){
    if(index >= 0 && index < this.childs.length){
	    if(this.ischildcompletebyindex(index)&&this.childs[index][4]){this.completechildbyindex(index);}
    }
};
//自动完成全部任务要点
Lagomoro_Mission.prototype.testallchild = function(){
	for(var i = 0;i < this.childs.length;i++){
		this.testchildbyindex(i);
    }
};
//强制完成任务
Lagomoro_Mission.prototype.complete = function(){
	if(!this.completed){
	    if(Number(PluginManager.parameters('Lagomoro_Mission')['toast'] || 1) === 1){
            $gameTemp.toast('完成任务 “' + this.name + '” !', this.color);
        }
        this.getreward();
	    this.completed = true;
	}
};
//获取任务是否达成
Lagomoro_Mission.prototype.iscomplete = function(){
    var isco = true;
    for(var i = 0;i < this.childs.length;i++){
        if(!this.ischildcompletedbyindex(i)){
            isco = false;
        }
    }
    return isco;
};
//获取任务是否完成
Lagomoro_Mission.prototype.iscompleted = function(){
    return this.completed;
};
//手动完成任务
Lagomoro_Mission.prototype.done = function(){
    if(this.iscomplete()){this.complete();}
	return this.iscompleted();
};
//自动完成任务
Lagomoro_Mission.prototype.test = function(){
	if(this.iscomplete()&&this.autocomplete){this.complete();}
};
//自动完成任务并且自动完成任务要点
Lagomoro_Mission.prototype.testall = function(){
    this.testallchild();
	this.test();
};
//获取奖励
Lagomoro_Mission.prototype.getreward = function(){
    for(var i = 0;i < this.reward.length;i++){
        if(this.reward[i][0] === 'EXP'){
            $gameParty.members().forEach(function(actor) {
                actor.changeExp(actor.currentExp() + this.reward[i][1], true);
            }.bind(this));
        }else if(this.reward[i][0] === 'MONEY'){
            $gameParty.gainGold(this.reward[i][1]);
        }else if(this.reward[i][0] === 'VAR'){
            eval(this.reward[i][1] + '+=' + this.reward[i][2]);
        }else if(this.reward[i][0] === 'ITEM'){
            $gameParty.gainItem($dataItems[this.reward[i][1]], this.reward[i][2]);
        }else if(this.reward[i][0] === 'WEAPON'){
            $gameParty.gainItem($dataWeapons[this.reward[i][1]], this.reward[i][2]);
        }else if(this.reward[i][0] === 'ARMOR'){
            $gameParty.gainItem($dataArmors[this.reward[i][1]], this.reward[i][2]);
        }
    }
};
// ======================================================================
// * Game_Temp
// ======================================================================
Game_Temp.prototype.Lagomoro_Mission_old_initialize = Game_Temp.prototype.initialize;
Game_Temp.prototype.initialize = function() {
    this.Lagomoro_Mission_old_initialize();
    this._mission_xlsx = [];
    this._mission_json = [];
    if(Number(PluginManager.parameters('Lagomoro_Mission')['usexlsx'] || 1) === 1){
        this.loadXlsx_Mission();
    }
    this.loadJson_Mission();
    console.log(this._mission_xlsx)
};
Game_Temp.prototype.loadXlsx_Mission = function() {
    var path = require('path');
    var base = path.join(path.dirname(process.mainModule.filename), 'data/Lagomoro_Mission.xlsx');

    if(!require('fs').existsSync(base)){
        return;
    }
    var xlstype = Number(PluginManager.parameters('Lagomoro_Mission')['xlssheet'] || 1) - 1;
    var workbook = XLSX.readFile(base);
    var worksheet = workbook.Sheets[workbook.SheetNames[xlstype]];
    var linenumber = 3 - xlstype;
    var temparray = [];
    while(!this.isXlsxEnd(worksheet,linenumber)){
        var linedata = {
            'A':this.readXlsxCell(worksheet,'A'+linenumber),
            'B':this.readXlsxCell(worksheet,'B'+linenumber),
            'C':this.readXlsxCell(worksheet,'C'+linenumber),
            'D':this.readXlsxCell(worksheet,'D'+linenumber),
            'E':this.readXlsxCell(worksheet,'E'+linenumber),
            'F':this.readXlsxCell(worksheet,'F'+linenumber),
            'G':this.readXlsxCell(worksheet,'G'+linenumber)
        }
        if(xlstype === 0){
            if(this.readXlsxCell(worksheet,'A'+linenumber)){
                if(temparray.length > 0){
                    this._mission_xlsx.push(temparray);
                }
                temparray = [];
                temparray.push(String(linedata.B));
                temparray.push(linedata.A);
                temparray.push(linedata.C);
                temparray.push([]);
                temparray.push(this.translateReward(linedata.D));
                temparray.push(linedata.E);
                temparray.push(linedata.F);
            }else{
                temparray[3].push([
                    String(linedata.B),
                    linedata.C,
                    (parseInt(linedata.D).toString() === linedata.D ? Number(linedata.D) : linedata.D),
                    (parseInt(linedata.E).toString() === linedata.E ? Number(linedata.E) : linedata.E),
                    linedata.F,
                    false
                ]);
            }
        }else{
            temparray.push(String(linedata.B));
            temparray.push(linedata.A);
            temparray.push(linedata.C);
            temparray.push(this.translateChild(linedata.D));
            temparray.push(this.translateReward(linedata.E));
            temparray.push(linedata.F);
            temparray.push(linedata.G);
            this._mission_xlsx.push(temparray);
            temparray = [];
        }
        linenumber++;
    }
    if(xlstype === 0 && temparray.length > 0){
        this._mission_xlsx.push(temparray);
    }
};
Game_Temp.prototype.translateChild = function(text) {
    if(!text) return [];
    var temp = text.split('|')
    for(var i = 0;i < temp.length;i++){
        temp[i] = temp[i].split(',');
        temp[i][2] = (parseInt(temp[i][2]).toString() === temp[i][2] ? Number(temp[i][2]) : temp[i][2]);
        temp[i][3] = (parseInt(temp[i][3]).toString() === temp[i][3] ? Number(temp[i][3]) : temp[i][3]);
        temp[i][4] = (temp[i][4] === 'TRUE' ? true : false);
        temp[i][5] = false;
    }
    return temp;
};
Game_Temp.prototype.translateReward = function(text) {
    if(!text) return [];
    var temp = text.split('|')
    for(var i = 0;i < temp.length;i++){
        temp[i] = temp[i].split(',');
        for(var a = 1;a < temp[i].length;a++){
            temp[i][a] = Number(temp[i][a]);
        }
    }
    return temp;
};
Game_Temp.prototype.readXlsxCell = function(worksheet,address) {
    var cell = worksheet[address];
    return (cell ? cell.v : undefined);
};
Game_Temp.prototype.isXlsxEnd = function(worksheet,linenumber) {
    var linename = ['A','B','C','D','E','F','G']
    for(var i = 0;i < 7;i++){
        if(this.readXlsxCell(worksheet,linename[i]+linenumber)){
            return false;
        }
    }
    return true;
};
Game_Temp.prototype.addMissionXlsx = function(id) {
    for(var i = 0;i < this._mission_xlsx.length;i++){
        if(this._mission_xlsx[i][0] === id){
            $gameParty.addmission(this._mission_xlsx[i][0],this._mission_xlsx[i][1],this._mission_xlsx[i][2],this._mission_xlsx[i][3],this._mission_xlsx[i][4],this._mission_xlsx[i][5],this._mission_xlsx[i][6]);
        }
    }
};

Game_Temp.prototype.loadJson_Mission = function() {
    var path = require('path');
    var base = path.join(path.dirname(process.mainModule.filename), 'data/Lagomoro_Mission.json');

    if(!require('fs').existsSync(base)){
        return;
    }
    try {
        var fs = require('fs');
        path.dirname(process.mainModule.filename)
        var filePath = base;
        if (fs.existsSync(filePath)) {
            data = fs.readFileSync(filePath, { encoding: 'utf8' });
        }
    } catch (e) {
        console.error(e);
    }
    if (data) {
        this._mission_json = JSON.parse(data);
    }
}
Game_Temp.prototype.addMissionJson = function(id) {
    for(var i = 0;i < this._mission_json.length;i++){
        if(this._mission_json[i].id === id){
            $gameParty.addmission(this._mission_json[i].id,this._mission_json[i].name,this._mission_json[i].description,this._mission_json[i].childs,this._mission_json[i].reward,this._mission_json[i].color,this._mission_json[i].autocomplete);
        }
    }
};
// ======================================================================
// * Game_System
// ======================================================================
Game_System.prototype.Lagomoro_Mission_old_initialize = Game_System.prototype.initialize;
Game_System.prototype.initialize = function() {
    this.Lagomoro_Mission_old_initialize();
    this._WindowOpen_Mission = Number(PluginManager.parameters('Lagomoro_Mission')['windowopen'] || 1) === 1 ? true : false;
};
// ======================================================================
// * Game_Party
// ======================================================================
Game_Party.prototype.Lagomoro_Mission_old_initialize = Game_Party.prototype.initialize;
Game_Party.prototype.initialize = function() {
    this.Lagomoro_Mission_old_initialize();
    this._missionlist = [];
};
//添加任务
Game_Party.prototype.addmission = function(id,name,description,childs,reward,color,autocomplete) {
    this._missionlist.push(new Lagomoro_Mission(id,name,description,childs,reward,color,autocomplete));
};
//添加要点
Game_Party.prototype.addmissionchild = function(id,child) {
	if(this.findmissionindexbyid(id)!==null){
         this._missionlist[this.findmissionindexbyid(id)].addchild(child);
	}
};
//根据id寻找任务
Game_Party.prototype.findmissionindexbyid = function(id){
    for(var i = 0;i < this._missionlist.length;i++){
        if(this._missionlist[i].id === id){
            return i;
        }
    }
	return null;
};
//删除任务
Game_Party.prototype.delmission = function(id) {
	if(this.findmissionindexbyid(id)!==null){
        this._missionlist.splice(this.findmissionindexbyid(id),1);
	}
};
//删除任务要点
Game_Party.prototype.delmissionchild = function(id,childid) {
	if(this.findmissionindexbyid(id)!==null){
        this._missionlist[this.findmissionindexbyid(id)].removechildbyid(childid);
	}
};
//刷新所有
Game_Party.prototype.testallmission = function(){
    for(var i = 0;i < this._missionlist.length;i++){
        this._missionlist[i].testall();
    }
};
//任务是否包含要点
Game_Party.prototype.ismissionchildexist = function(id,childid){
    if(this.findmissionindexbyid(id)!==null){
        return this._missionlist[this.findmissionindexbyid(id)].findchildindexbyid(childid) !== null;
	}
	return false;
};
//任务要点是否达成
Game_Party.prototype.ismissionchildcomplete = function(id,childid){
    if(this.findmissionindexbyid(id)!==null){
        return this._missionlist[this.findmissionindexbyid(id)].ischildcompletebyid(childid);
	}
	return false;
};
//任务要点是否完成
Game_Party.prototype.ismissionchildcompleted = function(id,childid){
    if(this.findmissionindexbyid(id)!==null){
        return this._missionlist[this.findmissionindexbyid(id)].ischildcompletedbyid(childid);
	}
	return false;
};
//任务是否达成
Game_Party.prototype.ismissioncomplete = function(id){
    if(this.findmissionindexbyid(id)!==null){
        return this._missionlist[this.findmissionindexbyid(id)].iscomplete();
	}
	return false;
};
//任务是否完成
Game_Party.prototype.ismissioncompleted = function(id){
    if(this.findmissionindexbyid(id)!==null){
        return this._missionlist[this.findmissionindexbyid(id)].iscompleted();
	}
	return false;
};
//增加一点任务要点完成度
Game_Party.prototype.upratemissionchild = function(id,childid){
    if(this.findmissionindexbyid(id)!==null){
        this._missionlist[this.findmissionindexbyid(id)].upchildratebyid(childid);
	}
};
//设置任务要点完成度
Game_Party.prototype.setratemissionchild = function(id,childid,num){
    if(this.findmissionindexbyid(id)!==null){
        this._missionlist[this.findmissionindexbyid(id)].setchildratebyid(childid,num);
	}
};
//强制完成任务要点
Game_Party.prototype.completemissionchild = function(id,childid){
    if(this.findmissionindexbyid(id)!==null){
        this._missionlist[this.findmissionindexbyid(id)].completechildbyid(childid);
	}
};
//手动完成任务要点
Game_Party.prototype.donemissionchild = function(id,childid){
    if(this.findmissionindexbyid(id)!==null){
        return this._missionlist[this.findmissionindexbyid(id)].donechildbyid(childid);
	}
	return false;
};
//强制完成任务
Game_Party.prototype.completemission = function(id){
    if(this.findmissionindexbyid(id)!==null){
        this._missionlist[this.findmissionindexbyid(id)].complete();
	}
};
//手动完成任务
Game_Party.prototype.donemission = function(id){
    if(this.findmissionindexbyid(id)!==null){
        return this._missionlist[this.findmissionindexbyid(id)].done();
	}
	return false;
};
//获取所有任务
Game_Party.prototype.getallmission = function(){
    return this._missionlist;
};
//获取所有未完成任务
Game_Party.prototype.getucmission = function(){
	var allmission = [];
	for(var i = 0;i < this._missionlist.length;i++){
		if(!this._missionlist[i].iscompleted()){
			allmission.push(this._missionlist[i]);
		}
	}
    return allmission;
};
//获取所有已完成任务
Game_Party.prototype.getcmission = function(){
	var allmission = [];
	for(var i = 0;i < this._missionlist.length;i++){
		if(this._missionlist[i].iscompleted()){
			allmission.push(this._missionlist[i]);
		}
	}
    return allmission;
};
//获取绘图行数
Game_Party.prototype.getmissiondrawline = function(){
	var templine = 0;
	var tempmission = this.getallmission();
	templine += tempmission.length;
	for(var i = 0;i < tempmission.length;i++){
		if(tempmission[i].description !== ''){
			templine++;
		}
		templine += tempmission[i].childs.length;
	}
	return templine;
};
// ======================================================================
// * Sprite_Character
// ======================================================================
Sprite_Character.prototype.Lagomoro_Mission_old_initialize = Sprite_Character.prototype.initialize;
Sprite_Character.prototype.initialize = function(character) {
    this.Lagomoro_Mission_old_initialize(character);
    if (character instanceof Game_Event) {
        var datas = character.event().note.match(/\<NPC:[^,]*,#[0-9a-f]{6},[-0-9]*,[-0-9]*\>/i);
        if (datas !== null) {
			datas = datas[0].slice(5,datas[0].length-1).split(',');
            this.drawNPC(datas[0].toString(),datas[1].toString(),parseInt(datas[2]),parseInt(datas[3]));
        };
        datas = character.event().note.match(/\<ICO:[0-9]*,[-0-9]*,[-0-9]*\>/i);
        if (datas !== null) {
			datas = datas[0].slice(5,datas[0].length-1).split(',');
            this.drawIco(parseInt(datas[0]),parseInt(datas[1]),parseInt(datas[2]));			    
        };
	    datas = character.event().note.match(/\<MIS:[^,]*,[^,]*,#[0-9a-f]{6},[-0-9]*,[-0-9]*\>/ig);
        if (datas !== null) {
			for(var i = 0;i < datas.length;i++){
			    var data = datas[i].slice(5,datas[i].length-1).split(',');
				var mission = data[0].toString();
				var child = data[1].toString();
				var havmis = $gameParty.findmissionindexbyid(mission)!==null;
				var miscop = $gameParty.ismissioncomplete(mission);
				var miscod = $gameParty.ismissioncompleted(mission);
				var havchi = $gameParty.ismissionchildexist(mission,child);
				var chicop = $gameParty.ismissionchildcomplete(mission,child);
				var chicod = $gameParty.ismissionchildcompleted(mission,child);
	            if((child===''&&(!havmis||(havmis&&!miscop&&!miscod)||(havmis&&miscop&&!miscod)))||(child!==''&&!miscod&&((havchi&&!chicop&&!chicod)||(havchi&&chicop&&!chicod)))){
                    this.drawMis(mission,child,data[2].toString(),parseInt(data[3]),parseInt(data[4]));
					break;
	            }else if(i === datas.length - 1){
					this.drawMisempty(parseInt(data[3]),parseInt(data[4]));
					break;
				}
			}
        };
    };
};
Sprite_Character.prototype.drawNPC = function(name,color,xadd,yadd) {
    this._NPCname = new Sprite();
    this._NPCname.bitmap = new Bitmap(100, 20);
    this._NPCname.bitmap.fontSize = 18;
    this._NPCname.bitmap.textColor = color;
    this._NPCname.bitmap.drawText(name, 0, 0, 100, 20, 'center');
    this._NPCname.anchor.x = 0.5;
    this._NPCname.anchor.y = 1;
    this._NPCname.x = xadd;
    this._NPCname.y = yadd;
    this.addChild(this._NPCname);
}
Sprite_Character.prototype.drawIco = function(iconIndex,xadd,yadd) {
    this._NPCico = new Sprite();
    this._NPCico.bitmap = new Bitmap(32, 32);
    var icos = ImageManager.loadSystem('IconSet');
    var pw = Window_Base._iconWidth;
    var ph = Window_Base._iconHeight;
    var sx = iconIndex % 16 * pw;
    var sy = Math.floor(iconIndex / 16) * ph;
    this._NPCico.bitmap.blt(icos, sx, sy, pw, ph, 0, 0);
    this._NPCico.anchor.x = 0.5;
    this._NPCico.anchor.y = 1;
    this._NPCico.x = xadd;
    this._NPCico.y = yadd;
    this.addChild(this._NPCico);
}
Sprite_Character.prototype.drawMis = function(mission,child,color,xadd,yadd) {
	var havmis = $gameParty.findmissionindexbyid(mission)!==null;
	var miscop = $gameParty.ismissioncomplete(mission);
	var miscod = $gameParty.ismissioncompleted(mission);
	var havchi = $gameParty.ismissionchildexist(mission,child);
	var chicop = $gameParty.ismissionchildcomplete(mission,child);
	var chicod = $gameParty.ismissionchildcompleted(mission,child);
	this._NPCmis = new Sprite();
    this._NPCmis.bitmap = new Bitmap(32, 32);
    this._NPCmis.bitmap.fontSize = 32;
    this._NPCmis.bitmap.textColor = ((havmis&&!miscop&&!miscod)||(havchi&&!chicop&&!chicod) ? '#999999' : color);
    this._NPCmis.bitmap.drawText((!havmis ? '？' : '！'), 0, 0, 32, 32, 'center');
    this._NPCmis.anchor.x = 0.5;
    this._NPCmis.anchor.y = 1;
    this._NPCmis.x = xadd;
    this._NPCmis.y = yadd;
    this.addChild(this._NPCmis);
};
Sprite_Character.prototype.drawMisempty = function(xadd,yadd) {
	this._NPCmis = new Sprite();
    this._NPCmis.bitmap = new Bitmap(32, 32);
    this._NPCmis.bitmap.fontSize = 32;
    this._NPCmis.anchor.x = 0.5;
    this._NPCmis.anchor.y = 1;
    this._NPCmis.x = xadd;
    this._NPCmis.y = yadd;
    this.addChild(this._NPCmis);
};
Sprite_Character.prototype.Lagomoro_Mission_update = Sprite_Character.prototype.update;
Sprite_Character.prototype.update = function() {
    this.Lagomoro_Mission_update();
	if(this._NPCmis){
        this.updatemis();
	}
};
Sprite_Character.prototype.updatemis = function() {
	this._NPCmis.bitmap.clear();
    var datas = this._character.event().note.match(/\<MIS:[^,]*,[^,]*,#[0-9a-f]{6},[-0-9]*,[-0-9]*\>/ig);
    if (datas !== null) {
	    for(var i = 0;i < datas.length;i++){
			var data = datas[i].slice(5,datas[i].length-1).split(',');
			var mission = data[0].toString();
			var child = data[1].toString();
			var havmis = $gameParty.findmissionindexbyid(mission)!==null;
			var miscop = $gameParty.ismissioncomplete(mission);
			var miscod = $gameParty.ismissioncompleted(mission);
			var havchi = $gameParty.ismissionchildexist(mission,child);
			var chicop = $gameParty.ismissionchildcomplete(mission,child);
			var chicod = $gameParty.ismissionchildcompleted(mission,child);
	        if((child===''&&(!havmis||(havmis&&!miscop&&!miscod)||(havmis&&miscop&&!miscod)))||(child!==''&&!miscod&&((havchi&&!chicop&&!chicod)||(havchi&&chicop&&!chicod)))){
				this._NPCmis.bitmap.textColor = ((havmis&&!miscop&&!miscod)||(havchi&&!chicop&&!chicod) ? '#999999' : data[2].toString());
                this._NPCmis.bitmap.drawText((!havmis ? '？' : '！'), 0, 0, 32, 32, 'center');
			    this._NPCmis.x = parseInt(data[3]);
                this._NPCmis.y = parseInt(data[4]);
				break;
	        }
		}
    };
};
// ======================================================================
// * Scene_Base
// ======================================================================
Scene_Base.prototype.Lagomoro_Mission_old_updateChildren = Scene_Base.prototype.updateChildren;
Scene_Base.prototype.updateChildren = function() {
    this.Lagomoro_Mission_old_updateChildren();
    $gameParty.testallmission();
};
// ======================================================================
// * Scene_Lagomoro_Mission
// ======================================================================
function Scene_Lagomoro_Mission() {this.initialize.apply(this, arguments);}
Scene_Lagomoro_Mission.prototype = Object.create(Scene_MenuBase.prototype);
Scene_Lagomoro_Mission.prototype.constructor = Scene_Lagomoro_Mission;
Scene_Lagomoro_Mission.prototype.initialize = function() {
    Scene_MenuBase.prototype.initialize.call(this);
};
Scene_Lagomoro_Mission.prototype.create = function() {
	Scene_MenuBase.prototype.create.call(this);
	this.createHelpWindow();
	this.createItemWindow();
};
Scene_Lagomoro_Mission.prototype.createHelpWindow = function() {
    this._helpWindow = new Lagomoro_Mission_Window_Help(200, 0, Graphics.boxWidth - 200, Graphics.boxHeight);
    this.addWindow(this._helpWindow);
};
Scene_Lagomoro_Mission.prototype.createItemWindow = function() {
    this._itemWindow = new Lagomoro_Mission_Window_ItemList(0, 0, 200, Graphics.boxHeight);
    this._itemWindow.setHelpWindow(this._helpWindow);
    this._itemWindow.setHandler('cancel', this.popScene.bind(this));
    this.addWindow(this._itemWindow);
	this._itemWindow.refresh();
	this._itemWindow.activate();
	this._itemWindow.selectLast();
};
// ======================================================================
// * Lagomoro_Mission_Window_ItemList
// ======================================================================
function Lagomoro_Mission_Window_ItemList() {this.initialize.apply(this, arguments);}
Lagomoro_Mission_Window_ItemList.prototype = Object.create(Window_Selectable.prototype);
Lagomoro_Mission_Window_ItemList.prototype.constructor = Lagomoro_Mission_Window_ItemList;
Lagomoro_Mission_Window_ItemList.prototype.initialize = function(x, y, width, height) {
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this._data = [];
};
Lagomoro_Mission_Window_ItemList.prototype.standardFontSize = function() {return 18;};
Lagomoro_Mission_Window_ItemList.prototype.maxCols = function() {return 1;};
Lagomoro_Mission_Window_ItemList.prototype.maxItems = function() {return this._data ? this._data.length : 1;};
Lagomoro_Mission_Window_ItemList.prototype.item = function() {
    var index = this.index();
    return this._data && index >= 0 ? this._data[index] : null;
};
Lagomoro_Mission_Window_ItemList.prototype.makeItemList = function() {
	this._data = $gameParty.getucmission().reverse().concat($gameParty.getcmission().reverse());
};
Lagomoro_Mission_Window_ItemList.prototype.selectLast = function() {this.select(0);};
Lagomoro_Mission_Window_ItemList.prototype.drawItem = function(index) {
    var item = this._data[index];
    if (item) {
        var rect = this.itemRect(index);
        rect.width -= this.textPadding();
        this.changePaintOpacity(!item.completed);
		this.contents.textColor = item.color;
        this.drawText(item.name, rect.x + 10, rect.y, rect.width - 20);
        this.changePaintOpacity(1);
    }
};
Lagomoro_Mission_Window_ItemList.prototype.updateHelp = function() {this.setHelpWindowItem(this.item());};
Lagomoro_Mission_Window_ItemList.prototype.refresh = function() {
    this.makeItemList();
    this.createContents();
    this.drawAllItems();
};
// ======================================================================
// * Lagomoro_Mission_Window_Help
// ======================================================================
function Lagomoro_Mission_Window_Help() {this.initialize.apply(this, arguments);}
Lagomoro_Mission_Window_Help.prototype = Object.create(Window_Base.prototype);
Lagomoro_Mission_Window_Help.prototype.constructor = Lagomoro_Mission_Window_Help;
Lagomoro_Mission_Window_Help.prototype.initialize = function(x, y, width, height) {
    Window_Base.prototype.initialize.call(this, x, y, width, height);
	this._item = null;
};
Lagomoro_Mission_Window_Help.prototype.standardPadding = function() {return 12;};
Lagomoro_Mission_Window_Help.prototype.standardFontSize = function() {return 18;};
Lagomoro_Mission_Window_Help.prototype.setItem = function(item) {
    if (this._item !== item) {
	    this._item = item;
	    this.refresh();
	}
};
Lagomoro_Mission_Window_Help.prototype.processNormalCharacter = function(textState) {
    var c = textState.text[textState.index];
    var w = this.textWidth(c);
	if (this.width - 2 * this.standardPadding() - textState.x >= w){
        this.contents.drawText(c, textState.x, textState.y, w * 2, textState.height);
		textState.index++;
        textState.x += w;
    }else{
        this.processNewLine(textState);
		textState.index--;
        this.processNormalCharacter(textState);
	}
};
Lagomoro_Mission_Window_Help.prototype.refresh = function() {
    this.contents.clear();
	var item = this._item;
	if(item){
		var y = 0;
		this.contents.textColor = item.color;
		this.contents.paintOpacity = item.completed ? this.translucentOpacity() : 255;
	    this.drawText(item.name + (item.completed ? '（已完成）' : ''), 6, y, this.width - 2*this.standardPadding() - 6, 'center');
		y += 36;
		this.contents.paintOpacity = 255;
		this.contents.textColor = '#ffffff';
		
        this.drawTextEx(item.description, 6, y);
		
		y = this.height/5*2 - 10;
		
		for(var i = 0;i < item.childs.length;i++){
		    this.contents.textColor = (item.childs[i][5] ? '#999999' : '#ffffff');
		    this.drawText('▪ ' + item.childs[i][1], 6, y, this.width - 2*this.standardPadding() - 6, 'left');
            this.drawText(item.getchildratebyindex(i), 6, y, this.width - 2*this.standardPadding() - 8, 'right');
		    this.contents.textColor = '#ffffff';
		    y += this.standardFontSize() + this.textPadding();
	    }
		
		y = this.height/5*4 - 5;
		this.drawText('任务奖励：', 6, y, this.width - 2*this.standardPadding() - 6, 'left');
		y += this.standardFontSize()+this.textPadding() + 3;
		
		var drawints = ['0 ' + TextManager.expA, '0' + TextManager.currencyUnit];
		for(var i = 0;i < item.reward.length;i++){
            if(item.reward[i][0] === 'EXP'){
				drawints[0] = item.reward[i][1] + ' ' + TextManager.expA;
            }else if(item.reward[i][0] === 'MONEY'){
                drawints[1] = item.reward[i][1] + TextManager.currencyUnit;
            }else if(item.reward[i][0] === 'ITEM'){
                drawints.splice(2,0,$dataItems[item.reward[i][1]].name + '*' + item.reward[i][2]);
            }else if(item.reward[i][0] === 'WEAPON'){
                drawints.splice(2,0,$dataWeapons[item.reward[i][1]].name + '*' + item.reward[i][2]);
            }else if(item.reward[i][0] === 'ARMOR'){
                drawints.splice(2,0,$dataArmors[item.reward[i][1]].name + '*' + item.reward[i][2]);
            }
        }
		
		var ssl = '';
		for(var i = 0;i < 2;i++){
            ssl += drawints[i];
			if(i < 1){ssl += '    ';}
        }
		this.drawText(ssl, 8, y, this.width - 2*this.standardPadding() - 6, 'left');
		y += this.standardFontSize()+this.textPadding();

		ssl = '';
		for(var i = 2;i < drawints.length;i++){
            ssl += drawints[i];
			if(i < drawints.length - 1){ssl += '    ';}
        }
		this.drawText(ssl, 8, y, this.width - 2*this.standardPadding() - 6, 'left');
	}
};
// ======================================================================
// * Scene_Menu
// ======================================================================
Scene_Menu.prototype.Lagomoro_old_createCommandWindow = Scene_Menu.prototype.createCommandWindow;
Scene_Menu.prototype.createCommandWindow = function() {
	this.Lagomoro_old_createCommandWindow();
    this._commandWindow.setHandler('mission',   this.command_Lagomoro_Mission.bind(this));
};
Scene_Menu.prototype.command_Lagomoro_Mission = function() {
    SceneManager.push(Scene_Lagomoro_Mission);
};
// ======================================================================
// * Window_MenuCommand
// ======================================================================
Window_MenuCommand.prototype.Lagomoro_old_addOriginalCommands = Window_MenuCommand.prototype.addOriginalCommands;
Window_MenuCommand.prototype.addOriginalCommands = function() {
	this.Lagomoro_old_addOriginalCommands();
	this.addCommand('任务', 'mission', this.areMainCommandsEnabled());
};

// ======================================================================
// * 优兔插件登记：作者Lagomoro<优兔>[6R:rpg-sheep][度娘:优加星爱兔子]
// ----------------------------------------------------------------------
// * Lagomoro_Toast核心
// ======================================================================
// ======================================================================
// * Scene_Base
// ======================================================================
Scene_Base.prototype.Lagomoro_Toast_updateChildren = Scene_Base.prototype.updateChildren;
Scene_Base.prototype.updateChildren = function() {
    this.Lagomoro_Toast_updateChildren();
    if($gameTemp){$gameTemp.updatetoast();}
};
// ======================================================================
// * Game_Temp
// ======================================================================
Game_Temp.prototype.Lagomoro_Toast_old_initialize = Game_Temp.prototype.initialize;
Game_Temp.prototype.initialize = function() {
    this.Lagomoro_Toast_old_initialize();
    this._toastlist = [];
};
Game_Temp.prototype.toast = function(text, color, func) {
	var sprite = new Sprite();
	sprite.bitmap = new Bitmap(Graphics.boxWidth, 36);
	sprite.bitmap.textColor = (color ? color : '#ffff00');
	sprite.bitmap.fontSize = 24;

	var width = sprite.bitmap.measureTextWidth(text) + 12;
	var colorm = (PluginManager.parameters('Lagomoro_Mission')['windowcolor']||'rgba(0, 0, 0, 0.4)');
	
    sprite.bitmap.fillRect(0, 3, width, 30, colorm);
    sprite.bitmap.fillRect(width, 5, 2, 26, colorm);
    sprite.bitmap.fillRect(width + 2, 7, 2, 22, colorm);
    sprite.bitmap.fillRect(width + 4, 9, 2, 18, colorm);
    sprite.bitmap.fillRect(width + 6, 11, 2, 14, colorm);
    sprite.bitmap.fillRect(width + 8, 13, 2, 10, colorm);
    sprite.bitmap.fillRect(width + 10, 15, 2, 6, colorm);
    sprite.bitmap.fillRect(width + 12, 17, 2, 2, colorm);

	sprite.bitmap.drawText(text, 6, 6, Graphics.boxWidth, 24, 'left');

	sprite._time = 0;
    sprite._tempy = 0;
    sprite.alpha = 0;
	this.alltoastmove(32);
	this._toastlist.push(sprite);
	SceneManager._scene.addChild(this._toastlist[this._toastlist.length - 1]);
};
Game_Temp.prototype.toastShow = function() {
    for(var i = 0;i < this._toastlist.length;i++){
        this._toastlist[i]._time = 0;
        SceneManager._scene.addChild(this._toastlist[i]);
    }
};
Game_Temp.prototype.alltoastmove = function(height) {
    for(var i = 0;i < this._toastlist.length;i++){
		this._toastlist[i]._tempy += height;
	}
};
Game_Temp.prototype.updatetoast = function() {
    for(var i = 0;i < this._toastlist.length;i++){
		if(this._toastlist[i]._time < 60){
			this._toastlist[i].alpha += 1/60;
		}else if(this._toastlist[i]._time < 660){
        }else if(this._toastlist[i]._time < 720){
            this._toastlist[i].alpha -= 1/60;
        }
        this._toastlist[i]._time ++;
        if(this._toastlist[i]._tempy > 0){
            this._toastlist[i].y ++;
            this._toastlist[i]._tempy --;
        }
        if(this._toastlist[i].y + this._toastlist[i].height > Graphics.boxHeight - 350 && this._toastlist.length > 1){
            if(this._toastlist[i]._time < 660){
                this._toastlist[i]._time = 660;
            }else if(this._toastlist[i]._time >= 720){
                SceneManager._scene.removeChild(this._toastlist.splice(i,1)[0]);
                i--;
			}
        }
	}
};