/*:
 * @plugindesc limit equip and item level
 * @author tiannian dtiannian@aliyun.com
 *
 * @param equip
 * @desc open equip's level limit 
 * @default false
 *
 * @param item
 * @desc open item's level limit 
 * @default false
 *
 * @help
 * Equip and Item note:
 * <LimitLevel:level>       #this item must greater than player level
 */

/*:zh
 * @plugindesc 限制装备和物品的使用等级
 * @author tiannian dtiannian@aliyun.com
 *
 * @param equip
 * @desc 开启装备等级限制 
 * @default false
 *
 * @param item
 * @desc 开启物品等级限制
 * @default false
 *
 * @help
 * Equip and Item note:
 * <LimitLevel:level>       #当玩家等级大于level时才可以使用
 */

(function(){
    var parameters = PluginManager.parameters('LimitLevel');
    var openEquipLimit = parameters['equip'] || false;
    var openItemLimit = parameters['item'] || false;
    if (openEquipLimit) {
        var testEq = Game_Actor.prototype.changeEquip;
        Game_Actor.prototype.changeEquip = function(slotId, item) {
            if (item === null) {
                //卸下装备
                testEq.call(this,slotId, item);
            } else {
                //检查装备等级
                var ll = item.meta.LimitLevel || 0;
                if (ll <= this._level) {
                    testEq.call(this,slotId, item);
                }
            }
        };
    }

    var Game_System_GetZzySEFEQLevel = Game_System.prototype.GetZzySEFEQLevel;
    Game_System.prototype.GetZzySEFEQLevel = function(eq) {
        var currentLevel = Game_System_GetZzySEFEQLevel.call(this, eq);
        if ((SceneManager._scene instanceof Scene_StoreOnline) && (SceneManager._scene._buyWindow._data.indexOf(eq) >= 0)) currentLevel = eq.zzySEF.streng;
        return currentLevel;
    };

    if (openItemLimit) {
        var testIt = Scene_ItemBase.prototype.useItem;
        Scene_ItemBase.prototype.useItem = function(target) {
            var ll = this.item().meta.LimitLevel || 0;
            if (ll <= this.user()._level) {
                testIt.call(this,target);
            }
        };
    }

    var Scene_StoreOnline_sendItem = Scene_StoreOnline.prototype.sendItem;
    Scene_StoreOnline.prototype.sendItem = function(price) {
	    if (this._item.zzySEF.typeId == 2) this._item.zzySEF.streng = $gameSystem._ZzySEFWeaponLevel[this._item.id];
	    else if (this._item.zzySEF.typeId == 3) this._item.zzySEF.streng = $gameSystem._ZzySEFArmorLevel[this._item.id];
        Scene_StoreOnline_sendItem.call(this, price);
    };

    var Scene_StoreOnline_gainItem = Scene_StoreOnline.prototype.gainItem;
    Scene_StoreOnline.prototype.gainItem = function(item) {
        Scene_StoreOnline_gainItem.call(this, item);
        if (!!item.baseItemId && !!item.etypeId) {
            if (item.etypeId == 1) $gameSystem._ZzySEFWeaponLevel[item.id] = item.zzySEF.streng;
            else $gameSystem._ZzySEFArmorLevel[item.id] = item.zzySEF.streng;
        }
    };

})()