/*:
 * @plugindesc v1.06 LiuYue_EquipTime 装备限时
 * @author 流逝的岁月
 *
 * @help
 * ============================================================================
 * 介绍
 * ============================================================================
 *
 * 给装备添加标签,对获取的装备显示存在,结束后销毁
 *
 * 以下是可能会用到的脚本信息
 * Zzy.ETF.GetTime(typeId,id)                 //这会返回指定装备所剩余的时长,单位为秒,若未有该装备,则返回undefined
 *                                            typeId:可输入'item'或'weapon'或'armor' 代表这是一个武器还是护甲
 *                                            id:输入装备在数据库中的ID值
 *
 * ps:请将此插件放在拥有兼容性插件之下
 *
 *
 *
 * 在数据库 - 武器/护甲  中添加以下备注
 *
 * <ZzyETF Time: x>                          //使装备为期限物品,时长为x秒
 * <ZzyETF Common: x>                        //装备消失后会调用的公共事件
 * <ZzyETF MapPrefix: x>                     //文本前缀
 * <ZzyETF UseItemCommon: x1 x2>            //使用物品后,将会开启延时调用公共事件,x1填写公共事件ID   x2填写等待的时长,单位为秒
 *
 *
 *
 *
 *
 *--------------------------------------------------------------------------
 *
 *
 *以下是一些插件指令的信息
 *---------------------Plugin Command--------------------
 *
 * ZzyETF IsShow x(true/false)                            //可填写true或是false,这会使所有计时器的显示开启或关闭
 *
 *
 *
 *
 *
 *
 *
 *
 * @param ---设置---
 * @default
 *
 * @param IsShow
 * @text 是否显示时间
 * @parent ---设置---
 * @type boolean
 * @on YES
 * @off NO
 * @desc 这是在地图上显示剩余时长
 * YES - true     NO - false
 * @default true
 *
 * @param TextSize
 * @text 显示文字大小
 * @parent ---设置---
 * @type Number
 * @desc 这是显示文本的大小,默认值为24
 * @default 24
 *
 * @param TextColor
 * @text 显示文字颜色
 * @parent ---设置---
 * @type Text
 * @desc 这是显示文本的颜色,可以使用两种格式的颜色码 #000000~#ffffff 或者 rgba(0~255,0~255,0~255,0~1),默认值天蓝色
 * @default #009BFF
 *
 * @param BorderSize
 * @text 显示文字边框宽度
 * @parent ---设置---
 * @type Number
 * @desc 这是显示文本的边框宽度,默认值为2
 * @default 2
 *
 * @param BorderColor
 * @text 显示文字边框颜色
 * @parent ---设置---
 * @type Text
 * @desc 这是显示文本边框的颜色,可以使用两种格式的颜色码 #000000~#ffffff 或者 rgba(0~255,0~255,0~255,0~1),默认值半透明黑色
 * @default rgba(0,0,0,0.5)
 *
 * @param ---队列---
 * @default
 *
 * @param ListX
 * @text 队列开始X位置
 * @parent ---队列---
 * @type Text
 * @desc 队列开始在屏幕上X的位置
 * @default Graphics.boxWidth/2
 *
 * @param ListY
 * @text 队列开始Y位置
 * @parent ---队列---
 * @type Text
 * @desc 队列开始在屏幕上Y的位置
 * @default 14
 *
 * @param ListOfx
 * @text 队列压入偏移X
 * @parent ---队列---
 * @type Text
 * @desc 队列开始在屏幕上X的位置
 * @default 0
 *
 * @param ListOfy
 * @text 队列压入偏移Y
 * @parent ---队列---
 * @type Text
 * @desc 队列开始在屏幕上Y的位置
 * @default 30
 *
 *
 *
 *
 * @param ---进制---
 * @default
 *
 * @param SecondSystem
 * @text 秒进制
 * @parent ---进制---
 * @type Number
 * @desc 秒钟进制格式
 * @default 60
 *
 * @param MinuteSystem
 * @text 分进制
 * @parent ---进制---
 * @type Number
 * @desc 分钟进制格式
 * @default 60
 *
 * @param TimeForamt
 * @text 秒进制
 * @parent ---进制---
 * @type Number
 * @desc 显示事件文本的格式  %1会被替换为秒数   %2会被替换为分数   %3会被替换为时数
 * @default %3:%2:%1
 *
 *
 *
 * @param ---兼容设置---
 * @default
 *
 * @param CompYEPItemCore
 * @parent ---兼容设置---
 * @text YEP装备核心
 * @type boolean
 * @on YES
 * @off NO
 * @desc 是否有安装YEP_ItemCore这个插件，如果有请打开这个选项来确保兼容性
 * YES - true     NO - false
 * @default true
 *
 *
 * @param CompJHMWMessage
 * @parent ---兼容设置---
 * @text 江湖梦网消息
 * @type boolean
 * @on YES
 * @off NO
 * @desc 是否有安装JHMW_Message这个插件，如果有请打开这个选项来确保兼容性
 * YES - true     NO - false
 * @default true
 *
 *
 */


var LiuYue = LiuYue || {};
LiuYue.LiuYue_EquipTime = true;//插件启动

var Zzy = Zzy || {};
Zzy.ETF = Zzy.ETF || {};
Zzy.ETF.version = 1.06;
Zzy.Parameters = PluginManager.parameters('LiuYue_EquipTime');
Zzy.Param = Zzy.Param || {};

Zzy.Param.ETFCompYEPItemCore = eval(String(Zzy.Parameters['CompYEPItemCore']));//YEP装备核心
Zzy.Param.ETFCompJHMWMessage = eval(String(Zzy.Parameters['CompJHMWMessage']));//江湖梦网消息


Zzy.Param.ETFIsShow = eval(String(Zzy.Parameters['IsShow']));//升级消耗道具ID
Zzy.Param.ETFTextSize = parseInt(Zzy.Parameters['TextSize']);//显示文字大小
Zzy.Param.ETFTextColor = String(Zzy.Parameters['TextColor']);//显示文字颜色
Zzy.Param.ETFBorderSize = parseInt(Zzy.Parameters['BorderSize']);//显示文字大小
Zzy.Param.ETFBorderColor = String(Zzy.Parameters['BorderColor']);//显示文字边框颜色
Zzy.Param.ETFSecondSystem = parseInt(Zzy.Parameters['SecondSystem']);//秒进制
Zzy.Param.ETFMinuteSystem = parseInt(Zzy.Parameters['MinuteSystem']);//分进制
Zzy.Param.ETFTimeForamt = String(Zzy.Parameters['TimeForamt']);//时间格式

Zzy.Param.ETFListX = String(Zzy.Parameters['ListX']);//队列开始X位置 
Zzy.Param.ETFListY = String(Zzy.Parameters['ListY']);//队列开始Y位置 
Zzy.Param.ETFListOfx = Number(Zzy.Parameters['ListOfx']);//队列压入偏移X 
Zzy.Param.ETFListOfy = Number(Zzy.Parameters['ListOfy']);//队列压入偏移Y 




Zzy.ETF.IsHaveYEPItemCore = function()//是否有安装其他需兼容的插件
{
	if(Imported && Imported.YEP_ItemCore)
	{
		return true;
	}
	return false;
}


Zzy.ETF.IsHaveJHMWMessage = function()//是否有安装其他需兼容的插件
{
	if(Imported && Imported.JHMW_Message)
	{
		return true;
	}
	return false;
}

//==================================================================
//Game_System
//==================================================================

Zzy.ETF.Game_System_initialize = Game_System.prototype.initialize;
Game_System.prototype.initialize = function() 
{
    Zzy.ETF.Game_System_initialize.call(this);
	this.ZzyETFInitData();//初始化数据
};

Game_System.prototype.ZzyETFInitData = function()
{
	this._ZzyETFEquipList = [];
	this._ZzyETFCommonList = [];//公共事件队列
	this._ZzyETFUseItemList = [];
	
	this._ZzyETFCommonIndex = 0;//下标
	
	this._ZzyETFIsShow = Zzy.Param.ETFIsShow;
	
}

Game_System.prototype.GetZzyETFEquipList = function()
{
	if(this._ZzyETFEquipList === undefined)
	{this._ZzyETFEquipList = [];}
	return this._ZzyETFEquipList;
}

Game_System.prototype.GetZzyETFCommonList = function()
{
	if(this._ZzyETFCommonList === undefined)
	{this._ZzyETFCommonList = [];}
	return this._ZzyETFCommonList;
}

Game_System.prototype.GetZzyETFUseItemList = function()
{
	if(this._ZzyETFUseItemList === undefined)
	{this._ZzyETFUseItemList = [];}
	return this._ZzyETFUseItemList;
}


Game_System.prototype.GetZzyETFIsShow = function()
{
	if(this._ZzyETFIsShow === undefined)
	{this._ZzyETFIsShow = Zzy.Param.ETFIsShow;}
	return this._ZzyETFIsShow;
}

Game_System.prototype.SetZzyETFIsShow = function(enable)
{
	this._ZzyETFIsShow = enable;
}

Game_System.prototype.IsZzyETFTime = function(eq)
{
	if(!eq.zzyETF)return false;
	return eq.zzyETF.time ? true : false;
}

Game_System.prototype.PushZzyETFCommonList = function(commonId)
{
	var cList = this.GetZzyETFCommonList();
	cList.push(commonId);
}

Game_System.prototype.ClearZzyETFCommonList = function()
{
	this._ZzyETFCommonList = [];//公共事件队列
	this._ZzyETFCommonIndex = 0;//下标	
}

Game_System.prototype.MakeZzyETFEquip = function(eq)//制作计数武器
{
	var info = {};
	if(DataManager.isItem(eq))
	{info.typeId = 1;}
	else if(DataManager.isWeapon(eq))
	{info.typeId = 2;}
	else if(DataManager.isArmor(eq))
	{info.typeId = 3;}
	info.id = eq.id;
	info.yxj = eq.meta.ZzyYxj || 0;
	info.atypeId = eq.atypeId;
	info.etypeId = eq.etypeId;
	info.time = Zzy.ETF.EvalTranslateOfEquip(eq.zzyETF.time,eq);
	info.commonId = eq.zzyETF.commonId ? eq.zzyETF.commonId : 0;
	info.prefix = eq.zzyETF.prefix ? eq.zzyETF.prefix : '';
	info.eq = eq;
	//$gameParty.gainItemLose(eq, -1);
	return info;
}



Game_Party.prototype.gainItemLose = function(item, amount, includeEquip) {
    var container = this.itemContainer(item);
    if (container) {
        var lastNumber = this.numItems(item);
        var newNumber = lastNumber + amount;
        container[item.id] = newNumber.clamp(0, this.maxItems(item));
        if (container[item.id] === 0) {
            delete container[item.id];
        }
        if (includeEquip && newNumber < 0) {
            this.discardMembersEquip(item, -newNumber);
        }
        $gameMap.requestRefresh();
    }
};

Game_System.prototype.PushZzyETFList = function(info)
{
	var eList = this.GetZzyETFEquipList();

	for(var i=0;i<eList.length;i++)
	{
		var eqInfo = eList[i];
		var eati = eqInfo.atypeId;var iati = info.atypeId;
		var eeti = eqInfo.etypeId;var ieti = info.etypeId;
		var eyxj = eqInfo.yxj || 0;var iyxj = info.yxj || 0;
		if(eqInfo.typeId == info.typeId && eati == iati && eeti == ieti)
		{
			eqInfo.id = info.id;
			eqInfo.yxj = info.yxj || 0;
			eqInfo.atypeId = info.atypeId;
			eqInfo.etypeId = info.etypeId;
			eqInfo.time = info.time;
			eqInfo.commonId = info.commonId;
			eqInfo.prefix = info.prefix;
			$gameParty.gainItemLose(eqInfo.eq, -1, true);
			eqInfo.eq = info.eq
			return false;
		}
		
	}
	eList.push(info);
	return true;
}

Game_System.prototype.PushZzyETFListRefresh = function(info)
{
	var eList = this.GetZzyETFEquipList();
	eList.push(info);
	return true;	
}

Game_System.prototype.ClearZzyETFList = function()
{
	this._ZzyETFEquipList = [];
	this._ZzyETFUseItemList = [];
}


Game_System.prototype.DeleteZzyETFListNull = function()
{
	//获得装备计时
	var eList = this.GetZzyETFEquipList();
	var tempList = [];
	for(var i=0;i<this._ZzyETFEquipList.length;i++)
	{
		if(this._ZzyETFEquipList[i])
		{
			tempList.push(this._ZzyETFEquipList[i]);
		}
	}
	this._ZzyETFEquipList = tempList;
	
	
	//使用道具计时
	var eList2 = this.GetZzyETFUseItemList();
	var tempList2 = [];
	for(var i=0;i<this._ZzyETFUseItemList.length;i++)
	{
		if(this._ZzyETFUseItemList[i])
		{
			tempList2.push(this._ZzyETFUseItemList[i]);
		}
	}
	this._ZzyETFUseItemList = tempList2;

}


Game_System.prototype.GetZzyETFSurplusTime = function(eq)//返回装备剩余时长
{
	var eList = this.GetZzyETFEquipList();
	for(var i=0;i<eList.length;i++)
	{
		var tInfo = eList[i];
		
		if(eq.id === tInfo.id)
		{
			if(DataManager.isItem(eq) && tInfo.typeId === 1)
			{
				return tInfo.time;
			}
			else if(DataManager.isWeapon(eq) && tInfo.typeId === 2)
			{
				return tInfo.time;
			}
			else if(DataManager.isArmor(eq) && tInfo.typeId === 3)
			{
				return tInfo.time;
			}			
		}

	}
	return undefined;
}

Game_System.prototype.GetZzyETFSurplusTimeEx = function(typeId,id)
{
	var eList = this.GetZzyETFEquipList();
	
	for(var i=0;i<eList.length;i++)
	{
		var tInfo = eList[i];
		if(typeId === tInfo.typeId && id === tInfo.id)
		{
			return tInfo.time;
		}
	}
	return undefined;
}


Game_System.prototype.PushZzyETFUseItemCommond = function(info)
{
	var list = this.GetZzyETFUseItemList();
	
	//检测是否重复
	for(var i=0;i<list.length;i++)
	{
		if(list[i] && (list[i].id === info.itemId))
		{
			//刷新计时
			list[i].time = info.comTime;
			return false;
		}
	}
	var tInfo = {};
	tInfo.id = info.itemId;
	tInfo.comId = info.comId;
	tInfo.time = info.comTime;
	tInfo.isUseItem = true;//标记:用于做区分
	tInfo.prefix = info.prefix;
	list.push(tInfo);
	return true;
}



//=============================================================================
//Window_ZzyETFShow
//=============================================================================

function Window_ZzyETFShow() {
    this.initialize.apply(this, arguments);
}

Window_ZzyETFShow.prototype = Object.create(Window_Base.prototype);
Window_ZzyETFShow.prototype.constructor = Window_ZzyETFShow;

Window_ZzyETFShow.prototype.initialize = function() 
{
	var x = 0;
	var y = 0;
	var width = Graphics.boxWidth;
	var height = Graphics.boxHeight;
    Window_Base.prototype.initialize.call(this, x, y, width, height);

	this.opacity = 0;//透明度
	this.sprArr = [];//对象池
	this.sprIndex = [];//标记ID数组
	
	this.CreateTimeList();//创造队列
};

Window_ZzyETFShow.prototype.CreateTimeList = function()//创造时间队列
{
	var listArr = $gameSystem.GetZzyETFEquipList();
	var index = 0;
	for(var i=0;i<listArr.length;i++)
	{
		var info = listArr[i];
		if(info.time > 0)
		{this.PushList(info,index);index++;}
	}
	
	//压入延迟调用计时
	listArr = $gameSystem.GetZzyETFUseItemList();
	for(var i=0;i<listArr.length;i++)
	{
		var info = listArr[i];
		if(info.time > 0)
		{this.PushList(info,index);index++;}
	}
	
}

Window_ZzyETFShow.prototype.ClearTimeList = function()
{
	for(var i=0;i<this.sprArr.length;i++)
	{
		if(!this.sprArr[i].IsFree())
		{this.sprArr[i].Clear();}
	}
}



Window_ZzyETFShow.prototype.PushList = function(info,index)
{
	var spr = this.RequestSpr();
	spr.ResetInfo(info);//添加
	spr.SetPositionOfList(index);//设置位置

}



Window_ZzyETFShow.prototype.GetListSpr = function(mapId,evId)
{
	for(var i=0;i<this.sprArr.length;i++)
	{
		if(!this.sprArr[i].IsFree() && 
		this.sprArr[i].isList && 
		this.sprArr[i].mapId === mapId && 
		this.sprArr[i].evId === evId)
		{
			return this.sprArr[i];
		}
	}
	return undefined;
}



Window_ZzyETFShow.prototype.RequestSpr = function()//申请一个对象,通过对象池管理,同时判断是否申请重复
{

	for(var i=0;i<this.sprArr.length;i++)
	{
		if(this.sprArr[i].IsFree())
		{
			var spr = this.sprArr[i];
			return spr;
		}
	}
	var spr2 = new Sprite_ZzyETF();
	this.sprArr.push(spr2);
	this.addChild(spr2);
	return spr2;
}

Window_ZzyETFShow.prototype.update = function()
{
	Window_Base.prototype.update.call(this);
	
}


Window_ZzyETFShow.prototype.RefreshList = function()
{
	var listArr = $gameSystem.GetZzyETFListArr();
	for(var i=0;i<listArr.length;i++)
	{
		var info = listArr[i];
		var spr = this.GetListSpr(info.mapId,info.evId);
		if(spr){spr.SetPositionOfList(i);}//设置位置
		
	}		
}

//=============================================================================
//Sprite_ZzyETF
//=============================================================================
function Sprite_ZzyETF() 
{
    this.initialize.apply(this, arguments);
}

Sprite_ZzyETF.prototype = Object.create(Sprite.prototype);
Sprite_ZzyETF.prototype.constructor = Sprite_ZzyETF;

Sprite_ZzyETF.prototype.initialize = function() 
{
    Sprite.prototype.initialize.call(this);
	this.DefaultBitmap();
	this.time = 0;//剩余时长
	this.rTime = 0;//记录时间
	this.isRefresh = false;//等待刷新
	this.prefix = '';
	this.info = undefined;
	this.index = 0;
	
	this.isFree = false;
	this.anchor.x = 0.5;
	this.anchor.y = 0.5;
};

Sprite_ZzyETF.prototype.DefaultBitmap = function()
{
	var w = this.DefaultBitmapWidth();
	var h = this.DefaultBitmapHeight();
	this.bitmap = new Bitmap(w,h);
}

Sprite_ZzyETF.prototype.DefaultBitmapWidth = function()
{
	return Graphics.boxWidth;
}

Sprite_ZzyETF.prototype.DefaultBitmapHeight = function()
{
	return Zzy.Param.ETFTextSize + Zzy.Param.ETFBorderSize*2;
}

Sprite_ZzyETF.prototype.refresh = function()
{
	this.bitmap.clear();
	//绘制新的内容
	var tiETFr = Zzy.Param.ETFTimeForamt;
	//时分秒转换
	
	var second = this.info.time%Zzy.Param.ETFSecondSystem;
	var minute = Math.floor(this.info.time/Zzy.Param.ETFSecondSystem)%Zzy.Param.ETFMinuteSystem;
	var hour = Math.floor(Math.floor(this.info.time/Zzy.Param.ETFSecondSystem)/Zzy.Param.ETFMinuteSystem);

	tiETFr = tiETFr.format(Zzy.ETF.FixZeroStart(second,2),Zzy.ETF.FixZeroStart(minute,2),Zzy.ETF.FixZeroStart(hour,2));
	
	this.bitmap.fontSize = Zzy.Param.ETFTextSize;
	this.bitmap.textColor = Zzy.Param.ETFTextColor;
	this.bitmap.outlineWidth = Zzy.Param.ETFBorderSize;
	this.bitmap.outlineColor = Zzy.Param.ETFBorderColor;
	
	var tText = (this.info.prefix === '') ? tiETFr : this.info.prefix + tiETFr;


	this.bitmap.drawText(tText, Zzy.Param.ETFBorderSize, Zzy.Param.ETFBorderSize, this.bitmap.width, this.bitmap.height, 'center');

}


Sprite_ZzyETF.prototype.update = function()
{
	if(this.IsFree()){return};
	Sprite.prototype.update.call(this);
	if(!this.info){this.Clear();return;}
	
	this.updateTime();
	this.updateDraw();//更新绘制
}



Sprite_ZzyETF.prototype.SetPositionOfList = function(index)
{
	var tx = eval(Zzy.Param.ETFListX) + Zzy.Param.ETFListOfx*index;
	var ty = eval(Zzy.Param.ETFListY) + Zzy.Param.ETFListOfy*index;
	this.x = tx;
	this.y = ty;
	this.index = index;
}


Sprite_ZzyETF.prototype.SetPrefixText = function(prefix)
{
	this.prefix = prefix;
}

Sprite_ZzyETF.prototype.updateDraw = function()
{
	if(this.isRefresh)
	{
		this.isRefresh = false;
		this.refresh();//绘制		
	}
}


Sprite_ZzyETF.prototype.updateTime = function()
{
	if(this.rTime !== this.info.time)
	{
		this.rTime = this.info.time;
		this.isRefresh = true;
	}
	if(this.info.time <= 0)
	{
		this.Clear();//清理
		//需要刷新队列
		Zzy.ETF.RefreshList();//刷新队列
	}
}

Sprite_ZzyETF.prototype.Clear = function()
{
	this.isFree = true;
	this.visible = false;
}



Sprite_ZzyETF.prototype.updatePosition = function()
{
	this.x = eval(Zzy.Param.ETFListX);
	this.y = eval(Zzy.Param.ETFListY);
}

Sprite_ZzyETF.prototype.ResetInfo = function(info)
{
	this.visible = $gameSystem.GetZzyETFIsShow();
	this.info = info;
	this.prefix = '';
	this.isFree = false;
	this.rTime = -1;
	this.isRefresh = true;
}

Sprite_ZzyETF.prototype.IsFree = function()//是否无效
{
	return this.isFree;
}


//=============================================================================
//Game_Interpreter
//=============================================================================

Zzy.ETF.Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function(command, args)//插件命令
{
	Zzy.ETF.Game_Interpreter_pluginCommand.call(this,command,args);

	if(command === 'ZzyETF')
	{
		this.ZzyETFCommand(args);
	}
}

Game_Interpreter.prototype.ZzyETFCommand = function(args)
{
	var command = String(args[0]);
	
	switch(command)
	{   	
		case 'IsShow':
			var enable = eval(String(args[1]));
			Zzy.ETF.IsShow(enable);
		break;	
	}
}


//==================================================================
//Scene_Map
//==================================================================
Zzy.ETF.Scene_Map_start = Scene_Map.prototype.start;
Scene_Map.prototype.start = function()
{
	Zzy.ETF.Scene_Map_start.call(this);
	this.ZzyETFCreateWindow();
	this._cZzyETFFrame = 0;
	
}

Scene_Map.prototype.ZzyETFCreateWindow = function()//创建窗口
{
	this._zzyETFWindow = new Window_ZzyETFShow();
	this.addChild(this._zzyETFWindow);
	
	
}

Zzy.ETF.Scene_Map_update = Scene_Map.prototype.update;
Scene_Map.prototype.update = function()
{
	Zzy.ETF.Scene_Map_update.call(this);
	
	this.updateZzyETFList();
	this.updateZzyETFCommon();//刷新公共事件
}

Scene_Map.prototype.updateZzyETFCommon = function()//公共事件刷新
{
	var cList = $gameSystem.GetZzyETFCommonList();
	if(!cList.length)return;
	
	if(!$gameTemp.isCommonEventReserved())
	{
		var commonId = cList[$gameSystem._ZzyETFCommonIndex];
		$gameTemp.reserveCommonEvent(commonId);
		$gameSystem._ZzyETFCommonIndex++;
	}
	
	if($gameSystem._ZzyETFCommonIndex >= cList.length)
	{$gameSystem.ClearZzyETFCommonList();}
	
	
}

Scene_Map.prototype.updateZzyETFList = function()
{
	this._cZzyETFFrame++;
	if(this._cZzyETFFrame >= 60)
	{this._cZzyETFFrame = 0;}
	else{return;}
	
	var eqList = $gameSystem.GetZzyETFEquipList();
	if(eqList.length)
	{
		var isChange = false;
		for(var i=0;i<eqList.length;i++)
		{
			var eqInfo = eqList[i];
			if(eqInfo.time > 0)
			{eqInfo.time--;}
			else
			{
				//装备限时消失
				var eq = undefined;
				var count = 0;
				var isEq = true;
				switch(eqInfo.typeId)//统计数量删除
				{
					case 1:
					eq = $dataItems[eqInfo.id];
					count = $gameParty.numItems(eq);
					isEq = false;
					break;

					case 2:
					eq = $dataWeapons[eqInfo.id];
					count = $gameParty._weapons[eqInfo.id];
					
					break;
					case 3:
					eq = $dataArmors[eqInfo.id];
					count = $gameParty._armors[eqInfo.id];
					break;
				}
				
				if(eq)//道具将会逐个消失
				{
					if(eqInfo.typeId === 1)
					{
						count--;
						//对下一个道具继续计时
						if(count > 0)
						{
							var info = $gameSystem.MakeZzyETFEquip(eq);
							$gameSystem.PushZzyETFListRefresh(info);//刷新替换计时
							Zzy.ETF.RefreshList();//刷新队列
						}					
						
						$gameParty.loseItem(eq,1,true);
					}
					else
					{
						$gameParty.loseItem(eq,count,true);					
					}
				}
				
				if(isEq)
				{
					
					//队伍装备剔除
					for(var j=0;j<$gameParty._actors.length;j++)
					{
						var actor = $gameActors.actor($gameParty._actors[j]);
						
						var eqArr = actor.equips();
						for(var k=0;k<eqArr.length;k++)
						{
							var teq = eqArr[k];
							if(teq === eq)
							{
								actor.discardEquip(eq);//丢弃装备
								
								if(Zzy.ETF.IsHaveJHMWMessage())
								{
									this.ZzyETFJHMWAddMessage(eq,-1);
								}
							}
						}
					}
				}
				//公共事件队列
				if(eqInfo.commonId)
				{$gameSystem.PushZzyETFCommonList(eqInfo.commonId);}
				
				//此处从数组中剔除
				eqList[i] = undefined;//剔除
				isChange = true;
			}
		}
		
		if(isChange)
		{$gameSystem.DeleteZzyETFListNull();}//删除空			
	}

	var usList = $gameSystem.GetZzyETFUseItemList();
	if(usList.length)
	{
		var isChange = false;
		for(var i=0;i<usList.length;i++)
		{
			var info = usList[i];
			if(info.time > 0)
			{info.time--;}
			else
			{
				var comId = info.comId;
				Zzy.ETF.ExeCommonEvent(comId);//调用执行公共事件
				usList[i] = undefined;//剔除
				isChange = true;
			}
		}
		if(isChange)
		{$gameSystem.DeleteZzyETFListNull();}
	}
	
}


Scene_Map.prototype.ZzyETFJHMWAddMessage = function(item,amount)
{
	if (!JHMW.Message.ItemText || this._showPopups || !item || !item.name || item.itypeId > 2 || amount == 0) return;
	if (amount > 0) var gainText = JHMW.Message.GainText;
    else var gainText = JHMW.Message.LoseText;
    if (item.textColor) var text = JHMW.Message.ItemText.format(gainText, amount, '\\c[' + item.textColor + ']\\i[' + item.iconIndex + ']' + item.name);
    else var text = JHMW.Message.ItemText.format(gainText, amount, '\\i[' + item.iconIndex + ']' + item.name);
    var location = JHMW.Message.ItemLocation;
    var color = Notification.nextColor;
    Notification.add(text, location, color, 0, 0);	
}

//==================================================================
//Scene_Battle
//==================================================================
Zzy.ETF.Scene_Battle_start = Scene_Battle.prototype.start;
Scene_Battle.prototype.start = function()
{
	Zzy.ETF.Scene_Battle_start.call(this);
	this.ZzyETFCreateWindow();
	this._cZzyETFFrame = 0;
	
}

Scene_Battle.prototype.ZzyETFCreateWindow = function()//创建窗口
{
	this._zzyETFWindow = new Window_ZzyETFShow();
	this.addChild(this._zzyETFWindow);
	
	
}

Zzy.ETF.Scene_Battle_update = Scene_Battle.prototype.update;
Scene_Battle.prototype.update = function()
{
	Zzy.ETF.Scene_Battle_update.call(this);
	
	this.updateZzyETFList();
	this.updateZzyETFCommon();//刷新公共事件
}

Scene_Battle.prototype.updateZzyETFCommon = function()//公共事件刷新
{
	var cList = $gameSystem.GetZzyETFCommonList();
	if(!cList.length)return;
	
	if(!$gameTemp.isCommonEventReserved())
	{
		var commonId = cList[$gameSystem._ZzyETFCommonIndex];
		$gameTemp.reserveCommonEvent(commonId);
		$gameSystem._ZzyETFCommonIndex++;
	}
	
	if($gameSystem._ZzyETFCommonIndex >= cList.length)
	{$gameSystem.ClearZzyETFCommonList();}
	
	
}

Scene_Battle.prototype.updateZzyETFList = function()
{
	this._cZzyETFFrame++;
	if(this._cZzyETFFrame >= 60)
	{this._cZzyETFFrame = 0;}
	else{return;}
	
	var eqList = $gameSystem.GetZzyETFEquipList();
	if(eqList.length)
	{
		var isChange = false;
		for(var i=0;i<eqList.length;i++)
		{
			var eqInfo = eqList[i];
			if(eqInfo.time > 0)
			{eqInfo.time--;}
			else
			{
				//装备限时消失
				var eq = undefined;
				var count = 0;
				var isEq = true;
				switch(eqInfo.typeId)//统计数量删除
				{
					case 1:
					eq = $dataItems[eqInfo.id];
					count = $gameParty.numItems(eq);
					isEq = false;
					break;

					case 2:
					eq = $dataWeapons[eqInfo.id];
					count = $gameParty._weapons[eqInfo.id];
					
					break;
					case 3:
					eq = $dataArmors[eqInfo.id];
					count = $gameParty._armors[eqInfo.id];
					break;
				}
				
				if(eq)//道具将会逐个消失
				{
					if(eqInfo.typeId === 1)
					{
						count--;
						//对下一个道具继续计时
						if(count > 0)
						{
							var info = $gameSystem.MakeZzyETFEquip(eq);
							$gameSystem.PushZzyETFListRefresh(info);//刷新替换计时
							Zzy.ETF.RefreshList();//刷新队列
						}					
						
						$gameParty.loseItem(eq,1,true);
					}
					else
					{
						$gameParty.loseItem(eq,count,true);					
					}
				}
				
				if(isEq)
				{
					
					//队伍装备剔除
					for(var j=0;j<$gameParty._actors.length;j++)
					{
						var actor = $gameActors.actor($gameParty._actors[j]);
						
						var eqArr = actor.equips();
						for(var k=0;k<eqArr.length;k++)
						{
							var teq = eqArr[k];
							if(teq === eq)
							{
								actor.discardEquip(eq);//丢弃装备
								
								if(Zzy.ETF.IsHaveJHMWMessage())
								{
									this.ZzyETFJHMWAddMessage(eq,-1);
								}
							}
						}
					}
				}
				//公共事件队列
				if(eqInfo.commonId)
				{$gameSystem.PushZzyETFCommonList(eqInfo.commonId);}
				
				//此处从数组中剔除
				eqList[i] = undefined;//剔除
				isChange = true;
			}
		}
		
		if(isChange)
		{$gameSystem.DeleteZzyETFListNull();}//删除空			
	}

	var usList = $gameSystem.GetZzyETFUseItemList();
	if(usList.length)
	{
		var isChange = false;
		for(var i=0;i<usList.length;i++)
		{
			var info = usList[i];
			if(info.time > 0)
			{info.time--;}
			else
			{
				var comId = info.comId;
				Zzy.ETF.ExeCommonEvent(comId);//调用执行公共事件
				usList[i] = undefined;//剔除
				isChange = true;
			}
		}
		if(isChange)
		{$gameSystem.DeleteZzyETFListNull();}
	}
	
}



Scene_Battle.prototype.ZzyETFJHMWAddMessage = function(item,amount)
{
	if (!JHMW.Message.ItemText || this._showPopups || !item || !item.name || item.itypeId > 2 || amount == 0) return;
	if (amount > 0) var gainText = JHMW.Message.GainText;
    else var gainText = JHMW.Message.LoseText;
    if (item.textColor) var text = JHMW.Message.ItemText.format(gainText, amount, '\\c[' + item.textColor + ']\\i[' + item.iconIndex + ']' + item.name);
    else var text = JHMW.Message.ItemText.format(gainText, amount, '\\i[' + item.iconIndex + ']' + item.name);
    var location = JHMW.Message.ItemLocation;
    var color = Notification.nextColor;
    Notification.add(text, location, color, 0, 0);	
}


//==================================================================
//Scene_MenuBase
//==================================================================
Zzy.ETF.Scene_MenuBase_start = Scene_MenuBase.prototype.start;
Scene_MenuBase.prototype.start = function()
{
	Zzy.ETF.Scene_MenuBase_start.call(this);
	this.ZzyETFCreateWindow();
	this._cZzyETFFrame = 0;
	
}

Scene_MenuBase.prototype.ZzyETFCreateWindow = function()//创建窗口
{
	this._zzyETFWindow = new Window_ZzyETFShow();
	this.addChild(this._zzyETFWindow);
	
	
}

Zzy.ETF.Scene_MenuBase_update = Scene_MenuBase.prototype.update;
Scene_MenuBase.prototype.update = function()
{
	Zzy.ETF.Scene_MenuBase_update.call(this);
	
	this.updateZzyETFList();
	this.updateZzyETFCommon();//刷新公共事件
}

Scene_MenuBase.prototype.updateZzyETFCommon = function()//公共事件刷新
{
	var cList = $gameSystem.GetZzyETFCommonList();
	
	if(!cList.length)return;
	
	if(!$gameTemp.isCommonEventReserved())
	{
		var commonId = cList[$gameSystem._ZzyETFCommonIndex];
		$gameTemp.reserveCommonEvent(commonId);
		$gameSystem._ZzyETFCommonIndex++;
	}
	
	if($gameSystem._ZzyETFCommonIndex >= cList.length)
	{$gameSystem.ClearZzyETFCommonList();}
	
	
}


Scene_MenuBase.prototype.updateZzyETFList = function()
{
	this._cZzyETFFrame++;
	if(this._cZzyETFFrame >= 60)
	{this._cZzyETFFrame = 0;}
	else{return;}
	
	var eqList = $gameSystem.GetZzyETFEquipList();
	if(eqList.length)
	{
		var isChange = false;
		for(var i=0;i<eqList.length;i++)
		{
			var eqInfo = eqList[i];
			if(eqInfo.time > 0)
			{eqInfo.time--;}
			else
			{
				//装备限时消失
				var eq = undefined;
				var count = 0;
				var isEq = true;
				switch(eqInfo.typeId)//统计数量删除
				{
					case 1:
					eq = $dataItems[eqInfo.id];
					count = $gameParty.numItems(eq);
					isEq = false;
					break;

					case 2:
					eq = $dataWeapons[eqInfo.id];
					count = $gameParty._weapons[eqInfo.id];
					
					break;
					case 3:
					eq = $dataArmors[eqInfo.id];
					count = $gameParty._armors[eqInfo.id];
					break;
				}
				
				if(eq)//道具将会逐个消失
				{
					if(eqInfo.typeId === 1)
					{
						count--;
						//对下一个道具继续计时
						if(count > 0)
						{
							var info = $gameSystem.MakeZzyETFEquip(eq);
							$gameSystem.PushZzyETFListRefresh(info);//刷新替换计时
							Zzy.ETF.RefreshList();//刷新队列
						}					
						
						$gameParty.loseItem(eq,1,true);
					}
					else
					{
						$gameParty.loseItem(eq,count,true);					
					}
				}
				
				if(isEq)
				{
					
					//队伍装备剔除
					for(var j=0;j<$gameParty._actors.length;j++)
					{
						var actor = $gameActors.actor($gameParty._actors[j]);
						
						var eqArr = actor.equips();
						for(var k=0;k<eqArr.length;k++)
						{
							var teq = eqArr[k];
							if(teq === eq)
							{
								actor.discardEquip(eq);//丢弃装备
								
								if(Zzy.ETF.IsHaveJHMWMessage())
								{
									this.ZzyETFJHMWAddMessage(eq,-1);
								}
							}
						}
					}
				}
				//公共事件队列
				if(eqInfo.commonId)
				{$gameSystem.PushZzyETFCommonList(eqInfo.commonId);}
				
				//此处从数组中剔除
				eqList[i] = undefined;//剔除
				isChange = true;
			}
		}
		
		if(isChange)
		{$gameSystem.DeleteZzyETFListNull();}//删除空			
	}

	var usList = $gameSystem.GetZzyETFUseItemList();
	if(usList.length)
	{
		var isChange = false;
		for(var i=0;i<usList.length;i++)
		{
			var info = usList[i];
			if(info.time > 0)
			{info.time--;}
			else
			{
				var comId = info.comId;
				Zzy.ETF.ExeCommonEvent(comId);//调用执行公共事件
				usList[i] = undefined;//剔除
				isChange = true;
			}
		}
		if(isChange)
		{$gameSystem.DeleteZzyETFListNull();}
	}
	
}

Scene_MenuBase.prototype.ZzyETFJHMWAddMessage = function(item,amount)
{
	if (!JHMW.Message.ItemText || this._showPopups || !item || !item.name || item.itypeId > 2 || amount == 0) return;
	if (amount > 0) var gainText = JHMW.Message.GainText;
    else var gainText = JHMW.Message.LoseText;
    if (item.textColor) var text = JHMW.Message.ItemText.format(gainText, amount, '\\c[' + item.textColor + ']\\i[' + item.iconIndex + ']' + item.name);
    else var text = JHMW.Message.ItemText.format(gainText, amount, '\\i[' + item.iconIndex + ']' + item.name);
    var location = JHMW.Message.ItemLocation;
    var color = Notification.nextColor;
    Notification.add(text, location, color, 0, 0);	
}


//================================================================
//Game_Party
//================================================================

Zzy.ETF.Game_Party_gainItem = Game_Party.prototype.gainItem;
Game_Party.prototype.gainItem = function(item, amount, includeEquip) 
{
	
	
	if(item && $gameSystem.IsZzyETFTime(item) && amount > 0)
	{
		
		if(!Zzy.ETF.IsHaveYEPItemCore())//插件开启
		{	
			var info = $gameSystem.MakeZzyETFEquip(item);
			$gameSystem.PushZzyETFList(info);//压入到内存中
		}
		else//此处是判断如果YEP道具类型没有开启独立化,那么将会正常压入到时间列表中
		{
			if (!DataManager.isIndependent(item))
			{
				var info = $gameSystem.MakeZzyETFEquip(item);
				$gameSystem.PushZzyETFList(info);//压入到内存中				
			}
		}
		
		if(SceneManager._scene instanceof Scene_Map)
		{
			Zzy.ETF.RefreshList();			
		}
	}
	
	
	Zzy.ETF.Game_Party_gainItem.call(this,item, amount, includeEquip);




};


if(Zzy.Param.ETFCompYEPItemCore)
{
		
	Zzy.ETF.Game_Party_gainIndependentItem = Game_Party.prototype.gainIndependentItem;
	Game_Party.prototype.gainIndependentItem = function(item, amount, includeEquip) 
	{
		var reArr = Zzy.ETF.Game_Party_gainIndependentItem.call(this,item,amount,includeEquip);
		
		for(var i=0;i<reArr.length;i++)
		{
			var eq = reArr[i];
			if(eq && $gameSystem.IsZzyETFTime(eq) && amount > 0)
			{
				var info = $gameSystem.MakeZzyETFEquip(eq);

				$gameSystem.PushZzyETFList(info);//压入到内存中
			}
		}
		
		
		return reArr;
	};
}



Zzy.ETF.Game_Party_consumeItem = Game_Party.prototype.consumeItem;
Game_Party.prototype.consumeItem = function(item) 
{
	if (DataManager.isItem(item) && item.consumable)
	{
		//检测道具是否具备事件倒计时
		if(item.zzyETF && item.zzyETF.useComId && item.zzyETF.useComTime)
		{
			
			var info = {};
			info.comId = item.zzyETF.useComId;
			info.comTime = item.zzyETF.useComTime;
			info.itemId = item.id;
			info.prefix = item.zzyETF.prefix ? item.zzyETF.prefix : '';
			$gameSystem.PushZzyETFUseItemCommond(info);//压入时长
			
			Zzy.ETF.RefreshList();
		}
	}
	
	Zzy.ETF.Game_Party_consumeItem.call(this,item);
};








//================================================================
//DataManager
//================================================================
Zzy.ETF.DataManager_isDatabaseLoaded = DataManager.isDatabaseLoaded;
DataManager.isDatabaseLoaded = function()
{
	if (!Zzy.ETF.DataManager_isDatabaseLoaded.call(this)) return false;
	
	this.ZzyETFLoadNoteCase1($dataArmors);//护甲
	this.ZzyETFLoadNoteCase1($dataWeapons);//武器
	this.ZzyETFLoadNoteCase1($dataItems);//道具
	return true;
}

DataManager.ZzyETFLoadNoteCase1 = function(objArr)
{
  for (var i = 1; i < objArr.length; i++) 
  {
    var obj = objArr[i];
    var noteData = obj.note.split(/[\r\n]+/);
	
	obj.zzyETF = {};
	//obj.zzyETF.suitEffect = [];//套装

	var selectMode = 'n';
	var suitType = '';
	var countIndex = 0;

	for(var j=0;j<noteData.length;j++)
	{
		var lineStr = noteData[j];
		if(lineStr.match(/<ZZYETF TIME:[ ](.*)>/i))
		{
			var timeEval = String(RegExp.$1);
			obj.zzyETF.time = timeEval;
		}
		else if(lineStr.match(/<ZZYETF COMMON:[ ](.*)>/i))
		{
			var commonId = parseInt(RegExp.$1);
			obj.zzyETF.commonId = commonId;
		}
		else if(lineStr.match(/<ZZYETF MAPPREFIX:[ ](.*)>/i))
		{
			var prefix = String(RegExp.$1);
			obj.zzyETF.prefix = prefix;
		}
		else if(lineStr.match(/<ZZYETF USEITEMCOMMON:[ ](.*)[ ](.*)>/i))
		{
			var comId = parseInt(RegExp.$1);
			var time = parseInt(RegExp.$2);
			
			obj.zzyETF.useComId = comId;
			obj.zzyETF.useComTime = time;
		}		
			
	}
  }
}

if(LiuYue.LiuYue_StrengEquip)
{
	
	
//==================================================================
//Scene_ZzySEFStreng
//==================================================================
Zzy.ETF.Scene_ZzySEFStreng_start = Scene_ZzySEFStreng.prototype.start;
Scene_ZzySEFStreng.prototype.start = function()
{
	Zzy.ETF.Scene_ZzySEFStreng_start.call(this);
	this.ZzyETFCreateWindow();
	this._cZzyETFFrame = 0;
	
}

Scene_ZzySEFStreng.prototype.ZzyETFCreateWindow = function()//创建窗口
{
	this._zzyETFWindow = new Window_ZzyETFShow();
	this.addChild(this._zzyETFWindow);
	
	
}



Zzy.ETF.Scene_ZzySEFStreng_update = Scene_ZzySEFStreng.prototype.update;
Scene_ZzySEFStreng.prototype.update = function()
{
	Zzy.ETF.Scene_ZzySEFStreng_update.call(this);
	
	this.updateZzyETFList();
	this.updateZzyETFCommon();//刷新公共事件
}

Scene_ZzySEFStreng.prototype.updateZzyETFCommon = function()//公共事件刷新
{
	var cList = $gameSystem.GetZzyETFCommonList();
	
	if(!cList.length)return;
	
	if(!$gameTemp.isCommonEventReserved())
	{
		var commonId = cList[$gameSystem._ZzyETFCommonIndex];
		$gameTemp.reserveCommonEvent(commonId);
		$gameSystem._ZzyETFCommonIndex++;
	}
	
	if($gameSystem._ZzyETFCommonIndex >= cList.length)
	{$gameSystem.ClearZzyETFCommonList();}
	
	
}



Scene_ZzySEFStreng.prototype.updateZzyETFList = function()
{
	this._cZzyETFFrame++;
	if(this._cZzyETFFrame >= 60)
	{this._cZzyETFFrame = 0;}
	else{return;}
	
	if(!$gameSystem._ZzyETFEquipList.length)return;
	
	var isChange = false;
	for(var i=0;i<$gameSystem._ZzyETFEquipList.length;i++)
	{
		var eqInfo = $gameSystem._ZzyETFEquipList[i];
		
		if(eqInfo.time > 0)
		{eqInfo.time--;}
		else
		{
			//装备限时消失
			var eq = undefined;
			var count = 0;
			var isEq = true;
			switch(eqInfo.typeId)//统计数量删除
			{
				case 1:
				eq = $dataItems[eqInfo.id];
				count = $gameParty.numItems(eq);
				isEq = false;
				break;

				case 2:
				eq = $dataWeapons[eqInfo.id];
				count = $gameParty._weapons[eqInfo.id];
				
				break;
				case 3:
				eq = $dataArmors[eqInfo.id];
				count = $gameParty._armors[eqInfo.id];
				break;
			}
			if(eq)
			{
				count = count ? count : 0;
				$gameParty.loseItem(eq,count,true);
			}
			
			if(isEq)
			{
				
				//队伍装备剔除
				for(var j=0;j<$gameParty._actors.length;j++)
				{
					var actor = $gameActors.actor($gameParty._actors[j]);
					
					var eqArr = actor.equips();
					for(var k=0;k<eqArr.length;k++)
					{
						var teq = eqArr[k];
						if(teq === eq)
						{
							actor.discardEquip(eq);//丢弃装备
							
							if(Zzy.ETF.IsHaveJHMWMessage())
							{
								this.ZzyETFJHMWAddMessage(eq,-1);
							}
						}
					}
				}
			}
			//公共事件队列
			if(eqInfo.commonId)
			{$gameSystem.PushZzyETFCommonList(eqInfo.commonId);}
			
			//此处从数组中剔除
			$gameSystem._ZzyETFEquipList[i] = undefined;//剔除
			isChange = true;
		}
	}
	
	if(isChange)
	{$gameSystem.DeleteZzyETFListNull();}//删除空
	
}


Scene_ZzySEFStreng.prototype.ZzyETFJHMWAddMessage = function(item,amount)
{
	if (!JHMW.Message.ItemText || this._showPopups || !item || !item.name || item.itypeId > 2 || amount == 0) return;
	if (amount > 0) var gainText = JHMW.Message.GainText;
    else var gainText = JHMW.Message.LoseText;
    if (item.textColor) var text = JHMW.Message.ItemText.format(gainText, amount, '\\c[' + item.textColor + ']\\i[' + item.iconIndex + ']' + item.name);
    else var text = JHMW.Message.ItemText.format(gainText, amount, '\\i[' + item.iconIndex + ']' + item.name);
    var location = JHMW.Message.ItemLocation;
    var color = Notification.nextColor;
    Notification.add(text, location, color, 0, 0);	
}
	
}




//--------------------------------Function-------------------------------

Zzy.ETF.EvalTranslateOfEquip = function(formula,equip)//eval转换
{
	var variables = $gameVariables._data;//全局变量
	var v = $gameVariables._data;
	var switchs = $gameSwitches._data;//全局开关
	var s = $gameSwitches._data;
	var eq = equip;



	return eval(formula);
}

Zzy.ETF.GetTime = function(typeId,id)//装备剩余时长
{
	switch(typeId)
	{
		case 'item':typeId = 1;break;
		case 'weapon':typeId = 2;break;
		case 'armor':typeId = 3;break;
	}
	
	
	
	return $gameSystem.GetZzyETFSurplusTimeEx(typeId,id);
}


Zzy.ETF.IsShow = function(enable)//获取名字
{
	$gameSystem.SetZzyETFIsShow(enable);
	Zzy.ETF.RefreshShow();
}


Zzy.ETF.RespawnTime = function(info)
{

}

Zzy.ETF.FixZeroStart = function(str,n)
{
	return (Array(n).join(0)+str).slice(-n);
}


Zzy.ETF.RefreshList = function()
{
	var pointer = SceneManager._scene._zzyETFWindow;
	pointer.ClearTimeList();
	pointer.CreateTimeList();//生产
	
}



//强制执行公共事件
Zzy.ETF.ExeCommonEvent = function(commonId) //执行公共事件
{
	var tempInter = new Game_ZzyETFInterpreter();//临时启动器
	tempInter.SetupZzyETFCommand(commonId);//启动一般保留事件

	while(true)
	{
		tempInter.update();
		if(Zzy.ETF.IsRunTempInterOk(tempInter)){break;}	
	}
};


Zzy.ETF.IsRunTempInterOk = function(tempInter)
{
	
	if(!tempInter._list)return true;
	var commonLen = tempInter._list.length;//事件长度
	if(tempInter._index >= commonLen)return true;
	
	return false;
}



//=============================================================
//Game_ZzyETFInterpreter
//=============================================================

function Game_ZzyETFInterpreter() {
    this.initialize.apply(this, arguments);
}

Game_ZzyETFInterpreter.prototype = Object.create(Game_Interpreter.prototype);
Game_ZzyETFInterpreter.prototype.constructor = Game_ZzyETFInterpreter;

Game_ZzyETFInterpreter.prototype.SetupZzyETFCommand = function(commonId) //存储事件信息
{
    var commonEvent = $dataCommonEvents[commonId];
    if (commonEvent) 
	{
        var eventId = this.isOnCurrentMap() ? this._eventId : 0;
		this.setup(commonEvent.list, eventId);
    }
    return true;
};

//重新update，排除SceneManager.isSceneChanging的检测
Game_ZzyETFInterpreter.prototype.update = function()
{
    while (this.isRunning()) {
        if (this.updateChild() || this.updateWait()) 
		{ break;}
        if (!this.executeCommand()) {break; }
        if (this.checkFreeze()) { break;  }
    }	
}












