﻿/*:
* @plugindesc 随机物品基本程序
* @author MaRcoLi Revise by Fanzi
*
* @param 浮窗背景
* @parent ---浮窗---
* @dir img/pictures/
* @type file
* @desc 浮动窗口背景图头名字，不同品质加 ID 后缀区分
* @default float_Title
*
* @param 浮窗停留
* @parent ---浮窗---
* @type number
* @min 240
* @desc 浮动窗口停留时长(单位帧)
* @default 600
*
* @help
=================================帮助文件======================================
*/
var Mac_BaseFun = Mac_BaseFun || {};
Mac_BaseFun.Parameters = PluginManager.parameters('Mac_BaseFun');
Mac_BaseFun.Param = Mac_BaseFun.Param || {};

Mac_BaseFun.Param.Bgd = String(Mac_BaseFun.Parameters['浮窗背景']);
Mac_BaseFun.Param.LayTime = parseInt(Mac_BaseFun.Parameters['浮窗停留'] || 240);
//==============随机范围整数=============
function Mac_RandomNum(Min,Max) {
    var Range = Max - Min;
    var Rand = Math.random();
    var num = Min + Math.round(Rand * Range);
    return num;
};
//==============随机范围 2位小数=============
function Mac_RandomDec(Min,Max) {
    var Range = Max - Min;
    var Rand = Math.random();
    var dec = Rand * Range;
    var num = Math.round((Min + dec) * 100) / 100;
    return num;
};
//==============数组名字排序=========
function Mac_compare(property) {
    return function(a,b) {
        var aa = a[property];
        var bb = b[property];
        if (aa < bb) return -1;
        if (aa > bb) return 1;
        return 0;
    }
};
//==================黑色=================
Window_Base.prototype.Color15 = function() {
    return this.textColor(15);
};
//===========================HelpWindow=================
function Mac_WinHelp() {
    this.initialize.apply(this, arguments);
};

Mac_WinHelp.prototype = Object.create(Window_Base.prototype);
Mac_WinHelp.prototype.constructor = Mac_WinHelp;

Mac_WinHelp.prototype.initialize = function(x, y, width, numLines) {
    var height = this.fittingHeight(numLines || 2);
    Window_Base.prototype.initialize.call(this, x, y, width, height);
    this._text = '';
};

Mac_WinHelp.prototype.setText = function(text) {
    if (this._text !== text) {
        this._text = text;
        this.refresh();
    }
};

Mac_WinHelp.prototype.clear = function() {
    this.setText('');
};

Mac_WinHelp.prototype.setItem = function(item) {
    this.setText(item ? item.description : '');
};

Mac_WinHelp.prototype.refresh = function() {
    this.contents.clear();
    this.drawTextEx(this._text, this.textPadding(), 0);
};
//===========================物品基本Window==========================
function Mac_itemWindow() {
    this.initialize.apply(this, arguments);
};

Mac_itemWindow.prototype = Object.create(Window_ItemList.prototype);
Mac_itemWindow.prototype.constructor = Mac_itemWindow;

Mac_itemWindow.prototype.initialize = function(x, y, width, height) {
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this._data = [];
};

//================物品栏间隔=======================
Mac_itemWindow.prototype.spacing = function() {
    return 1;
};

Mac_itemWindow.prototype.setItemWindow = function(itemWindow) {
    this._itemWindow = itemWindow;
};
Mac_itemWindow.prototype.setFloatWin = function(floatwin) {
    this._floatwin = floatwin;
};
Mac_itemWindow.prototype.setStatusWindow = function(statusWindow) {
    this._statusWindow = statusWindow;
    this.callUpdateHelp();
};

Mac_itemWindow.prototype.maxItems = function() {
    return this._data ? this._data.length : 1;
};
//======================物品绘制=================================

Mac_itemWindow.prototype.drawItem = function(index) {
    var item = this._data[index];
    if (item) {
        var numberWidth = this.numberWidth();
        var rect = this.itemRect(index);
        rect.width -= this.textPadding();
//        this.changePaintOpacity(this.isEnabled(item));
        this.drawItemName(item, rect.x, rect.y, rect.width );
        if (!EqManager.isIndependent(item)) {
            this.drawItemNumber(item, rect.x, rect.y, rect.width);
        }
        this.changePaintOpacity(1);
    }
};

Mac_itemWindow.prototype.drawItemNumber = function(item, x, y, width) {
    if (this.needsNumber()) {
        this.drawText(':', x, y, width - this.textWidth('00'), 'right');
        this.drawText($gameParty.numItems(item), x, y, width, 'right');
    }
};

Mac_itemWindow.prototype.drawItemName = function(item, x, y, width) {
    width = width || 312;
    if (item) {
        var iconBoxWidth = Window_Base._iconWidth + 4;
        this.ItemColor(item);
        this.drawIcon(item.iconIndex, x + 2, y + 2);
        this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
        this.resetTextColor();
    }
};

Mac_itemWindow.prototype.ItemColor = function(item) {
    if (DataManager.isItem(item)) return;
    var quality = item._quality;
    if (quality === 'Unique') this.changeTextColor(this.textColor(14));
    else if (quality === 'Rare') this.changeTextColor(this.textColor(30));
    else if (quality === 'Rumor') this.changeTextColor(this.textColor(1));
    else if (quality === 'Superior') this.changeTextColor(this.textColor(3));
    else this.changeTextColor(this.textColor(0));
};

Mac_itemWindow.prototype.drawActorHp = function(actor, x, y, width) {
    width = width || 186;
    var color1 = this.hpGaugeColor1();
    var color2 = this.hpGaugeColor2();
    this.drawGauge(x, y, width, actor.hpRate(), color1, color2);
    this.changeTextColor(this.systemColor());
    this.makeFontSmaller();
    this.drawText(TextManager.hpA, x, y, 44);
    this.drawCurrentAndMax(actor.hp, actor.mhp, x, y, width, this.hpColor(actor), this.normalColor());
    this.makeFontBigger();
};

Mac_itemWindow.prototype.drawActorMp = function(actor, x, y, width) {
    width = width || 186;
    var color1 = this.mpGaugeColor1();
    var color2 = this.mpGaugeColor2();
    this.drawGauge(x, y, width, actor.mpRate(), color1, color2);
    this.changeTextColor(this.systemColor());
    this.makeFontSmaller();
    this.drawText(TextManager.mpA, x, y, 44);
    this.drawCurrentAndMax(actor.mp, actor.mmp, x, y, width, this.mpColor(actor), this.normalColor());
    this.makeFontBigger();
};

Mac_itemWindow.prototype.updateHelp = function() {
    this.setHelpWindowItem(this.item());
    if (this._actor && this._statusWindow) {
        var actor = JsonEx.makeDeepCopy(this._actor);
        actor.forceChangeEquip(this._slotId, this.item());
        this._statusWindow.setTempActor(actor);
    }
    if (this.active && this.isEquipItem(this.item())) {
        this._floatwin.setEqItem(this.item());
    } else {
        this._floatwin.setEqItem(null);
    }
    this.updateShopHelp();
};

Mac_itemWindow.prototype.updateShopHelp = function() {
    if (this._shop ===true && this.isEquipItem(this.item()) ) {
        var actorarry = [];
        var num = $gameParty.allMembers().length;
        for ( var i=0; i<num; i++) {
            var actor = $gameParty.allMembers()[i];
            var tempactor = JsonEx.makeDeepCopy(actor);
            var enabled = actor.canEquip(this.item());
            if (enabled) {
                tempactor.forceChangeEquip(this.item().etypeId-1, this.item());
                actorarry.push(tempactor);
            } else {
                actorarry.push(actor);
            }
        }
        if (this._statusWindow) {
            this._statusWindow.setItem(this.item());
            this._statusWindow.setTempActor(actorarry);
        }
    } else if (this.active && this._statusWindow && this._shop === true) {
        this._statusWindow.setItem(null);
        this._statusWindow.setTempActor(null);
    }
};
Mac_itemWindow.prototype.isEquipItem = function(item) {
    return DataManager.isWeapon(item) || DataManager.isArmor(item);
};

Mac_itemWindow.prototype.refresh = function() {
//    if (!Imported.YEP_SaveCore) return;
    this.makeItemList();
    this.createContents();
    this.drawAllItems();
};
//=================================================
function Mac_floatWin() {
    this.initialize.apply(this, arguments);
};

Mac_floatWin.prototype = Object.create(Mac_itemWindow.prototype);
Mac_floatWin.prototype.constructor = Mac_floatWin;

Mac_floatWin.prototype.initialize = function(x, y) {
    var width = this.windowWidth();
    var height = this.windowHeight();
    Mac_itemWindow.prototype.initialize.call(this, 0, 0, width, height);
    this.active = false;
    this.timer=0;
    this.showwin=false;
    this.setTitlepic();
    this.hide();
    this.setFloatRect();
};

Mac_floatWin.prototype.windowWidth = function() {
    return 230;
};

Mac_floatWin.prototype.windowHeight = function() {
    return 400;
};

Mac_floatWin.prototype.setItemWindow = function(itemWindow) {
    this._itemWindow = itemWindow;
};
Mac_floatWin.prototype.setHideOnetime = function(a, b) {
    this._slotwin = a;
    this._slotwincmd = b;
};
Mac_floatWin.prototype.setTitlepic = function() {
    this.opacity=0;
    this.contentsOpacity = 0;
    this._title = new Sprite();
    this._title.x=18;
    this._title.y=18;
    this._title.opacity=0;
};
Mac_floatWin.prototype.setEqItem = function(item) {
    this.Eqitem = item;
    if (!this.Eqitem) {
        this.hide(); return;
    } else if (this.Eqitem._randomItem) {this.hide(); return;}
    this.contentsOpacity = 180;
    this._title.opacity = 180;
    this.winopen = false;
    this.refresh();
};
Mac_floatWin.prototype.setFloatRect = function() {
    this.rect = new Rectangle();
    this.rect.y = 34;
    this.rect.width =160;
    // this.rect.opacity=50;
};
Mac_floatWin.prototype.refresh = function() {
//    if (!Imported.YEP_LoadCore) return;
    this.contents.clear();
    this.showwin=true;
    this.timer=0;
    this.show();
    this.drawFloatRect();
};

Mac_floatWin.prototype.setwinplace = function() {
    var itmindex = this._itemWindow._index;
    var itemRect = this._itemWindow.itemRect(itmindex);
    if (this._slotwin) this._itemWindow.x +=144;
    if (this._slotwincmd) { this.hide(); this._slotwincmd=false;}
    this.x = this._itemWindow.x +itemRect.x+35;
    this.y = this._itemWindow.y +itemRect.y +36;
    if (this.y+this.rect.height+18+34 > Graphics.boxHeight){
        this.y = this._itemWindow.y +itemRect.y - (this.rect.height+34);
    }
};

Mac_floatWin.prototype.drawDarkRect = function(dx, dy, dw, dh) {
    var color = this.Color15();
    this.contents.paintOpacity =240;
    this.contents.fillRect(dx, dy, dw, dh, color);
    this.changePaintOpacity(true);
};
Mac_floatWin.prototype.drawFloatRect = function() {
    var Eqlength = this.Eqitem._disparams ? this.Eqitem._disparams.length : 0;
    var height=20;
    this.rect.height = 55 + Eqlength *height;
    this.setwinplace();
    this.selectTitlepic();
    this.drawDarkRect(this.rect.x, this.rect.y, this.rect.width, this.rect.height);
    this.makeFontSmaller();
    this.drawItemName(this.Eqitem, 0, 0, this.rect.width);
    this.darwitemRank();
    this.drawText(this.baseparam(), 0, 50, this.rect.width,'center');
    if (Eqlength) {
        for (var i=0; i<Eqlength; i++) {
            this.changeTextColor(this.mpGaugeColor1());
            this.drawText(this.Eqitem._disparams[i], 0, 50+height*(i+1), this.rect.width,'center');
        }
    }
    this.makeFontBigger();
    this.resetTextColor();
};
Mac_floatWin.prototype.darwitemRank = function() {
    if (this.Eqitem._quality === 'Unique') {
        this.changeTextColor(this.textColor(14));
        this.drawText('史诗', 0, 30, 100,'center');
    } else if (this.Eqitem._quality === 'Rare') {
        this.changeTextColor(this.textColor(30));
        this.drawText('极品', 0, 30, 100,'center');
    } else if (this.Eqitem._quality === 'Rumor') {
        this.changeTextColor(this.textColor(1));
        this.drawText('精品', 0, 30, 100,'center');
    } else if (this.Eqitem._quality === 'Superior') {
        this.changeTextColor(this.textColor(3));
        this.drawText('良品', 0, 30, 100,'center');
    } else this.drawText('普通', 0, 30, 100,'center');
    if (this.Eqitem._rank == 3) {
        this.changeTextColor(this.textColor(10));
        this.drawText('极品', 60, 30, this.rect.width - 60,'center');
    } else if (this.Eqitem._rank == 2) {
        this.changeTextColor(this.textColor(20));
        this.drawText('精华', 60, 30, this.rect.width - 60,'center');
    } else if (this.Eqitem._rank == 1) {
        this.changeTextColor(this.textColor(27));
        this.drawText('优秀', 60, 30, this.rect.width - 60,'center');
    } else {
        this.changeTextColor(this.textColor(8));
        this.drawText('一般', 60, 30, this.rect.width - 60,'center');
    }
};
Mac_floatWin.prototype.baseparam = function() {
    if (DataManager.isWeapon(this.Eqitem)) {
        if (this.Eqitem.Independent === true) {
            var baseItemId=this.Eqitem.baseItemId;
            var baseItem = $dataWeapons[baseItemId];
            var base = EqManager.loadRank(baseItem,this.Eqitem,2);
        } else {
            var base = this.Eqitem.params[2];}
            var param = '基础' + TextManager.param(2) + ' ' + base;
    } else {
        if (this.Eqitem.Independent === true) {
            var baseItemId=this.Eqitem.baseItemId;
            var baseItem = $dataArmors[baseItemId];
            var base = EqManager.loadRank(baseItem,this.Eqitem,3);
        } else {
            var base = this.Eqitem.params[3];
        }
        var param = '基础' + TextManager.param(3) + ' ' + base;
    }
    return param;
};
Mac_floatWin.prototype.drawItemName = function(item, x, y, width) {
    if (item) {
        this.ItemColor(item);
        this.drawText(item.name, x , y, width, 'center');
        this.resetTextColor();
    }
};

Mac_floatWin.prototype.update = function() {
    if (this.showwin) {
        this.timer++;
        if (this.contentsOpacity < 255 && this.winopen == false ) {
            this.contentsOpacity += 4;
            this._title.opacity += 4;
        } else {this.winopen = true;};
        if (this.timer > Mac_BaseFun.Param.LayTime) {
            if (this.contentsOpacity > 0) {
                this.contentsOpacity -= 12;
                this._title.opacity -= 12;
            } else {
                this.hide();
                this.timer=0;
                this.showwin=false;
            }
        }
    }
};

Mac_floatWin.prototype.selectTitlepic = function() {
    var picfile='';
    var quality=this.Eqitem._quality;
    if (this.Eqitem) {
        if (quality === 'Unique') {
            picfile = Mac_BaseFun.Param.Bgd + '4';
        } else if (quality === 'Rare') {
            picfile = Mac_BaseFun.Param.Bgd + '3';
        } else if (quality === 'Rumor') {
            picfile = Mac_BaseFun.Param.Bgd + '2';
        } else if (quality === 'Superior') {
            picfile = Mac_BaseFun.Param.Bgd + '1';
        } else {
            picfile = Mac_BaseFun.Param.Bgd + '0';
        }
        this._title.bitmap = ImageManager.loadPicture(picfile);
        this.addChildAt(this._title,1);
    }
};