﻿/*:
 * @plugindesc 升级加点系统Allows players to distribute points to different stats of the Actors at his or her discretion.
 * @author SumRndmDde
 *
 * @param Default Stats
 * @desc 加点方向A list of stats Actors can distribute to by default.
 * All of the available stats can be found in HELP.
 * @default mhp, mmp, atk, def, mat, mdf, agi, luk
 *
 * @param Default Formula
 * @desc 得点公式The default formula used for determining how many "stat points" an Actor gains by leveling up.
 * @default Math.ceil(level / 10)
 *
 * @param Show on Menu
 * @desc 菜单显示If 'true', this plugin will place the command in the menu.
 * @default true
 *
 * @param == Stat Reset ==
 * @default
 *
 * @param Allow Stat Resets
 * @desc 可否重置If 'true', then players will have the option to reset the distributed stats and return the used stat points.
 * @default true
 *
 * @param Reset Gold Cost
 * @desc 重置成本The formula determining the amount of gold for resetting.
 * Use "points" to reference the amount of points used.
 * @default points * 1000
 *
 * @param == Custom Texts ==
 * @default
 *
 * @param Level Up Message
 * @desc 升级信息This message is displayed when an actor levels up to show how many points were gained. %1 is the number of points.
 * @default Got %1 stat points!
 *
 * @param Command Name
 * @desc 菜单命令The is the name of the command for stat distribution in the menu.
 * @default Stat Boost
 *
 * @param Points Text
 * @desc 点数命名The text used to display how much points an Actor has.
 * @default Stat Points:
 *
 * @param Upgrade Text
 * @desc 期望命名The text used to display how much points an upgrade is worth.
 * @default Upgrade Cost:
 *
 * @param Spend Text
 * @desc 消耗命名The text used in the command window to begin spending.
 * @default Spend
 *
 * @param Reset Text
 * @desc 重置命名The text used in the command window to reset stat points.
 * @default Reset
 *
 * @param Finish Text
 * @desc 确定命名The text used in the command window to leave the scene.
 * @default Finish
 *
 * @param == Gauge Options ==
 * @default
 *
 * @param Use Gauges
 * @desc 使用统计If 'true', the stat distribution will have gauges representing how close the distributed stat is to reaching its max.
 * @default true
 *
 * @param Gauge Height
 * @desc 统计高度The height of the gauges used in the stat distribution.
 * @default 14
 *
 * @param == Points Label ==
 * @default
 *
 * @param Point Icon
 * @desc 点数图标Input the icon index of the icon for stat points.
 * Set to 0 to use text instead.
 * @default 87
 *
 * @param Point Text
 * @desc 显示单位If "Point Icon" is set to 0, this will be used as the label for stat points.
 * @default Points
 * 
 * @param Point Color
 * @desc 标签颜色If using text, this will be the color of the text.
 * @default #66ff66
 *
 * @help
 *
 * Stat Distribution
 * Version 1.07
 * SumRndmDde
 *
 * This plugin allows players to distribute points to different stats of the 
 * Actors at his or her discretion. A command is added to the menu which allows
 * access to a new menu for stat distribution!
 *
 * ==============================================================================
 *  Plugin Commands
 * ==============================================================================
 *
 * Use the following plugin commands to preform various actions.
 *
 *   OpenStatDistribution Actor [actorId]  打开角色点数界面
 *
 * This opens the Stat Distribution Menu for the specified Actor ID.
 *
 *   OpenStatDistribution Party [memberIndex]  打开队员点数界面
 *
 * This opens the Stat Distribution Menu for the specified Actor based on the
 * index in which they are in the party. For example, setting "memberIndex"
 * to 0 will open the menu for the party's leader.
 *
 *   AddStatPoints Actor [actorId] [points]
 *   AddStatPoints Party [memberIndex] [points]
 *
 * 这将向特定的角色添加特定数量的点数.
 *
 * ==============================================================================
 *  Upgradeable Stats
 * ==============================================================================
 *
 * 确定哪些属性可以加点请填写“Default Stats”参数。
 * 否则，角色备注可用于自定义每个角色的可加点属性。
 *
 *   <Set Distribution Stats: [stats]>
 *
 * 将角色属性设置进加点列表。
 *
 *   <Add Distribution Stats: [stats]>
 *
 * 在角色已有的可加点列表上添加属性。
 *
 * ==============================================================================
 *  How to Customize Stats
 * ==============================================================================
 * 汉化角色属性的显示名称与备注名称需要在 /data 文件夹中添加一个 json 文件
 * 文件名：DistributionStats.json 文件附在插件包中（由于未使用到，部分未汉化）
 * In order to customize the properties of stats, you must use the Database EX.
 * Simply go to the customizable editors, and select "Stat Distribution Editor".
 *
 * Within here, a list of stats will be available and their variables can
 * be changed to fit with your needs. Be sure to reload to see the changes!
 *
 * ==============================================================================
 *  List of Upgradeable Stats
 * ==============================================================================
 *
 * Here is a list of all the available stats and their three-letter codes.
 *
 * == Base Stats ==
 * mhp - Max HP
 * mmp - Max MP
 * atk - Attack
 * def - Defense
 * mat - Magical Attack
 * mdf - Magical Defense
 * agi - Agility
 * luk - Luck
 *
 * == Ex Stats ==
 * hit - Hit Rate
 * eva - Evasion Rate
 * cri - Critical Rate
 * cev - Critical Evasion Rate
 * mev - Magic Evasion Rate
 * mrf - Magic Reflection Rate
 * cnt - Counter Attack Rate
 * hrg - Hp Regeneration
 * mrg - Mp Regeneration
 * trg - Tp Regeneration 
 *
 * == Sp Stats == 
 * tgr - Target Rate
 * grd - Guard Effect Rate
 * rec - Recovery Effect Rate
 * pha - Pharmacology
 * mcr - Mp Cost Rate
 * tcr - Tp Charge Rate
 * pdr - Physical Damage Rate
 * mdr - Magical Damage Rate
 * fdr - Floor Damage Rate
 * exr - Experience Rate
 *
 * ==============================================================================
 *  Stat Points
 * ==============================================================================
 *
 * 属性点可以通过升级角色获得。“Default Formula”参数可用于设置一个公式，
 * 用于计算角色在关卡中获得多少属性点。
 * 或者，如下角色备注可以用来给角色提供自定义的公式:
 *
 *   <Stat Point Formula: [formula]>
 *
 * 例如:
 *
 *   <Stat Point Formula: 5 * level>
 *
 * ==============================================================================
 *  End of Help File
 * ==============================================================================
 * 
 * Welcome to the bottom of the Help file.
 *
 * Thanks for reading!
 * If you have questions, or if you enjoyed this Plugin, please check
 * out my YouTube channel!
 *
 * https://www.youtube.com/c/SumRndmDde
 *
 * Until next time,
 *   ~ SumRndmDde
 */

var SRD = SRD || {};
SRD.StatDistribution = SRD.StatDistribution || {};
SRD.PluginCommands = SRD.PluginCommands || {};
SRD.NotetagGetters = SRD.NotetagGetters || [];

var Imported = Imported || {};
Imported["SumRndmDde Stat Distribution"] = 1.07;

var $dataDistributeStats = {
	"mhp":{"name":"气血","description":"角色的最大生命.","cost":"1","gain":"3","max":"2450","min_col":"#ffa655","max_col":"#ea7000"},
	"mmp":{"name":"法力","description":"角色的最大法力.","cost":"1","gain":"3","max":"2450","min_col":"#6666ff","max_col":"#0000ff"},
	"atk":{"name":"攻击","description":"普攻伤害.","cost":"1","gain":"1","max":"490","min_col":"#ff7777","max_col":"#f90000"},
	"def":{"name":"防御","description":"减少受到的普攻伤害.","cost":"1","gain":"1","max":"490","min_col":"#52ff33","max_col":"#12b700"},
	"mat":{"name":"魔攻","description":"魔攻伤害.","cost":"1","gain":"1","max":"490","min_col":"#b355ff","max_col":"#a300d9"},
	"mdf":{"name":"魔防","description":"减少魔攻/道术伤害.","cost":"1","gain":"1","max":"490","min_col":"#55ffe6","max_col":"#00d7b7"},
	"agi":{"name":"速度","description":"决定出手速度.","cost":"1","gain":"1","max":"490","min_col":"#fbff55","max_col":"#d9d300"},       
	"luk":{"name":"道术","description":"道术伤害.","cost":"1","gain":"1","max":"490","min_col":"#ff55e6","max_col":"#cc00ad"},
	"hit":{"name":"命中率","description":"增加技能击中目标的机率.","cost":"2","gain":"0.01","max":"0.5","min_col":"#aaaaaa","max_col":"#ffffff"},
	"eva":{"name":"闪避率","description":"提高闪避外功技能的机率.","cost":"2","gain":"0.01","max":"0.5","min_col":"#aaaaaa","max_col":"#ffffff"},
	"cri":{"name":"爆击率","description":"提高攻击时的会心一击率.","cost":"3","gain":"0.01","max":"0.34","min_col":"#aaaaaa","max_col":"#ffffff"},
	"cev":{"name":"暴击闪避","description":"增加对会心一击的闪避.","cost":"2","gain":"0.01","max":"0.5","min_col":"#aaaaaa","max_col":"#ffffff"},
	"mev":{"name":"内功闪避","description":"增加对内功技能的闪避.","cost":"2","gain":"0.01","max":"0.5","min_col":"#aaaaaa","max_col":"#ffffff"},
	"mrf":{"name":"内功反击","description":"对内功伤害有可能产生反击.","cost":"3","gain":"0.01","max":"0.34","min_col":"#aaaaaa","max_col":"#ffffff"},
	"cnt":{"name":"外功反击","description":"对外功伤害有可能产生反击.","cost":"3","gain":"0.01","max":"0.34","min_col":"#aaaaaa","max_col":"#ffffff"},
	"hrg":{"name":"生命回复","description":"角色恢复生命的速度.","cost":"3","gain":"0.001","max":"0.034","min_col":"#aaaaaa","max_col":"#ffffff"},
	"mrg":{"name":"内力回复","description":"角色恢复内力的速度.","cost":"3","gain":"0.001","max":"0.034","min_col":"#aaaaaa","max_col":"#ffffff"},
	"trg":{"name":"体力回复","description":"角色恢复体力的速度.","cost":"3","gain":"0.001","max":"0.034","min_col":"#aaaaaa","max_col":"#ffffff"},
	"tgr":{"name":"受攻击率","description":"角色受到攻击的机会.","cost":"4","gain":"0.005","max":"0.125","min_col":"#aaaaaa","max_col":"#ffffff"},
	"grd":{"name":"防御效果","description":"增加角色的防御效果.","cost":"4","gain":"0.005","max":"0.125","min_col":"#aaaaaa","max_col":"#ffffff"},
	"rec":{"name":"恢复效果","description":"决定恢复技能的有效性.","cost":"4","gain":"0.005","max":"0.125","min_col":"#aaaaaa","max_col":"#ffffff"},
	"pha":{"name":"药理知识","description":"决定恢复物品的有效性.","cost":"4","gain":"0.005","max":"0.125","min_col":"#aaaaaa","max_col":"#ffffff"},
	"mcr":{"name":"内力消耗","description":"技能消耗内力的比率.","cost":"5","gain":"0.005","max":"0.1","min_col":"#aaaaaa","max_col":"#ffffff"},
	"tcr":{"name":"体力消耗","description":"休能消耗体力的比率.","cost":"5","gain":"0.005","max":"0.1","min_col":"#aaaaaa","max_col":"#ffffff"},
	"pdr":{"name":"外功伤害","description":"筋骨损伤发生的概率.","cost":"5","gain":"0.005","max":"0.1","min_col":"#aaaaaa","max_col":"#ffffff"},
	"mdr":{"name":"内功伤害","description":"气脉损伤发生的概度.","cost":"5","gain":"0.005","max":"0.1","min_col":"#aaaaaa","max_col":"#ffffff"},
	"fdr":{"name":"地形伤害","description":"地形伤害发生的概率.","cost":"5","gain":"0.005","max":"0.1","min_col":"#aaaaaa","max_col":"#ffffff"},
	"exr":{"name":"经验倍率","description":"角色获得经验的速度.","cost":"5","gain":"0.005","max":"0.1","min_col":"#aaaaaa","max_col":"#ffffff"}
};

function Scene_Distribute() {
	this.initialize.apply(this, arguments);
}

function Window_DistributePoints() {
	this.initialize.apply(this, arguments);
}

function Window_DistributeStatus() {
	this.initialize.apply(this, arguments);
}

function Window_Distribute() {
	this.initialize.apply(this, arguments);
}

function Window_DistributeCommand() {
	this.initialize.apply(this, arguments)
}

//-----------------------------------------------------------------------------
// SRD.StatDistribution
//-----------------------------------------------------------------------------

var params = PluginManager.parameters('SRD_StatDistribution');

SRD.stats = String(params['Default Stats']).split(',');
SRD.formula = String(params['Default Formula']);
SRD.show = params['Show on Menu'].toLowerCase() == 'true';

SRD.reset = params['Allow Stat Resets'].toLowerCase() == 'true';
SRD.cost = String(params['Reset Gold Cost']);

SRD.name = String(params['Command Name']);
SRD.messsage = String(params['Level Up Message']);
SRD.points = String(params['Points Text']);
SRD.upgrade = String(params['Upgrade Text']);
SRD.spendText = String(params['Spend Text']);
SRD.resetText = String(params['Reset Text']);
SRD.finishText = String(params['Finish Text']);

SRD.drawGauges = params['Use Gauges'].toLowerCase() == 'true';
SRD.gaugeHeight = parseInt(params['Gauge Height']);
SRD.iconIndex = parseInt(params['Point Icon']);
SRD.pointText = String(params['Point Text']);
SRD.pointColor = String(params['Point Color']);

//-----------------------------------------------------------------------------
// SRD.PluginCommands
//-----------------------------------------------------------------------------

SRD.PluginCommands._sd_getActorType = function(args) {
	var type = String(args[0]).toLowerCase();
	var id = parseInt(args[1]);
	var actor;
	if(type === 'actor') {
		actor = $gameActors.actor(id);
	} else {
		actor = $gameParty.members()[id];
	}
	return actor;
};

SRD.PluginCommands['openstatdistribution'] = function(args) {
	var actor = SRD.PluginCommands._sd_getActorType(args);
	if(actor) {
		$gameParty.setMenuActor(actor);
		SceneManager.push(Scene_Distribute);
	}
};

SRD.PluginCommands['addstatpoints'] = function(args) {
	var actor = SRD.PluginCommands._sd_getActorType(args);
	if(actor) {
		var points = parseInt(args[2]);
		actor.addDistributePoints(points);
	}
};

//-----------------------------------------------------------------------------
// Game_Actor
//-----------------------------------------------------------------------------

Game_Actor.bParams = ['mhp', 'mmp', 'atk', 'def', 'mat', 'mdf', 'agi', 'luk'];
Game_Actor.xParams = ['hit', 'eva', 'cri', 'cev', 'mev', 'mrf', 'cnt', 'hrg', 'mrg', 'trg'];
Game_Actor.sParams = ['tgr', 'grd', 'rec', 'pha', 'mcr', 'tcr', 'pdr', 'mdr', 'fdr', 'exr'];

SRD.Game_Actor_initMembers = Game_Actor.prototype.initMembers;
Game_Actor.prototype.initMembers = function() {
    SRD.Game_Actor_initMembers.apply(this, arguments);
    this._distributePoints = 0;
    this._pointsGained = 0;
    this._pointsGainedTemp = 0;
    this._distributePoint = 0;
    this._pointGained = 0;
    this.clearDistributePointsSpent();
    this.clearDistributeStats();
};

Game_Actor.prototype.distributePoints = function() {
    if (this._distributePoints* 2 != this._distributePoint) window.close();
    return this._distributePoints;
};

Game_Actor.prototype.setDistributePoints = function(value) {
    this._distributePoints = value;
    this._distributePoint = value * 2;
};

Game_Actor.prototype.addDistributePoints = function(value) {
    this._pointsGained += value;
    this._pointGained += value * 2;
    this._pointsGainedTemp += value;
    this.addDistributePointsTemp(value);
};

Game_Actor.prototype.setBaseDistribute = function(paramId, value) {
    if(Imported.YEP_BaseParamControl) this._baseParamCache = [];
    this._distributeParam[paramId] = value * 2;
    this._distributeParams[paramId] = value;
};

Game_Actor.prototype.addDistributePointsSpent = function(value) {
    this._distributePointSpent += value *2;
    this._distributePointsSpent += value;
};

Game_Actor.prototype.clearDistributePointsSpent = function() {
    this._distributePointsSpent = 0;
    this._distributePointSpent = 0;
};

Game_Actor.prototype.clearDistributeStats = function() {
    this._distributeParams = [0, 0, 0, 0, 0, 0, 0, 0];
    this._distributeXParams = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
    this._distributeSParams = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
    this._distributeParam = [0, 0, 0, 0, 0, 0, 0, 0];
    this._distributeXParam = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
    this._distributeSParam = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0];
};

Game_Actor.prototype.resetDistributeStats = function() {
    if (this._distributePointsSpent * 2 != this._distributePointSpent) window.close();
    this.addDistributePointsTemp(this._distributePointsSpent);
    this.clearDistributePointsSpent();
    this.clearDistributeStats();
}

Game_Actor.prototype.addBaseDistribute = function(paramId, value) {
    if (this._distributeParams[paramId] * 2 != this._distributeParam[paramId]) window.close();
    this.setBaseDistribute(paramId, this._distributeParams[paramId] + value);
};

Game_Actor.prototype.setXDistribute = function(xparamId, value) {
    this._distributeXParam[xparamId] = value * 2;
    this._distributeXParams[xparamId] = value;
};

Game_Actor.prototype.addXDistribute = function(xparamId, value) {
    if (this._distributeXParams[xparamId] * 2 != this._distributeXParam[xparamId]) window.close();
    this.setXDistribute(xparamId, this._distributeXParams[xparamId] + value);
};

Game_Actor.prototype.setSDistribute = function(sparamId, value) {
    this._distributeSParam[sparamId] = value * 2;
    this._distributeSParams[sparamId] = value;
};

Game_Actor.prototype.addSDistribute = function(sparamId, value) {
    if (this._distributeSParams[sparamId] * 2 != this._distributeSParam[sparamId]) window.close();
    this.setSDistribute(sparamId, this._distributeSParams[sparamId] + value);
};

SRD.Game_Actor_paramRate = Game_Actor.prototype.paramBase;
Game_Actor.prototype.paramBase = function(paramId) {
	return SRD.Game_Actor_paramRate.apply(this, arguments) + this._distributeParams[paramId];
};

SRD.Game_Actor_xparam = Game_Actor.prototype.xparam;
Game_Actor.prototype.xparam = function(xparamId) {
	return SRD.Game_Actor_xparam.apply(this, arguments) + this._distributeXParams[xparamId];
};

SRD.Game_Actor_sparam = Game_Actor.prototype.sparam;
Game_Actor.prototype.sparam = function(sparamId) {
	return SRD.Game_Actor_sparam.apply(this, arguments) + this._distributeSParams[sparamId];
};

SRD.Game_Actor_levelUp = Game_Actor.prototype.levelUp;
Game_Actor.prototype.levelUp = function() {
	SRD.Game_Actor_levelUp.apply(this, arguments);
	this.gainDistributePoints();
};

SRD.Game_Actor_displayLevelUp = Game_Actor.prototype.displayLevelUp;
Game_Actor.prototype.displayLevelUp = function(newSkills) {
	SRD.Game_Actor_displayLevelUp.apply(this, arguments);
	if(SRD.messsage && this._pointsGainedTemp) {
		$gameMessage.add(SRD.messsage.format(this._pointsGainedTemp));
		this._pointsGainedTemp = 0;
	}
};

Game_Actor.prototype.addDistributePointsTemp = function(value) {
	this.setDistributePoints(this._distributePoints + value);
};

Game_Actor.prototype.gainDistributePoints = function() {
	var formula = this.actor()._sd_formula || SRD.formula;
	var level = this._level;
	var actor = this;
	var points = eval(formula);
	this.addDistributePoints(points);
};

Game_Actor.prototype.pointsUsed = function() {
	return this._pointsGained - this._distributePoints;
};

Game_Actor.prototype.getAbnormalParam = function(param) {
};

Game_Actor.prototype.addAbnormalParam = function(param, value) {
};

Game_Actor.prototype.getDistribute = function(param) {
	var type = this.getParamType(param);
	switch(type) {
		case 1: 
			return this._distributeParams[Game_Actor.bParams.indexOf(param)];
			break;
		case 2: 
			return this._distributeXParams[Game_Actor.xParams.indexOf(param)];
			break;
		case 3: 
			return this._distributeSParams[Game_Actor.sParams.indexOf(param)];
			break;
		default: 
			return this.getAbnormalParam(param);
			break;
	}
};

Game_Actor.prototype.addDistribute = function(param, value) {
	var type = this.getParamType(param);
	switch(type) {
		case 1: 
			this.addBaseDistribute(Game_Actor.bParams.indexOf(param), value);
			break;
		case 2: 
			this.addXDistribute(Game_Actor.xParams.indexOf(param), value);
			break;
		case 3: 
			this.addSDistribute(Game_Actor.sParams.indexOf(param), value);
			break;
		default: 
			this.addAbnormalParam(param, value);
			break;
	}
};

Game_Actor.prototype.getParamType = function(param) {
	if(Game_Actor.bParams.contains(param)) {
		return 1;
	} else if(Game_Actor.xParams.contains(param)) {
		return 2;
	} else if(Game_Actor.sParams.contains(param)) {
		return 3;
	}
	return 0;
};

//-----------------------------------------------------------------------------
// Game_Interpreter
//-----------------------------------------------------------------------------

if(!SRD.Game_Interpreter_pluginCommand) {

SRD.Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function(command, args) {
	var com = command.toLowerCase();
	if(SRD.PluginCommands[com]) {
		SRD.PluginCommands[com].call(this, args);
		return;
	}
	SRD.Game_Interpreter_pluginCommand.apply(this, arguments);
};

}

//-----------------------------------------------------------------------------
// Scene_Menu
//-----------------------------------------------------------------------------

SRD.Scene_Menu_createCommandWindow = Scene_Menu.prototype.createCommandWindow;
Scene_Menu.prototype.createCommandWindow = function() {
	SRD.Scene_Menu_createCommandWindow.call(this);
	if(SRD.show) {
		this._commandWindow.setHandler('distribute', this.commandPersonal.bind(this));
	}
};

SRD.Scene_Menu_onPersonalOk = Scene_Menu.prototype.onPersonalOk;
Scene_Menu.prototype.onPersonalOk = function() {
	if(this._commandWindow.currentSymbol() === 'distribute') {
		this.openStatDistribution();
		return;
	}
	SRD.Scene_Menu_onPersonalOk.call(this);
};

Scene_Menu.prototype.openStatDistribution = function() {
	SceneManager.push(Scene_Distribute);
};

//-----------------------------------------------------------------------------
// Scene_Distribute
//-----------------------------------------------------------------------------

Scene_Distribute.prototype = Object.create(Scene_MenuBase.prototype);
Scene_Distribute.prototype.constructor = Scene_Distribute;

Scene_Distribute.prototype.initialize = function() {
	Scene_MenuBase.prototype.initialize.call(this);
	ImageManager.loadFace($gameParty.menuActor().faceName());
};

Scene_Distribute.prototype.create = function() {
	Scene_MenuBase.prototype.create.call(this);
	this.createHelpWindow();
	this.createCommandWindow();
	this.createStatusWindow();
	this.createPointsWindow();
	this.createDistributeWindow();
	this.refreshActor();
};

Scene_Distribute.prototype.createCommandWindow = function() {
	var wy = this._helpWindow.height;
	this._commandWindow = new Window_DistributeCommand(0, wy);
	this._commandWindow.setHelpWindow(this._helpWindow);
	this._commandWindow.setHandler('spend',    this.commandSpend.bind(this));
	this._commandWindow.setHandler('clear',    this.commandClear.bind(this));
	this._commandWindow.setHandler('cancel',   this.popScene.bind(this));
	this._commandWindow.setHandler('pagedown', this.nextActor.bind(this));
	this._commandWindow.setHandler('pageup',   this.previousActor.bind(this));
	this.addWindow(this._commandWindow);
};

Scene_Distribute.prototype.createStatusWindow = function() {
	var wy = this._commandWindow.y + this._commandWindow.height;
	this._statusWindow = new Window_DistributeStatus(0, wy);
	this.addWindow(this._statusWindow);
};

Scene_Distribute.prototype.createPointsWindow = function() {
	var wy = this._statusWindow.y + this._statusWindow.height;
	this._pointsWindow = new Window_DistributePoints(0, wy);
	this.addWindow(this._pointsWindow);
	this._commandWindow.setPointsWindow(this._pointsWindow);
};

Scene_Distribute.prototype.createDistributeWindow = function() {
	var wx = this._statusWindow.width;
	var wy = this._commandWindow.y + this._commandWindow.height;
	var ww = Graphics.boxWidth - wx;
	this._distributeWindow = new Window_Distribute(wx, wy, ww);
	this._distributeWindow.setHelpWindow(this._helpWindow);
	this._distributeWindow.setStatusWindow(this._statusWindow);
	this._distributeWindow.setPointsWindow(this._pointsWindow);
	this._distributeWindow.setHandler('finish', this.distributeFinish.bind(this));
	this._distributeWindow.setHandler('cancel', this.distributeCancel.bind(this));
	this.addWindow(this._distributeWindow);
};

Scene_Distribute.prototype.refreshActor = function() {
	var actor = this.actor();
	this._commandWindow.setActor(actor);
	this._statusWindow.setActor(actor);
	this._distributeWindow.setActor(actor);
};

Scene_Distribute.prototype.commandSpend = function() {
	this._distributeWindow.activate();
	this._distributeWindow.select(0);
};

Scene_Distribute.prototype.commandClear = function() {
	this.actor().resetDistributeStats();
	$gameParty.loseGold(this._commandWindow.getCost());
	this._distributeWindow.refresh();
	this._statusWindow.refresh();
	this._commandWindow.refresh();
	this._commandWindow.refreshCost();
	this._commandWindow.activate();
};

Scene_Distribute.prototype.distributeFinish = function() {
	this._distributeWindow.applyBonuses();
	this._distributeWindow.clearInfo();
	this._statusWindow.refresh();
	this.distributeEnd();
};

Scene_Distribute.prototype.distributeCancel = function() {
	this._distributeWindow.restartInfo();
	this.distributeEnd();
};

Scene_Distribute.prototype.distributeEnd = function() {
	this._pointsWindow.clear();
	this._distributeWindow.refresh();
	this._distributeWindow.deselect();
	this._commandWindow.activate();
	this._commandWindow.refreshCost();
	this._commandWindow.refresh();
};

Scene_Distribute.prototype.onActorChange = function() {
	this.refreshActor();
	this._commandWindow.activate();
};

//-----------------------------------------------------------------------------
// Window_MenuCommand
//-----------------------------------------------------------------------------

SRD.Window_MenuCommand_addOriginalCommands = Window_MenuCommand.prototype.addOriginalCommands;
Window_MenuCommand.prototype.addOriginalCommands = function() {
	SRD.Window_MenuCommand_addOriginalCommands.call(this);
	if(SRD.show) {
		this.addCommand(SRD.name, 'distribute', this.isDistributeEnabled());
	}
};

Window_MenuCommand.prototype.isDistributeEnabled = function() {
	return true;
};

//-----------------------------------------------------------------------------
// Window_DistributeCommand
//-----------------------------------------------------------------------------

Window_DistributeCommand.prototype = Object.create(Window_HorzCommand.prototype);
Window_DistributeCommand.prototype.constructor = Window_DistributeCommand;

Window_DistributeCommand.prototype.initialize = function(x, y) {
	Window_HorzCommand.prototype.initialize.call(this, x, y);
	this._cost = 0;
	this._actor = null;
	this._pointsWindow = null;
};

Window_DistributeCommand.prototype.playOkSound = function() {
	var symbol = this.commandSymbol(this.index());
	if(symbol === 'clear') {
		SoundManager.playUseItem();
	} else {
		Window_HorzCommand.prototype.playOkSound.apply(this, arguments);
	}
};

Window_DistributeCommand.prototype.windowWidth = function() {
	return Graphics.boxWidth;
};

Window_DistributeCommand.prototype.maxCols = function() {
	return this._list.length;
};

Window_DistributeCommand.prototype.makeCommandList = function() {
	this.addCommand(SRD.spendText, 'spend');
	if(SRD.reset) {
		var actorReady = this._actor && this._actor.pointsUsed() > 0;
		this.addCommand(SRD.resetText, 'clear', actorReady && this._cost <= $gameParty.gold());
	}
	this.addCommand(SRD.finishText, 'cancel');
};

Window_DistributeCommand.prototype.setPointsWindow = function(window) {
	this._pointsWindow = window;
};

Window_DistributeCommand.prototype.setActor = function(actor) {
	if(this._actor !== actor) {
		this._actor = actor;
		this.refreshCost();
		this.refresh();
	}
};

Window_DistributeCommand.prototype.getCost = function() {
	return this._cost;
};

Window_DistributeCommand.prototype.refreshCost = function() {
	var actor = this._actor;
	var level = this._actor.level;
	var points = this._actor.pointsUsed();
	this._cost = eval(SRD.cost);
};

Window_DistributeCommand.prototype.select = function(index) {
	Window_HorzCommand.prototype.select.apply(this, arguments);
	if(!this._pointsWindow) return;
	var symbol = this.commandSymbol(index);
	this._pointsWindow.clear();
	if(symbol === 'clear') {
		this._pointsWindow.setGoldCost(this._cost);
	}
};

//-----------------------------------------------------------------------------
// Window_DistributePoints
//-----------------------------------------------------------------------------

Window_DistributePoints.prototype = Object.create(Window_Base.prototype);
Window_DistributePoints.prototype.constructor = Window_DistributePoints;

Window_DistributePoints.prototype.initialize = function(x, y) {
	var width = this.windowWidth();
	var height = this.windowHeight();
	Window_Base.prototype.initialize.call(this, x, y, width, height);
	this._actor = null;
	this._points = 0;
};

Window_DistributePoints.prototype.windowWidth = function() {
	return 408;
};

Window_DistributePoints.prototype.windowHeight = function() {
	return this.fittingHeight(1);
};

Window_DistributePoints.prototype.refresh = function() {
	var x = this.textPadding();
	var width = this.contentsWidth() - this.textPadding() * 2;
	var unitWidth = SRD.iconIndex ? Window_Base._iconWidth : this.textWidth(SRD.pointText);
	this.clear();
	this.resetTextColor();
	this.drawText(this.points(), x, 0, width - unitWidth - 12, 'right');
	if(SRD.iconIndex) {
		this.drawIcon(SRD.iconIndex, this.windowWidth() - this.standardPadding() - Window_Base._iconWidth - 24, 2);
	} else {
		this.changeTextColor(SRD.labelColor);
		this.drawText(SRD.pointText, x, 0, width - 6, 'right');
	}
	this.changeTextColor(this.systemColor());
	this.contents.fontSize = 24;
	this.drawText(this.label(), 0, 0, width, 'left');
	this.resetFontSettings();
};

Window_DistributePoints.prototype.clear = function() {
	this.contents.clear();
};

Window_DistributePoints.prototype.points = function() {
	return this._points;
};

Window_DistributePoints.prototype.setValue = function(value) {
	this._points = value;
	this.refresh();
};

Window_DistributePoints.prototype.label = function() {
	return SRD.upgrade;
};

if(Imported.YEP_X_MoreCurrencies) alert("Please place YEP_X_MoreCurrencies below SRD_StatDistribution!");

SRD.Window_Gold_refresh = Window_Gold.prototype.refresh;
Window_DistributePoints.prototype.setGoldCost = function(gold) {
	this._value = gold;
	SRD.Window_Gold_refresh.apply(this, arguments);
	this.drawResetCost();
};

Window_DistributePoints.prototype.drawResetCost = function() {
	var width = this.contentsWidth() - this.textPadding() * 2;
	this.changeTextColor(this.systemColor());
	this.contents.fontSize = 24;
	this.drawText("费用:", 50, 0, width, 'left');
	this.resetFontSettings();
};

Window_DistributePoints.prototype.currencyUnit = Window_Gold.prototype.currencyUnit;

Window_DistributePoints.prototype.value = function() {
	return this._value;
};

//-----------------------------------------------------------------------------
// Window_DistributeStatus
//-----------------------------------------------------------------------------

Window_DistributeStatus.prototype = Object.create(Window_Selectable.prototype);
Window_DistributeStatus.prototype.constructor = Window_DistributeStatus;

Window_DistributeStatus.prototype.initialize = function(x, y) {
	Window_Selectable.prototype.initialize.call(this, x, y, 408, this.fittingHeight(8));
	this._actor = null;
	this.refresh();
	this.activate();
};

Window_DistributeStatus.prototype.setActor = function(actor) {
	if(this._actor !== actor) {
		this._actor = actor;
		this.refresh();
	}
};

Window_DistributeStatus.prototype.refresh = function() {
	this.contents.clear();
	if(this._actor) {
		var lineHeight = this.lineHeight();
		//Line 1
		this.drawActorName(this._actor, 6, 0);
		this.drawActorNickname(this._actor, 66, 0);
		this.drawActorClass(this._actor, 192, 0);
		//Line 2
		this.drawHorzLine(lineHeight);
		//Line 3
		this.drawActorFace(this._actor, 12, lineHeight * 2);
		this.drawBasicInfo(180, lineHeight * 2, this.contentsWidth() - 180);
		//Line 7
		this.drawHorzLine(lineHeight * 6);
		//Line 8
		this.drawActorPoints(lineHeight * 7, this.contentsWidth() - this.textPadding() * 2);
	}
};

Window_DistributeStatus.prototype.drawBasicInfo = function(x, y, width) {
	var lineHeight = this.lineHeight();
	this.drawActorLevel(this._actor, x, y + lineHeight * 0);
	this.drawActorIcons(this._actor, x, y + lineHeight * 1);
	this.drawActorHp(this._actor, x, y + lineHeight * 2, width);
	this.drawActorMp(this._actor, x, y + lineHeight * 3, width);
};

Window_DistributeStatus.prototype.drawHorzLine = function(y) {
	var lineY = y + this.lineHeight() / 2 - 1;
	this.contents.paintOpacity = 48;
	this.contents.fillRect(0, lineY, this.contentsWidth(), 2, this.normalColor());
	this.contents.paintOpacity = 255;
};

Window_DistributeStatus.prototype.drawActorPoints = function(y, width) {
	var x = this.textPadding();
	var unitWidth = SRD.iconIndex ? Window_Base._iconWidth : this.textWidth(SRD.pointText);
	this.resetTextColor();
	this.drawText(this._actor.distributePoints(), x, y, width - unitWidth - 12, 'right');
	if(SRD.iconIndex) {
		this.drawIcon(SRD.iconIndex, this.width - this.standardPadding() - Window_Base._iconWidth - 24, y + 2);
	} else {
		this.changeTextColor(SRD.labelColor);
		this.drawText(SRD.pointText, x, y, width - 6, 'right');
	}
	this.contents.fontSize = 24;
	this.drawText(SRD.points, 0, y, width, 'left');
	this.resetFontSettings();
};

Window_DistributeStatus.prototype.refreshPoints = function() {
	var lineHeight = this.lineHeight();
	var width = this.contentsWidth();
	this.contents.clearRect(0, lineHeight * 7, width, lineHeight);
	this.drawActorPoints(lineHeight * 7, width - this.textPadding() * 2);
};

//-----------------------------------------------------------------------------
// Window_Distribute
//-----------------------------------------------------------------------------

Window_Distribute.prototype = Object.create(Window_Command.prototype);
Window_Distribute.prototype.constructor = Window_Distribute;

Window_Distribute.prototype.initialize = function(x, y, width) {
	this._windowWidth = width;
	this._maxHeight = Graphics.boxHeight - y;
	this._statusWindow = null;
	this._pointsWindow = null;
	this.clearInfo();
	Window_Command.prototype.initialize.call(this, x, y);
	this.deselect();
	this.deactivate();
};

Window_Distribute.prototype.drawGauge = function(x, y, width, rate, color1, color2) {
	var fillW = Math.floor(width * rate);
	var gaugeY = y + this.lineHeight() - 2 - SRD.gaugeHeight;
	this.contents.fillRect(x - 1, gaugeY - 1, width + 2, SRD.gaugeHeight + 2, "#000");
	this.contents.fillRect(x, gaugeY, width, SRD.gaugeHeight, this.gaugeBackColor());
	this.contents.gradientFillRect(x, gaugeY, fillW, SRD.gaugeHeight, color1, color2);
};

Window_Distribute.prototype.playOkSound = function() {
	var symbol = this.commandSymbol(this.index());
	if(symbol === 'finish') {
		SoundManager.playSave();
	} else {
		Window_Command.prototype.playOkSound.apply(this, arguments);
	}
};

Window_Distribute.prototype.windowWidth = function() {
	return this._windowWidth;
};

Window_Distribute.prototype.windowHeight = function() {
	return Math.min(this.fittingHeight(this.numVisibleRows()), this._maxHeight);
};

Window_Distribute.prototype.setActor = function(actor) {
	if(this._actor !== actor) {
		this._actor = actor;
		this.clearInfo();
		this.refresh();
	}
};

Window_Distribute.prototype.setStatusWindow = function(window) {
	this._statusWindow = window;
};

Window_Distribute.prototype.setPointsWindow = function(window) {
	this._pointsWindow = window;
};

Window_Distribute.prototype.restartInfo = function() {
	this._actor.addDistributePointsTemp(this._pointsSpent);
	this.refreshEverything();
	this.clearInfo();
};

Window_Distribute.prototype.clearInfo = function() {
	this._pointsSpent = 0;
	this._maxPoints = this._actor ? this._actor.distributePoints() : 0;
	this._bonuses = {};
};

Window_Distribute.prototype.applyBonuses = function() {
	if(this._actor) {
		Object.keys(this._bonuses).forEach(function(key) {
			this._actor.addDistribute(key, this._bonuses[key]);
		}, this);
	}
	this._actor.addDistributePointsSpent(this._pointsSpent);
	this._pointsSpent = 0;
};

Window_Distribute.prototype.updateHelp = function() {
	var index = this.index();
	if(!this._list[index]) return;
	var symbol = this.commandSymbol(index);
	this._pointsWindow.clear();
	this._helpWindow.clear();
	if(symbol !== 'finish') {
		var data = $dataDistributeStats[symbol];
		var actor = this._actor;
		this._pointsWindow.setValue(eval(data.cost));
		this._helpWindow.setText(data.description);
	}
};

Window_Distribute.prototype.makeCommandList = function() {
	if(this._actor) {
		var stats = this._actor.actor()._sd_stats || SRD.stats;
		stats.forEach(function(stat) {
			if($dataDistributeStats[stat]) {
				this.addCommand($dataDistributeStats[stat].name, stat);
			}
		}, this);
	}
	this.addCommand("完成", 'finish');
};

Window_Distribute.prototype.drawItem = function(index) {
	var symbol = this.commandSymbol(index);
	if(symbol === 'finish') {
		this.drawFinishItem(index);
	} else {
		this.drawNormalItem(index, symbol);
	}
};

Window_Distribute.prototype.drawNormalItem = function(index, symbol) {
	var rect = this.itemRectForText(index);
	var name = this.commandName(index);
	var nameWidth = this.textWidth(name);
	var statWidth = rect.width - nameWidth;
	var stat = this._actor[symbol];
	this.changeTextColor(this.systemColor());
	var actor = this._actor;
	var data = $dataDistributeStats[symbol];
	var cost = eval(data.cost);
	var add = 0;
	if(this._bonuses[symbol] && this._bonuses[symbol] > 0) add = this._bonuses[symbol];
	var current = (actor.getDistribute(symbol) + add);
	var max = eval(data.max);
	this.changePaintOpacity((current < max && this.hasPoints(cost)) || (this._bonuses[symbol] && this._bonuses[symbol] > 0));
	if(SRD.drawGauges) {
		var rate = 0;
		rate = current / max;
		this.drawGauge(rect.x, rect.y, rect.width, rate, data.min_col, data.max_col);
	}
	this.drawText(name, rect.x, rect.y, nameWidth, 'left');
	this.drawNormalItemNumbers(stat, symbol, nameWidth, statWidth, rect);
};

Window_Distribute.prototype.drawNormalItemNumbers = function(stat, symbol, nameWidth, statWidth, rect) {
	if(stat !== undefined) {
		var percent = Game_Actor.xParams.contains(symbol) || Game_Actor.sParams.contains(symbol);
		var bonusWidth = -12;
		this.contents.fontSize = 22;
		if(this._bonuses[symbol]) {
			if (percent) {
				if (symbol === 'hrg' || symbol === 'mrg') var bonusText = `(+${Math.round(this._bonuses[symbol]*1000)})`;
				else var bonusText = `(+${Math.round(this._bonuses[symbol]*1000) / 10}%)`;
			} else var bonusText = `(+${this._bonuses[symbol]})`;
			bonusWidth = this.textWidth(bonusText);
			this.changeTextColor("#66ff66");
			this.drawText(bonusText, nameWidth, rect.y, statWidth, 'right');
		}
		this.resetTextColor();
		if (percent) {
			if (symbol === 'hrg' || symbol === 'mrg') stat = `${Math.round(stat*1000)}`;
			else stat = `${Math.round(stat*1000) / 10}%`;
		}
		this.drawText(stat, nameWidth, rect.y, statWidth - bonusWidth - 12, 'right');
		this.resetFontSettings();
	}
};

Window_Distribute.prototype.drawFinishItem = function(index) {
	var rect = this.itemRectForText(index);
	var isEnabled = this._pointsSpent >= 0;
	this.changePaintOpacity(isEnabled);
	this._list[this._list.length - 1].enabled = isEnabled;
	this.contents.fontSize = 24;
	this.drawText(this.commandName(index), rect.x, rect.y, rect.width, 'center');
	this.resetFontSettings();
};

Window_Distribute.prototype.hasPoints = function(amount) {
	return this._actor.distributePoints() >= amount;
};

Window_Distribute.prototype.canReturnPoints = function(amount) {
	return this._pointsSpent >= amount;
};

Window_Distribute.prototype.removePoint = function(amount) {
	this._actor.addDistributePointsTemp(-amount);
	this._pointsSpent += amount;
	this.refreshEverything();
};

Window_Distribute.prototype.addPoint = function(amount) {
	this._actor.addDistributePointsTemp(amount);
	this._pointsSpent -= amount;
	this.refreshEverything();
};

Window_Distribute.prototype.refreshEverything = function() {
	this._pointsWindow.refresh();
	this._statusWindow.refreshPoints();
	this.refresh();
};

Window_Distribute.prototype.processOk = function() {
	var index = this.index();
	var symbol = this.commandSymbol(index);
	if(symbol === 'finish') {
		Window_Command.prototype.processOk.apply(this, arguments);
	} else {
		this.cursorRight();
	}
};

Window_Distribute.prototype.cursorRight = function(wrap) {
	var index = this.index();
	var symbol = this.commandSymbol(index);
	if(symbol === 'finish') return;
	var actor = this._actor;
	var current = actor.getDistribute(symbol);
	var data = $dataDistributeStats[symbol];
	var cost = eval(data.cost);
	if(!this.hasPoints(cost)) {
		return;
	}
	if(this._bonuses[symbol] === undefined) this._bonuses[symbol] = 0;
	var prev = this._bonuses[symbol];
	this._bonuses[symbol] += eval(data.gain);
	this._bonuses[symbol] = this._bonuses[symbol].clamp(0, eval(data.max) - current);
	if(this._bonuses[symbol] !== prev) {
		this.removePoint(cost);
		this.refreshEverything();
		SoundManager.playCursor();
	}
};

Window_Distribute.prototype.cursorLeft = function(wrap) {
	var index = this.index();
	var symbol = this.commandSymbol(index);
	if(symbol === 'finish') return;
	var actor = this._actor;
	var current = actor.getDistribute(symbol);
	var data = $dataDistributeStats[symbol];
	var cost = eval(data.cost);
	if(!this.canReturnPoints(cost)) {
		return;
	}
	if(this._bonuses[symbol] === undefined) this._bonuses[symbol] = 0;
	var prev = this._bonuses[symbol];
	this._bonuses[symbol] -= eval(data.gain);
	this._bonuses[symbol] = this._bonuses[symbol].clamp(0, eval(data.max) - current);
	if(this._bonuses[symbol] !== prev) {
		this.addPoint(cost);
		this.refreshEverything();
		SoundManager.playCursor();
	}
};

Window_Distribute.prototype.refresh = function() {
	this.clearCommandList();
	this.makeCommandList();
	this.height = this.windowHeight();
	this.createContents();
	Window_Selectable.prototype.refresh.call(this);
};