//=============================================================================
// sound sepher Engine - Battle Speed Up 高機能版
// SSEP_BattleSpeedUp_v2.js
// Version: 2.01
//=============================================================================

var Imported = Imported || {};
Imported.SSEP_BattleSpeedUp = true;

var Sepher = Sepher || {};

//=============================================================================
/*:
 * @plugindesc [ver2.01] 这是一个增加战斗速度的插件。与Yanfly引擎兼容。 
 * 利用時は、必ずYanflyEngineの後に読み込んでください。
 * @author Shoichiro Sakamoto(sound sepher)
 *
 * @help
 * ------------------------------------------------------------------------------
 * ■sound sepher Engine - "Battle Speed UP" Plugin (Ver2.01 '15 11/07)
 * ------------------------------------------------------------------------------
 * 制作：坂本昌一郎 / Shoichiro Sakamoto (sound sepher)
 * Web ：http://sepher.jp/
 * 
 * ------------------------------------------------------------------------------
 * ■内容说明
 * ------------------------------------------------------------------------------
 * 这个插件是一个提高战斗速度的插件。
 * 支持正视图和侧视图。 
 * 
 * 还有 Yanfly Engine 的 BattleCore，动作序列包（X_ActSeqPack1 等），
 * 支持 YEP Active Time Battle、VictoryAftermath 和 Ellye 的 Active Time Battle。 
 * 
 * ------------------------------------------------------------------------------
 * ■使用方法
 * ------------------------------------------------------------------------------
 * 如果按住确定按钮，战斗将进入快进模式。
 * 此外，如果您按下屏幕上显示的切换按钮，它将始终快进。
 * 您可以通过按 shift (X) 键获得相同的效果。 
 * 
 * 所有这些也可以关闭。 
 * 
 * 您可以微调战斗速度和切换图像设置。
 * 如果您不明白设置项的含义，请使用默认状态。
 * 通常，更改常规设置的值就足够了。 
 * 
 * 它支持 Yanfly Engine 和 Ellye 的 ATB，
 * 但除非您将此插件放在插件管理器中的这些插件之后，否则它将无法正常运行。
 * 另外，在使用 Ellye 的 ATB 时，删除第 108 行“(function () {”和第 550 行“}) ();”
 * 以打开函数作用域。 
 * 
 * ------------------------------------------------------------------------------
 * ■注意事项 
 * ------------------------------------------------------------------------------
 * ・一定要在Yanfly Engine之后加载，避免使用时发生冲突。
 * ・我们不提供支持，因此请自担风险使用。
 * ・できればtxtやクレジットに、前述の著作権表記を掲載して頂けたら嬉しいです。
 * 
 * ------------------------------------------------------------------------------
 * ■参数说明 
 * ------------------------------------------------------------------------------
 * ・BattleSpeed (Default)：没有按OK按钮时的战斗速度的倍数。
 * ・BattleSpeed (Boost)　：是按下OK按钮时战斗速度的倍数。
 * 　　　　　　　　　　　 　它在 4 倍速以上没有效果。 
 * ・OkayKeyBoost　　　　 ：指定是否通过按住OK按钮来加速。
 * 　　　　　　　　　　　 　(true:是 / false:否)
 * ・VisibleSwitch　　　　：升压选择开关是否可以显示。。
 * 　　　　　　　　　　　 　隐藏此开关也会禁用 Boost Tggle 开关。
 * 　　　　　　　　　　　 　(true:显示 / false:隐藏)
 * ・BoostToggleSwitch　　：切换升压状态时要按下的按钮的名称。 
 * ------------------------------------------------------------------------------
 * ・StateIcon　　　　　　：调整敌人状态图标的显示速度。 
 * ・StateOverLay　　　　 ：调整队友状态动画的动画速度。 
 * ・Weapon　　　　　　　 ：队友在侧视图中挥动武器的速度。 
 * ・Motion　　　　　　　 ：队友和敌人的动画速度。 
 * 　　　　　　　　　　　 　影响待机动画、移动动画等的速度。
 * 　　　　　　　　　　　 　此外，武器和动作是同步的，
 * 　　　　　　　　　　　 　所以最好保持数值相同。 
 * ・Balloon　　　　　　　：气泡的显示速度。 
 * ・Damage　　　　　　　 ：伤害值的连续帧数。 
 * ・DamageMin　　　　　　：这是损坏值的最小连续帧数。 
 * 　　　　　　　　　　　 　如果提前设置损坏，将很难看到，因此这是一种预防措施。 
 * ------------------------------------------------------------------------------
 * ・LogAnime BaseDelay　 ：战斗日志窗口中显示的帧数。 
 * ・LogAnime NextDelay　 ：战斗日志窗口中的连续帧数。 
 * ・LogWaitCount Default ：将战斗日志窗口中显示的等待帧数增加指定倍数。 
 * 　　　　　　　　　　　 　
 * ・LogWaitCount Boost　 ：将战斗日志窗口中显示的等待帧数增加指定倍数。
 * 　　　　　　　　　　　 　（在升压状态）按住 OK 按钮可以使待机帧更快。 
 * 　　　　　　　　　　　 　
 * 　　　　　　　　　　　 　
 * ------------------------------------------------------------------------------
 * ・SE BoostON　　　　　 ：这是boost状态开启时的音效。 
 * ・SE BoostOFF　　　　　：这是boost状态关闭时的音效。 
 * ・SE Volume　　　　　　：音效的音量。 
 * ・SwitchImage　　　　　：要显示的开关图像的文件名。 
 * 　　　　　　　　　　　　浏览到系统文件夹system。 
 * ・SwitchX　　　　　　　：指定显示开关图像的位置 X。 
 * ・SwitchY　　　　　　　：指定显示开关图像的位置 Y。 
 * ・SwitchWidth　　　　　：指定一个开关的宽度大小。 
 * ・SwitchHeight　　　　 ：指定一个开关的垂直宽度大小。 
 * ・SwitchTop　　　　　　：指定要显示的开关图像的阶段数。
 * 　　　　　　　　　　　 　0 为最上面一行，随着数字的增加，该行向下移动。 
 * 　　　　　　　　　　　 　默认情况下，它指的是气泡的第二行[？]。
 * ・SwitchLeft　　　　　 ：指定切换动画的起始帧数。 
 * ・SwitchAnimePattern　 ：指定切换动画的最后一帧。 
 * 　　　　　　　　　　　 　默认情况下，循环播放 2 到 8 帧。 
 * ・SwitchAnimeSpeed　　 ：通过帧数指定切换动画的显示速度。 
 * ------------------------------------------------------------------------------
 * ★以下是 Yanfly Engine-BattleCore 的附加选项 
 * 
 * ・YEP Battle MotionWait：值越小，攻击效果的权重越快。 
 * ------------------------------------------------------------------------------
 * ★以下是 Yanfly Engine --Active Time Battle 的附加选项 
 * 
 * ・YEP ATB BoostSwitch　：安装 YEP ATB 时，指定是否升压。
 * 　　　　　　　　　　　 　(true:是 / false:否)
 * ------------------------------------------------------------------------------
 * ★以下是 Ellye 和 Yanfly ATB 的附加选项（均支持） 
 * 
 * ・ATB Speed(Default)　 ：将正常的 AT 仪表速度指定为倍数。 
 * ・ATB Speed(Boost)　　 ：指定倍数提升时 AT 仪表的速度。 
 * ------------------------------------------------------------------------------
 * ★以下是 Yanfly Engine-Victory Aftermath 的其他选项 
 * 
 * ・YEP Victory Motion　 ：如果在胜利时增加，重量会减少。
 * 　　　　　　　　　　　 　(true:缩短 / false:不缩短)
 * ------------------------------------------------------------------------------
 *
 * @param ---General Setting---
 * @default
 * @param BattleSpeed (Default)
 * @desc 默认动画速度。 初始值为 1。
 * 以指定数量的倍数增加武器攻击以外的动画速度。 
 * @default 1
 * @param BattleSpeed (Boost)
 * @desc 输入确定键时的动画速度。 初始值为 2。
 * 以指定数量的倍数增加武器攻击以外的动画速度。 
 * @default 2
 * @param OkayKeyBoost
 * @desc 是否通过按住OK键来加速。
 * 初始值为true。
 * @default true
 * @param VisibleSwitch
 * @desc 是否显示升压开关。 初始值为true。
 * 否则，BoostToggleSwitch也将禁用。
 * @default true
 * @param BoostToggleSwitch
 * @desc 输入按键名称以切换升压状态。
 * 初始值为 shift。 相当于打击垫上的 Y 按钮。 
 * @default shift
 * @param ---Detail Setting---
 * @default
 * @param StateIcon
 * @desc 指定状态图标的动画速度。 
 * 初期値は40。
 * @default 40
 * @param StateOverlay
 * @desc 指定状态叠加的动画速度 
 * 初期値は8。
 * @default 8
 * @param Weapon
 * @desc 指定侧视图中武器攻击的动画速度。
 * 初始值为 12。由于它与 Motion 同步，因此建议使用相同的值。 
 * @default 12
 * @param Motion
 * @desc 指定动作动作的动画速度。
 * 初始值为 12。由于它与武器同步，因此建议使用相同的值。 
 * @default 12
 * @param Balloon
 * @desc 指定气泡的动画速度。 
 * 初期値は12。
 * @default 12
 * @param Damage
 * @desc 指定损坏弹出窗口的连续帧数。
 * 初始值为 90。要小心，因为如果太早，它就不会被看到。 
 * @default 90
 * @param DamageMin
 * @desc 指定损坏弹出窗口的最小连续帧数。
 * 初始值为 60。 我们建议您保持原样。 
 * @default 60
 * @param --BattleLog Setting--
 * @default
 * @param LogAnime BaseDelay
 * @desc 指定在战斗日志窗口中显示的帧数。
 * 初始值为 8。 不受战速影响（默认）。 
 * @default 8
 * @param LogAnime NextDelay
 * @desc 指定战斗日志窗口中的连续帧数。
 * 初始值为 12。 不受战速影响（默认）。
 * @default 12
 * @param LogWaitCount Default
 * @desc 战斗日志中显示的待机帧加速指定倍数。
 * 初始值为 1。增加该值将增加战斗日志的速度。 
 * @default 1
 * @param LogWaitCount Boost
 * @desc 战斗日志提升期间的待机帧以指定倍数加速。
 * 初始值为 20。 增加该值将增加战斗日志的速度。 
 * @default 2
 * @param ---Switch Setting---
 * @default
 * @param SE BoostON
 * @desc 这是boost状态开启时的音效。 
 * 初期値はDecision2。
 * @default Decision2
 * @param SE BoostOFF
 * @desc 这是boost状态关闭时的音效。 
 * 初期値はDecision2。
 * @default Decision2
 * @param SE Volume
 * @desc 音效的音量。 初始值为 50。 
 * @default 50
 * @param SwitchImage
 * @desc 图像的文件名。浏览系统文件夹system。
 * 初期値はBalloon。
 * @default Balloon
 * @param SwitchX
 * @desc 开关位置X。初始值为10。 
 * @default 10
 * @param SwitchY
 * @desc 开关位置Y。初始值为10。 
 * @default 10
 * @param SwitchWidth
 * @desc 开关宽度的大小。 初始值为 48。 
 * @default 48
 * @param SwitchHeight
 * @desc 开关垂直宽度的大小。 初始值为 48。 
 * @default 48
 * @param SwitchTop
 * @desc 切换动画的阶段数。 0 是顶行。 
 * 初期値は2。
 * @default 2
 * @param SwitchLeft
 * @desc 切换动画的起始帧数。 0 是第一帧。
 * 初期値は1。
 * @default 1
 * @param SwitchAnimePattern
 * @desc 这是切换动画的最后一帧。 
 * 初期値は8。
 * @default 8
 * @param SwitchAnimeSpeed
 * @desc Switch 的动画速度。 初始值为 5。 
 * @default 5
 * @param ---YEP BattleCore---
 * @default
 * @param YEP Battle MotionWait
 * @desc 只有安装了 YEP BattleCore 才能设置。 
 * 初始值为 20。 减小该值将增加攻击效果的权重。 
 * @default 20
 * @param ---YEP ATB---
 * @default
 * @param YEP ATB BoostSwitch
 * @desc 仅当安装了 YEP ATB 时才可以设置。 
 * 初始值为真。 如果设置为 true，则 ATB 在提升时也会加速。
 * @default true
 * @param ---ATB Speed---
 * @default
 * @param ATB Speed(Default)
 * @desc 只有安装了 Ellye 或 YEP ATB 才可以设置。 
 * 初始值为 1。 以倍数指定默认 AT 仪表的速度。 
 * @default 1
 * @param ATB Speed(Boost)
 * @desc 只有安装了 Ellye 或 YEP ATB 才可以设置。 
 * 初始值为 2。 指定倍数提升时 AT 仪表的速度。 
 * @default 2
 * @param ---YEP Victory AM---
 * @default
 * @param YEP Victory Wait
 * @desc 仅当安装了 YEP Victory Aftermath 时才可以设置。 
 * 初期値はtrue。 提升时获胜时增加重量。 
 * @default true
 *
 */


//------------------------------------------------------------------------------
// 全局变量
//------------------------------------------------------------------------------

Sepher.Parameters = PluginManager.parameters('SSEP_BattleSpeedUp_v2');
Sepher.Param = Sepher.Param || {};

Sepher.Param.BattleSpeedDefault	= Number(Sepher.Parameters['BattleSpeed (Default)']);
Sepher.Param.BattleSpeedBoost	= Number(Sepher.Parameters['BattleSpeed (Boost)']);
Sepher.Param.OkayKeyBoost		= String(Sepher.Parameters['OkayKeyBoost']);
Sepher.Param.VisibleSwitch		= String(Sepher.Parameters['VisibleSwitch']);
Sepher.Param.BoostToggleSwitch	= String(Sepher.Parameters['BoostToggleSwitch']);
Sepher.Param.StateIcon			= Number(Sepher.Parameters['StateIcon']);
Sepher.Param.StateOverlay		= Number(Sepher.Parameters['StateOverlay']);
Sepher.Param.Weapon				= Number(Sepher.Parameters['Weapon']);
Sepher.Param.Balloon			= Number(Sepher.Parameters['Balloon']);
Sepher.Param.Motion				= Number(Sepher.Parameters['Motion']);
Sepher.Param.Damage				= Number(Sepher.Parameters['Damage']);
Sepher.Param.DamageMin			= Number(Sepher.Parameters['DamageMin']);
Sepher.Param.LogBase			= Number(Sepher.Parameters['LogAnime BaseDelay']);
Sepher.Param.LogNext			= Number(Sepher.Parameters['LogAnime NextDelay']);
Sepher.Param.logWaitDefault		= Number(Sepher.Parameters['LogWaitCount Default']);
Sepher.Param.logWaitBoost		= Number(Sepher.Parameters['LogWaitCount Boost']);
Sepher.Param.seBoostON			= String(Sepher.Parameters['SE BoostON']);
Sepher.Param.seBoostOFF			= String(Sepher.Parameters['SE BoostOFF']);
Sepher.Param.seVolume			= Number(Sepher.Parameters['SE Volume']);
Sepher.Param.SwitchImage		= String(Sepher.Parameters['SwitchImage']);
Sepher.Param.SwitchX			= Number(Sepher.Parameters['SwitchX']);
Sepher.Param.SwitchY			= Number(Sepher.Parameters['SwitchY']);
Sepher.Param.SwitchWidth		= Number(Sepher.Parameters['SwitchWidth']);
Sepher.Param.SwitchHeight		= Number(Sepher.Parameters['SwitchHeight']);
Sepher.Param.SwitchTop			= Number(Sepher.Parameters['SwitchTop']);
Sepher.Param.SwitchLeft			= Number(Sepher.Parameters['SwitchLeft']);
Sepher.Param.SwitchAnimePattern	= Number(Sepher.Parameters['SwitchAnimePattern']);
Sepher.Param.SwitchAnimeSpeed	= Number(Sepher.Parameters['SwitchAnimeSpeed']);
Sepher.Param.YEPMotionWait		= Number(Sepher.Parameters['YEP Battle MotionWait']);
Sepher.Param.YEPVictoryWait		= String(Sepher.Parameters['YEP Victory Wait']);
Sepher.Param.YEPBoostSwitch		= String(Sepher.Parameters['YEP ATB BoostSwitch']);
Sepher.Param.ATBSpeedDefault	= Number(Sepher.Parameters['ATB Speed(Default)']);
Sepher.Param.ATBSpeedBoost		= Number(Sepher.Parameters['ATB Speed(Boost)']);

// OkayKeyBoost型変換
if (Sepher.Param.OkayKeyBoost == 'true'){
	Sepher.Param.OkayKeyBoost = true;
}else{
	Sepher.Param.OkayKeyBoost = false;
};
// VisibleSwitch型変換
if (Sepher.Param.VisibleSwitch == 'true'){
	Sepher.Param.VisibleSwitch = true;
}else{
	Sepher.Param.VisibleSwitch = false;
};
// YEPVictoryWait型変換
if (Sepher.Param.YEPVictoryWait == 'true'){
	Sepher.Param.YEPVictoryWait = true;
}else{
	Sepher.Param.YEPVictoryWait = false;
};
// YEPBoostSwitch型変換
if (Sepher.Param.YEPBoostSwitch == 'true'){
	Sepher.Param.YEPBoostSwitch = true;
}else{
	Sepher.Param.YEPBoostSwitch = false;
};


//------------------------------------------------------------------------------
// Switch Setting
//------------------------------------------------------------------------------

if (Sepher.Param.VisibleSwitch){
	var QuickMode = false;
	var createUpper = Spriteset_Battle.prototype.createUpperLayer;
	Spriteset_Battle.prototype.createUpperLayer = function() {
		createUpper.apply(this, arguments);
		var button = new Sprite_Button();
		button.bitmap = ImageManager.loadSystem(Sepher.Param.SwitchImage);
		button.x = Sepher.Param.SwitchX;
		button.y = Sepher.Param.SwitchY;
		button.visible = true;
		button.opacity = 80;
		this.addChild(button);

		var animCount = 1;
		var duration = 1;
		button.updateFrame = function() {
			var quickModeKey = Input.isTriggered(Sepher.Param.BoostToggleSwitch);
			if (quickModeKey){
				button.quickModeToggleSwitch();
			}
			if (QuickMode) {
				button.opacity = 255;
				if (++duration == Sepher.Param.SwitchAnimeSpeed) {
					if (++animCount >= Sepher.Param.SwitchAnimePattern) {
						animCount = 1;
					}
					button.setFrame(animCount * Sepher.Param.SwitchWidth, Sepher.Param.SwitchTop * Sepher.Param.SwitchHeight, Sepher.Param.SwitchWidth, Sepher.Param.SwitchHeight);
					duration = 1;
				}
			}
		}
		button.setFrame(Sepher.Param.SwitchLeft * Sepher.Param.SwitchWidth, Sepher.Param.SwitchTop * Sepher.Param.SwitchHeight, Sepher.Param.SwitchWidth, Sepher.Param.SwitchHeight);

		button.setClickHandler(function () {
			QuickMode = !QuickMode;
			if (QuickMode) {
  	 			var sound = {
					name:   Sepher.Param.seBoostON,
					volume: Sepher.Param.seVolume,
					pitch:  100,
   					pan:    0
				};
   				AudioManager.playSe(sound);
				button.opacity = 255;
			}else{
   				var sound = {
					name:   Sepher.Param.seBoostOFF,
					volume: Sepher.Param.seVolume,
   					pitch:  100,
    				pan:    0
  				};
   				AudioManager.playSe(sound);
				button.setFrame(Sepher.Param.SwitchLeft * Sepher.Param.SwitchWidth, Sepher.Param.SwitchTop * Sepher.Param.SwitchHeight, Sepher.Param.SwitchWidth, Sepher.Param.SwitchHeight);
				button.opacity = 80;
			}
		});

		button.quickModeToggleSwitch = function() {
			QuickMode = !QuickMode;
			if (QuickMode) {
   				var sound = {
					name:   Sepher.Param.seBoostON,
					volume: Sepher.Param.seVolume,
					pitch:  100,
   					pan:    0
				};
   				AudioManager.playSe(sound);
				button.opacity = 255;
			}else{
   				var sound = {
					name:   Sepher.Param.seBoostOFF,
					volume: Sepher.Param.seVolume,
   					pitch:  100,
    				pan:    0
  				};
    			AudioManager.playSe(sound);
				button.setFrame(Sepher.Param.SwitchLeft * Sepher.Param.SwitchWidth, Sepher.Param.SwitchTop * Sepher.Param.SwitchHeight, Sepher.Param.SwitchWidth, Sepher.Param.SwitchHeight);
				button.opacity = 80;
			}
		};
	};
}

// Alias - launchBattle
var launchBattle = Scene_Map.prototype.launchBattle;
Scene_Map.prototype.launchBattle = function() {
	launchBattle.apply(this, arguments);
};

//------------------------------------------------------------------------------
// QuickMode Speed Setting
//------------------------------------------------------------------------------

Sprite_StateIcon.prototype.animationWait = function() {
	var speed = Sepher.Param.StateIcon / Sepher.Param.BattleSpeedDefault;
    return speed;
};

Sprite_StateOverlay.prototype.animationWait = function() {
	var speed = Sepher.Param.StateOverlay / Sepher.Param.BattleSpeedDefault;
    return speed;
};

Sprite_Weapon.prototype.animationWait = function() {
	var speed = Sepher.Param.Weapon / Sepher.Param.BattleSpeedDefault;
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
			speed = speed / Sepher.Param.BattleSpeedBoost;
		} else {
			speed = speed / Sepher.Param.BattleSpeedDefault;
		}
	} else {
		if (QuickMode){
			speed = speed / Sepher.Param.BattleSpeedBoost;
		} else {
			speed = speed / Sepher.Param.BattleSpeedDefault;
		}
	}
   	return speed;
};

Sprite_Balloon.prototype.waitTime = function() {
	var speed = Sepher.Param.Balloon / Sepher.Param.BattleSpeedDefault;
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
			speed = speed / Sepher.Param.BattleSpeedBoost;
		} else {
			speed = speed / Sepher.Param.BattleSpeedDefault;
		}
	} else {
		if (QuickMode){
			speed = speed / Sepher.Param.BattleSpeedBoost;
		} else {
			speed = speed / Sepher.Param.BattleSpeedDefault;
		}
	}
    return speed;
};

// Alias - startMove
var _SSEP_Battler_startMove = Sprite_Battler.prototype.startMove;
Sprite_Battler.prototype.startMove = function(x, y, duration) {
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
  			duration = duration / Sepher.Param.BattleSpeedBoost;
  		} else {
  			duration = duration / Sepher.Param.BattleSpeedDefault;
		}
	} else {
		if (QuickMode){
			duration = duration / Sepher.Param.BattleSpeedBoost;
  		} else {
  			duration = duration / Sepher.Param.BattleSpeedDefault;
		}
	}
	_SSEP_Battler_startMove.call(this, x, y, duration);
};


Sprite_Actor.prototype.motionSpeed = function() {
	var speed = Sepher.Param.Motion;
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
			speed = speed / Sepher.Param.BattleSpeedBoost;
		} else {
			speed = speed / Sepher.Param.BattleSpeedDefault;
		}
	} else {
		if (QuickMode){
			speed = speed / Sepher.Param.BattleSpeedBoost;
		} else {
			speed = speed / Sepher.Param.BattleSpeedDefault;
		}
	}
    return speed;
};

// Alias - Animation setup
var _SSEP_Animation_setup = Sprite_Animation.prototype.setup;
Sprite_Animation.prototype.setup = function(target, animation, mirror, delay) {
	delay = delay / Sepher.Param.BattleSpeedDefault;
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
			delay = delay / Sepher.Param.BattleSpeedBoost;
		} else {
			delay = delay / Sepher.Param.BattleSpeedDefault;
		}
	} else {
		if (QuickMode){
			delay = delay / Sepher.Param.BattleSpeedBoost;
		} else {
			delay = delay / Sepher.Param.BattleSpeedDefault;
		}
	}
	_SSEP_Animation_setup.call(this, target, animation, mirror, delay);
};

// Alias - Animation_setupRate
var _SSEP_Animation_setupRate = Sprite_Animation.prototype.setupRate;
Sprite_Animation.prototype.setupRate = function() {
    _SSEP_Animation_setupRate;
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
			this._rate = this._rate / Sepher.Param.BattleSpeedBoost;
		} else {
		    this._rate = this._rate / Sepher.Param.BattleSpeedDefault;
		}
	} else {
		if (QuickMode){
			this._rate = this._rate / Sepher.Param.BattleSpeedBoost;
		} else {
		    this._rate = this._rate / Sepher.Param.BattleSpeedDefault;
		}
	}
	this._rate = (Math.round(this._rate));
	if (this._rate < 1){
		this._rate = 1;
	}
};

var _SSEP_Damage_initialize = Sprite_Damage.prototype.initialize;
Sprite_Damage.prototype.initialize = function() {
	_SSEP_Damage_initialize.call(this);
    this._duration = Sepher.Param.Damage;
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
			this._duration = this._duration / Sepher.Param.BattleSpeedBoost;
		} else {
			this._duration = this._duration / Sepher.Param.BattleSpeedDefault;
		}
	} else {
		if (QuickMode){
			this._duration = this._duration / Sepher.Param.BattleSpeedBoost;
		} else {
			this._duration = this._duration / Sepher.Param.BattleSpeedDefault;
		}
	}
    if (this._duration <= Sepher.Param.DamageMin){
    	this._duration = Sepher.Param.DamageMin;
    }
};

// BattleLog Window

Window_BattleLog.prototype.animationBaseDelay = function() {
	var speed = Sepher.Param.LogBase;
    return speed;
};

Window_BattleLog.prototype.animationNextDelay = function() {
	var speed = Sepher.Param.LogNext;
    return speed;
};

Window_BattleLog.prototype.updateWaitCount = function() {
	var waitMax;
	var waitMin;
	if (Sepher.Param.OkayKeyBoost){
		if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
   	 	    waitMax = 3 * Sepher.Param.logWaitBoost;
    		waitMin = 1 * Sepher.Param.logWaitBoost;
   		}else{
   	 	    waitMax = 3 * Sepher.Param.logWaitDefault;
    		waitMin = 1 * Sepher.Param.logWaitDefault;
		}
	}else{
		if (QuickMode){
   	 	    waitMax = 3 * Sepher.Param.logWaitBoost;
    		waitMin = 1 * Sepher.Param.logWaitBoost;
   		}else{
   	 	    waitMax = 3 * Sepher.Param.logWaitDefault;
    		waitMin = 1 * Sepher.Param.logWaitDefault;
		}
	}

    if (this._waitCount > 0) {
        this._waitCount -= this.isFastForward() ? waitMax : waitMin;
        if (this._waitCount < 0) {
            this._waitCount = 0;
        }
    	return true;
    }
    return false;
};

//------------------------------------------------------------------------------
// Support - YanflyEngine BattleCore
//------------------------------------------------------------------------------

if (Imported.YEP_BattleEngineCore){

	//Overwrite actionPerformAction
	BattleManager.actionPerformAction = function() {
		wait = Sepher.Param.YEPMotionWait;
		if (Sepher.Param.OkayKeyBoost){
			if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
				wait = wait / Sepher.Param.BattleSpeedBoost;
			} else {
				wait = wait / Sepher.Param.BattleSpeedDefault;
			}
		} else {
			if (QuickMode){
				wait = wait / Sepher.Param.BattleSpeedBoost;
			} else {
				wait = wait / Sepher.Param.BattleSpeedDefault;
			}
		}
		
	    this._logWindow.performAction(this._subject, this._action);
			if (this._subject.isActor() && this._subject.isSpriteVisible) {
				this._logWindow._waitCount += wait;
				return false;
			}
 	   return true;
	};

	//Overwrite actionFloat
	BattleManager.actionFloat = function(name, actionArgs) {
		var movers = this.makeActionTargets(name);
		if (movers.length < 1) return true;
		var cmd = actionArgs[0];
		var frames = actionArgs[1] || 12;

		if (Sepher.Param.OkayKeyBoost){
			if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
				frames = frames / Sepher.Param.BattleSpeedBoost;
			} else {
				frames = frames / Sepher.Param.BattleSpeedDefault;
			}
		} else {
			if (QuickMode){
				frames = frames / Sepher.Param.BattleSpeedBoost;
			} else {
				frames = frames / Sepher.Param.BattleSpeedDefault;
			}
		}

	    var pixels = 0;
	    if (cmd.match(/(\d+)([%％])/i)) {
	    	var floatPeak = parseFloat(RegExp.$1 * 0.01);
		} else if (cmd.match(/(\d+)/i)) {
			pixels = parseInt(RegExp.$1);
			var floatPeak = 0.0;
    	} else {
    		var floatPeak = 1.0;
    	}
    	movers.forEach(function(mover) {
    		var floatRate = floatPeak + (pixels / mover.spriteHeight());
    		mover.spriteFloat(floatRate, frames);
    	});
    	return false;
	};

	//Overwrite actionJump
	BattleManager.actionJump = function(name, actionArgs) {
		var movers = this.makeActionTargets(name);
		if (movers.length < 1) return true;
		var cmd = actionArgs[0];
	    var frames = actionArgs[1] || 12;

		if (Sepher.Param.OkayKeyBoost){
			if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
				frames = frames / Sepher.Param.BattleSpeedBoost;
			} else {
				frames = frames / Sepher.Param.BattleSpeedDefault;
			}
		} else {
			if (QuickMode){
				frames = frames / Sepher.Param.BattleSpeedBoost;
			} else {
				frames = frames / Sepher.Param.BattleSpeedDefault;
			}
		}

	    var pixels = 0;
		if (cmd.match(/(\d+)([%％])/i)) {
			var jumpPeak = parseFloat(RegExp.$1 * 0.01);
    	} else if (cmd.match(/(\d+)/i)) {
    		pixels = parseInt(RegExp.$1);
    		var jumpPeak = 0.0;
    	} else {
    		var jumpPeak = 1.0;
    	}
    	movers.forEach(function(mover) {
    		var jumpRate = jumpPeak + (pixels / mover.spriteHeight());
    		mover.spriteJump(jumpRate, frames);
    	});
    	return true;
    };
}

//------------------------------------------------------------------------------
// Support - YanflyEngine Active Time Battle
//------------------------------------------------------------------------------

if (Imported.YEP_X_BattleSysATB){
	//Overwrite atbTickRate
	BattleManager.atbTickRate = function() {
	    var rate = 0.1 * ConfigManager.atbSpeed;
    	//Add
	    if (Sepher.Param.OkayKeyBoost){
			if (Sepher.Param.YEPBoostSwitch){
				if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
					rate = rate * Sepher.Param.ATBSpeedBoost;
				} else {
					rate = rate * Sepher.Param.ATBSpeedDefault;
				}
			}
		} else {
		    if (Sepher.Param.YEPBoostSwitch){
				if (QuickMode || TouchInput.isPressed()){
					rate = rate * Sepher.Param.ATBSpeedBoost;
				} else {
					rate = rate * Sepher.Param.ATBSpeedDefault;
				}
			}

		}
    	return rate;
	};
}

//------------------------------------------------------------------------------
// Support - YanflyEngine Victory Aftermath
//------------------------------------------------------------------------------

if (Imported.YEP_VictoryAftermath){
	//Overwrite isFinishedVictoryCheer
	BattleManager.isFinishedVictoryCheer = function() {
		if (Sepher.Param.YEPVictoryWait){
			if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
				return ++this._victoryCheerWait >= Yanfly.Param.VACheerWait / Sepher.Param.BattleSpeedBoost;
			} else {
				return ++this._victoryCheerWait >= Yanfly.Param.VACheerWait / Sepher.Param.BattleSpeedDefault;
			}
		} else {
			return ++this._victoryCheerWait >= Yanfly.Param.VACheerWait;
		}
	};
}

//------------------------------------------------------------------------------
// Support - Ellye's ATB
//------------------------------------------------------------------------------

//Check if there is Window_CTB function
if (typeof Window_CTB == "function"){
    //Changing the flow of battle
    //_BattleManager_update=BattleManager.update;
    BattleManager.update=function() {
        if(!this.isBusy()&&!this.updateEvent()) {
            switch(this._phase) {
                case 'atb':
                	//Add script
					if (Sepher.Param.OkayKeyBoost){
						if (QuickMode || Input.isPressed('ok') || TouchInput.isPressed()){
	                		var speed = Sepher.Param.ATBSpeedBoost;
						} else {
	                		var speed = Sepher.Param.ATBSpeedDefault;
						}
					} else {
						if (QuickMode){
	                		var speed = Sepher.Param.ATBSpeedBoost;
						} else {
	                		var speed = Sepher.Param.ATBSpeedDefault;
						}
					}
                    this.increaseAtbGauges(speed);
                    break;
                default:
                    _BattleManager_update.call(this);
                    break;
            }
        }
    };

    //Alias - increaseAtbGauses
    var _SSEP_increaseAtbGauges = BattleManager.increaseAtbGauges;
    BattleManager.increaseAtbGauges = function(speed) {
    	//Add variable　and Rate calculation
    	var copy_base_atb_increase = base_atb_increase;
        base_atb_increase = base_atb_increase * speed;
        //original method
        _SSEP_increaseAtbGauges.call(this);
        //return global variable
        base_atb_increase = copy_base_atb_increase;
    };
}