(function() {

    var parameters = PluginManager.parameters('StoragePlugin');
    var storageSize = Number(parameters['Storage Size']);

    //---------------------------------------------------------------------------
    // DataManager
    //---------------------------------------------------------------------------

    var _DataManager_createGameObjects = DataManager.createGameObjects;
    DataManager.createGameObjects = function() {
        _DataManager_createGameObjects.call(this);
        $gameStorage = new Game_Storage();
    };

    var _DataManager_makeSaveContents = DataManager.makeSaveContents;
    DataManager.makeSaveContents = function() {
        var contents = _DataManager_makeSaveContents.call(this);
        contents.storage = $gameStorage;
        return contents;
    };

    var _DataManager_extractSaveContents = DataManager.extractSaveContents;
    DataManager.extractSaveContents = function(contents) {
        _DataManager_extractSaveContents.call(this, contents);
        $gameStorage = contents.storage || new Game_Storage();     
    };

    //---------------------------------------------------------------------------
    // Game_Storage
    //---------------------------------------------------------------------------

    function Game_Storage() {
        this.initialize.apply(this, arguments);
    }

    Game_Storage.prototype.initialize = function() {
        this._items = {};
    };

    Game_Storage.prototype.getItemCount = function(itemId) {
        return this._items[itemId] || 0;
    };

    Game_Storage.prototype.addItem = function(itemId, count) {
        if (this._items[itemId]) {
            this._items[itemId] += count;
        } else {
            this._items[itemId] = count;
        }
    };

    Game_Storage.prototype.removeItem = function(itemId, count) {
        if (this._items[itemId]) {
            this._items[itemId] -= count;
            if (this._items[itemId] <= 0) {
                delete this._items[itemId];
            }
        }
    };

    Game_Storage.prototype.clear = function() {
        this._items = {};
    };

    Game_Storage.prototype.isFull = function() {
        var itemCount = Object.keys(this._items).length;
        return itemCount >= storageSize;
    };

    //---------------------------------------------------------------------------
    // Game_Party
    //---------------------------------------------------------------------------

    /* Override */
    Game_Party.prototype.gainItem = function(item, amount, includeEquip) {
        if (item && item.itypeId === 1) {
            $gameStorage.addItem(item.id, amount);
        } else {
            _Game_Party_gainItem.call(this, item, amount, includeEquip);
        }
    };

    //---------------------------------------------------------------------------
    // Game_Interpreter
    //---------------------------------------------------------------------------

    var _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === 'openStorage') {
            SceneManager.push(Scene_Storage);
        }
    };

    //---------------------------------------------------------------------------
    // Scene_Storage
    //---------------------------------------------------------------------------

    function Scene_Storage() {
        this.initialize.apply(this, arguments);
    }

    Scene_Storage.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_Storage.prototype.constructor = Scene_Storage;

    Scene_Storage.prototype.initialize = function() {
        Scene_MenuBase.prototype.initialize.call(this);
    };

    Scene_Storage.prototype.create = function() {
        Scene_MenuBase.prototype.create.call(this);
        this.createWindowLayer();
        this.createHelpWindow();
        this.createItemWindow();
    };

    Scene_Storage.prototype.createHelpWindow = function() {
        this._helpWindow = new Window_Help(1);
        this._helpWindow.setText("请选择一个物品");
        this.addWindow(this._helpWindow);
    };

    Scene_Storage.prototype.createItemWindow = function() {
        var x = 0;
        var y = this._helpWindow.height;
        var width = Graphics.boxWidth;
        var height = Graphics.boxHeight - y;
        this._itemWindow = new Window_StorageItem(x, y, width, height);
        this._itemWindow.setHelpWindow(this._helpWindow);
        this.addWindow(this._itemWindow);
    };

    //---------------------------------------------------------------------------
    // Window_StorageItem
    //---------------------------------------------------------------------------

    function Window_StorageItem() {
        this.initialize.apply(this, arguments);
    }

    Window_StorageItem.prototype = Object.create(Window_ItemList.prototype);
    Window_StorageItem.prototype.constructor = Window_StorageItem;

    Window_StorageItem.prototype.initialize = function(x, y, width, height) {
        Window_ItemList.prototype.initialize.call(this, x, y, width, height);
    };

    Window_StorageItem.prototype.isEnabled = function(item) {
        return $gameParty.canUse(item) || item.itypeId !== 1;
    };

    Window_StorageItem.prototype.makeItemList = function() {
        this._data = [];
        for (var itemId in $gameStorage._items) {
            var item = $dataItems[itemId];
            if (item && $gameStorage.getItemCount(itemId) > 0) {
                this._data.push(item);
            }
        }
    };

    Window_StorageItem.prototype.updateHelp = function() {
        Window_ItemList.prototype.updateHelp.call(this);
       ._item) {
            var itemId = this._item.id;
            var count = $gameStorage.getItemCount(itemId);
            var itemName = this;
            this._helpWindow.setText(itemName + ' x ' + count);
        }
    };

    Window_StorageItem.prototype.playOkSound = function() {
        SoundManager.playCursor();
    };

    Window_StorageItem.prototype.playBuzzerSound = function() {
        SoundManager.playBuzzer();
    };

    //---------------------------------------------------------------------------
    // Window_ItemList
    //---------------------------------------------------------------------------

    /* Override */
    Window_ItemList.prototype.isEnabled = function(item) {
        return $gameParty.canUse(item);
    };

})();