//=============================================================================
// XYZ_Formation.js
//=============================================================================
var Imported = Imported || {};
Imported.Formation = true;

var XYZ = XYZ || {};
XYZ.Formation = XYZ.Formation || {};
XYZ.Formation.version = 1.00;
/*:
 * @plugindesc 阵法.
 * @author DawnKing
 * 
 * @param ---主菜单配置---
 * @default 
 * 
 * @param 主菜单-启用
 * @parent ---主菜单配置---
 * @type boolean
 * @on 启用
 * @off 不启用
 * @desc 阵法是否显示在主菜单上
 * @default true
 * 
 * @param 主菜单-显示文本
 * @parent ---主菜单配置---
 * @type string
 * @desc 显示在主菜单上的按钮文本
 * @default 阵法
 * 
 * @param ---文本配置---
 * @default 
 * 
 * @param 文本-离阵
 * @parent ---文本配置---
 * @type string
 * @desc 阵法下人时显示的文本
 * @default 离阵
 * 
 * @param ---按钮配置---
 * @default 
 * 
 * @param 按钮-退出-启用
 * @parent ---按钮配置---
 * @type boolean
 * @on 启用
 * @off 不启用
 * @desc 按钮-退出是否显示
 * @default true
 * 
 * @param 按钮文本-切换阵法
 * @parent ---按钮配置---
 * @type string
 * @desc 按钮切换阵法显示的文本
 * @default 切换阵法
 * 
 * @param 按钮文本-设置阵位
 * @parent ---按钮配置---
 * @type string
 * @desc 按钮设置阵位显示的文本
 * @default 设置阵位
 * 
 * @param ---战斗配置---
 * @default 
 * 
 * @param 战斗-阵法生效提示
 * @parent ---战斗配置---
 * @type boolean
 * @on 启用
 * @off 不启用
 * @desc 阵法生效时，是否提示
 * @default true
 * 
 * @param 战斗文本-阵法生效
 * @parent ---战斗配置---
 * @type string
 * @desc 战斗中阵法生效时提示的文本。 %1-阵法名
 * @default 阵法生效：%1
 * 
 * @param ---数据配置---
 * @default 
 * 
 * @param 阵法数据
 * @parent ---数据配置---
 * @type struct<Formation>[]
 * @desc 阵法数据配置
 * @default []
 *
 * @help
 * 
 * 插件指令：
 * XYZFormation open                 #打开阵法界面
 * XYZFormation add    阵法ID        #解锁阵法ID的阵法
 * XYZFormation remove 阵法ID        #移除阵法ID的阵法
 * XYZFormation addType    阵法类型  #解锁阵法类型的全部阵法
 * XYZFormation removeType 阵法类型  #移除阵法类型的全部阵法
 * XYZFormation addAll              #解锁全部阵法
 * XYZFormation removeAll           #移除全部阵法
 * 第二个参数大小写皆可
 * 
 * 激活代码配置:
 * paramCheckBig     属性检查       参数1[属性ID] 参数2[检查值] 参数3[等于开启] 参数4[检查范围]    #检查一定范围内的属性总和大于属性值
 * paramCheckSmall   属性检查       参数1[属性ID] 参数2[检查值] 参数3[等于开启] 参数4[检查范围]   #检查一定范围内的属性总和小于属性值
 * 
 * 参数大全
 * 检查范围 阵位列表 如 1,2 检查1和2号位 ALL（全大写） 为所有阵位
 * 
 * 限制代码配置:
 * actorAllow  允许角色  参数1[角色列表]     #允许这些角色
 * actorRefuse 拒绝角色  参数1[角色列表]     #拒绝这些角色
 * classAllow  允许职业  参数1[职业列表]     #允许这些职业
 * classRefuse 拒绝职业  参数1[职业列表]     #拒绝这些职业
 * paramBig    大于属性  参数1[属性ID] 参数2[属性值] 参数3[等于开启]     #属性大于属性值的角色
 * paramSmall  小于属性  参数1[属性ID] 参数2[属性值] 参数3[等于开启]     #属性小于属性值的角色
 * 允许类限制 会把允许外的角色踢出名单
 * 拒绝类限制 会把拒绝的角色踢出名单
 * 
 * 参数大全
 * 角色列表：角色ID,角色ID  如 1,2即为选中1,2 ID的角色
 * 等于开启：1 或 true 开启 其他均为关闭 
 * 
 * 效果代号配置:
 * 1 addState 添加状态      参数1[状态ID]                 //如1 1 战斗开始时为自己添加状态1，该参数可写成数组形式如 2,3为添加两种状态
 * 2 paramUp  属性固定提升  参数1[属性ID] 参数2[属性值]    //如2 0 100 战斗开始时自身血量临时提升100,该值不受倍率因素影响
 * 3 paramPass 属性传递      参数1[属性ID] 参数2[属性传递率] 参数3[阵位] //如3 0 100 2 该阵位的角色会获得2号位100%的HP加成
 * 4 paramPass2 属性传递[原]  参数1[属性ID] 参数2[属性传递率] 参数3[阵位] //同上，但该获得的加成仅以角色原数值为基数，即没有装备的角色数值
 * 5 learnSkill 临时技能   参数1[技能ID]                  //如5,6 战斗开始时自身获得5,6两个技能，临时技能可与自身技能池重复，重复技能不会在战斗结束后移除
 * 6 addStateEnemy 敌人添加状态  参数1[状态ID]          //同1 对象为敌方全体
 * 7 paramLowEnemy 敌人属性固定降低  参数1[属性ID] 参数2[属性值]  //同2 对象为敌方全体
 * 8 paramPassEnemy 敌人属性削弱  参数1[属性ID] 参数2[属性削弱率] 参数3[阵位]  //同3 敌方全体下降该阵位上角色属性的一定比例
 * 9 paramPassEnemy2 敌人属性削弱  参数1[属性ID] 参数2[属性削弱率] 参数3[阵位]  //同4
 * 
 * 参数大全
 * 属性ID：0 MHP 1 MMP 2 ATK 3 DEF 4 MAT 5 MDF 6 AGI 7 LUK
 * 属性传递率：单位% 即100 100%传递削弱
 * 
 * 技能方法：
 * $gameFormation.switchFormation('阵法编号'); 可在战斗中进行切换阵法
 * 变阵的目标阵法需解锁后才会使变阵效果生效
 * 切换后阵眼站位保持原站位，故设置的变阵的激活条件和人数最好与原阵法保持一致，不然会有阵法失效情况
 * 在战斗结束后阵法会变更为原先的阵法
 * 变阵技能可通过临时技能附加给阵眼上的角色
 * 变阵可设置为隐藏模式
 * 
 * 该插件功能集中于此属性上
 * $gameFormation
 * $dataFormation
 * 
 */
/*~struct~Formation: 
 *
 * @param 阵法编号
 * @desc 阵法ID,不可重复,可使用字符。不要设0
 * @type number
 * @default 1
 * 
 * @param 阵法类型
 * @desc 阵法的类型,可自由定义
 * @default 初级型阵法
 * 
 * @param 阵法显示
 * @type boolean
 * @on 显示
 * @off 不显示
 * @desc 是否显示在阵法列表中
 * @default true
 *
 * @param 阵法名
 * @desc 阵法名称
 * @default 阵法的名字
 * 
 * @param 阵法人数
 * @desc 激活该阵法所需人数
 * @type number
 * @default 2
 * 
 * @param 阵法激活条件
 * @desc 激活该阵法所需的条件
 * @type struct<FormationActivate>[]
 * @default []
 * 
 * @param 阵法描述
 * @desc 该阵法的简单描述
 * @default 一个普通的阵法
 * 
 * @param 阵法位列表
 * @desc 该阵法各个位置上的效果,其数量最好等同于阵法人数即使该阵位没有效果
 * @type struct<FormationPostion>[]
 * @default []
 * 
 */
/*~struct~FormationPostion: 
 *
 * @param 阵法位
 * @desc 阵法位,1为1号位
 * @type number
 * @default 1
 * 
 * @param 阵法位名称
 * @desc 该阵法位的显示名称
 * @default 阵眼
 * 
 * @param 阵法位限制
 * @desc 该阵法位上的限制
 * @type struct<FormationLimit>[]
 * @default []
 * 
 * @param 阵法位描述
 * @desc 该阵法位的简单描述
 * @default 一个普通的阵法位
 *
 * @param 阵法效果
 * @desc 该阵法位上的阵法效果
 * @type struct<FormationEffect>[]
 * @default []
 * 
 */
/*~struct~FormationLimit: 
 *
 * @param 限制代码
 * @desc 具体设置参考help中内容 -Allow 允许 -Refuse 拒绝 -Big 大于 -Small 小于
 * @type combo
 * @option actorAllow 角色允许 角色ID
 * @option actorRefuse 角色拒绝 角色ID
 * @option classAllow 职业允许 职业ID
 * @option classRefuse 职业拒绝 职业ID
 * @option paramBig 属性大于 属性ID 属性值 等于开启(可选)
 * @option paramSmall 属性小于 属性ID 属性值 等于开启(可选)
 * @default actorAllow 角色允许 角色ID
 * 
 * @param 限制参数1
 * @desc 限制参数1
 * @default 
 * @param 限制参数2
 * @desc 限制参数2
 * @default 
 * @param 限制参数3
 * @desc 限制参数3
 * @default 
 * 
 */
/*~struct~FormationEffect: 
 *
 * @param 效果代号
 * @desc 具体设置参考help中内容
 * @type combo
 * @option addState 添加状态 状态ID
 * @option paramUp 属性固定提升 属性ID 属性值
 * @option paramPass 属性传递 属性ID 传递率 传递阵眼
 * @option paramPass2 属性传递原 属性ID 传递率 传递阵眼
 * @option learnSkill 临时技能 技能ID
 * @option addStateEnemy 敌人添加状态 状态ID
 * @option paramLowEnemy 敌人属性固定降低 属性ID 属性值
 * @option paramPassEnemy 敌人属性削弱 属性ID 属性削弱率 阵位
 * @option paramPassEnemy2 敌人属性削弱 属性ID 属性削弱率 阵位
 * @default addState
 * 
 * @param 效果参数1
 * @desc 效果参数1
 * @default 
 * @param 效果参数2
 * @desc 效果参数2
 * @default 
 * @param 效果参数3
 * @desc 效果参数3
 * @default 
 * @param 效果参数4
 * @desc 效果参数4
 * @default 
 * @param 效果参数5
 * @desc 效果参数5
 * @default 
 * 
 */
/*~struct~FormationActivate: 
 *
 * @param 激活代码
 * @desc 具体设置参考help中内容 -paramCheck 属性检查 -paramAllCheck 属性检查[全]
 * @type combo
 * @option paramCheckBig 属性检查大于 属性ID 检查值 检查范围 等于开启(可选)
 * @option paramCheckSmall 属性检查小于 属性ID 检查值 检查范围 等于开启(可选)
 * @default paramCheckBig
 * 
 * @param 激活参数1
 * @desc 激活参数1
 * @default 
 * @param 激活参数2
 * @desc 激活参数2
 * @default 
 * @param 激活参数3
 * @desc 激活参数3
 * @default 
 * @param 激活参数4
 * @desc 激活参数4
 * @default
 * 
 */

XYZ.Parameters = PluginManager.parameters('XYZ_Formation');

XYZ.Formation.showMain = String(XYZ.Parameters['主菜单-启用'] || 'true');
XYZ.Formation.showMainText = String(XYZ.Parameters['主菜单-显示文本'] || '阵法');
XYZ.Formation.levelFormationText = String(XYZ.Parameters['文本-离阵'] || '离阵');
XYZ.Formation.changeFormationText = String(XYZ.Parameters['按钮文本-切换阵法'] || '切换阵法');
XYZ.Formation.setPostionText = String(XYZ.Parameters['按钮文本-设置阵位'] || '设置阵位');
XYZ.Formation.exitBtnFlag = String(XYZ.Parameters['按钮-退出-启用'] || 'true');
XYZ.Formation.battleFormationTipFlag = String(XYZ.Parameters['战斗-阵法生效提示'] || 'true');
XYZ.Formation.battleFormationTipText = String(XYZ.Parameters['战斗文本-阵法生效'] || '阵法生效：%1');
// ============================================================================
// 数据加载
// ============================================================================
XYZ.loadJson = function(json, deep) {
    if (deep == 20) {
        throw new Error('Object too deep');
    }
    try {
        json = JSON.parse(json || "");
    } catch (error) {
        return json;
    }
    if (json.constructor === Object) {
        for (key in json)
            json[key] = XYZ.loadJson(json[key], deep + 1);
    } else if (json.constructor === Array) {
        for (var i = 0; i < json.length; i++)
            json[i] = XYZ.loadJson(json[i], deep + 1);
    }
    return json;
}
$dataFormation = {};
$dataFormation._data = XYZ.loadJson(XYZ.Parameters['阵法数据'], 0);

$dataFormation.getFormation = function(formationId) {
    for (item of this._data) {
        if (item['阵法编号'] == formationId) { return item; }
    }
    return false;
}

$dataFormation.getFormationIdList = function() {
    var list = [];
    this._data.forEach(function(item) { list.push(item['阵法编号']); })
    return list;
}

$dataFormation.getFormationNameList = function() {
    var list = [];
    this._data.forEach(function(item) { list.push(item['阵法名']); })
    return list;
}

$dataFormation.getFormationPostion = function(formationId) {
    var formation = this.getFormation(formationId);
    return formation['阵法位列表'] ? formation['阵法位列表'] : false;
}

// ============================================================================
// 类
// ============================================================================
//$gameFormation
function DK_Game_Formation() {
    this.initialize.apply(this, arguments);
}

DK_Game_Formation.prototype.initialize = function() {
    this._data = []; //已获得的阵法列表Id
    this._equipFormation = 0; //正在使用的阵法
    this._switchFormation = 0; //切换的阵法 切换阵法时会将原阵法存在此处
    this._actorPosition = [null]; //角色站位
    this._activate = false;
};

DK_Game_Formation.prototype.setActivate = function(flag) {
    this._activate = flag;
};
DK_Game_Formation.prototype.getActivate = function() {
    return this._activate;
};

DK_Game_Formation.prototype.setActorPostion = function(postionId, actorId) {
    //其他位置有该角色先清除
    if (this._actorPosition.indexOf(actorId) > 0) { this._actorPosition[this._actorPosition.indexOf(actorId)] = 0 }
    this._actorPosition[postionId] = actorId;
};

//获取装备中的阵法对象 checkFlag 检查装备阵法是否存在在习得阵法内
DK_Game_Formation.prototype.getEquipFormation = function(checkFlag) {
    checkFlag = checkFlag || true
    if (this._data.indexOf(String(this._equipFormation)) < 0 && checkFlag) {
        this.unEquipFormation();
        return false;
    }
    return $dataFormation.getFormation(this._equipFormation) || false;
};

//获取装备中的阵法对象的阵法名
DK_Game_Formation.prototype.getEquipFormationName = function() {
    return $dataFormation.getFormation(this._equipFormation)['阵法名'] || "未知阵法";
};

//获取阵法位上的角色ID
DK_Game_Formation.prototype.getActorPostion = function(postion) {
    return this._actorPosition[postion] || false;
};

//获取指定阵位上的角色
DK_Game_Formation.prototype.getActorByPostion = function(p) {
    if (this.getActorPostion(p)) {
        var actor = $gameActors.actor(this.getActorPostion(p));
    }
    return actor ? actor : false;
};

//获取阵位上的全部角色
DK_Game_Formation.prototype.getActorListByPostion = function(pList) {
    var list = [];
    for (p of pList) {
        list.push(this.getActorByPostion(p));
    }
    return list.length > 0 ? list : false;
};

//获取阵位上的全部角色
DK_Game_Formation.prototype.getActorAll = function(p) {
    var formation = this.getEquipFormation();
    var list = [];
    var len = formation['阵法人数'];
    for (var p = 1; p <= len; p++) {
        if (this.getActorPostion(p)) {
            var actor = $gameActors.actor(this.getActorPostion(p));
            list.push(actor);
        }
    }
    return list.length > 0 ? list : false;
};

//获取已解锁阵法列表
DK_Game_Formation.prototype.getUnlockList = function() {
    var list = [];
    for (id of this._data) {
        if ($dataFormation.getFormation(id)['阵法显示'] == true) {
            list.push($dataFormation.getFormation(id));
        }
    }
    return list;
};

//切换阵法
DK_Game_Formation.prototype.switchFormation = function(formationId) {
    if (!$dataFormation.getFormation(formationId) || !this.isExistsFormation(formationId)) {
        return;
    }
    if (formationId == 0) { return; }
    if ($gameParty.inBattle()) { BattleManager.disableFormation(); }
    if (this._switchFormation == 0) { this._switchFormation = this._equipFormation; }
    this._equipFormation = String(formationId);
    if ($gameParty.inBattle()) {
        BattleManager.useFormation(false);
        this.setSwitch(true);
    }
};

DK_Game_Formation.prototype.equipFormation = function(formationId) {
    if (formationId == 0) { return; }
    this._equipFormation = formationId;
    this._actorPosition = [null];
};

DK_Game_Formation.prototype.unEquipFormation = function() {
    this._equipFormation = 0;
    this._actorPosition = [null];
};

DK_Game_Formation.prototype.isExistsFormation = function(formationId) {
    if (this._data.indexOf(String(formationId)) < 0) {
        return false;
    }
    return true;
};

DK_Game_Formation.prototype.addFormation = function(formationId) {
    if ($dataFormation.getFormation(formationId) && !this.isExistsFormation(formationId)) {
        this._data.push(formationId);
    }
};

DK_Game_Formation.prototype.addFormationType = function(typeId) {
    var list = $dataFormation._data;
    list = list.filter(function(item) {
        return item['阵法类型'] == typeId;
    })
    list.forEach(function(item) {
        $gameFormation.addFormation(String(item['阵法编号']));
    });
};

DK_Game_Formation.prototype.addAllFormation = function() {
    var list = $dataFormation.getFormationIdList();
    list.forEach(function(item) {
        $gameFormation.addFormation(String(item));
    });
};

DK_Game_Formation.prototype.removeFormation = function(formationId) {
    this._data = this._data.filter(function(id) {
        return id != formationId;
    });
};

DK_Game_Formation.prototype.removeFormationType = function(typeId) {
    this._data = this._data.filter(function(id) {
        return $dataFormation.getFormation(id)['阵法类型'] != typeId;
    });
};

DK_Game_Formation.prototype.clearFormation = function() {
    this._data = [];
};

DK_Game_Formation.prototype.setSwitch = function(flag) {
    this._isSwitch = flag;
};

DK_Game_Formation.prototype.isSwitch = function() {
    return this._isSwitch;
};

DK_Game_Formation.prototype.isActivate = function() {
    var formation = this.getEquipFormation(false);
    if (formation) {
        var list = this._actorPosition.slice(1, Number(formation['阵法人数']) + 1);
        if (formation['阵法人数'] == list.length) {
            //人数验证 且无空位
            for (l of list) {
                if (!(l > 0)) { return false; }
            }
            var activateList = formation['阵法激活条件'] || [];
            for (activate of activateList) {
                activateId = activate['激活代码'].split(' ')[0];
                p1 = activate['激活参数1'];
                p2 = activate['激活参数2'];
                p3 = activate['激活参数3'];
                p4 = activate['激活参数4'];
                switch (activateId) {
                    case "paramCheckBig":
                        if (p3 == "ALL") {
                            p3 = $gameFormation.getActorAll();
                        } else {
                            p3 = String(p3).split(',');
                            p3 = $gameFormation.getActorListByPostion(p3)
                        }
                        p4 = p4 == 1 || p4 == 'true' ? true : false;
                        var v = 0;
                        for (actor of p3) {
                            v += actor.param(p1);
                        }
                        //逻辑相反
                        if (p4) {
                            if (v < p2) { return false; }
                        } else {
                            if (v <= p2) { return false; }
                        }
                        break;
                    case "paramCheckSmall":
                        if (p3 == "ALL") {
                            p3 = $gameFormation.getActorAll();
                        } else {
                            p3 = String(p3).split(',');
                            p3 = $gameFormation.getActorListByPostion(p3)
                        }
                        p4 = p4 == 1 || p4 == 'true' ? true : false;
                        var v = 0;
                        for (actor of p3) {
                            v += actor.param(p1);
                        }
                        //逻辑相反
                        if (p4) {
                            if (v > p2) { return false; }
                        } else {
                            if (v >= p2) { return false; }
                        }
                        break;
                }
            }
            return true;
        }
    }
    return false;
};

//激活效果
DK_Game_Formation.prototype.activateEffect = function() {
    var formation = this.getEquipFormation();
    var postion = formation['阵法位列表'] || [];
    for (p of postion) {
        var actor = this.getActorByPostion(p['阵法位']);
        if (actor) { this.activateEffect2(p, actor); }
    }
};

DK_Game_Formation.prototype.activateEffect2 = function(p, actor) {
    var effectList = p['阵法效果'] || [];
    for (effect of effectList) {
        this.activateEffect3(effect, actor);
    }
};

DK_Game_Formation.prototype.activateEffect3 = function(effect, actor) {
    effectId = effect['效果代号'].split(' ')[0];
    p1 = effect['效果参数1'];
    p2 = effect['效果参数2'];
    p3 = effect['效果参数3'];
    p4 = effect['效果参数4'];
    switch (effectId) {
        case 1:
        case "addState":
            //1 添加状态      参数1[状态ID]
            p1 = String(p1).split(',');
            for (state of p1) {
                actor.addState(state);
            }
            console.log('阵法生效，添加状态', p1);
            break;
        case 2:
        case "paramUp":
            //2 属性固定提升  参数1[属性ID] 参数2[属性值]  
            p2 = Number(p2);
            actor.addFormationParam(p1, p2);
            console.log('阵法生效，属性固定提升', p1, p2);
            break;
        case 3:
        case "paramPass":
            //3 属性传递      参数1[属性ID] 参数2[属性传递率] 参数3[阵位]
            var actor2 = this.getActorByPostion(p3);
            var rate = Number(p2);
            var value = actor2 ? Number(actor2.param(p1)) * rate / 100 : 0;
            actor.addFormationParam(p1, value);
            console.log('阵法生效，属性传递', p1, value);
            break;
        case 4:
        case "paramPass2":
            //4 属性传递[原]  参数1[属性ID] 参数2[属性传递率] 参数3[阵位]
            var actor2 = this.getActorByPostion(p3);
            var rate = Number(p2);
            var value = actor2 ? Number(actor2.paramBase(p1)) * rate / 100 : 0;
            actor.addFormationParam(p1, value);
            console.log('阵法生效，属性传递[原]', p1, value);
            break;
        case 5:
        case "learnSkill":
            //5 临时技能  参数1[技能ID]
            p1 = String(p1).split(',');
            actor.tempSkill = []; //临时技能
            for (skill of p1) {
                if (!actor.isLearnedSkill(skill)) {
                    actor.learnSkill(skill);
                    actor.tempSkill.push(skill);
                }
            }
            console.log('阵法生效，临时技能', p1);
            break;
        case 6:
        case "addStateEnemy":
            //6 添加状态敌方全体      参数1[状态ID]
            p1 = String(p1).split(',');
            for (state of p1) {
                $gameTroop._enemies.forEach(function(enemy) {
                    enemy.addState(state);
                })
            }
            console.log('阵法生效，敌方添加状态', p1);
            break;
        case 7:
        case "paramLowEnemy":
            //7 属性固定降低  参数1[属性ID] 参数2[属性值]  
            p2 = Number(p2);
            $gameTroop._enemies.forEach(function(enemy) {
                enemy.addFormationParam(p1, -p2);
            })
            console.log('阵法生效，属性固定降低', p1, p2);
            break;
        case 8:
        case "paramPassEnemy":
            //8 属性削弱      参数1[属性ID] 参数2[属性削弱率] 参数3[阵位]
            var actor2 = this.getActorByPostion(p3);
            var rate = Number(p2);
            var value = actor2 ? Number(actor2.param(p1)) * rate / 100 : 0;
            $gameTroop._enemies.forEach(function(enemy) {
                enemy.addFormationParam(p1, -value);
            })
            console.log('阵法生效，属性削弱', p1, value);
            break;
        case 9:
        case "paramPassEnemy2":
            //9 属性削弱原     参数1[属性ID] 参数2[属性削弱率] 参数3[阵位]
            var actor2 = this.getActorByPostion(p3);
            var rate = Number(p2);
            var value = actor2 ? Number(actor2.param(p1)) * rate / 100 : 0;
            $gameTroop._enemies.forEach(function(enemy) {
                enemy.addFormationParam(p1, -value);
            })
            console.log('阵法生效，属性削弱[原]', p1, value);
            break;
    }
};

//移除效果
DK_Game_Formation.prototype.disableEffect = function() {
    var formation = this.getEquipFormation();
    var postion = formation['阵法位列表'] || [];
    for (p of postion) {
        var actor = this.getActorByPostion(p['阵法位']);
        if (actor) { this.disableEffect2(p, actor); }
    }
};

DK_Game_Formation.prototype.disableEffect2 = function(p, actor) {
    var effectList = p['阵法效果'] || [];
    for (effect of effectList) {
        this.disableEffect3(effect, actor);
    }
};

DK_Game_Formation.prototype.disableEffect3 = function(effect, actor) {
    effectId = effect['效果代号'].split(' ')[0];
    p1 = effect['效果参数1'];
    p2 = effect['效果参数2'];
    p3 = effect['效果参数3'];
    p4 = effect['效果参数4'];
    switch (effectId) {
        case 1:
        case "addState":
            p1 = String(p1).split(',');
            for (state of p1) {
                actor.removeState(state);
            }
            break;
        case 2:
        case "paramUp":
        case 3:
        case "paramPass":
        case 4:
        case "paramPass2":
            actor.clearFormationParam();
            break;
        case 5:
        case "learnSkill":
            for (skill of actor.tempSkill) {
                actor.forgetSkill(skill);
            }
            break;
        case 6:
        case "addStateEnemy":
            p1 = String(p1).split(',');
            for (state of p1) {
                $gameTroop._enemies.forEach(function(enemy) {
                    enemy.removeState(state);
                })
            }
        case 7:
        case "paramLowEnemy":
            $gameTroop._enemies.forEach(function(enemy) {
                enemy.clearFormationParam();
            })
    }
};

// ============================================================================
// 重写 新增部分
// ============================================================================
//主菜单相关
if (XYZ.Formation.showMain == 'true') {
    var _formation_createCommandWindow = Scene_Menu.prototype.createCommandWindow;
    Scene_Menu.prototype.createCommandWindow = function() {
        _formation_createCommandWindow.call(this);
        this._commandWindow.setHandler('XYZFormation', this.commandFormation.bind(this));
    };

    Scene_Menu.prototype.commandFormation = function() {
        SceneManager.push(Scene_XYZ_Formation);
    };

    var _formation_addOriginalCommands = Window_MenuCommand.prototype.addOriginalCommands;
    Window_MenuCommand.prototype.addOriginalCommands = function() {
        _formation_addOriginalCommands.call(this);
        this.addCommand(XYZ.Formation.showMainText, 'XYZFormation', this.areMainCommandsEnabled());
    };
}

//存档相关
var _formation_DataManager_createGameObjects = DataManager.createGameObjects;
DataManager.createGameObjects = function() {
    _formation_DataManager_createGameObjects.call(this);
    $gameFormation = new DK_Game_Formation();
};

var _formation_DataManager_makeSaveContents = DataManager.makeSaveContents;
DataManager.makeSaveContents = function() {
    var contents = _formation_DataManager_makeSaveContents.call(this);
    contents.formation = $gameFormation;
    return contents;
};

var _formation_DataManager_extractSaveContents = DataManager.extractSaveContents;
DataManager.extractSaveContents = function(contents) {
    _formation_DataManager_extractSaveContents.call(this, contents);
    $gameFormation = contents.formation;
};

//战斗处理
var _formation_BattleManager_startBattle = BattleManager.startBattle;
BattleManager.startBattle = function() {
    _formation_BattleManager_startBattle.call(this);
    BattleManager.useFormation();
};

var _formation_BattleManager_endBattle = BattleManager.endBattle;
BattleManager.endBattle = function(result) {
    _formation_BattleManager_endBattle.call(this, result);
    BattleManager.disableFormation();
    BattleManager.reductionFormation();
};

BattleManager.useFormation = function(switchFlag) {
    switchFlag = switchFlag || false;
    if ($gameFormation.isActivate()) {
        if (XYZ.Formation.battleFormationTipFlag && !switchFlag) {
            var text = XYZ.Formation.battleFormationTipText.replace('%1', $gameFormation.getEquipFormationName())
            $gameMessage.add(text);
        }
        $gameFormation.setActivate(true);
        var formation = $gameFormation.getEquipFormation();
        if (formation) {
            $gameFormation.activateEffect();
        }
    }
};

BattleManager.disableFormation = function() {
    if ($gameFormation.isActivate()) {
        $gameFormation.setActivate(false);
        var formation = $gameFormation.getEquipFormation();
        if (formation) {
            $gameFormation.disableEffect();
        }
    }
};

BattleManager.reductionFormation = function() {
    if ($gameFormation.isSwitch()) {
        $gameFormation._equipFormation = $gameFormation._switchFormation;
        $gameFormation._switchFormation = 0;
        $gameFormation.setSwitch(false);
    }
};

Object.defineProperties(Game_Actor.prototype, {
    formation_hp: {
        get: function() { return this.formationParam(0); },
        set: function(v) { this.setFormationParam(0, v) },
        configurable: true
    },
    formation_mp: {
        get: function() { return this.formationParam(1); },
        set: function(v) { this.setFormationParam(1, v) },
        configurable: true
    },
    formation_atk: {
        get: function() { return this.formationParam(2); },
        set: function(v) { this.setFormationParam(2, v) },
        configurable: true
    },
    formation_def: {
        get: function() { return this.formationParam(3); },
        set: function(v) { this.setFormationParam(3, v) },
        configurable: true
    },
    formation_mat: {
        get: function() { return this.formationParam(4); },
        set: function(v) { this.setFormationParam(4, v) },
        configurable: true
    },
    formation_mdf: {
        get: function() { return this.formationParam(5); },
        set: function(v) { this.setFormationParam(5, v) },
        configurable: true
    },
    formation_agi: {
        get: function() { return this.formationParam(6); },
        set: function(v) { this.setFormationParam(6, v) },
        configurable: true
    },
    formation_luk: {
        get: function() { return this.formationParam(7); },
        set: function(v) { this.setFormationParam(7, v) },
        configurable: true
    }
});

var _formation_Game_BattlerBase_initMembers = Game_BattlerBase.prototype.initMembers;
Game_BattlerBase.prototype.initMembers = function() {
    _formation_Game_BattlerBase_initMembers.call(this);
    this.clearFormationParam();
};

Game_BattlerBase.prototype.clearFormationParam = function() {
    this._formationParam = [0, 0, 0, 0, 0, 0, 0, 0];
};

Game_BattlerBase.prototype.formationParam = function(paramId) {
    return this._formationParam[paramId] || 0;
};

Game_BattlerBase.prototype.addFormationParam = function(paramId, value) {
    this._formationParam[paramId] += value;
};

Game_BattlerBase.prototype.setFormationParam = function(paramId, value) {
    this._formationParam[paramId] = value;
};

var _formation_Game_BattlerBase_param = Game_BattlerBase.prototype.param;
Game_BattlerBase.prototype.param = function(paramId) {
    var value = _formation_Game_BattlerBase_param.call(this, paramId);
    if ($gameFormation.getActivate()) {
        value += this.formationParam(paramId);
        var maxValue = this.paramMax(paramId);
        var minValue = this.paramMin(paramId);
        return Math.round(value.clamp(minValue, maxValue));
    } else {
        return value;
    }
};

var _formation_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function(command, args) {
    _formation_Game_Interpreter_pluginCommand.call(this, command, args);
    if (command == 'XYZFormation') {
        switch (args[0].toUpperCase()) {
            case 'OPEN':
                SceneManager.push(Scene_XYZ_Formation);
                break;
            case 'ADD':
                $gameFormation.addFormation(String(args[1]));
                break;
            case 'REMOVE':
                $gameFormation.removeFormation(String(args[1]));
                break;
            case 'ADDALL':
                $gameFormation.addAllFormation();
                break;
            case 'REMOVEALL':
                $gameFormation.clearFormation();
                break;
            case 'ADDTYPE':
                $gameFormation.addFormationType(args[1]);
                break;
            case 'REMOVETYPE':
                $gameFormation.removeFormationType(args[1]);
                break;
        }
    }
};

Scene_MenuBase.prototype.setMap = function(key, value) {
    this.map.set(key, value);
    value.setSceneMap(this.map);
    this.addWindow(value);
};

Window_Base.prototype.setSceneMap = function(map) {
    this._sceneMap = map;
};
// ============================================================================
// scene
// ============================================================================
function Scene_XYZ_Formation() {
    this.initialize.apply(this, arguments);
}

Scene_XYZ_Formation.prototype = Object.create(Scene_MenuBase.prototype);
Scene_XYZ_Formation.prototype.constructor = Scene_XYZ_Formation;

Scene_XYZ_Formation.prototype.create = function() {
    var ww, wh, wx, wy;
    Scene_MenuBase.prototype.create.call(this);
    this._commandWindow = new Window_XYZ_Formation_Command(0, 0);
    this._commandWindow.setHandler('cancel', this.popScene.bind(this));

    wy = this._commandWindow.height;
    ww = this._commandWindow.width;
    wh = Graphics.boxHeight - wy;
    this._selectFormationWindow = new Window_XYZ_Formation_Selectable(0, wy, ww, wh);
    this._selectFormationWindow.setHandler('cancel', this.formationBack.bind(this));

    wx = this._commandWindow.width;
    ww = Graphics.boxWidth - wx;
    wh = Graphics.boxHeight * 3 / 4;
    this._selectFormationPostionWindow = new Window_XYZ_Formation_Postion_Selectable(wx, 0, ww, wh);
    this._selectFormationPostionWindow.setHandler('cancel', this.formationPostionBack.bind(this));
    wx = Graphics.boxWidth * 3 / 4;
    ww = Graphics.boxWidth / 4;
    this._selectActornWindow = new Window_XYZ_Actor_Selectable(wx, 0, ww, wh);
    this._selectActornWindow.setHandler('cancel', this.selectActorClose.bind(this));
    wx = this._commandWindow.width;
    wy = wh;
    ww = Graphics.boxWidth - wx;
    wh = Graphics.boxHeight - wh;
    this._messageWindow = new Window_XYZ_Formation_Message(wx, wy, ww, wh);

    this.map = new Map();
    this.setMap('command', this._commandWindow);
    this.setMap('selectFormation', this._selectFormationWindow);
    this.setMap('selectFormationPostion', this._selectFormationPostionWindow);
    this.setMap('selectActor', this._selectActornWindow);
    this.setMap('message', this._messageWindow);
};

Scene_XYZ_Formation.prototype.selectActorClose = function() {
    this._selectActornWindow.deselect();
    this._selectActornWindow.deactivate();
    this._selectActornWindow.close();
    this._selectFormationPostionWindow.activate();
};

Scene_XYZ_Formation.prototype.formationBack = function() {
    this._selectFormationWindow.deselect();
    this._selectFormationWindow.deactivate();
    this._commandWindow.activate();
};

Scene_XYZ_Formation.prototype.formationPostionBack = function() {
    this._selectFormationPostionWindow.deselect();
    this._selectFormationPostionWindow.deactivate();
    this._commandWindow.activate();
};

// ============================================================================
// window 
// ============================================================================
//功能列表
function Window_XYZ_Formation_Command() {
    this.initialize.apply(this, arguments);
}

Window_XYZ_Formation_Command.prototype = Object.create(Window_Command.prototype);
Window_XYZ_Formation_Command.prototype.constructor = Window_XYZ_Formation_Command;

Window_XYZ_Formation_Command.prototype.windowWidth = function() {
    return Graphics.boxWidth / 4;
};

Window_XYZ_Formation_Command.prototype.statusWidth = function() {
    return 120;
};

Window_XYZ_Formation_Command.prototype.makeCommandList = function() {
    this.addOptions();
};

Window_XYZ_Formation_Command.prototype.addOptions = function() {
    this.addCommand(XYZ.Formation.changeFormationText, 'changeFormation');
    this.addCommand(XYZ.Formation.setPostionText, 'postionSet');
    if (XYZ.Formation.exitBtnFlag == 'true') {
        this.addCommand('退出', 'exit');
    }
};

Window_XYZ_Formation_Command.prototype.isChangeFormationSymbol = function(symbol) {
    return symbol.contains('changeFormation');
};

Window_XYZ_Formation_Command.prototype.isPostionSetSymbol = function(symbol) {
    return symbol.contains('postionSet');
};

Window_XYZ_Formation_Command.prototype.isExitSymbol = function(symbol) {
    return symbol.contains('exit');
};

Window_XYZ_Formation_Command.prototype.processOk = function() {
    var index = this.index();
    var symbol = this.commandSymbol(index);
    if (this.isChangeFormationSymbol(symbol)) {
        this.intoWindow('selectFormation');
    } else if (this.isPostionSetSymbol(symbol)) {
        this.intoWindow('selectFormationPostion');
    } else if (this.isExitSymbol(symbol)) {
        this.callHandler('cancel');
    } else {
        this.callHandler('cancel');
    }
};

Window_XYZ_Formation_Command.prototype.intoWindow = function(window) {
    this.deactivate();
    this._sceneMap.get(window).select(0);
    this._sceneMap.get(window).activate();
};

//阵法选择
function Window_XYZ_Formation_Selectable() {
    this.initialize.apply(this, arguments);
}

Window_XYZ_Formation_Selectable.prototype = Object.create(Window_Selectable.prototype);
Window_XYZ_Formation_Selectable.prototype.constructor = Window_XYZ_Formation_Selectable;

Window_XYZ_Formation_Selectable.prototype.initialize = function(x, y, width, height) {
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this._list = this.makeList();
    this.deselect();
    this.refresh();
};

Window_XYZ_Formation_Selectable.prototype.isOkEnabled = function() {
    return true;
};

Window_XYZ_Formation_Selectable.prototype.makeList = function() {
    var list = $gameFormation.getUnlockList();
    return list;
};

Window_XYZ_Formation_Selectable.prototype.maxItems = function() {
    return this._list ? this._list.length : 0;
};

Window_XYZ_Formation_Selectable.prototype.update = function() {
    Window_Selectable.prototype.update.call(this);
    this.updateMessage();
};

Window_XYZ_Formation_Selectable.prototype.updateMessage = function() {
    if (this._sceneMap.get('message')) {
        var formation = this._list[this.index()] || false;
        if (formation) this._sceneMap.get('message').setMessage(formation['阵法描述']);
    }
};

Window_XYZ_Formation_Selectable.prototype.drawItem = function(index) {
    var item = this._list[index];
    var rect = this.itemRect(index);
    var width = rect.width - this.textPadding();
    this.drawFormationName(item, rect.x + 6, rect.y, width);
};

Window_XYZ_Formation_Selectable.prototype.drawFormationName = function(formation, x, y, width) {
    width = width || 128;
    this.resetTextColor();
    if ($gameFormation._equipFormation == formation['阵法编号']) {
        this.changeTextColor(this.hpGaugeColor1());
    }
    this.drawText(formation['阵法名'], x, y, width);
};

Window_XYZ_Formation_Selectable.prototype.processOk = function() {
    var index = this.index();
    var formation = this._list[index];
    if ($gameFormation._equipFormation == formation['阵法编号']) { return; }
    $gameFormation.equipFormation(formation['阵法编号']);
    this.refresh();
    if (this._sceneMap.get('selectFormationPostion')) {
        this._sceneMap.get('selectFormationPostion').makeList();
    }
};

//阵位设置
function Window_XYZ_Formation_Postion_Selectable() {
    this.initialize.apply(this, arguments);
}

Window_XYZ_Formation_Postion_Selectable.prototype = Object.create(Window_Selectable.prototype);
Window_XYZ_Formation_Postion_Selectable.prototype.constructor = Window_XYZ_Formation_Postion_Selectable;

Window_XYZ_Formation_Postion_Selectable.prototype.initialize = function(x, y, width, height) {
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this.deselect();
    this.makeList();
};

Window_XYZ_Formation_Postion_Selectable.prototype.isOkEnabled = function() {
    return true;
};

Window_XYZ_Formation_Postion_Selectable.prototype.makeList = function() {
    var equipFormation = $gameFormation.getEquipFormation();
    var postionList = equipFormation['阵法位列表'] || [];
    var postionModel = {
        '阵法位': 0,
        '阵法位名称': '阵眼',
        '阵法位描述': '一个普通的阵眼'
    }
    var len = Number(equipFormation['阵法人数']);
    var list = [];
    //将已有阵法位信息填充
    for (var i = 0; i < len; i++) {
        if (postionList[i]) {
            var postion = Number(postionList[i]['阵法位']);
            var actorId = $gameFormation.getActorPostion(postion) ? $gameFormation.getActorPostion(postion) : 0;
            list[postion - 1] = { postion: postionList[i], actorId: actorId };
        }
    }
    for (var i = 0; i < len; i++) {
        if (!list[i]) {
            var actorId = $gameFormation.getActorPostion(i + 1) ? $gameFormation.getActorPostion(i + 1) : 0;
            list[i] = { postion: postionModel, actorId: actorId };
        }
    }
    this._list = list;
    this.refresh();
};

Window_XYZ_Formation_Postion_Selectable.prototype.itemHeight = function() {
    var clientHeight = this.height - this.padding * 2;
    return Math.floor(clientHeight / this.numVisibleRows());
};

Window_XYZ_Formation_Postion_Selectable.prototype.numVisibleRows = function() {
    return 3;
};

Window_XYZ_Formation_Postion_Selectable.prototype.maxItems = function() {
    return this._list ? this._list.length : 0;
};

Window_XYZ_Formation_Postion_Selectable.prototype.update = function() {
    Window_Selectable.prototype.update.call(this);
    this.updateMessage();
};

Window_XYZ_Formation_Postion_Selectable.prototype.updateMessage = function() {
    if (this._sceneMap.get('message')) {
        var p = this._list[this.index()] || "";
        if (p) this._sceneMap.get('message').setMessage(p.postion['阵法位描述']);
    }
};

Window_XYZ_Formation_Postion_Selectable.prototype.drawItem = function(index) {
    var item = this._list[index];
    var rect = this.itemRect(index);
    var width = rect.width - this.textPadding();
    this.drawFormationPostionName(item.postion, rect.x + 6, rect.y, width);
    if (item.actorId !== 0) {
        var actor = $gameActors.actor(item.actorId);
        this.drawActorFace(actor, rect.x + 128, rect.y, 144, 144);
    }
};

Window_XYZ_Formation_Postion_Selectable.prototype.drawFormationPostionName = function(postion, x, y, width) {
    width = width || 128;
    this.resetTextColor();
    if ($gameFormation.isActivate()) {
        this.changeTextColor(this.hpGaugeColor1());
    }
    this.drawText(postion['阵法位名称'], x, y, width);
};

Window_XYZ_Formation_Postion_Selectable.prototype.processOk = function() {
    var index = this.index();
    var item = this._list[index];
    if (this._sceneMap.get('selectActor')) {
        this._sceneMap.get('selectActor').makeList(item);
        this._sceneMap.get('selectActor').showOpen();
        this.deactivate();
    }
};

Window_XYZ_Formation_Postion_Selectable.prototype.setActorPostion = function(actorId) {
    var index = Number(this.index()) + 1;
    $gameFormation.setActorPostion(index, actorId);
    this.makeList();
};

//选择角色
function Window_XYZ_Actor_Selectable() {
    this.initialize.apply(this, arguments);
}

Window_XYZ_Actor_Selectable.prototype = Object.create(Window_Selectable.prototype);
Window_XYZ_Actor_Selectable.prototype.constructor = Window_XYZ_Actor_Selectable;

Window_XYZ_Actor_Selectable.prototype.initialize = function(x, y, width, height) {
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this.hide();
    this.close();
};

Window_XYZ_Actor_Selectable.prototype.isOkEnabled = function() {
    return true;
};

Window_XYZ_Actor_Selectable.prototype.makeList = function(item) {
    var actorId = item.actorId || 0;
    var postion = item.postion || 0;
    var list = $gameParty.allMembers();
    list = list.filter(function(item) {
        return item._actorId != actorId;
    })
    if (postion !== 0) {
        var limit = postion['阵法位限制'];
        if (limit) {
            for (l of limit) {
                limitId = l['限制代码'].split(' ')[0];
                var p1 = l['限制参数1'];
                var p2 = l['限制参数2'];
                var p3 = l['限制参数3'];
                switch (limitId) {
                    case "actorAllow":
                        p1 = String(p1).split(',');
                        list = list.filter(function(item) {
                            return p1.indexOf(String(item._actorId)) >= 0;
                        })
                        break;
                    case "actorRefuse":
                        p1 = String(p1).split(',');
                        list = list.filter(function(item) {
                            return p1.indexOf(String(item._actorId)) < 0;
                        })
                        break;
                    case "classAllow":
                        p1 = String(p1).split(',');
                        list = list.filter(function(item) {
                            return p1.indexOf(String(item._classId)) >= 0;
                        })
                        break;
                    case "classRefuse":
                        p1 = String(p1).split(',');
                        list = list.filter(function(item) {
                            return p1.indexOf(String(item._classId)) < 0;
                        })
                        break;
                    case "paramBig":
                        p1 = Number(p1);
                        p2 = Number(p2);
                        p3 = p3 == 1 || p3 == 'true' || p3 == true ? true : false;
                        list = list.filter(function(item) {
                            return p3 ? item.param(p1) > p2 : item.param(p1) >= p2;
                        })
                        break;
                    case "paramSmall":
                        p1 = Number(p1);
                        p2 = Number(p2);
                        p3 = p3 == 1 || p3 == 'true' || p3 == true ? true : false;
                        list = list.filter(function(item) {
                            return p3 ? item.param(p1) < p2 : item.param(p1) <= p2;
                        })
                        break;
                }
            }
        }
    }
    list.unshift(XYZ.Formation.levelFormationText);
    this._list = list;
    this.refresh();
};

Window_XYZ_Actor_Selectable.prototype.maxItems = function() {
    return this._list ? this._list.length : 0;
};

Window_XYZ_Actor_Selectable.prototype.drawItem = function(index) {
    var item = this._list[index];
    var rect = this.itemRect(index);
    var width = rect.width - this.textPadding();
    this.resetTextColor();
    if (index == 0) {
        this.drawText(item, rect.x + 6, rect.y, width);
    } else {
        this.drawActorName(item, rect.x + 6, rect.y, width);
    }
};

Window_XYZ_Actor_Selectable.prototype.showOpen = function() {
    if (!this.visible) { this.show(); }
    this.activate();
    this.open();
    this.select(0);
};

Window_XYZ_Actor_Selectable.prototype.processOk = function() {
    var index = this.index();
    var actorId = index == 0 ? 0 : this._list[index]._actorId;
    if (this._sceneMap.get('selectFormationPostion')) {
        this._sceneMap.get('selectFormationPostion').setActorPostion(actorId);
        this.deactivate();
        this.close();
        this._sceneMap.get('selectFormationPostion').activate();
    }
};

//信息
function Window_XYZ_Formation_Message() {
    this.initialize.apply(this, arguments);
}

Window_XYZ_Formation_Message.prototype = Object.create(Window_Base.prototype);
Window_XYZ_Formation_Message.prototype.constructor = Window_XYZ_Formation_Message;

Window_XYZ_Formation_Message.prototype.setMessage = function(message) {
    this._message = message;
    this.refresh();
};

Window_XYZ_Formation_Message.prototype.refresh = function() {
    if (this._message) {
        this.contents.clear();
        this.drawTextEx(this._message, 6, 6);
    }
};