//=============================================================================
// Z_JSBB_ItemDropsRate.js
//=============================================================================
/*:
 * @plugindesc Version 1.2 根据物品品质(颜色)，技能，敌人品质动态调整物品掉率
 * @author 金色邦邦 b站搜索：金色邦邦
 * 参数
 * =============================================================================
 * @param Settings
 * @text 物品品质掉率设置
 * @default 
 *
 * @param ItemQuality
 * @text 物品品质掉率设置开关
 * @parent Settings
 * @type combo
 * @option true
 * @option false
 * @desc 控制是否根据物品注释中的颜色品质，进行掉率计算
 * @default true
 *
 * @param Variable1
 * @text 品质1掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质1物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable2
 * @text 品质2掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质2物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable3
 * @text 品质3掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质3物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable4
 * @text 品质4掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质4物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable5
 * @text 品质5掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质5物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable6
 * @text 品质6掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质6物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable7
 * @text 品质7掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质7物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable8
 * @text 品质8掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质8物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable9
 * @text 品质9掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质9物品/武器/护甲掉率的控制变量
 * @default 0
 *
 * @param Variable10
 * @text 品质10掉率变量
 * @parent Settings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质10物品/武器/护甲掉率的控制变量
 * @default 0
 * 
 * @param EnemySettings
 * @text 敌人品质掉率设置
 * @default 
 *
 * @param EnemyQuality
 * @text 敌人品质开关
 * @parent EnemySettings
 * @type combo
 * @option true
 * @option false
 * @desc 控制是否根据敌人注释中的颜色品质，再次进行掉率计算
 * @default false
 *
 * @param Enemy1
 * @text 敌人品质1掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质1敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy2
 * @text 敌人品质2掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质2敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy3
 * @text 敌人品质3掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质3敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy4
 * @text 敌人品质4掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质4敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy5
 * @text 敌人品质5掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质5敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy6
 * @text 敌人品质6掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质6敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy7
 * @text 敌人品质7掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 敌人设置品质7敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy8
 * @text 敌人品质8掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质8敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy9
 * @text 敌人品质9掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质9敌人掉率的控制变量
 * @default 0
 *
 * @param Enemy10
 * @text 敌人品质10掉率变量
 * @parent EnemySettings
 * @type number
 * @min 0
 * @decimals 0
 * @desc 设置品质10敌人掉率的控制变量
 * @default 0
 *
 * @help
 * =============================================================================
 * ★★★正式介绍★★★
 * =============================================================================
 * 一、功能概述:
 *   Drill_ItemTextColor插件的物品注释中的 <颜色:1>等作为物品品质，
 *   Drill_EnemyTextColor插件的敌人注释中的 <颜色:1>等作为敌人品质，
 *   动态设定每种品质物品/武器/防具，每种品质敌人的物品掉率。
 * *****************************************************************************
 * 二、物品品质注释标签 优先级：
 *   兼容物品/武器/防具备注中各种标签设置的物品品级，优先级如下：
 *		<高级颜色> =》<颜色> =》<Quality> =》<quality>
 *	 若以上标签都没取到值，则使用RMMV数据库中设置的掉率，
 *   若使用插件YEP_ExtraEnemyDrops，则使用YEP插件设置的物品掉率。
 * *****************************************************************************
 * 三、默认最高支持10个品级:
 *   常见项目够用了，不够的可自行拓展插件。
 * *****************************************************************************
 * 四、RMMV变量说明：
 *   在项目进程中，可以使用变量操作，对物品掉率进行动态调整。
 *	 这可以使我们灵活的调整掉率，例如：
 *		使用某物品后增加掉率；
 *	    学习技能后增加掉率；
 *		达成成就后增加掉率；
 *		完成任务后增加掉率；
 *	 等等能使用变量之处都可以调整物品掉率。
 *
 *	 变量值说明：
 *   变量=5，代表提升原掉率的5%，变量=15，代表提升原掉率的15%
 *   重要说明：
 *   不是加法！！！不是加法！！！是  原掉率 + 原掉率 * 变量/100 ！！！
 * *****************************************************************************
 * 五、使用示例：
 *   1.在插件中设置Variable1为变量1，RMMV中设置值=5;
 *     设置Enemy1为变量11，值=20;
 *     若不使用YEP_ExtraEnemyDrops插件，
 *          敌人注释示例：              物品Item 1注释示例：      
 *            <颜色:1>                      <颜色:1>
 *     若使用YEP_ExtraEnemyDrops插件，
 *          敌人注释示例：                  物品Item 1注释示例：
 *            <颜色:1>                      <颜色:1>
 *            <Enemy Drops>
 *            Item 1: 10%
 *            </Enemy Drops>
 *   敌人注释说明：
 *   敌人(敌人品质<颜色:1>)注释中掉落物品1(物品品质<颜色:1>)原掉率 = 10%，
 *
 *   开启ItemQuality开关,
 *   则插件计算后的掉率 =》 10% * (1 + (5/100)) = 10.5% 概率掉落，
 *   并且，所有<颜色:1>的物品掉落率都在原掉率基础上乘(1 + (5/100))
 *   若同时开启敌人品质掉率开关，
 *   则计算后的掉率=》10% * (1 + (5+20)/100) = 12.5% 概率掉落
 *
 *   2.物品原掉率 = 10% ，Variable1变量 = 900
 *   则计算后的掉率 =》 10% * (1 + (900/100)) = 100% 必定掉落
 *   3.新增技能Skill注释：
 *      <passiveIRUP:80>或
 *      <passiveIRUP:80%> (兼容dsPassiveSkill被动技能插件)  
 *      要使用技能注释，请打开ItemQuality开关。
 *     技能注释示例：
 *      <颜色:1>
 *      <passiveIRUP:80>
 *     注释说明：品质=1的物品/武器/防具掉率在原掉率基础上提升80%
 * *****************************************************************************
 * 六、拓展应用：
 *	 可拓展的很多，例如：
 *   可结合YEP_MainMenuVar插件，将提升的掉率显示在菜单中
 * =============================================================================
 * ★★★声明★★★
 * =============================================================================
 * 1.该插件可单独使用，同时支持YEP额外敌群掉落插件。
 * 2.已与以下插件兼容，将本插件放在以下插件的下方：
 *	  YEP_ItemCore
 *	  YEP_ExtraEnemyDrops
 *	  Drill_CoreOfColor
 *	  Drill_ItemTextColor
 *	  Drill_EnemyTextColor
 *	  MOG_TrPopUpBattle
 * 3.确保本插件在所有有关掉落率的插件之下。
 * 4.本插件完全免费，可用于商用/非商用项目。
 * 5.本插件可以随意魔改，限于JavaScript技能，出现错误本作者可能无法更改，
 *   若于其他插件冲突，请移除本插件。
 * 6.插件版权归原作者所有，遵守MIT协议。致谢名单中注明插件原作者：金色邦邦。
 * =============================================================================
 * ★★★版本★★★
 * =============================================================================
 * Version 1.3 增加简便实现技能增/减掉率
 * Version 1.2 增加与MOG_TrPopUpBattle掉落动画插件兼容
 * Version 1.1 增加敌人品质影响物品掉落率
 * Version 1.0 完成开发
 * =============================================================================
 */
(function() {
	var parameters = PluginManager.parameters('Z_JSBB_ItemDropsRate');
	var ItemQuality = String(parameters['ItemQuality'] || 'true');
	var Variable1 = String(parameters['Variable1'] || '0');
	var Variable2 = String(parameters['Variable2'] || '0');
	var Variable3 = String(parameters['Variable3'] || '0');
	var Variable4 = String(parameters['Variable4'] || '0');
	var Variable5 = String(parameters['Variable5'] || '0');
	var Variable6 = String(parameters['Variable6'] || '0');
	var Variable7 = String(parameters['Variable7'] || '0');
	var Variable8 = String(parameters['Variable8'] || '0');
	var Variable9 = String(parameters['Variable9'] || '0');
	var Variable10 = String(parameters['Variable10'] || '0');
	
	var EnemyQuality = String(parameters['EnemyQuality'] || 'false');
	var Enemy1 = String(parameters['Enemy1'] || '0');
	var Enemy2 = String(parameters['Enemy2'] || '0');
	var Enemy3 = String(parameters['Enemy3'] || '0');
	var Enemy4 = String(parameters['Enemy4'] || '0');
	var Enemy5 = String(parameters['Enemy5'] || '0');
	var Enemy6 = String(parameters['Enemy6'] || '0');
	var Enemy7 = String(parameters['Enemy7'] || '0');
	var Enemy8 = String(parameters['Enemy8'] || '0');
	var Enemy9 = String(parameters['Enemy9'] || '0');
	var Enemy10 = String(parameters['Enemy10'] || '0');
	
	var _mog_trPopBattle_gEnemy_initMembers = Game_Enemy.prototype.initMembers;
	Game_Enemy.prototype.initMembers = function() {
		_mog_trPopBattle_gEnemy_initMembers.call(this);
		this._treasure = {};
		this._treasure.needPopup = false;
		this._treasure.checked = false;
		this._treasure.item = [];
	};
		
	//打开物品品质掉落开关时，学习和遗忘技能才生效
	if(ItemQuality == 'true'){
		//学习技能后增加掉率；
		Game_Actor.prototype.learnSkill = function(skillId) {
			if (!this.isLearnedSkill(skillId)) {
				this._skills.push(skillId);
				this._skills.sort(function(a, b) {
					return a - b;
				});
				
				skillRate = '' + $dataSkills[skillId].meta.passiveIRUP;
				skillRate = skillRate.replace('%', '');
									
				var skillQuality = $dataSkills[skillId].meta.高级颜色;
				//兼容各种标签设置的物品品级
				if(skillQuality == undefined){
					skillQuality = $dataSkills[skillId].meta.颜色;
					if(skillQuality == undefined){
						skillQuality = $dataSkills[skillId].meta.Quality;
						if(skillQuality == undefined){
							skillQuality = $dataSkills[skillId].meta.quality;
						}
					}
				}
				
				var skillVar = 0;
				if(skillQuality != undefined && skillRate != undefined){
					switch(skillQuality){
					case '0':skillVar = Number(skillRate);break;
					case '1':
						skillVar = $gameVariables.value(Variable1) + Number(skillRate);
						$gameVariables.setValue(Variable1,skillVar)
						break;	
					case '2':
						skillVar = $gameVariables.value(Variable2) + Number(skillRate);
						$gameVariables.setValue(Variable2,skillVar)
						break;		
					case '3':
						skillVar = $gameVariables.value(Variable3) + Number(skillRate);
						$gameVariables.setValue(Variable3,skillVar)
						break;
					case '4':
						skillVar = $gameVariables.value(Variable4) + Number(skillRate);
						$gameVariables.setValue(Variable4,skillVar)
						break;
					case '5':
						skillVar = $gameVariables.value(Variable5) + Number(skillRate);
						$gameVariables.setValue(Variable5,skillVar)
						break;
					case '6':
						skillVar = $gameVariables.value(Variable6) + Number(skillRate);
						$gameVariables.setValue(Variable6,skillVar)
						break;
					case '7':
						skillVar = $gameVariables.value(Variable7) + Number(skillRate);
						$gameVariables.setValue(Variable7,skillVar)
						break;
					case '8':
						skillVar = $gameVariables.value(Variable8) + Number(skillRate);
						$gameVariables.setValue(Variable8,skillVar)
						break;
					case '9':
						skillVar = $gameVariables.value(Variable9) + Number(skillRate);
						$gameVariables.setValue(Variable9,skillVar)
						break;
					case '10':
						skillVar = $gameVariables.value(Variable10) + Number(skillRate);
						$gameVariables.setValue(Variable10,skillVar)
						break;
					}
				}
					
			}
		};
		
		//技能遗忘后，修改掉率变量
		Game_Actor.prototype.forgetSkill = function(skillId) {
			var index = this._skills.indexOf(skillId);
			if (index >= 0) {
				this._skills.splice(index, 1);
			}
			
			skillRate = '' + $dataSkills[skillId].meta.passiveIRUP;
			skillRate = skillRate.replace('%', '');
			
			var skillQuality = $dataSkills[skillId].meta.高级颜色;
			//兼容各种标签设置的物品品级
			if(skillQuality == undefined){
				skillQuality = $dataSkills[skillId].meta.颜色;
				if(skillQuality == undefined){
					skillQuality = $dataSkills[skillId].meta.Quality;
					if(skillQuality == undefined){
						skillQuality = $dataSkills[skillId].meta.quality;
					}
				}
			}
			
			var skillVar = 0;
			if(skillQuality != undefined && skillRate != undefined){
				switch(skillQuality){
				case '0':skillVar = Number(skillRate);break;
				case '1':
					skillVar = $gameVariables.value(Variable1) - Number(skillRate);
					$gameVariables.setValue(Variable1,skillVar)
					break;	
				case '2':
					skillVar = $gameVariables.value(Variable2) - Number(skillRate);
					$gameVariables.setValue(Variable2,skillVar)
					break;		
				case '3':
					skillVar = $gameVariables.value(Variable3) - Number(skillRate);
					$gameVariables.setValue(Variable3,skillVar)
					break;
				case '4':
					skillVar = $gameVariables.value(Variable4) - Number(skillRate);
					$gameVariables.setValue(Variable4,skillVar)
					break;
				case '5':
					skillVar = $gameVariables.value(Variable5) - Number(skillRate);
					$gameVariables.setValue(Variable5,skillVar)
					break;
				case '6':
					skillVar = $gameVariables.value(Variable6) - Number(skillRate);
					$gameVariables.setValue(Variable6,skillVar)
					break;
				case '7':
					skillVar = $gameVariables.value(Variable7) - Number(skillRate);
					$gameVariables.setValue(Variable7,skillVar)
					break;
				case '8':
					skillVar = $gameVariables.value(Variable8) - Number(skillRate);
					$gameVariables.setValue(Variable8,skillVar)
					break;
				case '9':
					skillVar = $gameVariables.value(Variable9) - Number(skillRate);
					$gameVariables.setValue(Variable9,skillVar)
					break;
				case '10':
					skillVar = $gameVariables.value(Variable10) - Number(skillRate);
					$gameVariables.setValue(Variable10,skillVar)
					break;
				}
			}
					
		};
	};	
	
	//掉率动态调整
	Game_Enemy.prototype.makeDropItems = function() {
		if (this._treasure.checked) {
			return this._treasure.item;
		} else {
			return this.enemy().dropItems.reduce(function(r, di) {
				if (di.kind > 0) {
					if(Math.random() * di.denominator < this.dropItemRate(di)){
						return r.concat(this.itemObject(di.kind, di.dataId));
					}else {
						return r;
					}
				} else {
					return r;
				}
			}.bind(this), []);
		};
};
	
	Game_Enemy.prototype.dropItemRate = function(di) {
		
			var rate = 0;		
			var ratio = 1;
			var item = Game_Enemy.prototype.itemObject(di.kind, di.dataId);
			var item1 = di.denominator;
			this._curItem = item;
			var item1 = this._curItem;
			
			if(ItemQuality == 'true'){
				
				var quality = item1.meta.高级颜色;
				//兼容各种标签设置的物品品级
				if(quality == undefined){
					quality = item1.meta.颜色;
					if(quality == undefined){
						quality = item1.meta.Quality;
						if(quality == undefined){
							quality = item1.meta.quality;
						}
					}
				}

				if(quality != undefined){
					switch(quality){
					case '0':rate = 0;break;
					case '1':rate = $gameVariables.value(Variable1);break;	
					case '2':rate = $gameVariables.value(Variable2);break;		
					case '3':rate = $gameVariables.value(Variable3);break;
					case '4':rate = $gameVariables.value(Variable4);break;	
					case '5':rate = $gameVariables.value(Variable5);break;
					case '6':rate = $gameVariables.value(Variable6);break;	
					case '7':rate = $gameVariables.value(Variable7);break;
					case '8':rate = $gameVariables.value(Variable8);break;
					case '9':rate = $gameVariables.value(Variable9);break;
					case '10':rate = $gameVariables.value(Variable10);break;
					}
				}else{
					rate = 0;
				}
				
			}
			
			if(EnemyQuality == 'true'){
				var enemy = $dataEnemies[this._enemyId];
				var quality = enemy.meta.高级颜色;
				if(quality == undefined){
					quality = enemy.meta.颜色;
					if(quality == undefined){
						quality = enemy.meta.Quality;
						if(quality == undefined){
							quality = enemy.meta.quality;
						}
					}
				}

				if(quality != undefined){
					switch(quality){
					case '0':rate += 0;break;
					case '1':rate += $gameVariables.value(Enemy1);break;	
					case '2':rate += $gameVariables.value(Enemy2);break;		
					case '3':rate += $gameVariables.value(Enemy3);break;
					case '4':rate += $gameVariables.value(Enemy4);break;	
					case '5':rate += $gameVariables.value(Enemy5);break;
					case '6':rate += $gameVariables.value(Enemy6);break;	
					case '7':rate += $gameVariables.value(Enemy7);break;
					case '8':rate += $gameVariables.value(Enemy8);break;
					case '9':rate += $gameVariables.value(Enemy9);break;
					case '10':rate += $gameVariables.value(Enemy10);break;
					}
				}else{
					rate = 0;
				}				
			}
			
			ratio = 1 + (rate/100)
		//debugger;	
		return $gameParty.hasDropItemDouble() ? 2*ratio : ratio;
	};
	
})();