﻿//-----------------------------------------------------------------------------
// JHMW_OnlineCore.js
//-----------------------------------------------------------------------------
/*:
 * @plugindesc 江湖梦网MV/MZ联网插件核心3.2版
 * @author 江湖梦网 Fanzi
 * @version 3.2
 * @url https://bbs.jhmeng.cn/
 *
 * @command SaveGame
 * @text 云存档
 * @desc 存档数据异步上传，立刻关闭游戏容易存不上
 *
 * @arg SaveMode
 * @text 存档模式
 * @desc false 为异步存档，游戏流畅但不能立即关闭游戏；on 为同步存档，须等待存档才继续游戏。
 * @type boolean
 * @default
 *
 * @command SceneRanking
 * @text 打开排行榜
 * @desc 可用在事件打开排行榜
 *
 * @command ShowTalk
 * @text 打开聊天窗
 * @desc 可用在事件打开聊天窗
 *
 * @command HideTalk
 * @text 关闭聊天窗
 * @desc 可用在事件关闭聊天窗
 *
 * @command ShowPlayer
 * @text 显示玩家同屏
 * @desc 可用在事件显示玩家同屏
 *
 * @command HidePlayer
 * @text 关闭玩家同屏
 * @desc 可用在事件关闭玩家同屏
 *
 * @command SendWord
 * @text 发送系统消息
 * @desc 可用在事件向全体玩家发送系统消息
 *
 * @arg TalkText
 * @text 消息内容
 * @desc 该内容支持显示文章的颜色代码 \C[n]
 * @default
 *
 * @command AskData
 * @text 查询服务器时间
 * @desc 查询服务器的北京时间返回给参数变量
 *
 * @arg YearVarId
 * @text 年变量
 * @desc 该变量ID用于记录服务器返回时间年份
 * @type variable
 * @default 6
 *
 * @arg MonthVarId
 * @text 月变量
 * @desc 该变量ID用于记录服务器返回时间月份(后面不带参数可不填)
 * @type variable
 * @default 7
 *
 * @arg DateVarId
 * @text 日变量
 * @desc 该变量ID用于记录服务器返回时间日期(后面不带参数可不填)
 * @type variable
 * @default 8
 *
 * @arg HourVarId
 * @text 时变量
 * @desc 该变量ID用于记录服务器返回时间钟点(24小时制，后面不带参数可不填)
 * @type variable
 * @default
 *
 * @arg MinuteVarId
 * @text 分变量
 * @desc 该变量ID用于记录服务器返回时间分钟(可选，后面不带参数时可不填)
 * @type variable
 * @default
 *
 * @arg SecondVarId
 * @text 秒变量
 * @desc 该变量ID用于记录服务器返回时间秒数(可选)
 * @type variable
 * @default
 *
 * @command AskDate
 * @text 查询服务器时间戳
 * @desc 查询服务器的十位时间戳返回给参数变量，两次时间戳之差为时间秒数。
 *
 * @arg CallBack
 * @text 返回变量
 * @desc 该变量ID用于记录服务器返回的十位时间戳(计算机绝对时间)
 * @type variable
 * @default 6
 *
 * @command AskValue
 * @text 访问记数变量
 * @desc 访问服务器记数变量，加一后返回新的值
 *
 * @arg VisitVarId
 * @text 返回值变量
 * @desc 该变量ID用于记录服务器返回的新值
 * @type variable
 * @default 6
 *
 * @command CheckValue
 * @text 查询记数变量
 * @desc 查询服务器记数变量，无操作返回当前值
 *
 * @arg RecordVarId
 * @text 返回值变量
 * @desc 该变量ID用于记录服务器返回的当前值
 * @type variable
 * @default 6
 *
 * @command ResetValue
 * @text 重置记数变量
 * @desc 将服务器记数变量重置归零
 *
 * @command MarkPlayer
 * @text 玩家封号
 * @desc 检测到作弊数据可自动封号
 *
 * @arg MarkType
 * @text 封号类型
 * @desc 开发者自定义封号类型，方便查询对应
 * @type number
 * @min 5
 * @default 5
 *
 * @command AskRank
 * @text 查自己排行
 * @desc 可用在事件中查看自己排名数据，执行后调用“排行记录”变量
 *
 * @command LoadDownload
 * @text 查看云存档
 * @desc 将玩家云存档保存成本地格式查看游戏数据
 *
 * @arg SaveFileId
 * @text 存档编号
 * @desc 保存成的本地存档文件编号
 * @type number
 * @min 0
 * @default 0
 *
 * @param ---主控---
 * @default
 *
 * @param  服务器名
 * @parent ---主控---
 * @desc 游戏在江湖梦网申请到的服务器名
 * @default game
 *
 * @param  后台程序
 * @parent ---主控---
 * @desc 游戏在江湖梦网申请到的后台程序名
 * @default game
 *
 * @param  游戏版本
 * @parent ---主控---
 * @desc 输入当前游戏的版本号，用于检测游戏更新
 * @type number
 * @min 0
 * @default 1
 *
 * @param  下载地址
 * @parent ---主控---
 * @desc 游戏更新的网站入口，论坛发布指引帖子填写地址
 * @default https://bbs.jhmeng.cn/
 *
 * @param  APK打包
 * @parent ---主控---
 * @desc 用于兼容APK打包后的弹出窗口，Website2APK打包填true，其它填false
 * @type boolean
 * @on Website 2 APK
 * @off 其它打包
 * @default false
 *
 * @param  统一变量
 * @parent ---主控---
 * @desc 指定一个变量号，开发者可在服务器修改该变量，登录赋值
 * @type variable
 * @default 2
 *
 * @param  挂机变量
 * @parent ---主控---
 * @desc 离线的服务器时间写入该号变量，单位为秒数
 * @type variable
 * @default 3
 *
 * @param  输入底色
 * @parent ---主控---
 * @desc 登录与聊天输入框的背景底色，填十六进制颜色码，不填为白色。
 * @default #000000
 * 
 * @param 公共事件
 * @parent ---主控---
 * @desc 读档进入游戏调用的公共事件id
 * @type common_event
 * @default
 *
 * @param ---登录---
 * @default
 *
 * @param  游戏名字
 * @parent ---登录---
 * @desc 登陆后显示欢迎语
 * @default 
 *
 * @param  窗口位置
 * @parent ---登录---
 * @desc 登陆窗口显示的高度
 * @type number
 * @min 0
 * @default 100
 *
 * @param  窗口透明度
 * @parent ---登录---
 * @desc 调节登录窗口的透明度
 * @type number
 * @min 0
 * @max 255
 * @default 255
 *
 * @param  弹窗背景
 * @parent ---登录---
 * @desc 弹出窗口背景图片 384 * 144 留空无背景
 * @require 1
 * @dir img/pictures/
 * @type file
 * @default popup
 *
 * @param  存档通知
 * @parent ---登录---
 * @desc 云存档成功后返回提示信息，留空不显示
 * @default 您已成功存档！
 *
 * @param ---排行---
 * @default
 *
 * @param  计算公式
 * @parent ---排行---
 * @desc 排行依据的计算公式，按脚本运行，默认内置公式为最大出战队员的八维属性总和
 * @default $gameParty.sumScore()
 *
 * @param  排行变量
 * @parent ---排行---
 * @desc 浏览排行榜后记录排行数据，查询排行后记录查询者排行数据
 * @type variable
 * @default 4
 *
 * @param  排名变量
 * @parent ---排行---
 * @desc 浏览排行榜后记录排行名次，查询排行后记录查询者排行名次
 * @type variable
 * @default 5
 *
 * @param  菜单排行
 * @parent ---排行---
 * @desc 主菜单指令列表增加显示排行的文本，留空不建立
 * @default 排行
 *
 * @param  排行内容
 * @parent ---排行---
 * @desc 排行界面显示数字序列的表头文本
 * @default 总战力
 *
 * @param  排行背景
 * @parent ---排行---
 * @desc 排行界面背景图片，留空无背景
 * @require 1
 * @dir img/pictures/
 * @type file
 * @default bgd
 *
 * @param  排行透明度
 * @parent ---排行---
 * @desc 调节排行窗口的透明度，独立于聊天窗口
 * @type number
 * @min 0
 * @max 255
 * @default 255
 *
 * @param ---同屏---
 * @default
 *
 * @param  在线时长
 * @parent ---同屏---
 * @desc 玩家多少分钟内有数据更新认定在线
 * @type number
 * @min 0
 * @default 1
 *
 * @param  显示上限
 * @parent ---同屏---
 * @desc 同个地图显示玩家人数限制
 * @type number
 * @min 0
 * @default 10
 *
 * @param  同屏模式
 * @parent ---同屏---
 * @desc true与角色同层穿透，适合按键操作；false为底层不穿透，适合鼠标操作
 * @type boolean
 * @on 同层穿透
 * @off 底层不穿透
 * @default false
 *
 * @param 同屏人物
 * @parent ---同屏---
 * @desc 同屏显示的玩家行走图：0 - 锁定队长   >0 - 对应ID角色
 * @type number
 * @min 0
 * @default 1
 *
 * @param  己方颜色
 * @parent ---同屏---
 * @desc 同屏显示名字区别自己与玩家的颜色，填十六进制颜色码，不填同为白色。
 * @default #ffea00
 *
 * @param  称号变量
 * @parent ---同屏---
 * @desc 同屏显示称号的变量号，将变量赋值字符串作为称号显示在玩家头顶
 * @type variable
 * @default 6
 *
 * @param  显名高度
 * @parent ---同屏---
 * @desc 显示玩家名字距离脚底原点Y位移
 * @type number
 * @min 0
 * @default 10
 *
 * @param  人名字号
 * @parent ---同屏---
 * @desc 显示玩家名字的字体大小
 * @type number
 * @min 1
 * @default 20
 *
 * @param ---聊天---
 * @default
 *
 * @param  聊天窗XY
 * @parent ---聊天---
 * @desc 聊天窗口显示位置(左下角)
 * @default 600,600
 *
 * @param  聊天窗宽
 * @parent ---聊天---
 * @desc 聊天窗口宽度
 * @type number
 * @min 1
 * @default 200
 *
 * @param  聊天明暗
 * @parent ---聊天---
 * @desc 聊天窗口、背景及信息明暗度
 * @type number
 * @min 0
 * @max 255
 * @default 255
 *
 * @param  聊天字号
 * @parent ---聊天---
 * @desc 聊天窗口显示字体大小
 * @type number
 * @min 1
 * @default 20
 *
 * @param  显示聊天
 * @parent ---聊天---
 * @desc 聊天窗口是否默认显示
 * @type boolean
 * @on 显示
 * @off 隐藏
 * @default true
 *
 * @param  系统说话
 * @parent ---聊天---
 * @desc 用于系统说话开关，提示获得物品等
 * @default 恭喜
 *
 * @help
 * 本插件用于将游戏网络化，为游戏提供服务器对接，接入本插件的游
 * 戏将可发布为PC终端、WEB终端、APP终端皆同步通讯，服务器端提
 * 供账号检测、密码保护、单线登录、玩家同屏、云存档、在线双频道
 * 聊天、玩家排行、离线时间、版本更新、统一变量、显示称号、服务
 * 器记数、获取服务器北京时间等功能，内购商城、玩家拍卖行、雇佣
 * 组队、玩家对战、签到等有另外配套插件。插件参数可供开发者自定
 * 义设计游戏，包括透明度与背景图片用于界面美化；排行变量由开发
 * 者在游戏中计算存入指定变量，该变量将与服务器进行通讯，排行名
 * 称自定义；在线时长为判定玩家在线的依据，即该时间内玩家有数据
 * 更新即为在线；显示上线控制同地图玩家显示数量；同屏模式用以调
 * 节其他玩家阻挡手机端的虚拟按钮操作；称号变量在游戏中赋值字符
 * 串用以显示玩家称号；玩家名字位置大小均自定义；聊天窗口大小、
 * 位置、透明度、字体均可自定义；挂机变量用以离线经验奖励调用，
 * 该数值为服务器离线时间单位秒数；统一变量用以指定变量由开发者
 * 在服务器修改，玩家登录即更新其值；插件提供一个系统给所有在线
 * 玩家全体发信息的指令，可结合系统说话参数做系统提示，详见后文。
 * 本插件由 江湖梦网 Fanzi 开发，入驻游戏开发者可免费安装使用，
 * 江湖梦网作为玩家交流管理平台，提供服务器并承载赞助代收与转发。
 *                     有意合作者请登录 江湖梦网 https://bbs.jhmeng.cn/
 *                                               联系QQ：12868880  Fanzi
 * 插件指令:
 * 游戏云存档：SaveGame
 * 需自动云存档则建立并行处理的，公共事件定期执行SaveGame指令，
 * 自动云存档建议600帧执行一次，定点存档直接运行SaveGame指令。
 * 同步存档：SaveGame true      即存档时游戏会等待完成（卡）
 * 显示聊天窗：ShowTalk
 * 隐藏聊天窗：HideTalk
 * 显示同屏玩家：ShowPlayer
 * 隐藏同屏玩家：HidePlayer
 * 打开排行榜：SceneRanking
 * 服务器记数：AskValue n   参数 n 填变量号(可选)新记录返回该变量
 * 服务器查数：CheckValue n   参数 n 填变量号用于查询返回该号变量
 * 服务器清数：ResetValue   开发者可设定记数值达到多少则清零
 * 事件发信息：SendWord \C[2]信息内容\C[0]   （颜色代码同显示文本）
 * 例如系统说话参数为：恭喜，插件指令为：SendWord 捡到金子，当某
 * 个玩家触发该指令时则全体玩家显示“恭喜触发的玩家捡到金子”。
 *
 * 读取服务器时间：AskData a b c...    (可带最多 6 个变量ID作为参数)
 * 注意空格隔开，执行此插件指令后变量一赋值年份，变量二赋值月份，
 * 变量三赋值日期，变量四赋值钟点（24小时制），变量五赋分钟，变量六
 * 赋值秒数，后面的参数可缺省，前面的参数就算不用也得填上。此功能可
 * 用于设计日常任务，若需计算时间差可用时间戳指令，两次相减为秒数。
 *
 * 读取服务器时间戳：AskDate n    执行后变量 n 会被赋值服务器时间戳。
 *
 * 游戏检测封号：MarkPlayer n   参数 n 为封号类型，用于后续查看分析
 * 封号类型 10 以内为系统封号，开发者在游戏中自定义条件分歧，封号类型
 * 使用 10 以后的，比如玩家修改等级封为 10 类，角色属性封为 11 类，变
 * 量为 12 类，这样当检测到玩家作弊会自动走封号程序，游戏后台查询时也
 * 能明确封号原因。
 *
 * 查自己排行：AskRank
 * 执行后“排行记录”变量自动赋值排名，当要使用自己排名做条件分歧时，
 * 请确保刷新一遍“排行记录”变量，浏览排行榜或运行插件指令都可刷新。
 * 
 *
 * 不想显示玩家同屏的地图可备注：<hidePlayer>
 * 备注上的地图将无法使用插件指令开启玩家同屏
 */
//-----------------------------------------------------------------------------

var Imported = Imported || {};
Imported.JHMW_OnlineCore = 3.2;

var JHMW = JHMW || {};
JHMW.Param = JHMW.Param || {};
JHMW.OnlineCore = JHMW.OnlineCore || {};
JHMW.OnlineCore.PluginName = 'JHMW_OnlineCore';
JHMW.OnlineCore.Parameters = PluginManager.parameters('JHMW_OnlineCore');

function Scene_Login() {
    this.initialize.apply(this, arguments);
};

function Scene_Ranking() {
    this.initialize.apply(this, arguments);
};

function Window_LoginCommand() {
    this.initialize.apply(this, arguments);
}

function Window_LoginInfo() {
    this.initialize.apply(this, arguments);
}

function Window_Ranking() {
    this.initialize.apply(this, arguments);
}

function Window_Action() {
    this.initialize.apply(this, arguments);
}

function Window_TalkList() {
    this.initialize.apply(this, arguments);
}

function Window_RankCmd() {
    this.initialize.apply(this, arguments);
}

function Sprite_Online() {
    this.initialize.apply(this, arguments);
};

var chrsz=8;var hexcase=0;var b64pad="";JHMW.MD5=function(instring){var hexcase=0;var b64pad="";function hex_md5(s){return rstr2hex(rstr_md5(str2rstr_utf8(s)))};function b64_md5(s){return rstr2b64(rstr_md5(str2rstr_utf8(s)))};function any_md5(s,e){return rstr2any(rstr_md5(str2rstr_utf8(s)),e)};function hex_hmac_md5(k,d){return rstr2hex(rstr_hmac_md5(str2rstr_utf8(k),str2rstr_utf8(d)))};function b64_hmac_md5(k,d){return rstr2b64(rstr_hmac_md5(str2rstr_utf8(k),str2rstr_utf8(d)))};function any_hmac_md5(k,d,e){return rstr2any(rstr_hmac_md5(str2rstr_utf8(k),str2rstr_utf8(d)),e)};function md5_vm_test(){return hex_md5("abc").toLowerCase()=="900150983cd24fb0d6963f7d28e17f72"};function rstr_md5(s){return binl2rstr(binl_md5(rstr2binl(s),s.length*8))};function rstr_hmac_md5(key,data){var bkey=rstr2binl(key);if(bkey.length>16)bkey=binl_md5(bkey,key.length*8);var ipad=Array(16),opad=Array(16);for(var i=0;i<16;i++){ipad[i]=bkey[i]^0x36363636;opad[i]=bkey[i]^0x5C5C5C5C}var hash=binl_md5(ipad.concat(rstr2binl(data)),512+data.length*8);return binl2rstr(binl_md5(opad.concat(hash),512+128))};function rstr2hex(input){try{hexcase}catch(e){hexcase=0};var hex_tab=hexcase?"0123456789ABCDEF":"0123456789abcdef";var output="";var x;for(var i=0;i<input.length;i++){x=input.charCodeAt(i);output+=hex_tab.charAt((x>>>4)&0x0F)+hex_tab.charAt(x&0x0F)}return output};function rstr2b64(input){try{b64pad}catch(e){b64pad=''};var tab="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";var output="";var len=input.length;for(var i=0;i<len;i+=3){var triplet=(input.charCodeAt(i)<<16)|(i+1<len?input.charCodeAt(i+1)<<8:0)|(i+2<len?input.charCodeAt(i+2):0);for(var j=0;j<4;j++){if(i*8+j*6>input.length*8)output+=b64pad;else output+=tab.charAt((triplet>>>6*(3-j))&0x3F)}}return output};function rstr2any(input,encoding){var divisor=encoding.length;var i,j,q,x,quotient;var dividend=Array(Math.ceil(input.length/2));for(i=0;i<dividend.length;i++){dividend[i]=(input.charCodeAt(i*2)<<8)|input.charCodeAt(i*2+1)}var full_length=Math.ceil(input.length*8/(Math.log(encoding.length)/Math.log(2)));var remainders=Array(full_length);for(j=0;j<full_length;j++){quotient=Array();x=0;for(i=0;i<dividend.length;i++){x=(x<<16)+dividend[i];q=Math.floor(x/divisor);x-=q*divisor;if(quotient.length>0||q>0)quotient[quotient.length]=q}remainders[j]=x;dividend=quotient}var output="";for(i=remainders.length-1;i>=0;i--)output+=encoding.charAt(remainders[i]);return output};function str2rstr_utf8(input){var output="";var i=-1;var x,y;while(++i<input.length){x=input.charCodeAt(i);y=i+1<input.length?input.charCodeAt(i+1):0;if(0xD800<=x&&x<=0xDBFF&&0xDC00<=y&&y<=0xDFFF){x=0x10000+((x&0x03FF)<<10)+(y&0x03FF);i++}if(x<=0x7F)output+=String.fromCharCode(x);else if(x<=0x7FF)output+=String.fromCharCode(0xC0|((x>>>6)&0x1F),0x80|(x&0x3F));else if(x<=0xFFFF)output+=String.fromCharCode(0xE0|((x>>>12)&0x0F),0x80|((x>>>6)&0x3F),0x80|(x&0x3F));else if(x<=0x1FFFFF)output+=String.fromCharCode(0xF0|((x>>>18)&0x07),0x80|((x>>>12)&0x3F),0x80|((x>>>6)&0x3F),0x80|(x&0x3F))}return output};function str2rstr_utf16le(input){var output="";for(var i=0;i<input.length;i++)output+=String.fromCharCode(input.charCodeAt(i)&0xFF,(input.charCodeAt(i)>>>8)&0xFF);return output};function str2rstr_utf16be(input){var output="";for(var i=0;i<input.length;i++)output+=String.fromCharCode((input.charCodeAt(i)>>>8)&0xFF,input.charCodeAt(i)&0xFF);return output};function rstr2binl(input){var output=Array(input.length>>2);for(var i=0;i<output.length;i++)output[i]=0;for(var i=0;i<input.length*8;i+=8)output[i>>5]|=(input.charCodeAt(i/8)&0xFF)<<(i%32);return output};function binl2rstr(input){var output="";for(var i=0;i<input.length*32;i+=8)output+=String.fromCharCode((input[i>>5]>>>(i%32))&0xFF);return output};function binl_md5(x,len){x[len>>5]|=0x80<<((len)%32);x[(((len+64)>>>9)<<4)+14]=len;var a=1732584193;var b=-271733879;var c=-1732584194;var d=271733878;for(var i=0;i<x.length;i+=16){var olda=a;var oldb=b;var oldc=c;var oldd=d;a=md5_ff(a,b,c,d,x[i+0],7,-680876936);d=md5_ff(d,a,b,c,x[i+1],12,-389564586);c=md5_ff(c,d,a,b,x[i+2],17,606105819);b=md5_ff(b,c,d,a,x[i+3],22,-1044525330);a=md5_ff(a,b,c,d,x[i+4],7,-176418897);d=md5_ff(d,a,b,c,x[i+5],12,1200080426);c=md5_ff(c,d,a,b,x[i+6],17,-1473231341);b=md5_ff(b,c,d,a,x[i+7],22,-45705983);a=md5_ff(a,b,c,d,x[i+8],7,1770035416);d=md5_ff(d,a,b,c,x[i+9],12,-1958414417);c=md5_ff(c,d,a,b,x[i+10],17,-42063);b=md5_ff(b,c,d,a,x[i+11],22,-1990404162);a=md5_ff(a,b,c,d,x[i+12],7,1804603682);d=md5_ff(d,a,b,c,x[i+13],12,-40341101);c=md5_ff(c,d,a,b,x[i+14],17,-1502002290);b=md5_ff(b,c,d,a,x[i+15],22,1236535329);a=md5_gg(a,b,c,d,x[i+1],5,-165796510);d=md5_gg(d,a,b,c,x[i+6],9,-1069501632);c=md5_gg(c,d,a,b,x[i+11],14,643717713);b=md5_gg(b,c,d,a,x[i+0],20,-373897302);a=md5_gg(a,b,c,d,x[i+5],5,-701558691);d=md5_gg(d,a,b,c,x[i+10],9,38016083);c=md5_gg(c,d,a,b,x[i+15],14,-660478335);b=md5_gg(b,c,d,a,x[i+4],20,-405537848);a=md5_gg(a,b,c,d,x[i+9],5,568446438);d=md5_gg(d,a,b,c,x[i+14],9,-1019803690);c=md5_gg(c,d,a,b,x[i+3],14,-187363961);b=md5_gg(b,c,d,a,x[i+8],20,1163531501);a=md5_gg(a,b,c,d,x[i+13],5,-1444681467);d=md5_gg(d,a,b,c,x[i+2],9,-51403784);c=md5_gg(c,d,a,b,x[i+7],14,1735328473);b=md5_gg(b,c,d,a,x[i+12],20,-1926607734);a=md5_hh(a,b,c,d,x[i+5],4,-378558);d=md5_hh(d,a,b,c,x[i+8],11,-2022574463);c=md5_hh(c,d,a,b,x[i+11],16,1839030562);b=md5_hh(b,c,d,a,x[i+14],23,-35309556);a=md5_hh(a,b,c,d,x[i+1],4,-1530992060);d=md5_hh(d,a,b,c,x[i+4],11,1272893353);c=md5_hh(c,d,a,b,x[i+7],16,-155497632);b=md5_hh(b,c,d,a,x[i+10],23,-1094730640);a=md5_hh(a,b,c,d,x[i+13],4,681279174);d=md5_hh(d,a,b,c,x[i+0],11,-358537222);c=md5_hh(c,d,a,b,x[i+3],16,-722521979);b=md5_hh(b,c,d,a,x[i+6],23,76029189);a=md5_hh(a,b,c,d,x[i+9],4,-640364487);d=md5_hh(d,a,b,c,x[i+12],11,-421815835);c=md5_hh(c,d,a,b,x[i+15],16,530742520);b=md5_hh(b,c,d,a,x[i+2],23,-995338651);a=md5_ii(a,b,c,d,x[i+0],6,-198630844);d=md5_ii(d,a,b,c,x[i+7],10,1126891415);c=md5_ii(c,d,a,b,x[i+14],15,-1416354905);b=md5_ii(b,c,d,a,x[i+5],21,-57434055);a=md5_ii(a,b,c,d,x[i+12],6,1700485571);d=md5_ii(d,a,b,c,x[i+3],10,-1894986606);c=md5_ii(c,d,a,b,x[i+10],15,-1051523);b=md5_ii(b,c,d,a,x[i+1],21,-2054922799);a=md5_ii(a,b,c,d,x[i+8],6,1873313359);d=md5_ii(d,a,b,c,x[i+15],10,-30611744);c=md5_ii(c,d,a,b,x[i+6],15,-1560198380);b=md5_ii(b,c,d,a,x[i+13],21,1309151649);a=md5_ii(a,b,c,d,x[i+4],6,-145523070);d=md5_ii(d,a,b,c,x[i+11],10,-1120210379);c=md5_ii(c,d,a,b,x[i+2],15,718787259);b=md5_ii(b,c,d,a,x[i+9],21,-343485551);a=safe_add(a,olda);b=safe_add(b,oldb);c=safe_add(c,oldc);d=safe_add(d,oldd)}return Array(a,b,c,d)};function md5_cmn(q,a,b,x,s,t){return safe_add(bit_rol(safe_add(safe_add(a,q),safe_add(x,t)),s),b)};function md5_ff(a,b,c,d,x,s,t){return md5_cmn((b&c)|((~b)&d),a,b,x,s,t)};function md5_gg(a,b,c,d,x,s,t){return md5_cmn((b&d)|(c&(~d)),a,b,x,s,t)};function md5_hh(a,b,c,d,x,s,t){return md5_cmn(b^c^d,a,b,x,s,t)};function md5_ii(a,b,c,d,x,s,t){return md5_cmn(c^(b|(~d)),a,b,x,s,t)};function safe_add(x,y){var lsw=(x&0xFFFF)+(y&0xFFFF);var msw=(x>>16)+(y>>16)+(lsw>>16);return(msw<<16)|(lsw&0xFFFF)};function bit_rol(num,cnt){return(num<<cnt)|(num>>>(32-cnt))};return hex_md5(instring).toUpperCase()};
var LZString={_keyStr:"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",_f:String.fromCharCode,compressToBase64:function(e){if(e==null)return"";var t="";var n,r,i,s,o,u,a;var f=0;e=LZString.compress(e);while(f<e.length*2){if(f%2==0){n=e.charCodeAt(f/2)>>8;r=e.charCodeAt(f/2)&255;if(f/2+1<e.length)i=e.charCodeAt(f/2+1)>>8;else i=NaN}else{n=e.charCodeAt((f-1)/2)&255;if((f+1)/2<e.length){r=e.charCodeAt((f+1)/2)>>8;i=e.charCodeAt((f+1)/2)&255}else r=i=NaN}f+=3;s=n>>2;o=(n&3)<<4|r>>4;u=(r&15)<<2|i>>6;a=i&63;if(isNaN(r)){u=a=64}else if(isNaN(i)){a=64}t=t+LZString._keyStr.charAt(s)+LZString._keyStr.charAt(o)+LZString._keyStr.charAt(u)+LZString._keyStr.charAt(a)}return t},decompressFromBase64:function(e){if(e==null)return"";var t="",n=0,r,i,s,o,u,a,f,l,c=0,h=LZString._f;e=e.replace(/[^A-Za-z0-9\+\/\=]/g,"");while(c<e.length){u=LZString._keyStr.indexOf(e.charAt(c++));a=LZString._keyStr.indexOf(e.charAt(c++));f=LZString._keyStr.indexOf(e.charAt(c++));l=LZString._keyStr.indexOf(e.charAt(c++));i=u<<2|a>>4;s=(a&15)<<4|f>>2;o=(f&3)<<6|l;if(n%2==0){r=i<<8;if(f!=64){t+=h(r|s)}if(l!=64){r=o<<8}}else{t=t+h(r|i);if(f!=64){r=s<<8}if(l!=64){t+=h(r|o)}}n+=3}return LZString.decompress(t)},compressToUTF16:function(e){if(e==null)return"";var t="",n,r,i,s=0,o=LZString._f;e=LZString.compress(e);for(n=0;n<e.length;n++){r=e.charCodeAt(n);switch(s++){case 0:t+=o((r>>1)+32);i=(r&1)<<14;break;case 1:t+=o(i+(r>>2)+32);i=(r&3)<<13;break;case 2:t+=o(i+(r>>3)+32);i=(r&7)<<12;break;case 3:t+=o(i+(r>>4)+32);i=(r&15)<<11;break;case 4:t+=o(i+(r>>5)+32);i=(r&31)<<10;break;case 5:t+=o(i+(r>>6)+32);i=(r&63)<<9;break;case 6:t+=o(i+(r>>7)+32);i=(r&127)<<8;break;case 7:t+=o(i+(r>>8)+32);i=(r&255)<<7;break;case 8:t+=o(i+(r>>9)+32);i=(r&511)<<6;break;case 9:t+=o(i+(r>>10)+32);i=(r&1023)<<5;break;case 10:t+=o(i+(r>>11)+32);i=(r&2047)<<4;break;case 11:t+=o(i+(r>>12)+32);i=(r&4095)<<3;break;case 12:t+=o(i+(r>>13)+32);i=(r&8191)<<2;break;case 13:t+=o(i+(r>>14)+32);i=(r&16383)<<1;break;case 14:t+=o(i+(r>>15)+32,(r&32767)+32);s=0;break}}return t+o(i+32)},decompressFromUTF16:function(e){if(e==null)return"";var t="",n,r,i=0,s=0,o=LZString._f;while(s<e.length){r=e.charCodeAt(s)-32;switch(i++){case 0:n=r<<1;break;case 1:t+=o(n|r>>14);n=(r&16383)<<2;break;case 2:t+=o(n|r>>13);n=(r&8191)<<3;break;case 3:t+=o(n|r>>12);n=(r&4095)<<4;break;case 4:t+=o(n|r>>11);n=(r&2047)<<5;break;case 5:t+=o(n|r>>10);n=(r&1023)<<6;break;case 6:t+=o(n|r>>9);n=(r&511)<<7;break;case 7:t+=o(n|r>>8);n=(r&255)<<8;break;case 8:t+=o(n|r>>7);n=(r&127)<<9;break;case 9:t+=o(n|r>>6);n=(r&63)<<10;break;case 10:t+=o(n|r>>5);n=(r&31)<<11;break;case 11:t+=o(n|r>>4);n=(r&15)<<12;break;case 12:t+=o(n|r>>3);n=(r&7)<<13;break;case 13:t+=o(n|r>>2);n=(r&3)<<14;break;case 14:t+=o(n|r>>1);n=(r&1)<<15;break;case 15:t+=o(n|r);i=0;break}s++}return LZString.decompress(t)},compressToUint8Array:function(e){var t=LZString.compress(e);var n=new Uint8Array(t.length*2);for(var r=0,i=t.length;r<i;r++){var s=t.charCodeAt(r);n[r*2]=s>>>8;n[r*2+1]=s%256}return n},decompressFromUint8Array:function(e){if(e===null||e===undefined){return LZString.decompress(e)}else{var t=new Array(e.length/2);for(var n=0,r=t.length;n<r;n++){t[n]=e[n*2]*256+e[n*2+1]}return LZString.decompress(String.fromCharCode.apply(null,t))}},compressToEncodedURIComponent:function(e){return LZString.compressToBase64(e).replace(/=/g,"$").replace(/\//g,"-")},decompressFromEncodedURIComponent:function(e){if(e)e=e.replace(/$/g,"=").replace(/-/g,"/");return LZString.decompressFromBase64(e)},compress:function(e){if(e==null)return"";var t,n,r={},i={},s="",o="",u="",a=2,f=3,l=2,c="",h=0,p=0,d,v=LZString._f;for(d=0;d<e.length;d+=1){s=e.charAt(d);if(!Object.prototype.hasOwnProperty.call(r,s)){r[s]=f++;i[s]=true}o=u+s;if(Object.prototype.hasOwnProperty.call(r,o)){u=o}else{if(Object.prototype.hasOwnProperty.call(i,u)){if(u.charCodeAt(0)<256){for(t=0;t<l;t++){h=h<<1;if(p==15){p=0;c+=v(h);h=0}else{p++}}n=u.charCodeAt(0);for(t=0;t<8;t++){h=h<<1|n&1;if(p==15){p=0;c+=v(h);h=0}else{p++}n=n>>1}}else{n=1;for(t=0;t<l;t++){h=h<<1|n;if(p==15){p=0;c+=v(h);h=0}else{p++}n=0}n=u.charCodeAt(0);for(t=0;t<16;t++){h=h<<1|n&1;if(p==15){p=0;c+=v(h);h=0}else{p++}n=n>>1}}a--;if(a==0){a=Math.pow(2,l);l++}delete i[u]}else{n=r[u];for(t=0;t<l;t++){h=h<<1|n&1;if(p==15){p=0;c+=v(h);h=0}else{p++}n=n>>1}}a--;if(a==0){a=Math.pow(2,l);l++}r[o]=f++;u=String(s)}}if(u!==""){if(Object.prototype.hasOwnProperty.call(i,u)){if(u.charCodeAt(0)<256){for(t=0;t<l;t++){h=h<<1;if(p==15){p=0;c+=v(h);h=0}else{p++}}n=u.charCodeAt(0);for(t=0;t<8;t++){h=h<<1|n&1;if(p==15){p=0;c+=v(h);h=0}else{p++}n=n>>1}}else{n=1;for(t=0;t<l;t++){h=h<<1|n;if(p==15){p=0;c+=v(h);h=0}else{p++}n=0}n=u.charCodeAt(0);for(t=0;t<16;t++){h=h<<1|n&1;if(p==15){p=0;c+=v(h);h=0}else{p++}n=n>>1}}a--;if(a==0){a=Math.pow(2,l);l++}delete i[u]}else{n=r[u];for(t=0;t<l;t++){h=h<<1|n&1;if(p==15){p=0;c+=v(h);h=0}else{p++}n=n>>1}}a--;if(a==0){a=Math.pow(2,l);l++}}n=2;for(t=0;t<l;t++){h=h<<1|n&1;if(p==15){p=0;c+=v(h);h=0}else{p++}n=n>>1}while(true){h=h<<1;if(p==15){c+=v(h);break}else p++}return c},decompress:function(e){if(e==null)return"";if(e=="")return null;var t=[],n,r=4,i=4,s=3,o="",u="",a,f,l,c,h,p,d,v=LZString._f,m={string:e,val:e.charCodeAt(0),position:32768,index:1};for(a=0;a<3;a+=1){t[a]=a}l=0;h=Math.pow(2,2);p=1;while(p!=h){c=m.val&m.position;m.position>>=1;if(m.position==0){m.position=32768;m.val=m.string.charCodeAt(m.index++)}l|=(c>0?1:0)*p;p<<=1}switch(n=l){case 0:l=0;h=Math.pow(2,8);p=1;while(p!=h){c=m.val&m.position;m.position>>=1;if(m.position==0){m.position=32768;m.val=m.string.charCodeAt(m.index++)}l|=(c>0?1:0)*p;p<<=1}d=v(l);break;case 1:l=0;h=Math.pow(2,16);p=1;while(p!=h){c=m.val&m.position;m.position>>=1;if(m.position==0){m.position=32768;m.val=m.string.charCodeAt(m.index++)}l|=(c>0?1:0)*p;p<<=1}d=v(l);break;case 2:return""}t[3]=d;f=u=d;while(true){if(m.index>m.string.length){return""}l=0;h=Math.pow(2,s);p=1;while(p!=h){c=m.val&m.position;m.position>>=1;if(m.position==0){m.position=32768;m.val=m.string.charCodeAt(m.index++)}l|=(c>0?1:0)*p;p<<=1}switch(d=l){case 0:l=0;h=Math.pow(2,8);p=1;while(p!=h){c=m.val&m.position;m.position>>=1;if(m.position==0){m.position=32768;m.val=m.string.charCodeAt(m.index++)}l|=(c>0?1:0)*p;p<<=1}t[i++]=v(l);d=i-1;r--;break;case 1:l=0;h=Math.pow(2,16);p=1;while(p!=h){c=m.val&m.position;m.position>>=1;if(m.position==0){m.position=32768;m.val=m.string.charCodeAt(m.index++)}l|=(c>0?1:0)*p;p<<=1}t[i++]=v(l);d=i-1;r--;break;case 2:return u}if(r==0){r=Math.pow(2,s);s++}if(t[d]){o=t[d]}else{if(d===i){o=f+f.charAt(0)}else{return null}}u+=o;t[i++]=f+o.charAt(0);r--;f=o;if(r==0){r=Math.pow(2,s);s++}}}};if(typeof module!=="undefined"&&module!=null){module.exports=LZString};
var _0xodZ='jsjiami.com.v6',_0xodZ_=['_0xodZ'],_0x23d3=[_0xodZ,'a8OsHcOAwpzDicKQfMK/w7NFFQ==','FgpBPA7CjEDDrHU=','RcOUCxzCtw==','aFXCgsKEwpw=','w5LDhALDrnY=','Hxl8wovDpg==','IlLCrzzCnA==','w4FmGMOmw5XDohjCnMOpw68qTw==','KsOxWG3Dqw==','b8OIw4h8Tn9YwqBOw60=','KAMZWsOQw7l1w5x6','wpHDg8Olw40v','woVkw5/DtGM=','QgcDwqLDiw==','LwzCtxDCsQ==','EQMNfcOiw752w5x6','wpgww4HCijM=','QiwDWsKKFcOzBw==','wrcZwrbDmwXClsKWwqLDt0U=','wr7CvDAKBg==','w43CoQMlJDzDlXZowrbDvG7CisKZ','wr/DpjPCtjQ=','Y2Mpw6LDtg5TB1Q=','X8K2ScKeCQ==','PH1yw6ds','DErCuDzCkcOww4R7','wqDDjmEzwo8=','TGDCqMKXwofDrcOISg==','LcObTW7Dv2w=','fsOTw7Zmw6Bw','ExHCkzHCk1RlwqZ9wqHDrw4=','f8Opw65Rw4s=','YsKZSsKQNcOGwq8g','wpPDoyjCih4fw6vCkQBkEUjCucOC','LBEydcOQw7l3w6t6w53Dl8KfE8OWw74=','w6LDqcOKHzHCoTUOw4Y=','KHwsCWI=','ecKlfMKAOA==','EkDDq3Qg','e8K3fsOZEQ==','wqXCicOJw50B','w7VpwozCicOnw43CsXFewprDvDE=','wqrDlE7DhsOx','wrbCicOJw7gBwpfChVVxwpU=','M1bDvG8hw6B+BSI=','wpvCkcOTw6QB','w7DDtAHDgHo=','wq0jw5HCiiE=','wqTDg30bwrI=','HQpJMxDCi1TDrWk=','DQVGEgk=','woDDj0RJwr7DmGkg','w5fCoRwkEg==','AS/CpTzDpwPCtUQ=','wozDgHjDhMOYRA==','UsKeS8KUFcOVwrYoA8OeTg==','wqvDjEA9woHDvzJPZsKd','Fw5qwqXDpg==','EsKTw7Z/wqgow6HCsQsAwoxsw6Y=','WcORw456Q1dawrlBw7w=','J8OJY3c=','McKJw4fDgsKXK8OqMCnDqnTCjRU=','wrjDtMOtw6RTfFF5w7cp','w6Fhwo7CgQ==','GUzCrg/Cl8Ojw6d6wrDDhsKb','wrzDu8O8w5NVcU4=','wpfDsRfCnTwew7XCmgR4B23CvsOWwr0=','w4VnGcO3w4PDsA==','5ou65o6x5Z6b5ZGn','ADfCpinDghM=','5Li75Lux6aGQ','ASFMwpjDtGw=','5Lid5LmN6aCa','MFQRLl4lw5vCpw==','TcK+VcOACw==','wqcmCMOpHw==','dETCrcKLwrM=','wpbDlhnCixA=','wobCtRoMDQ==','NsOJaWvDgw==','wo7DmsOqwrEA','w4s9SsKfZ8K+','BQ80ccOXcjFLwo3CvMOL','55ae5oi45ZG7776Y','woQLwr/DnQ==','5aytA8KT56KM77yR','6Lyy5LiJ6LaV5Y265q2o5Zyz5L6n55SK77+86KyE5o2M5LmA6Le05Y+w6Ye05pSJ55uW5byU5rmA5omo44GG','Q2TCtcKXwqDDqg==','6LS45Y2y56ed5q+Q5aej57iX55qj5b2M77+a5aSW6KKG55q25Y2+6K6V5Yu/6K+65Z2t5pWy5a2Y56CG44OA','w5XDmB/DqGk=','RMOOw4Vkew==','wrRzw5/DrHJ+wqo=','H8KTw51TwpAGw5Y=','ME1Rw7xqw60=','UsOWw5NjS0Y=','wrbDh1E9wqvDsTJJbg==','GxQvd8KVwr1dEw==','csONPT/Cljl3','woHDhcOkw44qbw==','KgzCmCjCr0t6woo=','w4xhwo/CiMObw5PCg3E=','c2k3w5jDthRcP0xJCxEX','w6lqAMOnw5TDoSM=','w43CoQMlMiLDn29Pwo/DtHfCjMKC','AS/CpTzDoADCo1Nkwo9Zw51twpc=','X8Osw7xKw5s=','YcKQV8KGDA==','wpsuw6TCoR4hDw==','GCbCswrCtw==','aBk4wq/Djg==','wqbDncOmw7d1','w6TCmjgnRHFfCA==','MSIUY8OP','E2MMDlE=','w5zDmg/DtkI=','w67DnsOUPhM=','wqzCrz0IN09Jw6M=','w6TChjYEHQ==','w6FvwozCmcOtw5TCjWdgwpTDsDHDiA==','dEoBw4/DuQ==','Z1jCtcK5wqA=','wpjDs8OJwpUA','w7cqWcKbfcKvwpw=','w6BpwpbCgMOpw4o=','f2TCrcKGwqvDrMOI','dcOJw4B+w7J5','JloYP2kjw5bCrA==','GcKBw57Dk8KtLMO3HQ==','HyvCsDrDgAXCrnhmwqRYw50=','w43CoQMlJDzDlXY=','BQUDb8Kpwr5JGMOxw7g=','w6zDlgvDi3cEW8OIwqPDr2vCvXo=','MMONd28=','wqYXDsOmDQ==','wqPDpm4Dwp0=','wosLwrfDnQTCrQ==','TGTCusKHwqDDqg==','wrDDgF/Dg8OlXFF6','wrjDi0lGwp3DhQ==','KFbDqFYh','bsK2Ch3Dky42CcKd','CT0AecKAwrda','GSvCrjzDqyE=','wrbDh1xPwpnDhw==','CwU7ccOWciF8wpzCq8ONVzrCqsKvwpw=','FjtJOT7Ch0I=','wqXCt8OEw4h0cQ==','LhBHwpzDuHl/','IsKqw4VfwogT','woDDkMO4woMfTXkVEsO1DsKeeF8XOw==','wrDDhMO/w5Qbc8OCHMKHw6w=','wrLDksOrwp8=','IMKlw5fDpsKL','w4PDhx/Dp1VMDw==','JA1sLxU=','JsONQ1bDiw==','woQnFsOGEQ==','wrbCrsOKw6scwrvCmQ==','eMOlbMKsasOV','GQIiXsK7','CsOJYn7Dn3zChms=','wp9Gw4vDuUVM','woTCiMOlw5dA','YcOBw5hmw7Y=','WMOsLsOBwo3DpsK5dMKz','wrrDo1bDkMOSWE4=','w5/DocOaFhnCvw==','wok6worDp8Kb','w5rDkznDp0U=','wrpmw4fDtFg=','wovDvkRDwoHDknU=','VGTCo8KXwovDjw==','wrLDpcO1wpETWm4=','w6rDlhXDjVQg','w7cfQ8KfasK+wp0=','w5XCsgA7Yg==','G8OJa3/DhXvCjHsS','wrQpwojDtcKF','VMKBXcOEHw==','d2A4w6/DkA==','wpY6wrrDtsKk','IUrCji3Cug==','RsKYS8KDDsOQwosuHsOWVDA=','wpvDkcOIwoMN','SsOkMMOZwq7DgcK8bcK+','SMOXw5thw7dAETnCnMKx','wqXCvCQzBkRWw6NB','HyHCoybDgQ==','wrPDqDvCgQo=','w47DvMO+CzU=','GVTCsz7CvQ==','ClbDqEgTw6d9BSI=','ZMOBDMOWwq0=','FMKiw51Zwq8Mw4rClw==','woQPwqrDnTHCs8KZwqvDp0PDp8Ohw5h+','wodCw4DDuU1ywqpRwoTCvU8Hwrc=','asK9WcOnOcKVw4nDoMO2ZMO2wpM=','wrJ8w5DCkcK9w4bDjWgEwoHCpTnClg==','ecOpdsK1bsOX','wpDCri8tGw==','wrzDicOmw4Yla8OQ','d8K2U8O8MsKDw5I=','wq7Dg1o9wq3DqDtIaMK2QsOkfUM=','wo7Ct8OKw5lcUjA=','OAwBfQ==','GgcXc8Obw6Nh','PEtRw7Q=','wqfCszAWDU9C','wrzDgsO/w5cu','c2nCtMKUwpXDtMOaSlfDoA==','d8OpbMKg','woAHwr3DjDHCs8KZwqvDp0M=','RcOpbsKkZcORLw==','PF7DsE4lw74=','wo3CjMOHw70xwrvCk1E=','AVLDskYqw7pq','AMOgW2rDvg==','w7huG8Onw6nDvCrCnA==','w4zDjgjDu15KDw==','wrbDh1E9wr8=','wp/CsQkqNA==','b8Oaw5FrREJF','w7zDmhnDlHMH','bsKUU8KCD8OTwrA=','TiIlwoI=','w5hpwrPCnsOM','BsKnw53Dk8K/','YjomecK1','Y8OFw4xnw4Q=','wqXDv8O8w6RuSg==','Xmknw4/DvRNH','wrbDh1E9wrzDiQ==','QSE6ZMKb','VGTCo8KXwpHDjw==','w77Djh/DrUVMGXjDjGw1wrBlwoxjwrY=','H1AALlQ+w58=','CMKFw4XDk8KQMcO+','wp9Gw4vDuVw=','csO4JzvCgSh2','w6LCgS09cWQ=','wpfDtR3CiwoDw73CowBuF3bCvsOBwr02','w77CpQY0GSDDiQ==','LMKiw4JGwqwVw4HCjSAAwqJVw4bCpg==','wrbCm8OQw68LwqrCmA==','McOYa3rDuHA=','dh5VasKFw6sjw5Qnw4DCiw==','w6PDpsK1w6wOYQ1swr4=','WcKIR8KfFw==','w6DDmxLDt0Q=','TCgwwoPDmsKY','wokkwrLDvww=','w4QuXMKKQ8K3wo7ChAhwB8KOw5vDgA==','wq/DoxnCijEQw7XCkg==','wqnClzcpDg==','w6zCiSokVn1VDwrCnEFhMQ==','DwJXKzPCrFHDtGQ=','SMOww5hyw6psDQ==','esKxS8O4PcKW','w7DDhxvDv0I=','w7BlwoTCn8Otw4nCkURbwpzDrSDDkg==','wozDv8OywqYH','MlbDt1cXw6Z2Bg==','BMKIw5zDgcKuKcOsASnDsQ==','fsO1PinCvzBkY0nDmg==','JcKFw5XDhMKbNsOlKCDDomrCgBM=','w51Qwo7CjMOxw5/Ciw==','I8Kvw5RTwps=','IALChSjCr0pvwpg=','w5/CuDUxaXFV','woPDiFhPwrQ=','wobCtMOMw5F+','RjcTwoPDqQ==','W8K8CgvDgg4O','wq7DgUdPwqvDlGY4w5E=','Egk2Z8OrZS1PwpHCpw==','w6o6ScKYdsKpwrY=','wqgzw5PCgAo=','woHDtsOlw6lfb04=','wrDDiExlwrM=','VsKycsOQLw==','wqnDpV8Owo4=','w4JmGsOnw63DvDTCjcOo','w6XCiR0XMA==','w77CkBwwDjHDiA==','NMKmw4lGwqc0','RnvCmcKHwpM=','wo8wwofDrMKtO34=','wqLCscOOw5lXSBs=','dsKlTcKGCQ==','dsOta8K1TsOTOTvCpsKkeMOwG3U=','wq3DjEUgwobDuy0=','OcKCw4LDtMKz','XcKQVsKTJMORwqYlA8O/UzDCiCM=','ccOrMMObwpfDjcKr','Li5EwonDpA==','w5/CjS81fmBU','wrLCvsOZw51A','wp/DkMO/woIPTHQEAcOoFMK9','SSgZVMKiDsOu','wosZwp3DgsKl','w51IAMOFw5w=','Y8OJw5h2w5tsFi7CgcKx','wpzDqj7CnCk=','MVnDqEoqw6tq','a8K7TcOwOcKIw7g=','wqvCvcOTw5FhRSIjw4I=','IAbClzjCmVA=','EzPCmj3ChUdy','SsK2DAPDhhA=','wqvCsTkeEQ==','wpQxI8O+MmvDmQ==','LiUXUcOT','wonCg8O9w7h+','VsOFw6VqfA==','wonDsw7CnRofw4A=','NgzCmTHCr0FhwoN2','wpnDnMO1wpU9Vng1Hw==','wq9qw57Di0w=','wr8EN8OrBVk=','WMOOw5h6w71sPCbCm8Kg','w5TCshwFHjnDnw==','ccKVw4HDmsKKLMOgHXE=','aRgedMKNwrtFBMKp','woc7wp/DkcKqM2g=','UsKmHCzDoQ==','wrbDmcO7wpsawpoD','wqDDqXTDicOM','wrNWw7DDpk0=','WnbCtsKRwp8=','U1nCrMKRwok=','w47DkMOZCTQ=','w7fDuR/DqHQ=','w5zDhRLDt15bP0PDm3E=','RMKFQ8OfFcOIw7J9','fB/DhSDDiF42wpMiwq3CqAZi','wo3DvEpswr0=','wpsewrXDgBU=','wrHCjMOow5Iy','wosGwrjDmyLCsMKcwrfDg0U=','OF7DiFsL','wo7DncO4woIrSw==','T8KWasOCEQ==','NMKIw5LDhMK9KsOpHQ3Dtw==','wovDh8O2wp0pV30zNMOiHsKs','w5LDnRjDu2I=','w5dBPcOww7k=','w5pOwpXCgMOF','wozDoF9PwrU=','w7DDgx/DrHNRGEnDqGA=','wrLDssOlw6J5cll1w4w4','JsKxw55fwqoLw4XCkRchwqlE','ElnDsWE0','bG/CrsKhwrU=','H292w5dO','GHsBJnc=','KysJRMOm','wprDrF4kwqU=','AwcCV8Oi','wonCnwwOBQ==','ASvCuC/DkR4=','wq8wwofDrMKtO07DmlLClQ==','a8KsTcOxOcKFw47DgcOl','HcOhZSLDnSAyZh/DlFQIfEFK','56C85a6+5YWN5pSh5riz5oqS5pex5b6F5bW35p6x5a+s5qO/','55u75b695rmh5omf','w5HDp8OJEhY=','5rGo5YWk6LW/5Y6n','a2Q8w50=','6YGJ5YaR5rid5omX','QcOKw59nS1k=','5pWm5oyj6ZW26K+e77+A5Lur5a+F5qCQ5aeM6Leu776w','woPDmw7CkDw=','wrnDjcOPw4JP','YyQLQ8Kd','GQbCkRbClg==','wqfCvDARIg==','wozDmFEDwqY=','A1XCrhvCsg==','HzQEdcOn','wojCn8Ogw74v','ZMOQw5h6w6c=','wroww7bCtjA=','wobDoMOpw44B','EzM5bcOS','LULDvkQy','w65VLsOLw64=','wqXCljsaMg==','wqoEw6bCkhY=','wopIw6LDgUo=','Q8O6Ay/Clg==','wr3CjisMGg==','wqM5wojDi8Kv','wqnCugoFEQ==','wr/CrC8UBA==','ScOrw753Xw==','w6xDwrLCjMOD','cigMXsKoH8OeG8KcwrnDgg==','w6ofWcKpZA==','wo7DncO4woIpUHgkNsO5','XcOxMCfCoDc=','GsKow6HDksK1','wqcAwrXDgA/CusK7wr3DsFQ=','e8K+DgvDrg4xCw==','wp7DgcO4woItXnEk','wo/Dt00dwos=','V8K2U8O8MsKDw6LDisOyZA==','w7LCoQY0PjrDnG4=','Axgzdg==','wqHCvcOMw79dSCUmw5XCow==','JcKpw6bDvsKY','BwcMeMOR','QcONw5RgYQ==','L0XDt00P','BjFYwpbDpg==','wpNRw5XDvlo=','wqgMNcOJDQ==','b2/Ct8KKwqvDvcO4XEDDtw==','RGTCuMKMwqHDvcO4W1PDoGE=','wpPDisOyw4dN','J8OAbmnDj3rCmGsnw7M=','TMO5NR3CgDFoe0LDjA==','IhstUcOM','wpjDr8Orw7N7','wpvDtBjCuxAcw7XClgty','Ly5XGRc=','WyQde8Kl','JAYFVcOaw7p/w4l8w5g=','FMKEw79Zwq0=','WsO1w7BWZg==','dCA8wo3DmQ==','QMKDVsKJKg==','wroTPMOxAA==','IsOaYHbDj33CnXwlw6g+wpQ=','c8KAfsOhLQ==','wqkDwozDpSk=','wpgnOcOlLw==','w7RfAMOVw40=','TiwVQMKA','wo0zI8O6CQ==','w4tpwpTCpcOk','dsOEw5BQw7xkEijCh8Kh','wrbDo8OIw5dD','wovDl0Yqwqk=','DCrCsgvDihvCq1x+wqU=','JsKGw4Nzwrk=','GsKJw4HDvMKX','Q8Onw7p4w5c=','Kn8hE3Y=','M8OxfHPDnA==','wqgJLsOtCGHDj8OuW8Kt','OMKSw7VHwp4=','D8Ktw51bwocGw6fCjCYr','NkFGw75vw7jCjMOuw6TDisOH','w5hIBcORw5Y=','w7jCuR0lZw==','w5AsQ8KSeg==','JkcZJnkiw43Cuxglw4DDlQ==','PMK4w7LDgsKP','AiPCuhzDiw==','w58WXMKWQw==','AVDCjSrCjQ==','b8KBTMO9DA==','FMOnW33Duw==','TsOVIQbCmw==','woR7w5jDo00=','wqPDt1tKwqg=','wpg9wofDqcKq','GMKOw5/Dn8KQIMOOFz7Dpg==','w4XCpRM+EzHDuWlawrPDpg==','w7DDgMO8HzM=','AkVXw7Bm','Pi/CoC3DtgPCpV51wqRS','wokKwr0=','EFQEKlc=','woLCs8OKw5lhUyAsw4LCq8OR','Y8K0FQ3DqA==','w4DCpBQ=','wpXDp0vDo8Ou','wobDjsOdw5xW','VcOEIhvCqw==','wpHCp8OVw48h','wq0ww6rCtTg7HcK4YcKqw6nCqA==','K203P0s=','wr7Dt8Oow4RU','PsKRw6TDgMKk','O8KFw53DkcKKLQ==','w4/ChjU5fnFkEwzCtw==','UzkjwrjDm8KPw50Mw5bCuQ==','VcOidMKoZcOAHzrCoMKP','wrfDlk9xwpzDsW8Q','C8OGY3LDonDCv2EUw6I=','wqzDg8OtwpoHwrYKQAA=','woQsw6nCsRU2P8KlUMKg','wpMqwpnDmsKwMmTDlkU=','wrgNJsO8Lg==','wp7DmcOwwpMP','W8KzEQ3Dgg==','B2hRw5B/','w6NDAsODw44=','IUvCrALClsOxw7N8wqbDlw==','QMK6ADHDkw8IBsKRw58=','GsOhZyLDnyA3Zh3DlFEIeUFK','ecKJTsKyBw==','U3HCt8KKwrE=','wprCksOPw6kA','w5/DmxjDr0c=','w7fCrBQ1EA==','wpI/woXDocKsM0TDm1Q=','X8OtKcOkwqw=','w5RswobCicOv','bMOkMsOWwpbDhcKRd8Ki','wrvDuAnCrio=','wqQcwpvDncK2','w4/DqcOAHxfCogwVw4Y=','QcKrFi/Dkg==','WMKFS8KmFA==','w7BLIsOGw4k=','EcKkw6fDssKN','wqYawr/DgcKw','KznCgRbCmw==','wp3CkcO1w74XwrfChUI=','LVvDrUAh','w7HDkQfDpmYYbMOowqfDvg==','wonDun0uwp4=','w63DhyzDkXQ=','PhbChSbCpg==','ZSEGUsKp','TmI9w4PDvQJ3AFJN','wq8ELMOwL2vDqMOje8KrwpY=','wrtCw4HDrGY=','wowjw6jCvTUyEcKv','wpDDmcOzwpcMwrAmSxc1','RiwdUMKhH8OpFsKPwrg=','5rig5ouX5ZCy5a+C','SMK5TcO0MQ==','wrDDicOmw4Y5UcOCCMKH','woQPI8O2JWvDqMOkaMK8','wr3DlMOrwpEHWmgkBcO+','5p2a5YmB5Zub5ZK7','PkTCsgrClQ==','acK8DAfDiA4ZBcKVw5Q=','wo3DjEUgwobDux1JbsKd','fcO8Iz/Cgjlxf17Dmw==','5ZKM5Y+w56up5bmi','HALChD3CkQ==','wq8PwrTDjDfCusKKwqHDq17Dhw==','w47Dkh/DmH8SR8O9wr3DvQ==','5rqm5oq854il5p6V','FQMTd8OY','WcOkMcOXwrXDgcK2cg==','B8KBw4HDl8KTIMO5HT7DsA==','5LiV6L+45Z2s5Z+y','wr3DlMOrwpEH','w5VpwozCicOnw43CoA==','w5HDnQHDkHwScMO3wr3Dqw==','woHCs8OOw51fQzcqw5XCvQ==','56uX5Y+W5Ly757yl','woTDj1pDwpU=','w6FmGMOmw5XDoh/CicOhw6srT8KA','wrgPwqvDiAzCusKMwrfDsEI=','56u55Y+G6YOP5pml5bme','w5DCiSsxfQ==','cG7Cq8KWwrXDmsOcVw==','byM9wo7DkcKPw70Mw4DCuQ==','5b6R56uY6IOj5pqR','TsOkLsOTwpQ=','w67DqcOYHivCuiYYw5fCuk8=','AkVXw7Bmw7jCu8Ojw7fDiw==','5ayd5qCB6YG755+z','wrXDkkjDkMOG','wqcAwrXDkDfCvsKUwqfDpw==','wp7CvMOQw5VcQwAgw5XCqw==','w63DqcOcGhXCqjEew4DCrA==','57u35Luf5Y6g6Yah','w63DqcOcGhU=','woQHKcOTImDDjg==','wqrDnVbDmMOFWH9wL8Oq','PS/CpCnDiBPCslhiwrI=','5o2V5p+a5Y2r6YW5','wrnCn8OUw6sI','JCDCpj3DkTXCqVF/wrRE','wpLDg1sowoXDuypDbsKL','6L2z5YS75bm+6Iu3','PS/CpCnDiA==','wrsNwrbDmwTCicKZwr7Dt1Q=','f8ORw4tnRFN1wr9Jw7w=','DlbDtkIpw6ttFDhD','5o2Q6KOu5Y+x6YaG','FMOJfXrDoQ==','TMOkMsOZwq/DicK0bMKz','w7lhGsOrw5TDsBPClsOyw60=','w7HCoQIwGjHDjmRJwrI=','5o+j5ZGN5Y+z6YSy','w6VsGcOww5/Dlj/CncOl','6K2p562K5YSw5b2w','DV/Dq1QQw7dpFA==','NgEoY8OOZTBNwovCoA==','5ZCN5bGD5qmw5b6l','Jktpw758w7jCvcOFw6TDi8OR','Y8OSw4F2','YMOew5VvRw==','w5fDgg3DrlxfBW3DimAuwpU=','w67Crhw4GTHDuW5JwqQ=','YcKQV8KGDMOCwrcuBcOC','5ZCs5bCC5Lur54qO','wo/DlsOtwp8P','CBsiecOZw7hnw5o=','wofCszAWDU9yw7hbw5E=','UW0jw4vDvgJAClJb','5bSU5pSK6aKm6IuD','BgweecKU','wqPDmkjDgsOfc11yOA==','WSMDWMKiH8OeHMKPwq4=','w4PDigzDv11bCEnDm2c=','56ai5Y+T5Y+96YWe','woHDu8O2w7FX','GQdEISLCkEM=','wpvDgERLwpbDkkQ7w4bCiw==','GQpXOSrCh0TDvHPCjA==','5pmP56W65Lih6Zqt','eMK+Cg/Dig==','w6chQ8KXfcK+wrvClABn','FMOJfXrDoXDCiGsUw7Q=','5Z+j57u95pWz6Zan','wpPDr8Oiw7Zfb2Q=','BgweecKUwrdcBMOmw7k=','5pmT5ZGD6aiO5buu','w7HCoQIwGg==','EFbDqUYXw6djFA==','5LqS5ZOS5ayv5Y6Z','w5DCiSsxfXFTGQzCoQ==','wrA/wpnDpMKuO3nDkFLCgw==','6IKe5aaH56q/esKh','wpbDg0Uiwr/DtzpSdA==','wrXDvhDCkREUw5vCmBdz','wrXDkkjDkMOGWEh6L8O8','6IG75aeY56qy5a2a','wq7DsRDCkzABw7nClAxiGg==','Aw3CmjXCkkdDwoBhwrQ=','cGDCqcKCwqjDvcOPVkDDoQ==','6IG65aaW5pqp5pqZ','wrPDjcOiw4Im','SsK5UcO+HsKBw4U=','SMK5TcO0McKDw5XDgMOycg==','5o+S6KKP6IG95pmd','w5F5wpHCmcOtw5fCrXVbwpY=','57K757ic6K2F6K+v','cCwjwobDkg==','wpbDg0Uiwq7DsTBS','6IC75aWp5a285YyK','w5JhwpDCjMOl','wpgJIMOoH2/Dh8Og','w6ZuBMOjw5fDsCTCnMOyw7s=','5pu156W46IOP5aex','GSHCmifDkhPCtH5xwrJT','wr8TOsO6','EMKiw4NTwoQ=','woYEIcOqGW/DhcOg','UcOrMMObwpfDjcKbdsKkw7E=','6I6x5Yyb5o2E6KCE','w5LCiTc7WWBCEQ==','5oyS6KKk5Yec5a+p','TMOkMsOZwrbDmMK5esK/w6BU','5o+06KCs6YGV5piM5biF','wpXDh0s6woHDqjsUXcKoaA==','BcO4ROaJiOWOiQ==','wrzCshAQFE9Dw5RIw4fDtg==','Yj8aVA==','wpPDmMO+wpohwpAsQA==','Pi5HwpTDr3lOw4Qqew==','wqrDsQ7CmRIUw6zCkhdl','5YSg5YSS5Lm95Lqq','worDhMO6wowswrQIQQ==','BgVJMSnCh3PDtnPCmg==','wr3DlMOqwoMdUG4l','PUTCtg7CscO6w5Z8','YsOzPTfCgTlGdV7DjQ==','GVbDqUYNw6B/Hg==','wpDCvsOQw6xeRzoqw5XCvQ==','woYjw7XCiBcyBcKvUMK2','wqLDm8O1wpkEWl8uBcOo','wp4yw6HCuQ82KMKrTsKu','fsKfScKOD8OCwoAkBcOU','fMK+FAXDtAktAQ==','wrQ/wofDrsKXO3XDgQ==','wqzDgsO8w4olesOgCsKQw7s=','w53DjgnDi0NbDg==','w5BhwozChsOhw5TCng==','wo3DlsOxwpUywrQCQQ==','OQMAYcK6wr1MBA==','OMKGw5XDmsKXK8Oo','E10ZPGomw43CsD44','Hk1Lw7o=','w4nCtAQhBG7ClS4=','DlbDtkIp','FgcXc8OHw5lzw4V3','wq7CgjE9dXpAUh3CvA9rNSPCkBJxwo7DnlnCoEPDiyTDocKuCMK9','Pi5HwpTDr3k=','csO4bMKxeMKfc3o=','NQUsZ8ORTiVFwpw=','XypDwpDDpHJqwoU7cMOBwrXDiMORwqdd','GQpXOSo=','w4HCiy05f3ppHRPCtw==','OcOQw5xj','BUMTJU4Gw4XCui8=','5oia5oqw5ZGZ','5py05oyI6KOI','5omF5Yms5Lq7','5oyL5oqU5LuO','5oi+5oqV5ZKq','6YKM5Lu46ZWr5Luy5aSG772f5o2t5ouS6IOf5o2M5ayQ77+M','5p6I5o+c6KKa','FH/DiXRqw4F3HSNeZsO2EGfDgTDDrE/DtcOEYldrworCiAPDosOoXD4tY8KKw7F4w7EYwrJBw4DDnnpTH8OswrfCn3jDoUQ4wrFWwpN8BkgKFjAR','5oqN5Yqy5Lq6','6KyM5Ymh6IGb6L2S5Z2N5b+e5Yy65LiIKsOPLeaUreiuieafteW8ke++mA==','5o+05oqr5Lus','6K2l5Ym36IKk6L6I5Zy45byU5Y+25LmRUMO8DuaVoOiuo+adkeW9sO+8vA==','ZTs0wonDi8Kmw5cQw4Y=','w5guXcKffsK+wpvCmB9x','5a6U5pe35b6F5YiF5oy36KC656+qYUbCmg==','f8O8PzXCuT1pb0k=','wp/lkq/vvKU=','IyFFwpbDiGhow4Y=','77y6XcKNwrg=','w7guXcKffg==','DVTDq1Ehw5h4HT9V','eiIIXsK5Dg==','cklGNw02wprCtWp7w5jCgWcvEFtwwpjDn3ccccKiOwxsd8Ok','NMKiw51ZwoUKw5fClw==','UcORKMOfwrI=','HUpJw7hlw7jCjMOpw7fDnQ==','woXCs8OQw5dmQzs7','wofDj15HwrHDmWE7','w43DusOLCxnCvSA1w5fCp1/Cs8K0PivChA==','Z8OVw4d7','FhlANSjClFXDkG/Cj8OLCQ==','wptvw7zDlXw=','Z2DCtsKGwozDtsOdXA==','wrPDjcOjw5A8cMORAQ==','Z8KxFAfDiQUUC8KKw5Q=','esK+FgXDjg4w','wrzDncOcw4J2','ZSgbeMKiDsO4AcKLwqrDnQ==','TG0hw7rDvwZNClJb','FsKMw5/DpsKSJMO0HT7DsA==','w6chQ8KXfcK+wqzCkh9n','wp/Ct8OLw6lBQzE=','VsKUUcKqIMOk','w7tywr/DqEtpTMO6GMO1wr3Dt2JJQcKNwoE=','QXIbw43CsSw9wptoLsOew6I=','GDNlworDq28=','wpHCvjMwDA==','woYLwq3Dng7CrcKTwpvDrEXDjMOyw5N6XsOOw7U=','V1nCrMKtwpA=','IQLClQ==','ccOsEcO8wpw=','OsKBw5A=','w4RqBsOuw5vDtjU=','wpkgw4bCmTo=','wrDCuMOdw4Q=','ccO7YMK7fA==','I2PChgHCvQ==','6L+u5oya5Lqq5Yi+5pyn5YqX5Zme77+c6K+M5qGZ5p2W57+B57ia6Yep5pak55m45b2344Ko','DDTCgA7Dlg==','RgI8ZQ==','YyI/wpPDmsKEw4pOw6bCpcOCOg==','w7/Dgx3DlXsUUsOswqbDoXzDt3DCtcOlUcKZd1zDk03Cj8OFwpTCmmYQwqkYJmPCpRRYw4vDtMOPb8Odw5bDmU4Uw7pfw6Mz','w6/CmDw+','eMOxCcORwp0=','JMKFw4fDpMKbNMO4HT/Dt1vCgAAqIcOQ','wq3DsTPCrRk=','bVkbw7/Dgw==','wo8wwofDqsKiOg==','wo8IwqHDoMKa','dcOhQMKAeQ==','cngww57DphQ=','w6vCihcIPw==','wqrCpMO3w60f','w7V5woTCjMOk','IiDCuiHDixPChVJiwqQ=','AgwAc8KtwrdQFQ==','bsOwL8Oa','V8O2E8Ocwr0=','XsOTw7t9w5c=','DBEueMOx','woLCt8OSw5hmQzs7','d8OKJMOXwp4=','PgbChSzCk0xzwoo=','LFoRJE8+','wq4IwpHDk8KC','wobCkMODw7gXwrHCmQ==','w6IeVsKLZQ==','R2gGw6jDtA==','woTDksOKw7Ft','eSYjYMKH','QsO2HQ/CpA==','woPDqWzDt8Om','NQUNasK6wr1MBMOVw74=','w57DhsOFFwg=','KAbClTPCmEdDwodywqPDqQ==','dMK3WMO6KcKS','PxZRwqvDgA==','Og5LPA==','CQIwXcOXbxtYwpHCow==','AEFDw6Nuw67Cp8OCw6TDjMOV','w7p1w7nDrQdiSMK2E8O0w7HDtC5ITcOBwoDDssO2worCtMO4Bh3CrsOhWyQ2wqLDuwwUw5RlZkFyFMKMw67CqTvDo8Owwp/DmsKiw7/DlMKxw6Y=','wpoLwr8=','wpI7wo3Dt8KmLWU=','w5TDlhfDtmM=','w5s/Q8KXZw==','wqTDqMOow4RTcFg=','wqRNw5/DpGV+','wqIxw4HCvRkmG8KvRg==','woXDkMOhwq8eUEMjHsOj','w5JmBMOnw5nDoTnClsOu','CCZU','wq3Cj8Oxw6sW','ADI2Q8Oc','NQHCrC7Dpg==','w7nCjwo3NA==','w5zDuS/DikY=','w7F1woDCnsO8w4g=','wrg3w6fCqw8h','wqfCszAGIEVVw7I=','wrgPwqvDiAw=','W8OvbMKoZMOLEjTCv8KP','WMKCYcKCA8OSwqQuEw==','wqZnwoY=','w7F0wpDChMOmw53CkHJO','Mi3CuSzDgAU=','PQ1DKC4=','wpgCIMOtLk3DhMOvfw==','wp7CvMOQw5VcQxcmw4rCqw==','KQcXc8OZ','NwcNYA==','MsKmw5BWwq4Cw4nChg==','wqfDhcOjw5MnfsOaJMKBw6oTTA==','wokNwq3DhhM=','wq3Cl8OVw7oJwr/CkmRnwpVsMw==','woHDkMO4wpQPTQ==','dsOWw5V9XnhXwr1e','w4BuGsO3w58=','ZmjCqcKQwrHDlsOaXlc=','wr0jw6nCrR4=','BsKqw4NBwp0tw4XCjjE=','wrfDhklQwpnDlHMxw4bCpw1lERI=','wrLCusOdw45TRTcqw5XCgMOUUcOm','fcKUZcOjHw==','CQjCkz/CmQ==','woTCn8OWw4MB','bG0hw6PDtw==','w63DhwzDi2Y6UsOowobDqg==','woXCmWk=','UW0iw5nDpAhGCw==','VG7Cl8KMwrLDvcOJcFPDoXc=','WsK6GQrDoAE6AQ==','w78df8OjenLCmcO3KQ==','6LSG5Y+25L6H5b+p6KGu5bKZ77636K+E6IGm57Gp5paD56qf5qOI6K6U6KWO5bGf44Ol','5p2b5YuM5ZiE57us5omd5Lu/77yk6K6W5Yen5rGi5Ya35ZCl56us5ZKl5YSo55qM44Gi','5L6055iq6LSO5Y+V5aei5aaz55m85byB77646KyZ5qKj5p+k5pmb5ZGM6KOk55mM5Y2C44Kz','5ruV5oq654uR5p+c5baY57uw5Y+E5pq177276K265pqO5pW55rip5oi05Yeu55io5b+l44KW','5L+z55iv6LWY5Y2p5a+256C/5Y6k5puY776l6K+j5qKB5pyC5pij5ZOO6KK855iJ5Y6944KH','5ruw5ouS6YGG6K+Q5YaZ546a5byz5bqv77yl6K2D6YWe5pWg5ZKs5Yum5ruS5omB44O2','6LyK5oyL5Lil5YiS5p+15YiW5Zmv772Y6K2D5qCu5p6G572Q57qo6Yan6K6a44KZ','PCBHNgA=','PxPCmjXCiA==','w47DvMOcJBzCqiYUw5bCug==','U3XCqcK8wrbDtMOSUFc=','HVXCrALCjA==','wq7Cn8OLw68swrDCjUo=','wqzDh8OzwpcW','wqvCvMORw4cX','EMKBw57Dk8K3K8OrFw==','L1MQJ1Mkw4k=','wpoLwrjDjS/CusKPwofDsVTDmw==','KQ42a8ONZQdHwovCtg==','w6RqEMOww5/DpjjCqcOsw6k7XsKL','wobDu8ONwo0u','AgMMc8O8w7l0w4c=','bWM2w4XDphM=','IcKuw7tiwrM=','KAx5wo7DjQ==','Zyw8woLDtsKEw5gM','MljDo0wxw7o=','GwsufsKB','wrEiwovDmi0=','wqc/wobDoMKKMGvDmg==','RMKwHwHDkhQ=','w7nCkxQUGQ==','w45XGcOmw50=','w41uwo7ChMOmw5/CuntFwpg=','UMOBw5l2w5pnGSY=','ASHCsSfDkAI=','w7nDsifDt1E=','cMKdfcKrIA==','wpbDvxvClwoF','wrvDtMOKwpwi','w4QgSMKRZsKv','LRbCuxbCtg==','CQY8bsOKbiE=','bsKgTcOXNg==','AijCsCTDjBjCow==','wp7DtMOow7lUeH5/w78p','wrPDmMO4wpEXwqE=','wqXDk2QkwqQ=','QsOaw4ZqZFNBwoVIw7zDgQ==','J8OHYX3DpWfCkQ==','wrzDlsOxwp0Hwrk=','w7jDuCbDrEE=','HDxHFBE=','wr7DpsOxwpE/','wowAIsO6AmDDjcOk','wqM3F8OaLA==','wqDDiDHCkTg=','ehUcwo7DuA==','w7/DlwnDun0aXsO5wqHDqg==','wpvCq8OMw459','QWXCv8KgwqrDtcOWUlzDtg==','NHYfBn4=','wqjDgsO9wpkG','wqxCw57DqEJ1wr5N','ZMOVw4dVw7Q=','wrzCuzI1Nw==','dWo/w6DDhw==','wpbDu8Opw7Vzc1t/','PSp6wqXDmA==','WWUSw6vDqg==','wqkIwqrDh8Kr','fsOTw7t9w79gESw=','w6YqWMKrYMK+wp0=','wojDg8O8wp4e','woUowo7Dq8K3','J8KBw5TDk8KN','HcOlRknDlQ==','woQLwrfDjhXCtw==','wofDtcOYwr8j','wo7DicO1w5c4XMOMC8KGw7cIV8OaUnI=','woPCt8Oaw45XVSsfw4vCr8OMWcOx','wr3DtcOlw7RdfFB1','XsOaw5BpS1tT','w6wtYMKtZg==','5a655L6j6K2F77yo','wrvDvBDCqBMQw6HCkhdl','GQMAccKXwrdrDsOmw68=','wrbDnMO0w4I/esO3BMKOw7U=','w44jQMKRYQ==','Tig7wrPDnA==','DErCuCPCncO9w5d7wqA=','wp8AI8O0DWHDhcO/','BXIFKVQ=','OS/CuiPDsRPCvkk=','LMKmw59Vwp0L','EMOJY3DDmHDChHo=','ZMOQw5h6w7Bs','wpo0JMOrKA==','SsOkMMOZwq3DjcKgbQ==','wonDllTDlsOfVQ==','W8OCL8OQwpc=','w5tZDMOUw5g=','IQE3Z8OqbiJH','WcOkMcOXwrDDhsK+dg==','w7LDgi7DvV4=','w5PCqSYfBw==','EVnDqEoqw6taHjhV','USwCVMKFFMO7HA==','IwnChy/Cvw==','wpNkw53DpFg=','FUVIw7RCw7PCqcOp','wrDDncOuwo0h','Hic0a8Ow','Gg5YwrHDrw==','CwMyQMO5','UMKYFgfDtA==','NXnDt28q','w5nDkgDDnFsZVcO3','VsK5bMODEA==','w7pHwozChMOb','RsOTIhLCgQ==','CsOJXE3DgA==','L8OmfFfDog==','w4fCiTQ1WXpBEw==','X8KuEyPDhg==','woLDlxLCkSw=','w7XDvR7DtXw=','NiFGwpjDiHJrw4Q=','eDcaa8KH','w59+F8OPw6g=','wpvCtTMIN1NBw7I=','NgEoY8OO','wqnDuBPCjysIw6jCkg==','IxY/bMOXTC1bwo0=','GHZhw7pH','wqpPw5/DnWd6wqFHwoLCgA==','wqTDm1tK','KUTCrQ7CscO6w5Z8','OBcZQsKy','ecKSTcOFGQ==','dsOHw4x7w7Q=','wrHDvkBuwrw=','wq4Sw63ClD8=','wpHDm1o9wo3DswpHcMKT','Rm08w4/DmglSAA==','B2dMw7dZ','w7/Du8OXFx4=','Ki/Cuy3DrBjCoFI=','S8OGNcOUwqs=','wqlQw4rDoW0=','wqkCwpXDoxI=','wpgCwpHDrCg=','HknCiC7CsQ==','wqLDklfDlMOiU1pw','w6NMH8Okw6g=','wrjDowXClBk=','DlcmAn0=','w5TDihPDu3lQGkM=','U8O1f8KyWg==','wokxw7zCtB0=','EBEEZMO7w7Z/w40=','woIYKMOsGg==','eMO0w5Bew4Y=','VMODesKNYw==','wobCqsOCw4cw','XcK1Cx/DiA==','V8O2w5NhXA==','w5fDiRnDuH8=','wqLDvsOCw5lw','wp5Jw4DDvGQ=','d8K8FwPDigE5AMKvw5jCqsK2ecOd','JMKFw4fDvsKfK8OpFCnDsQ==','BSQtTMO6','WiIOVcKLG8OwFg==','JwsPcg==','w6lsGcOvw5fDtD7CncOXw6EsX8KWew==','w47DrcOaMxnCoSEXw5fCrQ==','wpIzw5PCgi8=','VsK9SMOSPcKLw4Q=','wrbDh0ZG','QiwDWsKYH8OlBw==','w5XDhyzDlmI=','wq7DsRDCkysUw6DCgw==','XMOaw4lpXl4=','fMKYZ8KvGQ==','wqRNw5/DpGV+wptNwoLClg==','wpbDg0UiwrzDuyZS','NRcSfg==','c2DCrcKGwpLDvcOZ','w6fDihLDtVxXD1g=','wo7CrTM+LQ==','5paZ5o6Q6L2h5aat772p5LmP5a6z5qGG5aS16LeE77yC','w63Dkhs=','wprCn8OQw68=','TyMTwoLDmcKFw4wGw6HCvcOEOg==','wqfDmlpLwpbDkG4yw40=','IQLCnTnCr0N2wopQwr7DtA4yYcOiw5w=','w7DCoTMifA==','wr3Dv8Oqw7dOdQ==','ZMOVw69cRA==','w4fCtQEHBg==','Lx5UDjY=','wrDDmDHCryA8w73ChBZ3BEQ=','BwQ+','w6dPwqDCisO/','YGg1','N2tnw7Z8','wrrDgV/DnMOES1lWM8O/FQY=','w7HDoTHDv2o=','w7DDhBDDqlVQCF8=','wqTDj0FMwozDuHc1w5fChxd4','ccO1PcORwpDDnMKh','wpfDi3tHwrI=','wobDnFfDgcOZWE9sCcOgIhPCtsKWwrUC','C0TDoVEKw690FA==','wq3DjEUwwqvDsTpD','w43CrGw=','woHDu8O3w6NNck90','asOqEMOdwo7DjcKqWsK3w6dI','w4E8a8KbccKuwojCmAk=','V8OILQ==','w47DvMOcEhbCqCwdw4s=','wpQCIMO7Ln0=','Wwla','SGbClMKnwqs=','IgonYMO3','ZmQXw5zDkQ==','worDncOfwoYo','CANNwrLDqg==','a8KmVsKDIg==','wp7DgMO7woMeTQ==','a8KtXcOmKMKU','KUYyLlg/w4vCrD8=','JAbCjgPCiE1fwo16wr8=','wogbw43Clh4=','VT89wrPDlsKHw5s=','ZMK2FgU=','Bwo7eg==','wrkELsO7GG/DncOu','FAU7ZsOwYTJN','5paT5o+C6ZaJ6K2M772H5LqO5a+75qCG5aeT6La77729','UsKQcsOCA8Krw4TDlsOzYMO/woI=','dyAKW8K/','FgMXc8Omw6Jxw4t3w5nDnQ==','f8OhOA==','ISsseMO7','wqnCuTg=','woU9wrbDmAQ=','QwcjwrLDtw==','R8OBw4Zyw74=','BQwafcKqwqdLAsOxw6/Cvw==','UcObw4M=','E8Kiw4dXwroWw4fCgDErwqk=','w68EWcKESw==','wojCmsOC','TVLCtMKSwqA=','wpgnw6vCvC82BMK+','IjFfTcO9SsOhQcKBw7rDjXBuw4tbwoHDtcOMw7fCkDHDrU53bcODwonCjA==','w6fDihI=','w7TCiTU7','wpzDrsO9w6Jz','N8OYY3LDuA==','QsOTw5Fhw51oEiw=','RiwdUMKh','wqjCncOSw6MKwrDCpURpwoQ=','OXbDg2sQ','w7TCsxUjOTXDl2Q=','B1bChA7CmsOhw5d2wrA=','I2HDtQ==','w5s7XcKXfcK8wobCmxQ=','wo7CscOTw5hXVQ==','wo8Mwr/DgyQ=','worDnVbDiMOoUlh6','fz4rVMKuD8O6FsKZ','wqbCkMOKw6MLwrvCqEp2woQ=','figXbsK4FcOCEcKUwqU=','PBlJDC7Cj1U=','bGjCtcKI','CcOsOg==','wqXDhsOKw6ka','NBnCsw7Drg==','JBIUScOt','YnPClcKowos=','wojDvsOXw71K','PWBqw7Vl','HVDCohjCjMOm','Oh5HKzPCkA==','QMKiRMKhBw==','woE0worDvQ==','wqjDtw8=','HALChS/Ci01ywos=','wp9Mw7/Donx+wqphwpHCgEs=','dsOTw59Xw7J9Hg==','MMKFw4fDgsKXKMOo','ZcOMw4J8ZFdbwrU=','ccOcw5NnRVh4wrFWw7w=','WcOkQsK5Yw==','MBANQsOcw7p3','bCQ/wow=','woTDmVvDiQ==','aMOpecKlT8OEKDA=','w7LCjTg0VHVTGQ==','BnTCqcKPwrHDscOWVg8=','cxbChDDCiEttwoou','XcOIw59yw4M=','wo8SwpvDqMK2','wqfDmlp9wpzDkmQ7w5DCiw==','wpDDmMOiw7w4c8OKBsKH','LmYmPHU=','woPDsjnCnwg=','PQTCogzCmA==','R8OPw5FGw7w=','w4VqAsOUw5vDuSXCnA==','wpvDpcO0wpYq','VzsHwp/DsA==','CcOda1rDtg==','RMOuCS/CvA==','w48qW8KqesK2woo=','wovDoMOsw7NC','w4EhS8Kba8KUwok=','WMK7X8KJCQ==','wpfDgnHDtcOc','SsO+asKCTQ==','wqHDm8OTwqYn','woQ9wr3DrCc=','Nx4HXMKYwqZN','MQsMcw==','Ax4JasK3wrNFBA==','wqLDj8Okw4okccOtBMKPw7s=','w4nDnQzDklw=','w509Q8KqesK2woo=','DMKqw59Z','KAFEIA==','c2kww47Dhw5ZCg==','UmTCusKHwpHDscOWVg==','cngjw7XDtwJXAERN','wqfDmlp9wovDm243w5E=','Uz09wo7Diw==','bcOgKMOkwpjDhMKtfA==','BSDChR7Dsg==','Y8KUVsKCFcOjwqI/Fg==','DiLCuTvDgA==','w7zDq8OaEhfCoQsaw5/Cug==','wpTCq8OMw6x9','a8O3MMOmwpDDhcK9','BQJLMw==','CgwNf8Obw7JRw4dgw5k=','w4klTsKG','Cg89bcOWdA==','w4HCmzICcXpM','FH3Dq8Kfw7PDpMKOTwHDriMaOg==','EDNAwo/DoHJm','wpvDoxc=','fsK6VcOXPQ==','wqfDnkRLwow=','wpsjw7fCuRY=','Fw4YccKWwrxmAMO5w68=','NUcaH1Mnw4k=','w7LDpsOCEhbCqgYUw4DCug==','PgbClzjCrkNuwoQ=','GiEabsKU','AsKTw5bDhMKwJMOgHQ==','wpLDs8Kq','woJnw5jDi2A=','NsK4w5XDj8K1','wo4hw4TCojY=','wooSwpnDo8Kb','w4EDbsK7VA==','cnkzw5nDpxU=','L8Ktw51LwqoMw4DChg==','JAgNfMKrwrNGCg==','W8KrCjHDgwU0C8Kcw5Q=','QsKFV8K4EsOLwqooEg==','PwbCggrCnU51woo=','DFbDqkgSw691BC8=','Y8K9Mj/Dig==','w7PCjS0GcXhSGQ==','FgEOZMOQw4Fzw4Rnw5k=','wqHCjMOLw4E1','cMKCTsKxAMOLwrYu','dMOUO2fCvmnDiXJRw7trwo1MEAzDrcKzXcKs','d8Otag==','wqnCrjcJAkZEw7I=','ATzCuSHDqw==','LBrCmy7DnQ==','wqw7w7XCqA0=','wrjDjsOvwo4U','MMKZw4PDhsKI','DxLCuS3CiQ==','wprCi8OEw7kRwqw=','wqzDgsO9wo0Wwqc=','wobCkMOKw7MmwrHCj0A=','dsOKw5Vr','w5PCpRE1ITXDlnRe','wqE9wp/DrMKsMEPDlE3ClQ==','U8OBaQ==','w5JhwpHCnsO/w5XCi3A=','Eg8WbcOUZTZrwpjCoMOc','IBhhPSXCl1fDvGU=','WcKUXcK4FcOIwpwpHsOf','wr3DnWxHwprDgmAxw5A=','KyRv','w63Dhx/DkHwQWsO+wrY=','EwDCmTjCmVE=','Ix8ATMKQwr9N','wpPDnsOxwpU=','wpfDg0YHwpE=','CgNAOyzCtFHDtXTCmg==','wqnDj13DhSELAsOkw73DsiTCpDzDpMKqWsOb','ecOnbg==','wqgJKsO8IHjDisOnb8K8','wpjDkMOxw7F5','w4V/GsOrw44=','BAEVf8Oaw7lcw4l/w5k=','DxMeZ8OBdSNNwp0=','VcKcCg==','w5LCtAI4GTPDk2dC','IArDow==','wpDCnjcnKg==','acKyTsK/KA==','D8Kjw5jDrsK3','T8Ojw59Lw5o=','RMO3w4pbfw==','PxbClC/CiFA=','wrgPwqrDmhbCsMKKwrY=','wqXDtcOIw79NeE9Tw6w/wr0=','w5fDowrDrXY=','w4DCqhEp','ZCgOVcKaG8OxBsKY','c8O/XMKkacOQOzDCtg==','GSVTwqLDtXNSw4kxcA==','wrbDn8O1w5EFfsOOAA==','wr3CrzArCkdU','Kgk0aQ==','wrrCuD0bNUtdw6JM','D1saIlQvw6/Cpikv','w4V7BMOdw57DsDPClsOkw60=','M0EEFEkmw4XCqj4=','JQgYTsKYwr5dBA==','S2kcw7rDtg==','wprCuC8aF3xQw7tcw5E=','w7IdfcOjc3LCm8O3L8Klw5FOacKBOHDCjz4S','asK9TMOwKMKQw4DDicO1ZA==','CwEo','w6vDq8OiHBI=','JMKQw5/Dn8KK','wrXDkknDgsOcUk57','NFo6JE0vw57Cijo5w4E=','wpjDscO2w5p0','w4PDigzDv10=','w7dsAsOrw5XDux7CmMOtw60=','wpjDh0ZJ','WsOkwoE=','SUshw6/DoA==','ClFQw5J+','wpM3w7DCmw4=','GcK0w6FIwoI=','wq7Dpmlrwqs=','W8OkV8Kjeg==','bcOwPsOBwo3Dmg==','w57Dr0s=','HjrCpCHDixHCr1tp','csO+PjrCii8=','wqHCrhgaAV9Ww7JN','woALwqHDthXCsMKnwrDDq18=','wr7DncO+woY=','OMKOw5/Dj8K9KsOpHQ==','w708SsKMXcK6woLCmA==','H0VXw7pbw7HCrsO/w6DDig==','w5PDjw/DjcKTQQhjbcOzUw7DtMKPwrFKcg==','wpfDsQ4=','AC/CpCM=','wpnDmBDCtAo=','w7Fwwo7ChMO8','wqcuN8OnCg==','NsOqaH/DnA==','AhUGY8Oz','wr3DpxvCjTk=','wrvDhxjCoBc=','wpsbwrvDmhXCrQ==','EC8EQMOk','VXPCt8K3wqzDtcOe','CMOBYXA=','PsKTw7fDk8KcMMOqHSg=','wpLDtQTCpwsew4fClQx4','RMOuFTvCjSlif0g=','H2AQ','wpbDh0jDmMOFWlV5JA==','MUbCrw/CncOn','QG8lw4PDvAl6Dk1N','dXLCvsKRwovDucOWVg==','w7/DmQzDgQ==','w6zChEU=','wpwBwpXDhhbCusKKwpHDo0LDjA==','wr3CrTgeF091w7Zdw5U=','FjFiwpvCjsKWwosfwoXCoMKAI0gROQ==','wpjDhcO9wpEeWg==','wr4yw6E=','UMK7PybDqw==','wpMuwofDrMK3','ORHCmgjClU9l','w6chQ8KXfcK+','woHCs8OOw51f','CAhRMSjCjH7DuGzCmg==','MDdNwpvDpw==','w4TCmyMeYA==','LjBKwo7DslVjw5stag==','wr0AI8OqLg==','UcOVw4Z2','QsOaw4ZqfFNEwqNSw7bDnQ==','PAQe','W8KrCgfDiQc+AsKB','G8OLYH/DqWY=','dDw+eMKo','MiDCtyXDgD/CqE1lwrU=','wp4PwrXDnAQ=','w49Ew5c=','wqoJwoHDmQM=','w6ood8KOcQ==','PhpCwqvDpg==','GHLCsxPCkQ==','SsO2w7VYeA==','c2kww47DhQJGHElHHA==','5pSd5oyg5YWP5YSt5oq15Yup77+K5Y+S5Lu16YCp572E5buf5Y+Z5bun5rus5ou25Lmc44CO','wpDDmMOiw7wvesOACsKGw7s=','wqLCpsOOw6NBSiosw4I=','w7PCmDU5ZA==','KAYPXcOf','fcO+TcKoXg==','wrvCpsOvw4QC','wp3CvcOdw5hhRzUqw6PCr8OBXQ==','RcOpJS7CnGYqNQ==','YgnCnjHCmUxnw4Fwwr/CtR02YsOzwoA=','MMO1NMOC','wqXCusOQw4Zq','w63Dq8OjPRo=','akvCtRI4wr8tDXsCf8KEAyzDmC/CmEDCqMOqMEUyw57CnVfDv8KyRUMwK8OYwqFywqVG','YsOEPBjCuw==','MnBjw5bCpxsFExA=','wpgnw7PCvQkdHcKnRw==','wqLCt8OKw5lAaCIiw4I=','H1TDsEorw6BXECdV','w4ksW8KXfMK1wqHCnABn','wpbCs8ORw5lkQzE8w47CocOb','wqXDg0Qswr7DuyxVdcKXTQ==','wq8wwofDvMKVP2HDgEU=','wr7DtMOow6lsfFFlw6g=','w7lpEMOOw5PDuzU=','OQsKVMKQwrxN','w6VsGcOww5/DgzHClcO1w60=','wrghw6rCqh4FHcKmV8Kg','GwpLMxHCg1zDrGQ=','QsOew4llfFdawqVe','wqrDnVbDmMOFWA==','B8OHaXLDpA==','QmM3w4PDuw==','w6sgScKXew==','w7TCiw8yfA==','wpTDp3gawpg=','B8KBw4HDl8KT','YsKUU8KCE8OpwqImEg==','wq3Dll5Hwqo=','w6ksW8KXfMK1wqHCnABn','woMnwr7DncKh','VMONEcOlwqbDu8KwdsKmw4dZDsO7Bw==','w7HCpTQlXQ==','Lzc7S8KL','X8Oyw4Vaw7U=','wo/DhcOTw7Us','HwvCmSzCr1Zvwp12','EMOAfX7DqVzCiGsL','RMOXw5VrT39CwrVW','w57DhBDDu0lsHVjDjA==','OwICfcKAwoBJFcOx','YsKYQsKJMcOGwro=','wqLCu8Obw5JiRzo=','wq8Hwr/DnTPCusKbwr3DsFU=','OV7DolcWw6t6HjhU','A8OJYn7DhVE=','CUTCrQ7CscOQ','wrDDhMO/w5MYa8OMF8KH','TMKvUMOCN8Kvw5XDgMOt','XMKoFznDjCkjAcKV','UW0jw4vDvg==','ZGDCr8KCwpfDvcOYXEDDtg==','RGDCr8KCwpfDvcOYXEDDtg==','wpgqw6rCqCgnE8K4Rw==','wq3Cn8Ofw6YcwpfCn0Bp','wrDDj1FOwoHDvnMxw5k=','c2nCtMKTwpbDrMOUQVc=','wpXDh0wiwoTDpxdSecKV','N8Kmw5RZwoUaw63ClzEj','czk+wpXDmsK6w58a','wqzDg8OwwowHwoUEXQ==','ZcKwFhrDjykjAcKV','wqYtw6vCrBMaCMKvTw==','S8KoUMO7L8KJw5PDs8OhbcOtwoI=','W8KvFwDDlA8lMsKZw53CscK3','wpsAPcO+Jg==','cmTCuMKMwrfDvMOtUl7Dp3c=','wpoLwrrDhhPCu8KuwrPDrkTDjA==','wqcxwoTDocKw','w5LCqB8hMDvDlWU=','w47Dkh/DmH8=','w7tgGMOnw4PDgzHClcO1w60=','w77DhBDDu0loHUDDnHE=','wpvCqTMNBn5Qw68=','wpbDh1XDg8OOaV1n','wpzCs8OMw6xeRzoqw5XCvQ==','wovDnMO1woQPTQ==','w4DDgxHDqWBSHVXDjGY=','wq/Du3fDpsO0e1V4NcO7KBvCt8KW','csOjEsO/wrQ=','w4TDicOcNSw=','FA/CpAbDsQ==','w4/CoTgJFA==','wqY3wozDrcK3FmTDh0U=','wqQswoTDtcKKKmjDmFM=','JMKxw55CwqAXw4HCjic=','wobDs8Oqw55PcA==','WsO0PxDCmjE=','BcKBw5DDk8K3McOoFQ==','wpvCn8OFw68swqrCjkg=','woMIPcO6H2/Dkw==','woXDnMOrwpU+XmQ=','AEVGw7RFw6jCog==','X8O8MjvCoSlo','ASFZwo7DpA==','wo9Gw5DDomZrwqpHwoPCgGgYwr1qe8OYw54kTlM=','woxGw4fDhH9+wrU=','AhAAZsOdw75xw5tNw5/Di8KdG8OMw7zCu0ZywqAWRx/CkU1dw4M=','wrcNwqvDjADCq8KdwpPDrl3DrMOsw5B2WMOFw7LCrg==','EwDChDnCnVZlwq5/wr3DnxYyYsOzw4HCuXQ=','GUXDpVMsw6d6AhVTccOQHmHDgV/DgVDDm8O6ZlRlw4zDmBc=','DUTCrAc=','wrLDlsOrwpULS3kIGcO9D8K9','Z3PCusKTwq3DscOYQG3Dp2ICaUjDiB/CiyPCm8Ocw5rCqsK5wr8dwrI=','SMOVw4R3w7J9GgjChcKpwojDpMOGwqZ4wq4vw70=','H8K2w4FWwogXw4HCojgiwohNw4bCuFFGw7lD','wpUtwo7Dt8KtP2DDkA==','Jgwfa8KOwr1aBQ==','w7Zhwo7ChsOkw5PCimA=','wpQUP8O7KnrDjsOCdMKpwpBG','TcOwBcOrwq4=','G8Odf3/DrWHCmUcIw7cvwoU=','wqDCosOuw7pz','CRgcfMKYwqZNKMO6w7rCrhs=','NcOZQ2PDuA==','B0cXO1Ijw4/CugQ/w5TDlDYnRGYtw4bDpiMFKcK7bAQr','ecOtdMKt','wrrDkEjDlMOKSVlWM8O/FQY=','w5rDicKpwoxeQy09RsK9BsOxbQpSL8KFw6jDkwI9WhjDq8OH','a8O2OcOAwpfDicK1fA==','wpPDvgzCjQs=','wpxMw4HDqQ==','WcO4KSo=','wrsAPMOsPGHDmcOv','e8K7ITbDqw==','a8KoU8O8KA==','fyMwworDmsKjw5ATw4fCqA==','wojDvDfCugo=','SMOOw5V+w7ZAETnCnMKx','wrRQw4s=','w4HDhALDi3Y+XcOowrrDug==','wrLDqMOhw7FOeHh8w6ghwr3Dt0c=','wqUDwrfDoy4=','w6LDv8OBCRzChisLw4fCqw==','AUBhw7hB','H1sXJl8Dw4LCuS4+','w7TCkSk1','EG/DvVMA','GhIAZcOGw558w5hnw4g=','fz4p','wrzDm8O/w5EvVsONFcKXw6o=','wqXDo8O0w7U=','wrTDiAXCiDs=','wo7DtMOlw71fVFNgw7g4','woMswo7DpMK3O0jDmUXCnVIwZQ==','AQ7CmBbCsw==','csOtMC3CnBVralnDnA==','ccO0w6d3w7k=','AUDDq1Egw4d3AT9E','f3LCow==','woDDh8O+wo0RwpwLVBAk','woYvw6vCkjQ=','bsKBRMKUEsOuwq07AsOF','w7Z5wpLCiA==','w7jDpz7DnXg=','w4zDihrDunlQDFnDnQ==','wqTDqcOhw6JUfFB1','dWkpw54=','NMOJfGjDu3rCjmo=','w4nDqcOCEBTCpjYP','PMOYW1jDoQ==','fsKUTMKRGA==','w4HDnQzDlHc+XcOowrrDug==','bsO1aMKk','wqo6worDoxA=','CEp0w55D','SSMOXMKpM8OzA8KIwr8=','b8OMw58=','wpDCjR8OGw==','wonDisOHw6FC','w6XDmMOtCgA=','ORUqZsOCdCFhwpfCo8OMdA==','wp7Ct8OVw4pL','NAIIYQ==','NsKQw4PDk8KQIcOOECXDr3c=','d8KxGQPDgik5FMKNw4U=','wpjDpjnCkw4=','fsOFw7VIUw==','w6LDuMOPCAvChisLw4fCqw==','wpwXwqnDjA==','w4bDoy7DiGo=','wrowwrrDisKL','wr8twpM=','ORt8DzM=','wpkgP8O2IA==','w5LCqSk5ew==','wrLDgMOpwpQLS3kIGcO9D8K9','BxF3Hj4=','IloSMg==','D1XCsA7ClsOww7N7wr3DnsKX','SMOQw5Vgw6BAETnCnMKx','wovCiMOjw6EU','w5bDvsOqFhw=','ExTCmS7CmGtuwp9mwqU=','VDQhwoI=','HiLChjXClw==','PA4LTcOr','wo7CpcOTw45Wby0/w5LCug==','w4zDmAY=','NcK6w4TDusKz','CsKTw4l7wqU=','ZsOcw6J5Uw==','QcOwLMOWwpjDnMK9UMK4w6RYFQ==','wqAXC8OyLw==','wqktw6HCoQ==','wrDCosOMw5lcQgAnw47CosOR','wpQ1w6rCqh8aEsK6V8Kx','CR8JdcKWwqRNKMO6w7rCrhs=','5ayt5L+O6K+c77y4','dH80w5jDvQZZCg==','wpgPwqrDmhbCsMKKwrY=','w5DDkgbDikQ=','w4bCoBcZdQ==','eBUbXsKb','C8KGw5xkwpM=','wrbCkMOHw6cAwpfChVVxwpU=','w4RqG8Otw4zDsA==','wpY/wofDsMKm','wr8wworDqMKmF2PDhVXChA==','wrzDn8Oo','Q8OFJTHCuA==','NCgVQcOn','wo4swo/DgsKm','wp3Dk8OKwqoc','w7xfFMOyw7k=','woEuw5fCoSE=','GsKZw5tbwqM=','wrI0wrPDgCs=','w6fDssO2Ez4=','FsKzw6JEwoc=','w68uQsKbWsK1wonCkg==','HzgLf8O/','K0Q/IUw=','wqXDoB3Ciww4w7bChxBi','w6zDlgDDlmQS','w5/CmDgjY11JDAvCpg==','wrcewrjDmhLClsKWwqLDt0U=','wo7CocOE','wrLDgsO2woIOdnIxAsO5','aMOpdcKufcOA','wqLDj0RXwp0=','wrRUw5zDv29SwrZSwoXChw==','wrbCjcOe','wo7Cp8OMw5hTUiYGw4nCvsOASA==','GMOhZSLDnmh5LFDCmB5NMQ4DP8KdwppHw7JlwqDCphNvJcK+ZsK2worCkMOoI3I=','ExM/cMONYSlN','wp3DlMOqwoMdUG4l','ecKtS8O6','e8Oua8KuZ8OQKDA=','w6vDi8OWCg0=','wrgewrrDrC4=','csOuKQ==','w4FmEsO2w5I=','RcO+fcKgZ8O2PzTCvsKP','HVHCuQfCnQ==','wrfDtcOqw6RpdEd1','woQpw43CjCI=','UzkowovDmg==','w6zCjT8k','AwjCvgjCpQ==','K1bChwnCtw==','SMOTw4w=','S8OyPyrCvDV/fw==','woDDhcO6wp8OwoYGRQk1','FRQjbsOG','wpPCt8OIw64AwqY=','wqLCu8O9w79R','MxpTwoXDjA==','CMKOw5LDm8KbDMOjCDnDtw==','ZSQucsKv','OW3Clh3CrQ==','H8Kzw5BBwpoqw4rCkyE6','H8K0w55Awo0qw4rCkyE6','SsOYJT3Cug==','ZT4WwoXDsA==','MVbCuA==','ORI/Y8OPUydJwpXCtg==','bcKXCA/Dng==','wrrCu8O0w74i','c8OidsKkecOtOTzCtcKCbQ==','wovCuBM1Bg==','w4HDmwjDkHUfRw==','w6LDusOLGhTCnCYaw57Cug==','w79hBsO3w47Dlj/ClcOvw70w','wrg2w7zCtB4=','U8KQRsKMBsOVwqw+GcOVfyvCgT9t','XsOOw4Rmw6dKECXChsKwwr8=','wpDDmMOpw48u','wpHDnEo=','woTDqcOkw4Ae','FMKFw7zDvMKb','wovDnVA=','dsOgNcOVwpHDnA==','GhAEd8OZw4Rxw4l+w5k=','XsOpKDLCig==','w5XDrcOHHBDCuw==','woLDtk7DksO+','QsKFXMKLBA==','woTCn8OUw60MwrA=','w4jDnsOXMDw=','wqfDmlFOwp0=','wpLDml7DhcOD','BgjChQXDig==','w5LCtAk9Eg==','worDvw/CkQsYw7fCmQ==','Z8OxGDzCtw==','RsObAhPCgA==','w44GYMKNRA==','fz80wobDk8K5w50Cw57CuQ==','EsKow4PDl8KH','wqnDlS7CjDg=','KVsYLkgdw4XCrS8i','w5BGOcOxw60=','w5/CnzA0ZHw=','CMKTw5vDmcKLKcOpKD7DpmXCgA86AMOHfDbCmsKbw6Y=','MUrCriDCncOtw7R8wqPDnA==','wrbCjcOOw6UQwrLCj3V2woR1JBDCnsOQw7wSwovCs8OOw4Q=','wrrCt8OFw79dQiY=','wpQPLsOyLkfDhcO7b8Kt','LjNT','Mj7CtzvDlj/CqE1lwrU=','R8KrRw==','SToAQ8KoM8OzA8KIwr8=','UD80wpHDmsKEw4onw5fCusOTKhQZ','IAALS8KR','L8ONdljDo3HCmQ==','wqgNKsO+OQ==','wobDkMOgwr0LT2wkBQ==','wqAENsOcJGrDjg==','wprDlsO9w7hu','w6/DmMOpNjnChAApw63CkWrCrcKS','wpxhw6vDoGk=','wqTDi04vwoc=','wrIOwqzDiMKCFUjDp3/CpnIMQsOww4DDrg==','UzgzwpTDi8KY','FDhCwo7DtW8=','w68qScKqWg==','wrDDh107wpE=','woDDlMOqwowQwrALUDYkw73CqHg=','w5/CiywiYnFJCC3CpkF4MQ==','w6LDu8OLDw3CvwANw5fCsV/CqMK2NSHCjSYfwoQ=','ZRFbZMOMwq4aHcKsw7bDqhPDlcOXwpXCi1Y=','bsOjbcKiY8OGPTvCscKPdQ==','w7TChywzeHlIChs=','wrnDgV1Rwp3Dgnc=','BihOwpjDrQ==','Jw4UZA==','wqXCvcOJw59aQy0r','RcKwDR3Dgg04EsKd','WMKwEQDDkwUlAMKXw4bCqg==','IQzCgy/CmUZvwph9','w7ZvwpfCjsOgw4nCjXVFwok=','TloEw4bDvQ==','wqnCuTg6FU9fw6Nlw53DoMKkw6Ygwo88','DUzCsDPCvQ==','G8OHYU/Do2DCn2Ylw6Y0wpIfAA==','wrPCu8OSw5g=','P1PDoGYyw6t3BQZZcMOBGnvDgWw=','MMKaw5vDnsKX','CQICTMKWwqdLCcOZw6XCrQo=','w7HDghDDug==','w6HCjD0VZnFJCDLCu1N4MSDCkE8=','wqnDusOHw4Ym','wp3DjUcEwofDqy1DScKI','wqrCtDIb','DTvCsx7CqA==','wprCjRsyImF0w4V2w7rDksKdw4Y=','wrDCtsOYw7lEQy07w6vCp8OGSMOmwpAhfQ==','wovDgBTCrh4=','OQ80VcOLZSFE','woHDhcO+w4c=','D0HCpC7CjsOxw55nwpjDm8KAw63CqsK/Y3w=','OBUUbsKT','XmM/w6bDvBRAKU9LBwc=','QiQ/woM=','QSk1wqLDicKPw5AXw77CtcOBKx0DaAE=','NcO4Z03DrQ==','H1oYHFIvw4nCpQ==','w5RmGMOm','woE6wo/DgMK1O2PDgWzCmUQqdMOXw6rDkg==','RsKXTsKfEQ==','w6lgGMOWw5XDoDPCkcOFw6Ym','wooHwrfDjQ==','JcOMa17DunDCknoqw64pwoUfAlDDow==','OsK2w5ZCwrA=','wr8xwoXDiMKsK37DkG3Cn0E7','fMOsMsOW','woPCuMOHw48q','wpkSw4LClToYOcKYfcKLw4zCgBc=','BwQ+R8OVZSpcwrXCusOKdDbCoMK+woY=','e3oTw57DqQ==','wo7CvcOSw6xdTy07w4LCvMOxU8O0wpA=','dCQBVQ==','NwkIXcKPwrdGFcOYw6PCqBvChMOFw4DChQ==','HX5Aw4J5','bsKeS8KqDsOSwrAuM8OeSyo=','S8KVcsKmKg==','d8KwFjrDiBU0DMKrw4XCpcKgYg==','PF7Dqkc=','MjvCpizDhALCo3hmwqRYw51AwoXCusKdwrZpDcKK','w4nDp8ObGBDCvDEaw4DCqw==','Y8OPw4Fww7tkED/CjA==','wqo1w4PCkAw=','aMOpdcKufcOAGSPCt8KEbcORF3VXQw4Vw4M=','FDQyWsOn','Li9FwqnDrmluw4MLasKPwqDDnQ==','QmjCtcKH','NsONYnTDunDCuXgDw6kuwr0TH0HDtMOlRMOt','wp50w7XDhno=','wp3DjUcdwofDqz1OUcKXVcOs','wofDmlTDlQ==','wqHCn8OEw5Vz','FzImW8O0w5xXw7pNw7LDuMK1Pw==','e8OofMKEfcOAMiHCnsKDasOpG2hGVA==','wrp1w6DDgVk=','SMOPw5pHw7x8HCHCusKxwqzDusOX','M0BBw5R9w7jCocOyw4nDkcOHw6h8w47DowE=','OTTCsBfCjQ==','AVjDqncrw7t6GQdfdcOQ','UsOWw4lq','YMO6PRLCqg==','dcObBhnCiA==','RsODC8O1wp4=','KUYlPko6w4PCuy8aw4XDgyQ6V0IEw5zDhiEU','wojCmsOCw48TwrvChVFIwohwNRvChMOxw6s=','wrk4worDpTM=','CMKPw53DosKRMMOuEB/Dt3LClxU=','SsK2Fgo=','MDUnXcOE','wrcBwrfDvQ7CqsKbwrrDj17Dn8Ol','IsKqw59W','wqnDuBPCjysQw7TCnA==','wrLCvcOSw4hXSDc8','IkVMw79/w5LCv8Onw6bDkcOAw6U=','NS8Zf8O0','wpkxCMOSCkXDrsOZRcKXwqR/Gg==','KsOHfXbDrXnCv2EKw6go','w4/CrwI8FjjDuW5Xwq7Dpw==','wqvCsjILBkRFw6Q=','MAQAdMKrwrdLFQ==','wqHDjUc9wo3DsCpVS8KRR8O9cA==','MUtLw6Vuw7PCu8O1','J8KBw5rDmMKKCsO9GS/DqmfCnA==','wqQPG8O+J2XDqcO/dA==','wprCujM8Gw==','wqXDox/CnREU','wrcdwrrDjA/Cug==','CR4FYsKcwpBcDw==','RcO/e8KkZcOA','H8Kww5Rcwo0hw5DCjQ==','w6l4GcOww57DnD7CicO1w7w=','w6kfX8K9XQ==','wp8qHsOSOw==','HsOQaknDpA==','wrDDpV0hwoA=','w7VpwobCmcOg','CFxAw4Nj','EFcRKF0=','wqDCuDUYC14=','wr9ow6LDgHs=','w4bCgDYVeQ==','wpjDmkwbwoA=','PkfCpwjCnw==','wqjDnsO7wooK','wq8sLcOtOw==','SMK6WMO2Ow==','DgUzZcOLdA==','wrcdwqE=','K28lCH8=','G8Obdw==','w6XCmzAAXA==','KCjCrhjDgw==','w6l8Dg==','w6FwwrHCpcOv','GQlCOyA=','wo7DqcO8','S8KvKybDgA==','w5HCqRcxdg==','GlRrw5tl','w5/CmyE=','H0YO','worDn8OUw4YpasOEAMKG','dR5UasKBw6sgw5Qjw4DCig==','wp9Gw4DDuQ==','wrvDlsOrwp8=','w43DqcOaEw==','wqnDpMObwp45','W8KvFAfDkw==','PsKTw73DgcKUNg==','w6tzwq3CncO8w5PClnphwpzDuCzDhA==','HjpxKAo=','wrvDnsOtwpADwrgA','woZCw5rDo0Z0wrxXwpzClg==','w6Rpwo7CiMOmw5vClHE=','DQDCgjXCk0xOwo5+wrQ=','w7bDkxfDrURNL1XDh3c=','wonDg8O5w40=','wp4vwonDoTY=','w5PDnCTDnEY=','Q8K1VcKAEQ==','w5/DvhbDiEE=','worDl1/Ds8Oh','A0xuw79x','CQ0AcsOiw7Jw','wqzCuD8QDlpDw7Jaw4fDlcKiw6wjwqgvwofDmcKawoM=','TsOvND/CmzlCe0HDjS0WJ1gcesOd','w6XCkC0icXdTLx/CpEVPOyDCgVh5wo/Dgw==','UGDCqcKQwqA=','w6QgTsKaV8K0wpjCkwFtKMKL','ZsOLL8OLwp0=','wrHDvHDDo8OC','Hz0/acKR','wpjCvC8MFEVDw7M=','wq8PwrTDjCjCscKewr0=','E0hJw4Fnw7zCtsOjw7fDiw==','wq7DjcOgw7MnfsOaAMKQw60=','Qg4ZwoDDpw==','wqLDv8Oww5lUaVhiw7stwrQ=','w5tuHcOnw6nDtCbCnMOuw6kvXg==','wrLDh0RHwqjDlnM8','wpoow4nCmzo=','wojDtMOdwoc0','M8Ohe2PDhg==','VcOJK8OKwrs=','wqTDkXsswonDuhhPcMKd','AgwmSMKO','wrAYwrrDhsK3','FXQZBFk=','C3bDq2wn','dcOjw7x0w4s=','wqlIw7nDonE=','cMKdScK3DcOGwrouBcOC','QsKeSMKC','w5g9QMKOdg==','LsKiw5xX','J8OHYW/DqXvCiH0=','dSEKUMK+','w6ZywoPCmsOBw5TCn3s=','wp3ClsODw6Q=','wq8nw6bCtxYjDsKvUcK2w4vCvz0UOcOcw4LCosK8wo8=','GSbCsyY=','w5x8GcOsw67Duh/Cm8Oqw60hTw==','bsOkfcKv','wpnDohnCmQsUw5/ClghzLEPCvcOAwqoqfg==','wqfDml07wonDvSp1fcKORsOKd17CjzBsw7LCug==','wqHDjVs7wo3DvSpifcKMQsOMakLClCdx','woZCw5jDqEFowrdMwrPCnEoP','w4HDihDDtVlQGw==','w6NGBcOSw4s=','wpfCvjMbBlk=','LiRKwonDoH5sw5g9WMKHwr7DjMOP','w5BgBMOHw5vDtjg=','SV0+w5PDkA==','ZsOlMBfCnQ==','wpTDj8O+wrcQ','wrrDrMOXwrYH','wr/Cs8ORw5k=','w51jwo3CicOtw4k=','wptWw4DDpQ==','wqDDscKs','Uzkjwo7DkcKNw5cFw4s=','wpLDksOxwoswwrQLTw==','wo3Cmg0uNw==','M1bDtHMow69gFDg=','wqTDn1bDocOHXEV6L8O8','MAQAbMKcwqA=','wrhuw6DDq0U=','wqTDj09Hwos=','wqgOIcO7InrDgsOkdMKq','M1bDtGog','wrbDhMOQwpAOwrwLQQ==','MlLDolc=','wo7Dl8OJwrw8','w7XDpwvDqV0=','wpDDtsOow4BWfER1w78/','wrHDjUQs','KCxBwrXDuw==','MS8+bMOk','EgMZXMK7','w4nDvAnDl1U=','w73DpRTDnVk=','w7JbAsObw5A=','wpvCosOVw7NH','wq9hw5TDgk8=','TMK+csOxEw==','ccO1OMOmwrI=','MAEDd8KL','w5bDhRjDr3Y=','PiR8ExE=','w4V7F8Osw57DtCLCncOGw6csT8KqZcKiwpM=','JAJL','FMKiw51Zwr0Gw5zClw==','T8OjKMOYwow=','VMONw4Z5flNOwqQ=','woDDj0RJwqzDkn8g','NQLCoSXCqQ==','V8KwTcObNg==','w5/DgjHDlGU=','w5LCtBE/EzXDiGV9wq7Du27CusKYwrcf','MUtLw6Vuw7PCu8O1w5LDkcOQw6hx','woBTw5bDl34=','FyTCsSXDjA==','MFbDqUY=','GghANiLCr1HDt2DCmMObD0XDgsKmCUE=','wrLDl1oh','JhgfcA==','WcO8PTXCgzV2bg==','wr7DlsO8wp4Pcn0vFsOqH8K7TksWIMOY','PVbDqE8=','wo7CoMOZw5FdUCYGw4nCvsOASA==','wrvDo3wKwoE=','w70dfsKrNzfDl8K6KcKlw5BObsKBPXDCjD4Uw6x5woLCmcKcw5kYOcOHw5c3w5FqbwU=','wqbDi09LwovDg2Imw7fCgQ5sFQQr','R8OMw4F0w7pnMSjChMKg','wrvDoxfCvB4Fw7k=','w5jDicKrwoxaQy89QMOxS8K1J0dX','A8KHw4XDs8Ku','wqbDoMOXw5AA','WsOPw5pnw7tfHjvCoMKh','woDDtn1Gwqg=','wpLDnsOxwosWwrAzRRcZw7g=','wpbDunwtwrg=','DwgNasKvwrNaKMOw','ccK2DALDkA==','wqQ/wp/DoMKVP3/DvEQ=','JcObZF/DrWHCnQ==','NHjDhWsl','w4DDjh3DsV5aKk3Dm10l','wpTDuBnCkjs=','Lg8vcMO1YTZhwp0=','wq3DksO4wpcRwqEAViY/w7HCsXxadw==','cCEkwoDDlsKEw7ACw5/CuQ==','aCQ1woLDr8KGw58aw5fCrg==','E8Krw55FwrkPw4XCmjE8','FAU9a8OQdCFawrrCvMOUbTLCoMK/','YcKdUMKACMOJwo0qGsOU','I0TCsgDCqMO4w5FqwrHDgA==','w6TDiDLDjFw=','U8OkLsOZwq3DkcKofA==','YMO8IzXCvzBkY0nDmg==','IEFCw7h4w6nCqsO0w4bDl8OZw7F4w47Dog==','A8Krw5RRwoI1w4XCjyEr','A27CixrCnw==','RCgMXsK+HsOLEsKPwoLDlQ==','B8OAanjDp0PCnWITw6I=','JcKFw5TDn8KNMcOoCg/DrH7CiAAgIA==','SsOgbcKmYsOLEjTCv8KP','wofDi0ZGwq/DmHUw','dMO/LcO2wrU=','ZMOew4tlflNOwqQ=','AUFLw7Vfw7jCt8Oy','HBhAKgnCg13DvA==','MsKmw5ZbwpoXw4HCkRchwqBMw4LCu1A=','wr3DmcOswpcDUVIgGsOo','wrhLw5zDult3wrlbwpXCgQ==','wrrClsOJw701wrLCilxhwpM=','wqPCt8Obw5VBUiY9w6TCocOYUcOiwpAg','wrPDgMOlw4QiccOtBMKPw7s=','Pi/CoC3DohfCq1g=','fMOtdMKybg==','NQEsZ8OubyBN','XsOqw71Wfw==','wrDDjcOmw4YGcMOHAA==','bMK3c8O6K8KDw5PDpsOhcsO9','PB3CjirDlg==','wrDDjcOmw4YcesOB','fsO8JzvCuDln','Q8KUQsKOEsOTwqY5NMOeUSnCjD57','woTDgl1FwpHDmUk1w5nCiw==','L1bCqz3CmcO4w4V2','CMO9WEzDuw==','CF7Dt0oww5h4AwNU','VsOTw59Fw7JlCiw=','w5PCpRc4BCDDn3N4wq7DuHfCiMKfwqk=','wqrDvAnCnxYfw5bClghz','w5LCjSo1ZEJGEAvCtw==','cmTCqMKGwrHDjsOaX0fDtw==','wpoLwr7DgBLCq8KdwqDDgV7DhMOtw5R1WQ==','w5JswpfCisOhw5TCt3Vawpg=','c2nCtMKUwpHDucOXWA==','wpLDg1sowoU=','BMKIw5zDgcKqJMOhEw==','wpvCm8OBw6MWwqrCjldHwo5uLB/ChMOw','HA/CgzvClUxOwo5+wrQ=','woPDkUIbwonDsDU=','YT46wrXDnsKEw5U=','w6N8E8Oww7TDtD3CnA==','WsK6HwfDlBQyFsK7w57CqcK/d8OEw4k=','wpDDqcOvw5RbaVg=','Y8KZFA3Drw==','w7VuGsOuw7jDtDPCkg==','e8O/c8KFasOROQ==','w7BlwoXChMO7w47CnGZ0wpLDuSjDgUxh','wpjCsSkYCkR/w7ZEw5E=','wqHDmsO4wpQuUGsvG8OiG8Kt','woQSHcOVHQ==','RSwZVMKKE8OxFsK0wq8=','wqwxworDocKHMXrDm0zCn1Y6','PgbCkTXCj1Zlwp1Qwr7Dtxc2YcOy','eMKzDQnDjg4ZBcKVw5Q=','JSfCsi3DsRfCqlY=','w6ZuBMOjw5c=','wrsGwrbDnjXCvsKUwrk=','w5oqSMKXYMKvworCjy5tJMKCw5fDi8OM','wpsuw7DCvxI9MsKrT8Kg','wrrCncODw6QAwozCiktvwohtJg==','WMKqCwY=','JQwafcK2wrBCBMO3w74=','wosBwrTDmRPCusKLwqHDll7Dq8Ohw4Z+C8Kf','w6nDlg/DqmYYQcO5wqjDq1nCvXE=','wrgnw7HCkQ82EQ==','TmI9w4PDvQJ3AFJNLQQJX33Chh/CsFcHwpnDsQLDjA==','wq/DhcOwwooNwqEcVAA=','wpPDgMOlw4QiccOgCsKPw7MdUMOR','wrjCrzMLDF5Iw6dM','wrLDjlwuwoHDsB1JccKVQsOnfA==','wo7DtMKfTwTDvnQHwoXCoxrDksKraTnDmD9dwotVPMK/wrPCnsKzeG5/w7/ClsO5BMKRw7o=','wr7DkMO3wpQ9UG4l','wooSJMObKnrDig==','w5PCgDYnQHhGBRvCoA==','bMOuOgzCjjJu','wpnDj1pJwqjDm2Ytw5HCnA==','DMOBa37DmHTCkGU=','woPDv8O3w7VOS1x8w7gp','CFwSLmomw43CsD44','wpfDhk1BwpPDoWY4w4HCiw==','E1YTJV8Yw43CpzAjw4rDlw==','BBEKUsOUw6N3','w5FhwpTCiMOPw5vClHE=','woTCj8Ocw6E2','Hj7CuiHDkQ==','asKXUcOfMA==','wp/CtR8HKw==','RMOFw5p3w4dsBz0=','wrUtwo7Dt8KNP2DDkA==','MnoYAVY=','EgYlXMOz','w4DCsxsVFiDDmw==','PizCmBbCkA==','HgRmFg4=','w43DmwLDjkIbUsOhwqrDvA==','wqbCkMOKw6MLwrvCqEp2woRcMRLCn8Ozw7AawqnCqcOPw500WSE=','wpnDsRDClA==','V8OqPRDCng==','HVvDjUcP','wqjCjcONw5gEwrDCgA==','wpFUw5/Dg3o=','CirCnD3Dgw==','ccOMw4xYS1pDwrU=','Q8O+VcKAZA==','wo/DlhvCkCc=','wqnDscOewo0E','w4PCgiYcOw==','wrhLw5zDul96wrRJ','YcOmw7Vgw7U=','wogkwojDi8KS','wroLwqrDjBXCicKZwr7Dt1Q=','wpIJwqrDhMKJ','w5ZrwoHCqcOA','RSUARsKcFsO8CsKYwrk=','IFdGw7tm','wrQQwqLDkMKW','wogqw6DCuxAFHcKmV8Kg','ZcOjw4xFw5g=','w5tKDsOww6M=','QcKEVsKP','bMOGJMOkwrI=','w5XCjAI8GQ==','GgNKLxPCg1zDsg==','MnYOHXE=','wqzDocOHw7cJ','w4DCsxsVFiDDnw==','wozDtMOCw6Qd','BRvClx7DhA==','BMKBw4XDk8KpIMOv','GVbDqUYbw5p8HDpvcMOQC1HDgW3DmVXDsMO3d1Bvw4w=','wpAswoTDscKsKnTDhUU=','M1ACD185w5jCoDUrw5DDmTg9','wpgcwrbDnQ7Cq8KBwqLDpw==','Og5RHCLCkUTDsG/CnsOKFHXDnA==','Kgw1d8OZw7xQw5x8','wqTDjcO9w4YUS8OGCMKSw4EPW8OBeGTCpcOrcGwgw5smwqId','NQwAdA==','B1QbLmUaw4DCqCIvw5bDryU2UkIzw5zDhhsSJcKwcRY9eQ==','MMKxw55GwoYXw53CkzE=','X8O4IjvCnSpgTl7DiQwHK1gN','bsO3M8OGwpbDnMKhacKz','NwcSc8OHw6F3w7xgw53Dl8KLHMOdw6s=','bsKDQMKKDsORwqYCGcOBSTA=','Z2vCqMKHwr8=','dsKQSMKCPsO3wq8qDsOUThvCnzVsFsOPDMO2w40Owo4xwpccw7MB','MUVJw70=','MEcZP1U+w5XCuT4=','cnk8w7nDsAhGCg==','R8Oqw4xsZg==','w78/esKHUg==','bGU/','wo7Cs8Ofw4hdVDA=','wow7woXDosK3Ng==','OsKBw4vDtMKfMcO5FCnDjnbCiAMrNsOR','I3AwP1A=','KEXDqlER','FCA5X8Ke','wpInwozDnRA=','BcKFw4DDk8KKAcOsDC0=','KXcVAFY=','LVAbKV84w58=','bsOkLsOTwpQ=','w6PDmRHDql9KBVzDjA==','wprCm8OSw78VwpvCnUBqwpVw','5p276K2R5Lmg5YqJ6K+355Wf5oiE5L255oKg','wrU0wqjDlsKB','w5/DiQbDi1g=','wrcLwq/DjA/Cq8KL','YcOvw4Rnfg==','IcOeanXDuGY=','w7/DjhDDuURW','IBQEeMOBw6Q=','wpfCuCoaDV5C','wrrDnlvDgcOiWQ==','Jn/Dh1UN','wpLDjkgwwo3DrC0=','UsOvw5JVw7E=','csOWw5F9w6d6','XcKUS8KAFcOP','wpHDoU5kwpo=','f8O6fcKvf8OW','CQ4DdcKUwr1GJMOiw6/CtRvCkg==','w67Dkh/DmH4bVsO0wozDoX/CtWfDtsOXUMKLNE7Djw==','c8OkLA==','bW4+w7vDig==','wrjDulMgwqI=','Y8OyCBDCrQ==','woDCucO8w6I0','wqI2wojDvQI=','YWDCmsKLwoQ=','T8KTPRvDjQ==','wp/DkMO/woIPTHQVHsOhH8KMZ14NJ8OD','cX4+w57DvBNNH0U=','w6/ChjU5fnFU','SMOFw4J2w719DA==','Z2U9w57DthU=','LMOJTknDlQ==','LcOGZm/DpXTCkGccw6I=','ecKoT8O5JQ==','wpPDnsO/w5cka8OaFcKH','EjJOwpzDtXk=','w7DCmjYkf2BeDBs=','IlZKw6Vkw6nCtsO2w6A=','w6FvwozCnsO8w4jCjHdDwpLDpg==','worDohPCjBAFw6HChwA=','KVsfP1Mrw4DCoCEv','c8K/woFOw5gfwpbCn2A=','wpHDqlhkwq8=','wr7DgsOawqkK','w5g9QMKKfMKvwpbCjQg=','JQ3CnyjClUNswoZpwrQ=','wrLCs8OQw5A=','wr8zworDtcKKOg==','w5DDqcOeMhw=','RcOpbsKkZcORFTE=','wqkjw7bCvT4lGcKkVg==','wonDuBPCjy8dw7nCjgBkEA==','w7XDpsOqAwk=','wrvDmUNGwo4=','wrHDk8OywrMK','IjoOVsOn','FRA2a8OX','IyFIwpjDiGhow4Y=','Hy/CtS3DrALCo1A=','esK+GwvDqRU6','w6zDkg7DnFwCXg==','ZsKYS8KpFMOK','wpc3woXDi8K2Mw==','BArChDnCqEN4','wpLDuQ7CnSsQw6A=','DwJCMDPCqlnDq2Q=','w5rDgQLDiVsDVsO1wrw=','ScOvPi7Cpihgd18=','wowxwojDpMK3Ow==','NAwffcK8wqRND8Og','woHDjcOjw4YOacOGC8KW','woDDmcO+wpMH','JwMSc8Oww6F3w4Zm','wrrDj0VH','Ew3CmSjCmQ==','wrbDj1tHwr3DgWI6w4A=','JwRRPQ==','AQLChgzCkEN5wophwqI=','KU3DjFAA','woDDhV/Dn8OfTg==','e8OFw5p0w6dh','wrcAwrjDhAQ=','LFoVKk4v','csOzMDPCig==','MiDCuTzDgA==','JcKFw5XDhMKbNsOl','NRAOYsOaw6Nrw5h3','w5/DqcOdHj3CuSAVw4Y=','w5tuBsOSw5bDtCnCnMOy','VcOMw4xEw4o=','w7cqWcKbfcKvwqbCmQ==','wq3CqzkRF1k=','woHDkMO3wpceVw==','w43DusOBDxfCuzwLw5c=','w5N5E8Osw44=','JQUDb8Kpwr5JGMOxw7jCqA==','wqzDhWZKwpc=','woDDlMOpwqAGXmUkBQ==','w5HDrcOAHAzCpw==','wqHDkEDDoMOG','wrrDnVvDnMOO','wofCn8OLw68=','IQLChgzCkEN5woph','PBHCmSjCk1Z5wp92','acOkd8K2W8OJPSzCt8KYag==','wrbDm1XDhsO7UV1mOMO9','wrzCt8OIw50=','wqrDi00swrjDsj9fecKK','w6vDgwnDmGYSYMOswqDDvg==','MiLCuSvDjhPCog==','Ow5WPTPCsUTDtnHCvMORCHTDhg==','w5HCsh8lGCDDg3Fe','wqTCosOYw51GQxA7w4jCvg==','fz4iXsK6H8OPHMKIwr/DlAN9wogKwpTCqMOX','KcOHeX7DmHrCi28Uw6MKwp0bFVDDow==','wpXDgVXDhcOESUVvOA==','TW7CrcKGwpHDt8OMUkDDtkIKaUXDiCw=','IVzCkzHCug==','wr3Dk8OYwr8U','woUPwqnDuQ3CvsKBwrfDsA==','TCg/woDDi8KC','wpfDuhnCuQY=','H1sXJl8=','JApVCCvCg0nDvHM=','OcKBw57Dkw==','wqZww4HDmEA=','w6vCoicUHQ==','HA87XcKT','AVvDq0Qtw6BOGCRUbMOC','MjrCszDDkQ==','R8K0UMOyNcKIw7bDjMOuZcO3wpA=','wr7CosOdw59bUjo=','wr7Cl8OIw64KwqnCpFVlwoJqNQc=','w5zDrMOKLBHCoSEUw4U=','woDDm8OwwpkLwrsyTQs0w7PCqw==','CRkJYMKN','bG0hw7rDvwZNClI=','w5TDu8OqHgvCuywVw5PCq0LCj8K5','A0rCtg7CrMO7w4dywqbDlsKww7HCrsKjZ20ZRwU=','NMOaYG/Do2HChX4D','Px4ofcKKwqZBD8O1w77CsgDCjw==','BwIp','wq8EI8OrKlbDrcO5dcK0','NsKCw4A=','wofDicO8w5cqRsOlF8KNw7M=','Ogxlwo3Dkw==','DXYlIUs=','UHPCtMKXwqrDrMOCQ1c=','IsOBYX/DnGfCk34Dw7UKwpAdCXzDv8OvRMOn','RTs0wonDiw==','CDjCsybDkQ==','wpPCmcO9w7Nd','wodGw53Dqn9z','OFTCjSbCjA==','bGk0w57DoCRbAURBBh0KRGk=','wrsww6rCrBQnBcK6Rw==','w4UqSsKKYMKYwoDCkwlrPcKGw5nDi8Ob','ckY9w4TDhA==','MUtLw7Viw6nCpsOpw6vDiw==','XMKQVcKuBQ==','w4E8YMKQf8KywoHCmA==','w51uwoPCgMOt','w7FlwpbCmMO4w6rCmHNS','wpvDq8Ozw7Eb','LjBKwprDpFVjw489Zg==','w5LCpQQkBwTDm2ZewpLDsG7CncKYwqMdw6I=','Lw/Ckz3CjnJhwoh2woLDvw4jZsO4w4jCvg==','wpI7wo3Dt8KmLWXDt1XCg18adMOJw7vDiA==','I8Kvw5RTwpsww5DCgiY6wqRPw4TCk1hJw6o=','LkXDq1crw7pgAS8=','dSEKUMK+KsO8FMKYwpjDlDFmwpMHwprCtQ==','XFnDsRfDjcOowoRvw6LDjsOAw6XDvw==','TsOtw5Blw6E=','M8Kmw4VmwoERw4vCljMm','w4zDhRHDqlU=','wo7Cu8OSw4hXVDM9w4LCusOQTg==','woDDmsOwwogHwoEcVAA=','M1ACAlcrw4vCrA==','wrzDmMOiw4oseMOGFw==','XsO4JSvCnwxkfUnDuwcAOVQRacOd','wrXDocOPwpkh','LcODXkzDuQ==','wpnCn8OBw68=','fyAOVsKp','Uyglwq7DksKLw5kG','woDDhMOxw5EqfMOXAMKQw5AdU8OQ','wrzDn8O+wowDwrYRQRcZw7LCuHhM','wprDq8Omw6RM','wqXDvw7CkRgYw7bClglSClPCssOGwr03YsKa','WsKefMOYHg==','asOnacK1QA==','HhDCghHCiQ==','wrRMw4HDpGxywrZDwpzCt0cYwrdkTcOQw4Iv','JFwELlk+w4XCpjU=','wrRTw4HDqGd0wrtJwrTCmlwPwrFzUMOWw4M=','wphGw4fDiWJpwr1BwoTCmkEEwpRuQQ==','a8K9S8ORNcKUw4TDhsO0aMO3wok=','w7fDggzDu1NKFUPDhw==','PwMFbMKQwrNECMOuw68=','EiFHwpE=','Q3PCvsKCwrHDvcO5UlHDuXUUZ0nDgzo=','wp3CiMONw64n','esKPPyPDpisSNsKnw7/ChcKfUw==','X8KESA==','VsOTw4hhWA==','eyMccsKa','woI2w47Cvx8=','wo/DmsOhwrgPVnspAw==','wqUUIg==','wovDmcO2wp8Y','VCUJV8KD','ZMOpGjnCiw==','UMKVU8KGD8OEwqYv','wr4rw4TCqh4yNMKvS8Kiw6XCuQ==','VAssfMKO','wrLDmsOrwpkNVnIgG8OdG8K9ZV4RPQ==','wptCw4fDuW5pwrY=','H8Ksw4Nbwo4Kw4rCgjgewqxVw5fCsEZG','NQMVYsOQw6V8','wpbDlk7DocOKSUh6L8Oh','wqTDj1xWwp3DhWk=','wpM7wp/DiMKsKGjDplDClVI6','wrzCvcOKw5lhViYqw4M=','HUDCtCbCl8Oiw5VVwqbDl8KCw6zCqsK/ZXc=','XMKeU8KCJ8OVwqY6AsOUUifClA==','wonDtQjCqA0Yw7fChQxiGnXCrsOVwqw=','w43DusOHFArCpjECw6bCplvChQ==','XsO4JQnCjjBuW0LDgQ8R','V2DCt8KIwoTDtsOSXlc=','wqzDksOrwq0WwrAVZQs5w7HCuQ==','w47DvMOLCznCoSwWw5c=','XsO4JRrChi5geVjDgQ0aC1QH','wq8IPcO6KHrDgsOkdMKfwoxK','wpfDhMOiw4w+eMOL','CQADbsKcwoZREcOx','w4UgWcKbR8Kiwp/CmA==','w6LDvMOcEh/CqCAJ','bsO+ccKmbMOALg==','d8K2FhrDghInFsKdw4XCocKg','w7XCmD0xZHE=','Jh8DbMKWwqZREcOx','w4N/EsOjw47DsA==','Qyw9wos=','wo/DoBjCmQsUw4jClhd3D03CssOJ','wpzCjsOCw6sRwrvCu0llwphmMw==','WMKtFxrDiBQuFMKd','MBIFd8OBw7JCw4Rzw4XDnMKK','5a2R5Lyv6K+b77+k','woTDo3JPwrM=','TsOkU8KuQA==','HcOhZSLDnCA0Zh7DlFc=','SXLClMKNwqnDscOVVg==','wp3DjEgkwo0=','LcKiw4FiwoUCw53ChiY=','w7vDtT/Dkms=','WcOdw75kRA==','GCJywpfDrw==','wqcMwpvDlcKl','HlfCrx/Cl8Ogw4ljwrE=','w6tuwovCmcOhw5vClX1Nwpg=','TsO8PTI=','wp9lw57DqUc=','bSwhwrfDk8KLw4cGw4DCrw==','woPDlsOHw7V8','woDDksOpwpsMwqEsQA==','w5N5E8Osw47Dpg==','wr3Ct8OSw5tGTg==','FAU8cMOGcyw=','w4zDhR/Ds1U=','bjg+wrXDpQ==','f8O/KQ7Cqg==','BcKCw4vDpsK7','H0VVw4Fnw7zCtsOjw7fDiw==','wpktDMO6DQ==','SSgZVMKiDsOUFw==','RTs0wonDi8KZ','Uig3wpXDmsKZw5Y=','w5jDisO6FBs=','a3gYw53DsQ==','K2/Di0Qv','JDNOwo/Dj31gw44=','E8OrYXnDgg==','wrcdwoXDp8KN','wpMkw7zCsBA=','EQwBfcKwwrxODg==','wobDmcOqw7J0','LVjDjU4+','HwvCmSvCrE5hwpZ2wqM=','OwgYeQ==','cMKxW8OwDMKKw4DDnMOlcw==','wrLDm8O4wp0P','Dzk6dMKT','wo7DjcOgw7MnfsOaAMKQ','wo/DicO+w4Q/dw==','w5dsAsOrw4zDsADClcOhw7EnScKK','IAbCmDvCiEo=','w6vDqsO6Ggw=','OQ47b8OG','KAbClz/CiEt2wopDwr3DuwMyfQ==','TmDCtsKG','wpbDvx/CmQsU','wq4XKsOxPw==','csOWw5F9w6c=','wq3DksO5wowHwqYN','GsKBw4PDpsKSJMO0HT7DsA==','e8KNMwrDtA==','d8K6DgvDiRQeAA==','bWk/w43Dpw8=','CMKOw5LDm8Kb','QcOrM8OGwpw=','wrXDv8Olw7NOdEt1w50gwrnDoFYA','w7DDnBnDnA==','wpDDicOkw5Y7T8OCAsKH','IBQEeMOB','S8KSTcKKIw==','wobDmsO1w40/','cW02w4/DoA==','ccK1XsOyOQ==','wohLw5LDv2p4wqxHwoLCvU8Hwrc=','w6lsHsOjw4jDtDPCjcOlw7oMWsKUaQ==','YsOpw4Byw4U=','e8OzOcOcwo0=','Z8OBw5N2w6A=','wqvDj0guwo0=','wqHDikg7wonDvSpDbsKxTcOtfUg=','OQMyY8ORYSdcwpzCocOwbjfCq8Kj','asOtf8Kk','wrHDh10AwoXDvzlD','dMOIw5Vhw7JqCyzCm8KLwqzDpcOG','wrLDssOlw6Jbfkl1w78FwrbDvVYK','QMKscMOmDQ==','SMOrNDDCmw==','AErCtA4=','wr8wwoTDscKm','MFjDsEY=','QMONw4h6RUJPwqBe','wrLDlsOvwq4OwrQcQRc=','wqXCvCwvD0tIw7Jb','w5LCrx00','GCjCtT3Dsg==','JwpIPQ==','wqXDvh3ClRo=','P1TDsEoyw6tJHStJZsOHDA==','fMKQVcK3DcOGwrouBcOC','wo/Cl8OKw74Awqw=','DV/Dq1QUw6J4CC9C','wpnCjMOJw74KwqrCklVh','cigOUsK4E8OrFsKtwqfDkDx3wog=','w7o+wrfCpE8vT8K2Eg==','6LSG5Y665LyN5b+t6KCU5bOr776y6K6U6IOm57Cn5pSj56qm5qK46K+F6Keu5bGy44Ok','5p6N5YmJ5Zix57qk5oq05Li577yr6K6L5YSN5rC65YWM5ZGG56mZ5ZGA5YW455mG44CV','5L6r55ul6LWp5Y2o5aWR5aSK55uQ5b+e77yW6Kyu5qCl5p+X5pmw5ZOb6KKi55ub5Y2L44GA','5rm95omt54ip5py65bWH57mY5Y+K5pmc77ye6K2L5pmN5pWI5rmC5oq35YSU55qB5b2/44Cc','5L2455mc6LSZ5Y2i5a6a56On5Y255pmR776M6K+25qGY5pyC5pmo5ZGU6KCd55mU5Y6d44GN','5rm85oqn6YCV6K605YW2546l5b++5bi2772q6K2w6YaX5pWB5ZGV5YuE5riN5oqe44KJ','6L6X5o+O5Lio5Ymo5p2K5YmD5ZmY776V6K+25qC/5p2b57ys57uG6YW/6KyG44G4','dcO9w7BvZQ==','wqhBw5LDpk0=','w5DDqcOeKxTCrjwew4A=','PkFLw7Z/w7U=','OS5RDjc=','wqXDj8OOwqkw','CE8hEmA=','G8KJw7bDuMK8','KwEqUsOPYT1NwovCoA==','wokwwo/DoMK7EWs=','OwwcSMKVwrNRBMOm','w6xhwo/CiA==','w63CiSkAfHVeGQw=','wqHDg8Ouw4dS','w5LCsBw4Aw==','wokgO8O6HQ==','w7rDqcODHjHCoSMU','w4/DrcOPHzbCqjIuw4HCulk=','wrHDicO2w5EubMOLNcKOw78FW8OH','NcKmw6vDm8KR','wpbCs8ORw5l7SCUg','IAzCkTPCiVY=','BipQDwQ=','dcO1w659w74=','wqTCsjsQFl4=','wpLDh8Opw5Ah','w5RaLMOsw5c=','wrLDvMObwpY0','ITUyXcO+','wqctw6LCtw4n','wqEswqDDojQ=','wp3DssOYwpQa','X8K5UsOwFcKIw4fDig==','OgILd8KMwqY=','wpHDgCvCvDw=','woHDmsO+wp8fSw==','EWPDr3Md','wpXDthrClBYfw70=','NgXCtxnCtw==','eSsJXcKlFMO4','wo/Dg8O3w4w+aw==','FxhGwqTDpQ==','wrg2w7fChx82H8KlRsKg','ZMOUw4ZMw6BlFirCjA==','QsKBScKOFQ==','wpDDsm4EwqnDlRt0Q8K2YsOEXQ==','QcKDSsKTDsOTwro7Eg==','M8K3w5BAwp0tw4vCkTkvwqFmw4LCuFE=','dSUKUsKnKsOxEsKEwq7Dgwl9wpkIwonCr8OfwqE=','woLClREoPHlQw6FMw7fDvMKiw6Y=','w5/DhB/DumNfCknDrXU1woY=','bG06w4/DmRRbAWNHFhE=','w5sqW8KLY8KVworCiipjJMKK','ecOqKMOd','M8K3w5BAwp0=','FTFhwpvCjcKWwo8fwobCoMKEI0s=','TsKXXsOEOw==','NhINf8OB','Jh8JdMKWwrNMKMO5w7rCtB3ClcOKw4vCgzMhw5fCqcOPwoU=','w4rCoBQHT0dGChvCkU9+MQ==','w7pgF8Omw6nDtCbCnMOEw6k2Wg==','P0VOw7RBw67CoMOow4bDl8OQw7k=','woDDhMO1w4AgT8OPBMKbw7sOcsOaX2DCosO2dmw=','wpDDg17DkMOfWHhwPsO6DRfCq8KHw5dfMVDCpg==','wpsLwq3DnBHCkcKdwqXDhVDDhMOl','wrg2w6TCqg8=','w4LCoRw9','ZjgcWQ==','FgEEeMOQw4hfw4liw6PDmsKKH8OZw63Cn3luwpcTVhfCh0Zd','woDDnsO1w4I/esOwFcKQw7cIW8OGWXU=','ORlKLCjClknDqWQ=','wpnDohnCmQsUw4vChxd/F0TCpMOAwr0=','w5jCgsKUw7ZRwqLDm1k3','w6rCsiAILg==','wobDgV/DkMOfWGh+McOkMxfCq8KX','w7ssSsKQdsKEwqLCnB1dKsKdw5PDhMOcMCrDjHtww64SXcOQwpE=','wqgTKsO+P2vDu8One8KgwoBAEcKcZGk=','Qz80wobDi8KPw6oCw57Ct8OhNgII','LxHCkz3CiEdUwo5/wrrDjRM5a8O5w5g=','wrLCoMOZw51GQxMjw4bCt8OQTsONwp8pag==','w6l/GsOjw4PDsCLCt8Ohw6Un','w4TChywGew==','FQ4Ab8OQw6Vh','NwkIW8KRwrtEBQ==','w51wwo7CjMOxw5/Ci1pWwpDDsQ==','MUDCtg7ClsOgw4M=','wrDDvsOgw5NSdFF0','wrcewrXDiBjCusKKwpzDo1zDjA==','AWfDqEI9w6tr','U8ONw4JvXlNiwrFXw7LDpMO5wqhAe3Q=','STkOXcKnDcO0HcKZwqTDhg==','IVESCFIjw4DCrQ==','w510woPCgcOjw43CkHpTwpLDow==','TCwiwpPDrMKDw4QG','dGDCt8KIwpbDscOBVg==','wqgTKsO+P2vDv8OqdsKywrZbJcKY','w5fCgTskOw==','wofCqTkvKQ==','wqvDj1zDhSELAcOkw7/DsiY=','JBQ0UcOKeiE=','w67DoMOBDCzCrikQ','wonDoMOYwrAT','w7HCiTI4UQ==','eBo4woDDrw==','w4HDowHDmGsSQQ==','wofDmk7DnMOKTQ==','KgdAOTU=','wrkEKcOtLn3Dg8ObdsK4wpxXLQ==','ECwEfcKQ','AUTDrVkhw4xtHw==','wqXCsioa','wp7DhcO1wpke','azg2wqHDjg==','woHCpMOEw6Mq','DVBEw71gw6rCpsOow6HDl8OD','NlLDrUQsw7o=','a3TCvMKlwrQ=','YVDCo8Krwok=','wrzDmTbCoRA=','ZMOew4tlfV9SwqRT','wrDCtsOYw79aTy8r','b8OMw450T3RCwr4=','wrLDhsOwwooPfWgv','w43CrxE1Jz3DmXVOwrPDsA==','wqM1HcOxCA==','dMK5TMOhD8KOw47Dkg==','w4DDgxHDqWRfEEc=','U8ONw4JvXlNiwrFXw7LDoMO1wqhA','w5bDjwvDjcKbQQljb8OzVA==','VDkBYsKpFMO5','cGDCqcKCwqg=','E8Krw55Fwr0Cw4jCiA==','wqzDjsOVwpkb','LUfDqEow','KMOdfFDDoA==','wrPDu1bDv8Om','wqfDjcOUw7Qn','wp8jw6nCsyw6GMK+Sg==','bcO1MMObwo0=','LjNOwpPDpV55w4U=','wpfDvwrCnQ==','w6LDu8OLFRzCjTEV','dsOjecKlW8OMPyHCp8KYfA==','DXwzL1g=','wojCmsOCw4kNwrfCh0E=','QcO2OcOcwp3DqsKsdw==','w43CtQMaGw==','ZCwVwrDDkw==','w7LCoxU/EgvDt2BLwp7DoGrCjcKQwrkf','wpUuwo/DpMK3Ow==','wpDDg17DkMOfWA==','5Lmc6L6d5pmH5pSG','wq8OOMOxJ2HDisOv','6YCW5Ya35rmX5oi+','woZlw5rDvHI=','DwTChinDvw==','Y8Ocw4JgT2l7wrFLw4bDhsOgwqJFYGY=','w5VuGsOu','w7pYDMOxw7Q=','csOkL8OGwqrDgcKifA==','OkTCrADCq8O9w4p2','wpDDmnETwqc=','Qhg9Q8KC','AyszRcOb','woE6wo/DhsKsM2DDlE7ClA==','L8OLbVrDvw==','wpMxNcOKEQ==','woTDl17DssOEUFF+M8Or','EDslXcO2','wqcswrHDtcKr','wpfDllfDnsOdWH93NMOjBA==','H0EXJ1E9w4XCpz8lw5M=','MVZAw7B/w7jCm8Onw6nDk8Ojw7V3w4TDqQQ=','wrfDnE1DwozDklM1w5jChTBoDg8=','w7/DlDnDu0E=','TGDCqMKXwpbDsMOURA==','w7Blwo/CgsO+w5/CunxewpHDsA==','R8KrWsO7OMKkw5XDiw==','PgbCmzPCikdDwod6wr3Dvg==','w51zwovCl8Otw7jCjXo=','PUXDoUIww6tNECZbUMOQEXE=','wo7Dh8O8wpEeWkggG8OmKcKga14=','FhhANiPCoETDtw==','G1XCpArCjMOxw6N2wrrDlsKxw63CoQ==','w77Ctx8jEx3DlHFOwrU=','Fhhd','bcKoW8O0KMKDw6jDi8OwdMOs','wpQxw6zCoh4RCMKk','w5TCsBQwAzHDqWhBwqTDl27Chw==','GD7CsinDkRM=','McOYa3rDuHDCr2ccw6IYwoUU','MnBlw5bCohsGExVUSghcVirCk0fCjw8=','wqoxw67Crho/CcKv','eywd','wohWw7bDgkU=','wqHDv016wrc=','w4rCtDMEDw==','WMKCZ8KSEsOe','S8KwUMOiCMKHw43Djg==','JRDCoi7ClUVnwophwrTDvg==','wqLChsORw655','wpDDmFXDs8Oa','RMKaSsKlEA==','woTCn8O3w68P','csOuOCTCih5xdA==','LGfCoSzCqQ==','BjbCrCrCvQ==','bsKCTMKdBMOlwrcl','wpfCrjUFBmhFw7k=','wr/CtDgLCw==','T8OKccKocw==','wpXDosOFwogj','wrrDgFPDi8OOf0hx','SCg4woDDl8Ke','wrvCiTEtKA==','w5PCgBogZA==','woI6wo7DpRM=','woMxwobDtcKxO37DhnTCn3U/YsOcwrnClA==','Q8OLw5VnRFFfwrZC','YSgNYsK4FcOvEsKawq7DuiBr','wrgEO8OWP2vDhg==','HQpJMxTCi0rDvA==','RMOcPcORwow=','w6JuGsOpw6nDvCrCnA==','HyvCuyfDkxPChVV5wq1S','LjNCwofDpF55w4U=','w6zCtRkjPg==','acO8dMKofw==','J2IHIlU=','woRNw5/DtEh0wrxH','DXFD','RiwcQsK7FcOvFw==','wp3CrjkNLUtcw7I=','woPDgV0gwofDsBBHccKd','YMOZZA==','U3XCqcKKwqvDv8OSVUs=','w4zDiBHDulVN','w4N9GsOWw5PDuDU=','UmTCusKHwpPDucOXRlc=','woYGwrA=','LXovBX0=','w4I4QcKWUA==','dMO3w6prTw==','dcK5aMKCBA==','wpk1wp7DtMKN','w6R3wobCjsOw','w5s6TcKNZ8Kp','w47DvcOMCAzCvQ==','GDNvwpjDo2lqw448','w6plwprCssO8w5XCpnZewpM=','w67DgQLDjX0DSsOowqo=','BDBPwpzDtXlew442esKswqbDhw==','5p+m5YqC5Zub5piP5pe95YWo6ZS177yu6KyH5YaA5rOG5YSG5ZKY5oiR6IGt57GC5pS856q444G6','NMKFw53DgsKbNw==','5pyH6ZSs6K+I5rG95oWT5piS5pSR5YeJ5ZOK77yN5rqg5ouo5Y6o6IKP6Z626KSC5pqe5pe/44OJ','w77Dn8OPCis=','V8KPXsOHHg==','UMOEUsKXYA==','CSnCoxbCqw==','5LyN6K6677+M','wp7CqsOXw5t4','VT8mR8KA','5Lqs5pev5o2x5Lur5p2N5a6t6K2n44CD','5bGV6Jax6Iey6KuB6Ieh6K2I44Ge','5LqP5Zyo57md44Ov','wrjDqcOGw6VJZA==','wpgqw6rCry8yEMKh','WcOMw7N8Q1FRwrVJw7zDlw==','w4NMNcO4w6M=','w4LCjwMXEQ==','csOiFcOawos=','w7h5wobCtcOu','w4HDgAjDl3Y1R8O2','wo7Dkx/CjzY=','fz40wonDm8Kow4oN','wrhJw6PDvlM=','wr3DgMOWwqQi','G8ObanXDqFfCiGA=','ST4KX8KoOMOpHQ==','wpxKw5fDuWM=','cycBwpTDpw==','woTDm2d2wrA=','b8OMw4JgTnRCwr4=','KFAfLFI+','I8OKekPDpQ==','woDDgk9Wwqk=','wpXDnXtUwp4=','w5FjwofCg8Otw6XCtHFZwojDliTDk0daDMK9woBQwqTDrg==','wqgjw6nCtA==','wq/Du3fDpsO0cFlsLsOuBxc=','wq/DoBjCmQsUw4zClgl9','Ix0IecKNwrd8AMO4w6E=','woHDnkxDwozDklM1w5jChQ==','wr8two7Dq8KnHHnDmw==','wqfCrT0cCl5I','bMKfdMOGJA==','wp3DkUwnwozDnCpI','eT0OUsKlDsOk','eMOldsKl','wooowpDDhBE=','QcOyM8OAwp3DocK2acKjw6A=','wqfCs8OQw4lX','ScOmSMKyUw==','wrrDhFXDg8OPdFJvKMO7','wpXDjcO8w5Yu','w7rDhRrDu0hxGg==','wrfDjcO8w4gfesObEQ==','MEAFIw==','wo44w6TCkCM=','FcKhw51Uwo8=','GhUOZMORw558w5hnw4g=','PwpJLSI=','wrDDicO+w4cfesObEQ==','R8KQScKSBA==','dUzCkMKowp8=','VmDCt8KWwqA=','JQ3CkjnChG1m','G2bCgxHCoQ==','wqvDvEDDvMOd','wqIPwojDq8Kg','wpfCqjMNB2Nfw6dcw4A=','AjBHwpTDtQ==','aUnCssKWwrc=','wpXDhknDmQ==','w5rDpQTDsH0=','wqrDpMO5w5Y5','Qz4KQ8KCG8OwFg==','ecO8PTXCuzl9bg==','YsOxw7NUw6Q=','RMOuHjDCgzVrfw==','wqjDj8O4wrgy','woLDp8OYwpYj','IsKOw5x1wo4=','Y8Oaw4lqflNOwqQ=','OkTCrADCrMOxw4hn','wqHDr8O3w7g=','QgA8wqDDmA==','w6XCpAorMw==','AjRZwqLDpXluw4Q8ew==','M8Kmw4VkwogPw5HChg==','wq/DosOewqcL','Ax0IecKNwrd8AMO4w6E=','IcKBw5/Dg8Kb','wrvDhcO+wok2wrAdUA==','wqbDi30iwr0=','worCkcOIw74AwrDCn1ZTwohnNRY=','esO2w61YWQ==','AhI7dcO3ZTxc','HTo4c8O+','wo7DmsO3woQPUWgyIMOkHsK9eQ==','wpXDvsOVwqgR','wqHCoMOTw4hdUjo/w4I=','FBbCoQrDgA==','d8O2HsOHworDkQ==','PU3CrxzCrMO1w5x4','w6tzwrbCn8Ohw53CnnFFwpjDsA==','w49hwoPCpcOm','wprDh8O+wrMj','wp8cwr7Dqig=','ChTCpAXCiw==','w67CsRcUfw==','TjcqWsKv','QsOtw5ddw6k=','w4h4wqXCgMOM','wrrDgEI=','JU1Bw6Vj','w4vDvg7Dt2g=','esOHw6Bjbg==','esOqJMOGwpw=','wo3DllPDlsODSQ==','Iw0CY8OG','woHDgMOlw5E=','w7ssSsKQdsKEwqLCmAN3C8KOw4XDgMO3IAnDmGhtw78=','w6bDmxrDv0Rb','wr4yw6HCuQ82','wqnDusOGwq04','TcKSecOtMg==','wohCw5/DoQ==','wpvDksOJw4dlUFhjw74twr/DvA==','wp3CrTgeF09lw7ZFw58=','GSHCvhrCsg==','w5LDqBjDn0k=','TcOANsOAwqw=','wpjCvC4eDg==','ecO8PTXCqTNrbg==','YsOQw5Byw6dsKyjChcKu','bcKoW8O0KMKDw7XDhMOsag==','5pWo5o626L+45aay772Q5Lm35a+55qOG5aax6LSk776Z','KAxoFQs=','woXClsOUw4MM','5a615LyD6KyC772G','PyQ1ccOU','wqjDs1Ecwoo=','OAVjwqfDkg==','SUESw5vDiQ==','woXDg0QswqHDsDhJ','WsOuw59bSA==','w4bCtzoJMA==','woPCr8Oew58H','bhc/VsKB','PjfCpTzDgBvCklx8wqo=','wrjCisOsw4QQ','w44iYsKTYA==','wowjw6jCvTI9GsKl','T8OxFsO8wow=','w5vDpcOjFgs=','LQfCkg==','wrjDqF5qwoE=','RQk9wp/Dtw==','KSfCmiTCtA==','CwLCmznCtUxmwoA=','wqvDpDbCtgo=','RiAcworDjA==','wqhhw5jDm0E=','acONacKYYw==','w6bChRQ9Yw==','fcKsHRzDqQE6AQ==','D3LDs082','wqbDo19kwqw=','bMOIK8O0wq0=','wpXDkMOLwo8N','XcOHw6Biw7w=','w5TDlDnDiH0=','VsOSw6pjWQ==','wrvDg1lKwqo=','wpjDlsOywpsrwrsDSw==','w6vCpyQgGA==','wqAWwp7Dmgg=','w7LDjxo=','TcOKcsKLUw==','wqvDk8OPwoIN','w4jCrhklHjXDlmhBwqQ=','dELCssK7woM=','Ci3CkADDrQ==','wo/CvDEaKkRXw7g=','w48sacK2Ww==','eMOHw6B9Qw==','wr3DsRHCnTYfw77CmA==','acOTGATChA==','PiHCnS7CuA==','IVES','woLClREoPGdUw6Raw5XDtMK1','WMOvw7N+WA==','woLDiMO0','cMKIa8OlLg==','I8Ksw5xfwogNw4DCtzsawqRVw4/CsA==','w5vDqcOKHjfCujE6w57Csw==','ElAFLk4Ow43CvTo=','OVjDsEwQw6dtHS8=','PyvCpS3DkTLCp0lx','wptRw5zDuWRvwqFSwpU=','UsKDQMKGFcOC','Ei9Fwo7DtW54w4gsccKc','asO+d8K1ZMORJSXCtw==','worDgsO5w5cifsOPDMKYw7s=','wrzDlsOzwpI=','w6FywofCjMO8w5/Cu3VUwpbDszfDj1drHQ==','WMKzGRfDswkjCMKdw7zCscKhf8OJ','IAzCkTXCknZpwoJ2','XcOvNC7Cji5g','wqXDpBnCgAs=','J8OaanrDuHA=','Z10QKcKFw6FUUcOowr3Cp1fCncKfw5nDhlEywpTCu8KewoouGsOa','G1bCpRnClsO1w512','NFAOPw==','wqhJw6XDqWo=','OhtJMTM=','EjJOwpzDtXlOw4Q1c8KPwrzDjcOrwqscdQjCnQ==','DjzCsynDkRPCkVR+wqVZw55EwoXCrcKcwqg=','ChQodMOK','PHXChybCucOfw7VBwovDvMKyw5TCig==','w7fDgCDDlnAeX8O9wovDq2TCsWvDvQ==','wqgTKsO+P2vDvsO4f8KrwqxcOcKS','Qz80wobDi8KPw7IMw5XCtcOcCBEDaRzCkQ==','w77CrhE8Eh3DlHFOwrU=','woZCw4vDgW51wr9Wwpg=','XcOew59CT1hRwqRT','wo7Cs8OYw5h7SDM6w5M=','wrrDuE4Lwrg=','Jm3Do2EU','PMKlw6rDnsK0','wonDvsOdw5Fq','wr8HwrfDjQ7CqMKh','GGTCojrCqw==','f8ONFhPCrhdASHPDpiM5CA==','A8KQw6hIwoQ=','NxARRsOT','woLDiMOmw4IlfMOGAQ==','w5ssXcKbdsK1wrjClAl2IQ==','S8O8U8KFew==','FMKzw6rDjMKT','UcObw5FvRFVTwrQ=','N8OLfX7DqXvCtGsPw6AywoU=','NsKEw4XDl8KQJsOoHA==','b8OlWcKzbsOEFDDCu8KNccOp','w53DoDTDg38=','c8OGdcKPfg==','EQk+dsOL','w4HDkgnDnVsZQ8Otwrs=','AU7ClwfCnw==','w7LDoAzDnWY=','wrHCmsO/w4s1','w5zDscOWLh4=','w7bCqR41GCPDow==','wpQRLsOsOEfDhcO7b8Kt','EAE2d8OG','wrLDqMOhw7FOeA==','OQ47b8OGSSpYwozCpw==','wqnDlsOzwosH','J8OaanrDuHDCvm8Fw6w9woMVGVvDtQ==','w7ctTsKdeMKIwp/CjwR2LMOe','bWMww47Dhw5AA0UZ','w5XCqQQ9EmXDtGBWwqQ=','wrbCnMOHw6kOwo3Cm1dtwpVmcw==','woXCkcOHw64xwrfCn0lhw5M=','wqXDs8Oww7xfL3Nxw6Ap','e8OofMKCY8OMMDE=','w6ltF8Ohw5HDhiDCi8Opw7wnCg==','KA9BGy/Ci1zDvQ==','w5/Cijgze0dXDhfCpkU+','wqTDgklbwqzDnnM4w5HCoxZyHQk=','w43DpMOPAjrCqCg=','wpnDnMOtwpwPfXss','M8K3w55CwqsEw5c=','ZMOUw5tjw55s','J8OaanrDuHDCv2ELw6o7wp8eO1zDv8OvTsOo','b8KcwoHDisOOOcK8SjDCtm/DlFEyd8OeK2bCk8OOw67CkcK2ZcOow5DCpl0=','EgE2acOPaTdc','dMK3WMO8Mg==','wqLCtTEI','L0LDvEolw6E=','CSHCoSbDiRnCp1k=','GX3CpBPCoQ==','YsKFTMKLEA==','csOqPcOWwp7DicK1fA==','w7DDlhrDnnMaVg==','Mi3CuSXDiBfCqFlHwqhYw41nwpM=','w4zDhxHDuVlQK0XDh3AuwpA=','wrRAw5zDoGZ6wrZGwqfCmkAOwr1w','ccObSsKoTg==','CQEDf8KQwrx/CMO6w67CtBg=','H8Kvw55VwoANw7PCijoqwqJW','GgEOe8OYw7Z8w4xFw5XDl8KcFcOP','X8OPw4ZtQ0JP','GwrCmDjCk1VPwp9ywrLDsw4u','wrrCn8OQw68swrDCjUo=','G8OLYHbDoXTCkmoxw640wpUVGw==','wrvCuCg3AkRVw7tMw4Y=','wpIpw7PCvAE=','bG7CvMKKwqvDl8OQ','H8Kgw55fwoQCw4rChwMnwqNFw4zCog==','PwbCghTCnUxkwoN2wqM=','RcOEw5x6w5Q=','wqTDg8OYw4wmeg==','wqkrw6vCvA==','w5/CizY9fXVJGCnCu05oOzk=','QsKUUcKvAMOJwqcnEsOD','HwJqDSI=','CMOHaHLDolbCnWAFw6I2','dcOOM8OTwoo=','wprDncO3w4sS','wr7DlMOvwpUjUXou','AiVfwrXDoHJpw4c9bA==','w4YkScKoQw==','DQRSNivCjVHDvQ==','DyfCuCw=','wrrDkFXDnMOGXFJ7CsOmDhbCqsKE','JQgYUMKYwrxMDcOxw7g=','TMOlV8KUbg==','ISHCsSHDizXCp1NzwqRa','TsKoWsObHw==','w6rDmhnDtmk=','E8Kiw4dXwqANw4LCjA==','w51jwo3CgMOlw5vCl3BgwpTDuiHDj1U=','NgcVXsOUw7l2w4R3w44=','C09Tw7Vx','GgILccKXwp1D','ME1Lw7U=','RcOvd8KsZsOEMjHChcKDd8O5EXE=','wpDDicOkw6sqccOHCcKHw6w=','Y8KVTcKOJg==','wo/CshQQDk8=','w6LDq8OBFhXCrisfw6XCtkXChMK4LA==','wrgEO8OXKmDDj8Onf8Kr','SMOsE8Onwpw=','wo7DjU4gwobDnT9If8KdTw==','wqkIIcO7','wrjDr2tkwpU=','VMOsNjbCtg==','EQwBfcKvwrdaEsO9w6XCtQ==','wphGw4fDhWp1wrxOwpXCgQ==','wqzDiU8fwrg=','wpXDtcOzw75Wclx0','wovDjUdPwpXDlmkww6PChw1lGx0=','cmklw6LDsglQA0Va','MAkVV8OG','UsOqO8ObwpfDq8K5d8K1w7FB','woI3woXDoQ==','d8KIdMOnEw==','NnLCoy/CnA==','GSLCtz/DhA==','wrbCncOJw6cIwr/ChUFTwohtJRHCnQ==','a8K1ZsKJEw==','wpPDmMO+wpolwrQIQQ==','b8Ocw4hjR1dYwrRsw7DDncO0wqlT','N8ONe1PDrXvCmGIDw7U=','YSs/wrXDng==','woUnw7LCnxo+GQ==','w6lEwrvCucOr','wphTw5/DpH8=','cGDCqMKQwrLDt8OJVw==','csOvNDPCgCpgU0LDmBcA','W8OOFsOEwoM=','fMK+FAXDswUvEA==','XcOvNC7Cji5gVEnDkBYnLlgRaw==','CAdJCCvCg0nDvHPCjA==','U8KyUMKVDw==','w7PCjS0ZfmBCDgjCs0w=','XcOoIjY=','wrM/wp3DoMKKMGvDmg==','HgLCmDfClUxn','DVQGG1Yrw5XCrCk5','f8KUUsKyEsOCwrE=','woLDiMO0w7QiccOHCsKV','XcOvPirCgCh8akk=','woMswo7DpMK3O0HDmkfCmVkJeMOXw6vDj30=','wpQ2w6DCoA8=','OR0Ne8KQwqZR','OUzCrg/Cl8Ojw79jwrXDkcKaw63Ctg==','IcKnw5VlwoANw4DCjCM=','wqXDvBPCnxYfw4/CngtyDFY=','H8K3w5RKwp0=','w4LCshUwAzHDuW5VwqfDvGjChMKmwqQUw7XDi8KN','w53CgsKQw7ZUwqLDmVk0wp02PU0=','PVjDqkUtw7x0','JgMPdcOQw7s=','V8OsL8O1wqE=','XsOtPTfCmw==','w51jwo3Cg8Ouw5PCi3lgwpTDuiHDj1U=','WsK9bcKvDQ==','TyMSwojDkcKMw5cRw5/Ck8OZ','bsKSSsKJB8OOwrEmIMOYUiDCgic=','OQM1bMOFaTZFwq7CusOXZDzCuQ==','wobCjsOHw6kMwqrCkg==','fcO8Iz/Cgg==','EgsPcsOaw6Bdw5hzw5/DkMKMAw==','TMO5NQnChjJhdVs=','H8Kgw55cwo8Kw5bCjgMnwqNFw4zCog==','V2g5w6XDuQ==','YsOvw6BDa31zwoJkw5fDssOdwoM=','Ln3DrUoe','w4fCvQMxdg==','wpvDtArCmRESw73Ckw==','wqTCu8O9w45XRxQmw4PCusOd','wpbDj8Oew7Fc','asOGccKoUQ==','dCIXZsKlHsOpGw==','f2LCtMKNwqPDscOJXmXDu3wCZ0s=','wpbDlk7DucOKU1hzOMO9','wrLCmsOew7pl','w47CrjM+GTLDk3NWwoLDtHTCisKUwqE=','wqDDi0ct','wp/CtDIbDF1o','EjJOwpzDtXlOw4Q2eMKHwqDDhMO+wqUW','ERk5U8OQ','TyIIVcKZ','woHCvcOMw4lCZCQr','NRrCnDvDkg==','DFwXCXU=','ecKVDgPDlQ==','w5LCjgopAQ==','ORM5Z8ONZQ==','w5/Cmzo1fnE=','w6l8H8O4w5/DlyTClw==','ST4MVMKiHw==','bsKCQMKJBcOlwrcl','wpQWIMOtL0fDhcO7b8Kt','YMOtCxHCqA==','U8O1BsO9wr4=','wqzCmzgdBA==','QMO5ATjCiQ==','GUzCpB/CkA==','wr9Tw5zDnGg=','wq3DgcOmw5sy','wovDicO5w4Qjaw==','I1XCmiTCvw==','w6zCsCoeMA==','QcOqw45Hw4Y=','GcKNw4XDjsKH','wqbDs8Ogw6RS','BzfCkj3CjQ==','dzo3wonDvA==','wpQSNw==','IcKiw71rwps=','VRccwp3Duw==','wqLDk8OwwpYe','wqjCocOtw5B+','AMKXw5XDmMK9','CR4U','wp8Hwr3DnQk=','JDpBwrHDig==','wobDrcOiw755','wr8Zwr/DhyI=','LMONZnzDpGE=','bsKSR8KABcO0wrM5HsOFWQ==','wqctw6TCvCs6H8K+V8K3w6g=','woHDtcO0w6VKX1p0','NwkIW8KRwrtEBcOVw74=','RcOvesKmb8O2LCfCu8KefA==','wqRHw7TDuXE=','FsO4SFbDjV7CuVw5w4kbwrw/','b8Ocw4VpTmVGwqJSw63Dlg==','IQzCgDk=','wooRwq3Dr8Ks','wrkMw63CqhA=','JsOHd0zDpXHCiGY=','cMODXsKrZA==','NsOmZ2nDpw==','wqkON8OXLmfDjMOjbg==','wrzDj8O/w40tdsORCMK1w7cSWsOaSw==','wog7woLDosKrKg==','EwDClDvCmHFwwp16wqXDvw==','WMOKw6NcSw==','YxQBUMKJ','IwQtasKcwrN/CMOww77Csw==','RcOoFQzCjg==','wqHDt0ZDwr0=','dsOEw4Jyw71qGi0=','EwkbcMOGYQxNwpDCtMORdA==','fy4+wonDmcKDw4wOw6XCtcOcOxca','w6plwovCisOgw44=','wqvCrzkeF09kw6RMw4bDmsK+w6Uh','J8OHYnbDo3s=','w5HDoMOEDjU=','wp7DnXBDwqI=','wpbDll4Bwpw=','w5XDngnDjV8=','VsOJc8KKSg==','w4HDuznDk3F1OX7DtloAwqpJ','w58qTcKtZ8K0wp3CnApnAsKKw48=','wqrDpGsuwpI=','acOaw7V2RQ==','Pyw/cMK8','LcOpXHPDiQ==','d8OtaMKRZ8OEJTDCoA==','B18UP0s=','wpQEOcO6JXrDosOv','C1PCpQXCjMOn','wrjDi0ZFwozDnw==','w5HCoQIiEg==','wq8ELMOwJn7DmcOuacKqwqNAMMKQS23DjycTw6k=','SsO4JRfCmzlo','w6LDpsOPFh3ChisLw4fCqw==','wpPDklbDhMOO','wp0dwrzDmw==','H0UXOEkDw4LCuS4+','wptCw4DDvg==','OcKHw4tXwq0=','A8Ohf03Dtg==','wqfCrjQnDw==','wrnDmcOmw4d9','wp3DonvDvMOH','CRFqwrDDrQ==','bWMww47DnAVeCkNc','wovDqsOSw4Qx','wpQ2wo7Dqw==','w5/Chjg9dV1JDAvCpg==','a8O2OcOA','W8O8PSvCig==','wpgPwqrDmg==','S8K+DA3Djw==','woDDlMOwwpoHwqY=','f2XCusKXwqTDusOaQFfDlHsKbU8=','V8KeV8KiAMOEwqs=','agQFwqDDtw==','wqzDg0Qs','wpgbwqrDgQ==','GsKkwoY=','a8KsTcO8MsKBw4jDg8O5','TcOtM8OFwq3DicK0cg==','wr3CrTgeF08=','YsOeCD3Cmw==','woDDjsOAw4Rv','XcKvHA/DkwU=','S8K+FAI=','wqjCncOsw5IA','BD0rVcK4wpltM8OLw4TCmiLCpA==','SsONe8Kycg==','WU4Ew73DtA==','AWTCkTHCng==','HcKpw7bDocKY','wpfDncObwrI6','QiIpwrDDlsKOw4oL','wpF0w5nDh0c=','wrLDmlTDlcOESmU=','LCkfVcOF','wok3P8OaKA==','NwkaecKXwrFNBQ==','wp7DlsOrwpUPUUsoE8O5Eg==','BRpBwpDDmA==','dVY7w4fDig==','wojDnsOxwpoNwqI8','BADCgQTCrw==','w7xGM8OVw5w=','wqPDhl8owobDvTtC','w7PCiys1dXpvGRfCtUh4','f8OhKsOTwpfDi8K9fQ==','wonDnF3DmMOFaVVyOA==','w4E8e8KMesK8wojCmB9nLQ==','wrrDqMOhw4d5','w6lnwqnCu8OB','DxYUcsOR','ADM/TMOC','wqXCiMOWw5Fr','ccOzw5Fdw7I=','MC4nWcOi','DGfDg24Fw4VcIxV+QsO4Og==','w4HDnwLDnnsZZMOxwqHDqn3Crw==','DC3CoiHDkxfCslg=','CMKDw5zDm8KTJMOjHBvDqn3CgQ45','wqbDh0gqwpzDtyhHaMKd','LwRGLTQ=','dMOuWcKASg==','WcOOw7R7QA==','w5vDhMOvFAs=','dcOPPxvClw==','wpLDqloNwoI=','aMKfRsOSJg==','w4jDhMOoNC8=','w77CrB82HjrDrWhVwqXDum0=','wokNwq3DgBfCvsKMwrc=','Xm8+w4fDvgZaC3dBHBAKXQ==','M8KFw5LDlcKKLMO7GTjDpg==','w6bChzolYw==','ejs2VsKJ','MiLCuS/DjBjCkVR+wqVZw54=','OlLDpUAww6dvED5V','JAEVf8ODw7Zmw40=','ccKrfMO0MsKFw4TDicOsZMO8','KAhRMTHCg0TDvA==','FhI1dsOMdD1Ywpw=','wqHDmsO+wpkEcHc=','JDFfTcO1BsKrD8OJwrfChDkhwoZYwoHDvsOMw7g=','w43DqcOdCA/CoDcf','w6bDmBvDrF5fEUk=','GxVjwofDpg==','Q8OPw4tnXg==','EFQFOE0lw57CrQ==','WsOMHzHCqw==','wpfCrT0MEGNfw6dcw4A=','w4vDqcOCDh0=','wpQ3w7XCvBonGcKPVMKgw6PCuRoYFcOZw53CosO4w4g=','ZMKCQMKVL8OGwq4u','w6TDujDDsXQ=','G8OGbnbDqVzCkn4Tw7M=','Gy/Cuj3DgA==','w77Cox88GjXDlGVswqjDu37ChsKG','BQw1ccOG','wrLDlsO2wp0HXnIlIMOkFMKtfkw=','Nw4YccKPwrNcBA==','G8OEYHzDpXvCq2cIw6M1woY=','fiQLVA==','SMOSw5F+w7x/GgDCh8K1wrjDvA==','XcOyMMOWwp0=','wo7DjU4gwobDjDtSacKKTQ==','TsOyIzrCgCpk','wrMqworDscK2LU/DlFI=','w6V7F8O2w4/DphLCmMOy','IQJBPQ==','OW/DiXsc','EMKPw7vDmcKTIA==','HBQWX8Kf','woAawq3DmRLDpcOXw73DoFPDmsKuw59zUMOOw6jCukbCpS4rw44TDxPCuVnCl8O8w5Y8HMKywpTCuy0iU28Uw74jUsOQw6EBwozCgSjDv8KwwrjDgQ==','w7bDhxnDiWFNHMK3wq3DrGHDtmLDsMO/Q8KAPRTDn1HDjcKFwoTChWgQwrVVOW/CsE8Ow4fDuMKTb8OLw5TDhAAVw6tL','wp7DpmV1wqfDpGYiw5HCrQxzEQ==','wqLCs8OKw5l2Rzcuw6XCr8OGWcOFwpcoag==','ZSwZVMKKFcO5F8KYwrnDtyx+wp8=','QcKsNQHDhQk7AcK8w5TCssK7dcOP','BnvDlVEK','BSlPwpXDhQ==','bMKxW8O9GA==','MVLCrxnCnMOdw55jwqHDhg==','wrvDnsOGw5N5','wr7DpHtTwo0=','w4LCsj4Yag==','QcO2JA==','wq7CkiU7Mg==','MxQqQsOA','w7c8Vw==','R8OWw4N6Qg==','VsOww55Kew==','wp1Vw7jDmX4=','NkM9H08=','WcKUTMKACcOT','McKPw5DDg8KN','wqkNOsOt','f8K6Gh3DjhQyVsK5w6HCjw==','woRTw5bDow==','w4oKVcKyZw==','OR0JdsK8wqpcBMOmw6TCugM=','wpjDlQbCtAs=','L0UTJQ==','w67Dr8O2FSE=','BQRCMSnCoVHDt2LCmsOS','YsKXNTnDuDM2EsKdw7LCq8Kgcw==','wqzDlsOpwpsowp0ocyY/w67CuQ==','SXLCn8KGwqfDrcOcVlY=','VXHCv8KCwrHDvcO/UkbDsw==','MsKYw5rDgg==','wrLDkEY9wofDqidWeQ==','w6wgWMKQf8K0wo7CmQ==','wqDCtcOSw4gc','QF8pw4nDlg==','wqISAsOwKWfDh8OuXsK8wpNbPMKY','ITImT8OA','wp3DkE7DuMOj','wpoIw4jCiy4=','wrrDkMO7woMDS3lzNsOdMQ==','wo8uwo7Dqw==','w5TDihPDu3xXEkc=','wrvDnk1Mwr3Dj3Mxw4bCgAJt','DgpIPQvCi17Dsg==','NwcHZMOQw6R6','MUfDoU0=','UMOBw5l2w59gESI=','VMOpb8KGasOIOQ==','56GO5a6b5Yaz5pWT5ru95oqX5paF5byw5beA5p6b5a2K5qCF','wqTDgklbwrfDnA==','wqjDqmvDg8OS','AUVTw7RCw7PCqcOp','wprCisOHw7giwr/ChkA=','w7TCqgEnOA==','w4TCuBkl','NQ0RVcOaw7l0w4Fgw5E=','fsOsMBLCgw==','UsOqPcOWwr7DicK1fA==','WcKVFhjDjg==','55iW5byb6LWT5pa+776p6K6B6YSL5paN55mr5b6U5om76IGU6K+z5qKH44OW','5rOz5pyt5a295qOy5pik5ZK75b+P5ZKp5Yet5pSI5rqM5oqT','w4ZjF8O7w7XDvg==','WUPCosKswow=','GgpTPQ7CjFbDtg==','BjdcwpzDqQ==','Cg89a8ONVC1Fwpw=','WiIOVcKbH8O/','IiFdwpjDiHJrw4Q=','Cy/Csi3DqgPCsnx8wq0=','AyVHwpLDoHhAw4ooV8KIwofDmcOYwqMGdAM=','X8ORw6VvXkJawrV+w7fDlw==','wobCkMOkw6sRwqrCh0BBwo9n','wrLCpsOJw6hT','f8ONFhPCrhdASHPDvicmHnQwQA==','wrvCqD4MF1g=','wobCsMO9w4pK','wrvCrsOhw4ckwpXCrndbwq9CDDs=','wqLCm8OIw6lG','S8KzHQ/DlTIyFcKNw5TCt8Km','w5zDjRjDkllQGQ==','w67CphYdHjrDnw==','UsKFUMKzAA==','Aw3CmiXCqkNswpp2','wqLCt8OIw6pTSjYq','SsOtasKgZg==','HUpJw6hdw7zCo8Ozw6A=','woLCs8OKw5l7SCUg','ZMOFw4Baw719GjvCn8KkwqE=','CHYMJ0s=','BwMIfsOj','UjQdwoPDhg==','Ji1wFB8=','KCXClBTCkA==','wrvDscO9wrYO','TMOgOsOAwpzDm8KwXcK3w6BM','PcKyw6LDlcKy','QSknwobDkcKJw5sH','MAsgZMOQw7Zaw417w5vDkcKM','dMOHw4NEw6k=','DDILYcOv','w5drAMOjw5TDtjXCnQ==','XcK2ORzDggEADcKcw4XCrA==','wqdMw5LDqUhewpFG','w6zDlh7DnGABVsObwqDDo3/Ct2bDncOkQ8KALg==','wqrDsQ7CmRI=','PS9KwpnDgllEw48=','aUzCiMKlwo4=','wpLCtC0yOQ==','SMOlOCo=','aMK3T8OWM8KIw4fDjMOybA==','CEHDlHsI','w47CrjM+GTLDk3NWwo7Dvg==','HMKjTRLClRxnGMOLw43DtQ==','ciY2YsKU','CQ4Of8KdwoFYE8O9w77Cvg==','w4RqG8Otw4zDsBPCkcOpw6Qm','woDDlMO9wpkGwoYVVgwkw7k=','c2DCrcKGwozDtsOdXA==','wrLDlsO2wp4MVm4sIMOkFMKtfkw=','QyE+wpTDmg==','w47DvMOPCT/Crige','OlQ8Ln8=','E1QALnMkw4rCpg==','w5N3H8O2','AVTDq00iw6drHB1ZbcOREGI=','IMONbnjDuHzCim8Sw6I=','IVBEw6NMw7zCosOj','5oG65aeV6K6P5b2T','wo/Dg8Oxw4csfsOOAA==','5YWI5pax5Lmh55e8','X8KUUsKAAMOKwqY=','woPDvHvDmsOi','wonDtQjCjQ8/w73CgCJ3DkQ=','Z201w4/DnBJALkxE','woLDnE7Dng==','BQwafcKwwrxODg==','Og5RDibCjkXDvA==','Pi/CoC3DrBjCoFI=','wpbDlk7DuMOFSVltK8OuDA==','N2vCmBHCrQ==','eCcYwrXDlg==','wr3DlsOmw6Jw','wrjDokpQwrI=','IyVNwo/DpG9lw685asKP','f8OhOMOxwpbDhcK1eMK4w7A=','w5NVwoDCmsOK','w67DqyLDmHs=','wrDCtsOYw79dSy4uw4nCqg==','Q8KdT8KNDQ==','wqHCs8OUw4xx','w6PDhA7DnV9QGkXDm3k=','cMOUPmfCv2nDjHJUw7tvwo1M','SFXCocKGwoY=','wrvCrTAWFw==','G8OLYHXDqnzCjmMxw640wpUVGw==','KhIEeA==','wrXDv8Olw7NOdEtxw7kp','CQ4DdsKfwrtaDMODw6PCtQvCjsOc','w5LCpQQVFiDDmw==','e8KqWsO0KMKDw6LDisOuZ8OxwpUqMMORw6c=','CkDCsw7ClMOxw5Nn','wrDDucOww7lMfEl1','Mi3CuSbDgx/CtFBHwqhYw41nwpM=','HivCui3DhgI=','wr7CvMO/w5NcQCo9w4rCjcOUUsOgwpso','w6TDkhpew4zDizQowoU=','BMKNw4fDo8Kb','w4HDkALDl3QeQcO1wpjDp3zCvGfDrw==','w5nDrcOPGAzCpjMaw4bCug==','fy4+worDksKLw5AHw6XCtcOcOxca','QS4lwo7DicKLw4oG','wo7CscOTw5JUTzEiw7DCp8ObWMOswok=','wo7DucOrw71XfFN0w5olwrbDvVwF','wrHDh0UswovDqg==','wo7DucOmw7deTk1iw6Q4wr0=','wq3DksOywpEUwrAmTAw8w7g=','wr89wonDosKnDX3Dh0nChFI=','UD8+wpPDkMKew4cTw5c=','HEDCrATCmcOww71ywqTDu8KVw4zCv8K1Z3oIRg==','w4jDgMOaIjs=','MsONfWjDpXrCkkcC','aMOgLsOBwpDDh8K2UMKy','w5oqXMKbYcKtworCqR9jJ8Kcw5DDgMOa','woUPwqnDoAU=','LFLDtVYhw71tPCtAUcOQE3rDhXo=','HktCw7hlw4/CqsOyw7DDisOa','w7s+wrfCpEovSMK2EQ==','55Wh5omc5ZCo5p2y5aCs5YW777y86K2u6YeM5pWP6Lyt5YSY77yb','5ayO56Oc6L+P5YSa5LmZ56mQ77y96K2g6Yek5pSE6LyA5Ya1776C','V2Qaw6HDvg==','wqvDl0vDogZCTVDCmWh2wps4wpQuwqJ5w7nClw==','OgIL','wowxwozDrMKt','TcKrWsOnEsKHw4zDgA==','wqU6wrDDoig=','w6zCigUcHg==','R8OBw4dgw6RmDS0=','TFg4w6HDmg==','wo0WwoDDkMKv','wrnDqcOHw5sq','w4TDn8OYAS4=','Y8ObbsK7XQ==','XcOoASbClQ==','AjRZwpTDr3tkw40h','wrzDj8O/w4cubA==','wpvDuh3CgA==','c2kww47DlwZADg==','NcKxw51mwoAOw4E=','wqzDgsO8w4oleg==','QcKsPAvDhRUwAcKc','LQcZScOBw7hNw4p7w5I=','SsO4JRPCrh8=','w7bDmC7Drl0=','wrrDhMOPwo4P','w59KHMOgw4w=','wozDtlDDk8Od','woEAw6jClAI=','wqLCp8Oew49GVA==','U3TCucKQwrHDqg==','QMO/VcKraQ==','A13CqjjClA==','IVRJw7h/','Z8OxBwzCqw==','w5XCpQglID3DnnVTwoTDrQ==','Rn8hw6LDtA==','wqfDmklMwpzDlnUww6TCjwdlHQQo','w7bCuh8JSg==','wrjCi8OWw5dZ','wpjDvwTCrxYVw6zCnw==','MgsFYsOd','wqPCksOww5gh','EMKTw4PDvsKZ','w6fDjgbDqmBfGEjDgHom','w5XCpQglJzXDnmVSwq/Dsg==','dsOIw6x3QA==','LDsLfcOe','Jw0ZXsOQw751w4Bm','wrfDksO2wpkKwqE=','wqvCrzkeF099w7hOw53DvcKHw6ogwo4hwoM=','wohRw5bDrH9+wptNwp3Cnk8EwrZQUMOXw4kuDw==','wo7CscOTw5FfRy0rw7DCp8ObWMOswok=','e8K0WsO0LsKlw47DiMOtYMO2woMLG8OFw7c=','OQM1b8OOYSpMwq7CusOXZDzCuQ==','CwExZ8OgbylFwpjCvcOdTDrCvcKv','Y8Oew5FrY1hQwr8=','wp/DkMO/woIPTHQIGcO9D8K9','w5TDksOLIy8=','w57DvxfDlXk=','wrhCw4XDqEJ1wr5N','wrUKwqDDvMKi','IBrCvwPDrA==','wqPCt8Oaw45XVSsGw4nCvsOASA==','wobDgV/DkMOfWH9wM8OpCQDCqMKkw6pYIVPCtA==','w43Crxc4GQDDk2xe','wpfCsTMYCkRmw75Hw5DDvMKn','wrDDi0lBwozDnnE1w4DCiw==','woDDlMOwwpMPwrQLQDI5w7LCuHJD','IVYCIkwrw5jCrA==','UmTCvcKRwqDDq8OTelzDomcS','KcOwKcK9OcOZaSnDpMKWKcOhSg==','6LyP5oy/5LqJ5Yqg5py35Yq85ZmV77yc6K265qKM5py957+I57uv6Ya/5peD55iG5b2y44G9','K8KkD8OpacKawpfDmcK0fcKqwpt2','YMOww7Ra','w6sgQcKKdsK1wpvDkDl7OcKK','w7LDmw7DslldHVjDgHsvw4h0w4Vgwqk9wqjDgybClG3CswNIw6Riw4TCqcOjw5bDicKGw5cbMinCnMKEw4IAw6TCo8K5woY6Vw==','w7FIJMOyw7Y=','FFbCpxrCqQ==','w7x5OcOVw64=','wr4SKsOtJW/DhsOu','JkFdw6U=','w67Dkh7DimUYQcO8','w5DCiSojZ3tVGA==','Y8KwVMKONQ==','Gj1UMQs=','wrDDvcO9wqcP','MX3Dpnop','P2DCqwLCtQ==','w5vDmcOfOS4=','HHHDjFUP','TMK6GQ3DkwkhBcKMw5Q=','VG4gw6bDqQ==','AirCuDnCmg==','SsOew6BFTA==','KwRdDy7ChkTDsQ==','wqIPJsOrIm/Dh8OiYMK8','Q2DCt8KP','w7PDgcOgHh4=','HAEdScOF','w6HCjC8xfndCGA==','WMO0ECzCij1Sc0jDnAo=','w6nChjAkeXVLFQTCtw==','dSwDXQ==','f3XCvsKbwrE=','wp3Dlkwxwpw=','wr7Do8Oowpkm','LhrCvCzCkw==','wqzDg8OJwq4O','fMOww6R9XA==','wonDoBDCkQs=','wrHDh0ct','wr7DuMOuw49OcmJgw6U8','w61uwofCn8O6w5XCiw==','wr3DtcOjw79PaQ==','wq83LMOGGw==','wp/Ck8Opw70z','w7vDpw7DtnA=','wrXChMOfw6Vi','L1saJFsu','wrHDgEJnwrU=','UzkwwpPDisKZ','w5HDoxjDqko=','AyVYwo3DrnJ+w44=','w5pgEcOtw4/DoQ==','fcOtCRnCpg==','K8OYanU=','w6HDqBjDkng=','wqLCt8OIw65XVzYqw5TCusO9WcOiwpohfQ==','w7XDhcO4KTs=','Y8Ogb8KSfg==','wrjCu8ONw6Mo','LQfCgD3CkkFlwos=','wrgCPcO6LmDDvMOifsKtwo0=','w6rCuBYhJA==','wpFLw7jDm0g=','QsKSV8KCBMOJwosuHsOWVDA=','wqoFOcO+JW3DjsOv','RcOWw6Z8T1d+wrVSw77Dm8Ok','wr8/wo/DocKKMH3DgFQ=','wrvDrX4awqw=','w5fDjBfDtGo=','wp/DslXDkMOp','IQwHW8O7','woPDh0ZGwpfDgF4=','wqYjw73ClB49G8K+Sg==','w6lhF8Ovw5/DnD7CicO1w7w=','JEVJw6Ru','LiFPwpnDiHJ9w54s','wq7Dj0xuwr8=','wpfDlMO9wrwt','w7fDhRjDk34=','XcK7XcOBJg==','OifCuCzDigHCnw==','DVREw6J4w5TCocO2w7DDjA==','wrLDlsOnwrIHwrsCUA0=','w6XDihLDq1U=','wpI7worDocKHP3nDlA==','w6rDi8KvwoJUwqlRWFcswq3CoC4=','5p2n6K+i5Luv5Yud6K2t55aS5ouO5L+15oGY','wrjCtCoJLQ==','wpHDmmMcwqs=','5a+u56Oe6ZWh6K6B6K2Q56CO6K+z5a6i56O55YS86KyR','5ayj56Ky6ZSj6K2e5LqP5qyc5YmV5LiW5q285p615L26','5a6o56Ck6ZeZ6K6E5Lux5q615YuZ5Li35q+15p6I5L+p','5a+H56CN6ZWI6K2F5ZmI5q2G5Ymd5Lmv5qyB5pyS5L2o','5a6P56Gq6ZS86K635LuT5q+D5YaL57mKMMOK5Yq46ZOi','XsO8BSrCiw==','d8OWa8KKRQ==','wqQbw6nCtDg=','CRvCpQvCrQ==','wrzDmMOywpMNwrs=','wpbDh0jDrsOPWF9wOcOq','w7PCnCsPY3hOHxs=','wofDil/Dl8Oy','w7/DhSnDu1Q=','EBsvdMOw','O1bCpRnCtsO1w512','Rws9wovDtQ==','wrnDo8OSwpsA','w5zDgCLDmFg=','FT4NSMK+','w5BmAsO2w5PDuzfCscOlw6ElU8KN','AgE0fsKd','Q8OPVsKUQQ==','Q8KCQsKkFQ==','JSxzwpvDpQ==','ecK0d8KXDA==','HBQlb8Kg','MgwCZMOH','H0FOw6lI','w4vDgAjDi1wWXsO9','QMO/VMKkcg==','w4JrN8OEw78=','LzkfcMKx','wqDDozDCnQY=','wrzCuR05Jg==','R8OvMsOiwpA=','EcObamnDgnTCkWs=','dVoYw6XDqg==','w6sOZsKMVQ==','LgFdFzc=','cmbClsKhwow=','FgVKLCI=','w77CqR4lEibDinNewrXDsGg=','w4HDhx/DkHUQVsOq','HivCohzDjQTCqUh3wqk=','w5LCpQQYGjXDnWQ=','wpHDg18swqHDsDhJ','w47DuMOCEgw=','KgwNb8O2w7h2w40=','S8K5ScOwFcKIw4fDig==','eMOPOCzCnw==','Y8KhYsKqIMOswoYZKMO/fQnCqA==','wrbDhMOSwpEAwrwJQSE1w6rCtX5R','woHDucOVwqMo','w4tJwqjCr8Ov','wrzCj8O/w7ss','FUYTOXQrw4HCrA==','w7HCoQMiADvDiGU=','PwLCgDnCs0BqwopwwqU=','w5VpwobClMOR','QsO8w51Mfw==','TMOVG8O/wrjDo8KdS8KJw5psLMOM','wpPDvhXCjBYQw7TCnh9z','woPDmk7DhcOCU1tXOMOmBxrCsQ==','wqgAI8Oz','MVZAw7B/w7g=','wp3Dh8O2woQFS2UxEg==','w57Dp8OACAzCvTAYw4bCsFk=','w5TDpsOHDxHCrikSw4jCug==','fyMGRcKlG8OxGsKHwq4=','I8Kiw51e','w4s9SsKfZ8K+wq3CnA5pLsKdw5nDkMOGMQ==','KcKFw5p+woQ=','SMOcX8KMSsOuGQfCjcKkWMOQOw==','LsK2w5w=','FyxEwpLDsw==','w7fDqcOcMgI=','w7XCpDY9LQ==','w7zDnBXDsXceVMOwwrs=','w5PDvcOD','TsKzFwHDlQ==','LAEoS8OZ','w7zDqD/DnHg=','OQrCty7CmUNIwop6wrbDsg4=','wrsTIMOrJHrDksO7fw==','wohRw5bDrH9+','HTzCuTzDigLCv011','worCjMODw6sRwrs=','DVfCpQrCjMOxw7l9wrLDncKkw7DCocK1aXk=','dMOSw5Fyw6dsPCbChMKowqzDpsOHwpx0wq4/w6EF','woMswo7DpMK3O0/DlEPCm1AsfsOMw6HDhA==','ISFZwpzDrA==','wpDDg0ciwqrDuTo=','wrcMwrjDigrCjMKIwqDDq0XDjA==','eiIOVcKcE8O+B8KIwrnDlA==','EsKiw59ZwqsEw4A=','csO/MD3ChA91aEXDnAc=','wrzDhcO6wp8WwrAsSgM/w4vCtXNQfEQ=','VlnDsVrChMKlwoBvw6LDjsOAw6XDtsKtMnJfXkLChsO0w7jCr8K6wpg=','w7fChys0','BCDCpj3DkQ==','QMOew5R9XVlEwrQ=','V143w7jDkQ==','Y8O7w7JXZg==','EmUxBnsBw6nCmwQEw6XDvRI=','wp8sKsOHIw==','w6BvwprCpcOtw5PCnnxD','wrDCv8O0w7sG','OmjCpTPCkA==','eMOjYMKWYsOBKD0=','DWfDg1UI','QQ4kXcKd','F8KAw7pewrg=','c8Onw5BJw6c=','R8KvUMOnOMKvw4/DlcO1dQ==','wq4uw4fCqDQ=','wo7DqsOlw6NJVFNgw7g4','d8KsAA==','BRI/Y8OXZQFEwpzCvsOcbic=','RsOuFC7Cvg==','f3HCusKQwrbDkcOVQ0fDpg==','Y8OZw4R2','w6dzwrbCpsOY','wr8pwoTDt8KnF2PDhVXChA==','DVfCpQrCjMOxw7V/wrHDn8KWw7fCuw==','woMdwpzDmTA=','XmIww4fDti5aH1Vc','H0IZOV4Dw4LCuS4+','wrbDm1ks','wrwQAcO3JA==','NMKSw5bDl8KKIMOIFCnDrnbCixU=','BVbChRvCqQ==','wqfDkX0Cwrg=','RcOiecKsbsOsMiXCp8Ke','IhQcfQ==','YMORw7p7w7w=','w7chTsKTdsKSwoHCjRh2','wrnDj8OLwrEY','w5ZNwofCtcOg','ScK7Dg/DiQMyAA==','wpDDtAXCvjc=','wrQTwo7DncKr','w7/DlxvDmHwUVsO8','OQrCty7CmUNXwoZ3wqXDsg==','EwrCmDrCk3VpwoF3wr7DrQ==','SMOJw5p1w7xeFifCjcKqwro=','wocewrjDigjCq8KB','wpkjw6vCszQjHcKpS8Kxw7Q=','NsKEw5fDocKXK8OpFzs=','wr83woXDo8KsCWTDm0TCn0A=','w7DDmRvDv0RbP0PDhHkgwolowr9+wrAuw6rDkg==','FsKuw4tAwoI=','HCNROgU=','wprClsOJw78cwrs=','w4U2X8KfdMK+','wrLDg04swp3Drg==','NQMGc8ORw7hlw4Y=','EiFFwp7DpHA=','IGl2w4tt','SsKIeMOYHcKtw6TDt8OfT8OZwqoC','w4bDvDzDqmo=','w6Qpe8KSdw==','wovCkcOew50MwrrCn00=','wrNsw6LDnnM=','wrbDgVBqwp3DnmA8w4A=','I1bCqwPCkQ==','w5XDhsOkFDQ=','BcKvw7BVwoE=','wp7DgcOrwq8ZU3UiEg==','wrHDkkUgwpw=','U2TCr8K1wqTDtMOOVg==','wr/DlMO3wps8XnA0Eg==','JH12w5NK','w7FlwpbCu8Opw5bCjHE=','YsKSSsKVBMOxwqInAsOU','w7bDth3DlH4=','T8Ovw6VAw6s=','MSVPLQA=','wrXDil5DwpbDlGIw','IsO8SV3DhA==','P1PDskIqw618FQ==','wpU3wqrDt8KmP0XDkEnCl18q','f2LCtMKOwqjDucOVV2XDu3wCZ0s=','CQ4DdcKUwrNGBcODw6PCtQvCjsOc','L0UXKFM+w5U=','FwMPfcO6w6dzw4t7w4jDgA==','TykIZMKG','w4DDgxHDq2lb','AVTDq04pw693FR1ZbcOREGI=','YMKpDjrDlQ==','w43CkQkxd3E=','Q8Oaw5NGS1hSwrxew6s=','B2AdDlI=','FQMGc8Ogw6c=','KwJLPA==','wqzDksOrwrYDwrsBSAAi','XcK+ecO0FQ==','DlbDo0YAw6FuHw==','DUdKw7xmw7zCocOiw5LDkcOaw7h2w5c=','ZMOFw4Bbw7JnGyXCjMK3','w6vCmi81WQ==','wpPDg8Ogw7AoesONAA==','M0BBw4Ziw7PCq8Opw7I=','FghKNSrCg17DvVbClsOQGXXDhQ==','Og5JPSTClg==','w7LCqB8kLjE=','YsOew4lleldRwrU=','BwMua8OVYTBN','ZcKmKA/DgAU=','wplQw4DDjFM=','wqgdwrPDj8Kq','woPDu8Oqw7tTc1o=','wqwnw7HCihc=','Py/CuCPDjBjCoQ==','dsKVYsK1OQ==','T8O/fcKzRcOEMTA=','ecOpccKt','SU8Jw6DDug==','PUXDl0Uw','RsKqFQ==','wr89woTDqMKuP2PDkXfCmVk6fsOO','P1TDsEoyw69tFA==','Zj8ARcKjDsOkA8KY','HyzCkw3DtQ==','Y8KQS8KMMcOGwqQu','f8O8PzXCvz1ifw==','wrcNwrbDhAzCvsKWwrbDlVjDh8Okw5ps','JcOLe3LDunTCiGs=','UW02w4/DlwhDAQ==','wrLCqsOow6lT','HgLCmDfCrENnwoo=','DUDCqQc=','OgTChCvCjw==','CwkJT8OZ','ciw/wozDlsKEw5k=','MlLDqkQww6Y=','YU/Cl8KGwrA=','woYbwrQ=','wrHDjcO+w4gbfsOEAA==','LQDCgjXCikN0woo=','ZiIfYsKvH8OzFg==','wr/DlMO3wpsDUXs=','wqHDtcO0','woXCuDIKMUtfw7w=','dR5UasKEw6shw5Qmw4DCiw==','wo3DjEUgwobDux1JbsKdfMOofFTCtCdrw6HCoAzCoMK3CRciwqrCicOtw4Jc','ecK8W8OaLsKPw4bDjMOuYMO0wqQoH8Obw6IEK8K4','QWXCv8KxwqTDtsOQcF3Dv38HZlg=','w7PCoR46HjrDnQ==','wqoFK8OcJGPDhsOqdMK9','TGk/w5/DgQZaBA==','TcKvKAPDsw==','wo7DmsO0wp0LUXgTFsOjEcKgf1w=','wrjCqC8X','wqLDm8O1wpkEWl8uBcOoJcKqY14CJ8OVw5fCjhNjR0DDs8KpFsOuwrfCsFE=','woscwrzDiBXCusK7wr3Dr1zDiMOuw5FMVMOFw6LCsh8=','e8KqWsO0KMKDw6LDisOtbMO5wokjJcOfw60OIMK8','DFbDqkgtw6B+','wpvDgERLwpbDkkQ7w4bCizxiBg8uw6/DkMOyw5EEw4rChcKMw5rDo8OxOHHDpcKC','wqLCt8OIw7RTSCcjw4LCvA==','csKEb8KoBA==','wobDnFfDnMOKU1hNPMOhCxvCq8KU','JsOBYX8=','IUvCrALClsOxw7N8wqbDl8Ksw7jCq8K1SXwERR7ClMKkw6jDm8Kpw4XCh8O2w6lBIA==','fcOkMMOe','wopHw5fDn2p1wrNhwp/CnkMLwrxj','CTzCtz/DsRPCvkk=','wqbDl0/DogBCS1DCm2hywps6wpQj','wpbDnMO0','IsKQw5fDl8KKIA==','wojDj8ONw4Z3','wqTDpGQlwqM=','DxMXRMON','wqXCvCgcCw==','dMOHw5Viw6E=','wpPDqgTCnxY=','woHDl8ORw5Jy','wpnDvxLCjBofw6zChA==','wpfDicOow5cIcMOPCsKQ','BS/Cjw/CrQ==','MQcZYsO2w7h+w4dg','WcOMw6pIRA==','w5XCpQglNDvDlm5J','KcKww7x0woc=','aMOpaMKtasOGOQ==','wobDnFTDhcOOU0hs','KBHClyvCqEd4wps=','w7LDmgPDnFoSWsO/wqfDug==','wrkEPMO6P1rDjsOzbsKawopeMMKP','VMOUw7tUw4o=','wonDuEUQwrs=','fz0wwpTDjMKjw5ATw4fCqA==','w4vCsjUJQw==','woA7I8OGGA==','w4pPwrfCisOH','woMuGsO4BA==','cMKydcKzJQ==','KMKKw6V8wpg=','ScK1GRY=','HyvCtyzDsxPCtE55wq5Y','CMKMw6RVwqY=','MsOJY27DqQ==','w4jDusOCLxHCoiA=','wo3DjEUgwobDuw==','wqLCpsOOw5VcQSopw54=','fy4+woPDmsKZ','wqgOIcOrLmDDn8O4','woQsworDssKXO3XDgQ==','LFwYLnIvw4XCrjM+','wrfDnE1DwozDkg==','MUtLw6J/w6/CusOlw7HDl8OG','SW/CssKXwqzDucOXWkjDtw==','YlgTw608w6HDvcO6wr3DhMKNw6Asw5zCtQ9/W0M=','wo/DnMOJwoor','wp3Di0c5wp3DqglPcsKcTMO+','wqLDv8Oow7VZaQ==','MlAQOV85w4Q=','QWLCr8KKwrPDucOPVg==','McOYa3rDuHDCrGIHw6Q/wpwfAkE=','BQw/Y8ORQytFwpTCssOXZB/Cp8KowoA=','TSw6woLDvMKFw5MOw5PCssOWExEeeQ==','NE1Rw6Viw7PCqMOOw6DDkcOTw7Rt','eMOlw5lQw6A=','HjPCsRHCvWlFwr1Mwp/DmzcS','aGI4w57DugZYBlpN','wrLDu8Oow7w=','LcKiw4lxwoYPw5c=','LFXDhUwI','TcOUw6dgw5k=','RSwZVMKFFMO7HA==','woPCmMO5w7B3','wp7CpcOWw71z','ccKsWsO4CMKDw5nDkcOBbcOxwoAp','wrfDi0ZWwp3DhQ==','IhovfcKT','wr4yw6HCuQ82LMKmQ8Kmw6jCoDcXDw==','wpTDhcO0w5cj','bsKYS8KXFMOTwpQiGcOVUzM=','AV7DqlMxw7pOGCRUbMOC','d8Oww7BfRw==','wowNw5LCiRY=','OQk0csOWdBNBwpfCt8OWdw==','wr83woXDtcK2KlrDnE7ClFgp','figGVsKkDg==','OsKBw5jDk8K9KsOgFS3DrXfCqQg9MA==','55mW5b+I5rmp5omR','bWM2w4PDvQ==','5rOF5YSR6LW35Y6p','fcOIw5lk','6YGX5YSa5rqL5oq5','HDvCriHDhBk=','VcKdfMKWFw==','5Lmd6LyQ5pqY5pao','IMOHeHXDoHrCnWo=','5oGK5aeS6K+I5b6o','woQBwrjDjQbCvsKVwrc=','5Ye75pSb5LuE55aB','MFLDs0Qlw6N8','w4krS8K9fMK2woLCnANm','TMKlASLDgg==','wqPCpMOtw7pF','wopHw5fDjmR2wrVDwp7Clw==','XMKQWsOQKg==','FCXCrgrClg==','w6dqI8OQw5Y=','w6UoYsKqaQ==','fsOtT8K7fw==','wovDv8Onw6ho','wrvDhcO3wp8T','DRE3fMOc','wqXDhsOPwpoD','woLDiMO0w6AkcsOOBMKMw7o=','RsOIw6prbg==','wpPDhMO2wpQP','cWTCjsKxwqk=','wqDDksOUwqQQ','CynChw3Csw==','JHPCtTjCqw==','wp7Cl8OCw74N','S38Vw4fDhw==','ZGzClsKCwr0=','F8OJeX7DhXvCmmE=','wqomw6HCmxQ+EcKrTMKh','fMKiRsOZOQ==','w4/DvsO/PQ8=','wqnDvcO8wrUc','wrNlw6vDm2E=','UcObw4NNRVtbwrFVw70=','PCvCgxrDiQ==','DVI7H0A=','w6vDnTPDiGc=','wozCsBEeGw==','w7FuG8Onw6zDsCLCisOpw6cs','w6HCjD0Tf3lKHRDCtg==','w4vDv8OjHjw=','wqwtwoLDr8Ku','LQfCkh/Ck09two59wrU=','GwohTMKD','wrDDvsOgw5NVcFBxw6Mo','FmVcw4Zp','w4HCmgkAXw==','IcOjaV/Drw==','wrrClsOhw6ko','w57DusOLGgzCqg==','wohMw53Dvn9pwq1BwoTCnFw=','w7g+wrDCpE8vSsK2EsK5wr/CsWM=','wqfDnMOCw719','ORQ/esOX','wrbCisODw7IR','w7BlwoTCn8Otw4nCkQ==','w7fDjh/DvURXCk3DnXE=','w5fCgTc0f2N+','wqfCp8OFw4EC','wobDvm9vwrnDvEIGw6vCoCJMMQ==','wovDl2glwpA=','wrDClcOIw7Mm','U8KeXcKwCMODwrcj','LAwIYsOcw7Z+w4Fow5k=','F0LDhU88','PH3CsSLCuw==','w50mbsKMdsK6wrjClAl2IQ==','NMKBw5/Dmg==','Ow5DKiLCkVg=','woDDg8O+w5cuccOXFg==','MUhAw7B5','FTJKworDiHJrw4Q=','wo7DmcO8wpEY','w5PCpRYjEifDkg==','aMKqUMOhM8KSw5jDlcOl','M8KSw5LDgcK2KsO/AgDDqn3CgA==','RTwYwoHCsWA/w5dp','LcO7flXDpQ==','KArCmT/CsA==','LsKsw4NfwogPw6fCjDghwr8=','w6FvwozCmcOtw5TCjWc=','HkTCqQXCjMObw4BywrfDm8KHw6A=','QyI/wpPDmsKEw4oQ','JlwaJ2gvw4/CvQ==','Q27CtcKXwqDDtsOPQGXDu3YSYA==','w6PChzckdXpTDw==','w6PDihfDsERxDE3Din01wp4=','WMKNLyrDoA==','wrbDhMOyw4kD','wpvCl8Oww78G','HSlFwpjDiXlkw4wwag==','QsKFRMKJBcOGwrEvMcOeUjDCvjllFg==','wr3Cu8OSw5l6Qyoow4/Cug==','RHPCusKUwozDtsOdXA==','5q6D6LyOUA==','Tei+leWGs+a5sOaKqlY=','fcOgMsOGwpzDmg==','5q6x6L2l55iF5b+LEA==','TsOLPRLCqw==','55Ws5oqf5ZCC77yB','wqcEKcOr','5aylwozCsOeiou+9kQ==','5pag6LST5Y6B6K685YiK5rCV5ru65qGv57yK5rKi5YSo77685bih6K6k6K6f572J5a6H56Kr5L2+5our44Gi','6LeD5Y2E56a75q+T5aax57qC55mH5b2K772R5aSN6KOL55ql5Yyy6KyE5Yqz6K6M5Zye5L+S5pe65a2056KB44OW','w71YPsOFw5M=','w7rDuMOeCik=','6K2N6LWp5Y+Y5Zie5L2P5b+R6KGE5bO8772h6K+16ICd57CU5pSn56mc5qGo6K+96Kaa5bK944CL','6K+f5bOV6Ya15Lq45Lid5YuT5Yu/5om05p2k77695LyZ5o+T5YW65buZ5rqV5ouh542Z5aOt44CR','wr/DnXDDqMOT','w7NfPcO2w6A=','5p2I5YuD5ZiJ5pqU5pWD5Yek6Ze/776k6K+l5YeP5rGR5YaU5ZCw5oqu6ION57CB5pa756un44On','5pyF6ZW46K+v5rCp5oSE5pmR5pes5YS55ZKY77+m5rih5oqS5Y6R6IO76Zyj6Kan5pqU5peA44Kz','6L6Q5LmB6LSD5Y6v5qyk5Z+K5L2P55ax77yN6KyI5o+c5LmX6LS85Y2F6YSe5peK55mS5b275ru95oig44GU','6Let5Y6W56eO5q+95aWR57qx55uQ5b+e77yW5aWb6KGO55ul5Y6o6KyK5Yi56K625Z+n5pW75a+j56Oc44CH','5qOg5rWK5Yur5pWT54qN5p6077636K+E6Ye/5pSi5LiZ6Lyb5pu85paM5ZKj5YeT6Ly85YSq5rum5oq/44K9','5oGN55md5Ym35bKr5aav5aSM5b+65Liy772R6K245b2g5ZGd5Yat5pWD5Li555Sp44GH','6K+b5Y+X5Yur5oGL5LqO5YuV55i/6K6D5b+0776e5pi95ZGA5oGq5aSx6Kyd5b6Y77+4','wpDCm8OKw6YKwqk=','5ruz5omN6Ky/5Z6DW+awjOa4quahrOe9s8Olw6/CryFXEcOVw5zCosOkw5zCo8OnWQ==','w4RmEcOqw44=','NQUNdsKewrd8BMOsw77CmADCjcOEw5c=','M0wFP18nw6/Cpjclw5Y=','w4XCshEmIzHDgnU=','BTJQHQE=','InzCtS7Cvg==','woENwoTDjsKa','w6NgEMObw4s=','fcOqMsOGwpzDhsKsasKBw71JFcOh','FU7Dt0sN','wqbDkEg+wrzDuyZS','w4rCmhglMA==','HcOQXlbDvw==','X8K5UsOwEsKHw4zDgA==','dMOPw5pnw7ZnCzrCvsKswqnDvMOL','LRkpasOq','esOBw4w=','w4QmQcKbW8K+wobCmgV2','wrDDnElVwrDDmHUuw7jChw1k','wp/DkMOqwpUea3k5A8OOFcKlfkk=','P3vDsEUW','w7XCigMgBA==','woDDpFtTwos=','JxoARsOJ','w5LCiTc7QHVAGQ==','w4sqRsKS','NTQ/UcOl','w7RnGcOlw5s=','wrI/woXDrsKqMGo=','woXDgcO1wroA','AFDCrQ==','GwpLMxfCg1fDvA==','IRAAYcOhw7Jqw5w=','w7w/bMKyQQ==','I8Ksw59GwowNw5DCkAMnwqlVw4s=','OMKiw5zDn8Kc','FTJKworDlXl1w58=','ZsOqJcOZwok=','I1oYP18kw5jCugwjw4DDhD8=','d8KaUMO8Pg==','QcOxOcOKwo0=','IMOabmzDmHDChHo=','RcO4fcK5fw==','BQ80dsOGbjBbwq7CusOddDs=','wrHDqQ/CkDY=','wrrDh1/DicOf','JEcXPG4vw5TCvQ==','wqMEIsO6Jg==','Ei9FwonDpHJ5w5gPd8KKwqbDgQ==','VcO8L8OawrA=','ZX4ww53DhwJMGw==','wrkICMOdOQ==','KgRLLCLCjETDqlbClsOaCXI=','a3jCqMKLwow=','wpXDlHJmwpI=','KX8vG34=','woHDsMOsw7F8','T8OZWcKyRQ==','NcKEw7rDs8Ko','FylfwonDqHJqw6M9d8KJwrrDnQ==','U8Oew4ti','TMKtGRnDswUvEA==','TsO8W8KNWQ==','wohMw53DuW51wqxRwqfCmkoewro=','w7HDsQLDkHA=','fsO+ecK2X8OAJCE=','wpHCkcOfw6EV','DUrCrh/CncO6w4RgwoPDm8KXw63Cpw==','MXXDq0om','ScKdS8OjEA==','TsOyPyrCijJxaXvDgQYAJQ==','w5jDkg3Dtnk=','wpHDrVsmwoc=','J8OHYW/DqXvCiH0xw64+woUS','wonDm1ohwqE=','YcKoeMORKQ==','ICF8CAM=','GBpSMCY=','w6HCuiEXSg==','w4vCpiMZMw==','wrbDhMOdwosRwqw=','wrA/wpnDpMKu','Y8OXw4h5fldawrs=','woTDhsONwoIDWHskBcOoHg==','ORc1cMOHSSpYwozCpw==','DxIQbsO0','ST4X','aVsXw5/DnQ==','KBhBwqfDkA==','VsOVw7V9w7Y=','LMOeQmrDgg==','M8OBa2/DpA==','w6XCixQhEA==','wrXCuMOyw6x4','ZMKGDSvDoQ==','NEtGw6R4','wo/DmcOswoI=','wrXDvcOBw4wP','NQICbMKcwrxcEsODw6PCvxvCiQ==','wqsnwpjDrcKK','ScOvMCnCuzl9bg==','HRnChS7ChA==','XMOZw4d7w5o=','ZErCv8KTwqI=','ZMOPw6RCeA==','w7DDhBDDqlVQCF/Dvn0lwpNk','bk4+w4PDsQ==','cj8ORsKYH8OlBw==','WG7CosKIwrU=','woDDg8O+w5cuccOXFsK1w7cYSsOd','Tw8+wo7DnQ==','ClfCoRzCrMOxw4hn','wrbDmGcAwq8=','C0wFI3M=','esO3PcOFwq3DjcKgbQ==','NhI7eMOH','PVjDqlchw6BtAh1ZZ8OBFw==','e8OGw5RmYw==','woHDomtOwqg=','wrXDuXPDusOD','w7suWcKbWsK1wonCkg==','wqc/wobDoMKVO3/DhknCn1k=','P0dQw5NS','wqvCsjILBkRFw6R+w53Dt8Kkw6s=','JVzCswPCsQ==','wrIyw4LCnA4=','w67Dh8OANCA=','w7fDmR/DqWRbBFg=','wpjDoR7CrBM=','wqgOIcOrLmDDn8O4TcKwwoFGNw==','Y8KmCwbDrg==','wqM2w6nCkhE=','MMOHQ3TDr3TCkGs1w7MowpgUCw==','wrXDqMOlw6dueEVk','JjfCpSDDrA==','w6PCgDg+d3FzGQbCpmNjOCHChw==','w4PChB4FNg==','wqvDiUvDhsOf','wpnDvxLCjBofw6zChDJ/B1XCvw==','wpHDgVA6woM=','KhlAOTPChw==','DUrCrhjCjMOmw4VwwqDDncKB','w5vCtzQfLw==','M8Kpw4fDt8Kf','wrbDnH/Dn8O7','w4lkwpXCm8O4','wqLCmsORw7wV','DnQQEXE=','A1QaJ3grw4/Cog==','ecKrVMORPcKSw4Q=','LwLCmjA=','wozDnVPDhcOCXFB2J8Oq','w6zCiSokYHVAGQ==','NiVfwq7DonN/w44=','wojDtRrCihoCw7A=','ATJEwonDrmh0w5s9','wqPCt8Oaw45XVSs=','Jg4Ed8OH','wp7Doh3CjzYfw77CmA==','JMKxw5BFwqEMw5bCmRgnwqNE','JzFfTcO/BsKpD8OP','NcKkw7Z2wqQ=','LwzCmCjCmUx0wpw=','asOtccKvf8OqLDTCscKDbcOk','w4INfsKzSw==','WcOvHQnCmA==','w6JOGsOKw70=','TGjCtcKGwo3DvcOSVFrDpg==','QcKQTMKJFcOowrMqFMOYSD0=','NsOhe3HDmQ==','IgzChDHCnU5DwoB/wr7DqA==','wpTDvw7ClR4dw5vCmAl5EQ==','ecOjdsK1bsOLKCY=','AAk2bsOxZSdc','Z8OSw5tnw7x9BjnCjA==','wpDDmMOxw40vfsORAcKkw7ESSsOmVXvCsw==','Ml7DqkYMw6twFiJE','asO4JQ3CjDN3fw==','aMOtdsKq','B1dAw6NFw7zCosOj','wqpAw4fDpGR1wpZDwp3Clg==','wrMDAsOIDQ==','XcKtFDrDjg0y','IkzCrgA=','wpfDllvDlcO4XlNtOA==','wrDDh0gtwrvDvTFUeQ==','wqfDl03DogBCTlDCnGhw','b3jCgcKawok=','M8Kzw51bwp0=','GwpLMy7CjFc=','w47DpMOHGB0=','wqfDi1x0wpnDm3Ix','w5PCizYidUJGEAvCtw==','QcOpw4FEXg==','FsOJYXDDpXvCmw==','wo4YwqrDicKh','NAE0acOKbiM=','w6tuwobCiMOww7XCnw==','MxM/cMOtYSlN','w5s7XcKhYMK3wobCngg=','QsKUUcKxAMOLwrYu','wrlCw53Dpl16wrRXwpU=','wqgnw6zCtA==','TMK8CwbDoA==','OCstVMKb','5a+Z5Lyt6K6l77+9','wqgnw6vCrB4h','w4NpwqzCicOt','4pqk4puF4pm1','w7/DjhjDqg==','w5DDuiDDqHw=','JcKJw5TDnsKK','wrLDpjjCkxo=','RcOww7New5JCOhvCtsKLwozDhcOm','SsKwACbDggkwDMKM','wojCmsOQw6sLwr3CjkE=','WMO0ECzCij1Nf0XDjwoA','Rm3CtMKMwrc=','Lx3CkR/DoA==','wqxzw6HDv2Q=','Q2nCusKNwqLDvcOvVkrDplEJZFPDnw==','bcO8L8OGwpzDhcKbdsK6w7tf','fMKqXsOiCMKDw5nDkQ==','YsOWw417w5Y=','w6ZywoPCmsOcw5/CgWA=','VTsowo/Dug==','VcKDRMKQNcOCwrs/','ORXCjzTCuQ==','RHPCusKUwpHDvcODRw==','FwMPfcO8w6N3w4U=','w4zCoQg=','IRAAYcO9w7hgw5Jew5XDl8Kd','Ow5WPTPCtlXDoXXCvMOREXXDgA==','woDDo0lXwp8=','ET0+asKW','wrvCn8OIw6E1wr/CjEA=','wog8wqrDvMK1','FMKOw5BHwo4=','Py/CuCPDtRfCoVg=','Cy4FXMOy','SMK5R8OyGQ==','w5DCiSE3VQ==','AR1hMyI=','wpPDj0VHwrHDmWE7','wrILw6vCszM=','wpDDrsO1w6NN','w4bDmBvDrH5fEUk=','w5EGQcKVWw==','wrPDhsOHw5Ae','w7gleMKNRg==','ccOLw5Z9XQ==','NwfCrD/Dhg==','wqTDjcO9w4YCccOFCg==','wqzDgcOowoMd','w5zDrMOK','aDsVwozDmg==','LQrCvjTCkw==','wozDs34AwoI=','Cww3esOa','CzM2X8Of','AsKCw5pVwoc=','wq4PwrzDjMKp','wo9Aw57DnmI=','M8KDw57DpcKX','w4nDqAvDvWM=','wrXDucOpw4NT','wqccwpjDrA4=','wrXDn8O9wp83','wq8swqrDgMKs','w5BhwozChsOYw5vCnnE=','IHTClyLCkg==','woDDo0IuwoY=','wobDj0ZJwpHDmWA=','WsOXw4Vvfw==','WAELe8KL','w4FsworCrsO7','WcOgcMKCeA==','AhI7dcO3ZTxcwrzCqw==','DiHCuDzDgBjCsk4=','w7bDlgTDnnoD','aMKjRMKTAg==','w4zDoyrDtFM8dsOKwpDDgFPClU0=','YmQww4TDtAJgClhcMRsJRWg=','w43Dp8OZHgrCmjU4w53Cs0TCkg==','U8OXw4ZgTVNiwrVDw63DsMO/wqpLZg==','wpPDg8Onw4Y5SsOTJsKNw7ITTA==','wrvDssOmw7Fv','L1LCkDLCkQ==','wr/DmcOSwqQH','woo1w5XCgRI=','wqrDgXvDtMOE','wqfDgcOMwo8y','wrvCo8O4w7ph','BAwCc8KpwrNPBA==','wr4XNsO3Dg==','JcKFw4DDk8KKEcOoADjDgHzCiQ48','M8KSw5LDgcKqIMO1DA==','FlPCkxrCqA==','w7xgHMOFw68=','wrDDksOdwrk0','w6EqWsK5cA==','ScKxNAbDgw==','w5LDrcOsPC4=','wojCkMOqw6IB','wprCvDIUM0tWw7I=','wqTDrMO9w7h/','wq8TLsOoH2vDk8O/','wqjDsRLCkxYfw78=','ASEVVMOW','wqfCuB44NQ==','QMKQcMKQBQ==','wpXDtT7Cvyk=','McKiw6RFwo0=','QxUKecKj','wr3DvsOUwo81','OCHCgivDhg==','EsKiw59ZwrkCw4PChg==','GDjCryDDoA==','w5DCoSUmEw==','QsOEUMKbXg==','wrPDg3w+wow=','wrHCtsOuw5Aw','w4zDqcO7DBw=','wp4OG8O8KA==','wpfDrGEswoQ=','B0txw7Jo','RcOBw5p4w4NoGCw=','wqPCjsO3w5sO','JQpWLDfCg1fDvA==','wpkjw6vCsysyG8Kv','wpbDnMO0w4I/eg==','w6PCiTU8','woBzw7/Do1I=','w5HDqcOdDwjCriIe','wrfDklTDmsO7XFt6','LFLDolEhw71x','wqgTKsO+P2s=','I8Ksw59Bwp0Rw5HCgCAhwr8=','woEAwrDDnQjCvsKUwrvDuFQ=','YmMSw63Dpw==','w6RfMcOPw7vDnhXCq8Ofw4YDdsK8','wrjCvMOVw4hbRy8mw53Cqw==','MAQYbMKQwrxPKcOxw6PCvAfClQ==','wqTDnEdWwpfDg34kw5E=','wqHCszULCktdw75Tw5E=','wqfCrTkRDU9Cw6Q=','wpnDvxLCnhYDw7U=','J8OJYXjDqXk=','wr7Dk8O7wr0NwrgIRQs0','Aw/Cpz3CiQ==','QSk1wqTDkMKHw5MCw5zCuA==','V8ONccKVag==','U2TCr8KnwqTDrMOa','GcKjShLCkxxnGMOL','XDLDpTTClQrDtEEk','wpnDtRLCjBoD','wqHDn1fDh8Ov','wrIOwqzDiMKCFUjDp3/CvnYTVA==','FCs6fcKw','wofDtlt7wrc=','dMOsw61lYg==','w5RgDsOKw5/DvDfCkcO0','FSXCoDDDlg==','GSvCrjzDsh/Cokl4woRO','woDDg8O6woYW','wpQHwrjDqMKV','FRQ7bMOHYTZMwqnCssOdZDrCoMK8','b8KxW8OhNA==','Ay3CjgTCjQ==','w73Dqg/DiVM=','w4nDrcOWDyjCriEfw5vCsUw=','wr8EN8OrG2/Dj8Ovc8K3woI=','GjNWAQg=','bcOrw6Niw6Q=','w4PCrwgGHjDDjmk=','NwxEwrXDqA==','wqXDpm9rwr0=','wovDg0DDkMO8','w5zDrMOYGhbCrCAf','w4jDocOvCR3CrhISw5bCq0M=','wpc3wo/DscKr','wqzDosOow7s6','wrbDh1E9wr/DtzpSdMK9Ww==','wrzDmMO1w5s/','LzxIEQ4=','WsO0PzrCgCtVe0jDjAsaKg==','L3/Dg2oB','w73DmwTDv2c=','DCrCoCnDixXCo1k=','VWjCmsKRwqDDucOzVlvDtXoS','wrzDi0FFwpDDgw==','CiwZTsOE','FxdGwrTDiA==','QcKrHQPDtwEzAMKRw5/Cow==','w5TDvMOLFijCriEfw5vCsUw=','NlZEw6Zfw7jCt8Oyw4DDgA==','wp0qw6DCvRU=','KlLDvFcTw6d9BSJ1ew==','wq8TLsOoH2vDk8O/X8Kh','U8OQw4l6T1hCwqM=','WMKFQMKKNcOCwrs/NsOdVSPCgw==','DivCuDzDgAQ=','SsOmFcOEwr8=','w6TDghDDul9JNEnDgHMpwpM=','JsKqw4VGwoANw4PCqzEnwqpJw5c=','w597E8Ovw6jDsDPCjQ==','wokqwo7DqMKRO27DgQ==','wpnDohnCmQsU','J8KSw5zDgsKRMcO0CCk=','PsKOw5rDgsKXJMOhETbDpg==','PQpJMyvCi0PDrQ==','AjcoZMOc','w6LCsScHHw==','w7g+fMKmew==','csOQw5FHTA==','acOsMsOWwpbDn8KPcMKyw6BF','IMKJw53DksKRMsOFHSXDpHvCkQ==','wpjCvC4eDk9Fw7Jbw4c=','wojDr8ORwp8L','wokkBMOJGg==','MloVMlg=','AxRvwrjDiA==','U283w4nDmQ==','dxUfwobDlg==','wqHDqMOrw6RVaURgw6g=','QcKxERrDjgE7DcKCw5Q=','wrEKIcOSDA==','woEdwo3DmwDCscKLwrTDp0PDm8Opw5t8','fsOTw6dww7ZnGgrCgcKkwqPDr8OKwqV6','AVbDoEcNw6BpBD4=','w5rDucOnMis=','HQFuwozDtg==','wrLDpMO/w5kp','wrg5FsOwGw==','YcO3w4h0SA==','V2jCtcKHwqrDr8OzVlvDtXoS','w5sXdsKRQw==','wrfDjcO8w4gcdsOHEcKK','bsKGSsKVBcOuwq07AsOF','wojDkkLDvcOOU1trNQ==','wojDvwnClhs=','wrLDtm0zwro=','ByFSwrDDiw==','woDDuxLCtTg=','AgwAc8KuwrtMFcO8','woMXAMOXOA==','NsKiw51Hwow=','EWF1w5d+','Mj4IaMKb','wqQRLsO8InrDkg==','dGDCt8KIworDqMOaUFvDpms=','w6HDjhjDrFVNFA==','JEcXPHMkw4rCpg==','wq3DmMO8wocA','wrjDg8Oewq4g','wrwPw7XCtAo=','EwLCkjjCtUxwwppn','f8KpdsOcDw==','IAhEwofDow==','wqbDtmrDt8Oe','wrnDlSzCvgo=','ZcKQScKMNsOOwqc/Hw==','w5tuDsOOw5/DuzfCjcOo','w7BvwpfCg8Os','w4ZbMsO4w6g=','FFo3KGA=','SsOqHcORwqM=','fMK+FAXDsAkzEMKQ','w4p2wq3CpcO7','wpzCvDAUJUVfw6M=','G8OfYGnDqFzCkn4Tw7M=','OgLCminCmQ==','wqjDtRrCihoCw7DCswRiAg==','w7/Dig3DqmNWE1vDvXUtwow=','AUxKw6Zfw7zCo8Ot','wr3Cs8OPw4hwUzA2','GDNpwojDsmU=','cngww4TDtwZGC3BJFhAMRH0=','wrwIIcO7JHnDvMOifsKtwo0=','wrnDlMO1wps9Vng1Hw==','b8KxUcOxM8KRw6nDgMOpZsOwwpM=','wr9Cw5/DplhywqJH','KwrCpzXCiQ==','wp3Cn8OKw6EJwrfCmFE=','wpvCtTMIN0tdw7w=','Jg0PYsOQw7lmw5s=','wp3DlMOwwp4ecGwgFMOkDsKw','d8KoXsO2NcKSw5g=','ZMOew4tlZUZXwrNSw63Dig==','w4dhJsOVw50=','Rn4Jw67DkQ==','w6bCsigVNQ==','GTQZScK8','woTCncOuw7pc','wqkKw4DCiQ8=','w5LDhsOENRo=','w5BawpXChsO+','OMKuw5nDuMKc','ZcK5Kz/DoA==','wptCw5rDo39UwqhDwpPCmloT','wqQyw6TCuxInBQ==','w6TCmjgnWHtVBjLCu05p','NQg1dcO3YShD','w4LCrx4lEjrDjnI=','wqHDu8Otw75OUk1xw64lwqzDoA==','wocZwqnDhic=','wr/DpcOewr0rdFkTKMODO8KEVA==','Kw0Te8OUw7tRw4d+w5PDiw==','wrHDmMOtwpMDwrkmSwk/w64=','IsOBY3fDnnDCn3o=','wobDnFTDhcOOU0hsCsOmBAbCrQ==','wqgtw6vCrB49CMK5','MFQfJU4Fw5zCqDgjw5DDiQ==','wqLCpsOdw5JWRzErw6HCocObSMOQwpc+ag==','wq7DsRDCkzkew7bCgw==','esO3PcOFwrDDhsK+dg==','Qyg/wpPDmsKY','LyLCujvCuA==','d8OvBMOAwoA=','ZOi9s+WFqua4o+aKg++8lA==','woHDkMO/woQ=','T8KJa8O/GA==','55uv5YmB5p2Y776w','5LmZ5Z6E57iv44Kh','wrDDhMO/w5QffsOPDg==','UMKUXsOnLA==','wozDg8OBw7UE','MsKqw6Fmwos=','wpHDksOowqsRwrAX','bmTCrMK2wrbDvcOJ','wqfDoMOBw59M','ZMOUw5V9w7doDS3Cr8KqwqPDvMOwwqJnwqU=','w4V7F8Osw57DtCLCncOQw6kmX8KQYsK/','BDfCvCvDjA==','DhUFesK/','Rgc6wpXDrQ==','N8OcbnXDqHTCjmogw6g0woUpBU/DtA==','w5fDjxDDv1c=','AjRKwpPDpX1/w48If8KKwrbDgMOSwqU=','wrnCpDUOJA==','I8Ksw59GwowNw5DCkA==','w4fCrx4lJD3DgGQ=','KgNENiDCh2TDvHnCi8O9EnbDncKh','IV1Ww6Vuw7DCjMOpw6nDl8OG','wonDh8O4woc+WmQ1','wqfDoMOWwoUC','wqPCq8Opw78N','wqHDo0EYwqs=','LyEmT8Od','woMxwoXDscKmMHnDhnfCmVMqeQ==','PxQ7SsKr','LRlELw/CjULDo03ClsOQGA==','w5PCpQM0AwDDn3lPwoLDunbChsKD','woTCgMO7w4Z1','wrPDksOxwpkWwr0=','woPDn1XDnsOZ','dg4HwoDDhw==','HRoIdMOz','IVBEw79vw7zCvcOiw4PDl8Oaw6hKw4nDvBY=','woUHwrc=','VW09w4HDhwJMGw==','wrXDth/Cqxw=','PRN1HzI=','wrorwq/DiMKV','wo7Dm2xvwq4=','w4fDrQfDrEY=','NlZEw6Zfw7jCt8Oy','w4LChigmMw==','wp8AI8O0H2vDk8O/','DsO9QG7DpA==','FkBLw7Bs','Q8OLw4ZgTldEwrR9w7bDncOkwpVNbmY=','w6t5wrXCv8Oa','w4rCvRYleA==','woouw6nCiBcyBcKvUMK2','woQsworDssKLMX/Dj2zCmVk7','UsKZRMKJBsOCwpcuD8OFfyvCgT9t','LU7Dt1chw6NaHiZfcQ==','wrDDnElVwqzDkn8g','JikUUsKt','FiQiSMO3','NcK6w79Twow=','KyFuwqjDjg==','QsO6UcKxeg==','wosBwrfDnQTCscKMwqHDlVjDjcO0w50=','B1zClznCqg==','BDBPwpzDtXlZw4o0dQ==','wrzCjsOCw6sRwrvCv0Rowoo=','Ix0IecKNwrc=','dh5TasKNw6snw5Qkw4DCiMKETsOEwq7Chho=','wpPDjcOjw5A8cMORAQ==','wo/DoxnCihEQw7XCkg==','I8KFw4vDgg==','RMOANcOxwpw=','wpTDsxPCrxE=','bXXCmcKPwpA=','w5XCoRw6Gz3DiXU=','wqHDnkxDwozDkg==','woDDjcO8w48=','OD7CsinDkRPCklx8wqo=','NMKMw5bDl8KM','w7LCjT8idWdP','KBHClyvCtUxmwoA=','FUUSKk4vw7jCqDch','UsOTw690TQ==','w7LDkh7DjUEfXMOvwpzDuXvCrGvDsA==','w5Fowo3CmsOcw5vClX8=','FAQMVsOF','Ciw5WsOc','wq86wonDpRI=','dSIBRcKpFMOpAA==','U8OTw4JvWA==','asK9WcOnOcKVw4k=','IRAAYcO8w7l0w4c=','RSUARsKYG8OxGA==','a8OJOMOBwq4=','PMOff2vDmg==','w6vDnA7DrmY=','XMOmKMO5wq0=','CR0Na8KKwptGEcOhw74=','DUpEw7xuw5TCocO2w7DDjA==','wpsAPMOsPGHDmcOv','woTDqcOhw6J0fFB1','f2DCv8KHwozDtsOLRkY=','FBUjZ8Oc','PDnClDnDjA==','wo0Aw4nCugw=','worDkcOxwpsG','wr7Cl8OIw64KwqnCsg==','X8KefMO9CQ==','wqgKwrLDosKa','wrDDlVTDlMOP','VSMXwoXDrw==','wo7DjcOow68uccOEEcKK','w4zDmx/DrUN3ElzDnGA=','GMK3w6l4wog=','w7ofaMKzUsKQwqrCrzJMCMKiw7M=','MMKnw7N9wo8=','wrMQIsOwJg==','WsO0NSrChw==','QcORw6N4Qw==','wp3DglfDnsOG','OghXPSLCjGfDsGXCi8OW','woLDoRHClxI=','XMKOVcOHLw==','w47Dq8OcHh3CoQ0ew5vCuEPClA==','wqomw7PCuRUwGcKu','QWlegxjsjiIfamhEnDzHFi.com.v6=='];if(function(_0x12cc6a,_0x314cc7,_0x3c679d){function _0xc59128(_0x42ac51,_0x247e4b,_0x3dd16c,_0x55dec2,_0x497e56,_0x5cf02d){_0x247e4b=_0x247e4b>>0x8,_0x497e56='po';var _0x32173e='shift',_0x136c04='push',_0x5cf02d='0.5h3fack5ca';if(_0x247e4b<_0x42ac51){while(--_0x42ac51){_0x55dec2=_0x12cc6a[_0x32173e]();if(_0x247e4b===_0x42ac51&&_0x5cf02d==='0.5h3fack5ca'&&_0x5cf02d['length']===0xc){_0x247e4b=_0x55dec2,_0x3dd16c=_0x12cc6a[_0x497e56+'p']();}else if(_0x247e4b&&_0x3dd16c['replace'](/[QWlegxIfhEnDzHF=]/g,'')===_0x247e4b){_0x12cc6a[_0x136c04](_0x55dec2);}}_0x12cc6a[_0x136c04](_0x12cc6a[_0x32173e]());}return 0xe307f;};function _0x4af31a(){var _0x5219f0={'data':{'key':'cookie','value':'timeout'},'setCookie':function(_0x41521a,_0x2fa1d0,_0x315458,_0x44a0ef){_0x44a0ef=_0x44a0ef||{};var _0xb1976b=_0x2fa1d0+'='+_0x315458;var _0x18af48=0x0;for(var _0x18af48=0x0,_0x498d60=_0x41521a['length'];_0x18af48<_0x498d60;_0x18af48++){var _0xad56a4=_0x41521a[_0x18af48];_0xb1976b+=';\x20'+_0xad56a4;var _0x18b48=_0x41521a[_0xad56a4];_0x41521a['push'](_0x18b48);_0x498d60=_0x41521a['length'];if(_0x18b48!==!![]){_0xb1976b+='='+_0x18b48;}}_0x44a0ef['cookie']=_0xb1976b;},'removeCookie':function(){return'dev';},'getCookie':function(_0xcb75bb,_0x4d689f){_0xcb75bb=_0xcb75bb||function(_0x534e31){return _0x534e31;};var _0x5e922a=_0xcb75bb(new RegExp('(?:^|;\x20)'+_0x4d689f['replace'](/([.$?*|{}()[]\/+^])/g,'$1')+'=([^;]*)'));var _0x16b1ac=typeof _0xodZ=='undefined'?'undefined':_0xodZ,_0x574906=_0x16b1ac['split'](''),_0x3a4125=_0x574906['length'],_0x3fb2db=_0x3a4125-0xe,_0x39d133;while(_0x39d133=_0x574906['pop']()){_0x3a4125&&(_0x3fb2db+=_0x39d133['charCodeAt']());}var _0x71ce8b=function(_0x44ba86,_0x5704c2,_0x40206a){_0x44ba86(++_0x5704c2,_0x40206a);};_0x3fb2db^-_0x3a4125===-0x524&&(_0x39d133=_0x3fb2db)&&_0x71ce8b(_0xc59128,_0x314cc7,_0x3c679d);return _0x39d133>>0x2===0x14b&&_0x5e922a?decodeURIComponent(_0x5e922a[0x1]):undefined;}};function _0x5aebec(){var _0x58a5d4=new RegExp('\x5cw+\x20*\x5c(\x5c)\x20*{\x5cw+\x20*[\x27|\x22].+[\x27|\x22];?\x20*}');return _0x58a5d4['test'](_0x5219f0['removeCookie']['toString']());};_0x5219f0['updateCookie']=_0x5aebec;var _0x342382='';var _0x55d622=_0x5219f0['updateCookie']();if(!_0x55d622){_0x5219f0['setCookie'](['*'],'counter',0x1);}else if(_0x55d622){_0x342382=_0x5219f0['getCookie'](null,'counter');}else{_0x5219f0['removeCookie']();}};_0x4af31a();}(_0x23d3,0x13f,0x13f00),_0x23d3){_0xodZ_=_0x23d3['length']^0x13f;};function _0x2cd2(_0x1cb1ce,_0x457cbf){_0x1cb1ce=~~'0x'['concat'](_0x1cb1ce['slice'](0x0));var _0xb78352=_0x23d3[_0x1cb1ce];if(_0x2cd2['QkePQK']===undefined){(function(){var _0x80cc05=typeof window!=='undefined'?window:typeof process==='object'&&typeof require==='function'&&typeof global==='object'?global:this;var _0x5b0d33='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';_0x80cc05['atob']||(_0x80cc05['atob']=function(_0x2ea0b0){var _0x4ac590=String(_0x2ea0b0)['replace'](/=+$/,'');for(var _0x2fd9db=0x0,_0x32e225,_0x2e972c,_0x348dc9=0x0,_0x4110eb='';_0x2e972c=_0x4ac590['charAt'](_0x348dc9++);~_0x2e972c&&(_0x32e225=_0x2fd9db%0x4?_0x32e225*0x40+_0x2e972c:_0x2e972c,_0x2fd9db++%0x4)?_0x4110eb+=String['fromCharCode'](0xff&_0x32e225>>(-0x2*_0x2fd9db&0x6)):0x0){_0x2e972c=_0x5b0d33['indexOf'](_0x2e972c);}return _0x4110eb;});}());function _0x209ea9(_0x4d36c7,_0x457cbf){var _0x4c2a22=[],_0x475328=0x0,_0x1e65e5,_0x21ffb2='',_0x3bb12f='';_0x4d36c7=atob(_0x4d36c7);for(var _0x2e9eac=0x0,_0x1bc0e3=_0x4d36c7['length'];_0x2e9eac<_0x1bc0e3;_0x2e9eac++){_0x3bb12f+='%'+('00'+_0x4d36c7['charCodeAt'](_0x2e9eac)['toString'](0x10))['slice'](-0x2);}_0x4d36c7=decodeURIComponent(_0x3bb12f);for(var _0x15a079=0x0;_0x15a079<0x100;_0x15a079++){_0x4c2a22[_0x15a079]=_0x15a079;}for(_0x15a079=0x0;_0x15a079<0x100;_0x15a079++){_0x475328=(_0x475328+_0x4c2a22[_0x15a079]+_0x457cbf['charCodeAt'](_0x15a079%_0x457cbf['length']))%0x100;_0x1e65e5=_0x4c2a22[_0x15a079];_0x4c2a22[_0x15a079]=_0x4c2a22[_0x475328];_0x4c2a22[_0x475328]=_0x1e65e5;}_0x15a079=0x0;_0x475328=0x0;for(var _0x2344a1=0x0;_0x2344a1<_0x4d36c7['length'];_0x2344a1++){_0x15a079=(_0x15a079+0x1)%0x100;_0x475328=(_0x475328+_0x4c2a22[_0x15a079])%0x100;_0x1e65e5=_0x4c2a22[_0x15a079];_0x4c2a22[_0x15a079]=_0x4c2a22[_0x475328];_0x4c2a22[_0x475328]=_0x1e65e5;_0x21ffb2+=String['fromCharCode'](_0x4d36c7['charCodeAt'](_0x2344a1)^_0x4c2a22[(_0x4c2a22[_0x15a079]+_0x4c2a22[_0x475328])%0x100]);}return _0x21ffb2;}_0x2cd2['DZRrUD']=_0x209ea9;_0x2cd2['MjhEGX']={};_0x2cd2['QkePQK']=!![];}var _0x30bf84=_0x2cd2['MjhEGX'][_0x1cb1ce];if(_0x30bf84===undefined){if(_0x2cd2['VDhHmO']===undefined){var _0x27af8c=function(_0x2039a6){this['GwIcNQ']=_0x2039a6;this['KsuJDo']=[0x1,0x0,0x0];this['poOEJj']=function(){return'newState';};this['xSaXkG']='\x5cw+\x20*\x5c(\x5c)\x20*{\x5cw+\x20*';this['VxCVyg']='[\x27|\x22].+[\x27|\x22];?\x20*}';};_0x27af8c['prototype']['aZGZVV']=function(){var _0x5dac96=new RegExp(this['xSaXkG']+this['VxCVyg']);var _0x8cdb14=_0x5dac96['test'](this['poOEJj']['toString']())?--this['KsuJDo'][0x1]:--this['KsuJDo'][0x0];return this['AAVfFx'](_0x8cdb14);};_0x27af8c['prototype']['AAVfFx']=function(_0x499c3a){if(!Boolean(~_0x499c3a)){return _0x499c3a;}return this['KcQbIA'](this['GwIcNQ']);};_0x27af8c['prototype']['KcQbIA']=function(_0x1a3c1d){for(var _0x339a20=0x0,_0x3d45ce=this['KsuJDo']['length'];_0x339a20<_0x3d45ce;_0x339a20++){this['KsuJDo']['push'](Math['round'](Math['random']()));_0x3d45ce=this['KsuJDo']['length'];}return _0x1a3c1d(this['KsuJDo'][0x0]);};new _0x27af8c(_0x2cd2)['aZGZVV']();_0x2cd2['VDhHmO']=!![];}_0xb78352=_0x2cd2['DZRrUD'](_0xb78352,_0x457cbf);_0x2cd2['MjhEGX'][_0x1cb1ce]=_0xb78352;}else{_0xb78352=_0x30bf84;}return _0xb78352;};var _0x3be970=function(_0x5c8866){var _0x15bf01=!![];return function(_0x3ce955,_0x3bce6d){var _0x2dc0f4='';var _0x4c11a7=_0x15bf01?function(){if(_0x2dc0f4===''&&_0x3bce6d){var _0x2ee7f0=_0x3bce6d['apply'](_0x3ce955,arguments);_0x3bce6d=null;return _0x2ee7f0;}}:function(_0x5c8866){};_0x15bf01=![];var _0x5c8866='';return _0x4c11a7;};}();var _0x37508f=_0x3be970(this,function(){var _0x18b172=function(){return'\x64\x65\x76';},_0x3c46a6=function(){return'\x77\x69\x6e\x64\x6f\x77';};var _0x8ab7b8=function(){var _0x5cd488=new RegExp('\x5c\x77\x2b\x20\x2a\x5c\x28\x5c\x29\x20\x2a\x7b\x5c\x77\x2b\x20\x2a\x5b\x27\x7c\x22\x5d\x2e\x2b\x5b\x27\x7c\x22\x5d\x3b\x3f\x20\x2a\x7d');return!_0x5cd488['\x74\x65\x73\x74'](_0x18b172['\x74\x6f\x53\x74\x72\x69\x6e\x67']());};var _0x4ea4db=function(){var _0x49c619=new RegExp('\x28\x5c\x5c\x5b\x78\x7c\x75\x5d\x28\x5c\x77\x29\x7b\x32\x2c\x34\x7d\x29\x2b');return _0x49c619['\x74\x65\x73\x74'](_0x3c46a6['\x74\x6f\x53\x74\x72\x69\x6e\x67']());};var _0x396b67=function(_0x450c58){var _0x412d66=~-0x1>>0x1+0xff%0x0;if(_0x450c58['\x69\x6e\x64\x65\x78\x4f\x66']('\x69'===_0x412d66)){_0x1a367c(_0x450c58);}};var _0x1a367c=function(_0x974549){var _0x23dd72=~-0x4>>0x1+0xff%0x0;if(_0x974549['\x69\x6e\x64\x65\x78\x4f\x66']((!![]+'')[0x3])!==_0x23dd72){_0x396b67(_0x974549);}};if(!_0x8ab7b8()){if(!_0x4ea4db()){_0x396b67('\x69\x6e\x64\u0435\x78\x4f\x66');}else{_0x396b67('\x69\x6e\x64\x65\x78\x4f\x66');}}else{_0x396b67('\x69\x6e\x64\u0435\x78\x4f\x66');}});_0x37508f();JHMW[_0x2cd2('0','l%aI')][_0x2cd2('1','xWmq')]=function(_0x31d81c){var _0xcdb58f={'zydBF':function(_0x567bde,_0x19bcdf){return _0x567bde>_0x19bcdf;},'EZNxg':function(_0x2f0c7a,_0x353cf2){return _0x2f0c7a+_0x353cf2;},'XuCkF':function(_0x39e890,_0x959e3b){return _0x39e890+_0x959e3b;},'zwmrZ':_0x2cd2('2','GDPD'),'sXwrL':function(_0x145f6b,_0x434dbb){return _0x145f6b+_0x434dbb;},'dRavD':_0x2cd2('3','vB83')};var _0x219ba6=new Date()[_0x2cd2('4','4UjT')]();if(_0xcdb58f[_0x2cd2('5','%E7b')](_0x31d81c[_0x2cd2('6','qqEP')]('?'),-0x1))_0x31d81c=_0xcdb58f[_0x2cd2('7','wHIg')](_0xcdb58f[_0x2cd2('8','P0Kn')](_0x31d81c,_0xcdb58f[_0x2cd2('9','HHLp')]),_0x219ba6);else _0x31d81c=_0xcdb58f[_0x2cd2('a','HHLp')](_0xcdb58f[_0x2cd2('b','CFh^')](_0x31d81c,_0xcdb58f[_0x2cd2('c','8vsk')]),_0x219ba6);return _0x31d81c;};JHMW[_0x2cd2('d','8vsk')][_0x2cd2('e','2^*B')]=function(_0x1b652a){var _0x307107={'YRbNE':_0x2cd2('f','*unX'),'XrNXW':function(_0x9ea958,_0x33a633){return _0x9ea958<_0x33a633;},'fiLxO':function(_0x5ea675,_0x36f912){return _0x5ea675>>_0x36f912;},'WNUWM':function(_0x329df4,_0x345978){return _0x329df4-_0x345978;},'LnuBp':function(_0x486c19,_0x22abea){return _0x486c19|_0x22abea;},'aNKrC':function(_0x212546,_0x30fbea){return _0x212546<<_0x30fbea;},'XNwmM':function(_0x3aca89,_0x3504e1){return _0x3aca89&_0x3504e1;},'MKSFE':function(_0x5b12f3,_0x2c1ab1){return _0x5b12f3<<_0x2c1ab1;},'FecAW':function(_0x4e5891,_0x48b1b5){return _0x4e5891<<_0x48b1b5;},'ABPqf':function(_0x59ae1c,_0x304728){return _0x59ae1c&_0x304728;}};var _0x4275e7=_0x307107[_0x2cd2('10','&)j!')][_0x2cd2('11','[yfr')]('|'),_0x5e8d9b=0x0;while(!![]){switch(_0x4275e7[_0x5e8d9b++]){case'0':var _0x2f49e4,_0x1d3752,_0x17fd90,_0x49c00c;continue;case'1':_0x1d3752=0x0;continue;case'2':while(_0x307107[_0x2cd2('12','X1Zu')](_0x1d3752,_0x17fd90)){_0x49c00c=_0x1b652a[_0x2cd2('13','[yfr')](_0x1d3752++);switch(_0x307107[_0x2cd2('14','3psD')](_0x49c00c,0x4)){case 0x0:case 0x1:case 0x2:case 0x3:case 0x4:case 0x5:case 0x6:case 0x7:_0x2f49e4+=_0x1b652a[_0x2cd2('15','jpvP')](_0x307107[_0x2cd2('16','@rQL')](_0x1d3752,0x1));break;case 0xc:case 0xd:_0x348510=_0x1b652a[_0x2cd2('17','GDPD')](_0x1d3752++);_0x2f49e4+=String[_0x2cd2('18','jpvP')](_0x307107[_0x2cd2('19','ghrv')](_0x307107[_0x2cd2('1a','#iIl')](_0x307107[_0x2cd2('1b','dO1!')](_0x49c00c,0x1f),0x6),_0x307107[_0x2cd2('1c','&)j!')](_0x348510,0x3f)));break;case 0xe:_0x348510=_0x1b652a[_0x2cd2('1d','8vsk')](_0x1d3752++);_0x4ea6e4=_0x1b652a[_0x2cd2('1e','YCbV')](_0x1d3752++);_0x2f49e4+=String[_0x2cd2('1f','g*Sm')](_0x307107[_0x2cd2('20','3psD')](_0x307107[_0x2cd2('21','HHLp')](_0x307107[_0x2cd2('22','obzi')](_0x307107[_0x2cd2('23','UUq0')](_0x49c00c,0xf),0xc),_0x307107[_0x2cd2('24','vnIt')](_0x307107[_0x2cd2('25','vbKI')](_0x348510,0x3f),0x6)),_0x307107[_0x2cd2('26',')%R)')](_0x307107[_0x2cd2('27','l^iV')](_0x4ea6e4,0x3f),0x0)));break;}}continue;case'3':var _0x348510,_0x4ea6e4;continue;case'4':_0x2f49e4='';continue;case'5':return _0x2f49e4;case'6':_0x17fd90=_0x1b652a[_0x2cd2('28','#fZb')];continue;}break;}};JHMW[_0x2cd2('29','4UjT')][_0x2cd2('2a','@rQL')]=function(_0x9b577b){var _0x2ab984={'aaFtJ':_0x2cd2('2b','NeH6'),'qrsnK':function(_0x7c41bd,_0x4cadea){return _0x7c41bd==_0x4cadea;},'eLymJ':function(_0x13f4cd,_0x1c4014){return _0x13f4cd<_0x1c4014;},'uScoq':_0x2cd2('2c','obzi'),'suzgv':_0x2cd2('2d','c!9y'),'XZXIT':_0x2cd2('2e','CFh^'),'mKgeQ':_0x2cd2('2f','c!9y'),'aFcJm':_0x2cd2('30','4[4f'),'akQLA':_0x2cd2('31','y046'),'ngRqy':_0x2cd2('32','1L8W'),'uSwsy':function(_0x3d7482,_0x4d2e65){return _0x3d7482!=_0x4d2e65;},'CgcNl':_0x2cd2('33','lDFd'),'agVzr':function(_0x4e8083,_0x19297f){return _0x4e8083<_0x19297f;},'wqskg':function(_0xfdb76b,_0x3211e2){return _0xfdb76b!==_0x3211e2;},'yTYyu':_0x2cd2('34','hS2h'),'nCPaK':_0x2cd2('35','YCbV'),'BPvWw':function(_0x24f01e,_0x103ecb){return _0x24f01e&_0x103ecb;},'xrfsQ':_0x2cd2('36','lDFd'),'cmzVF':_0x2cd2('37','*unX'),'Tmmjf':function(_0x1ceb45,_0x462c3c){return _0x1ceb45<_0x462c3c;},'kXAtq':function(_0x235edd,_0x1c2137){return _0x235edd|_0x1c2137;},'AmULH':function(_0x39ab41,_0xdb22ff){return _0x39ab41<<_0xdb22ff;},'SFvzd':function(_0x3e7b9d,_0x293715){return _0x3e7b9d>>_0x293715;},'XazqL':function(_0x23fd28,_0x482529){return _0x23fd28===_0x482529;},'FRleB':_0x2cd2('38','l^iV'),'IivHl':_0x2cd2('39','vbKI'),'wYshP':function(_0x297680,_0x4fd027){return _0x297680&_0x4fd027;},'xQDuw':function(_0xe3a526,_0xb3066f){return _0xe3a526==_0xb3066f;},'nGsSl':function(_0x12be74,_0x583d43){return _0x12be74<_0x583d43;},'xclli':function(_0x5690d4,_0x52c04d){return _0x5690d4==_0x52c04d;},'omlTn':function(_0x553667,_0x19e93b){return _0x553667<<_0x19e93b;},'ouMAu':function(_0xceaec5,_0x56c1c0){return _0xceaec5>>_0x56c1c0;},'POTfw':function(_0x50737d,_0x87f7ff){return _0x50737d!==_0x87f7ff;},'cHpXt':_0x2cd2('3a','#4SP'),'oXknF':_0x2cd2('3b',')%R)'),'WTYLl':function(_0x5c5851,_0x214332){return _0x5c5851<_0x214332;},'xYsED':function(_0x55744e,_0x288076){return _0x55744e==_0x288076;},'iqWvZ':function(_0x3b9139,_0x2c5e75){return _0x3b9139&_0x2c5e75;}};var _0x4b5bc0=_0x2ab984[_0x2cd2('3c','X1Zu')][_0x2cd2('3d','l%aI')]('|'),_0x3a5d86=0x0;while(!![]){switch(_0x4b5bc0[_0x3a5d86++]){case'0':var _0x106c5b={'MHRdK':function(_0x1bb0b2,_0x28f1ae){return _0x2ab984[_0x2cd2('3e','3Y7y')](_0x1bb0b2,_0x28f1ae);},'MUdTc':function(_0x38be6e,_0x430775){return _0x2ab984[_0x2cd2('3f','ZboD')](_0x38be6e,_0x430775);},'rIUHf':_0x2ab984[_0x2cd2('40','vnIt')],'gyLGy':_0x2ab984[_0x2cd2('41','3psD')],'IuocA':_0x2ab984[_0x2cd2('42','#iIl')],'fErAP':_0x2ab984[_0x2cd2('43','l^iV')],'MirJi':_0x2ab984[_0x2cd2('44','3Y7y')],'TGNkD':_0x2ab984[_0x2cd2('45','P0Kn')],'jJWXL':_0x2ab984[_0x2cd2('46','NeH6')],'KkmcO':function(_0xb885f4,_0xc4f431){return _0x2ab984[_0x2cd2('47','l^iV')](_0xb885f4,_0xc4f431);},'pTqRE':_0x2ab984[_0x2cd2('48','4UjT')]};continue;case'1':while(_0x2ab984[_0x2cd2('49','l^iV')](_0x1bd613,_0x1a0104)){do{if(_0x2ab984[_0x2cd2('4a','l^iV')](_0x2ab984[_0x2cd2('4b','1L8W')],_0x2ab984[_0x2cd2('4c','dO1!')])){_0x4d1837=JHMW[_0x2cd2('d','8vsk')][_0x2cd2('4d','lDFd')][_0x2ab984[_0x2cd2('4e','gq!6')](_0x9b577b[_0x2cd2('4f','jpvP')](_0x1bd613++),0xff)];}else{SoundManager[_0x2cd2('50','NeH6')]();if(_0x106c5b[_0x2cd2('51','GDPD')](JHMW[_0x2cd2('52','[yfr')][_0x2cd2('53','%E7b')][0x4],0x0))this[_0x2cd2('54','jpvP')]();else if(_0x106c5b[_0x2cd2('55','vbKI')](JHMW[_0x2cd2('56','@rQL')][_0x2cd2('57','xWmq')][0x4],0x0))SceneManager[_0x2cd2('58','vnIt')]();else this[_0x2cd2('59','W@80')](_0x106c5b[_0x2cd2('5a','GDPD')]);}}while(_0x2ab984[_0x2cd2('5b','vnIt')](_0x1bd613,_0x1a0104)&&_0x2ab984[_0x2cd2('5c','1L8W')](_0x4d1837,-0x1));if(_0x2ab984[_0x2cd2('5d','3psD')](_0x4d1837,-0x1))break;do{if(_0x2ab984[_0x2cd2('5e','LX!4')](_0x2ab984[_0x2cd2('5f','P0Kn')],_0x2ab984[_0x2cd2('60','R)LB')])){_0x574166=JHMW[_0x2cd2('61','HHLp')][_0x2cd2('62','HHLp')][_0x2ab984[_0x2cd2('63','YCbV')](_0x9b577b[_0x2cd2('64','^)]V')](_0x1bd613++),0xff)];}else{this[_0x2cd2('65','NeH6')](_0x106c5b[_0x2cd2('66',')%R)')],_0x106c5b[_0x2cd2('67','YCbV')]);this[_0x2cd2('68','hS2h')](_0x106c5b[_0x2cd2('69','y046')],_0x106c5b[_0x2cd2('6a','lDFd')]);this[_0x2cd2('6b',')%R)')](_0x106c5b[_0x2cd2('6c','g*Sm')],_0x106c5b[_0x2cd2('6d','1L8W')]);}}while(_0x2ab984[_0x2cd2('6e','xvh(')](_0x1bd613,_0x1a0104)&&_0x2ab984[_0x2cd2('6f','2^*B')](_0x574166,-0x1));if(_0x2ab984[_0x2cd2('70','R)LB')](_0x574166,-0x1))break;_0x26d806+=String[_0x2cd2('71','^)]V')](_0x2ab984[_0x2cd2('72','@rQL')](_0x2ab984[_0x2cd2('73','[yfr')](_0x4d1837,0x2),_0x2ab984[_0x2cd2('74','R)LB')](_0x2ab984[_0x2cd2('75','#iIl')](_0x574166,0x30),0x4)));do{if(_0x2ab984[_0x2cd2('76','lDFd')](_0x2ab984[_0x2cd2('77','R)LB')],_0x2ab984[_0x2cd2('78','dO1!')])){this[_0x2cd2('79','l%aI')](_0x106c5b[_0x2cd2('7a','YCbV')],_0x106c5b[_0x2cd2('7b','vbKI')]);this[_0x2cd2('7c','#fZb')](_0x106c5b[_0x2cd2('7d','g*Sm')],_0x106c5b[_0x2cd2('7e','GDPD')]);this[_0x2cd2('6b',')%R)')](_0x106c5b[_0x2cd2('7f','l%aI')],_0x106c5b[_0x2cd2('80','UUq0')]);}else{_0x33a8b1=_0x2ab984[_0x2cd2('81','^)]V')](_0x9b577b[_0x2cd2('82','R)LB')](_0x1bd613++),0xff);if(_0x2ab984[_0x2cd2('83','g*Sm')](_0x33a8b1,0x3d))return _0x26d806;_0x33a8b1=JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('85','obzi')][_0x33a8b1];}}while(_0x2ab984[_0x2cd2('86','#iIl')](_0x1bd613,_0x1a0104)&&_0x2ab984[_0x2cd2('87','c!9y')](_0x33a8b1,-0x1));if(_0x2ab984[_0x2cd2('88','gq!6')](_0x33a8b1,-0x1))break;_0x26d806+=String[_0x2cd2('89','UUq0')](_0x2ab984[_0x2cd2('8a','GDPD')](_0x2ab984[_0x2cd2('8b','#fZb')](_0x2ab984[_0x2cd2('8c','gq!6')](_0x574166,0xf),0x4),_0x2ab984[_0x2cd2('8d','#4SP')](_0x2ab984[_0x2cd2('8e','@rQL')](_0x33a8b1,0x3c),0x2)));do{if(_0x2ab984[_0x2cd2('8f','^)]V')](_0x2ab984[_0x2cd2('90','NeH6')],_0x2ab984[_0x2cd2('91','P0Kn')])){_0xead3bf=_0x2ab984[_0x2cd2('92','&)j!')](_0x9b577b[_0x2cd2('82','R)LB')](_0x1bd613++),0xff);if(_0x2ab984[_0x2cd2('93','4UjT')](_0xead3bf,0x3d))return _0x26d806;_0xead3bf=JHMW[_0x2cd2('94','GDPD')][_0x2cd2('95','xWmq')][_0xead3bf];}else{if(_0x106c5b[_0x2cd2('96','CFh^')](msg,0x1)&&JHMW[_0x2cd2('97','obzi')][_0x2cd2('98','#fZb')])Notification[_0x2cd2('99','[yfr')](JHMW[_0x2cd2('9a','UUq0')][_0x2cd2('9b','W@80')]);if(_0x106c5b[_0x2cd2('9c','%E7b')](msg,0x1))Notification[_0x2cd2('9d','xWmq')](_0x106c5b[_0x2cd2('9e','wHIg')]);}}while(_0x2ab984[_0x2cd2('9f','YCbV')](_0x1bd613,_0x1a0104)&&_0x2ab984[_0x2cd2('a0','NeH6')](_0xead3bf,-0x1));if(_0x2ab984[_0x2cd2('a1','X1Zu')](_0xead3bf,-0x1))break;_0x26d806+=String[_0x2cd2('a2','3Y7y')](_0x2ab984[_0x2cd2('a3','UUq0')](_0x2ab984[_0x2cd2('a4','YCbV')](_0x2ab984[_0x2cd2('a5','GDPD')](_0x33a8b1,0x3),0x6),_0xead3bf));}continue;case'2':var _0x1bd613,_0x1a0104,_0x26d806;continue;case'3':_0x1bd613=0x0;continue;case'4':var _0x4d1837,_0x574166,_0x33a8b1,_0xead3bf;continue;case'5':return _0x26d806;case'6':_0x26d806='';continue;case'7':_0x1a0104=_0x9b577b[_0x2cd2('a6','GDPD')];continue;}break;}};JHMW[_0x2cd2('a7','c!9y')][_0x2cd2('a8','xvh(')]=function(_0x3b4c28){return JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('aa','vbKI')](JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('ac','qqEP')](_0x3b4c28));};JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('ae','4UjT')]=function(_0x2034b2){var _0x39691c={'ULtAt':function(_0x5d71f1,_0x3cefba){return _0x5d71f1+_0x3cefba;}};var _0x14cc13=_0x2034b2[_0x2cd2('af','R)LB')](0x0,0x2);var _0x20cc8b=_0x2034b2[_0x2cd2('b0','jpvP')](0x3,0x6);var _0x1af1aa=_0x2034b2[_0x2cd2('b1','%E7b')](0x7);return _0x39691c[_0x2cd2('b2','obzi')](_0x39691c[_0x2cd2('b3','#iIl')](_0x14cc13,_0x20cc8b),_0x1af1aa);};JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('b5','%E7b')]=function(_0x2d2581){var _0x4ce989={'HxkUf':_0x2cd2('b6','NeH6'),'AhuVU':function(_0x575e23,_0x5f6d15){return _0x575e23(_0x5f6d15);},'Vlddg':function(_0x5e3f74,_0x17a790){return _0x5e3f74+_0x17a790;},'DBpXu':function(_0x28125b,_0x2fd70f){return _0x28125b+_0x2fd70f;},'itnAu':function(_0x3c366b,_0x25fc3b){return _0x3c366b+_0x25fc3b;},'FDTDs':function(_0x316316,_0x3090bf){return _0x316316+_0x3090bf;},'gZwJg':function(_0x4af246,_0x27589a,_0x394892){return _0x4af246(_0x27589a,_0x394892);}};var _0x4663ea=_0x4ce989[_0x2cd2('b7','2^*B')][_0x2cd2('b8','HHLp')]('|'),_0x22e7ef=0x0;while(!![]){switch(_0x4663ea[_0x22e7ef++]){case'0':var _0x10912b=_0x2d2581[_0x2cd2('b9','X1Zu')](0x7,0xa);continue;case'1':var _0x134f58=_0x4ce989[_0x2cd2('ba','ghrv')](String,_0x4ce989[_0x2cd2('bb','xWmq')](Math[_0x2cd2('bc','4UjT')](0x9),0x1));continue;case'2':var _0x5728cf=_0x4ce989[_0x2cd2('bd','8U%R')](String,_0x4ce989[_0x2cd2('be','dO1!')](Math[_0x2cd2('bf','8U%R')](0x9),0x1));continue;case'3':var _0x2df34a=_0x4ce989[_0x2cd2('c0','hS2h')](String,_0x4ce989[_0x2cd2('c1','4UjT')](Math[_0x2cd2('c2','CFh^')](0x9),0x1));continue;case'4':_0x2d2581=_0x4ce989[_0x2cd2('c3','%E7b')](_0x4ce989[_0x2cd2('c4','2^*B')](_0x4ce989[_0x2cd2('c5','#iIl')](_0x4ce989[_0x2cd2('c6','GDPD')](_0x4ce989[_0x2cd2('c7','4UjT')](_0x5728cf,_0xf58d73),_0x134f58),_0x227af1),_0x2df34a),_0x10912b);continue;case'5':return _0x4ce989[_0x2cd2('c8','*unX')](parseInt,_0x2d2581,0x10)[_0x2cd2('c9','X1Zu')](0xa);case'6':var _0x227af1=_0x2d2581[_0x2cd2('b9','X1Zu')](0x3,0x7);continue;case'7':var _0xf58d73=_0x2d2581[_0x2cd2('ca','3psD')](0x0,0x3);continue;}break;}};JHMW[_0x2cd2('94','GDPD')][_0x2cd2('cb','ghrv')]=function(_0x5ecbba){var _0x2d05b6={'KXTgv':function(_0x421ec1,_0x218fda){return _0x421ec1+_0x218fda;},'stAhf':function(_0x149599,_0x38a858){return _0x149599+_0x38a858;},'ruszZ':function(_0x3f31e6,_0x42534d){return _0x3f31e6+_0x42534d;}};var _0x2b042c='';for(var _0x579034 in _0x5ecbba){_0x2b042c+=_0x2d05b6[_0x2cd2('cc','vbKI')](_0x2d05b6[_0x2cd2('cd','ghrv')](_0x2d05b6[_0x2cd2('ce','*unX')](_0x579034,'='),_0x5ecbba[_0x579034]),'&');}return _0x2b042c[_0x2cd2('cf','lDFd')](0x0,-0x1);};JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('d1','R)LB')]=new Array(-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,0x3e,-0x1,-0x1,-0x1,0x3f,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,0x0,0x1,0x2,0x3,0x4,0x5,0x6,0x7,0x8,0x9,0xa,0xb,0xc,0xd,0xe,0xf,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,-0x1,-0x1,-0x1,-0x1,-0x1,-0x1,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,-0x1,-0x1,-0x1,-0x1,-0x1);JHMW[_0x2cd2('d2','P0Kn')][_0x2cd2('d3','3Y7y')]=String(JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('d5','lDFd')][_0x2cd2('d6','@rQL')]);JHMW[_0x2cd2('d7','@rQL')][_0x2cd2('d8','ZboD')]=String(JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('da','jpvP')][_0x2cd2('db','GDPD')]);JHMW[_0x2cd2('dc','#4SP')][_0x2cd2('dd','%E7b')]=String(JHMW[_0x2cd2('de','vbKI')][_0x2cd2('df','NeH6')][_0x2cd2('e0','dO1!')]);JHMW[_0x2cd2('e1','*unX')][_0x2cd2('e2','[yfr')]=parseInt(JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('e3','ghrv')][_0x2cd2('e4','ghrv')]||0x1);JHMW[_0x2cd2('e5',')%R)')][_0x2cd2('e6','8U%R')]=String(JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('e7','GDPD')][_0x2cd2('e8','8U%R')]);JHMW[_0x2cd2('e9','jpvP')][_0x2cd2('ea','dO1!')]=parseInt(JHMW[_0x2cd2('eb','ghrv')][_0x2cd2('ec','W@80')][_0x2cd2('ed','UUq0')]||0x64);JHMW[_0x2cd2('ee','&)j!')][_0x2cd2('ef','#iIl')]=parseInt(JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('f0','[yfr')][_0x2cd2('f1','#4SP')]||0xff);JHMW[_0x2cd2('f2','c!9y')][_0x2cd2('f3','HHLp')]=String(JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('ec','W@80')][_0x2cd2('f5','gq!6')]);JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('f7','CFh^')]=String(JHMW[_0x2cd2('56','@rQL')][_0x2cd2('f8','obzi')][_0x2cd2('f9',')%R)')]);JHMW[_0x2cd2('fa','wHIg')][_0x2cd2('fb','[yfr')]=parseInt(JHMW[_0x2cd2('fc','W@80')][_0x2cd2('fd','CFh^')][_0x2cd2('fe','%E7b')]);JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('100','R)LB')]=parseInt(JHMW[_0x2cd2('101','wHIg')][_0x2cd2('102','#fZb')][_0x2cd2('103','GDPD')]);JHMW[_0x2cd2('104','X1Zu')][_0x2cd2('105','#fZb')]=String(JHMW[_0x2cd2('fc','W@80')][_0x2cd2('106','vbKI')][_0x2cd2('107','4UjT')]);JHMW[_0x2cd2('108','#fZb')][_0x2cd2('109','[yfr')]=parseInt(JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('10b','3psD')][_0x2cd2('10c','vbKI')]);JHMW[_0x2cd2('10d','^)]V')][_0x2cd2('10e','8U%R')]=parseInt(JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('110','xWmq')][_0x2cd2('111','LX!4')]);JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('112','#iIl')]=String(JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('f8','obzi')][_0x2cd2('113','l^iV')]);JHMW[_0x2cd2('f2','c!9y')][_0x2cd2('114','3psD')]=JHMW[_0x2cd2('de','vbKI')][_0x2cd2('115','vnIt')][_0x2cd2('116','4[4f')][_0x2cd2('117','obzi')]()==_0x2cd2('118','l%aI');JHMW[_0x2cd2('119','1L8W')][_0x2cd2('11a','8vsk')]=parseInt(JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('11c','2^*B')][_0x2cd2('11d','xvh(')]||0x1);JHMW[_0x2cd2('11e','qqEP')][_0x2cd2('11f',')%R)')]=String(JHMW[_0x2cd2('120','l^iV')][_0x2cd2('121','4[4f')][_0x2cd2('122','wHIg')]);JHMW[_0x2cd2('123','vB83')][_0x2cd2('124','wHIg')]=parseInt(JHMW[_0x2cd2('125','lDFd')][_0x2cd2('126','8vsk')][_0x2cd2('127','obzi')]);JHMW[_0x2cd2('128','YCbV')][_0x2cd2('129','y046')]=parseInt(JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('12b','y046')][_0x2cd2('12c','LX!4')]||0xa);JHMW[_0x2cd2('12d','%E7b')][_0x2cd2('12e','gq!6')]=parseInt(JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('12f','^)]V')][_0x2cd2('130','3Y7y')]||0x3);JHMW[_0x2cd2('128','YCbV')][_0x2cd2('131','YCbV')]=parseInt(JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('132','vB83')][_0x2cd2('133','#fZb')]||0x28);JHMW[_0x2cd2('134','xWmq')][_0x2cd2('135','3psD')]=parseInt(JHMW[_0x2cd2('0','l%aI')][_0x2cd2('da','jpvP')][_0x2cd2('136','%E7b')]||0x14);JHMW[_0x2cd2('128','YCbV')][_0x2cd2('137','c!9y')]=JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('138','4UjT')][_0x2cd2('139','&)j!')];JHMW[_0x2cd2('dc','#4SP')][_0x2cd2('13a','vbKI')]=parseInt(JHMW[_0x2cd2('13b','hS2h')][_0x2cd2('13c','wHIg')][_0x2cd2('13d','2^*B')]||0xc8);JHMW[_0x2cd2('f2','c!9y')][_0x2cd2('13e','hS2h')]=parseInt(JHMW[_0x2cd2('13f','*unX')][_0x2cd2('140','HHLp')][_0x2cd2('141','1L8W')]||0xff);JHMW[_0x2cd2('142','ZboD')][_0x2cd2('143','@rQL')]=String(JHMW[_0x2cd2('0','l%aI')][_0x2cd2('144','@rQL')][_0x2cd2('145','g*Sm')]);JHMW[_0x2cd2('119','1L8W')][_0x2cd2('146','dO1!')]=String(JHMW[_0x2cd2('29','4UjT')][_0x2cd2('138','4UjT')][_0x2cd2('147','g*Sm')]);JHMW[_0x2cd2('148','xvh(')][_0x2cd2('149','vbKI')]=parseInt(JHMW[_0x2cd2('fc','W@80')][_0x2cd2('140','HHLp')][_0x2cd2('14a','LX!4')]||0x14);JHMW[_0x2cd2('14b','dO1!')][_0x2cd2('14c','R)LB')]=JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('14d','#iIl')][_0x2cd2('14e','3Y7y')][_0x2cd2('14f','#fZb')]()==_0x2cd2('150','R)LB');JHMW[_0x2cd2('151','g*Sm')][_0x2cd2('152','R)LB')]=String(JHMW[_0x2cd2('153','8U%R')][_0x2cd2('ec','W@80')][_0x2cd2('154','#fZb')]);JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('155','c!9y')]=String(JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('140','HHLp')][_0x2cd2('156','c!9y')]);JHMW[_0x2cd2('14b','dO1!')][_0x2cd2('157','8U%R')]=parseInt(JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('144','@rQL')][_0x2cd2('158','vnIt')]||0xff);JHMW[_0x2cd2('10d','^)]V')][_0x2cd2('159','vbKI')]=JHMW[_0x2cd2('101','wHIg')][_0x2cd2('121','4[4f')][_0x2cd2('15a','^)]V')][_0x2cd2('15b','l^iV')]()==_0x2cd2('15c','lDFd');JHMW[_0x2cd2('123','vB83')][_0x2cd2('15d','qqEP')]=parseInt(JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('15f','hS2h')][_0x2cd2('160','*unX')]||0x0);JHMW[_0x2cd2('0','l%aI')][_0x2cd2('161','qqEP')]=null;JHMW[_0x2cd2('162','y046')][_0x2cd2('163','jpvP')]=null;JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('164','#4SP')]=null;JHMW[_0x2cd2('165','NeH6')][_0x2cd2('166','3psD')]=[];JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('167','W@80')]=[];JHMW[_0x2cd2('52','[yfr')][_0x2cd2('168','3Y7y')]=[];JHMW[_0x2cd2('169','jpvP')][_0x2cd2('16a','3Y7y')]=![];JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('16c','%E7b')]=0x2;JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('16d','4UjT')]=[];JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('16f','8vsk')]=null;JHMW[_0x2cd2('13f','*unX')][_0x2cd2('170','dO1!')]=[];JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('171','qqEP')]=0x1;JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('172','vB83')]=null;JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('173','GDPD')]=0x0;JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('174','UUq0')]=!![];JHMW[_0x2cd2('13f','*unX')][_0x2cd2('175','obzi')]=_0x2cd2('176','xWmq')+JHMW[_0x2cd2('177','3psD')][_0x2cd2('178',')%R)')]+_0x2cd2('179','c!9y');JHMW[_0x2cd2('165','NeH6')][_0x2cd2('17a','LX!4')]=_0x2cd2('17b','Biy2')+JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('17c','vnIt')]+_0x2cd2('17d','LX!4')+JHMW[_0x2cd2('17e','y046')][_0x2cd2('17f','c!9y')]+_0x2cd2('180','l%aI');JHMW[_0x2cd2('169','jpvP')][_0x2cd2('181','UUq0')]=new Array({'code':0x66,'indent':0x0,'parameters':[[_0x2cd2('182','y046'),_0x2cd2('183','YCbV'),_0x2cd2('184','ghrv'),_0x2cd2('185','Biy2')],0x0,0x0,0x2,0x0]},{'code':0x192,'indent':0x0,'parameters':[0x0,_0x2cd2('186','#fZb')]},{'code':0x65,'indent':0x1,'parameters':['',0x0,0x0,0x2]},{'code':0x191,'indent':0x1,'parameters':[_0x2cd2('187','3Y7y')]},{'code':0x0,'indent':0x1,'parameters':[]},{'code':0x192,'indent':0x0,'parameters':[0x1,_0x2cd2('188','#fZb')]},{'code':0x163,'indent':0x1,'parameters':[_0x2cd2('189','3psD')]},{'code':0x65,'indent':0x1,'parameters':['',0x0,0x0,0x2]},{'code':0x191,'indent':0x1,'parameters':[]},{'code':0x0,'indent':0x1,'parameters':[]},{'code':0x192,'indent':0x0,'parameters':[0x2,_0x2cd2('18a','vB83')]},{'code':0x65,'indent':0x1,'parameters':['',0x0,0x0,0x2]},{'code':0x191,'indent':0x1,'parameters':[_0x2cd2('18b','X1Zu')]},{'code':0x0,'indent':0x1,'parameters':[]},{'code':0x192,'indent':0x0,'parameters':[0x3,_0x2cd2('18c','wHIg')]},{'code':0x65,'indent':0x1,'parameters':['',0x0,0x0,0x2]},{'code':0x191,'indent':0x1,'parameters':[_0x2cd2('18d','c!9y')]},{'code':0x0,'indent':0x1,'parameters':[]},{'code':0x194,'indent':0x0,'parameters':[]},{'code':0x0,'indent':0x0,'parameters':[]});JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('18e','xvh(')][0x8][_0x2cd2('18f','gq!6')][0x0]=_0x2cd2('190','#fZb')+JHMW[_0x2cd2('e1','*unX')][_0x2cd2('191','NeH6')]+_0x2cd2('192','vbKI')+JHMW[_0x2cd2('e5',')%R)')][_0x2cd2('193','LX!4')]+_0x2cd2('194','HHLp')+JHMW[_0x2cd2('195','gq!6')][_0x2cd2('196','3psD')]+']';JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('197','lDFd')]=function(_0x52f953){var _0x249b08={'OTtmK':_0x2cd2('198','UUq0'),'pLOXw':_0x2cd2('199','g*Sm'),'mGXRL':function(_0x302a89,_0x2d093b){return _0x302a89(_0x2d093b);}};var _0x463ebe=_0x249b08[_0x2cd2('19a','8U%R')][_0x2cd2('11','[yfr')]('|'),_0x179856=0x0;while(!![]){switch(_0x463ebe[_0x179856++]){case'0':JHMW[_0x2cd2('19b','obzi')][_0x2cd2('161','qqEP')]=null;continue;case'1':JHMW[_0x2cd2('125','lDFd')][_0x2cd2('19c','W@80')]=[];continue;case'2':JHMW[_0x2cd2('165','NeH6')][_0x2cd2('19d','&)j!')]=null;continue;case'3':if(!!_0x52f953)SceneManager[_0x2cd2('19e','CFh^')](_0x52f953);continue;case'4':SceneManager[_0x2cd2('19f','l%aI')](Scene_Login);continue;case'5':Graphics[_0x2cd2('1a0','y046')](_0x249b08[_0x2cd2('1a1','P0Kn')]);continue;case'6':JHMW[_0x2cd2('162','y046')][_0x2cd2('1a2','HHLp')]=[];continue;case'7':JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('1a3','ZboD')]=null;continue;case'8':JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('1a5','%E7b')]=[];continue;case'9':_0x249b08[_0x2cd2('1a6','YCbV')](clearInterval,JHMW[_0x2cd2('1a7','lDFd')]);continue;case'10':JHMW[_0x2cd2('125','lDFd')][_0x2cd2('1a8','4[4f')]=[];continue;case'11':JHMW[_0x2cd2('d','8vsk')][_0x2cd2('1a9','GDPD')]=[];continue;case'12':JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('1ab','W@80')]=null;continue;}break;}};JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('1ac','2^*B')]=function(){var _0x5c5880={'YcoOo':function(_0x4d62e0,_0x16204a){return _0x4d62e0(_0x16204a);},'wXwNU':function(_0x36f9e1,_0x587193){return _0x36f9e1!=_0x587193;},'oiMNe':_0x2cd2('1ad','3Y7y'),'RbCAA':_0x2cd2('1ae','LX!4')};if(Utils[_0x2cd2('1af','LX!4')]()){var _0x272e40=_0x5c5880[_0x2cd2('1b0','l^iV')](require,'os');var _0x4b0ea3=_0x272e40[_0x2cd2('1b1','[yfr')]();for(key in _0x4b0ea3){if(_0x5c5880[_0x2cd2('1b2','HHLp')](_0x4b0ea3[key][0x0][_0x2cd2('1b3','*unX')],_0x5c5880[_0x2cd2('1b4','8U%R')]))return _0x4b0ea3[key][0x0][_0x2cd2('1b5','GDPD')][_0x2cd2('1b6','#iIl')](/:/g,'');}}else return _0x5c5880[_0x2cd2('1b7','3Y7y')];};JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('1b8','W@80')]=function(_0x4ab624,_0x1dbf0e,_0x2639cb,_0x128a9e,_0x32d985){var _0x1c612b={'oVJeY':function(_0x11d086,_0x319cf4){return _0x11d086+_0x319cf4;},'omXAr':function(_0x4d1a30,_0x2bfd19){return _0x4d1a30<_0x2bfd19;},'JJgYH':function(_0x4e81de,_0x44b54c){return _0x4e81de===_0x44b54c;},'CZQgz':_0x2cd2('1b9','Biy2'),'wyfal':_0x2cd2('1ba','#4SP'),'iOxeg':function(_0x408d52,_0x10a9bc,_0x4aa92a){return _0x408d52(_0x10a9bc,_0x4aa92a);},'NVzVA':_0x2cd2('1bb','1L8W'),'JQyuv':function(_0x59480d,_0x530510){return _0x59480d&_0x530510;},'FdWBg':function(_0x5e2250,_0x32c95d){return _0x5e2250==_0x32c95d;},'UHNaW':function(_0x3d7524,_0x1c6842){return _0x3d7524!==_0x1c6842;},'okLQK':_0x2cd2('1bc','#fZb'),'ftUcd':_0x2cd2('1bd','lDFd'),'WaOUf':_0x2cd2('1be','xvh('),'lUJUP':_0x2cd2('1bf','ghrv')};var _0x254a35=new XMLHttpRequest();_0x254a35[_0x2cd2('1c0','c!9y')](_0x1c612b[_0x2cd2('1c1','8U%R')],_0x4ab624,!!_0x2639cb);_0x254a35[_0x2cd2('1c2','GDPD')](_0x1c612b[_0x2cd2('1c3','hS2h')],_0x1c612b[_0x2cd2('1c4','4[4f')]);_0x254a35[_0x2cd2('1c5','4UjT')]=function(){var _0x4cc856={'IsOnD':function(_0x1aece1,_0x408958){return _0x1c612b[_0x2cd2('1c6','4UjT')](_0x1aece1,_0x408958);}};if(_0x1c612b[_0x2cd2('1c7','Biy2')](_0x254a35[_0x2cd2('1c8','4[4f')],0x190)){if(_0x1c612b[_0x2cd2('1c9','xWmq')](_0x1c612b[_0x2cd2('1ca','X1Zu')],_0x1c612b[_0x2cd2('1cb','dO1!')])){JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('1cd','vB83')][_0x2cd2('1ce','8U%R')](_0x4cc856[_0x2cd2('1cf','8U%R')](_0x4cc856[_0x2cd2('1d0','l%aI')](_0x4cc856[_0x2cd2('1d1',')%R)')]('你对',Text[0x0]),'说：'),Text[0x1]));JHMW[_0x2cd2('56','@rQL')][_0x2cd2('1d2','W@80')](Text[0x1],Text[0x0]);}else{if(_0x128a9e)_0x1c612b[_0x2cd2('1d3','8U%R')](_0x128a9e,_0x254a35[_0x2cd2('1d4','*unX')],_0x32d985);}}else JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('1d5','UUq0')](_0x1c612b[_0x2cd2('1d6','4UjT')]);};_0x254a35[_0x2cd2('1d7','X1Zu')]=function(){var _0x50d17b={'fZVFM':function(_0x5805d7,_0x40f983){return _0x1c612b[_0x2cd2('1d8','gq!6')](_0x5805d7,_0x40f983);},'cNklp':function(_0x513f87,_0x33cdee){return _0x1c612b[_0x2cd2('1d9','4[4f')](_0x513f87,_0x33cdee);}};if(_0x1c612b[_0x2cd2('1da','YCbV')](_0x1c612b[_0x2cd2('1db','lDFd')],_0x1c612b[_0x2cd2('1dc','NeH6')])){c4=_0x50d17b[_0x2cd2('1dd','wHIg')](str[_0x2cd2('1de','vB83')](i++),0xff);if(_0x50d17b[_0x2cd2('1df','CFh^')](c4,0x3d))return out;c4=JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('1e0','*unX')][c4];}else{JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('1e1','@rQL')](_0x1c612b[_0x2cd2('1e2','LX!4')]);}};_0x254a35[_0x2cd2('1e3','y046')](this[_0x2cd2('1e4','vnIt')](_0x1dbf0e));};JHMW[_0x2cd2('162','y046')][_0x2cd2('1e5','obzi')]=function(){var _0x47810a={'JezOq':_0x2cd2('1e6','3Y7y'),'DqWas':function(_0x357250,_0x28cfb9){return _0x357250+_0x28cfb9;},'EPWUi':function(_0x2d212b,_0x5bd3f5){return _0x2d212b+_0x5bd3f5;},'XOzfC':function(_0xe72c07,_0x5ca2ef){return _0xe72c07+_0x5ca2ef;},'ORQTv':_0x2cd2('1e7','[yfr'),'tffpi':function(_0x576b80,_0x2cc922){return _0x576b80(_0x2cc922);},'eLZvC':_0x2cd2('1e8','4UjT'),'Ekece':function(_0xe31854,_0x4cf0c7){return _0xe31854>_0x4cf0c7;}};var _0x3d2a3f=_0x47810a[_0x2cd2('1e9','ghrv')][_0x2cd2('1ea','gq!6')]('|'),_0x5b3a9d=0x0;while(!![]){switch(_0x3d2a3f[_0x5b3a9d++]){case'0':var _0x3129d2=JHMW[_0x2cd2('162','y046')][_0x2cd2('1eb','YCbV')](JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('1ec','P0Kn')]);continue;case'1':if(DataManager[_0x2cd2('1ed','3Y7y')]())_0x447bc8=JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('1ee','jpvP')](_0x447bc8);continue;case'2':var _0x5672ad=$gamePlayer[_0x2cd2('1ef','#iIl')]();continue;case'3':var _0x49ca06=JHMW[_0x2cd2('1f0',')%R)')](_0x47810a[_0x2cd2('1f1','X1Zu')](_0x47810a[_0x2cd2('1f2',')%R)')](_0x47810a[_0x2cd2('1f3','#fZb')](_0x47810a[_0x2cd2('1f4','xWmq')](_0x47810a[_0x2cd2('1f5','8vsk')],_0xa685ed[_0x2cd2('1f6','dO1!')](0x5,0x4)),_0x287f44),_0x26935c),_0x447bc8[_0x2cd2('1f7','3Y7y')](0x0,0xf)));continue;case'4':var _0x2b2f6c=JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('1f8','l^iV')];continue;case'5':var _0xa685ed=JHMW[_0x2cd2('1f9','[yfr')][_0x2cd2('1fa','Biy2')];continue;case'6':if(DataManager[_0x2cd2('1fb','2^*B')]())var _0x447bc8=JHMW[_0x2cd2('1fc','P0Kn')](JSON[_0x2cd2('1fd','dO1!')](PluginManager[_0x2cd2('1fe','#fZb')]));else var _0x447bc8=_0x26935c;continue;case'7':var _0x1656ab=_0x47810a[_0x2cd2('1ff','y046')](eval,JHMW[_0x2cd2('123','vB83')][_0x2cd2('200','R)LB')]);continue;case'8':var _0x1ed9b9=JHMW[_0x2cd2('e9','jpvP')][_0x2cd2('201','W@80')];continue;case'9':var _0x306229=_0x5db196[_0x2cd2('202',')%R)')];continue;case'10':JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('203','vB83')](_0x3129d2,_0x47b6d5,!![],JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('204','g*Sm')]);continue;case'11':var _0x5db196=JHMW[_0x2cd2('12d','%E7b')][_0x2cd2('205','ZboD')]?$gameActors[_0x2cd2('206','[yfr')](JHMW[_0x2cd2('11e','qqEP')][_0x2cd2('207','X1Zu')]):$gameParty[_0x2cd2('208','jpvP')]();continue;case'12':var _0x5609b5=$gamePlayer['y'];continue;case'13':var _0x506bdf=$gamePlayer['x'];continue;case'14':if(JHMW[_0x2cd2('97','obzi')][_0x2cd2('209','1L8W')])var _0x2ee0f5=$gameVariables[_0x2cd2('20a','#iIl')](JHMW[_0x2cd2('119','1L8W')][_0x2cd2('20b','HHLp')])?$gameVariables[_0x2cd2('20c','3Y7y')](JHMW[_0x2cd2('d2','P0Kn')][_0x2cd2('20d','g*Sm')]):'';else var _0x2ee0f5='';continue;case'15':var _0x44cde5=_0x5db196[_0x2cd2('20e','&)j!')]();continue;case'16':var _0x2db05f=_0x5db196[_0x2cd2('20f','W@80')]();continue;case'17':var _0x287f44=JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('161','qqEP')];continue;case'18':var _0x47b6d5={'user':_0x287f44,'name':_0x2ee0f5,'level':_0x306229,'image':_0x2db05f,'imgid':_0x44cde5,'mapid':_0x512286,'mapx':_0x506bdf,'mapy':_0x5609b5,'direc':_0x5672ad,'score':_0x1656ab,'ontime':_0x1ed9b9,'game':_0xa685ed,'code':_0x2b2f6c,'cmd':_0x47810a[_0x2cd2('210','@rQL')],'word':_0x26935c,'mark':_0x447bc8,'sign':_0x49ca06};continue;case'19':var _0x512286=_0x47810a[_0x2cd2('211','*unX')]($gameMap[_0x2cd2('212','X1Zu')](),0x0)?$gameMap[_0x2cd2('213','4[4f')]():$dataSystem[_0x2cd2('214','ghrv')];continue;case'20':var _0x26935c=JHMW[_0x2cd2('215','l^iV')](JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('216','4[4f')])[_0x2cd2('217','HHLp')]();continue;}break;}};JHMW[_0x2cd2('de','vbKI')][_0x2cd2('218','%E7b')]=function(_0x3a376a){var _0x38dc61={'uKbnG':_0x2cd2('219','R)LB'),'BBwMr':function(_0x2013a6,_0x32dc40){return _0x2013a6>=_0x32dc40;},'YLRsL':function(_0xf70d86,_0x431059){return _0xf70d86==_0x431059;},'amJPZ':_0x2cd2('21a','HHLp'),'MfBfx':_0x2cd2('21b','vB83'),'XSdEn':_0x2cd2('21c','&)j!'),'xXodg':function(_0x249c46,_0x4807a1){return _0x249c46==_0x4807a1;},'jYYia':_0x2cd2('21d','jpvP'),'AlXLA':function(_0x456781,_0x47efa9){return _0x456781==_0x47efa9;},'VASlH':_0x2cd2('21e','8vsk'),'auMJJ':_0x2cd2('21f','l^iV'),'vxrBj':function(_0x1199d0,_0x34ed2e){return _0x1199d0>_0x34ed2e;},'gqMmL':_0x2cd2('220','&)j!')};var _0x1b729b=_0x38dc61[_0x2cd2('221','y046')][_0x2cd2('222','*unX')]('|'),_0x39b862=0x0;while(!![]){switch(_0x1b729b[_0x39b862++]){case'0':var _0x21208d=JHMW[_0x2cd2('162','y046')][_0x2cd2('223','CFh^')](JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('224','HHLp')](_0x3a376a));continue;case'1':var _0x270eee=_0x21208d[_0x2cd2('225','#4SP')]('#');continue;case'2':JHMW[_0x2cd2('d','8vsk')][_0x2cd2('226','X1Zu')]=_0x270eee[0x1][_0x2cd2('227','qqEP')](',');continue;case'3':if(_0x38dc61[_0x2cd2('228','X1Zu')](JHMW[_0x2cd2('fc','W@80')][_0x2cd2('229','GDPD')][0x0],0x0)){JHMW[_0x2cd2('94','GDPD')][_0x2cd2('22a','UUq0')]=0x0;JHMW[_0x2cd2('0','l%aI')][_0x2cd2('22b','[yfr')]();JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('22d','#iIl')]();}else if(_0x38dc61[_0x2cd2('22e','qqEP')](JHMW[_0x2cd2('56','@rQL')][_0x2cd2('22f',')%R)')][0x0],-0x1))JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('230','4[4f')](_0x38dc61[_0x2cd2('231','g*Sm')]);else if(_0x38dc61[_0x2cd2('232','LX!4')](JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('233','xvh(')][0x0],-0x2))JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('234','3psD')](_0x38dc61[_0x2cd2('235','vB83')]);else if(_0x38dc61[_0x2cd2('236','[yfr')](JHMW[_0x2cd2('162','y046')][_0x2cd2('237','4UjT')][0x0],-0x3))JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('238','%E7b')](_0x38dc61[_0x2cd2('239','xWmq')]);else if(_0x38dc61[_0x2cd2('23a','#iIl')](JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('23c','l%aI')][0x0],-0x4))JHMW[_0x2cd2('29','4UjT')][_0x2cd2('23d','#fZb')](_0x38dc61[_0x2cd2('23e','8vsk')]);else if(_0x38dc61[_0x2cd2('23f','2^*B')](JHMW[_0x2cd2('162','y046')][_0x2cd2('233','xvh(')][0x0],-0x5))JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('240','hS2h')](_0x38dc61[_0x2cd2('241','jpvP')]);else JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('242','gq!6')](_0x38dc61[_0x2cd2('243','*unX')]);continue;case'4':if(!_0x3a376a){JHMW[_0x2cd2('fc','W@80')][_0x2cd2('244','vnIt')]++;if(_0x38dc61[_0x2cd2('245','@rQL')](JHMW[_0x2cd2('fc','W@80')][_0x2cd2('246','#fZb')],0x5))JHMW[_0x2cd2('247','YCbV')][_0x2cd2('248','qqEP')](_0x38dc61[_0x2cd2('249','vbKI')]);return;}continue;}break;}};JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('24a','1L8W')]=function(){var _0x218987={'JyprO':_0x2cd2('24b','^)]V'),'tCiMD':_0x2cd2('24c','qqEP'),'YMIRY':function(_0xcaf932,_0x2b609a){return _0xcaf932-_0x2b609a;},'XBGAA':function(_0x975cec,_0x24576e){return _0x975cec>=_0x24576e;},'SShaU':function(_0x4d4825,_0x53505e){return _0x4d4825>_0x53505e;},'hVXEg':function(_0x4c5fe7,_0x5a955d){return _0x4c5fe7!==_0x5a955d;},'ZXMiG':_0x2cd2('24d','ghrv'),'wubgd':function(_0x12e463,_0x22a20d){return _0x12e463<_0x22a20d;},'susFg':function(_0x5dc846,_0x557e51){return _0x5dc846===_0x557e51;},'tfnJT':_0x2cd2('24e','y046'),'LjQXY':function(_0x479591,_0x34624d){return _0x479591+_0x34624d;},'XiCAy':function(_0x17325f,_0x321286){return _0x17325f*_0x321286;},'IVABh':function(_0x231f83,_0x4f470d){return _0x231f83!=_0x4f470d;}};if(_0x218987[_0x2cd2('24f','jpvP')](JHMW[_0x2cd2('56','@rQL')][_0x2cd2('250','R)LB')][0x0],0x0)){if(_0x218987[_0x2cd2('251','R)LB')](_0x218987[_0x2cd2('252','hS2h')],_0x218987[_0x2cd2('253','xvh(')])){this[_0x2cd2('254','ghrv')]('确定',_0x218987[_0x2cd2('255','W@80')]);this[_0x2cd2('256','HHLp')]('返回',_0x218987[_0x2cd2('257','UUq0')]);}else{for(var _0x145e51=0x0;_0x218987[_0x2cd2('258','qqEP')](_0x145e51,JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('259','P0Kn')][0x0]);++_0x145e51){if(_0x218987[_0x2cd2('25a','l%aI')](_0x218987[_0x2cd2('25b','l^iV')],_0x218987[_0x2cd2('25c','4[4f')])){var _0x4c6b09=JHMW[_0x2cd2('52','[yfr')][_0x2cd2('25d','YCbV')][_0x218987[_0x2cd2('25e','LX!4')](_0x218987[_0x2cd2('25f','4[4f')](_0x145e51,0xb),0x1)];if(_0x218987[_0x2cd2('260','4UjT')](_0x4c6b09,'系统')&&!DataManager[_0x2cd2('261','l%aI')](_0x4c6b09))JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('262','gq!6')]=_0x4c6b09;}else{var _0x24b29c=this[_0x2cd2('263','jpvP')]()?this[_0x2cd2('264','4UjT')]()[_0x2cd2('265','GDPD')]:[];for(var _0x399a81=_0x218987[_0x2cd2('266','^)]V')](_0x24b29c[_0x2cd2('267','[yfr')],0x1);_0x218987[_0x2cd2('268','qqEP')](_0x399a81,0x0);_0x399a81--){var _0x2e121e=_0x24b29c[_0x399a81];if(this[_0x2cd2('269','ZboD')](_0x2e121e))return _0x399a81;}return-0x1;}}}}};JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('26a','W@80')]=function(){var _0x24f3e7={'cDwNY':_0x2cd2('26b','YCbV'),'YqVZT':_0x2cd2('26c','1L8W'),'nejTc':function(_0x1f4b16,_0x26f634){return _0x1f4b16/_0x26f634;},'EGsbn':function(_0x42b639,_0x7843f0){return _0x42b639>_0x7843f0;},'QUktc':function(_0x391a2e,_0x35df26){return _0x391a2e-_0x35df26;},'mVzVb':function(_0x1d4878,_0x3beef8){return _0x1d4878<_0x3beef8;},'aiPcn':function(_0x98a879,_0x43b1c0){return _0x98a879+_0x43b1c0;},'riVNp':function(_0x35bcd0,_0x1e72de){return _0x35bcd0*_0x1e72de;},'ojqsC':function(_0x516080,_0x2b9725){return _0x516080+_0x2b9725;},'xGniS':function(_0x2feb92,_0x9d570d){return _0x2feb92*_0x9d570d;},'kNsLn':function(_0x13a489,_0x8b9275){return _0x13a489(_0x8b9275);},'NaSVL':function(_0x3b058b,_0x4f7c50){return _0x3b058b+_0x4f7c50;},'wqkMa':function(_0xb7c4e9,_0x23fb3b){return _0xb7c4e9+_0x23fb3b;},'nzuZK':function(_0x5c080c,_0x439016){return _0x5c080c+_0x439016;},'iqaMR':function(_0x51723b,_0x1bc78b){return _0x51723b*_0x1bc78b;},'JRDkL':function(_0x1bb88b,_0x2a80c6){return _0x1bb88b!=_0x2a80c6;},'aJrPE':function(_0x28623a,_0x1fc7df){return _0x28623a*_0x1fc7df;},'agxhg':function(_0x2fb1ce,_0x14c4ee){return _0x2fb1ce==_0x14c4ee;},'ePhLD':function(_0x4502c2,_0x1c35d4){return _0x4502c2+_0x1c35d4;},'UCifR':function(_0x40fdc5,_0x410311){return _0x40fdc5+_0x410311;},'Bsylf':function(_0x4c52a1,_0x26db10){return _0x4c52a1*_0x26db10;},'AlLJs':function(_0x12247c,_0x557484){return _0x12247c===_0x557484;},'plHEI':_0x2cd2('26d','gq!6'),'NbPIG':function(_0x31d7ec,_0x5c2992){return _0x31d7ec==_0x5c2992;},'IygsQ':function(_0x4bdb72,_0x14fc5c){return _0x4bdb72+_0x14fc5c;},'oTdMU':function(_0x2cc3c3,_0x99822e){return _0x2cc3c3+_0x99822e;},'NObLh':_0x2cd2('26e','g*Sm'),'ujsqo':function(_0x98193d,_0x25a7f8){return _0x98193d*_0x25a7f8;},'gItov':function(_0x2cb48b,_0x20b675){return _0x2cb48b+_0x20b675;},'IztAm':function(_0x378de6,_0x5507ef){return _0x378de6+_0x5507ef;},'sdFIJ':function(_0x4e854a,_0x3ba74b){return _0x4e854a+_0x3ba74b;},'FlRHR':function(_0x5efd94,_0x582f75){return _0x5efd94-_0x582f75;},'MiBHx':function(_0x27033a,_0x46f3e9){return _0x27033a!=_0x46f3e9;}};JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('26f','hS2h')]=[];JHMW[_0x2cd2('270','vB83')][_0x2cd2('271','ZboD')]=!![];var _0x259bd3=Math[_0x2cd2('272','gq!6')](_0x24f3e7[_0x2cd2('273','xvh(')](Graphics[_0x2cd2('274','#4SP')],JHMW[_0x2cd2('fa','wHIg')][_0x2cd2('275','R)LB')]));if(_0x24f3e7[_0x2cd2('276','UUq0')](JHMW[_0x2cd2('94','GDPD')][_0x2cd2('277','#fZb')][_0x2cd2('278','g*Sm')],_0x259bd3))JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('279','^)]V')][_0x2cd2('27a','l%aI')](0x0,_0x24f3e7[_0x2cd2('27b','R)LB')](JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('27c','8U%R')][_0x2cd2('27d','wHIg')],_0x259bd3));if(_0x24f3e7[_0x2cd2('27e','8U%R')](JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('23c','l%aI')][0x0],0x0)){for(var _0x2ab6ca=0x0;_0x24f3e7[_0x2cd2('27f','#iIl')](_0x2ab6ca,JHMW[_0x2cd2('19b','obzi')][_0x2cd2('280','vnIt')][0x0]);++_0x2ab6ca){var _0x2af146=JHMW[_0x2cd2('13b','hS2h')][_0x2cd2('281','8U%R')][_0x24f3e7[_0x2cd2('282','8vsk')](_0x24f3e7[_0x2cd2('283','xWmq')](_0x2ab6ca,0xb),0x1)];var _0x49dc38=JHMW[_0x2cd2('284','3psD')][_0x2cd2('285','lDFd')][_0x24f3e7[_0x2cd2('286','*unX')](_0x24f3e7[_0x2cd2('287','P0Kn')](_0x2ab6ca,0xb),0x2)];var _0x1f2d83=JHMW[_0x2cd2('52','[yfr')][_0x2cd2('288','obzi')][_0x24f3e7[_0x2cd2('289','qqEP')](_0x24f3e7[_0x2cd2('28a','vnIt')](_0x2ab6ca,0xb),0x3)];var _0x2268fa=_0x24f3e7[_0x2cd2('28b','LX!4')](parseInt,JHMW[_0x2cd2('56','@rQL')][_0x2cd2('281','8U%R')][_0x24f3e7[_0x2cd2('28c',')%R)')](_0x24f3e7[_0x2cd2('28d','%E7b')](_0x2ab6ca,0xb),0x4)]);var _0x490a2c=_0x24f3e7[_0x2cd2('28e','3psD')](parseInt,JHMW[_0x2cd2('284','3psD')][_0x2cd2('28f','ghrv')][_0x24f3e7[_0x2cd2('290','@rQL')](_0x24f3e7[_0x2cd2('291','dO1!')](_0x2ab6ca,0xb),0x5)]);var _0x30b409=_0x24f3e7[_0x2cd2('292','NeH6')](parseInt,JHMW[_0x2cd2('61','HHLp')][_0x2cd2('237','4UjT')][_0x24f3e7[_0x2cd2('293','^)]V')](_0x24f3e7[_0x2cd2('287','P0Kn')](_0x2ab6ca,0xb),0x6)]);var _0x456a20=_0x24f3e7[_0x2cd2('294','^)]V')](parseInt,JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('295','c!9y')][_0x24f3e7[_0x2cd2('296','%E7b')](_0x24f3e7[_0x2cd2('297','hS2h')](_0x2ab6ca,0xb),0x7)]);var _0x393ddb=_0x24f3e7[_0x2cd2('298','ghrv')](parseInt,JHMW[_0x2cd2('29','4UjT')][_0x2cd2('299','LX!4')][_0x24f3e7[_0x2cd2('29a','lDFd')](_0x24f3e7[_0x2cd2('29b','#iIl')](_0x2ab6ca,0xb),0x8)]);var _0x5cbe90=JHMW[_0x2cd2('fa','wHIg')][_0x2cd2('29c','l^iV')]?0x1:0x0;var _0x40130d=[];var _0x3c1008={'characterIndex':_0x2268fa,'characterName':_0x1f2d83,'direction':_0x393ddb,'pattern':0x1,'tileId':0x0};_0x40130d[0x0]={'conditions':_0x490a2c,'directionFix':![],'moveFrequency':0x3,'moveSpeed':0x4,'moveType':0x2,'priorityType':_0x5cbe90,'stepAnime':!![],'walkAnime':!![],'through':JHMW[_0x2cd2('29d','vnIt')][_0x2cd2('29e','hS2h')],'trigger':0x0,'image':_0x3c1008,'list':JHMW[_0x2cd2('29','4UjT')][_0x2cd2('29f','vnIt')]};var _0x3c8e1d={'id':_0x2ab6ca,'name':_0x2af146,'note':_0x49dc38,'x':_0x30b409,'y':_0x456a20,'pages':_0x40130d};if(_0x24f3e7[_0x2cd2('2a0','obzi')](_0x2af146,'系统'))JHMW[_0x2cd2('de','vbKI')][_0x2cd2('2a1','P0Kn')][_0x2cd2('2a2','&)j!')](_0x3c8e1d);if(JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('2a3','#4SP')][_0x24f3e7[_0x2cd2('2a4','vB83')](_0x24f3e7[_0x2cd2('2a5','@rQL')](_0x2ab6ca,0xb),0xa)]){if(_0x24f3e7[_0x2cd2('2a6','l%aI')](_0x2af146,'系统'))var _0x2b0f07=_0x24f3e7[_0x2cd2('2a7','&)j!')](_0x24f3e7[_0x2cd2('2a8','3Y7y')](JHMW[_0x2cd2('134','xWmq')][_0x2cd2('2a9','vbKI')],JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('2aa','4[4f')][_0x24f3e7[_0x2cd2('2ab','obzi')](_0x24f3e7[_0x2cd2('2ac','CFh^')](_0x2ab6ca,0xb),0xb)]),JHMW[_0x2cd2('61','HHLp')][_0x2cd2('2ad','#fZb')][_0x24f3e7[_0x2cd2('2ae','8U%R')](_0x24f3e7[_0x2cd2('2af','P0Kn')](_0x2ab6ca,0xb),0xa)]);else{if(_0x24f3e7[_0x2cd2('2b0','[yfr')](_0x24f3e7[_0x2cd2('2b1','[yfr')],_0x24f3e7[_0x2cd2('2b2','#4SP')])){if(JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('2b3','wHIg')][_0x24f3e7[_0x2cd2('2b4','#iIl')](_0x24f3e7[_0x2cd2('2b5','hS2h')](_0x2ab6ca,0xb),0xb)]){if(_0x24f3e7[_0x2cd2('2b6','UUq0')](JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('2b7','8vsk')][_0x24f3e7[_0x2cd2('2b8','Biy2')](_0x24f3e7[_0x2cd2('2b9','3Y7y')](_0x2ab6ca,0xb),0xb)],JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('2ba',')%R)')]))var _0x2b0f07=_0x24f3e7[_0x2cd2('2bb','R)LB')](_0x24f3e7[_0x2cd2('2bc','l%aI')](_0x2af146,_0x24f3e7[_0x2cd2('2bd','Biy2')]),JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('25d','YCbV')][_0x24f3e7[_0x2cd2('2be','X1Zu')](_0x24f3e7[_0x2cd2('2bf','%E7b')](_0x2ab6ca,0xb),0xa)]);}else var _0x2b0f07=_0x24f3e7[_0x2cd2('2c0','1L8W')](_0x24f3e7[_0x2cd2('2c1','ghrv')](_0x2af146,'说：'),JHMW[_0x2cd2('120','l^iV')][_0x2cd2('295','c!9y')][_0x24f3e7[_0x2cd2('2c2','YCbV')](_0x24f3e7[_0x2cd2('2c3','P0Kn')](_0x2ab6ca,0xb),0xa)]);}else{this[_0x2cd2('2c4','%E7b')][_0x2cd2('2c5','GDPD')](_0x24f3e7[_0x2cd2('2c6','vnIt')],this[_0x2cd2('2c7','lDFd')][_0x2cd2('2c8',')%R)')](this));this[_0x2cd2('2c9','#iIl')][_0x2cd2('2ca','CFh^')](_0x24f3e7[_0x2cd2('2cb','3Y7y')],this[_0x2cd2('2cc','@rQL')][_0x2cd2('2cd','&)j!')](this));}}var _0x207c7a=JHMW[_0x2cd2('de','vbKI')][_0x2cd2('2ce','lDFd')][_0x24f3e7[_0x2cd2('2cf','8vsk')](JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('2d0','hS2h')][_0x2cd2('2d1','1L8W')],0x1)];if(_0x24f3e7[_0x2cd2('2d2','2^*B')](_0x207c7a,_0x2b0f07))JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('2d4','vbKI')][_0x2cd2('2d5',')%R)')](_0x2b0f07);}}}};JHMW[_0x2cd2('13b','hS2h')][_0x2cd2('2d6','HHLp')]=function(_0x59cd3f){var _0x206b1b={'bJOaZ':_0x2cd2('2d7','8vsk'),'pIjrl':function(_0x15399b,_0x18b209){return _0x15399b>=_0x18b209;},'TjHRn':function(_0x3aeb7e,_0x2203fb){return _0x3aeb7e===_0x2203fb;},'fuqVq':_0x2cd2('2d8','l^iV'),'eOBgw':_0x2cd2('2d9','X1Zu'),'CeSeJ':function(_0x2d4288,_0x2c64ae){return _0x2d4288(_0x2c64ae);},'hgODn':function(_0xda7d87,_0x55baeb){return _0xda7d87+_0x55baeb;},'ghFvB':function(_0x198f39,_0x5103c7){return _0x198f39+_0x5103c7;},'yCfOk':function(_0x38e17a,_0x1375c7){return _0x38e17a+_0x1375c7;},'ZWsdC':_0x2cd2('2da','ghrv'),'CYHNe':_0x2cd2('2db','X1Zu')};_0x59cd3f=!_0x59cd3f;$gameSystem[_0x2cd2('2dc','xvh(')]();var _0x4089e5=JsonEx[_0x2cd2('2dd','&)j!')](DataManager[_0x2cd2('2de','*unX')]());if(_0x206b1b[_0x2cd2('2df','c!9y')](_0x4089e5[_0x2cd2('2e0','YCbV')],0x5b8d80)){if(_0x206b1b[_0x2cd2('2e1','1L8W')](_0x206b1b[_0x2cd2('2e2','xWmq')],_0x206b1b[_0x2cd2('2e3','y046')])){if(Imported[_0x2cd2('2e4','hS2h')])Notification[_0x2cd2('2e5','vnIt')](_0x206b1b[_0x2cd2('2e6','dO1!')]);else $gameMessage[_0x2cd2('2e7','4[4f')](_0x206b1b[_0x2cd2('2e8','obzi')]);return;}else{Graphics[_0x2cd2('2e9','wHIg')](_0x206b1b[_0x2cd2('2ea','8vsk')]);this[_0x2cd2('2eb','8vsk')][_0x2cd2('2ec','&)j!')]=0x0;this[_0x2cd2('2ed','8U%R')]=0x0;}}var _0x3b914d=_0x206b1b[_0x2cd2('2ee','&)j!')](encodeURIComponent,LZString[_0x2cd2('2ef','wHIg')](_0x4089e5));var _0xf03f8b=JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('2f0','3psD')];var _0x3cf25e=JHMW[_0x2cd2('dc','#4SP')][_0x2cd2('17f','c!9y')];var _0x480b8d=JHMW[_0x2cd2('56','@rQL')][_0x2cd2('2f1','vbKI')];var _0x643ed4=JHMW[_0x2cd2('2f2','c!9y')](JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('2f3','YCbV')])[_0x2cd2('2f4','8U%R')]();if(DataManager[_0x2cd2('2f5','gq!6')]())var _0x28d45b=JHMW[_0x2cd2('2f6','Biy2')](JSON[_0x2cd2('2f7','CFh^')](PluginManager[_0x2cd2('2f8','R)LB')]));else var _0x28d45b=_0x643ed4;var _0x170f78=JHMW[_0x2cd2('2f9','lDFd')](_0x206b1b[_0x2cd2('2fa','HHLp')](_0x206b1b[_0x2cd2('2fb',')%R)')](_0x206b1b[_0x2cd2('2fc','4[4f')](_0x206b1b[_0x2cd2('2fd','jpvP')](_0x206b1b[_0x2cd2('2fe','LX!4')](_0x206b1b[_0x2cd2('2ff','2^*B')],_0x3cf25e[_0x2cd2('300','jpvP')](0x5,0x4)),_0xf03f8b),_0x643ed4),_0x480b8d),_0x28d45b[_0x2cd2('301','@rQL')](0x0,0xf)));if(DataManager[_0x2cd2('302','UUq0')]())_0x28d45b=JHMW[_0x2cd2('101','wHIg')][_0x2cd2('303','*unX')](_0x28d45b);var _0x3e6d20={'data':_0x3b914d,'user':_0xf03f8b,'game':_0x3cf25e,'cmd':_0x206b1b[_0x2cd2('304','3Y7y')],'word':_0x643ed4,'mark':_0x28d45b,'sign':_0x170f78};var _0x248fcb=JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('305','xvh(')](JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('306','%E7b')]);JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('307','vnIt')](_0x248fcb,_0x3e6d20,![],JHMW[_0x2cd2('165','NeH6')][_0x2cd2('308','R)LB')]);};JHMW[_0x2cd2('270','vB83')][_0x2cd2('309','vnIt')]=function(_0x84df12){var _0x596d11={'amejs':function(_0x450a21,_0x52e40a){return _0x450a21==_0x52e40a;},'GKvzX':function(_0x1dc531,_0x3cf9e3){return _0x1dc531!=_0x3cf9e3;},'mSoqe':_0x2cd2('30a','ZboD'),'cJrUH':function(_0x5aa489,_0x437aaf){return _0x5aa489==_0x437aaf;}};if(Imported[_0x2cd2('30b','@rQL')]){if(_0x596d11[_0x2cd2('30c','lDFd')](_0x84df12,0x1)&&JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('30d',')%R)')])Notification[_0x2cd2('30e','8U%R')](JHMW[_0x2cd2('29d','vnIt')][_0x2cd2('f7','CFh^')]);if(_0x596d11[_0x2cd2('30f','vnIt')](_0x84df12,0x1))Notification[_0x2cd2('310','l^iV')](_0x596d11[_0x2cd2('311','[yfr')]);}else{if(_0x596d11[_0x2cd2('312','xvh(')](_0x84df12,0x1)&&JHMW[_0x2cd2('313','l%aI')][_0x2cd2('314','vB83')])$gameMessage[_0x2cd2('315','1L8W')](JHMW[_0x2cd2('f2','c!9y')][_0x2cd2('316','g*Sm')]);if(_0x596d11[_0x2cd2('317','gq!6')](_0x84df12,0x1))$gameMessage[_0x2cd2('318','X1Zu')](_0x596d11[_0x2cd2('319','HHLp')]);}};JHMW[_0x2cd2('101','wHIg')][_0x2cd2('31a','3Y7y')]=function(_0x4f2ad3,_0x3999bf){var _0x1116e4={'MtyrI':_0x2cd2('31b','lDFd'),'gAGHT':function(_0xf1b052,_0x46684e){return _0xf1b052==_0x46684e;},'gbfjE':function(_0x9cfd61,_0x2ec208){return _0x9cfd61(_0x2ec208);},'FjZJQ':function(_0x2c14ee,_0x27d35b){return _0x2c14ee+_0x27d35b;},'YWeFK':function(_0x142eef,_0x2cf21c){return _0x142eef+_0x2cf21c;},'BrNKN':function(_0x459ee6,_0x474cb1){return _0x459ee6+_0x474cb1;},'YdSmp':function(_0x2b283f,_0x487e3b){return _0x2b283f+_0x487e3b;},'oDOdn':_0x2cd2('31c','8vsk'),'qSaFf':_0x2cd2('31d','c!9y')};var _0x1c2b8f=_0x1116e4[_0x2cd2('31e','YCbV')][_0x2cd2('31f','^)]V')]('|'),_0x3da537=0x0;while(!![]){switch(_0x1c2b8f[_0x3da537++]){case'0':var _0x2f69fe=JHMW[_0x2cd2('153','8U%R')][_0x2cd2('320','l%aI')];continue;case'1':var _0x35f1ba=JHMW[_0x2cd2('321','lDFd')][_0x2cd2('322','X1Zu')];continue;case'2':if(_0x1116e4[_0x2cd2('323','3psD')](_0x3999bf,JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('324','xWmq')]))_0x3999bf='系统';continue;case'3':if(DataManager[_0x2cd2('325','#4SP')]())var _0x19e237=JHMW[_0x2cd2('326','#4SP')](JSON[_0x2cd2('327','gq!6')](PluginManager[_0x2cd2('328','W@80')]));else var _0x19e237=_0x3f5fa8;continue;case'4':_0x4f2ad3=_0x1116e4[_0x2cd2('329','[yfr')](encodeURIComponent,_0x4f2ad3);continue;case'5':var _0xe6360d=JHMW[_0x2cd2('fc','W@80')][_0x2cd2('32a','wHIg')];continue;case'6':if(DataManager[_0x2cd2('32b','lDFd')]())_0x19e237=JHMW[_0x2cd2('32c','X1Zu')][_0x2cd2('32d','lDFd')](_0x19e237);continue;case'7':var _0x4c2485=JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('32e','y046')](JHMW[_0x2cd2('125','lDFd')][_0x2cd2('32f','HHLp')]);continue;case'8':var _0x32df85=JHMW[_0x2cd2('330','^)]V')](_0x1116e4[_0x2cd2('331','ZboD')](_0x1116e4[_0x2cd2('332','#fZb')](_0x1116e4[_0x2cd2('333','vnIt')](_0x1116e4[_0x2cd2('334','HHLp')](_0x1116e4[_0x2cd2('335','YCbV')](_0x1116e4[_0x2cd2('336','obzi')],_0x35f1ba[_0x2cd2('337','#4SP')](0x5,0x4)),_0x2f69fe),_0x3f5fa8),_0xe6360d),_0x19e237[_0x2cd2('338','y046')](0x0,0xf)));continue;case'9':var _0x52a11a={'user':_0x2f69fe,'game':_0x35f1ba,'cmd':_0x1116e4[_0x2cd2('339','2^*B')],'text':_0x4f2ad3,'target':_0x3999bf,'word':_0x3f5fa8,'mark':_0x19e237,'sign':_0x32df85};continue;case'10':if(!_0x3999bf)var _0x3999bf='X';continue;case'11':JHMW[_0x2cd2('d','8vsk')][_0x2cd2('33a','4UjT')](_0x4c2485,_0x52a11a,!![]);continue;case'12':var _0x3f5fa8=JHMW[_0x2cd2('33b','wHIg')](JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('33c','*unX')])[_0x2cd2('33d','P0Kn')]();continue;}break;}};JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('33e','l%aI')]=function(_0x42643f){var _0x48f9a6={'ChZxh':_0x2cd2('33f','GDPD')};var _0x2acaa4={'user':JHMW[_0x2cd2('153','8U%R')][_0x2cd2('340','1L8W')],'game':JHMW[_0x2cd2('1f9','[yfr')][_0x2cd2('341','1L8W')],'cmd':_0x48f9a6[_0x2cd2('342','Biy2')]};var _0x15d96d=JHMW[_0x2cd2('d','8vsk')][_0x2cd2('343',')%R)')](JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('344','xvh(')]);JHMW[_0x2cd2('fc','W@80')][_0x2cd2('345','wHIg')](_0x15d96d,_0x2acaa4,![],JHMW[_0x2cd2('de','vbKI')][_0x2cd2('346','Biy2')],_0x42643f);};JHMW[_0x2cd2('94','GDPD')][_0x2cd2('347','c!9y')]=function(_0x27016e,_0x11bb40){var _0x1e61af={'Zzhcx':function(_0x1180be,_0x55bd1c){return _0x1180be>_0x55bd1c;},'iJznh':function(_0x362774,_0x5323f7){return _0x362774+_0x5323f7;},'rqKDw':function(_0x36741d,_0x148597){return _0x36741d+_0x148597;},'PrrCF':_0x2cd2('348','HHLp'),'LnJVM':function(_0x312f5e,_0x4a230d){return _0x312f5e+_0x4a230d;},'lSdEF':_0x2cd2('349','*unX'),'nSPwO':function(_0x553ce5,_0x487192){return _0x553ce5<_0x487192;},'ybEgw':function(_0x14889b,_0x49d443){return _0x14889b!==_0x49d443;},'qgTPd':_0x2cd2('34a','l%aI'),'PoeUo':_0x2cd2('34b','4UjT'),'DRkhH':function(_0x226a9f,_0x2c1627){return _0x226a9f(_0x2c1627);},'wvVxO':function(_0x139ef3,_0x1364ee){return _0x139ef3<_0x1364ee;},'MudAz':function(_0x309e49,_0x2d711d){return _0x309e49+_0x2d711d;},'isXqS':function(_0x5befc8,_0x330463){return _0x5befc8+_0x330463;}};var _0x31f32f=JHMW[_0x2cd2('94','GDPD')][_0x2cd2('34c','&)j!')](JHMW[_0x2cd2('120','l^iV')][_0x2cd2('34d','ZboD')](_0x27016e));var _0x43581c=_0x31f32f[_0x2cd2('227','qqEP')](',');for(var _0x311f4d=0x0;_0x1e61af[_0x2cd2('34e','UUq0')](_0x311f4d,0x6);_0x311f4d++){if(_0x1e61af[_0x2cd2('34f','hS2h')](_0x1e61af[_0x2cd2('350','*unX')],_0x1e61af[_0x2cd2('351','l%aI')])){if(_0x11bb40[_0x311f4d])$gameVariables[_0x2cd2('352','#iIl')](_0x11bb40[_0x311f4d],_0x1e61af[_0x2cd2('353','qqEP')](parseInt,_0x43581c[_0x1e61af[_0x2cd2('354','xvh(')](_0x311f4d,0x3)?_0x1e61af[_0x2cd2('355','^)]V')](_0x311f4d,0x1):_0x1e61af[_0x2cd2('356','NeH6')](_0x311f4d,0x2)]));}else{var _0x8ca48e=new Date()[_0x2cd2('357','gq!6')]();if(_0x1e61af[_0x2cd2('358','YCbV')](url[_0x2cd2('359','gq!6')]('?'),-0x1))url=_0x1e61af[_0x2cd2('35a','2^*B')](_0x1e61af[_0x2cd2('35b','wHIg')](url,_0x1e61af[_0x2cd2('35c','Biy2')]),_0x8ca48e);else url=_0x1e61af[_0x2cd2('35b','wHIg')](_0x1e61af[_0x2cd2('35d','jpvP')](url,_0x1e61af[_0x2cd2('35e','[yfr')]),_0x8ca48e);return url;}}};JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('35f','vB83')]=function(_0x2e38b1){var _0x2af782={'WnakN':_0x2cd2('360',')%R)')};var _0x2bc2c3={'user':JHMW[_0x2cd2('153','8U%R')][_0x2cd2('361','vB83')],'game':JHMW[_0x2cd2('d7','@rQL')][_0x2cd2('362','ZboD')],'cmd':_0x2af782[_0x2cd2('363','ghrv')]};var _0x525c64=JHMW[_0x2cd2('de','vbKI')][_0x2cd2('364','gq!6')](JHMW[_0x2cd2('61','HHLp')][_0x2cd2('365','g*Sm')]);JHMW[_0x2cd2('d','8vsk')][_0x2cd2('366','y046')](_0x525c64,_0x2bc2c3,![],JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('367','4[4f')],_0x2e38b1);};JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('368','HHLp')]=function(_0x12a256,_0x6c00e8){var _0x3eb45c={'hnSVW':function(_0x2e4d75,_0x56be65){return _0x2e4d75(_0x56be65);}};var _0x485ff4=JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('369','4[4f')](JHMW[_0x2cd2('284','3psD')][_0x2cd2('36a','&)j!')](_0x12a256));var _0x4a13bb=_0x485ff4[_0x2cd2('36b','xvh(')](',');if(_0x6c00e8)$gameVariables[_0x2cd2('36c','8U%R')](_0x6c00e8,_0x3eb45c[_0x2cd2('36d','#fZb')](parseInt,_0x4a13bb[0x1]));};JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('36e','2^*B')]=function(){var _0x45e808={'EypPO':_0x2cd2('36f','#fZb')};var _0x53502a={'user':JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('320','l%aI')],'game':JHMW[_0x2cd2('e9','jpvP')][_0x2cd2('370','CFh^')],'cmd':_0x45e808[_0x2cd2('371','W@80')]};var _0x3e2672=JHMW[_0x2cd2('120','l^iV')][_0x2cd2('372','8U%R')](JHMW[_0x2cd2('120','l^iV')][_0x2cd2('373','y046')]);JHMW[_0x2cd2('374',')%R)')][_0x2cd2('375','gq!6')](_0x3e2672,_0x53502a,![],JHMW[_0x2cd2('0','l%aI')][_0x2cd2('376','vnIt')]);};JHMW[_0x2cd2('101','wHIg')][_0x2cd2('377','c!9y')]=function(_0x49fecb){var _0xd6c27={'fbjBa':_0x2cd2('378','HHLp'),'LLvvm':_0x2cd2('379','LX!4'),'iDkFk':function(_0x393bed,_0x79d908){return _0x393bed+_0x79d908;},'aXfyK':function(_0x235a48,_0x546b76){return _0x235a48+_0x546b76;},'EcAzM':function(_0x1da706,_0x55da84){return _0x1da706+_0x55da84;},'jLrfX':function(_0x29a61d,_0x36d59c){return _0x29a61d+_0x36d59c;},'iLAEG':_0x2cd2('37a','hS2h')};var _0x3dc64e=_0xd6c27[_0x2cd2('37b','@rQL')][_0x2cd2('37c','&)j!')]('|'),_0x210d2d=0x0;while(!![]){switch(_0x3dc64e[_0x210d2d++]){case'0':var _0x13cd4d=JHMW[_0x2cd2('37d','3Y7y')][_0x2cd2('37e','vB83')];continue;case'1':var _0x3e5b68=JHMW[_0x2cd2('56','@rQL')][_0x2cd2('37f','UUq0')](JHMW[_0x2cd2('380','CFh^')][_0x2cd2('32f','HHLp')]);continue;case'2':JHMW[_0x2cd2('125','lDFd')][_0x2cd2('345','wHIg')](_0x3e5b68,_0x369b34,![],JHMW[_0x2cd2('d','8vsk')][_0x2cd2('381','*unX')]);continue;case'3':var _0x369b34={'user':_0x3d9af3,'game':_0x13cd4d,'cmd':_0xd6c27[_0x2cd2('382','vB83')],'actor':_0x49fecb,'sign':_0x3497b4};continue;case'4':var _0x3d9af3=JHMW[_0x2cd2('61','HHLp')][_0x2cd2('383','GDPD')];continue;case'5':var _0x3497b4=JHMW[_0x2cd2('384','qqEP')](_0xd6c27[_0x2cd2('385','P0Kn')](_0xd6c27[_0x2cd2('386','GDPD')](_0xd6c27[_0x2cd2('387','3Y7y')](_0xd6c27[_0x2cd2('388','4UjT')](_0xd6c27[_0x2cd2('389','gq!6')],_0x13cd4d[_0x2cd2('38a','4[4f')](0x5,0x4)),_0x3d9af3),_0x49fecb),_0x2e5ec8));continue;case'6':var _0x2e5ec8=JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('38b','g*Sm')];continue;}break;}};JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('38c','vB83')]=function(_0x386e56){var _0x469603={'KbJQm':function(_0x5724d2,_0x391f4d){return _0x5724d2(_0x391f4d);},'HrmKP':function(_0x3957ee,_0x1c7d2b){return _0x3957ee(_0x1c7d2b);}};var _0x36dbd6=JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('38d','%E7b')](JHMW[_0x2cd2('de','vbKI')][_0x2cd2('38e','2^*B')](_0x386e56));var _0x35efe0=_0x36dbd6[_0x2cd2('227','qqEP')](',');$gameVariables[_0x2cd2('38f','*unX')](JHMW[_0x2cd2('29d','vnIt')][_0x2cd2('390','3psD')],_0x469603[_0x2cd2('391','%E7b')](parseInt,_0x35efe0[0x1]));$gameVariables[_0x2cd2('392','c!9y')](JHMW[_0x2cd2('17e','y046')][_0x2cd2('393',')%R)')],_0x469603[_0x2cd2('394','X1Zu')](parseInt,_0x35efe0[0x2]));};JHMW[_0x2cd2('284','3psD')][_0x2cd2('395','2^*B')]=function(_0x120fde){var _0xd37a97={'lroiN':_0x2cd2('396','^)]V'),'ATMfx':function(_0x448fbb,_0x262e4b){return _0x448fbb+_0x262e4b;},'gyppv':function(_0x2a75af,_0x17cdc5){return _0x2a75af+_0x17cdc5;},'CqOqu':_0x2cd2('397','Biy2'),'UaoNy':_0x2cd2('398','l^iV')};var _0xec5ac0=_0xd37a97[_0x2cd2('399','#fZb')][_0x2cd2('227','qqEP')]('|'),_0x24d569=0x0;while(!![]){switch(_0xec5ac0[_0x24d569++]){case'0':var _0x4721a4=JHMW[_0x2cd2('a7','c!9y')][_0x2cd2('383','GDPD')];continue;case'1':var _0x399740=JHMW[_0x2cd2('384','qqEP')](_0xd37a97[_0x2cd2('39a','#fZb')](_0xd37a97[_0x2cd2('39a','#fZb')](_0xd37a97[_0x2cd2('39b','3Y7y')](_0xd37a97[_0x2cd2('39c','qqEP')](_0xd37a97[_0x2cd2('39d','GDPD')](_0xd37a97[_0x2cd2('39e','*unX')],_0x7fbd18[_0x2cd2('39f','X1Zu')](0x5,0x4)),_0x4721a4),_0x445414),_0x2e90fa),_0x39ba3e[_0x2cd2('3a0','qqEP')](0x0,0xf)));continue;case'2':var _0x2e90fa=JHMW[_0x2cd2('165','NeH6')][_0x2cd2('3a1','X1Zu')];continue;case'3':JHMW[_0x2cd2('94','GDPD')][_0x2cd2('3a2','l%aI')](_0x1db37a,_0x1f180b,![],JHMW[_0x2cd2('d','8vsk')][_0x2cd2('3a3','xWmq')],_0x120fde);continue;case'4':var _0x7fbd18=JHMW[_0x2cd2('148','xvh(')][_0x2cd2('3a4','4UjT')];continue;case'5':var _0x445414=JHMW[_0x2cd2('3a5','8U%R')](JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('3a6','dO1!')])[_0x2cd2('3a7','vnIt')]();continue;case'6':if(DataManager[_0x2cd2('3a8','y046')]())_0x39ba3e=JHMW[_0x2cd2('101','wHIg')][_0x2cd2('3a9','2^*B')](_0x39ba3e);continue;case'7':if(DataManager[_0x2cd2('3aa','&)j!')]())var _0x39ba3e=JHMW[_0x2cd2('3ab','vnIt')](JSON[_0x2cd2('3ac','ghrv')](PluginManager[_0x2cd2('3ad','*unX')]));else var _0x39ba3e=_0x445414;continue;case'8':var _0x1db37a=JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('3ae','vB83')](JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('3af','qqEP')]);continue;case'9':var _0x1f180b={'user':_0x4721a4,'game':_0x7fbd18,'cmd':_0xd37a97[_0x2cd2('3b0','vbKI')],'word':_0x445414,'vermd':_0x39ba3e,'sign':_0x399740};continue;}break;}};JHMW[_0x2cd2('de','vbKI')][_0x2cd2('3b1','y046')]=function(_0x550caa){var _0x55ea22={'IJuaC':_0x2cd2('3b2','ghrv'),'XCkXI':function(_0x109d0d,_0x2714d0){return _0x109d0d+_0x2714d0;},'tHmUU':_0x2cd2('3b3','Biy2'),'IPgTd':_0x2cd2('3b4','R)LB')};var _0x334dd3=_0x55ea22[_0x2cd2('3b5','YCbV')][_0x2cd2('3b6','#iIl')]('|'),_0x2d9795=0x0;while(!![]){switch(_0x334dd3[_0x2d9795++]){case'0':var _0x26714a=JHMW[_0x2cd2('97','obzi')][_0x2cd2('3b7',')%R)')];continue;case'1':if(DataManager[_0x2cd2('3b8','vnIt')]())var _0x1ea7a5=JHMW[_0x2cd2('3b9','@rQL')](JSON[_0x2cd2('3ba','xWmq')](PluginManager[_0x2cd2('328','W@80')]));else var _0x1ea7a5=_0x47dd63;continue;case'2':var _0x25aaf6=JHMW[_0x2cd2('3bb','#fZb')](_0x55ea22[_0x2cd2('3bc','l^iV')](_0x55ea22[_0x2cd2('3bd','2^*B')](_0x55ea22[_0x2cd2('3be','GDPD')](_0x55ea22[_0x2cd2('3bf','l%aI')](_0x55ea22[_0x2cd2('3c0','1L8W')],_0x26714a[_0x2cd2('3c1','*unX')](0x5,0x4)),_0x5b1efe),_0x47dd63),_0x1ea7a5[_0x2cd2('39f','X1Zu')](0x0,0xf)));continue;case'3':var _0x47dd63=JHMW[_0x2cd2('330','^)]V')](JHMW[_0x2cd2('29','4UjT')][_0x2cd2('3c2','[yfr')])[_0x2cd2('3c3','YCbV')]();continue;case'4':var _0x15a8f3={'user':_0x5b1efe,'game':_0x26714a,'cmd':_0x55ea22[_0x2cd2('3c4','ghrv')],'word':_0x47dd63,'vermd':_0x1ea7a5,'sign':_0x25aaf6};continue;case'5':JHMW[_0x2cd2('52','[yfr')][_0x2cd2('3c5','xWmq')](_0x74060d,_0x15a8f3,![],JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('3c6','lDFd')],_0x550caa);continue;case'6':if(DataManager[_0x2cd2('3c7','Biy2')]())_0x1ea7a5=JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('3c8','LX!4')](_0x1ea7a5);continue;case'7':var _0x5b1efe=JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('3c9','ZboD')];continue;case'8':var _0x74060d=JHMW[_0x2cd2('120','l^iV')][_0x2cd2('3ca','l^iV')](JHMW[_0x2cd2('380','CFh^')][_0x2cd2('3cb','vnIt')]);continue;}break;}};JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('3cc','l^iV')]=function(_0x4efe0a,_0x6cbd47){var _0x22687d={'JeMPe':function(_0x3ab30b,_0x5d1fb6){return _0x3ab30b(_0x5d1fb6);}};var _0x1c2a94=JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('3ce','#iIl')](JHMW[_0x2cd2('380','CFh^')][_0x2cd2('3cf','UUq0')](_0x4efe0a));var _0x4103e5=_0x1c2a94[_0x2cd2('3b6','#iIl')](',');if(_0x6cbd47)$gameVariables[_0x2cd2('3d0','vB83')](_0x6cbd47,_0x22687d[_0x2cd2('3d1','4[4f')](parseInt,_0x4103e5[0x1]));};JHMW[_0x2cd2('13f','*unX')][_0x2cd2('3d2','l^iV')]=function(){var _0x51b1d9={'VcLgj':_0x2cd2('3d3','R)LB'),'IkrJN':_0x2cd2('3d4','@rQL'),'HGpEs':function(_0x336bb1,_0xf0e202){return _0x336bb1+_0xf0e202;},'XuuCu':function(_0x1d08ab,_0x179a01){return _0x1d08ab+_0x179a01;},'YwPzk':function(_0x5d9470,_0x518542){return _0x5d9470+_0x518542;},'zHAIS':function(_0x43800f,_0x1745df){return _0x43800f+_0x1745df;},'AhObq':_0x2cd2('3d5','vnIt')};var _0x521b93=_0x51b1d9[_0x2cd2('3d6','CFh^')][_0x2cd2('3d7','GDPD')]('|'),_0x20b7e6=0x0;while(!![]){switch(_0x521b93[_0x20b7e6++]){case'0':var _0x39a287=JHMW[_0x2cd2('384','qqEP')](JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('3d8','wHIg')])[_0x2cd2('3d9','UUq0')]();continue;case'1':var _0x22fb6c={'user':_0x5353fb,'game':_0x55c31b,'cmd':_0x51b1d9[_0x2cd2('3da','YCbV')],'word':_0x39a287,'vermd':_0x36c027,'sign':_0x583345};continue;case'2':var _0x55c31b=JHMW[_0x2cd2('3db','8vsk')][_0x2cd2('3dc','#iIl')];continue;case'3':var _0x3f6f97=JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('3ca','l^iV')](JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('3dd','&)j!')]);continue;case'4':var _0x583345=JHMW[_0x2cd2('3de','l%aI')](_0x51b1d9[_0x2cd2('3df','4[4f')](_0x51b1d9[_0x2cd2('3e0','obzi')](_0x51b1d9[_0x2cd2('3e1','3Y7y')](_0x51b1d9[_0x2cd2('3e2','g*Sm')](_0x51b1d9[_0x2cd2('3e3','&)j!')](_0x51b1d9[_0x2cd2('3e4','Biy2')],_0x55c31b[_0x2cd2('3e5','8U%R')](0x5,0x4)),_0x5353fb),_0x39a287),_0x5dac4e),_0x36c027[_0x2cd2('1f6','dO1!')](0x0,0xf)));continue;case'5':if(DataManager[_0x2cd2('2f5','gq!6')]())var _0x36c027=JHMW[_0x2cd2('3e6','8vsk')](JSON[_0x2cd2('3e7','#fZb')](PluginManager[_0x2cd2('3e8','NeH6')]));else var _0x36c027=_0x39a287;continue;case'6':if(DataManager[_0x2cd2('3e9','l^iV')]())_0x36c027=JHMW[_0x2cd2('56','@rQL')][_0x2cd2('3ea','[yfr')](_0x36c027);continue;case'7':JHMW[_0x2cd2('101','wHIg')][_0x2cd2('3eb','qqEP')](_0x3f6f97,_0x22fb6c,!![]);continue;case'8':var _0x5dac4e=JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('3ec','GDPD')];continue;case'9':var _0x5353fb=JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('3ed','gq!6')];continue;}break;}};JHMW[_0x2cd2('0','l%aI')][_0x2cd2('3ee','obzi')]=function(_0x39f480){var _0x43b1f6={'cHlLu':_0x2cd2('3ef','wHIg'),'lOxxA':function(_0x2c299e,_0x4394e3){return _0x2c299e+_0x4394e3;},'rBgdP':function(_0x2674df,_0x2d26cb){return _0x2674df+_0x2d26cb;},'GwguF':function(_0x4fa519,_0x4145a5){return _0x4fa519+_0x4145a5;},'AWdXh':_0x2cd2('3f0','hS2h'),'UMeVQ':_0x2cd2('3f1','#fZb')};var _0x2c0ade=_0x43b1f6[_0x2cd2('3f2','hS2h')][_0x2cd2('3f3','dO1!')]('|'),_0x472e04=0x0;while(!![]){switch(_0x2c0ade[_0x472e04++]){case'0':var _0x410b47=JHMW[_0x2cd2('384','qqEP')](_0x43b1f6[_0x2cd2('3f4','R)LB')](_0x43b1f6[_0x2cd2('3f5','^)]V')](_0x43b1f6[_0x2cd2('3f6',')%R)')](_0x43b1f6[_0x2cd2('3f7','hS2h')](_0x43b1f6[_0x2cd2('3f8','hS2h')],_0x31e139[_0x2cd2('39f','X1Zu')](0x5,0x4)),_0x57f120),_0x2be19b),_0x54651a[_0x2cd2('3f9','[yfr')](0x0,0xf)));continue;case'1':var _0x585a1e={'user':_0x57f120,'game':_0x31e139,'cmd':_0x43b1f6[_0x2cd2('3fa',')%R)')],'mark':_0x39f480,'word':_0x2be19b,'vermd':_0x54651a,'sign':_0x410b47};continue;case'2':var _0x435315=JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('3fb','HHLp')](JHMW[_0x2cd2('94','GDPD')][_0x2cd2('3fc','^)]V')]);continue;case'3':if(DataManager[_0x2cd2('3fd','GDPD')]())_0x54651a=JHMW[_0x2cd2('eb','ghrv')][_0x2cd2('3fe','hS2h')](_0x54651a);continue;case'4':if(DataManager[_0x2cd2('3ff','NeH6')]())var _0x54651a=JHMW[_0x2cd2('400','obzi')](JSON[_0x2cd2('401','wHIg')](PluginManager[_0x2cd2('402','#4SP')]));else var _0x54651a=_0x2be19b;continue;case'5':var _0x31e139=JHMW[_0x2cd2('f2','c!9y')][_0x2cd2('403','4[4f')];continue;case'6':var _0x57f120=JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('404','HHLp')];continue;case'7':JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('405','ghrv')](_0x435315,_0x585a1e,!![]);continue;case'8':var _0x2be19b=JHMW[_0x2cd2('406','xWmq')](JHMW[_0x2cd2('162','y046')][_0x2cd2('216','4[4f')])[_0x2cd2('407','[yfr')]();continue;}break;}};JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('408','l^iV')]=function(){var _0x523e20={'xdGHL':_0x2cd2('409','xvh('),'Awfff':_0x2cd2('40a','jpvP'),'DszNp':function(_0x2777be,_0x103d6d){return _0x2777be+_0x103d6d;},'bqQId':function(_0x56aa9b,_0xed23b9){return _0x56aa9b+_0xed23b9;},'BgXpb':function(_0x48a597,_0x317d0a){return _0x48a597+_0x317d0a;},'OZiVg':function(_0x1acb26,_0xfc30e2){return _0x1acb26+_0xfc30e2;},'vWsxi':function(_0x361e03,_0x51d28b){return _0x361e03+_0x51d28b;},'zIRVR':_0x2cd2('40b','3Y7y')};var _0x4f5883=_0x523e20[_0x2cd2('40c','%E7b')][_0x2cd2('40d','4UjT')]('|'),_0x296c89=0x0;while(!![]){switch(_0x4f5883[_0x296c89++]){case'0':var _0x2bbe6e=JHMW[_0x2cd2('380','CFh^')][_0x2cd2('40e','*unX')](JHMW[_0x2cd2('94','GDPD')][_0x2cd2('40f','gq!6')]);continue;case'1':var _0x3341a7=JHMW[_0x2cd2('410','W@80')][_0x2cd2('411','y046')];continue;case'2':var _0x560b0f={'user':_0x3dfa87,'word':_0x7b09a0,'game':_0x3341a7,'data':_0x3bf02d,'cmd':_0x523e20[_0x2cd2('412','LX!4')],'sign':_0x3a6d7c};continue;case'3':var _0x7b09a0=_0x523e20[_0x2cd2('413','c!9y')]('',Graphics[_0x2cd2('414','LX!4')][_0x2cd2('415','R)LB')]);continue;case'4':JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('416','1L8W')](_0x2bbe6e,_0x560b0f,![],JHMW[_0x2cd2('29','4UjT')][_0x2cd2('417','1L8W')],_0x3bf02d);continue;case'5':var _0x3bf02d=JHMW[_0x2cd2('418','LX!4')](JSON[_0x2cd2('419','%E7b')](PluginManager[_0x2cd2('41a','^)]V')]));continue;case'6':var _0x3dfa87=_0x523e20[_0x2cd2('41b','lDFd')]('',Graphics[_0x2cd2('41c','#fZb')][_0x2cd2('41d','[yfr')]);continue;case'7':var _0x3a6d7c=JHMW[_0x2cd2('41e','dO1!')](_0x523e20[_0x2cd2('41f','[yfr')](_0x523e20[_0x2cd2('420','gq!6')](_0x523e20[_0x2cd2('421','LX!4')](_0x523e20[_0x2cd2('422','#4SP')](_0x523e20[_0x2cd2('423','1L8W')],_0x3341a7[_0x2cd2('3c1','*unX')](0x5,0x4)),_0x3dfa87),_0x7b09a0),_0x3bf02d));continue;}break;}};JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('424','4[4f')]=function(_0x5d92f6,_0x134659){var _0x226d09={'mdnKj':function(_0x3f4c07,_0x251a78){return _0x3f4c07==_0x251a78;},'grUiU':function(_0x21f1fe,_0x95e5ca){return _0x21f1fe(_0x95e5ca);},'RXINg':_0x2cd2('425','#fZb')};var _0x2e251e=JHMW[_0x2cd2('eb','ghrv')][_0x2cd2('426','ZboD')](JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('427','W@80')](_0x5d92f6));var _0x246f85=_0x2e251e[_0x2cd2('428','c!9y')]('#');if(_0x226d09[_0x2cd2('429',')%R)')](_0x246f85[0x1],_0x134659))_0x226d09[_0x2cd2('42a','Biy2')](alert,_0x226d09[_0x2cd2('42b','X1Zu')]);};JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('42c','W@80')]=function(){var _0x4ef5b5={'Cofih':function(_0x748dc5,_0xae1313){return _0x748dc5+_0xae1313;},'tcVbl':function(_0x3584db,_0x6f5f34){return _0x3584db+_0x6f5f34;},'VEQSp':_0x2cd2('42d','NeH6'),'yxveR':_0x2cd2('42e','*unX'),'cyUXb':_0x2cd2('42f','8U%R'),'qMmuM':function(_0x5f119d,_0x307fe2){return _0x5f119d!==_0x307fe2;},'yZWSr':_0x2cd2('430','W@80'),'HRqIf':_0x2cd2('431','CFh^'),'liCVg':_0x2cd2('432','3psD'),'lfNMM':function(_0x1a8756,_0x3a1e11){return _0x1a8756===_0x3a1e11;},'yArNT':_0x2cd2('433','NeH6'),'naHXc':_0x2cd2('434','4[4f')};JHMW[_0x2cd2('d2','P0Kn')][_0x2cd2('435','3Y7y')]=$dataJHMW[_0x2cd2('436','W@80')];JHMW[_0x2cd2('119','1L8W')][_0x2cd2('437','3psD')]=$dataJHMW[_0x2cd2('438','gq!6')];JHMW[_0x2cd2('d7','@rQL')][_0x2cd2('439','W@80')]=$dataJHMW[_0x2cd2('43a','vbKI')];JHMW[_0x2cd2('177','3psD')][_0x2cd2('43b','4UjT')]=$dataJHMW[_0x2cd2('43c','YCbV')];JHMW[_0x2cd2('17e','y046')][_0x2cd2('43d','#iIl')]=$dataJHMW[_0x2cd2('43e','vB83')];JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('43f','#iIl')]=$dataJHMW[_0x2cd2('440','3Y7y')];JHMW[_0x2cd2('e5',')%R)')][_0x2cd2('441','y046')]=$dataJHMW[_0x2cd2('442','1L8W')];JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('443','wHIg')]=_0x4ef5b5[_0x2cd2('444','^)]V')](_0x4ef5b5[_0x2cd2('445','4[4f')](_0x4ef5b5[_0x2cd2('446','gq!6')](_0x4ef5b5[_0x2cd2('447','c!9y')](_0x4ef5b5[_0x2cd2('448','vbKI')],JHMW[_0x2cd2('449','GDPD')][_0x2cd2('44a','2^*B')]),_0x4ef5b5[_0x2cd2('44b','&)j!')]),JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('44c','gq!6')]),_0x4ef5b5[_0x2cd2('44d','4UjT')]);if(Imported[_0x2cd2('44e','8U%R')]){if(_0x4ef5b5[_0x2cd2('44f','c!9y')](_0x4ef5b5[_0x2cd2('450','vB83')],_0x4ef5b5[_0x2cd2('451','l%aI')])){var _0x28e1a6=_0x4ef5b5[_0x2cd2('452','ZboD')][_0x2cd2('b8','HHLp')]('|'),_0x4e367a=0x0;while(!![]){switch(_0x28e1a6[_0x4e367a++]){case'0':JHMW[_0x2cd2('453','*unX')][_0x2cd2('454','^)]V')]=$dataJHMW[_0x2cd2('455','1L8W')];continue;case'1':JHMW[_0x2cd2('123','vB83')][_0x2cd2('456','8vsk')]=$dataJHMW[_0x2cd2('457','vB83')];continue;case'2':JHMW[_0x2cd2('37d','3Y7y')][_0x2cd2('458','2^*B')]=$dataJHMW[_0x2cd2('459','W@80')];continue;case'3':JHMW[_0x2cd2('134','xWmq')][_0x2cd2('45a','[yfr')]=$dataJHMW[_0x2cd2('45b','3psD')];continue;case'4':JHMW[_0x2cd2('449','GDPD')][_0x2cd2('45c','^)]V')]=$dataJHMW[_0x2cd2('45d','#4SP')];continue;case'5':JHMW[_0x2cd2('45e','ZboD')][_0x2cd2('45f','@rQL')]=$dataJHMW[_0x2cd2('460','%E7b')];continue;case'6':JHMW[_0x2cd2('461','4[4f')][_0x2cd2('462','HHLp')]=$dataJHMW[_0x2cd2('463','HHLp')];continue;case'7':JHMW[_0x2cd2('464','3Y7y')][_0x2cd2('465','X1Zu')]=$dataJHMW[_0x2cd2('466','&)j!')];continue;case'8':JHMW[_0x2cd2('467','HHLp')][_0x2cd2('468','vbKI')]=$dataJHMW[_0x2cd2('469','g*Sm')];continue;case'9':JHMW[_0x2cd2('321','lDFd')][_0x2cd2('46a','xvh(')]=$dataJHMW[_0x2cd2('46b','qqEP')];continue;case'10':JHMW[_0x2cd2('45e','ZboD')][_0x2cd2('46c','%E7b')]=$dataJHMW[_0x2cd2('46d','3Y7y')];continue;case'11':JHMW[_0x2cd2('e1','*unX')][_0x2cd2('46e','@rQL')]=$dataJHMW[_0x2cd2('46f','%E7b')];continue;case'12':JHMW[_0x2cd2('470','R)LB')][_0x2cd2('471','HHLp')]=$dataJHMW[_0x2cd2('472','[yfr')];continue;case'13':JHMW[_0x2cd2('464','3Y7y')][_0x2cd2('473','4UjT')]=$dataJHMW[_0x2cd2('474','xWmq')];continue;case'14':JHMW[_0x2cd2('475','ghrv')][_0x2cd2('476','#iIl')]=$dataJHMW[_0x2cd2('477','8vsk')];continue;case'15':JHMW[_0x2cd2('e1','*unX')][_0x2cd2('478','l^iV')]=$dataJHMW[_0x2cd2('479','wHIg')];continue;}break;}}else{return JHMW[_0x2cd2('fc','W@80')][_0x2cd2('47a','W@80')][_0x2cd2('47b','jpvP')](function(_0x2bacca){return!!_0x2bacca&&JHMW[_0x2cd2('32c','X1Zu')][_0x2cd2('47c','8vsk')];});}}if(Imported[_0x2cd2('47d','wHIg')]){if(_0x4ef5b5[_0x2cd2('47e','8U%R')](_0x4ef5b5[_0x2cd2('47f','CFh^')],_0x4ef5b5[_0x2cd2('480','#fZb')])){var _0x762006=_0x4ef5b5[_0x2cd2('481','xWmq')][_0x2cd2('31f','^)]V')]('|'),_0x201a14=0x0;while(!![]){switch(_0x762006[_0x201a14++]){case'0':JHMW[_0x2cd2('482','4UjT')][_0x2cd2('483','4UjT')]=$dataJHMW[_0x2cd2('484','g*Sm')];continue;case'1':JHMW[_0x2cd2('e9','jpvP')][_0x2cd2('485','YCbV')]=$dataJHMW[_0x2cd2('486','NeH6')];continue;case'2':JHMW[_0x2cd2('134','xWmq')][_0x2cd2('487','GDPD')]=$dataJHMW[_0x2cd2('488','X1Zu')];continue;case'3':JHMW[_0x2cd2('449','GDPD')][_0x2cd2('489','R)LB')]=$dataJHMW[_0x2cd2('48a','jpvP')];continue;case'4':JHMW[_0x2cd2('ee','&)j!')][_0x2cd2('48b','obzi')]=$dataJHMW[_0x2cd2('48c','NeH6')];continue;}break;}}else{data=JsonEx[_0x2cd2('48d','LX!4')](LZString[_0x2cd2('48e','P0Kn')](localStorage[_0x2cd2('48f','P0Kn')](key)));}}};JHMW[_0x2cd2('490',')%R)')]=Graphics[_0x2cd2('491','[yfr')];Graphics[_0x2cd2('492','*unX')]=function(){JHMW[_0x2cd2('493','3psD')][_0x2cd2('494','#4SP')](this);this[_0x2cd2('495','jpvP')]();};JHMW[_0x2cd2('496','HHLp')]=Graphics[_0x2cd2('497','l%aI')];Graphics[_0x2cd2('498','g*Sm')]=function(){var _0x3133eb={'SuYYW':_0x2cd2('499','4UjT'),'qpRFA':_0x2cd2('49a','vB83'),'qqLxt':_0x2cd2('49b','dO1!')};this[_0x2cd2('49c','R)LB')](_0x3133eb[_0x2cd2('49d','8U%R')]);this[_0x2cd2('49e','^)]V')](_0x3133eb[_0x2cd2('49f','W@80')]);this[_0x2cd2('4a0','vB83')](_0x3133eb[_0x2cd2('4a1','^)]V')]);JHMW[_0x2cd2('4a2','UUq0')][_0x2cd2('4a3','Biy2')](this);};Graphics[_0x2cd2('4a4','wHIg')]=function(){var _0x19b382={'SdYXL':_0x2cd2('4a5','jpvP'),'rlKBu':_0x2cd2('4a6','8U%R'),'MmnJO':_0x2cd2('4a7','hS2h'),'SdDiJ':_0x2cd2('4a8','P0Kn'),'NXypD':_0x2cd2('4a9','NeH6'),'fTSdj':_0x2cd2('4aa','R)LB')};var _0x567fb8=_0x19b382[_0x2cd2('4ab','%E7b')][_0x2cd2('4ac','@rQL')]('|'),_0x598f23=0x0;while(!![]){switch(_0x567fb8[_0x598f23++]){case'0':this[_0x2cd2('4ad','xvh(')]['id']=_0x19b382[_0x2cd2('4ae','hS2h')];continue;case'1':this[_0x2cd2('4af','l%aI')][_0x2cd2('4b0','P0Kn')]={'xs':![],'x':0x0,'y':0x0,'width':0x64,'height':0x14,'fontSize':0x12};continue;case'2':this[_0x2cd2('4b1','ghrv')]=document[_0x2cd2('4b2','YCbV')](_0x19b382[_0x2cd2('4b3','[yfr')]);continue;case'3':this[_0x2cd2('4b4','CFh^')]['id']=_0x19b382[_0x2cd2('4b5','obzi')];continue;case'4':this[_0x2cd2('4b6','UUq0')][_0x2cd2('4b7','c!9y')]=_0x19b382[_0x2cd2('4b8','3psD')];continue;case'5':this[_0x2cd2('4b9',')%R)')][_0x2cd2('4ba','xvh(')]={'xs':![],'x':0x0,'y':0x0,'width':0x64,'height':0x14,'fontSize':0x12};continue;case'6':this[_0x2cd2('4bb','ZboD')][_0x2cd2('4bc','YCbV')]=_0x19b382[_0x2cd2('4bd','hS2h')];continue;case'7':this[_0x2cd2('4be','YCbV')]=document[_0x2cd2('4bf','4UjT')](_0x19b382[_0x2cd2('4c0','*unX')]);continue;case'8':this[_0x2cd2('4c1','NeH6')]['id']=_0x19b382[_0x2cd2('4c2','l%aI')];continue;case'9':this[_0x2cd2('4c3','3psD')][_0x2cd2('4c4','HHLp')]={'xs':![],'x':0x0,'y':0x0,'width':0x64,'height':0x14,'fontSize':0x12};continue;case'10':this[_0x2cd2('4c5','qqEP')]=document[_0x2cd2('4b2','YCbV')](_0x19b382[_0x2cd2('4c6','3Y7y')]);continue;case'11':this[_0x2cd2('4c7','2^*B')][_0x2cd2('4c8','dO1!')]=_0x19b382[_0x2cd2('4c9','ghrv')];continue;}break;}};Graphics[_0x2cd2('4ca','8vsk')]=function(_0x3284c2,_0x264492,_0x4b6d8b,_0xe366d4,_0x23918e,_0x1b90a7,_0x22ce99){var _0x1ff939={'xpTCm':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b==_0xe366d4;},'Oeivy':_0x2cd2('4cb','YCbV'),'BTSJq':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b||_0xe366d4;},'ZnQOH':_0x2cd2('4cc','4[4f'),'XPCqx':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b||_0xe366d4;},'bvEkq':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b==_0xe366d4;},'NzRFy':_0x2cd2('4cd','^)]V'),'ppYWt':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b||_0xe366d4;},'RApik':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b||_0xe366d4;},'kvDmd':_0x2cd2('4ce','CFh^'),'bZwLM':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b||_0xe366d4;},'JPxIL':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b||_0xe366d4;},'VcEwy':function(_0x4b6d8b,_0xe366d4){return _0x4b6d8b||_0xe366d4;}};if(_0x1ff939[_0x2cd2('4cf','^)]V')](_0x3284c2,_0x1ff939[_0x2cd2('4d0','2^*B')])){this[_0x2cd2('4d1','ghrv')][_0x2cd2('4d2','Biy2')]=_0x1ff939[_0x2cd2('4d3','[yfr')](_0x264492,_0x1ff939[_0x2cd2('4d4','obzi')]);this[_0x2cd2('4d5','lDFd')][_0x2cd2('4d6','1L8W')]={'xs':!![],'x':_0x4b6d8b,'y':_0xe366d4,'width':_0x1ff939[_0x2cd2('4d7','l^iV')](_0x23918e,0x64),'height':_0x1ff939[_0x2cd2('4d8','YCbV')](_0x1b90a7,0x14),'fontSize':_0x1ff939[_0x2cd2('4d9','CFh^')](_0x22ce99,0x12)};this[_0x2cd2('4da','vnIt')](_0x1ff939[_0x2cd2('4db','W@80')]);document[_0x2cd2('4dc','vB83')][_0x2cd2('4dd','GDPD')](this[_0x2cd2('4de','%E7b')]);}else if(_0x1ff939[_0x2cd2('4df','hS2h')](_0x3284c2,_0x1ff939[_0x2cd2('4e0','1L8W')])){this[_0x2cd2('4e1','CFh^')][_0x2cd2('4e2','[yfr')]=_0x1ff939[_0x2cd2('4e3','ghrv')](_0x264492,_0x1ff939[_0x2cd2('4e4','4UjT')]);this[_0x2cd2('4e1','CFh^')][_0x2cd2('4e5','4UjT')]={'xs':!![],'x':_0x4b6d8b,'y':_0xe366d4,'width':_0x1ff939[_0x2cd2('4e6','y046')](_0x23918e,0x64),'height':_0x1ff939[_0x2cd2('4e7','R)LB')](_0x1b90a7,0x14),'fontSize':_0x1ff939[_0x2cd2('4e8','c!9y')](_0x22ce99,0x12)};this[_0x2cd2('4e9','jpvP')](_0x1ff939[_0x2cd2('4ea','y046')]);document[_0x2cd2('4eb','UUq0')][_0x2cd2('4ec','#4SP')](this[_0x2cd2('4ed','l%aI')]);}else if(_0x1ff939[_0x2cd2('4ee','X1Zu')](_0x3284c2,_0x1ff939[_0x2cd2('4ef','CFh^')])){this[_0x2cd2('4f0','*unX')][_0x2cd2('4f1','xvh(')]=_0x1ff939[_0x2cd2('4f2','*unX')](_0x264492,_0x1ff939[_0x2cd2('4f3','vnIt')]);this[_0x2cd2('4f4','W@80')][_0x2cd2('4f5','8vsk')]={'xs':!![],'x':_0x4b6d8b,'y':_0xe366d4,'width':_0x1ff939[_0x2cd2('4f6','GDPD')](_0x23918e,0x64),'height':_0x1ff939[_0x2cd2('4f7','g*Sm')](_0x1b90a7,0x14),'fontSize':_0x1ff939[_0x2cd2('4f8','1L8W')](_0x22ce99,0x12)};this[_0x2cd2('4f9','8U%R')](_0x1ff939[_0x2cd2('4fa','R)LB')]);document[_0x2cd2('4fb','3Y7y')][_0x2cd2('4fc','W@80')](this[_0x2cd2('4fd','3Y7y')]);}};Graphics[_0x2cd2('4fe','vB83')]=function(_0xc65ddb){var _0x386561={'JlRyZ':function(_0x16a304,_0x5177eb){return _0x16a304+_0x5177eb;},'ZZjiJ':function(_0x496efd,_0x5548e7){return _0x496efd+_0x5548e7;},'ZzXhF':function(_0x28cedf,_0x261487){return _0x28cedf*_0x261487;},'VpSvn':_0x2cd2('4ff','&)j!'),'kqIjv':function(_0x3d9ab2,_0x4e381e){return _0x3d9ab2*_0x4e381e;},'nXtoW':function(_0x5c9bf5,_0x9c4ac9){return _0x5c9bf5==_0x9c4ac9;},'KEmVz':_0x2cd2('500','4[4f'),'RHOCD':_0x2cd2('501','[yfr'),'nrdGe':function(_0x1af848,_0x167895){return _0x1af848===_0x167895;},'pfSZv':_0x2cd2('502','8vsk'),'JPbpC':_0x2cd2('503','c!9y')};if(_0x386561[_0x2cd2('504','lDFd')](_0xc65ddb,_0x386561[_0x2cd2('505','g*Sm')])){this[_0x2cd2('506','X1Zu')][_0x2cd2('507','#iIl')]();this[_0x2cd2('4d1','ghrv')][_0x2cd2('508','4UjT')]=null;this[_0x2cd2('509','4UjT')][_0x2cd2('50a','ZboD')]['xs']=![];}else if(_0x386561[_0x2cd2('50b','NeH6')](_0xc65ddb,_0x386561[_0x2cd2('50c','vnIt')])){if(_0x386561[_0x2cd2('50d','4UjT')](_0x386561[_0x2cd2('50e','jpvP')],_0x386561[_0x2cd2('50f','#iIl')])){var _0x2b69b0=_0x386561[_0x2cd2('510','3Y7y')](_0x386561[_0x2cd2('511','g*Sm')](JHMW[_0x2cd2('270','vB83')][_0x2cd2('1a2','HHLp')][_0x386561[_0x2cd2('512','[yfr')](_0x386561[_0x2cd2('513','CFh^')](i,0xb),0x1)],_0x386561[_0x2cd2('514','g*Sm')]),JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('515','gq!6')][_0x386561[_0x2cd2('516',')%R)')](_0x386561[_0x2cd2('517','UUq0')](i,0xb),0xa)]);Notification[_0x2cd2('2e5','vnIt')](_0x2b69b0);}else{this[_0x2cd2('518','hS2h')][_0x2cd2('519','ghrv')]();this[_0x2cd2('51a','c!9y')][_0x2cd2('415','R)LB')]=null;this[_0x2cd2('51b','[yfr')][_0x2cd2('51c','W@80')]['xs']=![];}}else{this[_0x2cd2('51d','jpvP')][_0x2cd2('51e','Biy2')]();this[_0x2cd2('4fd','3Y7y')][_0x2cd2('51f','&)j!')]=null;this[_0x2cd2('520','P0Kn')][_0x2cd2('521','X1Zu')]['xs']=![];}};Graphics[_0x2cd2('522','W@80')]=function(_0x46f4f5){var _0x203457={'VCxqu':_0x2cd2('523','NeH6'),'PpcEO':function(_0x197bd9,_0x4d7ecf){return _0x197bd9*_0x4d7ecf;},'OkHTY':function(_0x377589,_0x294264){return _0x377589+_0x294264;},'EsGbO':function(_0x433e4f,_0x286d78){return _0x433e4f*_0x286d78;},'siACc':function(_0x29e791,_0x2f37f9){return _0x29e791==_0x2f37f9;},'BZxxM':_0x2cd2('524','vnIt'),'WHVvU':_0x2cd2('525','jpvP'),'gEtcU':function(_0x582ea9,_0x159d78){return _0x582ea9+_0x159d78;},'EHpay':function(_0x39c7c7,_0x17a979){return _0x39c7c7/_0x17a979;},'SERtG':function(_0x5cb019,_0xdc9064){return _0x5cb019-_0xdc9064;},'CeOJe':function(_0x110632,_0x802862){return _0x110632*_0x802862;},'uVyKD':_0x2cd2('526','@rQL'),'kFSMo':function(_0x491ec4,_0x717069){return _0x491ec4+_0x717069;},'JlIbX':_0x2cd2('527','Biy2'),'fIOsW':function(_0x3de931,_0x5eaaf2){return _0x3de931*_0x5eaaf2;}};var _0x4843ee=_0x203457[_0x2cd2('528','CFh^')][_0x2cd2('40d','4UjT')]('|'),_0x1f812b=0x0;while(!![]){switch(_0x4843ee[_0x1f812b++]){case'0':var _0x42fb52=_0x203457[_0x2cd2('529','[yfr')](_0x53a2c2[_0x2cd2('52a','NeH6')][_0x2cd2('52b','#iIl')],this[_0x2cd2('52c','Biy2')]);continue;case'1':_0x53a2c2[_0x2cd2('52d','#4SP')][_0x2cd2('52e','YCbV')]=_0x203457[_0x2cd2('52f','3Y7y')](_0x545cde,'px');continue;case'2':_0x53a2c2[_0x2cd2('530','xvh(')][_0x2cd2('531','c!9y')]=_0x203457[_0x2cd2('532','*unX')](_0x2fa975,'px');continue;case'3':var _0x545cde=_0x203457[_0x2cd2('533','#4SP')](_0x53a2c2[_0x2cd2('534','l%aI')][_0x2cd2('535','NeH6')],this[_0x2cd2('536','qqEP')]);continue;case'4':_0x53a2c2[_0x2cd2('537','vnIt')][_0x2cd2('538','X1Zu')]=0xc;continue;case'5':if(_0x203457[_0x2cd2('539','W@80')](_0x46f4f5,_0x203457[_0x2cd2('53a','LX!4')]))var _0x53a2c2=this[_0x2cd2('53b','GDPD')];else if(_0x203457[_0x2cd2('53c','lDFd')](_0x46f4f5,_0x203457[_0x2cd2('53d','#4SP')]))var _0x53a2c2=this[_0x2cd2('53e','g*Sm')];else var _0x53a2c2=this[_0x2cd2('53f','g*Sm')];continue;case'6':var _0x262739=_0x203457[_0x2cd2('540','NeH6')](_0x203457[_0x2cd2('541','xvh(')](_0x53a2c2[_0x2cd2('542','#4SP')]['y'],this[_0x2cd2('543','vnIt')]),_0x203457[_0x2cd2('544','%E7b')](_0x203457[_0x2cd2('545','X1Zu')](window[_0x2cd2('546','Biy2')],_0x203457[_0x2cd2('547','l^iV')](this[_0x2cd2('548','ghrv')],this[_0x2cd2('549','CFh^')])),0x2));continue;case'7':if(JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('54a','#iIl')])_0x53a2c2[_0x2cd2('54b','3Y7y')][_0x2cd2('54c','2^*B')]=JHMW[_0x2cd2('177','3psD')][_0x2cd2('54d','l%aI')];continue;case'8':_0x53a2c2[_0x2cd2('54e','ZboD')][_0x2cd2('54f','wHIg')]=_0x203457[_0x2cd2('550','ZboD')](_0x262739,'px');continue;case'9':var _0x2b58b9=_0x203457[_0x2cd2('551','GDPD')](_0x53a2c2[_0x2cd2('552','&)j!')][_0x2cd2('553','8U%R')],this[_0x2cd2('554',')%R)')]);continue;case'10':_0x53a2c2[_0x2cd2('555','NeH6')][_0x2cd2('556','CFh^')]=_0x203457[_0x2cd2('557','wHIg')](_0x2b58b9,'px');continue;case'11':_0x53a2c2[_0x2cd2('558','2^*B')][_0x2cd2('559','X1Zu')]=_0x203457[_0x2cd2('55a','CFh^')];continue;case'12':_0x53a2c2[_0x2cd2('55b','&)j!')][_0x2cd2('55c','wHIg')]=_0x203457[_0x2cd2('55d','#fZb')](_0x42fb52,'px');continue;case'13':_0x53a2c2[_0x2cd2('55e','xWmq')][_0x2cd2('55f','hS2h')]=_0x203457[_0x2cd2('560','NeH6')];continue;case'14':var _0x2fa975=_0x203457[_0x2cd2('561','NeH6')](_0x203457[_0x2cd2('562','gq!6')](_0x53a2c2[_0x2cd2('521','X1Zu')]['x'],this[_0x2cd2('563','xvh(')]),_0x203457[_0x2cd2('564','GDPD')](_0x203457[_0x2cd2('565','hS2h')](window[_0x2cd2('566','UUq0')],_0x203457[_0x2cd2('567','#iIl')](this[_0x2cd2('568','c!9y')],this[_0x2cd2('554',')%R)')])),0x2));continue;}break;}};Input[_0x2cd2('569','GDPD')]=function(_0xed79bd){switch(_0xed79bd){case 0x8:case 0x9:case 0x21:case 0x22:case 0x25:case 0x26:case 0x27:case 0x28:return!![];}return![];};Input[_0x2cd2('56a','#4SP')]=function(_0x14a468){var _0x405771={'vmgSh':function(_0x36a3fe,_0x16c5a9){return _0x36a3fe===_0x16c5a9;},'KLyhT':function(_0x1bf152,_0x558e9a){return _0x1bf152==_0x558e9a;},'wBXmb':function(_0x176d4e,_0x21f698){return _0x176d4e>=_0x21f698;},'figfo':function(_0xceafd2,_0x3d835b){return _0xceafd2(_0x3d835b);},'GefTI':function(_0x5ce0b8,_0x4237a8){return _0x5ce0b8===_0x4237a8;}};if(this[_0x2cd2('56b','X1Zu')](_0x14a468[_0x2cd2('56c','W@80')])){if(Graphics&&(!Graphics[_0x2cd2('56d','R)LB')][_0x2cd2('56e','LX!4')]['xs']&&!Graphics[_0x2cd2('56f','#fZb')][_0x2cd2('570','@rQL')]['xs']&&!Graphics[_0x2cd2('571','lDFd')][_0x2cd2('542','#4SP')]['xs']))_0x14a468[_0x2cd2('572','xvh(')]();}if(_0x405771[_0x2cd2('573','vB83')](_0x14a468[_0x2cd2('574','^)]V')],0x90))this[_0x2cd2('575','R)LB')]();var _0x184e9b=this[_0x2cd2('576','jpvP')][_0x14a468[_0x2cd2('577','R)LB')]];if(_0x405771[_0x2cd2('578','YCbV')](Utils[_0x2cd2('579','CFh^')],'MV')&&_0x405771[_0x2cd2('57a','P0Kn')](_0x405771[_0x2cd2('57b','vbKI')](parseInt,Utils[_0x2cd2('57c','4UjT')][_0x2cd2('57d','xvh(')](0x2,0x1)),0x5)){if(ResourceHandler[_0x2cd2('57e','LX!4')]()&&_0x405771[_0x2cd2('57f','gq!6')](_0x184e9b,'ok'))ResourceHandler[_0x2cd2('580','vbKI')]();else if(_0x184e9b)this[_0x2cd2('581','qqEP')][_0x184e9b]=!![];}else{if(_0x184e9b)this[_0x2cd2('582','c!9y')][_0x184e9b]=!![];}};TouchInput[_0x2cd2('583','CFh^')]=function(){var _0x523361={'OVUln':_0x2cd2('584','vB83'),'cipXE':_0x2cd2('585','Biy2'),'gzhhi':_0x2cd2('586','c!9y'),'JVWem':_0x2cd2('587','&)j!'),'AXEBT':function(_0x53bc2f,_0xe1a0d1){return _0x53bc2f==_0xe1a0d1;},'qPhVa':_0x2cd2('588','LX!4'),'nxxvj':_0x2cd2('589',')%R)'),'wfkxp':_0x2cd2('58a','W@80'),'zugpY':_0x2cd2('58b','%E7b'),'jFaEO':function(_0x585b6c,_0x3ca4f4){return _0x585b6c==_0x3ca4f4;},'zvBtz':_0x2cd2('58c','%E7b'),'OZeSr':_0x2cd2('58d','*unX'),'zdWAK':_0x2cd2('58e','dO1!')};var _0x10de51=_0x523361[_0x2cd2('58f','4[4f')][_0x2cd2('428','c!9y')]('|'),_0x1b12aa=0x0;while(!![]){switch(_0x10de51[_0x1b12aa++]){case'0':document[_0x2cd2('590','l^iV')](_0x523361[_0x2cd2('591','#4SP')],this[_0x2cd2('592','^)]V')][_0x2cd2('593','W@80')](this));continue;case'1':document[_0x2cd2('594','3psD')](_0x523361[_0x2cd2('595','GDPD')],this[_0x2cd2('596','vB83')][_0x2cd2('597','8vsk')](this));continue;case'2':document[_0x2cd2('598','c!9y')](_0x523361[_0x2cd2('599','ZboD')],this[_0x2cd2('59a','vbKI')][_0x2cd2('59b','l^iV')](this));continue;case'3':if(_0x523361[_0x2cd2('59c','*unX')](Utils[_0x2cd2('59d','l^iV')],'MZ')){var _0x179526={'passive':![]};document[_0x2cd2('59e','W@80')](_0x523361[_0x2cd2('59f','hS2h')],this[_0x2cd2('5a0','vnIt')][_0x2cd2('5a1','ZboD')](this),_0x179526);window[_0x2cd2('5a2','#4SP')](_0x523361[_0x2cd2('5a3','vB83')],this[_0x2cd2('5a4','4[4f')][_0x2cd2('5a5','xvh(')](this));}else document[_0x2cd2('5a6','xvh(')](_0x523361[_0x2cd2('5a7','^)]V')],this[_0x2cd2('5a8','UUq0')][_0x2cd2('5a9','#iIl')](this));continue;case'4':document[_0x2cd2('5aa','4UjT')](_0x523361[_0x2cd2('5ab','2^*B')],this[_0x2cd2('5ac','#iIl')][_0x2cd2('5ad','[yfr')](this));continue;case'5':document[_0x2cd2('5ae','^)]V')](_0x523361[_0x2cd2('5af','g*Sm')],this[_0x2cd2('5b0','4UjT')][_0x2cd2('5b1','8U%R')](this));continue;case'6':if(_0x523361[_0x2cd2('5b2','X1Zu')](Utils[_0x2cd2('5b3','3Y7y')],'MV'))document[_0x2cd2('5b4','vnIt')](_0x523361[_0x2cd2('5b5','4[4f')],this[_0x2cd2('5b6','W@80')][_0x2cd2('5b7','lDFd')](this));continue;case'7':document[_0x2cd2('5b8','vB83')](_0x523361[_0x2cd2('5b9','obzi')],this[_0x2cd2('5ba','2^*B')][_0x2cd2('597','8vsk')](this));continue;case'8':document[_0x2cd2('5a2','#4SP')](_0x523361[_0x2cd2('5bb','2^*B')],this[_0x2cd2('5bc','%E7b')][_0x2cd2('5bd','3psD')](this));continue;}break;}};TouchInput[_0x2cd2('5be','#fZb')]=function(){var _0x2073e8={'pMxiA':function(_0x239dcd,_0x3506b4){return _0x239dcd==_0x3506b4;},'QVSLR':_0x2cd2('5bf','CFh^'),'uWFKq':_0x2cd2('5c0','l%aI'),'MglLE':function(_0x5709ca,_0x205fb3){return _0x5709ca===_0x205fb3;},'XFWGg':_0x2cd2('5c1','3Y7y')};document[_0x2cd2('5c2','Biy2')](_0x2073e8[_0x2cd2('5c3',')%R)')],this[_0x2cd2('5c4','LX!4')][_0x2cd2('5c5','HHLp')](this));document[_0x2cd2('5c6','^)]V')](_0x2073e8[_0x2cd2('5c7','P0Kn')],this[_0x2cd2('5c8','vbKI')][_0x2cd2('5c9','wHIg')](this));if(_0x2073e8[_0x2cd2('5ca','W@80')](Utils[_0x2cd2('5cb',')%R)')],'MZ')){var _0x56838e={'passive':![]};document[_0x2cd2('5cc','Biy2')](_0x2073e8[_0x2cd2('5cd','P0Kn')],this[_0x2cd2('5ce','l%aI')][_0x2cd2('2cd','&)j!')](this),_0x56838e);document[_0x2cd2('5cf','obzi')](_0x2073e8[_0x2cd2('5d0','*unX')],this[_0x2cd2('5d1','3psD')][_0x2cd2('5d2','1L8W')](this),_0x56838e);}else{if(_0x2073e8[_0x2cd2('5d3','NeH6')](_0x2073e8[_0x2cd2('5d4','NeH6')],_0x2073e8[_0x2cd2('5d5','8U%R')])){var _0x2c734a=Utils[_0x2cd2('5d6','UUq0')]();document[_0x2cd2('5d7','X1Zu')](_0x2073e8[_0x2cd2('5d8','[yfr')],this[_0x2cd2('5d9','GDPD')][_0x2cd2('5da','%E7b')](this),_0x2c734a?{'passive':![]}:![]);document[_0x2cd2('5cf','obzi')](_0x2073e8[_0x2cd2('5db',')%R)')],this[_0x2cd2('5dc','[yfr')][_0x2cd2('5dd','g*Sm')](this),_0x2c734a?{'passive':![]}:![]);}else{if(JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('5de','hS2h')]){this[_0x2cd2('5df','W@80')][_0x2cd2('5e0','obzi')]=0x64;if(_0x2073e8[_0x2cd2('5e1',')%R)')](Utils[_0x2cd2('5e2','R)LB')],'MV'))var _0x34e7f5=this[_0x2cd2('5e3','^)]V')]();else var _0x34e7f5=ColorManager[_0x2cd2('5e4','xWmq')]();this[_0x2cd2('5e5','l^iV')][_0x2cd2('5e6','vB83')](0x0,y,this[_0x2cd2('5e7','vbKI')](),0x2,_0x34e7f5);this[_0x2cd2('5e8','obzi')][_0x2cd2('5e9','GDPD')]=0xff;}}}};TouchInput[_0x2cd2('5ea','R)LB')]=function(){var _0x4b39b6={'RgoCx':function(_0x3ea7fd,_0x154f44){return _0x3ea7fd instanceof _0x154f44;},'APpCN':function(_0x53159b,_0x40fb07){return _0x53159b>=_0x40fb07;},'TKQMp':function(_0x29099c,_0x435e2d){return _0x29099c>=_0x435e2d;},'ZxeRh':function(_0xe3d148,_0x653046){return _0xe3d148<=_0x653046;},'rGthh':function(_0x10d766,_0x34e759){return _0x10d766+_0x34e759;},'Pbgcg':function(_0x3acf21,_0x2f8427){return _0x3acf21+_0x2f8427;},'FhoEi':function(_0x75e29c,_0x5b5aad){return _0x75e29c>=_0x5b5aad;},'dMbrp':function(_0x452120,_0x29f1c3){return _0x452120<=_0x29f1c3;},'kZSCE':function(_0x404240,_0x409761){return _0x404240>=_0x409761;},'esiPL':function(_0x3419e7,_0x33717e){return _0x3419e7>=_0x33717e;},'EfxPf':function(_0xbfdf72,_0x44e55e){return _0xbfdf72-_0x44e55e;},'cpSHg':function(_0x5dbf1d,_0x1d948f){return _0x5dbf1d<=_0x1d948f;},'QANaf':function(_0x1188f1,_0x23ef1f){return _0x1188f1+_0x23ef1f;},'HpNJn':function(_0x2fb7d8,_0x1aeb0d){return _0x2fb7d8+_0x1aeb0d;}};if(_0x4b39b6[_0x2cd2('5eb','l^iV')](SceneManager[_0x2cd2('5ec','hS2h')],Scene_Map)){var _0x14eb8a=SceneManager[_0x2cd2('5ed','[yfr')][_0x2cd2('5ee','vB83')];var _0x16ba9d=SceneManager[_0x2cd2('5ef','Biy2')][_0x2cd2('5f0','g*Sm')];var _0x527c23=Graphics[_0x2cd2('5f1','#iIl')];if(_0x14eb8a&&_0x4b39b6[_0x2cd2('5f2','gq!6')](this['x'],_0x14eb8a['x'])&&_0x4b39b6[_0x2cd2('5f3','R)LB')](this['y'],_0x14eb8a['y'])&&_0x4b39b6[_0x2cd2('5f4','^)]V')](this['x'],_0x4b39b6[_0x2cd2('5f5','vbKI')](_0x14eb8a['x'],_0x14eb8a[_0x2cd2('5f6','dO1!')]))&&_0x4b39b6[_0x2cd2('5f7','obzi')](this['y'],_0x4b39b6[_0x2cd2('5f8','UUq0')](_0x14eb8a['y'],_0x14eb8a[_0x2cd2('5f9','l^iV')])))return!![];else if(_0x16ba9d&&_0x4b39b6[_0x2cd2('5fa','P0Kn')](this['x'],_0x16ba9d['x'])&&_0x4b39b6[_0x2cd2('5fb','c!9y')](this['y'],_0x16ba9d['y'])&&_0x4b39b6[_0x2cd2('5fc','vbKI')](this['x'],_0x4b39b6[_0x2cd2('5fd','#4SP')](_0x16ba9d['x'],_0x16ba9d[_0x2cd2('5fe','qqEP')]))&&_0x4b39b6[_0x2cd2('5ff','R)LB')](this['y'],_0x4b39b6[_0x2cd2('600','@rQL')](_0x16ba9d['y'],_0x16ba9d[_0x2cd2('601','vnIt')])))return!![];else if(_0x527c23[_0x2cd2('602','[yfr')]['xs']&&_0x4b39b6[_0x2cd2('603','UUq0')](this['x'],_0x527c23[_0x2cd2('604','^)]V')]['x'])&&_0x4b39b6[_0x2cd2('605','c!9y')](this['y'],_0x4b39b6[_0x2cd2('606','#fZb')](_0x527c23[_0x2cd2('607','#iIl')]['y'],0xa))&&_0x4b39b6[_0x2cd2('608','dO1!')](this['x'],_0x4b39b6[_0x2cd2('609','y046')](_0x527c23[_0x2cd2('4c4','HHLp')]['x'],_0x527c23[_0x2cd2('60a','YCbV')][_0x2cd2('5f6','dO1!')]))&&_0x4b39b6[_0x2cd2('60b','%E7b')](this['y'],_0x4b39b6[_0x2cd2('60c','c!9y')](_0x4b39b6[_0x2cd2('60d','obzi')](_0x527c23[_0x2cd2('60e','c!9y')]['y'],_0x527c23[_0x2cd2('60f','UUq0')][_0x2cd2('556','CFh^')]),0xa)))return!![];}return![];};DataManager[_0x2cd2('610','ZboD')]=function(){var _0x46bfc3={'DQBnS':_0x2cd2('611',')%R)'),'WQTpM':_0x2cd2('612','P0Kn'),'vAPHW':function(_0x31fb32,_0x39018e){return _0x31fb32+_0x39018e;},'MoIeT':function(_0x54406c,_0x183b9f){return _0x54406c+_0x183b9f;},'rDpgp':_0x2cd2('613','qqEP'),'LUhVq':function(_0x20d22b,_0x4dc896){return _0x20d22b(_0x4dc896);},'odeBJ':_0x2cd2('614','CFh^'),'QhKnz':function(_0x2c0db4,_0x5305a8){return _0x2c0db4(_0x5305a8);}};var _0x16fe16=_0x46bfc3[_0x2cd2('615','jpvP')][_0x2cd2('616','%E7b')]('|'),_0xd43385=0x0;while(!![]){switch(_0x16fe16[_0xd43385++]){case'0':if(!Utils[_0x2cd2('617','GDPD')]()||!Utils[_0x2cd2('618','dO1!')](_0x46bfc3[_0x2cd2('619','y046')]))return!![];continue;case'1':var _0x25c9f0=_0x34d8ce[_0x2cd2('61a','qqEP')](process[_0x2cd2('61b','P0Kn')][_0x2cd2('61c','dO1!')]);continue;case'2':var _0x32102e=JHMW[_0x2cd2('17e','y046')][_0x2cd2('61d','*unX')][_0x2cd2('227','qqEP')]('_');continue;case'3':return!_0x5dfab4[_0x2cd2('61e','8vsk')](_0x34d8ce[_0x2cd2('61f','ZboD')](_0x25c9f0,_0x46bfc3[_0x2cd2('620','[yfr')](_0x46bfc3[_0x2cd2('621','ghrv')](_0x46bfc3[_0x2cd2('622','2^*B')],_0x32102e[0x1]),'/')));case'4':var _0x34d8ce=_0x46bfc3[_0x2cd2('623','8vsk')](require,_0x46bfc3[_0x2cd2('624','wHIg')]);continue;case'5':var _0x5dfab4=_0x46bfc3[_0x2cd2('625','obzi')](require,'fs');continue;}break;}};DataManager[_0x2cd2('626',')%R)')]=function(_0xe7eb81){var _0x592763=LZString[_0x2cd2('627','l^iV')](_0xe7eb81);this[_0x2cd2('628','NeH6')]();this[_0x2cd2('629','c!9y')](JsonEx[_0x2cd2('62a','HHLp')](_0x592763));return!![];};DataManager[_0x2cd2('62b','gq!6')]=function(_0x166df1){var _0x10237e={'QjLCA':function(_0x3cf9c9,_0x559a8c){return _0x3cf9c9==_0x559a8c;},'WCByV':function(_0x128fff,_0x31b78a){return _0x128fff!==_0x31b78a;},'wItxJ':_0x2cd2('62c','8U%R'),'KLwxB':_0x2cd2('62d','wHIg'),'TaJPw':function(_0x4a9459,_0x449277){return _0x4a9459(_0x449277);},'PFQCt':function(_0x251d01,_0xec0f43){return _0x251d01===_0xec0f43;},'UAoOc':_0x2cd2('62e','vB83'),'bCHgX':function(_0xe44eb1,_0x4d1116){return _0xe44eb1(_0x4d1116);}};JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('3ed','gq!6')]=null;JHMW[_0x2cd2('29','4UjT')][_0x2cd2('62f','l^iV')]=null;JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('630','[yfr')]=[];JHMW[_0x2cd2('61','HHLp')][_0x2cd2('631','obzi')]=[];JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('632','ZboD')]=[];_0x10237e[_0x2cd2('633','xvh(')](clearInterval,JHMW[_0x2cd2('634','YCbV')]);var _0x5c84fe=this[_0x2cd2('635','#iIl')](_0x166df1);var _0x47920e=StorageManager[_0x2cd2('636','&)j!')](_0x5c84fe);return new Promise((_0x241ff0,_0x5ebaad)=>{var _0x127897={'BkJoz':function(_0x2bc0d5,_0x17a332){return _0x10237e[_0x2cd2('637','3Y7y')](_0x2bc0d5,_0x17a332);}};if(_0x10237e[_0x2cd2('638','qqEP')](_0x10237e[_0x2cd2('639','^)]V')],_0x10237e[_0x2cd2('63a','8U%R')])){var _0x26f643=StorageManager[_0x2cd2('63b','vbKI')](_0x47920e);if(_0x26f643){_0x10237e[_0x2cd2('63c','vB83')](_0x241ff0,_0x26f643);}else{if(_0x10237e[_0x2cd2('63d','4UjT')](_0x10237e[_0x2cd2('63e','UUq0')],_0x10237e[_0x2cd2('63f','3psD')])){_0x10237e[_0x2cd2('640','l%aI')](_0x5ebaad,e);}else{var _0x2db73f={'prope':function(_0x4d77a8,_0x199a60){return _0x127897[_0x2cd2('641','P0Kn')](_0x4d77a8,_0x199a60);}};return JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('642','2^*B')][_0x2cd2('643','2^*B')](function(_0x25a984){return _0x2db73f[_0x2cd2('644','gq!6')](_0x25a984[_0x2cd2('645','g*Sm')],user);});}}}else{this[_0x2cd2('646','^)]V')][_0x2cd2('647','lDFd')]();this[_0x2cd2('648','dO1!')]();}})[_0x2cd2('649','X1Zu')](_0x2ecd7a=>LZString[_0x2cd2('64a','3Y7y')](_0x2ecd7a))[_0x2cd2('64b','#fZb')](_0x4575ad=>StorageManager[_0x2cd2('64c','#iIl')](_0x4575ad))[_0x2cd2('64d','Biy2')](_0x3df641=>{this[_0x2cd2('64e','hS2h')]();this[_0x2cd2('64f','vbKI')](_0x3df641);this[_0x2cd2('650','vbKI')]();});};DataManager[_0x2cd2('651','P0Kn')]=function(){var _0x17e0c0={'EGQQT':_0x2cd2('652','8vsk'),'HQoyC':function(_0x441812,_0x4d7bb3){return _0x441812===_0x4d7bb3;},'KxaIr':_0x2cd2('653','#iIl'),'WYNFm':function(_0x513155,_0x5103e4){return _0x513155(_0x5103e4);}};PluginManager[_0x2cd2('654','l^iV')]=[];this[_0x2cd2('655','LX!4')][_0x2cd2('656','#iIl')](function(_0x5aa936){if(_0x17e0c0[_0x2cd2('657','4[4f')](_0x17e0c0[_0x2cd2('658','NeH6')],_0x17e0c0[_0x2cd2('659','qqEP')])){var _0x517fe9=_0x17e0c0[_0x2cd2('65a','jpvP')](eval,_0x5aa936[_0x2cd2('65b','W@80')]);PluginManager[_0x2cd2('65c','dO1!')][_0x2cd2('65d','P0Kn')](JHMW[_0x2cd2('65e','jpvP')](JSON[_0x2cd2('65f','xvh(')](_0x517fe9)));}else{var _0x1d05d6=!![];this[_0x2cd2('65','NeH6')](JHMW[_0x2cd2('148','xvh(')][_0x2cd2('660','qqEP')],_0x17e0c0[_0x2cd2('661','l^iV')],_0x1d05d6);}});};DataManager[_0x2cd2('662','3psD')]=function(){var _0x97dcfb={'SMSfN':function(_0x4b0e0c,_0x2c6abe){return _0x4b0e0c==_0x2c6abe;}};return JHMW[_0x2cd2('380','CFh^')][_0x2cd2('663','wHIg')][_0x2cd2('664','vB83')](function(_0x57bbcc){return _0x97dcfb[_0x2cd2('665','P0Kn')](_0x57bbcc[_0x2cd2('666','&)j!')][0x0][_0x2cd2('667','R)LB')],$gameMap[_0x2cd2('668','3psD')]());});};DataManager[_0x2cd2('669','qqEP')]=function(_0x2b1188){var _0x41b226={'YljHz':function(_0x51e451,_0x118b61){return _0x51e451/_0x118b61;},'WOdnG':function(_0x284184,_0x526ab4){return _0x284184-_0x526ab4;},'DnuDB':function(_0x534451,_0xe7a4f){return _0x534451<=_0xe7a4f;},'cVydK':function(_0x3f888f,_0x31ac51){return _0x3f888f+_0x31ac51;},'DTtYj':function(_0x12507c,_0x3b0eb7){return _0x12507c*_0x3b0eb7;},'JpiOu':_0x2cd2('66a','3psD'),'DBgOD':function(_0x430459,_0x2503be){return _0x430459===_0x2503be;},'TfMdO':_0x2cd2('66b','jpvP'),'opdTK':_0x2cd2('66c','ghrv'),'zjgmi':function(_0x24adc5,_0x1d5daf){return _0x24adc5==_0x1d5daf;}};return JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('66d','YCbV')][_0x2cd2('66e','vbKI')](function(_0x10d033){var _0x38d1e1={'EnfqF':function(_0x53b06c,_0x14e03c){return _0x41b226[_0x2cd2('66f','LX!4')](_0x53b06c,_0x14e03c);},'wOYKV':function(_0x35ffa6,_0x42fd5b){return _0x41b226[_0x2cd2('670','vnIt')](_0x35ffa6,_0x42fd5b);},'Qftju':function(_0x401e10,_0x194372){return _0x41b226[_0x2cd2('671','vB83')](_0x401e10,_0x194372);},'yaWyU':function(_0x1f7ad3,_0x3952dd){return _0x41b226[_0x2cd2('672','ghrv')](_0x1f7ad3,_0x3952dd);},'OhrNj':function(_0x309110,_0x2e25b2){return _0x41b226[_0x2cd2('673','ghrv')](_0x309110,_0x2e25b2);},'LiOJU':function(_0x10bd0e,_0x22db4e){return _0x41b226[_0x2cd2('674','#iIl')](_0x10bd0e,_0x22db4e);},'kpeZu':_0x41b226[_0x2cd2('675','W@80')]};if(_0x41b226[_0x2cd2('676','P0Kn')](_0x41b226[_0x2cd2('677','@rQL')],_0x41b226[_0x2cd2('678','8U%R')])){var _0x195c70=Math[_0x2cd2('679','vB83')](_0x38d1e1[_0x2cd2('67a','8vsk')](_0x38d1e1[_0x2cd2('67b','y046')](line2,line1),this[_0x2cd2('67c','#iIl')]()));var _0x281d75=Math[_0x2cd2('67d','y046')](_0x195c70,JHMW[_0x2cd2('fc','W@80')][_0x2cd2('67e','g*Sm')][_0x2cd2('2e0','YCbV')]);for(var _0x401203=0x1;_0x38d1e1[_0x2cd2('67f','8U%R')](_0x401203,_0x281d75);++_0x401203){this[_0x2cd2('680','1L8W')](JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('681','&)j!')][_0x38d1e1[_0x2cd2('682','*unX')](JHMW[_0x2cd2('380','CFh^')][_0x2cd2('19c','W@80')][_0x2cd2('267','[yfr')],_0x401203)],0x0,_0x38d1e1[_0x2cd2('683','@rQL')](0x4,_0x38d1e1[_0x2cd2('684','8vsk')](_0x401203,this[_0x2cd2('685','xWmq')]())),this[_0x2cd2('686','obzi')](),_0x38d1e1[_0x2cd2('687','P0Kn')]);}}else{return _0x41b226[_0x2cd2('688','#fZb')](_0x10d033[_0x2cd2('689','3psD')],_0x2b1188);}});};JHMW[_0x2cd2('68a','y046')]=SceneManager[_0x2cd2('68b','vbKI')];SceneManager[_0x2cd2('68c','vB83')]=function(_0x394782){var _0x4de630={'yAUCi':_0x2cd2('68d','NeH6')};JHMW[_0x2cd2('68e','jpvP')][_0x2cd2('68f','3psD')](this,_0x394782);Graphics[_0x2cd2('690','W@80')](_0x4de630[_0x2cd2('691','vbKI')]);};if(Utils[_0x2cd2('579','CFh^')]=='MZ'){var _0xb2bc71=_0x2cd2('692','R)LB')[_0x2cd2('3b6','#iIl')]('|'),_0x15d9e2=0x0;while(!![]){switch(_0xb2bc71[_0x15d9e2++]){case'0':PluginManager[_0x2cd2('693','&)j!')](JHMW[_0x2cd2('165','NeH6')][_0x2cd2('694','l%aI')],_0x2cd2('695','hS2h'),_0x1ae5a8=>{var _0x4f9099={'TgvEP':_0x2cd2('696','jpvP'),'ELGsK':function(_0x5e4888,_0x509c02){return _0x5e4888(_0x509c02);},'TXUdP':function(_0x55a1bc,_0x57e40a){return _0x55a1bc(_0x57e40a);},'Yitlw':function(_0x51e42a,_0x5bac4a){return _0x51e42a(_0x5bac4a);},'jOAHa':function(_0x33c926,_0x594778){return _0x33c926(_0x594778);},'nhejD':function(_0x56ff50,_0x2aa0d8){return _0x56ff50(_0x2aa0d8);}};var _0x315b4c=_0x4f9099[_0x2cd2('697','GDPD')][_0x2cd2('616','%E7b')]('|'),_0x308663=0x0;while(!![]){switch(_0x315b4c[_0x308663++]){case'0':_0x157df5[0x1]=_0x4f9099[_0x2cd2('698','ZboD')](Number,_0x1ae5a8[_0x2cd2('699','l%aI')])||0x0;continue;case'1':_0x157df5[0x4]=_0x4f9099[_0x2cd2('69a','&)j!')](Number,_0x1ae5a8[_0x2cd2('69b','qqEP')])||0x0;continue;case'2':_0x157df5[0x0]=_0x4f9099[_0x2cd2('69c','vbKI')](Number,_0x1ae5a8[_0x2cd2('69d','vB83')])||0x0;continue;case'3':_0x157df5[0x2]=_0x4f9099[_0x2cd2('69e','%E7b')](Number,_0x1ae5a8[_0x2cd2('69f','4UjT')])||0x0;continue;case'4':JHMW[_0x2cd2('52','[yfr')][_0x2cd2('6a0','^)]V')](_0x157df5);continue;case'5':var _0x157df5=[];continue;case'6':_0x157df5[0x5]=_0x4f9099[_0x2cd2('6a1','3psD')](Number,_0x1ae5a8[_0x2cd2('6a2','8vsk')])||0x0;continue;case'7':_0x157df5[0x3]=_0x4f9099[_0x2cd2('6a3','hS2h')](Number,_0x1ae5a8[_0x2cd2('6a4','vnIt')])||0x0;continue;}break;}});continue;case'1':PluginManager[_0x2cd2('6a5','qqEP')](JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('6a6','xvh(')],_0x2cd2('6a7','xvh('),_0x4405d2=>{JHMW[_0x2cd2('169','jpvP')][_0x2cd2('6a8','g*Sm')]=![];});continue;case'2':PluginManager[_0x2cd2('6a9','vnIt')](JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('6aa','2^*B')],_0x2cd2('6ab','#4SP'),_0x118891=>{var _0x241063={'wcLRl':function(_0x1c91d7,_0x4be27e){return _0x1c91d7(_0x4be27e);}};var _0x26239f=_0x241063[_0x2cd2('6ac','8vsk')](Number,_0x118891[_0x2cd2('6ad','8U%R')]);JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('6ae','NeH6')](_0x26239f);});continue;case'3':PluginManager[_0x2cd2('6af','obzi')](JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('6a6','xvh(')],_0x2cd2('6b0','g*Sm'),_0x3ba8e1=>{var _0x53d2d0={'mKKqg':function(_0x105359,_0x1516dd){return _0x105359(_0x1516dd);}};var _0x3a0d18=_0x53d2d0[_0x2cd2('6b1','#4SP')](Number,_0x3ba8e1[_0x2cd2('6b2','lDFd')]);JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('6b3','^)]V')](_0x3a0d18);});continue;case'4':PluginManager[_0x2cd2('6b4','GDPD')](JHMW[_0x2cd2('d','8vsk')][_0x2cd2('6b5','Biy2')],_0x2cd2('6b6','&)j!'),_0x2474af=>{var _0x5677f3={'jzqDL':function(_0x46da2e,_0x5c2b16){return _0x46da2e(_0x5c2b16);}};var _0x8c22e1=_0x5677f3[_0x2cd2('6b7','8U%R')](String,_0x2474af[_0x2cd2('6b8','1L8W')]);JHMW[_0x2cd2('380','CFh^')][_0x2cd2('6b9','obzi')](_0x8c22e1,JHMW[_0x2cd2('d','8vsk')][_0x2cd2('6ba','y046')]);});continue;case'5':PluginManager[_0x2cd2('6bb','g*Sm')](JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('6bc','jpvP')],_0x2cd2('6bd','P0Kn'),_0x306df4=>{JHMW[_0x2cd2('270','vB83')][_0x2cd2('6be','X1Zu')]=!![];});continue;case'6':PluginManager[_0x2cd2('6bf','W@80')](JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('6c0','ZboD')],_0x2cd2('6c1','#fZb'),_0x1aac08=>{var _0x2a86f8={'nUZXU':function(_0x4ae1ca,_0x2a33ed){return _0x4ae1ca==_0x2a33ed;},'QSXbs':_0x2cd2('6c2','Biy2')};if(!_0x1aac08[_0x2cd2('6c3','vnIt')]||_0x2a86f8[_0x2cd2('6c4','1L8W')](_0x1aac08[_0x2cd2('6c5','ZboD')][_0x2cd2('6c6','@rQL')](),_0x2a86f8[_0x2cd2('6c7','#fZb')]))JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('6c8','ZboD')]();else JHMW[_0x2cd2('29','4UjT')][_0x2cd2('6c9','NeH6')](!![]);});continue;case'7':PluginManager[_0x2cd2('6ca','2^*B')](JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('6cb','&)j!')],_0x2cd2('6cc','#4SP'),_0x54dff1=>{var _0x2767bf={'LUWWw':function(_0x1ae05a,_0x26bbba){return _0x1ae05a(_0x26bbba);}};var _0x1b982a=_0x2767bf[_0x2cd2('6cd','^)]V')](Number,_0x54dff1[_0x2cd2('6ce','3psD')]);JHMW[_0x2cd2('270','vB83')][_0x2cd2('6cf','l%aI')](_0x1b982a);});continue;case'8':PluginManager[_0x2cd2('6d0','xWmq')](JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('6d1','hS2h')],_0x2cd2('6d2','c!9y'),_0x43ea9b=>{JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('6d3','HHLp')]();});continue;case'9':PluginManager[_0x2cd2('6d4','[yfr')](JHMW[_0x2cd2('13f','*unX')][_0x2cd2('6d5','dO1!')],_0x2cd2('6d6','HHLp'),_0x35bbf5=>{JHMW[_0x2cd2('6d7','vbKI')][_0x2cd2('6d8','GDPD')]=!![];});continue;case'10':PluginManager[_0x2cd2('6d9','X1Zu')](JHMW[_0x2cd2('52','[yfr')][_0x2cd2('6da','*unX')],_0x2cd2('6db','vbKI'),_0xae430c=>{JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('6dc','xvh(')](JHMW[_0x2cd2('0','l%aI')][_0x2cd2('6dd','#iIl')]);});continue;case'11':PluginManager[_0x2cd2('6de','%E7b')](JHMW[_0x2cd2('19b','obzi')][_0x2cd2('6a6','xvh(')],_0x2cd2('6df','YCbV'),_0x38432d=>{var _0x44b001={'KFlcH':function(_0x130080,_0x384654){return _0x130080(_0x384654);}};var _0x3a1e17=_0x44b001[_0x2cd2('6e0','%E7b')](Number,_0x38432d[_0x2cd2('6e1','#iIl')]);JHMW[_0x2cd2('52','[yfr')][_0x2cd2('6e2','Biy2')](_0x3a1e17);});continue;case'12':PluginManager[_0x2cd2('6e3','dO1!')](JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('6e4','l^iV')],_0x2cd2('6e5','jpvP'),_0x150c54=>{var _0x32f304={'OsRJV':function(_0x13190b,_0x340a6d){return _0x13190b(_0x340a6d);}};var _0xdab918=_0x32f304[_0x2cd2('6e6','R)LB')](Number,_0x150c54[_0x2cd2('6e7','lDFd')]);DataManager[_0x2cd2('6e8','4UjT')](_0xdab918);});continue;case'13':PluginManager[_0x2cd2('6e9','*unX')](JHMW[_0x2cd2('d','8vsk')][_0x2cd2('6ea','%E7b')],_0x2cd2('6eb','#fZb'),_0x5aad02=>{JHMW[_0x2cd2('6ec','#iIl')][_0x2cd2('6ed','[yfr')]=![];});continue;case'14':PluginManager[_0x2cd2('6ee','gq!6')](JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('6ef','3Y7y')],_0x2cd2('6f0','X1Zu'),_0x2450b2=>{SceneManager[_0x2cd2('6f1','%E7b')](Scene_Ranking);});continue;}break;}}else{StorageManager[_0x2cd2('6f2','vB83')]=function(_0x476500,_0x49344b){var _0x539f46=LZString[_0x2cd2('6f3','[yfr')](JsonEx[_0x2cd2('1fd','dO1!')](_0x49344b));var _0x5b885e=this[_0x2cd2('6f4','ghrv')](_0x476500);localStorage[_0x2cd2('6f5','3Y7y')](_0x5b885e,_0x539f46);};JHMW[_0x2cd2('6f6','4[4f')]=Game_Interpreter[_0x2cd2('6f7','qqEP')][_0x2cd2('6f8','ZboD')];Game_Interpreter[_0x2cd2('6f9','l^iV')][_0x2cd2('6fa','vbKI')]=function(_0x41d576,_0x23ce43){var _0x1f906a={'mqzkS':_0x2cd2('6fb','CFh^'),'rOnJl':function(_0x5dede3,_0x13f907){return _0x5dede3===_0x13f907;},'WhCxH':_0x2cd2('6fc','jpvP'),'WdDJF':_0x2cd2('6fd','R)LB'),'WoCNI':_0x2cd2('6fe','c!9y'),'zwlNq':function(_0x3c0ea3,_0x38f3e9){return _0x3c0ea3===_0x38f3e9;},'ClIdK':_0x2cd2('6ff','NeH6'),'gdJuf':_0x2cd2('395','2^*B'),'YrMAo':function(_0x1a9a33,_0x3ae111){return _0x1a9a33===_0x3ae111;},'uFghX':_0x2cd2('700','&)j!'),'vFAsf':function(_0x1b9a3c,_0xc12144){return _0x1b9a3c===_0xc12144;},'bBVML':_0x2cd2('701','^)]V'),'hzcNQ':_0x2cd2('702','YCbV'),'rWAAJ':function(_0x3a8130,_0x312a03){return _0x3a8130===_0x312a03;},'TkcDH':_0x2cd2('703','UUq0'),'rscjm':function(_0xf32cf8,_0x57d799){return _0xf32cf8===_0x57d799;},'TNIUU':_0x2cd2('704','&)j!'),'rCxVK':function(_0x49881f,_0x4dfbc2){return _0x49881f===_0x4dfbc2;},'mExrY':_0x2cd2('705','UUq0'),'tLrmn':_0x2cd2('6d8','GDPD'),'OMWTB':_0x2cd2('706',')%R)'),'oXRGV':function(_0x53b2d3,_0x433df3){return _0x53b2d3===_0x433df3;},'hUAVa':_0x2cd2('707','dO1!')};var _0x42c605=_0x1f906a[_0x2cd2('708','X1Zu')][_0x2cd2('709','#fZb')]('|'),_0x1018ce=0x0;while(!![]){switch(_0x42c605[_0x1018ce++]){case'0':if(_0x1f906a[_0x2cd2('70a','@rQL')](_0x41d576,_0x1f906a[_0x2cd2('70b','l^iV')]))JHMW[_0x2cd2('d','8vsk')][_0x2cd2('70c','l%aI')](_0x23ce43[0x0],JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('70d','4UjT')]);continue;case'1':if(_0x1f906a[_0x2cd2('70e','UUq0')](_0x41d576,_0x1f906a[_0x2cd2('70f',')%R)')]))JHMW[_0x2cd2('a7','c!9y')][_0x2cd2('710','xWmq')](_0x23ce43);continue;case'2':if(_0x1f906a[_0x2cd2('711','*unX')](_0x41d576,_0x1f906a[_0x2cd2('712','y046')]))JHMW[_0x2cd2('eb','ghrv')][_0x2cd2('713','ghrv')]=!![];continue;case'3':JHMW[_0x2cd2('714','X1Zu')][_0x2cd2('715','hS2h')](this,_0x41d576,_0x23ce43);continue;case'4':if(_0x1f906a[_0x2cd2('716','NeH6')](_0x41d576,_0x1f906a[_0x2cd2('717','3psD')]))JHMW[_0x2cd2('de','vbKI')][_0x2cd2('718','X1Zu')](JHMW[_0x2cd2('125','lDFd')][_0x2cd2('2ba',')%R)')]);continue;case'5':if(_0x1f906a[_0x2cd2('719','P0Kn')](_0x41d576,_0x1f906a[_0x2cd2('71a','#fZb')]))JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('71b','1L8W')](_0x23ce43[0x0]);continue;case'6':if(_0x1f906a[_0x2cd2('71c','Biy2')](_0x41d576,_0x1f906a[_0x2cd2('71d','hS2h')]))JHMW[_0x2cd2('13f','*unX')][_0x2cd2('6ab','#4SP')](_0x23ce43[0x0]);continue;case'7':if(_0x1f906a[_0x2cd2('71e','qqEP')](_0x41d576,_0x1f906a[_0x2cd2('71f','xWmq')]))JHMW[_0x2cd2('17e','y046')][_0x2cd2('720','P0Kn')]=![];continue;case'8':if(_0x1f906a[_0x2cd2('721','l%aI')](_0x41d576,_0x1f906a[_0x2cd2('722','4UjT')]))JHMW[_0x2cd2('94','GDPD')][_0x2cd2('723','[yfr')]();continue;case'9':if(_0x1f906a[_0x2cd2('724','4UjT')](_0x41d576,_0x1f906a[_0x2cd2('725','dO1!')]))JHMW[_0x2cd2('162','y046')][_0x2cd2('726','lDFd')]=![];continue;case'10':if(_0x1f906a[_0x2cd2('727','obzi')](_0x41d576,_0x1f906a[_0x2cd2('728','4UjT')]))JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('729','3Y7y')](_0x23ce43[0x0]);continue;case'11':if(_0x1f906a[_0x2cd2('72a','l%aI')](_0x41d576,_0x1f906a[_0x2cd2('72b','#iIl')]))SceneManager[_0x2cd2('72c','2^*B')](Scene_Ranking);continue;case'12':if(_0x1f906a[_0x2cd2('72d','8U%R')](_0x41d576,_0x1f906a[_0x2cd2('72e','xWmq')]))JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('72f','y046')]=!![];continue;case'13':if(_0x1f906a[_0x2cd2('730','UUq0')](_0x41d576,_0x1f906a[_0x2cd2('731','ZboD')]))JHMW[_0x2cd2('19b','obzi')][_0x2cd2('732','xWmq')](_0x23ce43[0x0]);continue;case'14':if(_0x1f906a[_0x2cd2('733','ZboD')](_0x41d576,_0x1f906a[_0x2cd2('734','#fZb')]))JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('735','GDPD')](_0x23ce43[0x0]);continue;}break;}};}JHMW[_0x2cd2('736','3psD')]=Game_Temp[_0x2cd2('737','4UjT')][_0x2cd2('738','UUq0')];Game_Temp[_0x2cd2('739','[yfr')][_0x2cd2('73a','y046')]=function(_0x173c9a,_0x2844bf){if(TouchInput[_0x2cd2('73b',')%R)')]())return;JHMW[_0x2cd2('73c','ZboD')][_0x2cd2('73d','vB83')](this,_0x173c9a,_0x2844bf);};JHMW[_0x2cd2('73e','UUq0')]=Game_Player[_0x2cd2('73f','g*Sm')][_0x2cd2('740','NeH6')];Game_Player[_0x2cd2('741','8U%R')][_0x2cd2('742',')%R)')]=function(_0x18f2e2,_0x3ae6ec,_0x20bdfa,_0x45daaf,_0x6bc169){var _0x16b360={'Gjsdz':_0x2cd2('68d','NeH6')};Graphics[_0x2cd2('743','2^*B')](_0x16b360[_0x2cd2('744','HHLp')]);JHMW[_0x2cd2('745','2^*B')][_0x2cd2('746','obzi')](this,_0x18f2e2,_0x3ae6ec,_0x20bdfa,_0x45daaf,_0x6bc169);};Game_Party[_0x2cd2('747','UUq0')][_0x2cd2('748','4[4f')]=function(){var _0x18e2aa={'cEFtj':function(_0x33200a,_0x37aace){return _0x33200a<_0x37aace;},'vrnrU':function(_0x1ab0af,_0x1d0459){return _0x1ab0af===_0x1d0459;},'BMUGg':_0x2cd2('749','1L8W'),'zIUtq':_0x2cd2('74a','gq!6'),'iBcKl':function(_0x19520c,_0x3ab592){return _0x19520c<_0x3ab592;}};var _0x276d38=0x0;var _0x26ae9d=Math[_0x2cd2('74b','4[4f')](this[_0x2cd2('74c','W@80')][_0x2cd2('74d','4UjT')],this[_0x2cd2('74e','GDPD')]());for(var _0x2483fa=0x0;_0x18e2aa[_0x2cd2('74f','UUq0')](_0x2483fa,_0x26ae9d);_0x2483fa++){if(_0x18e2aa[_0x2cd2('750','3psD')](_0x18e2aa[_0x2cd2('751','vB83')],_0x18e2aa[_0x2cd2('752','[yfr')])){JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('753','GDPD')]();}else{for(var _0x426065=0x0;_0x18e2aa[_0x2cd2('754','UUq0')](_0x426065,0x8);_0x426065++){_0x276d38+=this[_0x2cd2('755','UUq0')]()[_0x2483fa][_0x2cd2('756','8U%R')](_0x426065);}}}return _0x276d38;};Game_Map[_0x2cd2('757','8vsk')][_0x2cd2('758','X1Zu')]=function(){var _0x1d5569={'lboQY':function(_0x1f0924,_0x4904af){return _0x1f0924(_0x4904af);},'zXziJ':_0x2cd2('759','ghrv'),'NoYNB':function(_0x59a257,_0x540115){return _0x59a257===_0x540115;},'iGZhQ':_0x2cd2('75a','4UjT'),'JXQTc':_0x2cd2('75b','8vsk'),'QPciT':function(_0x230cf8,_0x3bb714){return _0x230cf8<_0x3bb714;},'xHCvI':function(_0x2a4b90,_0x2efd12){return _0x2a4b90<_0x2efd12;},'EOfFb':function(_0x34231b,_0x47c496){return _0x34231b+_0x47c496;}};this[_0x2cd2('75c','[yfr')]=[];for(var _0x5e63f3=0x0;_0x1d5569[_0x2cd2('75d','1L8W')](_0x5e63f3,$dataMap[_0x2cd2('75e','^)]V')][_0x2cd2('75f','8vsk')]);_0x5e63f3++){if($dataMap[_0x2cd2('760',')%R)')][_0x5e63f3])this[_0x2cd2('761','l^iV')][_0x5e63f3]=new Game_Event(this[_0x2cd2('762','wHIg')],_0x5e63f3);}for(var _0x440f04=0x0;_0x1d5569[_0x2cd2('763','3psD')](_0x440f04,JHMW[_0x2cd2('e1','*unX')][_0x2cd2('764','vbKI')]);_0x440f04++){this[_0x2cd2('761','l^iV')][_0x1d5569[_0x2cd2('765','l%aI')](_0x440f04,$dataMap[_0x2cd2('766','l%aI')][_0x2cd2('767','2^*B')])]=new _0x5efa25(_0x1d5569[_0x2cd2('768','&)j!')](_0x440f04,$dataMap[_0x2cd2('769','Biy2')][_0x2cd2('267','[yfr')]));}this[_0x2cd2('76a','vB83')]=this[_0x2cd2('76b','ghrv')]()[_0x2cd2('76c','8U%R')](function(_0x32a59a){var _0x1592f8={'AaAhA':function(_0x391133,_0x120205){return _0x1d5569[_0x2cd2('76d','4[4f')](_0x391133,_0x120205);},'gLEuj':_0x1d5569[_0x2cd2('76e','vbKI')]};if(_0x1d5569[_0x2cd2('76f','NeH6')](_0x1d5569[_0x2cd2('770','X1Zu')],_0x1d5569[_0x2cd2('771','[yfr')])){_0x1592f8[_0x2cd2('772','HHLp')](alert,_0x1592f8[_0x2cd2('773','%E7b')]);JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('340','1L8W')]=null;}else{return new Game_CommonEvent(_0x32a59a['id']);}});this[_0x2cd2('774','jpvP')]();};Game_Map[_0x2cd2('775','4[4f')][_0x2cd2('776','c!9y')]=function(){var _0x496ce0={'haARY':function(_0x55812a,_0xe5ada9){return _0x55812a instanceof _0xe5ada9;}};return this[_0x2cd2('777','l%aI')][_0x2cd2('778','4[4f')](function(_0x21725a){return _0x496ce0[_0x2cd2('779','^)]V')](_0x21725a,_0x5efa25);});};function _0x5efa25(){this[_0x2cd2('77a','^)]V')][_0x2cd2('77b','@rQL')](this,arguments);}_0x5efa25[_0x2cd2('77c','ZboD')]=Object[_0x2cd2('77d','LX!4')](Game_Event[_0x2cd2('77e','c!9y')]);_0x5efa25[_0x2cd2('77f','obzi')][_0x2cd2('780','dO1!')]=_0x5efa25;_0x5efa25[_0x2cd2('781','hS2h')][_0x2cd2('782','UUq0')]=function(_0x290b07){var _0x389aaf={'DZTTD':_0x2cd2('783','g*Sm'),'HnDxq':function(_0x18b833,_0xad957d){return _0x18b833===_0xad957d;},'owkdv':_0x2cd2('784','&)j!'),'ndmMh':_0x2cd2('785','qqEP'),'wzHsD':function(_0x13b03b,_0x39f8e2){return _0x13b03b-_0x39f8e2;}};Game_Character[_0x2cd2('786','gq!6')][_0x2cd2('787','*unX')][_0x2cd2('788','W@80')](this);this[_0x2cd2('789','4UjT')]=$gameMap[_0x2cd2('78a','CFh^')]();this[_0x2cd2('78b','Biy2')]=_0x290b07;if(this[_0x2cd2('78c','3Y7y')]()&&this[_0x2cd2('78d','hS2h')]()){if(_0x389aaf[_0x2cd2('78e','CFh^')](_0x389aaf[_0x2cd2('78f','&)j!')],_0x389aaf[_0x2cd2('790','qqEP')])){var _0x550b7d=_0x389aaf[_0x2cd2('791','vnIt')][_0x2cd2('792','vnIt')]('|'),_0x1376d6=0x0;while(!![]){switch(_0x550b7d[_0x1376d6++]){case'0':JHMW[_0x2cd2('449','GDPD')][_0x2cd2('793','LX!4')]=$dataJHMW[_0x2cd2('794','#fZb')];continue;case'1':JHMW[_0x2cd2('123','vB83')][_0x2cd2('795','%E7b')]=$dataJHMW[_0x2cd2('796','ghrv')];continue;case'2':JHMW[_0x2cd2('e9','jpvP')][_0x2cd2('797','2^*B')]=$dataJHMW[_0x2cd2('798','4UjT')];continue;case'3':JHMW[_0x2cd2('142','ZboD')][_0x2cd2('799','*unX')]=$dataJHMW[_0x2cd2('79a','hS2h')];continue;case'4':JHMW[_0x2cd2('79b','y046')][_0x2cd2('79c','ghrv')]=$dataJHMW[_0x2cd2('79d','NeH6')];continue;}break;}}else{this[_0x2cd2('79e','4UjT')](this[_0x2cd2('79f','vB83')]()['x'],this[_0x2cd2('7a0','ZboD')]()['y']);this[_0x2cd2('7a1','qqEP')]=this[_0x2cd2('7a2',')%R)')]()[_0x2cd2('7a3','&)j!')];this[_0x2cd2('7a4','*unX')]=this[_0x2cd2('7a5','&)j!')]()[_0x2cd2('7a6','y046')];JHMW[_0x2cd2('0','l%aI')][_0x2cd2('7a7','*unX')][_0x389aaf[_0x2cd2('7a8','3psD')](_0x290b07,$dataMap[_0x2cd2('7a9','wHIg')][_0x2cd2('7aa','l%aI')])]=this[_0x2cd2('7ab','[yfr')];}}else{this[_0x2cd2('7ac','UUq0')]($gamePlayer['x'],$gamePlayer['y']);this[_0x2cd2('7ad','NeH6')]='';this[_0x2cd2('7ae','#fZb')]='';}this[_0x2cd2('7af','GDPD')]();};_0x5efa25[_0x2cd2('7b0',')%R)')][_0x2cd2('7b1','CFh^')]=function(){var _0x39f9ab={'BlxWY':function(_0x1e58e5,_0x538226){return _0x1e58e5-_0x538226;}};return DataManager[_0x2cd2('7b2','#iIl')]()[_0x39f9ab[_0x2cd2('7b3','l%aI')](this[_0x2cd2('7b4','gq!6')],$dataMap[_0x2cd2('7b5','l^iV')][_0x2cd2('7b6','jpvP')])];};_0x5efa25[_0x2cd2('7b7','CFh^')][_0x2cd2('7b8','#iIl')]=function(){var _0x45d146={'xkNho':function(_0x3774df,_0x2a5b14){return _0x3774df<_0x2a5b14;},'DczQm':function(_0x100c4f,_0x56f8e1){return _0x100c4f==_0x56f8e1;}};var _0x3e60de={'id':0x0,'name':'','note':'','pages':[{'conditions':{'actorId':0x1,'actorValid':![],'itemId':0x1,'itemValid':![],'selfSwitchCh':'A','selfSwitchValid':![],'switch1Id':0x1,'switch1Valid':![],'switch2Id':0x1,'switch2Valid':![],'variableId':0x1,'variableValid':![],'variableValue':0x0},'directionFix':![],'image':{'characterIndex':0x0,'characterName':'','direction':0x2,'pattern':0x0,'tileId':0x0},'list':[{'code':0x0,'indent':0x0,'parameters':[]}],'moveFrequency':0x3,'moveRoute':{'list':[{'code':0x0,'parameters':[]}],'repeat':!![],'skippable':![],'wait':![]},'moveSpeed':0x3,'moveType':0x0,'priorityType':0x0,'stepAnime':![],'through':![],'trigger':0x0,'walkAnime':![]}],'x':0x1,'y':0x0};if(!this[_0x2cd2('7b9','vB83')]())return _0x3e60de;for(var _0x1b847a=0x0;_0x45d146[_0x2cd2('7ba','&)j!')](_0x1b847a,DataManager[_0x2cd2('7bb','jpvP')]()[_0x2cd2('7bc','CFh^')]);_0x1b847a++){if(_0x45d146[_0x2cd2('7bd','wHIg')](this[_0x2cd2('7be','wHIg')],DataManager[_0x2cd2('7bb','jpvP')]()[_0x1b847a][_0x2cd2('7bf','X1Zu')])){_0x3e60de=DataManager[_0x2cd2('7c0','*unX')]()[_0x1b847a];break;}}return _0x3e60de;};_0x5efa25[_0x2cd2('7c1','*unX')][_0x2cd2('7c2','Biy2')]=function(){return JHMW[_0x2cd2('380','CFh^')][_0x2cd2('7c3','wHIg')]&&!$dataMap[_0x2cd2('7c4','W@80')][_0x2cd2('7c5','vbKI')];};_0x5efa25[_0x2cd2('6f9','l^iV')][_0x2cd2('7c6','ghrv')]=function(){if(this[_0x2cd2('7c7','#fZb')])this[_0x2cd2('7c8','y046')]();Game_Character[_0x2cd2('7c9','xWmq')][_0x2cd2('7ca','W@80')][_0x2cd2('746','obzi')](this);if(!this[_0x2cd2('7cb','lDFd')]())this[_0x2cd2('7cc','^)]V')]();};_0x5efa25[_0x2cd2('7cd','wHIg')][_0x2cd2('7ce','HHLp')]=function(){var _0xcdd22a={'bdGAv':function(_0x2c3d43,_0x381b02){return _0x2c3d43<_0x381b02;},'mjeAy':function(_0x249c41,_0x58164c){return _0x249c41==_0x58164c;},'MSrUK':function(_0x37f487,_0x1fae28){return _0x37f487!==_0x1fae28;},'JbWEj':_0x2cd2('7cf','#4SP')};var _0x47f63d=null;for(var _0x3a4ddf=0x0;_0xcdd22a[_0x2cd2('7d0','qqEP')](_0x3a4ddf,DataManager[_0x2cd2('7d1','[yfr')]()[_0x2cd2('7d2','xvh(')]);_0x3a4ddf++){if(_0xcdd22a[_0x2cd2('7d3','hS2h')](this[_0x2cd2('7d4','UUq0')],DataManager[_0x2cd2('7d5','y046')]()[_0x3a4ddf][_0x2cd2('7d6','GDPD')])){if(_0xcdd22a[_0x2cd2('7d7','P0Kn')](_0xcdd22a[_0x2cd2('7d8','xWmq')],_0xcdd22a[_0x2cd2('7d9','vB83')])){this[_0x2cd2('7da','3psD')]=new Window_LoginInfo(this[_0x2cd2('7db','#fZb')]);this[_0x2cd2('7dc','@rQL')][_0x2cd2('7dd','W@80')]=JHMW[_0x2cd2('f6','8U%R')][_0x2cd2('7de','X1Zu')];this[_0x2cd2('7df','CFh^')](this[_0x2cd2('7e0','qqEP')]);this[_0x2cd2('7e1','vB83')]=null;}else{_0x47f63d=DataManager[_0x2cd2('7e2','4[4f')]()[_0x3a4ddf];break;}}}if(!this[_0x2cd2('7e3','CFh^')](_0x47f63d))this[_0x2cd2('7e4','#4SP')](_0x47f63d);};_0x5efa25[_0x2cd2('7e5','^)]V')][_0x2cd2('7e6','vB83')]=function(_0x5e7460){var _0xcc7d8c={'KLNpR':function(_0x588161,_0x579ffc){return _0x588161<_0x579ffc;},'MCSjq':function(_0x1bf929,_0x57e0e4){return _0x1bf929+_0x57e0e4;}};if(!_0x5e7460)return!![];var _0x376e84=Math[_0x2cd2('7e7','vnIt')](this[_0x2cd2('7e8','R)LB')](_0x5e7460['x']));var _0x456b16=Math[_0x2cd2('7e9','GDPD')](this[_0x2cd2('7ea','ZboD')](_0x5e7460['y']));return _0xcc7d8c[_0x2cd2('7eb','LX!4')](_0xcc7d8c[_0x2cd2('7ec','UUq0')](_0x376e84,_0x456b16),0x1);};_0x5efa25[_0x2cd2('7ed','HHLp')][_0x2cd2('7ee','^)]V')]=function(){var _0x394cb2={'BKAOo':function(_0x5b253b,_0x580771){return _0x5b253b-_0x580771;},'VqMMt':function(_0x205cab,_0x500ba8){return _0x205cab>=_0x500ba8;}};var _0x2139e6=this[_0x2cd2('7ef','xvh(')]()?this[_0x2cd2('7f0','#fZb')]()[_0x2cd2('666','&)j!')]:[];for(var _0xc28933=_0x394cb2[_0x2cd2('7f1','W@80')](_0x2139e6[_0x2cd2('7f2','P0Kn')],0x1);_0x394cb2[_0x2cd2('7f3','#4SP')](_0xc28933,0x0);_0xc28933--){var _0x2fb3e7=_0x2139e6[_0xc28933];if(this[_0x2cd2('7f4','4[4f')](_0x2fb3e7))return _0xc28933;}return-0x1;};_0x5efa25[_0x2cd2('7f5','3Y7y')][_0x2cd2('7f6','gq!6')]=function(_0x168546){var _0x14be14={'sJlnW':function(_0x2f266e,_0x2cd2db){return _0x2f266e!=_0x2cd2db;}};if(_0x14be14[_0x2cd2('7f7','4[4f')](_0x168546[_0x2cd2('7f8','obzi')],$gameMap[_0x2cd2('7f9','2^*B')]()))return![];if(!DataManager[_0x2cd2('7fa','gq!6')](this[_0x2cd2('7fb','dO1!')]))return![];return!![];};_0x5efa25[_0x2cd2('739','[yfr')][_0x2cd2('7fc','dO1!')]=function(){var _0x46c086={'xGcRP':function(_0x1f80f4,_0x3c68ec){return _0x1f80f4>=_0x3c68ec;}};if(_0x46c086[_0x2cd2('7fd','ZboD')](this[_0x2cd2('7fe','LX!4')],0x0))this[_0x2cd2('7ff','xWmq')]();else this[_0x2cd2('800','*unX')]();this[_0x2cd2('801','4UjT')]();this[_0x2cd2('802','g*Sm')]();};_0x5efa25[_0x2cd2('803','3psD')][_0x2cd2('804','lDFd')]=function(){var _0x50b906={'YMdvr':_0x2cd2('805','#4SP')};var _0x50e7ad=_0x50b906[_0x2cd2('806','l%aI')][_0x2cd2('31f','^)]V')]('|'),_0x2b3304=0x0;while(!![]){switch(_0x50e7ad[_0x2b3304++]){case'0':this[_0x2cd2('807','g*Sm')](!![]);continue;case'1':this[_0x2cd2('808','8vsk')]='';continue;case'2':this[_0x2cd2('7d4','UUq0')]='';continue;case'3':this[_0x2cd2('809','W@80')]=null;continue;case'4':this[_0x2cd2('80a','qqEP')]=0x0;continue;case'5':this[_0x2cd2('80b','UUq0')]('',0x0);continue;case'6':this[_0x2cd2('80c','ZboD')]=null;continue;}break;}};_0x5efa25[_0x2cd2('7f5','3Y7y')][_0x2cd2('80d','NeH6')]=function(){var _0x38de7e={'tvkdB':function(_0x15dc67,_0x5352){return _0x15dc67==_0x5352;},'mnsCV':function(_0x15cd4c,_0x960f04){return _0x15cd4c/_0x960f04;},'ItKgd':function(_0x48e2a8,_0xe34139){return _0x48e2a8-_0xe34139;},'BhffO':function(_0xbd91c1,_0x55d1da){return _0xbd91c1/_0x55d1da;},'Kqbtv':function(_0x2cb1d9,_0x7d1390){return _0x2cb1d9!==_0x7d1390;},'BFCMB':function(_0x6f87b6,_0x49e6e5){return _0x6f87b6!==_0x49e6e5;},'pkqtK':_0x2cd2('80e','qqEP'),'RstMu':_0x2cd2('80f','^)]V')};var _0x1bd1cd=this[_0x2cd2('810','X1Zu')]();var _0xc5fb55=_0x1bd1cd[_0x2cd2('811','lDFd')];this[_0x2cd2('812','xvh(')](_0xc5fb55[_0x2cd2('813','ZboD')],_0xc5fb55[_0x2cd2('814','qqEP')]);if(_0x38de7e[_0x2cd2('815','YCbV')](this[_0x2cd2('816','hS2h')],_0xc5fb55[_0x2cd2('1ef','#iIl')])){if(_0x38de7e[_0x2cd2('817','@rQL')](_0x38de7e[_0x2cd2('818','Biy2')],_0x38de7e[_0x2cd2('819','*unX')])){this[_0x2cd2('81a','P0Kn')]=_0xc5fb55[_0x2cd2('81b','UUq0')];this[_0x2cd2('81c','P0Kn')]=0x0;this[_0x2cd2('81d','P0Kn')](![]);this[_0x2cd2('81e','@rQL')](_0xc5fb55[_0x2cd2('81f','8vsk')]);}else{Scene_MenuBase[_0x2cd2('7b7','CFh^')][_0x2cd2('820','vB83')][_0x2cd2('821','LX!4')](this);this[_0x2cd2('822','HHLp')]();if(_0x38de7e[_0x2cd2('823','X1Zu')](Utils[_0x2cd2('824','%E7b')],'MV'))this[_0x2cd2('825','2^*B')]=Math[_0x2cd2('826','1L8W')](_0x38de7e[_0x2cd2('827','lDFd')](_0x38de7e[_0x2cd2('828','3Y7y')](Graphics[_0x2cd2('829','jpvP')],0xa0),0x23));else this[_0x2cd2('82a','R)LB')]=Math[_0x2cd2('82b','jpvP')](_0x38de7e[_0x2cd2('82c','lDFd')](_0x38de7e[_0x2cd2('82d','NeH6')]($dataSystem[_0x2cd2('82e','2^*B')][_0x2cd2('82f','3Y7y')],0xa0),0x23));}}if(_0x38de7e[_0x2cd2('830','lDFd')](this[_0x2cd2('831','jpvP')],_0xc5fb55[_0x2cd2('832','P0Kn')])){this[_0x2cd2('833','g*Sm')]=_0xc5fb55[_0x2cd2('834',')%R)')];this[_0x2cd2('835','wHIg')](_0xc5fb55[_0x2cd2('836','&)j!')]);}this[_0x2cd2('837','4UjT')](_0x1bd1cd[_0x2cd2('838','W@80')]);this[_0x2cd2('839','#4SP')](_0x1bd1cd[_0x2cd2('83a','2^*B')]);this[_0x2cd2('83b','hS2h')](_0x1bd1cd[_0x2cd2('83c','CFh^')]);this[_0x2cd2('83d','NeH6')](_0x1bd1cd[_0x2cd2('83e','HHLp')]);this[_0x2cd2('83f','qqEP')](_0x1bd1cd[_0x2cd2('840','CFh^')]);this[_0x2cd2('841','NeH6')](_0x1bd1cd[_0x2cd2('842','R)LB')]);this[_0x2cd2('807','g*Sm')](_0x1bd1cd[_0x2cd2('843','ZboD')]);this[_0x2cd2('844','vB83')]=_0x1bd1cd[_0x2cd2('845','gq!6')];this[_0x2cd2('846','CFh^')]=_0x1bd1cd[_0x2cd2('847','Biy2')];this[_0x2cd2('848','%E7b')]=null;};_0x5efa25[_0x2cd2('803','3psD')][_0x2cd2('849','c!9y')]=function(){Game_Character[_0x2cd2('84a','vB83')][_0x2cd2('84b','#iIl')][_0x2cd2('84c','xvh(')](this);this[_0x2cd2('84d','hS2h')]();this[_0x2cd2('84e','X1Zu')]();};_0x5efa25[_0x2cd2('84f','%E7b')][_0x2cd2('850',')%R)')]=function(){var _0x2d4cf4={'GRpPf':function(_0x189ad0,_0x4a1a2f){return _0x189ad0(_0x4a1a2f);},'tFmdL':function(_0x1edb4a,_0x549f3a){return _0x1edb4a-_0x549f3a;},'eBToc':function(_0xa4891,_0x4a9666){return _0xa4891==_0x4a9666;},'jtIwb':function(_0xbc336e,_0x409a38){return _0xbc336e+_0x409a38;},'uXOgk':function(_0x2e1379,_0x16d5c6){return _0x2e1379*_0x16d5c6;},'WCnbN':function(_0x88ba12,_0x511836){return _0x88ba12+_0x511836;},'Xfyhk':_0x2cd2('851','gq!6'),'soImz':function(_0x138928,_0x2a486c){return _0x138928*_0x2a486c;},'eFRky':function(_0x136828,_0x56e1c4){return _0x136828!==_0x56e1c4;},'ibYjn':_0x2cd2('852','&)j!'),'RLCeF':function(_0x571587,_0x4dfbfd){return _0x571587-_0x4dfbfd;},'NuoRZ':function(_0x5ba0c6,_0x707d61){return _0x5ba0c6===_0x707d61;},'RbxPE':_0x2cd2('853','Biy2'),'YTVlj':function(_0x44e5ab,_0x526f78){return _0x44e5ab>_0x526f78;},'VbTat':_0x2cd2('854','NeH6'),'SRKdS':function(_0xeecaeb,_0x55c528){return _0xeecaeb-_0x55c528;},'zchmB':function(_0x6925f6,_0x4511cf){return _0x6925f6!=_0x4511cf;},'uItaV':function(_0x15ba44,_0x5da791){return _0x15ba44!=_0x5da791;},'XtOsQ':function(_0x488485,_0xc102f3){return _0x488485!=_0xc102f3;}};if(!DataManager[_0x2cd2('855','HHLp')](this[_0x2cd2('856','vbKI')])||!this[_0x2cd2('857','g*Sm')]()){if(_0x2d4cf4[_0x2cd2('858','ghrv')](_0x2d4cf4[_0x2cd2('859','1L8W')],_0x2d4cf4[_0x2cd2('85a','LX!4')])){var _0x383b65=_0x2d4cf4[_0x2cd2('85b','4UjT')](parseInt,position[0x1]);Window_Base[_0x2cd2('85c','#4SP')][_0x2cd2('85d','dO1!')][_0x2cd2('85e','NeH6')](this,x,_0x2d4cf4[_0x2cd2('85f','P0Kn')](_0x383b65,height),width,height);}else{delete JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('860','xvh(')][_0x2d4cf4[_0x2cd2('861','YCbV')](this[_0x2cd2('862','qqEP')],$dataMap[_0x2cd2('863','#iIl')][_0x2cd2('864','W@80')])];this[_0x2cd2('865','vnIt')]();}}if(this[_0x2cd2('866','8vsk')]&&!JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('47c','8vsk')]){if(_0x2d4cf4[_0x2cd2('867','xvh(')](_0x2d4cf4[_0x2cd2('868','NeH6')],_0x2d4cf4[_0x2cd2('869','GDPD')])){delete JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('86a','obzi')][_0x2d4cf4[_0x2cd2('86b','R)LB')](this[_0x2cd2('86c','lDFd')],$dataMap[_0x2cd2('86d','xvh(')][_0x2cd2('27d','wHIg')])];this[_0x2cd2('86e','xvh(')]();}else{if(_0x2d4cf4[_0x2cd2('86f','CFh^')](JHMW[_0x2cd2('19b','obzi')][_0x2cd2('250','R)LB')][_0x2d4cf4[_0x2cd2('870','4[4f')](_0x2d4cf4[_0x2cd2('871','3psD')](i,0xb),0xb)],JHMW[_0x2cd2('120','l^iV')][_0x2cd2('872','LX!4')]))var _0x5ef955=_0x2d4cf4[_0x2cd2('873','^)]V')](_0x2d4cf4[_0x2cd2('874','4UjT')](actname,_0x2d4cf4[_0x2cd2('875','3Y7y')]),JHMW[_0x2cd2('380','CFh^')][_0x2cd2('876','vB83')][_0x2d4cf4[_0x2cd2('877','YCbV')](_0x2d4cf4[_0x2cd2('878','3psD')](i,0xb),0xa)]);}}if(JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('879','*unX')]&&!$dataMap[_0x2cd2('87a','vB83')][_0x2cd2('87b','@rQL')]&&!this[_0x2cd2('87c','jpvP')]&&_0x2d4cf4[_0x2cd2('87d','vB83')](DataManager[_0x2cd2('87e','ZboD')]()[_0x2cd2('87f','ZboD')],this[_0x2cd2('880','#iIl')]()[_0x2cd2('881','*unX')])){var _0x400b78=_0x2d4cf4[_0x2cd2('882','CFh^')][_0x2cd2('428','c!9y')]('|'),_0x245b2d=0x0;while(!![]){switch(_0x400b78[_0x245b2d++]){case'0':this[_0x2cd2('883','vnIt')]=this[_0x2cd2('884','*unX')]()[_0x2cd2('885','HHLp')];continue;case'1':this[_0x2cd2('886','hS2h')](this[_0x2cd2('887','R)LB')]()['x'],this[_0x2cd2('888','l%aI')]()['y']);continue;case'2':this[_0x2cd2('889','qqEP')]();continue;case'3':JHMW[_0x2cd2('d','8vsk')][_0x2cd2('88a','GDPD')][_0x2d4cf4[_0x2cd2('88b','%E7b')](this[_0x2cd2('88c','%E7b')],$dataMap[_0x2cd2('766','l%aI')][_0x2cd2('88d','4[4f')])]=this[_0x2cd2('88e','GDPD')];continue;case'4':this[_0x2cd2('88f','8U%R')]=this[_0x2cd2('890','YCbV')]()[_0x2cd2('891','ghrv')];continue;case'5':this[_0x2cd2('892','ZboD')]();continue;}break;}}if(this[_0x2cd2('893',')%R)')]()&&(_0x2d4cf4[_0x2cd2('894','2^*B')](this[_0x2cd2('895','ZboD')]()[_0x2cd2('896','4[4f')][0x0][_0x2cd2('897','@rQL')][_0x2cd2('898','P0Kn')],this[_0x2cd2('899','#iIl')])||_0x2d4cf4[_0x2cd2('89a','l%aI')](this[_0x2cd2('89b','8U%R')]()[_0x2cd2('89c','l%aI')][0x0][_0x2cd2('89d','vbKI')][_0x2cd2('89e','vbKI')],this[_0x2cd2('89f','vnIt')]))){var _0x2aecbc=this[_0x2cd2('8a0','Biy2')]()[_0x2cd2('89d','vbKI')];this[_0x2cd2('8a1','vbKI')](_0x2aecbc[_0x2cd2('8a2','l%aI')],_0x2aecbc[_0x2cd2('8a3','YCbV')]);}if(this[_0x2cd2('888','l%aI')]()&&_0x2d4cf4[_0x2cd2('8a4','@rQL')](this[_0x2cd2('7a4','*unX')],this[_0x2cd2('8a5','NeH6')]()[_0x2cd2('8a6','#4SP')]))this[_0x2cd2('8a7','4UjT')]=this[_0x2cd2('887','R)LB')]()[_0x2cd2('8a8','3psD')];};_0x5efa25[_0x2cd2('8a9','1L8W')][_0x2cd2('8aa','qqEP')]=function(){var _0xf77ee3={'ufcuW':function(_0x14cd2,_0x492717){return _0x14cd2==_0x492717;}};return DataManager[_0x2cd2('8ab','l^iV')]()[_0x2cd2('8ac','xWmq')](function(_0x540d47){return!!_0x540d47&&_0xf77ee3[_0x2cd2('8ad','#fZb')](_0x540d47[_0x2cd2('8ae','y046')],this[_0x2cd2('8af','hS2h')]);},this);};_0x5efa25[_0x2cd2('7e5','^)]V')][_0x2cd2('8b0','3psD')]=function(){return JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('8b1','2^*B')][_0x2cd2('8b2','X1Zu')](function(_0x2fa714){return!!_0x2fa714&&JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('8b3','3psD')];});};_0x5efa25[_0x2cd2('8b4','X1Zu')][_0x2cd2('8b5','lDFd')]=function(){var _0x198195={'pYjWh':_0x2cd2('8b6','3Y7y'),'BAteV':function(_0x3e5906,_0x9cb5fc){return _0x3e5906>=_0x9cb5fc;},'bFXmo':function(_0x2f2b67,_0x510bbd){return _0x2f2b67==_0x510bbd;},'OAuWC':_0x2cd2('8b7','xvh('),'bUZnm':function(_0x1200d9,_0x235ea7){return _0x1200d9==_0x235ea7;},'qkysj':_0x2cd2('8b8','c!9y'),'mKDhV':_0x2cd2('8b9','R)LB'),'dWSKK':function(_0x11a5fd,_0x1e5533){return _0x11a5fd==_0x1e5533;},'IByKU':_0x2cd2('8ba',')%R)'),'pGAdp':function(_0x4766c8,_0x41fe39){return _0x4766c8==_0x41fe39;},'kPWDC':_0x2cd2('8bb','@rQL'),'OTkPY':_0x2cd2('8bc','^)]V'),'zfAEK':function(_0x17948b,_0x5d9e37){return _0x17948b>_0x5d9e37;},'fXmYd':_0x2cd2('8bd','y046'),'CbakF':function(_0x1bde9c,_0x509c9b){return _0x1bde9c<_0x509c9b;},'pEtVp':function(_0x53add2,_0xf24fef){return _0x53add2===_0xf24fef;},'HzWYZ':_0x2cd2('8be','1L8W'),'LiENB':function(_0x4046be,_0x35f979){return _0x4046be==_0x35f979;}};var _0x486639=null;for(var _0x21d9d3=0x0;_0x198195[_0x2cd2('8bf','P0Kn')](_0x21d9d3,DataManager[_0x2cd2('8c0','CFh^')]()[_0x2cd2('8c1','obzi')]);_0x21d9d3++){if(_0x198195[_0x2cd2('8c2','y046')](_0x198195[_0x2cd2('8c3','jpvP')],_0x198195[_0x2cd2('8c4','UUq0')])){if(_0x198195[_0x2cd2('8c5','GDPD')](JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('8c6','vnIt')][_0x2cd2('8c7','4UjT')](DataManager[_0x2cd2('8c8','vB83')]()[_0x21d9d3][_0x2cd2('8c9','dO1!')]),-0x1)){_0x486639=DataManager[_0x2cd2('8ca','c!9y')]()[_0x21d9d3];break;}}else{var _0x5ce3d6=_0x198195[_0x2cd2('8cb','YCbV')][_0x2cd2('8cc','xWmq')]('|'),_0x3d8620=0x0;while(!![]){switch(_0x5ce3d6[_0x3d8620++]){case'0':if(_0x198195[_0x2cd2('8cd','R)LB')](JHMW[_0x2cd2('120','l^iV')][_0x2cd2('8ce','CFh^')][0x0],0x0)){JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('22a','UUq0')]=0x0;JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('8cf','CFh^')]();JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('8d0','ZboD')]();}else if(_0x198195[_0x2cd2('8d1','GDPD')](JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('8d2','W@80')][0x0],-0x1))JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('8d3','*unX')](_0x198195[_0x2cd2('8d4','y046')]);else if(_0x198195[_0x2cd2('8d5','l%aI')](JHMW[_0x2cd2('19b','obzi')][_0x2cd2('166','3psD')][0x0],-0x2))JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('8d6','l^iV')](_0x198195[_0x2cd2('8d7','ZboD')]);else if(_0x198195[_0x2cd2('8d8','#iIl')](JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('259','P0Kn')][0x0],-0x3))JHMW[_0x2cd2('153','8U%R')][_0x2cd2('240','hS2h')](_0x198195[_0x2cd2('8d9','qqEP')]);else if(_0x198195[_0x2cd2('8da',')%R)')](JHMW[_0x2cd2('270','vB83')][_0x2cd2('288','obzi')][0x0],-0x4))JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('8db','3Y7y')](_0x198195[_0x2cd2('8dc','[yfr')]);else if(_0x198195[_0x2cd2('8dd','jpvP')](JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('8de','@rQL')][0x0],-0x5))JHMW[_0x2cd2('d','8vsk')][_0x2cd2('8df','vB83')](_0x198195[_0x2cd2('8e0','hS2h')]);else JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('8e1','jpvP')](_0x198195[_0x2cd2('8e2','3psD')]);continue;case'1':if(!msg){JHMW[_0x2cd2('94','GDPD')][_0x2cd2('8e3','hS2h')]++;if(_0x198195[_0x2cd2('8e4','*unX')](JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('8e5','lDFd')],0x5))JHMW[_0x2cd2('125','lDFd')][_0x2cd2('8e6','ZboD')](_0x198195[_0x2cd2('8e7','LX!4')]);return;}continue;case'2':var _0x2caa86=JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('8e8','3Y7y')](JHMW[_0x2cd2('61','HHLp')][_0x2cd2('8e9','l%aI')](msg));continue;case'3':JHMW[_0x2cd2('52','[yfr')][_0x2cd2('22f',')%R)')]=_0x515450[0x1][_0x2cd2('616','%E7b')](',');continue;case'4':var _0x515450=_0x2caa86[_0x2cd2('8ea','2^*B')]('#');continue;}break;}}}return _0x486639;};if(Utils[_0x2cd2('8eb','vbKI')]=='MZ'){Scene_Boot[_0x2cd2('8ec','2^*B')][_0x2cd2('8ed','g*Sm')]=function(){this[_0x2cd2('8ee','lDFd')]();if(Imported[_0x2cd2('8ef','l^iV')])JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('8f0','8vsk')]();else DataManager[_0x2cd2('8f1','4[4f')]();DataManager[_0x2cd2('8f2','gq!6')]();SceneManager[_0x2cd2('8f3','8U%R')](Scene_Login);};}else{Scene_Boot[_0x2cd2('8a9','1L8W')][_0x2cd2('8f4','g*Sm')]=function(){var _0x2d9ebe={'VOaQg':_0x2cd2('8f5','xvh(')};var _0x399c4d=_0x2d9ebe[_0x2cd2('8f6','@rQL')][_0x2cd2('8f7',')%R)')]('|'),_0x457c55=0x0;while(!![]){switch(_0x399c4d[_0x457c55++]){case'0':SoundManager[_0x2cd2('8f8','vB83')]();continue;case'1':if(Imported[_0x2cd2('8f9','c!9y')])JHMW[_0x2cd2('380','CFh^')][_0x2cd2('8fa','#iIl')]();else DataManager[_0x2cd2('8fb','obzi')]();continue;case'2':this[_0x2cd2('8fc','ZboD')]();continue;case'3':this[_0x2cd2('8fd','wHIg')]();continue;case'4':DataManager[_0x2cd2('8fe','[yfr')]();continue;case'5':Scene_Base[_0x2cd2('775','4[4f')][_0x2cd2('8ff','3Y7y')][_0x2cd2('900','xWmq')](this);continue;case'6':SceneManager[_0x2cd2('901','lDFd')](Scene_Login);continue;}break;}};}JHMW[_0x2cd2('902',')%R)')]=Scene_Map[_0x2cd2('757','8vsk')][_0x2cd2('903','ZboD')];Scene_Map[_0x2cd2('904','y046')][_0x2cd2('905','hS2h')]=function(){var _0x257500={'KrPYY':_0x2cd2('906','X1Zu')};var _0x3dc945=_0x257500[_0x2cd2('907','xWmq')][_0x2cd2('227','qqEP')]('|'),_0x4a0dce=0x0;while(!![]){switch(_0x3dc945[_0x4a0dce++]){case'0':this[_0x2cd2('908','wHIg')]();continue;case'1':JHMW[_0x2cd2('909','gq!6')][_0x2cd2('73d','vB83')](this);continue;case'2':this[_0x2cd2('90a','R)LB')]();continue;case'3':this[_0x2cd2('90b','xvh(')]();continue;case'4':this[_0x2cd2('90c','*unX')]();continue;}break;}};Scene_Map[_0x2cd2('8ec','2^*B')][_0x2cd2('90d','W@80')]=function(){var _0xd6996={'DouVk':function(_0xe13d1a,_0x2bdac4){return _0xe13d1a<_0x2bdac4;}};this[_0x2cd2('90e','#iIl')]=new Sprite_Online();for(var _0x2a953e=0x0;_0xd6996[_0x2cd2('90f','c!9y')](_0x2a953e,JHMW[_0x2cd2('195','gq!6')][_0x2cd2('910',')%R)')]);_0x2a953e++){this[_0x2cd2('911','vB83')](this[_0x2cd2('912','dO1!')][_0x2cd2('913','#4SP')][_0x2a953e]);}this[_0x2cd2('914','YCbV')](this[_0x2cd2('915','[yfr')][_0x2cd2('916','3psD')]);};Scene_Map[_0x2cd2('7ed','HHLp')][_0x2cd2('917','1L8W')]=function(){this[_0x2cd2('918','lDFd')]=new Window_TalkList();this[_0x2cd2('919','UUq0')](this[_0x2cd2('91a','dO1!')]);this[_0x2cd2('91b','xvh(')]=JHMW[_0x2cd2('101','wHIg')][_0x2cd2('91c','HHLp')];};Scene_Map[_0x2cd2('7e5','^)]V')][_0x2cd2('91d','R)LB')]=function(){var _0x12a657={'VWGNq':function(_0x3e1c09,_0x191b4b){return _0x3e1c09===_0x191b4b;},'qakhA':_0x2cd2('91e','xWmq'),'XWigP':_0x2cd2('91f','l^iV'),'FAhei':_0x2cd2('920','ghrv'),'KugFq':function(_0x3e0fac,_0x3cc00a){return _0x3e0fac-_0x3cc00a;},'hZbiO':function(_0x462961,_0x4567f2){return _0x462961(_0x4567f2);},'AQxHL':function(_0x3682de,_0x42596a){return _0x3682de+_0x42596a;},'FIJYo':function(_0x12d867,_0x112ad7){return _0x12d867(_0x112ad7);},'hTRnC':_0x2cd2('921','vnIt')};if(JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('922','CFh^')]){if(_0x12a657[_0x2cd2('923','qqEP')](_0x12a657[_0x2cd2('924','c!9y')],_0x12a657[_0x2cd2('925','xvh(')])){this[_0x2cd2('926','ghrv')][_0x2cd2('927','wHIg')][_0x2cd2('928','y046')]();this[_0x2cd2('929','R)LB')]();}else{var _0x3a3952=_0x12a657[_0x2cd2('92a','vB83')][_0x2cd2('709','#fZb')]('|'),_0x214454=0x0;while(!![]){switch(_0x3a3952[_0x214454++]){case'0':this[_0x2cd2('92b','3psD')][_0x2cd2('92c','l^iV')](_0x21cdba,_0x1c6acf);continue;case'1':var _0x4c1ddd=JHMW[_0x2cd2('97','obzi')][_0x2cd2('144','@rQL')][_0x2cd2('92d','jpvP')](',');continue;case'2':var _0x1c6acf=_0x12a657[_0x2cd2('92e','xvh(')](_0x12a657[_0x2cd2('92f','X1Zu')](parseInt,_0x4c1ddd[0x1]),this[_0x2cd2('930','obzi')][_0x2cd2('931','3psD')]);continue;case'3':var _0x21cdba=_0x12a657[_0x2cd2('932','HHLp')](_0x12a657[_0x2cd2('933','HHLp')](_0x12a657[_0x2cd2('934','hS2h')](parseInt,_0x4c1ddd[0x0]),JHMW[_0x2cd2('e5',')%R)')][_0x2cd2('935','1L8W')]),0x18);continue;case'4':this[_0x2cd2('936','W@80')](this[_0x2cd2('937','1L8W')]);continue;case'5':this[_0x2cd2('938','jpvP')]=new Sprite(ImageManager[_0x2cd2('939','xWmq')](_0x12a657[_0x2cd2('93a','R)LB')]));continue;}break;}}}this[_0x2cd2('93b','@rQL')]=JHMW[_0x2cd2('dc','#4SP')][_0x2cd2('93c','8vsk')];};Scene_Map[_0x2cd2('8b4','X1Zu')][_0x2cd2('93d','1L8W')]=function(){var _0x29a190={'syJgy':_0x2cd2('93e','wHIg'),'lusKl':function(_0x4361fe,_0x431bca){return _0x4361fe-_0x431bca;},'VHlNM':function(_0x1996b4,_0x256ce9){return _0x1996b4+_0x256ce9;},'DaDWl':function(_0x4bd72b,_0xeaab53){return _0x4bd72b(_0xeaab53);},'MIEdb':_0x2cd2('93f','lDFd')};if(JHMW[_0x2cd2('940','HHLp')][_0x2cd2('941','g*Sm')]){var _0x57e7c5=_0x29a190[_0x2cd2('942','qqEP')][_0x2cd2('943','3psD')]('|'),_0x3fbe58=0x0;while(!![]){switch(_0x57e7c5[_0x3fbe58++]){case'0':var _0x239c3d=_0x29a190[_0x2cd2('944','^)]V')](_0x29a190[_0x2cd2('945','wHIg')](_0x29a190[_0x2cd2('946','ZboD')](parseInt,_0x8ce758[0x0]),JHMW[_0x2cd2('ee','&)j!')][_0x2cd2('947','3Y7y')]),0x46);continue;case'1':var _0x8ce758=JHMW[_0x2cd2('9a','UUq0')][_0x2cd2('e3','ghrv')][_0x2cd2('948','8U%R')](',');continue;case'2':this[_0x2cd2('949','LX!4')][_0x2cd2('94a','hS2h')](_0x239c3d,_0x265dd3);continue;case'3':this[_0x2cd2('94b','CFh^')]=new Sprite(ImageManager[_0x2cd2('94c','Biy2')](_0x29a190[_0x2cd2('94d','UUq0')]));continue;case'4':this[_0x2cd2('94e','X1Zu')](this[_0x2cd2('94f','8U%R')]);continue;case'5':var _0x265dd3=_0x29a190[_0x2cd2('950','xWmq')](_0x29a190[_0x2cd2('951','xvh(')](parseInt,_0x8ce758[0x1]),0x3c);continue;}break;}}};JHMW[_0x2cd2('952','xWmq')]=Scene_Map[_0x2cd2('803','3psD')][_0x2cd2('953','4UjT')];Scene_Map[_0x2cd2('7cd','wHIg')][_0x2cd2('954','wHIg')]=function(){var _0x4c6e56={'kcbAs':_0x2cd2('955','GDPD'),'XPzUZ':_0x2cd2('956','R)LB'),'UYDKC':_0x2cd2('957','lDFd'),'GrZph':_0x2cd2('32','1L8W'),'LWzsN':function(_0x35f34a,_0x409af5){return _0x35f34a!=_0x409af5;},'RxXZO':function(_0x33f5cd,_0x862abe){return _0x33f5cd===_0x862abe;},'TURrN':_0x2cd2('958','P0Kn'),'FIRSn':_0x2cd2('959','#fZb'),'agTBS':function(_0x162357,_0x1c8f53){return _0x162357!=_0x1c8f53;}};JHMW[_0x2cd2('95a','1L8W')][_0x2cd2('95b','#iIl')](this);if(_0x4c6e56[_0x2cd2('95c','#iIl')](this[_0x2cd2('95d','8U%R')],JHMW[_0x2cd2('169','jpvP')][_0x2cd2('95e','#4SP')])){if(_0x4c6e56[_0x2cd2('95f','vbKI')](_0x4c6e56[_0x2cd2('960','lDFd')],_0x4c6e56[_0x2cd2('961',')%R)')])){this[_0x2cd2('962','4UjT')](_0x4c6e56[_0x2cd2('963','^)]V')],_0x4c6e56[_0x2cd2('964','R)LB')]);this[_0x2cd2('965','wHIg')](_0x4c6e56[_0x2cd2('966',')%R)')],_0x4c6e56[_0x2cd2('967','4UjT')]);}else{this[_0x2cd2('968','wHIg')](this[_0x2cd2('969','UUq0')]);this[_0x2cd2('96a','obzi')]();this[_0x2cd2('93d','1L8W')]();this[_0x2cd2('96b','&)j!')]();}}if(_0x4c6e56[_0x2cd2('96c','ghrv')](this[_0x2cd2('96d','HHLp')],JHMW[_0x2cd2('fa','wHIg')][_0x2cd2('6d8','GDPD')])){this[_0x2cd2('96e','dO1!')](this[_0x2cd2('96f','@rQL')]);this[_0x2cd2('970','*unX')](this[_0x2cd2('971','dO1!')]);this[_0x2cd2('972','3psD')]();this[_0x2cd2('973','jpvP')]();}if(this[_0x2cd2('974','y046')])this[_0x2cd2('975','#4SP')]();if(Graphics[_0x2cd2('976','xWmq')][_0x2cd2('977','y046')]['xs'])this[_0x2cd2('978','@rQL')]();if(this[_0x2cd2('979','3Y7y')])this[_0x2cd2('97a','xWmq')]();this[_0x2cd2('912','dO1!')][_0x2cd2('97b','#fZb')]();};Scene_Map[_0x2cd2('775','4[4f')][_0x2cd2('97c','^)]V')]=function(){var _0x1398da={'MuirI':_0x2cd2('97d','4[4f'),'gWqio':_0x2cd2('97e','3Y7y'),'mOYNG':function(_0x54711a,_0x3a0cd4){return _0x54711a+_0x3a0cd4;},'jwnhC':function(_0x446b14,_0x5db9e2){return _0x446b14+_0x5db9e2;},'DHMee':function(_0x416765,_0x58d9af){return _0x416765+_0x58d9af;},'ykuqN':function(_0x55f57e,_0x58c940){return _0x55f57e+_0x58c940;},'fwdcx':_0x2cd2('97f','lDFd'),'sTmRK':function(_0x512e34,_0x39f717){return _0x512e34===_0x39f717;},'ukoBq':_0x2cd2('980','P0Kn'),'maQej':function(_0x78abf4,_0x411130){return _0x78abf4>=_0x411130;},'BBaGQ':function(_0x2c8077,_0x4a3f31){return _0x2c8077<=_0x4a3f31;},'JUZvA':function(_0x586915,_0x4b70ed){return _0x586915+_0x4b70ed;},'UFiix':function(_0x544e59,_0x509640){return _0x544e59<=_0x509640;},'ShCpt':_0x2cd2('981','&)j!'),'jTWLr':_0x2cd2('982','xWmq'),'ZYacu':function(_0x5e7e57,_0x4fd7ee){return _0x5e7e57==_0x4fd7ee;}};if(!$gameMessage[_0x2cd2('983','2^*B')]()&&JHMW[_0x2cd2('12d','%E7b')][_0x2cd2('984','@rQL')]&&TouchInput[_0x2cd2('985','*unX')]()){if(_0x1398da[_0x2cd2('986','W@80')](_0x1398da[_0x2cd2('987','wHIg')],_0x1398da[_0x2cd2('988','2^*B')])){var _0x1b77b4=TouchInput['x'];var _0x442810=TouchInput['y'];if(_0x1398da[_0x2cd2('989','X1Zu')](_0x1b77b4,this[_0x2cd2('98a','NeH6')]['x'])&&_0x1398da[_0x2cd2('989','X1Zu')](_0x442810,this[_0x2cd2('5ee','vB83')]['y'])&&_0x1398da[_0x2cd2('98b','#4SP')](_0x1b77b4,_0x1398da[_0x2cd2('98c','*unX')](this[_0x2cd2('98d','2^*B')]['x'],this[_0x2cd2('98e','l^iV')][_0x2cd2('98f','l^iV')]))&&_0x1398da[_0x2cd2('990','Biy2')](_0x442810,_0x1398da[_0x2cd2('991','qqEP')](this[_0x2cd2('98d','2^*B')]['y'],this[_0x2cd2('992','wHIg')][_0x2cd2('993','xvh(')]))){if(_0x1398da[_0x2cd2('994','l^iV')](_0x1398da[_0x2cd2('995','c!9y')],_0x1398da[_0x2cd2('996','[yfr')])){var _0x922081=LZString[_0x2cd2('997','4UjT')](JsonEx[_0x2cd2('998','1L8W')](file));var _0xde0d45=this[_0x2cd2('999','lDFd')](filename);localStorage[_0x2cd2('99a','R)LB')](_0xde0d45,_0x922081);}else{JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('99b','y046')]+=_0x1398da[_0x2cd2('99c','8U%R')](JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('99d','#iIl')],0x3)?-0x2:0x1;this[_0x2cd2('99e','#fZb')](this[_0x2cd2('99f','LX!4')]);}}}else{var _0x97f33f=_0x1398da[_0x2cd2('9a0','xWmq')][_0x2cd2('9a1','Biy2')]('|'),_0x22d61a=0x0;while(!![]){switch(_0x97f33f[_0x22d61a++]){case'0':var _0x20e404={'user':_0x4d5137,'game':_0x53bf8b,'cmd':_0x1398da[_0x2cd2('9a2','UUq0')],'word':_0x623210,'vermd':_0x46dc45,'sign':_0x4a53fb};continue;case'1':var _0x656893=JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('9a3','P0Kn')];continue;case'2':var _0x623210=JHMW[_0x2cd2('9a4','UUq0')](JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('9a5','lDFd')])[_0x2cd2('2f4','8U%R')]();continue;case'3':var _0x4d5137=JHMW[_0x2cd2('165','NeH6')][_0x2cd2('9a6','l^iV')];continue;case'4':var _0x53bf8b=JHMW[_0x2cd2('14b','dO1!')][_0x2cd2('9a7','vbKI')];continue;case'5':if(DataManager[_0x2cd2('3aa','&)j!')]())var _0x46dc45=JHMW[_0x2cd2('9a8','NeH6')](JSON[_0x2cd2('9a9','HHLp')](PluginManager[_0x2cd2('9aa','8vsk')]));else var _0x46dc45=_0x623210;continue;case'6':var _0x50d0a6=JHMW[_0x2cd2('120','l^iV')][_0x2cd2('9ab','#iIl')](JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('3fc','^)]V')]);continue;case'7':JHMW[_0x2cd2('270','vB83')][_0x2cd2('3a2','l%aI')](_0x50d0a6,_0x20e404,![],JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('9ac','HHLp')],args);continue;case'8':var _0x4a53fb=JHMW[_0x2cd2('9ad','3Y7y')](_0x1398da[_0x2cd2('9ae','UUq0')](_0x1398da[_0x2cd2('9af','gq!6')](_0x1398da[_0x2cd2('9b0','1L8W')](_0x1398da[_0x2cd2('9b1','2^*B')](_0x1398da[_0x2cd2('9b2','4UjT')](_0x1398da[_0x2cd2('9b3','dO1!')],_0x53bf8b[_0x2cd2('9b4','gq!6')](0x5,0x4)),_0x4d5137),_0x623210),_0x656893),_0x46dc45[_0x2cd2('9b5','CFh^')](0x0,0xf)));continue;case'9':if(DataManager[_0x2cd2('9b6','LX!4')]())_0x46dc45=JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('9b7','dO1!')](_0x46dc45);continue;}break;}}}};Scene_Map[_0x2cd2('9b8','ghrv')][_0x2cd2('9b9','LX!4')]=function(){var _0x3c84ed={'pUAYi':function(_0x87394b,_0x5c1dcb){return _0x87394b(_0x5c1dcb);},'diTkU':_0x2cd2('9ba','P0Kn'),'JIJVs':_0x2cd2('9bb','GDPD'),'XXYeK':_0x2cd2('9bc','@rQL'),'uCCzY':function(_0x3c97e2,_0x3ff41d){return _0x3c97e2!==_0x3ff41d;},'cOsFf':_0x2cd2('9bd','CFh^'),'lgIhr':_0x2cd2('9be','@rQL'),'zydXf':function(_0x28756e,_0x1f59e1){return _0x28756e>=_0x1f59e1;},'tCcwI':function(_0x1433b7,_0x2f9b2a){return _0x1433b7>=_0x2f9b2a;},'SjPsX':function(_0x40ae3d,_0x24b182){return _0x40ae3d<=_0x24b182;},'PuOTH':function(_0xc86d88,_0x413e02){return _0xc86d88+_0x413e02;},'gbuXi':function(_0x431968,_0x105af1){return _0x431968===_0x105af1;},'TlgtQ':_0x2cd2('9bf','Biy2'),'AsSvf':_0x2cd2('9c0','*unX'),'tGKSx':function(_0x26d144,_0xafe469,_0x392f6b){return _0x26d144(_0xafe469,_0x392f6b);},'bFImp':function(_0x2abb68,_0x3bf60c){return _0x2abb68>_0x3bf60c;},'EzaHX':function(_0x2448c0,_0x1f612e){return _0x2448c0+_0x1f612e;},'Ublff':_0x2cd2('9c1','#fZb'),'UMKKZ':function(_0x120a9f,_0xa8cff8){return _0x120a9f>_0xa8cff8;},'NOzMv':_0x2cd2('9c2','W@80'),'BQcnc':_0x2cd2('9c3','lDFd'),'IHiur':function(_0x2c0d57,_0x3120c7){return _0x2c0d57==_0x3120c7;},'INznM':_0x2cd2('9c4','xWmq'),'uQGGw':_0x2cd2('9c5','Biy2'),'oRAfI':function(_0x5e9543,_0x4b1a6a){return _0x5e9543+_0x4b1a6a;},'bMmGg':function(_0x3caea9,_0x3685bb){return _0x3caea9+_0x3685bb;},'DdzzD':_0x2cd2('9c6','dO1!')};if(!$gameMessage[_0x2cd2('9c7','YCbV')]()&&JHMW[_0x2cd2('dc','#4SP')][_0x2cd2('9c8','3Y7y')]&&TouchInput[_0x2cd2('9c9','1L8W')]()){if(_0x3c84ed[_0x2cd2('9ca','#iIl')](_0x3c84ed[_0x2cd2('9cb','xWmq')],_0x3c84ed[_0x2cd2('9cc','8U%R')])){var _0x145505=TouchInput['x'];var _0x414982=TouchInput['y'];if(_0x3c84ed[_0x2cd2('9cd','dO1!')](_0x145505,this[_0x2cd2('9ce','ghrv')]['x'])&&_0x3c84ed[_0x2cd2('9cf','hS2h')](_0x414982,this[_0x2cd2('9d0','xvh(')]['y'])&&_0x3c84ed[_0x2cd2('9d1','P0Kn')](_0x145505,_0x3c84ed[_0x2cd2('9d2','jpvP')](this[_0x2cd2('9d3','^)]V')]['x'],this[_0x2cd2('9d4','lDFd')][_0x2cd2('9d5','P0Kn')]))&&_0x3c84ed[_0x2cd2('9d6','xvh(')](_0x414982,_0x3c84ed[_0x2cd2('9d7','&)j!')](this[_0x2cd2('949','LX!4')]['y'],this[_0x2cd2('9d8','1L8W')][_0x2cd2('9d9','UUq0')]))){if(_0x3c84ed[_0x2cd2('9da','^)]V')](_0x3c84ed[_0x2cd2('9db','&)j!')],_0x3c84ed[_0x2cd2('9dc','&)j!')])){JHMW[_0x2cd2('9dd','dO1!')][_0x2cd2('9de','3Y7y')](this);if(Imported[_0x2cd2('9df','wHIg')]&&JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('9e0','hS2h')]){this[_0x2cd2('9e1','vB83')]();JHMW[_0x2cd2('d','8vsk')][_0x2cd2('9e2','&)j!')]=![];}}else{this[_0x2cd2('9e3','4UjT')][_0x2cd2('9e4','l^iV')]=0x78;_0x3c84ed[_0x2cd2('9e5','@rQL')](setTimeout,function(){this[_0x2cd2('9e6','vbKI')][_0x2cd2('9e7','lDFd')]=0xff;}[_0x2cd2('9e8','Biy2')](this),0x64);if(_0x3c84ed[_0x2cd2('9e9','[yfr')](Graphics[_0x2cd2('9ea','8U%R')][_0x2cd2('9eb','W@80')][_0x2cd2('864','W@80')],0x0)&&_0x3c84ed[_0x2cd2('9ec','Biy2')](Graphics[_0x2cd2('9ed','wHIg')][_0x2cd2('9ee','ZboD')][_0x2cd2('9ef','8vsk')](':'),0x0)){JHMW[_0x2cd2('52','[yfr')][_0x2cd2('9f0','ZboD')][_0x2cd2('9f1','UUq0')](_0x3c84ed[_0x2cd2('9f2','3Y7y')](_0x3c84ed[_0x2cd2('9f3','g*Sm')],Graphics[_0x2cd2('9f4',')%R)')][_0x2cd2('9f5','y046')]));JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('9f6','ZboD')](Graphics[_0x2cd2('4b1','ghrv')][_0x2cd2('9f7','2^*B')]);}else if(_0x3c84ed[_0x2cd2('9f8','HHLp')](Graphics[_0x2cd2('976','xWmq')][_0x2cd2('9f9','HHLp')][_0x2cd2('9fa','*unX')](':'),0x0)){if(_0x3c84ed[_0x2cd2('9fb','#4SP')](_0x3c84ed[_0x2cd2('9fc','wHIg')],_0x3c84ed[_0x2cd2('9fd','4UjT')])){var _0x2c2cf0=Graphics[_0x2cd2('9fe','l^iV')][_0x2cd2('415','R)LB')][_0x2cd2('9ff','LX!4')](':');if(_0x3c84ed[_0x2cd2('a00','HHLp')](_0x2c2cf0[0x0],'系统'))JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('6b8','1L8W')][_0x2cd2('a01','wHIg')](_0x3c84ed[_0x2cd2('a02','8vsk')]);else if(_0x3c84ed[_0x2cd2('a03','ZboD')](_0x2c2cf0[0x0],JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('a04','lDFd')]))JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('a05','NeH6')][_0x2cd2('9f1','UUq0')](_0x3c84ed[_0x2cd2('a06','l%aI')]);else if(DataManager[_0x2cd2('a07','NeH6')](_0x2c2cf0[0x0])){JHMW[_0x2cd2('eb','ghrv')][_0x2cd2('27c','8U%R')][_0x2cd2('72c','2^*B')](_0x3c84ed[_0x2cd2('a08','jpvP')](_0x3c84ed[_0x2cd2('a09','jpvP')](_0x3c84ed[_0x2cd2('a0a','g*Sm')]('你对',_0x2c2cf0[0x0]),'说：'),_0x2c2cf0[0x1]));JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('a0b','1L8W')](_0x2c2cf0[0x1],_0x2c2cf0[0x0]);}else JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('a0c','#4SP')][_0x2cd2('a0d','YCbV')](_0x3c84ed[_0x2cd2('a0e','xvh(')](_0x2c2cf0[0x0],_0x3c84ed[_0x2cd2('a0f','xWmq')]));}else{var _0xcc0c2c=JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('a10','LX!4')](JHMW[_0x2cd2('19b','obzi')][_0x2cd2('8e9','l%aI')](msg));var _0xcac95a=_0xcc0c2c[_0x2cd2('616','%E7b')](',');if(args)$gameVariables[_0x2cd2('a11','g*Sm')](args,_0x3c84ed[_0x2cd2('a12','qqEP')](parseInt,_0xcac95a[0x1]));}}JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('a13','vB83')]=!![];Graphics[_0x2cd2('4c3','3psD')][_0x2cd2('a14','GDPD')]='';}}}else{this[_0x2cd2('a15','qqEP')](_0x3c84ed[_0x2cd2('a16','vbKI')],0x0,0x64,this[_0x2cd2('a17','X1Zu')](),_0x3c84ed[_0x2cd2('a18','1L8W')]);this[_0x2cd2('a19','vnIt')](_0x3c84ed[_0x2cd2('a1a',')%R)')],0x0,0x96,this[_0x2cd2('a1b','jpvP')](),_0x3c84ed[_0x2cd2('a1c','qqEP')]);}}};Scene_Map[_0x2cd2('a1d','W@80')][_0x2cd2('978','@rQL')]=function(){var _0x5165ef={'MaaHn':function(_0x52366f,_0x3b77bb){return _0x52366f!==_0x3b77bb;},'wrgCI':_0x2cd2('a1e','#fZb'),'FwRYw':function(_0x2a1865,_0x318e59){return _0x2a1865>=_0x318e59;},'nYNDo':function(_0x37688b,_0x498d96){return _0x37688b>=_0x498d96;},'XzEkc':function(_0x43b55c,_0x4705c2){return _0x43b55c-_0x4705c2;},'UMcNz':function(_0x42bd2e,_0x3ae9c8){return _0x42bd2e<=_0x3ae9c8;},'JxGmD':function(_0x4f50c8,_0x197ede){return _0x4f50c8+_0x197ede;},'doxte':function(_0x37bee7,_0x534fcd){return _0x37bee7+_0x534fcd;}};if(!$gameMessage[_0x2cd2('a1f','8U%R')]()&&JHMW[_0x2cd2('475','ghrv')][_0x2cd2('a20','#4SP')]&&TouchInput[_0x2cd2('a21','dO1!')]()){if(_0x5165ef[_0x2cd2('a22','dO1!')](_0x5165ef[_0x2cd2('a23','jpvP')],_0x5165ef[_0x2cd2('a24','[yfr')])){return 0xa;}else{var _0x543aff=TouchInput['x'];var _0x23e16f=TouchInput['y'];var _0x4a17d0=Graphics[_0x2cd2('4b1','ghrv')];if(_0x5165ef[_0x2cd2('a25','*unX')](_0x543aff,_0x4a17d0[_0x2cd2('602','[yfr')]['x'])&&_0x5165ef[_0x2cd2('a26','c!9y')](_0x23e16f,_0x5165ef[_0x2cd2('a27','lDFd')](_0x4a17d0[_0x2cd2('52a','NeH6')]['y'],0xa))&&_0x5165ef[_0x2cd2('a28','l%aI')](_0x543aff,_0x5165ef[_0x2cd2('a29','dO1!')](_0x4a17d0[_0x2cd2('a2a','wHIg')]['x'],_0x4a17d0[_0x2cd2('602','[yfr')][_0x2cd2('a2b','obzi')]))&&_0x5165ef[_0x2cd2('a2c','ghrv')](_0x23e16f,_0x5165ef[_0x2cd2('a2d','1L8W')](_0x5165ef[_0x2cd2('a2e','8U%R')](_0x4a17d0[_0x2cd2('4e5','4UjT')]['y'],_0x4a17d0[_0x2cd2('60f','UUq0')][_0x2cd2('a2f','wHIg')]),0xa)))_0x4a17d0[_0x2cd2('a30',')%R)')]();else _0x4a17d0[_0x2cd2('a31','ZboD')]();}}};JHMW[_0x2cd2('a32','gq!6')]=Scene_MenuBase[_0x2cd2('904','y046')][_0x2cd2('a33','8vsk')];Scene_MenuBase[_0x2cd2('775','4[4f')][_0x2cd2('a34','3Y7y')]=function(){var _0x34c06c={'UBHFN':function(_0x3b5869,_0x3f073f){return _0x3b5869===_0x3f073f;},'ACfAy':_0x2cd2('a35','qqEP'),'SEjrU':_0x2cd2('a36','@rQL')};JHMW[_0x2cd2('9dd','dO1!')][_0x2cd2('a37','P0Kn')](this);if(Imported[_0x2cd2('a38','YCbV')]&&JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('a39','l^iV')]){if(_0x34c06c[_0x2cd2('a3a','*unX')](_0x34c06c[_0x2cd2('a3b','8vsk')],_0x34c06c[_0x2cd2('a3c','8U%R')])){return JHMW[_0x2cd2('a3d','l^iV')][_0x2cd2('a3e','NeH6')];}else{this[_0x2cd2('a3f','l%aI')]();JHMW[_0x2cd2('247','YCbV')][_0x2cd2('271','ZboD')]=![];}}};Scene_MenuBase[_0x2cd2('737','4UjT')][_0x2cd2('a40','@rQL')]=function(){var _0x24a2fb={'WFjJX':function(_0x13d6f3,_0x92a9a3){return _0x13d6f3-_0x92a9a3;},'FfVrg':function(_0x1a0d32,_0x4ec7eb){return _0x1a0d32(_0x4ec7eb);},'hPTpr':_0x2cd2('a41','@rQL'),'YDosw':function(_0x212226,_0x2c399a){return _0x212226<_0x2c399a;},'jQxUb':function(_0x12b289,_0x31ebf7){return _0x12b289+_0x31ebf7;},'IEHZS':function(_0x2d90d5,_0x44ef8a){return _0x2d90d5*_0x44ef8a;},'HMCqZ':function(_0x2e0544,_0x448f7a){return _0x2e0544==_0x448f7a;},'gwJXG':function(_0x59787d,_0x2032e2){return _0x59787d*_0x2032e2;},'xZPgM':function(_0x32e33b,_0x38e52a){return _0x32e33b+_0x38e52a;},'QtJNu':function(_0x2ddc3b,_0x1bde37){return _0x2ddc3b+_0x1bde37;},'fmMms':function(_0x49d7a4,_0x3f377d){return _0x49d7a4*_0x3f377d;},'lFvHy':function(_0x1f62b2,_0x4c9903){return _0x1f62b2===_0x4c9903;},'eDlxH':_0x2cd2('a42','y046'),'CBkVJ':function(_0x3b0d8a,_0x5cca25){return _0x3b0d8a==_0x5cca25;},'sAqYh':function(_0x37d7d3,_0xb189ef){return _0x37d7d3+_0xb189ef;},'QEwlr':function(_0x15ccc9,_0x47c873){return _0x15ccc9===_0x47c873;},'rMwFT':_0x2cd2('a43','X1Zu'),'JgTqo':function(_0x3d791a,_0x4fb4f9){return _0x3d791a+_0x4fb4f9;},'omqhR':_0x2cd2('a44','*unX'),'HxGsi':function(_0xf3c33e,_0x162c65){return _0xf3c33e*_0x162c65;},'TCiXF':function(_0x42c541,_0x54ab0f){return _0x42c541+_0x54ab0f;},'gcFHH':function(_0x3efe25,_0x4bb639){return _0x3efe25+_0x4bb639;},'DNIZk':function(_0x154186,_0x39d50d){return _0x154186+_0x39d50d;},'rBkrD':function(_0xc33ef0,_0x16c41c){return _0xc33ef0*_0x16c41c;}};for(var _0x195d90=0x0;_0x24a2fb[_0x2cd2('a45','vnIt')](_0x195d90,JHMW[_0x2cd2('125','lDFd')][_0x2cd2('876','vB83')][0x0]);++_0x195d90){if(JHMW[_0x2cd2('162','y046')][_0x2cd2('250','R)LB')][_0x24a2fb[_0x2cd2('a46','vbKI')](_0x24a2fb[_0x2cd2('a47','LX!4')](_0x195d90,0xb),0xa)]){if(_0x24a2fb[_0x2cd2('a48','4[4f')](JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('a49','vbKI')][_0x24a2fb[_0x2cd2('a4a','1L8W')](_0x24a2fb[_0x2cd2('a4b','xWmq')](_0x195d90,0xb),0x1)],'系统')){var _0x4ccd94=_0x24a2fb[_0x2cd2('a4c','X1Zu')](_0x24a2fb[_0x2cd2('a4d','lDFd')](JHMW[_0x2cd2('119','1L8W')][_0x2cd2('a4e','#fZb')],JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('259','P0Kn')][_0x24a2fb[_0x2cd2('a4f','X1Zu')](_0x24a2fb[_0x2cd2('a50','gq!6')](_0x195d90,0xb),0xb)]),JHMW[_0x2cd2('120','l^iV')][_0x2cd2('a51','3Y7y')][_0x24a2fb[_0x2cd2('a52','8U%R')](_0x24a2fb[_0x2cd2('a53','CFh^')](_0x195d90,0xb),0xa)]);Notification[_0x2cd2('a54','*unX')](_0x4ccd94);}else{if(_0x24a2fb[_0x2cd2('a55','&)j!')](_0x24a2fb[_0x2cd2('a56','xvh(')],_0x24a2fb[_0x2cd2('a57','*unX')])){if(JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('a58','*unX')][_0x24a2fb[_0x2cd2('a59','hS2h')](_0x24a2fb[_0x2cd2('a5a','xvh(')](_0x195d90,0xb),0xb)]){if(_0x24a2fb[_0x2cd2('a5b','P0Kn')](JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('237','4UjT')][_0x24a2fb[_0x2cd2('a5c','Biy2')](_0x24a2fb[_0x2cd2('a5d','c!9y')](_0x195d90,0xb),0xb)],JHMW[_0x2cd2('380','CFh^')][_0x2cd2('a5e','%E7b')])){if(_0x24a2fb[_0x2cd2('a5f','3psD')](_0x24a2fb[_0x2cd2('a60','&)j!')],_0x24a2fb[_0x2cd2('a61','8U%R')])){var _0x4ccd94=_0x24a2fb[_0x2cd2('a62','qqEP')](_0x24a2fb[_0x2cd2('a63','l%aI')](JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('299','LX!4')][_0x24a2fb[_0x2cd2('a64','ghrv')](_0x24a2fb[_0x2cd2('a65','1L8W')](_0x195d90,0xb),0x1)],_0x24a2fb[_0x2cd2('a66','&)j!')]),JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('a67','qqEP')][_0x24a2fb[_0x2cd2('a68','xWmq')](_0x24a2fb[_0x2cd2('a69','[yfr')](_0x195d90,0xb),0xa)]);Notification[_0x2cd2('a6a','8vsk')](_0x4ccd94);}else{var _0x430c6c=_0x24a2fb[_0x2cd2('a6b','Biy2')](_0x24a2fb[_0x2cd2('a6c','jpvP')](parseInt,position[0x1]),height);var _0x52f1dd={'x':x,'y':_0x430c6c,'width':width,'height':height};Window_Base[_0x2cd2('84a','vB83')][_0x2cd2('a6d','xWmq')][_0x2cd2('4a3','Biy2')](this,_0x52f1dd);}}}else{var _0x4ccd94=_0x24a2fb[_0x2cd2('a6e','HHLp')](_0x24a2fb[_0x2cd2('a6f','#fZb')](JHMW[_0x2cd2('284','3psD')][_0x2cd2('a70','l^iV')][_0x24a2fb[_0x2cd2('a71','gq!6')](_0x24a2fb[_0x2cd2('a72','1L8W')](_0x195d90,0xb),0x1)],'说：'),JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('a73','hS2h')][_0x24a2fb[_0x2cd2('a74','NeH6')](_0x24a2fb[_0x2cd2('a75','*unX')](_0x195d90,0xb),0xa)]);Notification[_0x2cd2('a76','UUq0')](_0x4ccd94);}}else{if(Imported[_0x2cd2('a77','l^iV')])Notification[_0x2cd2('a54','*unX')](_0x24a2fb[_0x2cd2('a78','1L8W')]);else $gameMessage[_0x2cd2('a79','ZboD')](_0x24a2fb[_0x2cd2('a7a','@rQL')]);return;}}}}};Scene_GameEnd[_0x2cd2('7b0',')%R)')][_0x2cd2('a7b','g*Sm')]=function(){this[_0x2cd2('a7c','CFh^')]();JHMW[_0x2cd2('0','l%aI')][_0x2cd2('a7d','UUq0')]();};Scene_Gameover[_0x2cd2('8a9','1L8W')][_0x2cd2('a7e','3psD')]=function(){JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('a7f','#fZb')]();};Scene_Login[_0x2cd2('a80','P0Kn')]=Object[_0x2cd2('a81','2^*B')](Scene_Base[_0x2cd2('7c9','xWmq')]);Scene_Login[_0x2cd2('8ec','2^*B')][_0x2cd2('a82','LX!4')]=Scene_Login;Scene_Login[_0x2cd2('7b0',')%R)')][_0x2cd2('782','UUq0')]=function(){Scene_Base[_0x2cd2('a83','Biy2')][_0x2cd2('a84','ZboD')][_0x2cd2('a85','qqEP')](this);this[_0x2cd2('a86','dO1!')]();this[_0x2cd2('a87','%E7b')]();this[_0x2cd2('a88','*unX')]=0x0;};Scene_Login[_0x2cd2('7b7','CFh^')][_0x2cd2('a89','NeH6')]=function(_0x345bd5){this[_0x2cd2('a8a','hS2h')]=_0x345bd5;};Scene_Login[_0x2cd2('a1d','W@80')][_0x2cd2('a8b','^)]V')]=function(){var _0x40d917={'CjVda':_0x2cd2('a8c','vB83'),'ltrvi':function(_0x340a97,_0x407635){return _0x340a97==_0x407635;},'xZgBP':_0x2cd2('49a','vB83'),'kEYhJ':function(_0x5e2ec9,_0x20ecb5){return _0x5e2ec9+_0x20ecb5;},'XdYAP':function(_0x1973f0,_0xf44c5c){return _0x1973f0+_0xf44c5c;},'vAbQS':function(_0x38a4b4,_0x1c24f3){return _0x38a4b4==_0x1c24f3;},'CSYzm':function(_0x2aea6b,_0x349744){return _0x2aea6b-_0x349744;},'QpKDp':function(_0x10907f,_0x434a1b){return _0x10907f/_0x434a1b;},'iJmNu':function(_0x1acc96,_0x3de162){return _0x1acc96/_0x3de162;},'okWlg':_0x2cd2('a8d','#4SP'),'lSadt':_0x2cd2('a8e','UUq0'),'ayxUf':function(_0x18bd5e,_0x59ee7b){return _0x18bd5e+_0x59ee7b;}};var _0x5e2e49=_0x40d917[_0x2cd2('a8f','P0Kn')][_0x2cd2('a90','y046')]('|'),_0x5ec88a=0x0;while(!![]){switch(_0x5e2e49[_0x5ec88a++]){case'0':this[_0x2cd2('a91','LX!4')]();continue;case'1':this[_0x2cd2('a92','#fZb')]();continue;case'2':if(_0x40d917[_0x2cd2('a93','vnIt')](Utils[_0x2cd2('a94','#4SP')],'MZ')||Utils[_0x2cd2('a95','ghrv')]())this[_0x2cd2('a96','R)LB')]();continue;case'3':this[_0x2cd2('a97','xvh(')]();continue;case'4':Graphics[_0x2cd2('a98','xWmq')][_0x2cd2('a99','P0Kn')]=0xf;continue;case'5':Graphics[_0x2cd2('51b','[yfr')][_0x2cd2('a9a','1L8W')]=0xf;continue;case'6':Graphics[_0x2cd2('a9b','W@80')](_0x40d917[_0x2cd2('a9c','vbKI')],_0x40d917[_0x2cd2('a9d','3psD')],_0x38f0fb,_0x40d917[_0x2cd2('a9e','GDPD')](_0x40d917[_0x2cd2('a9f','YCbV')](JHMW[_0x2cd2('f2','c!9y')][_0x2cd2('aa0','[yfr')],_0xc87d67),0x82),0xc8,0x19,0x18);continue;case'7':if(_0x40d917[_0x2cd2('aa1','#4SP')](Utils[_0x2cd2('aa2','NeH6')],'MZ')){var _0x38f0fb=_0x40d917[_0x2cd2('aa3','g*Sm')](_0x40d917[_0x2cd2('aa4','vnIt')]($dataSystem[_0x2cd2('aa5','ZboD')][_0x2cd2('aa6','gq!6')],0x2),0x14);var _0xc87d67=_0x40d917[_0x2cd2('aa7','Biy2')](_0x40d917[_0x2cd2('aa8','GDPD')]($dataSystem[_0x2cd2('aa9','1L8W')][_0x2cd2('aaa','^)]V')],$dataSystem[_0x2cd2('aab','GDPD')][_0x2cd2('aac','Biy2')]),0x2);}else{var _0x38f0fb=_0x40d917[_0x2cd2('aad','ghrv')](_0x40d917[_0x2cd2('aae','Biy2')](Graphics[_0x2cd2('aaf','vnIt')],0x2),0x14);var _0xc87d67=0x0;}continue;case'8':Graphics[_0x2cd2('ab0','ghrv')](_0x40d917[_0x2cd2('ab1','#4SP')],_0x40d917[_0x2cd2('ab2','ghrv')],_0x38f0fb,_0x40d917[_0x2cd2('ab3','X1Zu')](_0x40d917[_0x2cd2('ab4','CFh^')](JHMW[_0x2cd2('12d','%E7b')][_0x2cd2('ab5','xWmq')],_0xc87d67),0x50),0xc8,0x19,0x18);continue;case'9':Graphics[_0x2cd2('ab6','R)LB')][_0x2cd2('ab7','vnIt')]='';continue;case'10':Scene_Base[_0x2cd2('9b8','ghrv')][_0x2cd2('ab8','YCbV')][_0x2cd2('95b','#iIl')](this);continue;case'11':Graphics[_0x2cd2('ab9','vnIt')][_0x2cd2('aba','qqEP')]='';continue;}break;}};Scene_Login[_0x2cd2('747','UUq0')][_0x2cd2('abb','^)]V')]=function(){this[_0x2cd2('abc','gq!6')]=new Sprite(ImageManager[_0x2cd2('abd','4[4f')]($dataSystem[_0x2cd2('abe','xWmq')]));this[_0x2cd2('abf','X1Zu')]=new Sprite(ImageManager[_0x2cd2('ac0','X1Zu')]($dataSystem[_0x2cd2('ac1','YCbV')]));this[_0x2cd2('ac2','Biy2')](this[_0x2cd2('ac3','#iIl')]);this[_0x2cd2('ac4','y046')](this[_0x2cd2('ac5','c!9y')]);};Scene_Login[_0x2cd2('781','hS2h')][_0x2cd2('ac6','&)j!')]=function(){AudioManager[_0x2cd2('ac7','CFh^')]($dataSystem[_0x2cd2('ac8','jpvP')]);AudioManager[_0x2cd2('ac9','g*Sm')]();AudioManager[_0x2cd2('aca','l%aI')]();};Scene_Login[_0x2cd2('7cd','wHIg')][_0x2cd2('acb','^)]V')]=function(){var _0x4775c6={'kDYTc':_0x2cd2('acc','GDPD'),'EKJvz':_0x2cd2('acd','vnIt'),'bCurn':function(_0x1a4bf2,_0x12b1a7){return _0x1a4bf2(_0x12b1a7);},'kWRiE':function(_0x5459b9,_0x9005b4){return _0x5459b9+_0x9005b4;},'Ykvdz':_0x2cd2('ace','@rQL'),'RdhiG':_0x2cd2('acf','l^iV'),'ViOUe':_0x2cd2('ad0','3psD'),'kKoas':function(_0xa08de1,_0x6bde3c){return _0xa08de1==_0x6bde3c;},'yqghY':function(_0x5324bb,_0x45f383){return _0x5324bb(_0x45f383);},'nkfVP':_0x2cd2('ad1','#fZb'),'VpeNC':function(_0x22d018,_0x4eaab4){return _0x22d018<=_0x4eaab4;},'lACFm':function(_0x481488,_0x118a13){return _0x481488!=_0x118a13;},'oPKrO':function(_0x2fe8bb,_0x2cb26e){return _0x2fe8bb!==_0x2cb26e;},'XWcDd':_0x2cd2('ad2','#4SP'),'tlawa':_0x2cd2('ad3','2^*B'),'ZDCnr':_0x2cd2('ad4','8U%R'),'AfnRa':_0x2cd2('ad5','ghrv')};this[_0x2cd2('ad6','#fZb')]=new Window_LoginCommand(this[_0x2cd2('ad7','8vsk')]);this[_0x2cd2('ad8','P0Kn')]['y']=_0x4775c6[_0x2cd2('ad9','Biy2')](this[_0x2cd2('ada','vB83')]['y'],this[_0x2cd2('adb','g*Sm')][_0x2cd2('9d9','UUq0')]);this[_0x2cd2('adc',')%R)')][_0x2cd2('add','1L8W')]=JHMW[_0x2cd2('313','l%aI')][_0x2cd2('ade','*unX')];if(!JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('adf','X1Zu')]){this[_0x2cd2('ae0','^)]V')][_0x2cd2('ae1','l^iV')](_0x4775c6[_0x2cd2('ae2','3Y7y')],this[_0x2cd2('ae3','HHLp')][_0x2cd2('59b','l^iV')](this));this[_0x2cd2('ae4','g*Sm')][_0x2cd2('ae5','*unX')](_0x4775c6[_0x2cd2('ae6','l%aI')],this[_0x2cd2('ae7','ZboD')][_0x2cd2('ae8','3Y7y')](this));this[_0x2cd2('ae9','c!9y')][_0x2cd2('aea','2^*B')](_0x4775c6[_0x2cd2('aeb','y046')],this[_0x2cd2('aec','^)]V')][_0x2cd2('2cd','&)j!')](this));}else if(_0x4775c6[_0x2cd2('aed','8U%R')](_0x4775c6[_0x2cd2('aee','ZboD')](parseInt,JHMW[_0x2cd2('162','y046')][_0x2cd2('aef','jpvP')][0x0]),0x0)){this[_0x2cd2('adc',')%R)')][_0x2cd2('af0','LX!4')](_0x4775c6[_0x2cd2('af1','gq!6')],this[_0x2cd2('af2','y046')][_0x2cd2('af3','#fZb')](this));this[_0x2cd2('af4','wHIg')][_0x2cd2('af5','vB83')](_0x4775c6[_0x2cd2('af6','Biy2')],this[_0x2cd2('af7','#fZb')][_0x2cd2('5da','%E7b')](this));}else if(_0x4775c6[_0x2cd2('af8','@rQL')](_0x4775c6[_0x2cd2('af9','8vsk')](parseInt,JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('afa','g*Sm')][0x0]),0x5)){this[_0x2cd2('afb','dO1!')][_0x2cd2('afc',')%R)')](_0x4775c6[_0x2cd2('afd','obzi')],this[_0x2cd2('afe','vB83')][_0x2cd2('aff','obzi')](this));this[_0x2cd2('b00','Biy2')][_0x2cd2('b01','ZboD')](_0x4775c6[_0x2cd2('b02','2^*B')],this[_0x2cd2('b03','l^iV')][_0x2cd2('5c9','wHIg')](this));this[_0x2cd2('b04','CFh^')][_0x2cd2('b05','R)LB')](_0x4775c6[_0x2cd2('b06','8U%R')],this[_0x2cd2('b07','vbKI')][_0x2cd2('b08','R)LB')](this));}else if(_0x4775c6[_0x2cd2('b09','&)j!')](_0x4775c6[_0x2cd2('b0a','NeH6')](parseInt,JHMW[_0x2cd2('32c','X1Zu')][_0x2cd2('afa','g*Sm')][0x1]),JHMW[_0x2cd2('151','g*Sm')][_0x2cd2('b0b','vB83')])){this[_0x2cd2('2c9','#iIl')][_0x2cd2('b0c','P0Kn')](_0x4775c6[_0x2cd2('b0d','vbKI')],this[_0x2cd2('b0e','YCbV')][_0x2cd2('af3','#fZb')](this));this[_0x2cd2('b0f','&)j!')][_0x2cd2('b10','4[4f')](_0x4775c6[_0x2cd2('b11','vnIt')],this[_0x2cd2('b12','8U%R')][_0x2cd2('b13','4UjT')](this));}else{if(_0x4775c6[_0x2cd2('b14','@rQL')](_0x4775c6[_0x2cd2('b15','#4SP')],_0x4775c6[_0x2cd2('b16','#fZb')])){this[_0x2cd2('b17','X1Zu')][_0x2cd2('b0c','P0Kn')](_0x4775c6[_0x2cd2('b18','2^*B')],this[_0x2cd2('b19','qqEP')][_0x2cd2('593','W@80')](this));this[_0x2cd2('b1a','1L8W')][_0x2cd2('b1b','^)]V')](_0x4775c6[_0x2cd2('b1c','xvh(')],this[_0x2cd2('b1d','3Y7y')][_0x2cd2('af3','#fZb')](this));}else{var _0x4da1aa=_0x4775c6[_0x2cd2('b1e','dO1!')][_0x2cd2('b1f','P0Kn')]('|'),_0x2eb937=0x0;while(!![]){switch(_0x4da1aa[_0x2eb937++]){case'0':JHMW[_0x2cd2('165','NeH6')][_0x2cd2('b20','HHLp')]=null;continue;case'1':Graphics[_0x2cd2('b21','NeH6')](_0x4775c6[_0x2cd2('b22','8U%R')]);continue;case'2':JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('3c9','ZboD')]=null;continue;case'3':JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('b23','%E7b')]=[];continue;case'4':if(!!text)SceneManager[_0x2cd2('b24','NeH6')](text);continue;case'5':JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('b25','y046')]=[];continue;case'6':_0x4775c6[_0x2cd2('b26','2^*B')](clearInterval,JHMW[_0x2cd2('b27','c!9y')]);continue;case'7':SceneManager[_0x2cd2('b28','NeH6')](Scene_Login);continue;case'8':JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('b29','4UjT')]=null;continue;case'9':JHMW[_0x2cd2('101','wHIg')][_0x2cd2('b2a','*unX')]=[];continue;case'10':JHMW[_0x2cd2('125','lDFd')][_0x2cd2('b2b','UUq0')]=[];continue;case'11':JHMW[_0x2cd2('61','HHLp')][_0x2cd2('b2c','2^*B')]=null;continue;case'12':JHMW[_0x2cd2('270','vB83')][_0x2cd2('259','P0Kn')]=[];continue;}break;}}}this[_0x2cd2('b2d','ZboD')](this[_0x2cd2('b00','Biy2')]);};Scene_Login[_0x2cd2('b2e','NeH6')][_0x2cd2('b2f','4UjT')]=function(){this[_0x2cd2('ada','vB83')]=new Window_LoginInfo(this[_0x2cd2('b30','3Y7y')]);this[_0x2cd2('7dc','@rQL')][_0x2cd2('b31','vB83')]=JHMW[_0x2cd2('123','vB83')][_0x2cd2('b32','#4SP')];this[_0x2cd2('b33','g*Sm')](this[_0x2cd2('b34','hS2h')]);this[_0x2cd2('b35','g*Sm')]=null;};Scene_Login[_0x2cd2('a80','P0Kn')][_0x2cd2('b36','xWmq')]=function(){var _0x104088={'IisGX':_0x2cd2('b37','X1Zu'),'kLHHl':_0x2cd2('b38','3psD'),'VdhOj':function(_0xcfa437,_0x420223){return _0xcfa437==_0x420223;},'pJiiZ':function(_0x18340a,_0x3d6cc7){return _0x18340a-_0x3d6cc7;},'GUZaf':function(_0x2adfef,_0x19a6e7){return _0x2adfef/_0x19a6e7;},'cHbFW':_0x2cd2('b39',')%R)')};var _0x26cd47=_0x104088[_0x2cd2('b3a','8U%R')][_0x2cd2('b3b','NeH6')]('|'),_0x47b413=0x0;while(!![]){switch(_0x26cd47[_0x47b413++]){case'0':this[_0x2cd2('b3c','dO1!')][_0x2cd2('ae5','*unX')](_0x104088[_0x2cd2('b3d','2^*B')],this[_0x2cd2('b3e','xvh(')][_0x2cd2('2cd','&)j!')](this));continue;case'1':this[_0x2cd2('b3f','2^*B')]=new Window_Action(_0xf9f59b,_0x821795,0x190);continue;case'2':this[_0x2cd2('b40','vnIt')][_0x2cd2('b41','X1Zu')]=JHMW[_0x2cd2('b42','NeH6')][_0x2cd2('b43',')%R)')];continue;case'3':this[_0x2cd2('b44','NeH6')](this[_0x2cd2('b45','g*Sm')]);continue;case'4':if(_0x104088[_0x2cd2('b46','4[4f')](Utils[_0x2cd2('b47','1L8W')],'MZ'))var _0xf9f59b=_0x104088[_0x2cd2('b48','3psD')](_0x104088[_0x2cd2('b49','c!9y')]($dataSystem[_0x2cd2('b4a','hS2h')][_0x2cd2('b4b','W@80')],0x2),0xc8);else var _0xf9f59b=_0x104088[_0x2cd2('b4c','YCbV')](_0x104088[_0x2cd2('b4d','Biy2')](Graphics[_0x2cd2('b4e','lDFd')],0x190),0x2);continue;case'5':this[_0x2cd2('b4f','HHLp')][_0x2cd2('b50','wHIg')](_0x104088[_0x2cd2('b51','W@80')],this[_0x2cd2('b52','xWmq')][_0x2cd2('b53','vbKI')](this));continue;case'6':var _0x821795=JHMW[_0x2cd2('37d','3Y7y')][_0x2cd2('b54','l^iV')];continue;}break;}};Scene_Login[_0x2cd2('a83','Biy2')][_0x2cd2('b55','LX!4')]=function(){var _0x164a39={'sNzxv':function(_0x2b4403,_0x4ef4c6){return _0x2b4403 instanceof _0x4ef4c6;},'MpZOG':function(_0x4f9298,_0x3e9454){return _0x4f9298>=_0x3e9454;},'dFdbg':function(_0x43b50c,_0x16eb6c){return _0x43b50c<=_0x16eb6c;},'mdPff':function(_0x7bdc61,_0x4f635f){return _0x7bdc61+_0x4f635f;},'TpoQc':function(_0x41cf1f,_0x536ecc){return _0x41cf1f<=_0x536ecc;},'Nmvxy':function(_0x578c3a,_0x2bf9fc){return _0x578c3a+_0x2bf9fc;},'VJzTU':function(_0x124d82,_0x2bbd02){return _0x124d82<=_0x2bbd02;},'KTdaq':function(_0x4eb777,_0x434317){return _0x4eb777<=_0x434317;},'WwfnC':function(_0x58c62e,_0x5819a1){return _0x58c62e+_0x5819a1;},'aaLYr':function(_0x4d1a4c,_0x51ba8a){return _0x4d1a4c>=_0x51ba8a;},'uZMzD':function(_0x1485ca,_0x5157b2){return _0x1485ca>=_0x5157b2;},'Ofift':function(_0x577817,_0x1c0309){return _0x577817-_0x1c0309;},'ysQlL':function(_0xcc8373,_0x24e773){return _0xcc8373<=_0x24e773;},'UzjLK':function(_0x16d8ea,_0x1cc95f){return _0x16d8ea<=_0x1cc95f;},'XTJsw':function(_0x2fab4f,_0x18e311){return _0x2fab4f===_0x18e311;},'LiaBO':_0x2cd2('b56','vnIt'),'QJvmr':_0x2cd2('b57','lDFd'),'OdGtz':function(_0x5f237d,_0x1de3d6){return _0x5f237d==_0x1de3d6;},'jOFjo':function(_0x35378d,_0x242240){return _0x35378d/_0x242240;},'rNhrk':function(_0x29844b,_0x58f652){return _0x29844b-_0x58f652;},'huDRa':function(_0x62cedb,_0x5c2ddf){return _0x62cedb/_0x5c2ddf;},'uYnaE':function(_0x1612e7,_0x1a935e){return _0x1612e7-_0x1a935e;}};if(JHMW[_0x2cd2('6ec','#iIl')][_0x2cd2('b58','W@80')]){if(_0x164a39[_0x2cd2('b59','#fZb')](_0x164a39[_0x2cd2('b5a','UUq0')],_0x164a39[_0x2cd2('b5b','%E7b')])){if(_0x164a39[_0x2cd2('b5c','xWmq')](SceneManager[_0x2cd2('b5d','vnIt')],Scene_Map)){var _0x378cf6=SceneManager[_0x2cd2('b5e','c!9y')][_0x2cd2('b5f','#iIl')];var _0x45e8a3=SceneManager[_0x2cd2('b60','lDFd')][_0x2cd2('b61','2^*B')];var _0x503558=Graphics[_0x2cd2('b62','R)LB')];if(_0x378cf6&&_0x164a39[_0x2cd2('b63','NeH6')](this['x'],_0x378cf6['x'])&&_0x164a39[_0x2cd2('b64','8U%R')](this['y'],_0x378cf6['y'])&&_0x164a39[_0x2cd2('b65','l^iV')](this['x'],_0x164a39[_0x2cd2('b66','NeH6')](_0x378cf6['x'],_0x378cf6[_0x2cd2('b67','#4SP')]))&&_0x164a39[_0x2cd2('b68','P0Kn')](this['y'],_0x164a39[_0x2cd2('b69','ZboD')](_0x378cf6['y'],_0x378cf6[_0x2cd2('b6a','ZboD')])))return!![];else if(_0x45e8a3&&_0x164a39[_0x2cd2('b6b','#4SP')](this['x'],_0x45e8a3['x'])&&_0x164a39[_0x2cd2('b6c','xWmq')](this['y'],_0x45e8a3['y'])&&_0x164a39[_0x2cd2('b6d','l%aI')](this['x'],_0x164a39[_0x2cd2('b6e','GDPD')](_0x45e8a3['x'],_0x45e8a3[_0x2cd2('b6f','YCbV')]))&&_0x164a39[_0x2cd2('b70','*unX')](this['y'],_0x164a39[_0x2cd2('b71','xvh(')](_0x45e8a3['y'],_0x45e8a3[_0x2cd2('5f9','l^iV')])))return!![];else if(_0x503558[_0x2cd2('b72','R)LB')]['xs']&&_0x164a39[_0x2cd2('b73','g*Sm')](this['x'],_0x503558[_0x2cd2('51c','W@80')]['x'])&&_0x164a39[_0x2cd2('b74','xvh(')](this['y'],_0x164a39[_0x2cd2('b75','jpvP')](_0x503558[_0x2cd2('607','#iIl')]['y'],0xa))&&_0x164a39[_0x2cd2('b76','W@80')](this['x'],_0x164a39[_0x2cd2('b77','GDPD')](_0x503558[_0x2cd2('60a','YCbV')]['x'],_0x503558[_0x2cd2('b78','vB83')][_0x2cd2('b79','[yfr')]))&&_0x164a39[_0x2cd2('b7a','LX!4')](this['y'],_0x164a39[_0x2cd2('b7b','YCbV')](_0x164a39[_0x2cd2('b7c','[yfr')](_0x503558[_0x2cd2('51c','W@80')]['y'],_0x503558[_0x2cd2('542','#4SP')][_0x2cd2('b7d','^)]V')]),0xa)))return!![];}return![];}else{this[_0x2cd2('b7e','2^*B')]=new Sprite(ImageManager[_0x2cd2('b7f','3Y7y')](JHMW[_0x2cd2('11e','qqEP')][_0x2cd2('b80','YCbV')]));this[_0x2cd2('b81','vB83')](this[_0x2cd2('b82','Biy2')],0x1);if(_0x164a39[_0x2cd2('b83','P0Kn')](Utils[_0x2cd2('b84','^)]V')],'MV'))this[_0x2cd2('b85','1L8W')][_0x2cd2('b86','*unX')](_0x164a39[_0x2cd2('b87','4UjT')](_0x164a39[_0x2cd2('b88','3Y7y')](Graphics[_0x2cd2('b89','^)]V')],0x180),0x2),_0x164a39[_0x2cd2('b8a','Biy2')](_0x164a39[_0x2cd2('b8b','^)]V')](Graphics[_0x2cd2('b8c','R)LB')],this[_0x2cd2('b8d','ZboD')][_0x2cd2('b8e','4UjT')]),0x2));else this[_0x2cd2('b8f','*unX')][_0x2cd2('b86','*unX')](_0x164a39[_0x2cd2('b90','1L8W')](_0x164a39[_0x2cd2('b91','lDFd')]($dataSystem[_0x2cd2('aa9','1L8W')][_0x2cd2('b92','vB83')],0x180),0x2),_0x164a39[_0x2cd2('b93','NeH6')](_0x164a39[_0x2cd2('b94','&)j!')]($dataSystem[_0x2cd2('b95','l%aI')][_0x2cd2('b96','vnIt')],this[_0x2cd2('b97','xvh(')][_0x2cd2('b98','dO1!')]),0x2));}}};Scene_Login[_0x2cd2('8a9','1L8W')][_0x2cd2('b99','l^iV')]=function(){var _0x1003df={'Gjbtq':function(_0x130152,_0x2b45a2){return _0x130152-_0x2b45a2;},'JITGH':function(_0xfd970b,_0x354234){return _0xfd970b(_0x354234);},'LEkKA':function(_0x33715f,_0x2e8206){return _0x33715f==_0x2e8206;},'hFBgz':_0x2cd2('b9a','^)]V'),'YeRxo':function(_0x30a2d4,_0x1b4a9a){return _0x30a2d4!==_0x1b4a9a;},'iAShE':_0x2cd2('b9b','CFh^'),'yDzeD':function(_0x5e15be,_0x3bb24d){return _0x5e15be!==_0x3bb24d;},'GIpVz':_0x2cd2('b9c','&)j!'),'oshXl':_0x2cd2('b9d','vbKI'),'hCbWG':function(_0x493123,_0x13264f){return _0x493123===_0x13264f;},'xQAMl':_0x2cd2('b9e','ghrv')};if(_0x1003df[_0x2cd2('b9f','Biy2')](Utils[_0x2cd2('ba0','8vsk')],'MV')){var _0x6b6f03=null;var _0x438609=StorageManager[_0x2cd2('ba1','gq!6')](_0x1003df[_0x2cd2('ba2','vbKI')]);try{if(_0x1003df[_0x2cd2('ba3','1L8W')](_0x1003df[_0x2cd2('ba4','vB83')],_0x1003df[_0x2cd2('ba5','^)]V')])){return DataManager[_0x2cd2('ba6','Biy2')]()[_0x1003df[_0x2cd2('ba7','UUq0')](this[_0x2cd2('ba8','R)LB')],$dataMap[_0x2cd2('ba9','#4SP')][_0x2cd2('baa','&)j!')])];}else{_0x6b6f03=JsonEx[_0x2cd2('bab','xWmq')](LZString[_0x2cd2('bac','R)LB')](localStorage[_0x2cd2('bad','NeH6')](_0x438609)));}}catch(_0x6ae168){}if(_0x6b6f03){Graphics[_0x2cd2('bae','CFh^')][_0x2cd2('baf','wHIg')]=_0x6b6f03[_0x2cd2('bb0','[yfr')];Graphics[_0x2cd2('bb1','UUq0')][_0x2cd2('baf','wHIg')]=_0x6b6f03[_0x2cd2('bb2','P0Kn')];}}else{if(_0x1003df[_0x2cd2('bb3','g*Sm')](_0x1003df[_0x2cd2('bb4','^)]V')],_0x1003df[_0x2cd2('bb5','l^iV')])){try{if(_0x1003df[_0x2cd2('bb6','YCbV')](_0x1003df[_0x2cd2('bb7','wHIg')],_0x1003df[_0x2cd2('bb8','LX!4')])){StorageManager[_0x2cd2('bb9','4[4f')](_0x1003df[_0x2cd2('bba','ZboD')])[_0x2cd2('bbb','4UjT')](_0x6b6f03=>{Graphics[_0x2cd2('bbc','c!9y')][_0x2cd2('41d','[yfr')]=_0x6b6f03[_0x2cd2('bbd','8U%R')];Graphics[_0x2cd2('ab6','R)LB')][_0x2cd2('bbe','NeH6')]=_0x6b6f03[_0x2cd2('bbf','[yfr')];})[_0x2cd2('bc0','%E7b')](()=>{return;});}else{PluginManager[_0x2cd2('bc1','qqEP')]=[];this[_0x2cd2('bc2','HHLp')][_0x2cd2('bc3','2^*B')](function(_0x39ad64){var _0x58913b=_0x1003df[_0x2cd2('bc4','xvh(')](eval,_0x39ad64[_0x2cd2('bc5','vbKI')]);PluginManager[_0x2cd2('41a','^)]V')][_0x2cd2('bc6','[yfr')](JHMW[_0x2cd2('bc7','GDPD')](JSON[_0x2cd2('bc8','@rQL')](_0x58913b)));});}}catch(_0x1d8562){}}else{JHMW[_0x2cd2('e1','*unX')][_0x2cd2('bc9','8U%R')]=!![];}}};Scene_Login[_0x2cd2('6f9','l^iV')][_0x2cd2('bca','l^iV')]=function(){var _0x940c4e={'AcJXe':function(_0x4c0028,_0x2aed6d){return _0x4c0028==_0x2aed6d;},'PAcsy':function(_0x570d29,_0x1bb192){return _0x570d29===_0x1bb192;},'XBUWg':_0x2cd2('bcb','NeH6'),'oAQZf':_0x2cd2('bcc','YCbV'),'JIEWf':function(_0x367fee,_0x471229){return _0x367fee-_0x471229;},'zhBBP':function(_0x45611d,_0x58378a){return _0x45611d/_0x58378a;},'zWjJL':function(_0x324844,_0xa22fe4){return _0x324844+_0xa22fe4;},'BVpEc':function(_0x9c8a2f,_0x14b2b9){return _0x9c8a2f/_0x14b2b9;},'tZjmY':function(_0x5cdf07,_0x1471aa){return _0x5cdf07+_0x1471aa;},'HcwXS':function(_0x12e69a,_0x1c487d){return _0x12e69a/_0x1c487d;},'kreWC':function(_0xb68707,_0x557f8a){return _0xb68707>=_0x557f8a;},'kgKVI':function(_0x3dc67c,_0x580bcb){return _0x3dc67c>=_0x580bcb;},'Jtudd':function(_0x40dd30,_0x109208){return _0x40dd30-_0x109208;},'fSeNa':function(_0x20c5a2,_0x111283){return _0x20c5a2<_0x111283;},'uLFOW':function(_0x353eab,_0x5cc18f){return _0x353eab==_0x5cc18f;},'nbAAA':function(_0x2233f1,_0x3300eb){return _0x2233f1>=_0x3300eb;},'iqSuj':function(_0x4ee8b7,_0x28139f){return _0x4ee8b7>=_0x28139f;},'fLAos':function(_0x584927,_0x19320b){return _0x584927<_0x19320b;},'XRnEx':function(_0x566928,_0x7564b4){return _0x566928+_0x7564b4;},'PHsDj':function(_0x491918,_0x226258){return _0x491918<_0x226258;},'pGyGz':function(_0x58b34d,_0x381111){return _0x58b34d+_0x381111;},'lvYgE':function(_0x2cd5f5,_0x1f2eb0){return _0x2cd5f5==_0x1f2eb0;}};Scene_Base[_0x2cd2('741','8U%R')][_0x2cd2('bcd','%E7b')][_0x2cd2('bce','%E7b')](this);if(_0x940c4e[_0x2cd2('bcf','X1Zu')](Utils[_0x2cd2('bd0','vB83')],'MV')){if(_0x940c4e[_0x2cd2('bd1','Biy2')](_0x940c4e[_0x2cd2('bd2','4[4f')],_0x940c4e[_0x2cd2('bd3','#4SP')])){return JHMW[_0x2cd2('470','R)LB')][_0x2cd2('935','1L8W')];}else{var _0x2febbf=_0x940c4e[_0x2cd2('bd4','GDPD')](_0x940c4e[_0x2cd2('bd5','jpvP')](Graphics[_0x2cd2('bd6','xvh(')],0x2),0x14);var _0x1ba602=_0x940c4e[_0x2cd2('bd7','P0Kn')](JHMW[_0x2cd2('6d7','vbKI')][_0x2cd2('bd8','wHIg')],0x75);var _0x49fd78=0xc8;var _0x36d2f3=0x28;}}else{var _0x2febbf=_0x940c4e[_0x2cd2('bd9','vnIt')](_0x940c4e[_0x2cd2('bda','R)LB')]($dataSystem[_0x2cd2('bdb','vB83')][_0x2cd2('bdc','jpvP')],0x2),0x28);var _0x1ba602=_0x940c4e[_0x2cd2('bdd','LX!4')](_0x940c4e[_0x2cd2('bde','4[4f')](JHMW[_0x2cd2('29d','vnIt')][_0x2cd2('bdf','qqEP')],_0x940c4e[_0x2cd2('be0','*unX')](_0x940c4e[_0x2cd2('be1','#iIl')]($dataSystem[_0x2cd2('be2','vbKI')][_0x2cd2('be3','c!9y')],$dataSystem[_0x2cd2('be4','8U%R')][_0x2cd2('b96','vnIt')]),0x2)),0x78);var _0x49fd78=0xf0;var _0x36d2f3=0x3c;}this[_0x2cd2('be5','wHIg')]++;if(TouchInput[_0x2cd2('be6','gq!6')]()){var _0x281e38=TouchInput['x'];var _0xb2347c=TouchInput['y'];if(_0x940c4e[_0x2cd2('be7','YCbV')](_0x281e38,_0x2febbf)&&_0x940c4e[_0x2cd2('be8','dO1!')](_0xb2347c,_0x940c4e[_0x2cd2('be9',')%R)')](_0x1ba602,_0x36d2f3))&&_0x940c4e[_0x2cd2('bea','vnIt')](_0x281e38,_0x940c4e[_0x2cd2('beb','W@80')](_0x2febbf,_0x49fd78))&&_0x940c4e[_0x2cd2('bec','l%aI')](_0xb2347c,_0x1ba602)){if(_0x940c4e[_0x2cd2('bed',')%R)')](Utils[_0x2cd2('bee','3psD')],'MV'))this[_0x2cd2('bef','ghrv')][_0x2cd2('bf0','#fZb')]();this[_0x2cd2('bf1','GDPD')][_0x2cd2('bf2','vbKI')]();Graphics[_0x2cd2('a98','xWmq')][_0x2cd2('bf3','y046')]();}else if(_0x940c4e[_0x2cd2('bf4','Biy2')](_0x281e38,_0x2febbf)&&_0x940c4e[_0x2cd2('bf5','1L8W')](_0xb2347c,_0x1ba602)&&_0x940c4e[_0x2cd2('bf6','CFh^')](_0x281e38,_0x940c4e[_0x2cd2('bf7','NeH6')](_0x2febbf,_0x49fd78))&&_0x940c4e[_0x2cd2('bf8','vbKI')](_0xb2347c,_0x940c4e[_0x2cd2('bf9','@rQL')](_0x1ba602,_0x36d2f3))){if(_0x940c4e[_0x2cd2('bfa','CFh^')](Utils[_0x2cd2('bee','3psD')],'MV'))this[_0x2cd2('bfb','xWmq')][_0x2cd2('bfc','[yfr')]();this[_0x2cd2('bfd','4[4f')][_0x2cd2('bfe','GDPD')]();Graphics[_0x2cd2('ab6','R)LB')][_0x2cd2('bff','c!9y')]();}else{if(_0x940c4e[_0x2cd2('c00','lDFd')](Utils[_0x2cd2('5cb',')%R)')],'MV'))this[_0x2cd2('c01','#fZb')][_0x2cd2('c02','3psD')]();this[_0x2cd2('ae4','g*Sm')][_0x2cd2('c03',')%R)')]();}}if(TouchInput[_0x2cd2('c04','@rQL')]())this[_0x2cd2('b1a','1L8W')][_0x2cd2('c05','y046')]();};Scene_Login[_0x2cd2('c06','vnIt')][_0x2cd2('c07','jpvP')]=function(){var _0x417b3a={'jUHzg':_0x2cd2('c08','lDFd'),'wQNoD':function(_0x59b296,_0x1e5d73){return _0x59b296+_0x1e5d73;},'Cwldd':_0x2cd2('c09','CFh^'),'gXMXX':_0x2cd2('c0a','8vsk')};var _0x23d6d6=_0x417b3a[_0x2cd2('c0b','LX!4')][_0x2cd2('c0c','1L8W')]('|'),_0x98e322=0x0;while(!![]){switch(_0x23d6d6[_0x98e322++]){case'0':JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('c0d','UUq0')]=_0x417b3a[_0x2cd2('c0e','NeH6')]('',Graphics[_0x2cd2('c0f','l^iV')][_0x2cd2('c10','CFh^')]);continue;case'1':TouchInput[_0x2cd2('c11','3Y7y')]();continue;case'2':JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('c12','2^*B')]=_0x417b3a[_0x2cd2('c13','8vsk')]('',Graphics[_0x2cd2('c14','^)]V')][_0x2cd2('c15','#fZb')]);continue;case'3':this[_0x2cd2('c16','xWmq')][_0x2cd2('c17','vnIt')]();continue;case'4':this[_0x2cd2('c18','jpvP')][_0x2cd2('c19','vB83')]();continue;case'5':this[_0x2cd2('c1a','^)]V')][_0x2cd2('c1b','lDFd')]();continue;case'6':Graphics[_0x2cd2('c1c','l%aI')](_0x417b3a[_0x2cd2('c1d','8U%R')]);continue;case'7':this[_0x2cd2('c1e','vbKI')]();continue;case'8':if(window[_0x2cd2('c1f','NeH6')]&&window[_0x2cd2('c20','4UjT')])window[_0x2cd2('c21','#iIl')][_0x2cd2('c22','y046')]();continue;case'9':Graphics[_0x2cd2('690','W@80')](_0x417b3a[_0x2cd2('c23','3psD')]);continue;}break;}};Scene_Login[_0x2cd2('757','8vsk')][_0x2cd2('c24','GDPD')]=function(){var _0x55a3bd={'jDBCC':function(_0x3a00da,_0x120e4c){return _0x3a00da>=_0x120e4c;},'jJSqu':function(_0xa95d10,_0x4d099d){return _0xa95d10>=_0x4d099d;},'BZgHz':function(_0x4a152f,_0x5eb1d2){return _0x4a152f-_0x5eb1d2;},'fOyDQ':function(_0x326ea3,_0x539749){return _0x326ea3<=_0x539749;},'vvKTu':function(_0x472718,_0x2c0f88){return _0x472718+_0x2c0f88;},'XLQrN':function(_0x2e0312,_0x1d72df){return _0x2e0312!==_0x1d72df;},'tidhD':_0x2cd2('c25','vB83'),'bEzLt':_0x2cd2('c26','[yfr'),'SgXnY':_0x2cd2('c27','ghrv')};if(Imported[_0x2cd2('c28','&)j!')])DataManager[_0x2cd2('c29','W@80')]();if(Imported[_0x2cd2('8f9','c!9y')])DataManager[_0x2cd2('c2a','lDFd')]();if(Utils[_0x2cd2('c2b','%E7b')]()){if(_0x55a3bd[_0x2cd2('c2c','3psD')](_0x55a3bd[_0x2cd2('c2d','LX!4')],_0x55a3bd[_0x2cd2('c2e','@rQL')])){var _0x523a53=TouchInput['x'];var _0x19708d=TouchInput['y'];var _0x1c4e7e=Graphics[_0x2cd2('c2f','#4SP')];if(_0x55a3bd[_0x2cd2('c30','YCbV')](_0x523a53,_0x1c4e7e[_0x2cd2('4d6','1L8W')]['x'])&&_0x55a3bd[_0x2cd2('c31','&)j!')](_0x19708d,_0x55a3bd[_0x2cd2('c32','c!9y')](_0x1c4e7e[_0x2cd2('c33','8U%R')]['y'],0xa))&&_0x55a3bd[_0x2cd2('c34','l^iV')](_0x523a53,_0x55a3bd[_0x2cd2('c35',')%R)')](_0x1c4e7e[_0x2cd2('607','#iIl')]['x'],_0x1c4e7e[_0x2cd2('c36','gq!6')][_0x2cd2('c37','1L8W')]))&&_0x55a3bd[_0x2cd2('c38','1L8W')](_0x19708d,_0x55a3bd[_0x2cd2('c39','P0Kn')](_0x55a3bd[_0x2cd2('c3a','UUq0')](_0x1c4e7e[_0x2cd2('521','X1Zu')]['y'],_0x1c4e7e[_0x2cd2('607','#iIl')][_0x2cd2('c3b','2^*B')]),0xa)))_0x1c4e7e[_0x2cd2('c3c','GDPD')]();else _0x1c4e7e[_0x2cd2('c3d','R)LB')]();}else{if(!JHMW[_0x2cd2('1f9','[yfr')][_0x2cd2('c3e','%E7b')])window[_0x2cd2('c3f','P0Kn')](_0x55a3bd[_0x2cd2('c40','gq!6')]);else Website2APK[_0x2cd2('c41','vB83')](_0x55a3bd[_0x2cd2('c42','hS2h')]);}}else window[_0x2cd2('c43','UUq0')](_0x55a3bd[_0x2cd2('c44','CFh^')]);};Scene_Login[_0x2cd2('737','4UjT')][_0x2cd2('c45','y046')]=function(){if(Imported[_0x2cd2('c46','%E7b')])DataManager[_0x2cd2('c47','qqEP')]();if(!DataManager[_0x2cd2('c48','HHLp')]())JHMW[_0x2cd2('0','l%aI')][_0x2cd2('c49','HHLp')]();SceneManager[_0x2cd2('c4a','GDPD')]();};Scene_Login[_0x2cd2('c4b','vbKI')][_0x2cd2('c4c','gq!6')]=function(){var _0x12677e={'dPGYu':function(_0x332ae7,_0x48e215){return _0x332ae7!==_0x48e215;},'xctIH':_0x2cd2('c4d','X1Zu'),'QJMSU':_0x2cd2('c4e','4[4f')};if(Utils[_0x2cd2('c4f','R)LB')]()){if(_0x12677e[_0x2cd2('c50',')%R)')](_0x12677e[_0x2cd2('c51','wHIg')],_0x12677e[_0x2cd2('c52','3Y7y')])){if(!JHMW[_0x2cd2('12d','%E7b')][_0x2cd2('c53','jpvP')])window[_0x2cd2('c54','4UjT')](JHMW[_0x2cd2('ee','&)j!')][_0x2cd2('c55','8vsk')]);else Website2APK[_0x2cd2('c56','&)j!')](JHMW[_0x2cd2('10d','^)]V')][_0x2cd2('c57','y046')]);}else{this[_0x2cd2('c58',')%R)')]();}}else window[_0x2cd2('c59','3psD')](JHMW[_0x2cd2('10d','^)]V')][_0x2cd2('c5a','l%aI')]);};Scene_Login[_0x2cd2('7ed','HHLp')][_0x2cd2('c5b','Biy2')]=function(){var _0x244b9b={'MYQry':function(_0x398b44,_0xb9c6fe){return _0x398b44==_0xb9c6fe;},'UjqvO':function(_0x56448d,_0x2aad42){return _0x56448d<_0x2aad42;},'SqaLl':_0x2cd2('c5c','HHLp')};SoundManager[_0x2cd2('c5d','&)j!')]();if(_0x244b9b[_0x2cd2('c5e','wHIg')](JHMW[_0x2cd2('270','vB83')][_0x2cd2('c5f','obzi')][0x4],0x0))this[_0x2cd2('c60','X1Zu')]();else if(_0x244b9b[_0x2cd2('c61','xWmq')](JHMW[_0x2cd2('56','@rQL')][_0x2cd2('adf','X1Zu')][0x4],0x0))SceneManager[_0x2cd2('c62','xWmq')]();else this[_0x2cd2('c63',')%R)')](_0x244b9b[_0x2cd2('c64','NeH6')]);};Scene_Login[_0x2cd2('757','8vsk')][_0x2cd2('c65','8U%R')]=function(){var _0x12074c={'BaihV':function(_0x2795ef,_0x31a4ef){return _0x2795ef-_0x31a4ef;},'ryLdy':function(_0x1d8561,_0x148b1e){return _0x1d8561/_0x148b1e;},'oFULX':function(_0x3e8f11,_0x356073){return _0x3e8f11===_0x356073;},'dFbHl':_0x2cd2('c66','%E7b'),'yByOI':function(_0x48afea,_0x2c9039){return _0x48afea>_0x2c9039;},'wwwah':function(_0x2c3be6,_0x24d270){return _0x2c3be6<_0x24d270;},'ctuTa':function(_0x4e9fa0,_0x17fc7d){return _0x4e9fa0(_0x17fc7d);},'WbAvx':function(_0x328753,_0x48b511){return _0x328753==_0x48b511;},'sItUt':function(_0x5aa40f,_0x4b8cd8){return _0x5aa40f>=_0x4b8cd8;},'HCzlq':function(_0x1f78fc,_0xaf8b36,_0x2b686d){return _0x1f78fc(_0xaf8b36,_0x2b686d);},'IMSFK':_0x2cd2('c67','#fZb'),'ZiqMZ':function(_0x5c5845,_0x3fbb21){return _0x5c5845<_0x3fbb21;},'VvPXL':_0x2cd2('c68','obzi')};SoundManager[_0x2cd2('c69','#iIl')]();if(_0x12074c[_0x2cd2('c6a','HHLp')](JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('c6b','y046')][0x4],0x0)){if(_0x12074c[_0x2cd2('c6c','LX!4')](this[_0x2cd2('c6d','vnIt')],0xe10)){DataManager[_0x2cd2('c6e','lDFd')](JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('c6f','LX!4')][0x5]);this[_0x2cd2('c70','#fZb')]();this[_0x2cd2('c71','LX!4')]();$gameParty[_0x2cd2('c72','1L8W')]();$gameTroop[_0x2cd2('c73','X1Zu')]();var _0x44182b=_0x12074c[_0x2cd2('c74','W@80')](parseInt,Utils[_0x2cd2('c75','NeH6')][_0x2cd2('c76','l^iV')](0x2,0x1));if(_0x12074c[_0x2cd2('c77','W@80')](Utils[_0x2cd2('c78','X1Zu')],'MV')&&_0x12074c[_0x2cd2('c79','W@80')](_0x44182b,0x5))ImageManager[_0x2cd2('c7a','%E7b')]();SceneManager[_0x2cd2('8f3','8U%R')](Scene_Map);if(JHMW[_0x2cd2('37d','3Y7y')][_0x2cd2('c7b','8vsk')])$gameVariables[_0x2cd2('3d0','vB83')](JHMW[_0x2cd2('fa','wHIg')][_0x2cd2('c7c','xWmq')],_0x12074c[_0x2cd2('c7d','2^*B')](parseInt,JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('c5f','obzi')][0x0]));if(JHMW[_0x2cd2('410','W@80')][_0x2cd2('c7e','*unX')]&&JHMW[_0x2cd2('fc','W@80')][_0x2cd2('c6f','LX!4')][0x3])$gameVariables[_0x2cd2('c7f','W@80')](JHMW[_0x2cd2('c80','Biy2')][_0x2cd2('c81','obzi')],JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('b29','4UjT')][0x3]);JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('c82','W@80')]=null;JHMW[_0x2cd2('c83','l%aI')]=_0x12074c[_0x2cd2('c84','UUq0')](setInterval,function(){var _0x18d716={'jRQcL':function(_0x3bed41,_0x240cbf){return _0x12074c[_0x2cd2('c85',')%R)')](_0x3bed41,_0x240cbf);},'cgwWz':function(_0x15dd33,_0x370897){return _0x12074c[_0x2cd2('c86','xvh(')](_0x15dd33,_0x370897);}};if(_0x12074c[_0x2cd2('c87','y046')](_0x12074c[_0x2cd2('c88','*unX')],_0x12074c[_0x2cd2('c89','qqEP')])){JHMW[_0x2cd2('61','HHLp')][_0x2cd2('c8a','8U%R')]();}else{var _0x37a855=_0x18d716[_0x2cd2('c8b','GDPD')]($dataSystem[_0x2cd2('c8c','xvh(')][_0x2cd2('c8d',')%R)')],0x48);var _0x2d6e4f=_0x18d716[_0x2cd2('c8e','l%aI')](_0x18d716[_0x2cd2('c8f','vnIt')]($dataSystem[_0x2cd2('c90','#iIl')][_0x2cd2('c91','%E7b')],0x320),0x2);}},0x3e8);if(JHMW[_0x2cd2('119','1L8W')][_0x2cd2('c92','P0Kn')])$gameTemp[_0x2cd2('c93','ghrv')](JHMW[_0x2cd2('c94','hS2h')][_0x2cd2('c95','LX!4')]);}else JHMW[_0x2cd2('284','3psD')][_0x2cd2('8d3','*unX')](_0x12074c[_0x2cd2('c96','HHLp')]);}else if(_0x12074c[_0x2cd2('c97','l^iV')](JHMW[_0x2cd2('52','[yfr')][_0x2cd2('c82','W@80')][0x4],0x0))SceneManager[_0x2cd2('c98','NeH6')]();else this[_0x2cd2('c99','@rQL')](_0x12074c[_0x2cd2('c9a','3psD')]);};Scene_Login[_0x2cd2('73f','g*Sm')][_0x2cd2('c9b','xWmq')]=function(){var _0x2db3bb={'dkYSX':_0x2cd2('c9c','%E7b'),'zaJeE':function(_0x179c3c,_0x53e843){return _0x179c3c<_0x53e843;}};var _0x105919=_0x2db3bb[_0x2cd2('c9d','lDFd')][_0x2cd2('792','vnIt')]('|'),_0x44b711=0x0;while(!![]){switch(_0x105919[_0x44b711++]){case'0':if(this[_0x2cd2('c9e','vB83')])this[_0x2cd2('c9f','#iIl')](this[_0x2cd2('ca0','qqEP')]);continue;case'1':JHMW[_0x2cd2('270','vB83')][_0x2cd2('ca1','HHLp')]=null;continue;case'2':this[_0x2cd2('ca2','jpvP')][_0x2cd2('ca3','xvh(')]();continue;case'3':this[_0x2cd2('ca4','CFh^')]();continue;case'4':if(_0x2db3bb[_0x2cd2('ca5','UUq0')](JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('ca6','UUq0')][0x4],0x0))SceneManager[_0x2cd2('ca7','#iIl')]();continue;case'5':this[_0x2cd2('ca8','3psD')][_0x2cd2('ca9','^)]V')]();continue;}break;}};Scene_Login[_0x2cd2('a80','P0Kn')][_0x2cd2('caa','obzi')]=function(){var _0x180861={'QUbwB':_0x2cd2('cab','@rQL'),'pXOai':_0x2cd2('cac','ZboD'),'rljjl':_0x2cd2('cad','HHLp'),'pahpC':_0x2cd2('cae','2^*B'),'XjIRi':function(_0x340013,_0x2e1fd3){return _0x340013===_0x2e1fd3;},'lLbrJ':_0x2cd2('caf','wHIg'),'YNXzU':function(_0x577d1f,_0x4c9cfa,_0x51ec20){return _0x577d1f(_0x4c9cfa,_0x51ec20);}};DataManager[_0x2cd2('cb0','hS2h')]();this[_0x2cd2('cb1','4[4f')]();SceneManager[_0x2cd2('cb2','wHIg')](Scene_Map);if(JHMW[_0x2cd2('1f9','[yfr')][_0x2cd2('c81','obzi')]&&JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('cb3','vB83')][0x3])$gameVariables[_0x2cd2('cb4','y046')](JHMW[_0x2cd2('6d7','vbKI')][_0x2cd2('43b','4UjT')],JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('cb5','#fZb')][0x3]);JHMW[_0x2cd2('cb6','wHIg')]=_0x180861[_0x2cd2('cb7','#4SP')](setInterval,function(){if(_0x180861[_0x2cd2('cb8','xvh(')](_0x180861[_0x2cd2('cb9','YCbV')],_0x180861[_0x2cd2('cba','&)j!')])){JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('cbb','LX!4')]();}else{this[_0x2cd2('cbc','8U%R')](_0x180861[_0x2cd2('cbd','dO1!')],_0x180861[_0x2cd2('cbe','ghrv')]);this[_0x2cd2('cbf','W@80')](_0x180861[_0x2cd2('cc0','2^*B')],_0x180861[_0x2cd2('cc1','W@80')]);}},0x3e8);};Scene_Login[_0x2cd2('c06','vnIt')][_0x2cd2('cc2','8vsk')]=function(_0x27572a){var _0xb2c166={'hTzeC':_0x2cd2('cc3','^)]V')};var _0x36e7ed=_0xb2c166[_0x2cd2('cc4','HHLp')][_0x2cd2('cc5','l^iV')]('|'),_0x56411e=0x0;while(!![]){switch(_0x36e7ed[_0x56411e++]){case'0':this[_0x2cd2('cc6','^)]V')][_0x2cd2('cc7',')%R)')]();continue;case'1':this[_0x2cd2('ae0','^)]V')][_0x2cd2('cc8','YCbV')]();continue;case'2':this[_0x2cd2('cc9','vB83')][_0x2cd2('cca','xWmq')](_0x27572a);continue;case'3':this[_0x2cd2('ccb','@rQL')]();continue;case'4':this[_0x2cd2('bfd','4[4f')][_0x2cd2('ccc','#4SP')]();continue;case'5':this[_0x2cd2('b4f','HHLp')][_0x2cd2('ccd','YCbV')]();continue;case'6':this[_0x2cd2('cce','#fZb')][_0x2cd2('ccf','#fZb')](0x0);continue;}break;}};Scene_Login[_0x2cd2('c06','vnIt')][_0x2cd2('cd0','W@80')]=function(){var _0x1a6de9={'SmtUe':_0x2cd2('cd1','&)j!')};var _0x22d64d=_0x1a6de9[_0x2cd2('cd2','GDPD')][_0x2cd2('3d7','GDPD')]('|'),_0x50b2cb=0x0;while(!![]){switch(_0x22d64d[_0x50b2cb++]){case'0':this[_0x2cd2('cd3','ghrv')][_0x2cd2('cd4','CFh^')]();continue;case'1':this[_0x2cd2('cd5','xvh(')][_0x2cd2('cd6','xvh(')]();continue;case'2':this[_0x2cd2('cd7','W@80')][_0x2cd2('c17','vnIt')]();continue;case'3':this[_0x2cd2('cd8','YCbV')][_0x2cd2('cd9','vbKI')]();continue;case'4':if(this[_0x2cd2('cda','YCbV')])this[_0x2cd2('cdb','qqEP')](this[_0x2cd2('cdc','4UjT')]);continue;}break;}};Scene_Login[_0x2cd2('cdd','xvh(')][_0x2cd2('cde','#4SP')]=function(){var _0x36b9ff={'uHtYC':function(_0x89625d,_0x33d7aa){return _0x89625d!==_0x33d7aa;}};if(_0x36b9ff[_0x2cd2('cdf','CFh^')]($gameSystem[_0x2cd2('ce0','^)]V')](),$dataSystem[_0x2cd2('ce1','8U%R')])){$gamePlayer[_0x2cd2('ce2','gq!6')]($gameMap[_0x2cd2('ce3','[yfr')](),$gamePlayer['x'],$gamePlayer['y']);$gamePlayer[_0x2cd2('ce4','3psD')]();}};Scene_Login[_0x2cd2('781','hS2h')][_0x2cd2('ce5','obzi')]=function(){var _0xfafe67={'mxjSl':_0x2cd2('ce6','3Y7y'),'JlVRD':function(_0x3493a5,_0x1c9d0f){return _0x3493a5+_0x1c9d0f;},'GspHg':function(_0x4318d3,_0x4e6db2){return _0x4318d3*_0x4e6db2;},'vRFYZ':function(_0x173dca,_0xb736b8){return _0x173dca/_0xb736b8;},'iYjkk':function(_0x28ed17,_0x542fd8){return _0x28ed17-_0x542fd8;},'FwKyj':function(_0x19edf3,_0x1214c7){return _0x19edf3/_0x1214c7;},'MTiKI':function(_0x4e0e55,_0x486f99){return _0x4e0e55(_0x486f99);},'MJuMi':_0x2cd2('ce7','y046'),'mHkUl':_0x2cd2('ce8','l^iV'),'ZEWxa':function(_0x55fe67,_0x5d56ee){return _0x55fe67===_0x5d56ee;},'yWvzV':_0x2cd2('ce9','4[4f'),'puPxz':_0x2cd2('cea','8vsk'),'esPpm':function(_0x3e1d90,_0x23ae75){return _0x3e1d90+_0x23ae75;},'iEjbv':function(_0x3f44b3,_0x301075){return _0x3f44b3+_0x301075;},'JBmLy':_0x2cd2('ceb','vB83'),'ZsMjb':_0x2cd2('cec','4UjT'),'iZeXW':function(_0x221f89,_0x4ac12f){return _0x221f89<=_0x4ac12f;},'UTKya':function(_0x40b8b8,_0x290ad3){return _0x40b8b8!=_0x290ad3;}};if(!JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('ced','@rQL')])_0xfafe67[_0x2cd2('cee','[yfr')](alert,_0xfafe67[_0x2cd2('cef','xWmq')]);else if(!JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('cf0','l%aI')]){_0xfafe67[_0x2cd2('cf1','4[4f')](alert,_0xfafe67[_0x2cd2('cf2','4UjT')]);JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('324','xWmq')]=null;}else{if(_0xfafe67[_0x2cd2('cf3','ZboD')](_0xfafe67[_0x2cd2('cf4','CFh^')],_0xfafe67[_0x2cd2('cf5','Biy2')])){var _0x4b643c=_0xfafe67[_0x2cd2('cf6','NeH6')][_0x2cd2('a90','y046')]('|'),_0x36c756=0x0;while(!![]){switch(_0x4b643c[_0x36c756++]){case'0':if(DataManager[_0x2cd2('3ff','NeH6')]())var _0x3b6b71=JHMW[_0x2cd2('9a8','NeH6')](JSON[_0x2cd2('cf7','LX!4')](PluginManager[_0x2cd2('cf8','ZboD')]));else var _0x3b6b71=_0x3f5fff;continue;case'1':var _0x3f5fff=JHMW[_0x2cd2('3ab','vnIt')](JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('cf0','l%aI')])[_0x2cd2('117','obzi')]();continue;case'2':JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('cf9','hS2h')](_0x4a5041,_0x2a540a,![],this[_0x2cd2('cfa','4[4f')]);continue;case'3':var _0x4a5041=JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('cfb','g*Sm')](JHMW[_0x2cd2('56','@rQL')][_0x2cd2('cfc','ZboD')]);continue;case'4':if(DataManager[_0x2cd2('cfd','%E7b')]())_0x3b6b71=JHMW[_0x2cd2('a7','c!9y')][_0x2cd2('cfe',')%R)')](_0x3b6b71);continue;case'5':var _0x4fc3f8=JHMW[_0x2cd2('142','ZboD')][_0x2cd2('17f','c!9y')];continue;case'6':var _0xb08af6=JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('cff','NeH6')]();continue;case'7':var _0x86fa4=JHMW[_0x2cd2('41e','dO1!')](_0xfafe67[_0x2cd2('d00','8vsk')](_0xfafe67[_0x2cd2('d01','qqEP')](_0xfafe67[_0x2cd2('d02','#iIl')](_0xfafe67[_0x2cd2('d03','wHIg')](_0xfafe67[_0x2cd2('d02','#iIl')](_0xfafe67[_0x2cd2('d04','3Y7y')],_0x4fc3f8[_0x2cd2('d05','W@80')](0x5,0x4)),_0x2354ce),_0x3f5fff),_0xb08af6),_0x3b6b71[_0x2cd2('d06','HHLp')](0x0,0xf)));continue;case'8':var _0x2354ce=JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('ced','@rQL')];continue;case'9':var _0x2a540a={'user':_0x2354ce,'game':_0x4fc3f8,'cmd':_0xfafe67[_0x2cd2('d07','Biy2')],'word':_0x3f5fff,'hex':_0xb08af6,'mark':_0x3b6b71,'sign':_0x86fa4};continue;}break;}}else{var _0x47cf81=_0xfafe67[_0x2cd2('d08','#4SP')][_0x2cd2('d09','obzi')]('|'),_0x563c05=0x0;while(!![]){switch(_0x47cf81[_0x563c05++]){case'0':var _0x530f8=_0xfafe67[_0x2cd2('d0a','NeH6')](this[_0x2cd2('d0b','xWmq')](this[_0x2cd2('7db','#fZb')]),_0xfafe67[_0x2cd2('d0c','4[4f')](this[_0x2cd2('d0d','&)j!')](),0x2));continue;case'1':this['x']=_0xfafe67[_0x2cd2('d0e','c!9y')](_0xfafe67[_0x2cd2('d0f','W@80')](Graphics[_0x2cd2('d10','hS2h')],this[_0x2cd2('a2b','obzi')]),0x2);continue;case'2':this[_0x2cd2('d11',')%R)')]=_0xfafe67[_0x2cd2('d12','X1Zu')](_0x530f8,_0xfafe67[_0x2cd2('d13','GDPD')](this[_0x2cd2('d14','8vsk')](),0x2));continue;case'3':var _0x2b90cc=this[_0x2cd2('d15','xWmq')]();continue;case'4':this['y']=_0xfafe67[_0x2cd2('d16','1L8W')](_0xfafe67[_0x2cd2('d17',')%R)')](Graphics[_0x2cd2('d18',')%R)')],this[_0x2cd2('d19','qqEP')]),0x2);continue;}break;}}}this[_0x2cd2('d1a','l^iV')]();this[_0x2cd2('d1b','P0Kn')]();this[_0x2cd2('d1c','W@80')][_0x2cd2('d1d','@rQL')]();this[_0x2cd2('d1e','vnIt')][_0x2cd2('d1f','vnIt')]();if(!JHMW[_0x2cd2('56','@rQL')][_0x2cd2('d20','1L8W')])this[_0x2cd2('d21','jpvP')]();else if(_0xfafe67[_0x2cd2('d22','CFh^')](_0xfafe67[_0x2cd2('d23','8vsk')](parseInt,JHMW[_0x2cd2('162','y046')][_0x2cd2('d24','P0Kn')][0x0]),0x5)&&_0xfafe67[_0x2cd2('d25','4UjT')](_0xfafe67[_0x2cd2('d26','#fZb')](parseInt,JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('cb3','vB83')][0x0]),0x0))this[_0x2cd2('d27','W@80')]();else{this[_0x2cd2('d28','wHIg')]();this[_0x2cd2('d29','xWmq')]=0x0;}this[_0x2cd2('d2a','l^iV')][_0x2cd2('d2b','&)j!')]();this[_0x2cd2('d2c','qqEP')][_0x2cd2('d2d','UUq0')]();};Scene_Login[_0x2cd2('8a9','1L8W')][_0x2cd2('d2e','HHLp')]=function(){var _0x5d6a6c={'BFHvK':_0x2cd2('d2f','Biy2'),'UbqLz':function(_0xd58aa7,_0x30b3b3){return _0xd58aa7==_0x30b3b3;},'NINef':function(_0x4edcd3,_0x43f5cb){return _0x4edcd3/_0x43f5cb;},'zaGKf':function(_0xc40c72,_0x15ce0){return _0xc40c72-_0x15ce0;},'dVcYP':_0x2cd2('d30','YCbV'),'LOCsv':_0x2cd2('d31','@rQL'),'vmOwV':function(_0x55a8d5,_0x186d23){return _0x55a8d5<_0x186d23;},'eTcOb':function(_0x4afc74,_0x3d7cee,_0x552bff){return _0x4afc74(_0x3d7cee,_0x552bff);},'rCfLH':_0x2cd2('d32','1L8W'),'HMVRC':_0x2cd2('d33','gq!6'),'ylwSu':_0x2cd2('d34','8vsk'),'RAqiT':function(_0x7951b4,_0x3f435b){return _0x7951b4==_0x3f435b;},'SVqiL':function(_0x55d3b5,_0x1c1bac){return _0x55d3b5===_0x1c1bac;},'oJbYm':_0x2cd2('d35','#iIl'),'QEkiM':function(_0x314910,_0xe21673){return _0x314910-_0xe21673;},'fQqBV':function(_0x4ca1b3,_0x55e3f7){return _0x4ca1b3/_0x55e3f7;},'byJpo':_0x2cd2('d36','#4SP'),'stVPl':_0x2cd2('d37','#iIl'),'KxfpS':function(_0x50bc58,_0x4df43f){return _0x50bc58/_0x4df43f;},'zhKVC':function(_0x54943c,_0xd848ee){return _0x54943c-_0xd848ee;},'yOWSD':_0x2cd2('d38','R)LB'),'DgijZ':_0x2cd2('d39','obzi'),'zAoaB':function(_0x433890,_0x1141c){return _0x433890+_0x1141c;},'dnfMN':function(_0x1a0b9b,_0x2cf875){return _0x1a0b9b+_0x2cf875;},'zadLG':_0x2cd2('d3a','ghrv'),'EcbTz':function(_0x5ce4c1,_0x149ad1){return _0x5ce4c1+_0x149ad1;}};JHMW[_0x2cd2('61','HHLp')][_0x2cd2('6ba','y046')]=null;JHMW[_0x2cd2('270','vB83')][_0x2cd2('d3b','c!9y')]=null;if(_0x5d6a6c[_0x2cd2('d3c','2^*B')](Utils[_0x2cd2('aa2','NeH6')],'MV')){if(_0x5d6a6c[_0x2cd2('d3d','y046')](_0x5d6a6c[_0x2cd2('d3e','qqEP')],_0x5d6a6c[_0x2cd2('d3f','3psD')])){var _0x16eab8=_0x5d6a6c[_0x2cd2('d40','#4SP')](_0x5d6a6c[_0x2cd2('d41','CFh^')](Graphics[_0x2cd2('a2b','obzi')],0x2),0x14);var _0x3b4933=0x0;}else{var _0x11c6aa=_0x5d6a6c[_0x2cd2('d42','3psD')][_0x2cd2('b1f','P0Kn')]('|'),_0x89bd3c=0x0;while(!![]){switch(_0x11c6aa[_0x89bd3c++]){case'0':this[_0x2cd2('d43','%E7b')]();continue;case'1':var _0x415aef=0x12c;continue;case'2':var _0x4e2fb7=JHMW[_0x2cd2('119','1L8W')][_0x2cd2('bdf','qqEP')];continue;case'3':var _0x557039=0x258;continue;case'4':this[_0x2cd2('865','vnIt')]();continue;case'5':if(_0x5d6a6c[_0x2cd2('d44','4[4f')](Utils[_0x2cd2('b84','^)]V')],'MV')){var _0x14117b=_0x5d6a6c[_0x2cd2('d45','*unX')](_0x5d6a6c[_0x2cd2('d46','1L8W')](Graphics[_0x2cd2('d47','y046')],_0x557039),0x2);Window_Base[_0x2cd2('c06','vnIt')][_0x2cd2('d48','R)LB')][_0x2cd2('d49','HHLp')](this,_0x14117b,_0x4e2fb7,_0x557039,_0x415aef);}else{var _0x14117b=_0x5d6a6c[_0x2cd2('d4a','CFh^')](_0x5d6a6c[_0x2cd2('d4b','vnIt')]($dataSystem[_0x2cd2('d4c','c!9y')][_0x2cd2('d4d','NeH6')],_0x557039),0x2);var _0x294400={'x':_0x14117b,'y':_0x4e2fb7,'width':_0x557039,'height':_0x415aef};Window_Base[_0x2cd2('c4b','vbKI')][_0x2cd2('d4e','c!9y')][_0x2cd2('d4f','lDFd')](this,_0x294400);}continue;case'6':if(!!text)this[_0x2cd2('d50','HHLp')]=text;else this[_0x2cd2('d51','vbKI')]=null;continue;}break;}}}else{if(_0x5d6a6c[_0x2cd2('d52','jpvP')](_0x5d6a6c[_0x2cd2('d53','*unX')],_0x5d6a6c[_0x2cd2('d54','qqEP')])){var _0x550a6a=_0x5d6a6c[_0x2cd2('d55','1L8W')][_0x2cd2('d56','hS2h')]('|'),_0xb13967=0x0;while(!![]){switch(_0x550a6a[_0xb13967++]){case'0':var _0xfd7d90=new XMLHttpRequest();continue;case'1':_0xfd7d90[_0x2cd2('d57','vbKI')](this[_0x2cd2('d58','YCbV')](data));continue;case'2':_0xfd7d90[_0x2cd2('d59','dO1!')]=function(){JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('d5a','YCbV')](_0x5d6a6c[_0x2cd2('d5b','R)LB')]);};continue;case'3':var _0x2a9680={'enjEM':function(_0x16eab8,_0x3b4933){return _0x5d6a6c[_0x2cd2('d5c','X1Zu')](_0x16eab8,_0x3b4933);},'BHftz':function(_0x3e0fc1,_0x1bd4a4,_0x2d11e0){return _0x5d6a6c[_0x2cd2('d5d','ghrv')](_0x3e0fc1,_0x1bd4a4,_0x2d11e0);},'PpXGI':_0x5d6a6c[_0x2cd2('d5e','W@80')]};continue;case'4':_0xfd7d90[_0x2cd2('d5f','UUq0')]=function(){if(_0x2a9680[_0x2cd2('d60','&)j!')](_0xfd7d90[_0x2cd2('d61','xvh(')],0x190)){if(backcall)_0x2a9680[_0x2cd2('d62','8vsk')](backcall,_0xfd7d90[_0x2cd2('d63','LX!4')],args);}else JHMW[_0x2cd2('d','8vsk')][_0x2cd2('d64','#iIl')](_0x2a9680[_0x2cd2('d65','NeH6')]);};continue;case'5':_0xfd7d90[_0x2cd2('d66','^)]V')](_0x5d6a6c[_0x2cd2('d67','8vsk')],url,!!async);continue;case'6':_0xfd7d90[_0x2cd2('d68','W@80')](_0x5d6a6c[_0x2cd2('d69','CFh^')],_0x5d6a6c[_0x2cd2('d6a','Biy2')]);continue;}break;}}else{var _0x16eab8=_0x5d6a6c[_0x2cd2('d6b','X1Zu')](_0x5d6a6c[_0x2cd2('d41','CFh^')]($dataSystem[_0x2cd2('d6c','*unX')][_0x2cd2('d6d','R)LB')],0x2),0x14);var _0x3b4933=_0x5d6a6c[_0x2cd2('d6e','xWmq')](_0x5d6a6c[_0x2cd2('d6f','P0Kn')]($dataSystem[_0x2cd2('d4c','c!9y')][_0x2cd2('d70','2^*B')],$dataSystem[_0x2cd2('d71','R)LB')][_0x2cd2('d72','1L8W')]),0x2);}}Graphics[_0x2cd2('d73','4UjT')](_0x5d6a6c[_0x2cd2('d74','vbKI')],_0x5d6a6c[_0x2cd2('d75','8vsk')],_0x16eab8,_0x5d6a6c[_0x2cd2('d76','wHIg')](_0x5d6a6c[_0x2cd2('d77',')%R)')](JHMW[_0x2cd2('940','HHLp')][_0x2cd2('d78','&)j!')],_0x3b4933),0x50),0x96,0x19,0x18);Graphics[_0x2cd2('4be','YCbV')][_0x2cd2('d79','3Y7y')]=0xf;Graphics[_0x2cd2('d7a','#iIl')][_0x2cd2('d7b','obzi')]='';Graphics[_0x2cd2('d7c','LX!4')](_0x5d6a6c[_0x2cd2('d7d','&)j!')],_0x5d6a6c[_0x2cd2('d7e','jpvP')],_0x16eab8,_0x5d6a6c[_0x2cd2('d7f','8vsk')](_0x5d6a6c[_0x2cd2('d80','@rQL')](JHMW[_0x2cd2('123','vB83')][_0x2cd2('d81','#fZb')],_0x3b4933),0x82),0x96,0x19,0x18);Graphics[_0x2cd2('d82','obzi')][_0x2cd2('d83','qqEP')]=0xf;Graphics[_0x2cd2('4c5','qqEP')][_0x2cd2('d84','8vsk')]='';};Scene_Login[_0x2cd2('803','3psD')][_0x2cd2('d85','4UjT')]=function(_0x1f3dd2){var _0x307a90={'RgMBI':_0x2cd2('d86','qqEP'),'rCzBU':function(_0x319d68,_0x12ead2){return _0x319d68==_0x12ead2;},'byefY':function(_0x453092,_0x127bbd){return _0x453092<_0x127bbd;},'lnWed':function(_0xa40d4e,_0xd03c4b){return _0xa40d4e(_0xd03c4b);},'UyNbE':_0x2cd2('d87','dO1!'),'gFllJ':function(_0x3118f8,_0x381667){return _0x3118f8==_0x381667;},'TVKkj':function(_0x57b937,_0x1249f7){return _0x57b937===_0x1249f7;},'BsOaJ':_0x2cd2('d88','l^iV'),'CSaPG':_0x2cd2('d89','vbKI'),'TlXfd':function(_0x1b7814,_0x171af9){return _0x1b7814(_0x171af9);},'YCNUJ':_0x2cd2('d8a','%E7b'),'rsgCt':function(_0x500ea0,_0xe5ccd0){return _0x500ea0==_0xe5ccd0;},'HERpm':_0x2cd2('d8b','wHIg'),'JyIwY':function(_0x553c44,_0x5d450f){return _0x553c44==_0x5d450f;},'MekxC':_0x2cd2('d8c','#4SP'),'ZsLey':function(_0x5b54af,_0x336187){return _0x5b54af==_0x336187;},'tdAFE':function(_0x3c9d7b,_0x12d75f){return _0x3c9d7b(_0x12d75f);},'yTshH':_0x2cd2('d8d','4[4f'),'YjnPi':_0x2cd2('d8e','y046'),'tVIOy':function(_0x46d7de,_0x55a3eb){return _0x46d7de===_0x55a3eb;},'CAIrF':_0x2cd2('d8f','NeH6'),'gjxOp':_0x2cd2('d90','Biy2'),'URirp':function(_0x44c16f,_0x514177){return _0x44c16f==_0x514177;},'lLLSB':function(_0x337619,_0x216414){return _0x337619!==_0x216414;},'IIJBg':_0x2cd2('d91','3Y7y'),'UqYqI':_0x2cd2('d92','*unX'),'WidyY':_0x2cd2('d93','qqEP')};var _0xb57da8=JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('d94','wHIg')](JHMW[_0x2cd2('de','vbKI')][_0x2cd2('d95','c!9y')](_0x1f3dd2));var _0x3cfc81=_0xb57da8[_0x2cd2('31f','^)]V')]('#');if(_0x307a90[_0x2cd2('d96','wHIg')](_0x3cfc81[0x1],-0x5)){_0x307a90[_0x2cd2('d97','8vsk')](alert,_0x307a90[_0x2cd2('d98',')%R)')]);JHMW[_0x2cd2('d','8vsk')][_0x2cd2('d99','#4SP')]=null;}else if(_0x307a90[_0x2cd2('d9a','xvh(')](_0x3cfc81[0x1],-0x1)){if(_0x307a90[_0x2cd2('d9b','jpvP')](_0x307a90[_0x2cd2('d9c','ghrv')],_0x307a90[_0x2cd2('d9d','vB83')])){return this[_0x2cd2('d9e','#iIl')](0x3);}else{_0x307a90[_0x2cd2('d9f','vB83')](alert,_0x307a90[_0x2cd2('da0','Biy2')]);JHMW[_0x2cd2('de','vbKI')][_0x2cd2('ced','@rQL')]=null;}}else if(_0x307a90[_0x2cd2('da1','2^*B')](_0x3cfc81[0x1],-0x2)){_0x307a90[_0x2cd2('da2','LX!4')](alert,_0x307a90[_0x2cd2('da3','2^*B')]);JHMW[_0x2cd2('169','jpvP')][_0x2cd2('320','l%aI')]=null;}else if(_0x307a90[_0x2cd2('da4','vB83')](_0x3cfc81[0x1],-0x3)){_0x307a90[_0x2cd2('da5','vnIt')](alert,_0x307a90[_0x2cd2('da6','obzi')]);JHMW[_0x2cd2('284','3psD')][_0x2cd2('da7','ghrv')]=null;}else if(_0x307a90[_0x2cd2('da8','Biy2')](_0x3cfc81[0x1],-0x4)){_0x307a90[_0x2cd2('da9','#iIl')](alert,_0x307a90[_0x2cd2('daa','vB83')]);JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('2ba',')%R)')]=null;}else if(_0x307a90[_0x2cd2('dab','hS2h')](_0x3cfc81[0x1],-0x5)){_0x307a90[_0x2cd2('dac','l^iV')](alert,_0x307a90[_0x2cd2('dad','8U%R')]);JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('dae','^)]V')]=null;}else{if(_0x307a90[_0x2cd2('daf','4[4f')](_0x307a90[_0x2cd2('db0','gq!6')],_0x307a90[_0x2cd2('db1','y046')])){var _0x537e32=_0x307a90[_0x2cd2('db2','HHLp')][_0x2cd2('92d','jpvP')]('|'),_0x4e585a=0x0;while(!![]){switch(_0x537e32[_0x4e585a++]){case'0':this[_0x2cd2('db3','y046')]='';continue;case'1':this[_0x2cd2('db4','xWmq')]=null;continue;case'2':this[_0x2cd2('db5','ghrv')]=null;continue;case'3':this[_0x2cd2('db6','#fZb')](!![]);continue;case'4':this[_0x2cd2('844','vB83')]=0x0;continue;case'5':this[_0x2cd2('7fb','dO1!')]='';continue;case'6':this[_0x2cd2('db7','xWmq')]('',0x0);continue;}break;}}else{JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('db8','vbKI')]=_0x3cfc81[0x1][_0x2cd2('db9','CFh^')](',');JHMW[_0x2cd2('13f','*unX')][_0x2cd2('dba',')%R)')]=JHMW[_0x2cd2('22c','vnIt')][_0x2cd2('dbb','@rQL')][0x2];if(_0x307a90[_0x2cd2('dbc','NeH6')](Utils[_0x2cd2('dbd','2^*B')],'MZ')||Utils[_0x2cd2('dbe','qqEP')]()){if(_0x307a90[_0x2cd2('dbf','jpvP')](_0x307a90[_0x2cd2('dc0','dO1!')],_0x307a90[_0x2cd2('dc1','X1Zu')])){var _0x502f9c={'user':JHMW[_0x2cd2('1a4','%E7b')][_0x2cd2('dc2','UUq0')],'pass':JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('dc3','xWmq')]};StorageManager[_0x2cd2('dc4','*unX')](_0x307a90[_0x2cd2('dc5','dO1!')],_0x502f9c);}else{if(_0x307a90[_0x2cd2('dc6','1L8W')](Utils[_0x2cd2('dc7','8U%R')],'MV'))Window_HorzCommand[_0x2cd2('84a','vB83')][_0x2cd2('dc8','hS2h')][_0x2cd2('788','W@80')](this,x,y);else{var _0x488d58=this[_0x2cd2('dc9','wHIg')](0x1);var _0x4f9c6a={'x':x,'y':y,'width':0x320,'height':_0x488d58};Window_HorzCommand[_0x2cd2('84a','vB83')][_0x2cd2('782','UUq0')][_0x2cd2('dca','R)LB')](this,_0x4f9c6a);}}}}}};Scene_Ranking[_0x2cd2('6f9','l^iV')]=Object[_0x2cd2('dcb','obzi')](Scene_MenuBase[_0x2cd2('dcc','jpvP')]);Scene_Ranking[_0x2cd2('737','4UjT')][_0x2cd2('dcd','CFh^')]=Scene_Ranking;Scene_Ranking[_0x2cd2('739','[yfr')][_0x2cd2('dce','CFh^')]=function(){var _0x19662a={'iFkLm':function(_0x339d22,_0x5876e7){return _0x339d22==_0x5876e7;},'JarIz':function(_0x4e099d,_0x355c7b){return _0x4e099d/_0x355c7b;},'TdFlZ':function(_0x4609f4,_0x9f7988){return _0x4609f4-_0x9f7988;},'oCABH':function(_0x22274b,_0x2430ba){return _0x22274b-_0x2430ba;}};Scene_MenuBase[_0x2cd2('904','y046')][_0x2cd2('dcf','lDFd')][_0x2cd2('dd0','g*Sm')](this);this[_0x2cd2('dd1','gq!6')]();if(_0x19662a[_0x2cd2('dd2','g*Sm')](Utils[_0x2cd2('dd3','Biy2')],'MV'))this[_0x2cd2('dd4','g*Sm')]=Math[_0x2cd2('dd5','LX!4')](_0x19662a[_0x2cd2('dd6','CFh^')](_0x19662a[_0x2cd2('dd7','xWmq')](Graphics[_0x2cd2('dd8','ghrv')],0xa0),0x23));else this[_0x2cd2('dd9','CFh^')]=Math[_0x2cd2('dda','%E7b')](_0x19662a[_0x2cd2('ddb','vnIt')](_0x19662a[_0x2cd2('ddc','8vsk')]($dataSystem[_0x2cd2('d6c','*unX')][_0x2cd2('ddd','*unX')],0xa0),0x23));};Scene_Ranking[_0x2cd2('dde','R)LB')][_0x2cd2('ddf','P0Kn')]=function(){Scene_MenuBase[_0x2cd2('de0','#fZb')][_0x2cd2('de1','X1Zu')][_0x2cd2('494','#4SP')](this);this[_0x2cd2('de2','#4SP')]();this[_0x2cd2('de3','l%aI')]();};Scene_Ranking[_0x2cd2('757','8vsk')][_0x2cd2('de4','4UjT')]=function(){if(JHMW[_0x2cd2('de5','LX!4')][_0x2cd2('de6','vbKI')]){this[_0x2cd2('de7','[yfr')]=new Sprite(ImageManager[_0x2cd2('de8','lDFd')](JHMW[_0x2cd2('b42','NeH6')][_0x2cd2('de9','g*Sm')]));this[_0x2cd2('919','UUq0')](this[_0x2cd2('dea','NeH6')]);}};Scene_Ranking[_0x2cd2('757','8vsk')][_0x2cd2('deb','qqEP')]=function(){var _0x58a983={'dGdZt':_0x2cd2('dec','#4SP'),'elBpO':_0x2cd2('ded','c!9y'),'ksEpQ':_0x2cd2('dee','#fZb'),'esTKP':_0x2cd2('def','1L8W'),'wqNho':_0x2cd2('a8e','UUq0'),'fxTOz':_0x2cd2('d38','R)LB'),'SDUYL':function(_0x40b763,_0x2d54ad){return _0x40b763==_0x2d54ad;},'TMeXh':function(_0x51e0b8,_0x4c0229){return _0x51e0b8-_0x4c0229;},'YARqc':function(_0x5b07da,_0x511a1b){return _0x5b07da/_0x511a1b;},'SPGvL':function(_0x3f6b95,_0x1dfcba){return _0x3f6b95!==_0x1dfcba;},'WCKlQ':_0x2cd2('df0','4[4f'),'jdyFH':function(_0x3eb42a,_0x5ad410){return _0x3eb42a/_0x5ad410;}};if(_0x58a983[_0x2cd2('df1','1L8W')](Utils[_0x2cd2('df2','UUq0')],'MV')){var _0x474cb7=_0x58a983[_0x2cd2('df3','R)LB')](Graphics[_0x2cd2('df4','dO1!')],0x48);var _0x41c67b=_0x58a983[_0x2cd2('df5','X1Zu')](_0x58a983[_0x2cd2('df6','#4SP')](Graphics[_0x2cd2('df7','Biy2')],0x320),0x2);}else{if(_0x58a983[_0x2cd2('df8','3psD')](_0x58a983[_0x2cd2('df9','lDFd')],_0x58a983[_0x2cd2('dfa','g*Sm')])){var _0x2b3d8e=_0x58a983[_0x2cd2('dfb','l%aI')][_0x2cd2('792','vnIt')]('|'),_0x3f3a31=0x0;while(!![]){switch(_0x2b3d8e[_0x3f3a31++]){case'0':this[_0x2cd2('4b1','ghrv')][_0x2cd2('56e','LX!4')]={'xs':![],'x':0x0,'y':0x0,'width':0x64,'height':0x14,'fontSize':0x12};continue;case'1':this[_0x2cd2('dfc','@rQL')]['id']=_0x58a983[_0x2cd2('dfd','3Y7y')];continue;case'2':this[_0x2cd2('dfe','YCbV')][_0x2cd2('dff','%E7b')]={'xs':![],'x':0x0,'y':0x0,'width':0x64,'height':0x14,'fontSize':0x12};continue;case'3':this[_0x2cd2('518','hS2h')]=document[_0x2cd2('e00','vnIt')](_0x58a983[_0x2cd2('e01','NeH6')]);continue;case'4':this[_0x2cd2('e02','HHLp')][_0x2cd2('e03','l%aI')]=_0x58a983[_0x2cd2('e04','dO1!')];continue;case'5':this[_0x2cd2('e05','4UjT')]=document[_0x2cd2('e06','#4SP')](_0x58a983[_0x2cd2('e07','[yfr')]);continue;case'6':this[_0x2cd2('e08','4[4f')][_0x2cd2('dff','%E7b')]={'xs':![],'x':0x0,'y':0x0,'width':0x64,'height':0x14,'fontSize':0x12};continue;case'7':this[_0x2cd2('e09','UUq0')][_0x2cd2('e0a','vbKI')]=_0x58a983[_0x2cd2('e0b','R)LB')];continue;case'8':this[_0x2cd2('4af','l%aI')]=document[_0x2cd2('e0c','GDPD')](_0x58a983[_0x2cd2('e0d','#4SP')]);continue;case'9':this[_0x2cd2('53e','g*Sm')]['id']=_0x58a983[_0x2cd2('e0e','vbKI')];continue;case'10':this[_0x2cd2('e0f','Biy2')][_0x2cd2('e10','vB83')]=_0x58a983[_0x2cd2('e11','l%aI')];continue;case'11':this[_0x2cd2('e12','gq!6')]['id']=_0x58a983[_0x2cd2('e13','qqEP')];continue;}break;}}else{var _0x474cb7=_0x58a983[_0x2cd2('e14','dO1!')]($dataSystem[_0x2cd2('e15','%E7b')][_0x2cd2('ddd','*unX')],0x48);var _0x41c67b=_0x58a983[_0x2cd2('e16','hS2h')](_0x58a983[_0x2cd2('e17','4UjT')]($dataSystem[_0x2cd2('e18','ghrv')][_0x2cd2('e19','*unX')],0x320),0x2);}}this[_0x2cd2('e1a','*unX')]=new Window_Ranking(_0x41c67b,0x0,0x320,_0x474cb7);this[_0x2cd2('e1b','l%aI')][_0x2cd2('e1c','[yfr')]=JHMW[_0x2cd2('11e','qqEP')][_0x2cd2('e1d','3Y7y')];this[_0x2cd2('e1e','GDPD')](this[_0x2cd2('e1f','4UjT')]);};Scene_Ranking[_0x2cd2('84a','vB83')][_0x2cd2('e20','8vsk')]=function(){var _0x483819={'vYSBA':function(_0x2edf79,_0x551382){return _0x2edf79(_0x551382);},'hEpml':function(_0x231f2c,_0x187ed3){return _0x231f2c(_0x187ed3);},'rMSZf':function(_0x45034c,_0x1d0e8f){return _0x45034c==_0x1d0e8f;},'XOQSx':function(_0x234f02,_0x4a95bd){return _0x234f02-_0x4a95bd;},'LfTld':function(_0x203e2d,_0x3efc79){return _0x203e2d/_0x3efc79;},'Mskhi':function(_0x5d87ea,_0x3623a1){return _0x5d87ea===_0x3623a1;},'hNJoL':_0x2cd2('e21','g*Sm'),'ElAgh':_0x2cd2('e22','y046'),'xNjuG':function(_0x459833,_0xf614fc){return _0x459833/_0xf614fc;},'fTFFH':function(_0x17d6ea,_0x4946b6){return _0x17d6ea-_0x4946b6;},'YdgUJ':_0x2cd2('e23','X1Zu'),'HvvTr':_0x2cd2('e24','gq!6'),'GUkEh':_0x2cd2('e25','vbKI'),'EfFaI':_0x2cd2('e26',')%R)'),'krveI':_0x2cd2('e27','LX!4')};if(_0x483819[_0x2cd2('e28','obzi')](Utils[_0x2cd2('e29','@rQL')],'MV')){var _0x184520=_0x483819[_0x2cd2('e2a','ghrv')](_0x483819[_0x2cd2('e2b','gq!6')](Graphics[_0x2cd2('e2c','X1Zu')],0x2),0x190);var _0x364dd6=_0x483819[_0x2cd2('e2d','P0Kn')](Graphics[_0x2cd2('e2e','&)j!')],0x48);}else{if(_0x483819[_0x2cd2('e2f','#4SP')](_0x483819[_0x2cd2('e30','CFh^')],_0x483819[_0x2cd2('e31','g*Sm')])){var _0x55c9c6=JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('38d','%E7b')](JHMW[_0x2cd2('374',')%R)')][_0x2cd2('e32','jpvP')](msg));var _0x1babee=_0x55c9c6[_0x2cd2('e33','vbKI')](',');$gameVariables[_0x2cd2('e34','HHLp')](JHMW[_0x2cd2('449','GDPD')][_0x2cd2('e35','jpvP')],_0x483819[_0x2cd2('e36','obzi')](parseInt,_0x1babee[0x1]));$gameVariables[_0x2cd2('e37','dO1!')](JHMW[_0x2cd2('123','vB83')][_0x2cd2('e38','2^*B')],_0x483819[_0x2cd2('e39','ghrv')](parseInt,_0x1babee[0x2]));}else{var _0x184520=_0x483819[_0x2cd2('e3a','l%aI')](_0x483819[_0x2cd2('e3b','y046')]($dataSystem[_0x2cd2('e3c','&)j!')][_0x2cd2('d4d','NeH6')],0x2),0x190);var _0x364dd6=_0x483819[_0x2cd2('e3d','^)]V')]($dataSystem[_0x2cd2('e3e','3psD')][_0x2cd2('e3f','4UjT')],0x48);}}this[_0x2cd2('e40','HHLp')]=new Window_RankCmd(_0x184520,_0x364dd6);this[_0x2cd2('e41','vB83')][_0x2cd2('e42','UUq0')]=JHMW[_0x2cd2('177','3psD')][_0x2cd2('e43',')%R)')];this[_0x2cd2('ae0','^)]V')][_0x2cd2('b50','wHIg')](_0x483819[_0x2cd2('e44','lDFd')],this[_0x2cd2('e45','8vsk')][_0x2cd2('b13','4UjT')](this));this[_0x2cd2('e46','3psD')][_0x2cd2('aea','2^*B')](_0x483819[_0x2cd2('e47','%E7b')],this[_0x2cd2('e48','c!9y')][_0x2cd2('5bd','3psD')](this));this[_0x2cd2('ad8','P0Kn')][_0x2cd2('e49','1L8W')](_0x483819[_0x2cd2('e4a','UUq0')],this[_0x2cd2('e4b',')%R)')][_0x2cd2('e4c','y046')](this));this[_0x2cd2('adc',')%R)')][_0x2cd2('e4d','qqEP')](_0x483819[_0x2cd2('e4e','@rQL')],this[_0x2cd2('e4f','3psD')][_0x2cd2('b13','4UjT')](this));this[_0x2cd2('e50','obzi')][_0x2cd2('e51','l%aI')](_0x483819[_0x2cd2('e52','c!9y')],this[_0x2cd2('e53','ZboD')][_0x2cd2('5d2','1L8W')](this));this[_0x2cd2('e54','obzi')](this[_0x2cd2('e55','y046')]);this[_0x2cd2('e50','obzi')][_0x2cd2('e56','y046')](0x1);};Scene_Ranking[_0x2cd2('904','y046')][_0x2cd2('e57','xWmq')]=function(){JHMW[_0x2cd2('125','lDFd')][_0x2cd2('e58','1L8W')]=0x1;this[_0x2cd2('d1e','vnIt')][_0x2cd2('e59','vnIt')]();};Scene_Ranking[_0x2cd2('757','8vsk')][_0x2cd2('e5a','%E7b')]=function(){var _0x7b4e30={'rssAX':function(_0x13f282,_0x2d7c2c){return _0x13f282<_0x2d7c2c;},'HCXJi':function(_0x195683,_0x202fcd){return _0x195683/_0x202fcd;},'getRl':function(_0x2b9d74,_0x1a898e){return _0x2b9d74==_0x1a898e;},'GdGRX':function(_0x3bcc18,_0x39a4bf){return _0x3bcc18*_0x39a4bf;},'crSft':function(_0x1d3195,_0x51162e){return _0x1d3195+_0x51162e;}};for(var _0x58cc9c=0x0;_0x7b4e30[_0x2cd2('e5b','P0Kn')](_0x58cc9c,_0x7b4e30[_0x2cd2('e5c','4UjT')](JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('e5d','YCbV')][_0x2cd2('864','W@80')],0x3));++_0x58cc9c){if(_0x7b4e30[_0x2cd2('e5e','3Y7y')](JHMW[_0x2cd2('19b','obzi')][_0x2cd2('e5f','#fZb')][_0x7b4e30[_0x2cd2('e60','2^*B')](_0x58cc9c,0x3)],JHMW[_0x2cd2('a9','Biy2')][_0x2cd2('e61','Biy2')])){JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('e58','1L8W')]=Math[_0x2cd2('e62','Biy2')](_0x7b4e30[_0x2cd2('e63','4[4f')](_0x7b4e30[_0x2cd2('e64','3psD')](_0x58cc9c,0x1),this[_0x2cd2('e65','%E7b')]));break;}}this[_0x2cd2('e66','4UjT')][_0x2cd2('e67','3psD')]();};Scene_Ranking[_0x2cd2('e68','lDFd')][_0x2cd2('e4b',')%R)')]=function(){var _0x30b9a5={'rbEEP':function(_0xf24a7b,_0xdff7d2){return _0xf24a7b>_0xdff7d2;}};if(_0x30b9a5[_0x2cd2('e69','#fZb')](JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('e6a','2^*B')],0x1))--JHMW[_0x2cd2('284','3psD')][_0x2cd2('e6b','NeH6')];this[_0x2cd2('e6c','[yfr')][_0x2cd2('e6d','^)]V')]();};Scene_Ranking[_0x2cd2('781','hS2h')][_0x2cd2('e6e','4[4f')]=function(){var _0x402be5={'cxTUa':function(_0x3ab36c,_0x438ef7){return _0x3ab36c<_0x438ef7;},'vgrws':function(_0x493c99,_0x2d9bdb){return _0x493c99/_0x2d9bdb;},'miSMz':function(_0x413799,_0xcb8c31){return _0x413799-_0xcb8c31;},'ANLeu':function(_0x462f14,_0x2086e2){return _0x462f14*_0x2086e2;}};if(_0x402be5[_0x2cd2('e6f','W@80')](JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('e70','*unX')],Math[_0x2cd2('e71','#4SP')](_0x402be5[_0x2cd2('e72','*unX')](_0x402be5[_0x2cd2('e73','vnIt')](JHMW[_0x2cd2('de','vbKI')][_0x2cd2('e74','xvh(')][_0x2cd2('e75','3psD')],0x1),_0x402be5[_0x2cd2('e76','HHLp')](this[_0x2cd2('e77','[yfr')],0x3)))))++JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('e78','ZboD')];this[_0x2cd2('ad8','P0Kn')][_0x2cd2('e79','*unX')]();};Scene_Ranking[_0x2cd2('741','8U%R')][_0x2cd2('e7a','lDFd')]=function(){JHMW[_0x2cd2('0','l%aI')][_0x2cd2('e7b','jpvP')]=[];SceneManager[_0x2cd2('e7c','YCbV')]();};if(JHMW[_0x2cd2('410','W@80')][_0x2cd2('e7d','l^iV')]){var _0x2b9820=_0x2cd2('e7e',')%R)')[_0x2cd2('3d7','GDPD')]('|'),_0x55040a=0x0;while(!![]){switch(_0x2b9820[_0x55040a++]){case'0':JHMW[_0x2cd2('e7f','vbKI')]=Window_MenuCommand[_0x2cd2('85c','#4SP')][_0x2cd2('e80','@rQL')];continue;case'1':Window_MenuCommand[_0x2cd2('8b4','X1Zu')][_0x2cd2('e81','HHLp')]=function(){var _0x334982={'epPmT':_0x2cd2('e82','xWmq')};var _0x54504a=!![];this[_0x2cd2('e83','R)LB')](JHMW[_0x2cd2('e9','jpvP')][_0x2cd2('e84','4[4f')],_0x334982[_0x2cd2('e85','%E7b')],_0x54504a);};continue;case'2':Scene_Menu[_0x2cd2('904','y046')][_0x2cd2('e86','jpvP')]=function(){SceneManager[_0x2cd2('e87','l^iV')](Scene_Ranking);};continue;case'3':JHMW[_0x2cd2('e88','jpvP')]=Scene_Menu[_0x2cd2('84a','vB83')][_0x2cd2('e89','[yfr')];continue;case'4':Scene_Menu[_0x2cd2('a80','P0Kn')][_0x2cd2('e8a','@rQL')]=function(){var _0x31452e={'CuJOe':_0x2cd2('e8b','3psD')};JHMW[_0x2cd2('e8c','&)j!')][_0x2cd2('d49','HHLp')](this);this[_0x2cd2('ae0','^)]V')][_0x2cd2('e8d','W@80')](_0x31452e[_0x2cd2('e8e','2^*B')],this[_0x2cd2('e8f','wHIg')][_0x2cd2('e90','^)]V')](this));};continue;case'5':Window_MenuCommand[_0x2cd2('c4b','vbKI')][_0x2cd2('e80','@rQL')]=function(){JHMW[_0x2cd2('e91','#4SP')][_0x2cd2('e92','8U%R')](this);this[_0x2cd2('e93','P0Kn')]();};continue;}break;}};Window_Base[_0x2cd2('739','[yfr')][_0x2cd2('e94','#fZb')]=function(_0x51d7f9,_0x361c20,_0x372f19,_0x46dc7c,_0xbaaf6a){var _0x33c506={'CtOGY':_0x2cd2('e95','8vsk'),'KZlYS':function(_0x361c20,_0x372f19){return _0x361c20+_0x372f19;},'HOUgO':function(_0x361c20,_0x372f19){return _0x361c20+_0x372f19;},'ACPTD':_0x2cd2('e96','ZboD'),'hITNq':_0x2cd2('e97','GDPD'),'isMFn':function(_0x5cb4ad,_0x5a991a){return _0x5cb4ad(_0x5a991a);},'cgaqr':function(_0x361c20,_0x372f19){return _0x361c20!==_0x372f19;},'izxgi':_0x2cd2('e98','YCbV'),'PMUBH':_0x2cd2('e99','vbKI'),'ILySQ':function(_0x361c20,_0x372f19){return _0x361c20==_0x372f19;}};var _0x40ffc8=_0x33c506[_0x2cd2('e9a','vnIt')](String,_0x51d7f9)[_0x2cd2('e9b','l^iV')](/\\c\[(\d+)\]/i);if(_0x40ffc8){if(_0x33c506[_0x2cd2('e9c','l%aI')](_0x33c506[_0x2cd2('e9d','hS2h')],_0x33c506[_0x2cd2('e9e','YCbV')])){this[_0x2cd2('e9f','hS2h')][_0x2cd2('ea0','ZboD')]=_0x33c506[_0x2cd2('ea1','*unX')](Utils[_0x2cd2('59d','l^iV')],'MV')?this[_0x2cd2('ea2',')%R)')](_0x33c506[_0x2cd2('ea3','1L8W')](Number,_0x40ffc8[0x1])):ColorManager[_0x2cd2('ea4','xWmq')](_0x33c506[_0x2cd2('ea5','g*Sm')](Number,_0x40ffc8[0x1]));_0x51d7f9=_0x51d7f9[_0x2cd2('ea6','Biy2')](/\\c\[(\d+)\]/gi,'');this[_0x2cd2('ea7','wHIg')][_0x2cd2('ea8','*unX')](_0x51d7f9,_0x361c20,_0x372f19,_0x46dc7c,this[_0x2cd2('ea9','ghrv')](),_0xbaaf6a);this[_0x2cd2('eaa','R)LB')]();}else{var _0x5ab765=_0x33c506[_0x2cd2('eab','l%aI')][_0x2cd2('1ea','gq!6')]('|'),_0x3b3037=0x0;while(!![]){switch(_0x5ab765[_0x3b3037++]){case'0':var _0x3d4cef=JHMW[_0x2cd2('c94','hS2h')][_0x2cd2('dd','%E7b')];continue;case'1':var _0x1f5778=_0x33c506[_0x2cd2('eac','vbKI')]('',Graphics[_0x2cd2('ead','xvh(')][_0x2cd2('c15','#fZb')]);continue;case'2':var _0x4cb48b=JHMW[_0x2cd2('326','#4SP')](_0x33c506[_0x2cd2('eae','c!9y')](_0x33c506[_0x2cd2('eaf','R)LB')](_0x33c506[_0x2cd2('eb0','dO1!')](_0x33c506[_0x2cd2('eb1','R)LB')](_0x33c506[_0x2cd2('eb2','2^*B')],_0x3d4cef[_0x2cd2('338','y046')](0x5,0x4)),_0x264b1f),_0x1f5778),_0x265557));continue;case'3':var _0x15240d={'user':_0x264b1f,'word':_0x1f5778,'game':_0x3d4cef,'data':_0x265557,'cmd':_0x33c506[_0x2cd2('eb3','g*Sm')],'sign':_0x4cb48b};continue;case'4':JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('eb4','%E7b')](_0x1afda4,_0x15240d,![],JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('eb5','#fZb')],_0x265557);continue;case'5':var _0x264b1f=_0x33c506[_0x2cd2('eb6','g*Sm')]('',Graphics[_0x2cd2('bbc','c!9y')][_0x2cd2('eb7','^)]V')]);continue;case'6':var _0x1afda4=JHMW[_0x2cd2('374',')%R)')][_0x2cd2('eb8','CFh^')](JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('eb9','vbKI')]);continue;case'7':var _0x265557=JHMW[_0x2cd2('1fc','P0Kn')](JSON[_0x2cd2('eba','W@80')](PluginManager[_0x2cd2('ebb','xvh(')]));continue;}break;}}}else this[_0x2cd2('ebc','R)LB')][_0x2cd2('ebd','4UjT')](_0x51d7f9,_0x361c20,_0x372f19,_0x46dc7c,this[_0x2cd2('ebe','UUq0')](),_0xbaaf6a);};Window_LoginCommand[_0x2cd2('904','y046')]=Object[_0x2cd2('ebf','&)j!')](Window_Command[_0x2cd2('6f9','l^iV')]);Window_LoginCommand[_0x2cd2('8ec','2^*B')][_0x2cd2('ec0','obzi')]=Window_LoginCommand;Window_LoginCommand[_0x2cd2('dde','R)LB')][_0x2cd2('ec1','HHLp')]=function(_0x3a9a15){var _0x32ed6b={'PkVtI':_0x2cd2('ec2','obzi'),'oEmCs':function(_0x586442,_0x1a2761){return _0x586442==_0x1a2761;}};var _0x2c5370=_0x32ed6b[_0x2cd2('ec3','qqEP')][_0x2cd2('b3b','NeH6')]('|'),_0x2fae2f=0x0;while(!![]){switch(_0x2c5370[_0x2fae2f++]){case'0':this[_0x2cd2('ec4','vbKI')]=_0x3a9a15;continue;case'1':this[_0x2cd2('ec5','YCbV')](0x0);continue;case'2':var _0x50e05e=0x258;continue;case'3':this[_0x2cd2('ec6','UUq0')]();continue;case'4':this[_0x2cd2('ec7','HHLp')]();continue;case'5':this[_0x2cd2('ec8','^)]V')]();continue;case'6':this[_0x2cd2('ec9','vnIt')]();continue;case'7':this[_0x2cd2('eca','xvh(')]();continue;case'8':var _0x34d385=this[_0x2cd2('ecb','obzi')](0x1);continue;case'9':if(_0x32ed6b[_0x2cd2('ecc','l%aI')](Utils[_0x2cd2('ecd','*unX')],'MV'))Window_Command[_0x2cd2('b2e','NeH6')][_0x2cd2('ece','4[4f')][_0x2cd2('ecf','YCbV')](this,0x0,0x0,_0x50e05e,_0x34d385);else{var _0x3a316e={'x':0x0,'y':0x0,'width':_0x50e05e,'height':_0x34d385};Window_Command[_0x2cd2('757','8vsk')][_0x2cd2('d4e','c!9y')][_0x2cd2('900','xWmq')](this,_0x3a316e);}continue;}break;}};Window_LoginCommand[_0x2cd2('7b7','CFh^')][_0x2cd2('ed0','g*Sm')]=function(){var _0x4b2d3e={'rbAoL':function(_0x56ddb5,_0x28fa1a){return _0x56ddb5<=_0x28fa1a;},'ZtSsJ':function(_0x26eeba,_0x42ded1){return _0x26eeba(_0x42ded1);},'RJELE':function(_0xf4d2a8,_0x17adec){return _0xf4d2a8!=_0x17adec;},'OwjAA':function(_0x2c6a9e,_0x395cb8){return _0x2c6a9e(_0x395cb8);}};if(!JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('c82','W@80')])return 0x3;else if(_0x4b2d3e[_0x2cd2('ed1','3psD')](_0x4b2d3e[_0x2cd2('ed2','l%aI')](parseInt,JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('ed3','lDFd')][0x0]),0x5)&&_0x4b2d3e[_0x2cd2('ed4','W@80')](_0x4b2d3e[_0x2cd2('ed5','W@80')](parseInt,JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('d24','P0Kn')][0x0]),0x0))return 0x3;else return 0x2;};Window_LoginCommand[_0x2cd2('c4b','vbKI')][_0x2cd2('ed6','@rQL')]=function(){var _0x328d02={'twCej':_0x2cd2('ed7','&)j!')};return _0x328d02[_0x2cd2('ed8','vB83')];};Window_LoginCommand[_0x2cd2('85c','#4SP')][_0x2cd2('ed9','3Y7y')]=function(){var _0x3f4c13={'GOWQm':function(_0x42432e,_0x25e7a2){return _0x42432e+_0x25e7a2;}};this[_0x2cd2('eda','ZboD')]=this[_0x2cd2('edb','2^*B')][_0x2cd2('98f','l^iV')];this['x']=this[_0x2cd2('edc','3psD')]['x'];this['y']=_0x3f4c13[_0x2cd2('edd','1L8W')](_0x3f4c13[_0x2cd2('ede','3Y7y')](this[_0x2cd2('edf','vnIt')]['y'],this[_0x2cd2('ee0','4UjT')][_0x2cd2('ee1','lDFd')]),0xa);};Window_LoginCommand[_0x2cd2('7c9','xWmq')][_0x2cd2('ee2','GDPD')]=function(){var _0x423a26={'GJqQO':function(_0x71eb1d,_0x3281ac){return _0x71eb1d-_0x3281ac;},'JVuSS':function(_0x59128c,_0x201e24){return _0x59128c/_0x201e24;},'dzyLe':_0x2cd2('ee3','NeH6'),'rvQFw':_0x2cd2('ee4','4[4f'),'DHeEv':_0x2cd2('ee5','NeH6'),'XFXVj':_0x2cd2('ee6','l%aI'),'QeURl':_0x2cd2('ee7','GDPD'),'MgMTz':_0x2cd2('ee8','#fZb'),'daWzt':function(_0x561a62,_0x50e04a){return _0x561a62==_0x50e04a;},'ZecxR':function(_0x2822c6,_0x160fac){return _0x2822c6(_0x160fac);},'drhaq':function(_0x113dbf,_0x167cde){return _0x113dbf===_0x167cde;},'HsVji':_0x2cd2('ee9','2^*B'),'vwMeD':_0x2cd2('eea','vB83'),'Lsijm':_0x2cd2('eeb','^)]V'),'JsDmT':function(_0x209d5c,_0x263810){return _0x209d5c<=_0x263810;},'DmMax':function(_0x233868,_0x4156df){return _0x233868(_0x4156df);},'xvMVW':function(_0x92706d,_0x8c182d){return _0x92706d!=_0x8c182d;},'DAyWb':_0x2cd2('eec','%E7b'),'ArPPO':_0x2cd2('eed','[yfr'),'eKfDc':_0x2cd2('eee','8vsk'),'ShGcM':_0x2cd2('eef','3psD')};if(!JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('cb5','#fZb')]){this[_0x2cd2('ef0','gq!6')](_0x423a26[_0x2cd2('ef1','%E7b')],_0x423a26[_0x2cd2('ef2','W@80')]);this[_0x2cd2('ef3','P0Kn')](_0x423a26[_0x2cd2('ef4','@rQL')],_0x423a26[_0x2cd2('ef5','*unX')]);this[_0x2cd2('e83','R)LB')](_0x423a26[_0x2cd2('ef6','#iIl')],_0x423a26[_0x2cd2('ef7','gq!6')]);}else if(_0x423a26[_0x2cd2('ef8','Biy2')](_0x423a26[_0x2cd2('ef9','YCbV')](parseInt,JHMW[_0x2cd2('380','CFh^')][_0x2cd2('ed3','lDFd')][0x0]),0x0)){if(_0x423a26[_0x2cd2('efa','qqEP')](_0x423a26[_0x2cd2('efb',')%R)')],_0x423a26[_0x2cd2('efc','jpvP')])){this[_0x2cd2('efd','ZboD')](_0x423a26[_0x2cd2('efe','1L8W')],_0x423a26[_0x2cd2('eff','qqEP')]);this[_0x2cd2('ef0','gq!6')](_0x423a26[_0x2cd2('f00','HHLp')],_0x423a26[_0x2cd2('f01','jpvP')]);}else{var _0x1cf923=_0x423a26[_0x2cd2('f02','*unX')](_0x423a26[_0x2cd2('f03','#4SP')](Graphics[_0x2cd2('f04','X1Zu')],0x2),0x14);var _0x15069b=0x0;}}else if(_0x423a26[_0x2cd2('f05','4[4f')](_0x423a26[_0x2cd2('f06','HHLp')](parseInt,JHMW[_0x2cd2('a7','c!9y')][_0x2cd2('f07','^)]V')][0x0]),0x5)){this[_0x2cd2('f08','3Y7y')](_0x423a26[_0x2cd2('f09','@rQL')],_0x423a26[_0x2cd2('f0a','CFh^')]);this[_0x2cd2('cbc','8U%R')](_0x423a26[_0x2cd2('f0b','jpvP')],_0x423a26[_0x2cd2('f0c','P0Kn')]);this[_0x2cd2('f0d','1L8W')](_0x423a26[_0x2cd2('f0e','#fZb')],_0x423a26[_0x2cd2('f0f','UUq0')]);}else if(_0x423a26[_0x2cd2('f10','8vsk')](_0x423a26[_0x2cd2('f11','l^iV')](parseInt,JHMW[_0x2cd2('10a','1L8W')][_0x2cd2('cb3','vB83')][0x1]),JHMW[_0x2cd2('321','lDFd')][_0x2cd2('f12','#iIl')])){this[_0x2cd2('f13','c!9y')](_0x423a26[_0x2cd2('f14','CFh^')],_0x423a26[_0x2cd2('f15','4UjT')]);this[_0x2cd2('f16','*unX')](_0x423a26[_0x2cd2('ef6','#iIl')],_0x423a26[_0x2cd2('f17','vB83')]);}else{this[_0x2cd2('f18','YCbV')](_0x423a26[_0x2cd2('f19','obzi')],_0x423a26[_0x2cd2('f1a','c!9y')]);this[_0x2cd2('965','wHIg')](_0x423a26[_0x2cd2('f1b','^)]V')],_0x423a26[_0x2cd2('f1c','X1Zu')]);}};Window_LoginInfo[_0x2cd2('7b0',')%R)')]=Object[_0x2cd2('f1d','CFh^')](Window_Base[_0x2cd2('77f','obzi')]);Window_LoginInfo[_0x2cd2('7b7','CFh^')][_0x2cd2('f1e','P0Kn')]=Window_LoginInfo;Window_LoginInfo[_0x2cd2('a1d','W@80')][_0x2cd2('820','vB83')]=function(_0x3f19cb){var _0x101638={'vFFmG':_0x2cd2('f1f','3Y7y'),'NYcKg':function(_0x7420b8,_0x216a43){return _0x7420b8==_0x216a43;},'IuAlx':function(_0xa7cc7b,_0x13c315){return _0xa7cc7b/_0x13c315;},'YknyC':function(_0x30d6fb,_0x3dac81){return _0x30d6fb-_0x3dac81;},'RXqIC':function(_0x3dacd3,_0x4b41be){return _0x3dacd3-_0x4b41be;}};var _0x247892=_0x101638[_0x2cd2('f20','YCbV')][_0x2cd2('943','3psD')]('|'),_0xedf63=0x0;while(!![]){switch(_0x247892[_0xedf63++]){case'0':if(!!_0x3f19cb)this[_0x2cd2('f21','vnIt')]=_0x3f19cb;else this[_0x2cd2('f22','X1Zu')]=null;continue;case'1':this[_0x2cd2('f23','dO1!')]();continue;case'2':this[_0x2cd2('f24','8vsk')]();continue;case'3':var _0x3edf6d=0x258;continue;case'4':var _0x11feb9=JHMW[_0x2cd2('d7','@rQL')][_0x2cd2('f25','c!9y')];continue;case'5':var _0x299d0b=0x12c;continue;case'6':if(_0x101638[_0x2cd2('f26','X1Zu')](Utils[_0x2cd2('f27','&)j!')],'MV')){var _0x3b9372=_0x101638[_0x2cd2('f28','vbKI')](_0x101638[_0x2cd2('f29','X1Zu')](Graphics[_0x2cd2('f2a','2^*B')],_0x3edf6d),0x2);Window_Base[_0x2cd2('7c1','*unX')][_0x2cd2('f2b',')%R)')][_0x2cd2('dca','R)LB')](this,_0x3b9372,_0x11feb9,_0x3edf6d,_0x299d0b);}else{var _0x3b9372=_0x101638[_0x2cd2('f2c','3psD')](_0x101638[_0x2cd2('f2d','#4SP')]($dataSystem[_0x2cd2('e18','ghrv')][_0x2cd2('f2e','gq!6')],_0x3edf6d),0x2);var _0x463f7d={'x':_0x3b9372,'y':_0x11feb9,'width':_0x3edf6d,'height':_0x299d0b};Window_Base[_0x2cd2('9b8','ghrv')][_0x2cd2('85d','dO1!')][_0x2cd2('f2f','GDPD')](this,_0x463f7d);}continue;}break;}};Window_LoginInfo[_0x2cd2('6f9','l^iV')][_0x2cd2('f30','y046')]=function(){this[_0x2cd2('f31','ZboD')][_0x2cd2('f32','obzi')]();this[_0x2cd2('f33','LX!4')]();};Window_LoginInfo[_0x2cd2('7c1','*unX')][_0x2cd2('f34','jpvP')]=function(){this[_0x2cd2('f35','xWmq')]();};Window_LoginInfo[_0x2cd2('f36','@rQL')][_0x2cd2('f37','GDPD')]=function(_0xe2c5a6){var _0x3cff38={'iSqNi':_0x2cd2('f38','LX!4'),'diocL':function(_0x17b623,_0xe2c5a6){return _0x17b623==_0xe2c5a6;},'pRWDG':function(_0x2eff81,_0xe2c5a6){return _0x2eff81-_0xe2c5a6;},'UhbjH':function(_0x50a5e7,_0xe2c5a6){return _0x50a5e7+_0xe2c5a6;},'riVuc':function(_0x9bd0ad,_0xe2c5a6){return _0x9bd0ad/_0xe2c5a6;}};var _0x341e57=_0x3cff38[_0x2cd2('f39','^)]V')][_0x2cd2('11','[yfr')]('|'),_0x31ad83=0x0;while(!![]){switch(_0x341e57[_0x31ad83++]){case'0':if(_0x3cff38[_0x2cd2('f3a','*unX')](Utils[_0x2cd2('824','%E7b')],'MV'))var _0x1714eb=this[_0x2cd2('f3b','g*Sm')]();else var _0x1714eb=ColorManager[_0x2cd2('f3b','g*Sm')]();continue;case'1':this[_0x2cd2('f3c','dO1!')][_0x2cd2('f3d','#4SP')]=0xff;continue;case'2':this[_0x2cd2('f3e','xvh(')][_0x2cd2('f3f','UUq0')](0x0,_0x129b34,this[_0x2cd2('f40','HHLp')](),0x2,_0x1714eb);continue;case'3':this[_0x2cd2('f41','c!9y')][_0x2cd2('f42','8vsk')]=0x64;continue;case'4':var _0x129b34=_0x3cff38[_0x2cd2('f43','%E7b')](_0x3cff38[_0x2cd2('f44','ZboD')](_0xe2c5a6,_0x3cff38[_0x2cd2('f45','X1Zu')](this[_0x2cd2('f46','LX!4')](),0x2)),0x1);continue;}break;}};Window_LoginInfo[_0x2cd2('f36','@rQL')][_0x2cd2('f47','2^*B')]=function(){return 0x18;};Window_LoginInfo[_0x2cd2('7c9','xWmq')][_0x2cd2('f48','W@80')]=function(){return 0x1e;};Window_LoginInfo[_0x2cd2('77f','obzi')][_0x2cd2('f49','HHLp')]=function(){var _0x317e3b={'AzZDj':function(_0x155fc5,_0x458882){return _0x155fc5<_0x458882;},'STeSF':function(_0x1e6acc,_0x3a52ab){return _0x1e6acc/_0x3a52ab;},'Bhoga':function(_0x3ea56e,_0x540ab3){return _0x3ea56e-_0x540ab3;},'htlJj':function(_0x13cb23,_0x207863){return _0x13cb23*_0x207863;},'JpqxA':function(_0x228d7d,_0x26990c){return _0x228d7d>=_0x26990c;},'hWFuN':function(_0x57180b,_0x3de539){return _0x57180b>=_0x3de539;},'YXjZQ':function(_0x11f6a1,_0x300e98){return _0x11f6a1-_0x300e98;},'AuAne':function(_0x5c08d5,_0x123d19){return _0x5c08d5<=_0x123d19;},'hvMqN':function(_0x21cc2e,_0x4c644f){return _0x21cc2e+_0x4c644f;},'DKdpg':function(_0x95f8a1,_0xec856d){return _0x95f8a1<=_0xec856d;},'djNPJ':function(_0x2f9551,_0x59c5ca){return _0x2f9551+_0x59c5ca;},'LYuEF':function(_0x11188e,_0x44d9e0){return _0x11188e+_0x44d9e0;},'aSoKY':_0x2cd2('f4a','xWmq'),'UofYq':_0x2cd2('f4b','#fZb'),'KyshI':_0x2cd2('f4c','8U%R'),'kZhtG':function(_0x9f0dd0,_0x51cdd2){return _0x9f0dd0+_0x51cdd2;},'YxQMs':_0x2cd2('f4d','8vsk'),'aLtfR':function(_0x26da8b,_0x54d2b1){return _0x26da8b!==_0x54d2b1;},'TJsqs':_0x2cd2('f4e','NeH6'),'TpCLR':_0x2cd2('f4f','^)]V'),'oBoib':_0x2cd2('f50','R)LB'),'xoykp':_0x2cd2('f51','ZboD'),'hemem':_0x2cd2('f52','UUq0'),'riGBr':_0x2cd2('f53','wHIg'),'iJYPD':function(_0xf8cb7e,_0x4826aa){return _0xf8cb7e(_0x4826aa);},'PjhaF':function(_0x517329,_0x2983f0){return _0x517329===_0x2983f0;},'UUAsN':_0x2cd2('f54','#iIl'),'bdIEV':_0x2cd2('f55','CFh^'),'QEtvL':_0x2cd2('f56','gq!6'),'SOroo':_0x2cd2('f57','%E7b'),'ypGDu':function(_0x4356f5,_0x59a502){return _0x4356f5==_0x59a502;},'Qqwha':function(_0x2013cc,_0x5c3904){return _0x2013cc===_0x5c3904;},'aRxGZ':_0x2cd2('f58','wHIg'),'jfSHD':_0x2cd2('f59','#iIl'),'VQQoD':_0x2cd2('f5a',')%R)'),'Qzsrx':_0x2cd2('f5b','Biy2'),'tzNIG':_0x2cd2('f5c','y046'),'spZnr':_0x2cd2('f5d','R)LB'),'ULClP':function(_0x53447c,_0x3bd52a){return _0x53447c!=_0x3bd52a;},'PJIKh':function(_0x4223c9,_0x1e2124){return _0x4223c9(_0x1e2124);},'mcuBY':_0x2cd2('f5e','HHLp'),'SOnOX':_0x2cd2('f5f','wHIg'),'bqbTl':_0x2cd2('f60','HHLp'),'bDnTA':_0x2cd2('f61','X1Zu'),'Nzqwt':_0x2cd2('f62','3Y7y'),'Scysk':_0x2cd2('f63','#iIl')};this[_0x2cd2('f64','vB83')](this[_0x2cd2('f65','UUq0')]());if(JHMW[_0x2cd2('13f','*unX')][_0x2cd2('c6f','LX!4')])this[_0x2cd2('f66','xWmq')](_0x317e3b[_0x2cd2('f67','y046')](_0x317e3b[_0x2cd2('f68','#4SP')](_0x317e3b[_0x2cd2('f69','4UjT')],JHMW[_0x2cd2('380','CFh^')][_0x2cd2('340','1L8W')]),_0x317e3b[_0x2cd2('f6a','#iIl')]),0x0,0x5,this[_0x2cd2('f6b','8U%R')](),_0x317e3b[_0x2cd2('f6c','3psD')]);else this[_0x2cd2('f6d','vbKI')](_0x317e3b[_0x2cd2('f6e','xWmq')](_0x317e3b[_0x2cd2('f6f','^)]V')],JHMW[_0x2cd2('148','xvh(')][_0x2cd2('f70','@rQL')]),0x0,0x5,this[_0x2cd2('f71','l%aI')](),_0x317e3b[_0x2cd2('f72','vnIt')]);var _0x6d3bda=Math[_0x2cd2('f73','l%aI')](0x20,this[_0x2cd2('f74','gq!6')]());this[_0x2cd2('f75','&)j!')](_0x6d3bda);this[_0x2cd2('f76','jpvP')]();if(!JHMW[_0x2cd2('13f','*unX')][_0x2cd2('d24','P0Kn')]){if(_0x317e3b[_0x2cd2('f77','3psD')](_0x317e3b[_0x2cd2('f78','xWmq')],_0x317e3b[_0x2cd2('f79','&)j!')])){if(_0x317e3b[_0x2cd2('f7a','vnIt')](JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('f7b','c!9y')],Math[_0x2cd2('f7c','gq!6')](_0x317e3b[_0x2cd2('f7d','vnIt')](_0x317e3b[_0x2cd2('f7e','#iIl')](JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('f7f','4UjT')][_0x2cd2('baa','&)j!')],0x1),_0x317e3b[_0x2cd2('f80','jpvP')](this[_0x2cd2('f81','#4SP')],0x3)))))++JHMW[_0x2cd2('165','NeH6')][_0x2cd2('f82','y046')];this[_0x2cd2('ae9','c!9y')][_0x2cd2('e59','vnIt')]();}else{this[_0x2cd2('f83',')%R)')](_0x317e3b[_0x2cd2('f84','gq!6')],0x50,0x41,this[_0x2cd2('f85','g*Sm')](),_0x317e3b[_0x2cd2('f86','GDPD')]);this[_0x2cd2('f87','LX!4')](_0x317e3b[_0x2cd2('f88','8U%R')],0x50,0x6e,this[_0x2cd2('f89','UUq0')](),_0x317e3b[_0x2cd2('f8a','@rQL')]);if(this[_0x2cd2('f8b','8U%R')]){this[_0x2cd2('f8c','^)]V')](this[_0x2cd2('f8d','Biy2')],0x0,0xb4,this[_0x2cd2('f8e','vnIt')](),_0x317e3b[_0x2cd2('f8f','hS2h')]);this[_0x2cd2('f90','wHIg')]=null;}else{this[_0x2cd2('f91','UUq0')](_0x317e3b[_0x2cd2('f92','R)LB')],0x0,0xa0,this[_0x2cd2('f93','LX!4')](),_0x317e3b[_0x2cd2('f94','8U%R')]);this[_0x2cd2('f95','4[4f')](_0x317e3b[_0x2cd2('f96','R)LB')],0x0,0xc8,this[_0x2cd2('f97','y046')](),_0x317e3b[_0x2cd2('f98','HHLp')]);}}}else if(_0x317e3b[_0x2cd2('f99','&)j!')](_0x317e3b[_0x2cd2('f9a','UUq0')](parseInt,JHMW[_0x2cd2('16b','2^*B')][_0x2cd2('b29','4UjT')][0x0]),0x0)){if(_0x317e3b[_0x2cd2('f9b','YCbV')](_0x317e3b[_0x2cd2('f9c','Biy2')],_0x317e3b[_0x2cd2('f9d','GDPD')])){var _0x422d26=this[_0x2cd2('f9e','LX!4')](0x1);var _0x2707de={'x':x,'y':y,'width':0x320,'height':_0x422d26};Window_HorzCommand[_0x2cd2('7b7','CFh^')][_0x2cd2('787','*unX')][_0x2cd2('f9f','1L8W')](this,_0x2707de);}else{this[_0x2cd2('fa0','%E7b')](_0x317e3b[_0x2cd2('fa1','Biy2')],0x50,0x41,this[_0x2cd2('fa2','P0Kn')](),_0x317e3b[_0x2cd2('fa3','ghrv')]);this[_0x2cd2('fa4','Biy2')](_0x317e3b[_0x2cd2('fa5','X1Zu')],0x50,0x6e,this[_0x2cd2('fa6','#4SP')](),_0x317e3b[_0x2cd2('fa7','3psD')]);this[_0x2cd2('a15','qqEP')](_0x317e3b[_0x2cd2('fa8','@rQL')],0x0,0xa0,this[_0x2cd2('fa9','NeH6')](),_0x317e3b[_0x2cd2('faa','8vsk')]);this[_0x2cd2('ea8','*unX')](_0x317e3b[_0x2cd2('fab','vbKI')],0x0,0xc8,this[_0x2cd2('fac','^)]V')](),_0x317e3b[_0x2cd2('fad','vbKI')]);}}else if(_0x317e3b[_0x2cd2('fae','@rQL')](_0x317e3b[_0x2cd2('faf','y046')](parseInt,JHMW[_0x2cd2('a7','c!9y')][_0x2cd2('b29','4UjT')][0x0]),0x0)){if(_0x317e3b[_0x2cd2('fb0','y046')](_0x317e3b[_0x2cd2('fb1','c!9y')],_0x317e3b[_0x2cd2('fb2','xWmq')])){if(!$gameMessage[_0x2cd2('fb3','qqEP')]()&&JHMW[_0x2cd2('fb4','4UjT')][_0x2cd2('fb5','1L8W')]&&TouchInput[_0x2cd2('fb6','jpvP')]()){var _0x207b4a=TouchInput['x'];var _0x60368e=TouchInput['y'];var _0x281f28=Graphics[_0x2cd2('fb7','vnIt')];if(_0x317e3b[_0x2cd2('fb8',')%R)')](_0x207b4a,_0x281f28[_0x2cd2('fb9','lDFd')]['x'])&&_0x317e3b[_0x2cd2('fba','4[4f')](_0x60368e,_0x317e3b[_0x2cd2('fbb','LX!4')](_0x281f28[_0x2cd2('4d6','1L8W')]['y'],0xa))&&_0x317e3b[_0x2cd2('fbc','l%aI')](_0x207b4a,_0x317e3b[_0x2cd2('fbd','^)]V')](_0x281f28[_0x2cd2('977','y046')]['x'],_0x281f28[_0x2cd2('60e','c!9y')][_0x2cd2('fbe','^)]V')]))&&_0x317e3b[_0x2cd2('fbf','xWmq')](_0x60368e,_0x317e3b[_0x2cd2('fc0','W@80')](_0x317e3b[_0x2cd2('fc1','%E7b')](_0x281f28[_0x2cd2('4d6','1L8W')]['y'],_0x281f28[_0x2cd2('977','y046')][_0x2cd2('c3b','2^*B')]),0xa)))_0x281f28[_0x2cd2('fc2','obzi')]();else _0x281f28[_0x2cd2('fc3','jpvP')]();}}else{this[_0x2cd2('f87','LX!4')](_0x317e3b[_0x2cd2('fc4','ZboD')],0x0,0x64,this[_0x2cd2('fc5','vB83')](),_0x317e3b[_0x2cd2('fc6','4UjT')]);this[_0x2cd2('fc7','NeH6')](_0x317e3b[_0x2cd2('fc8','*unX')],0x0,0x96,this[_0x2cd2('f93','LX!4')](),_0x317e3b[_0x2cd2('fc9','l%aI')]);}}else if(_0x317e3b[_0x2cd2('fca','HHLp')](_0x317e3b[_0x2cd2('f9a','UUq0')](parseInt,JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('f07','^)]V')][0x0]),0x5)){this[_0x2cd2('a15','qqEP')](_0x317e3b[_0x2cd2('fcb','1L8W')],0x50,0x41,this[_0x2cd2('fcc','8vsk')](),_0x317e3b[_0x2cd2('fcd','4[4f')]);this[_0x2cd2('fce','lDFd')](_0x317e3b[_0x2cd2('fcf','HHLp')],0x50,0x6e,this[_0x2cd2('fd0','ZboD')](),_0x317e3b[_0x2cd2('fd1','xvh(')]);this[_0x2cd2('fd2','#4SP')](_0x317e3b[_0x2cd2('fd3','vbKI')],0x0,0xa0,this[_0x2cd2('f71','l%aI')](),_0x317e3b[_0x2cd2('fd4','UUq0')]);this[_0x2cd2('fd5','8U%R')](_0x317e3b[_0x2cd2('fd6',')%R)')],0x0,0xc8,this[_0x2cd2('fd7','3psD')](),_0x317e3b[_0x2cd2('fd8','1L8W')]);}else if(_0x317e3b[_0x2cd2('fd9','&)j!')](_0x317e3b[_0x2cd2('fda','wHIg')](parseInt,JHMW[_0x2cd2('120','l^iV')][_0x2cd2('fdb','gq!6')][0x1]),JHMW[_0x2cd2('97','obzi')][_0x2cd2('fdc','4UjT')]))this[_0x2cd2('fce','lDFd')](_0x317e3b[_0x2cd2('fdd','obzi')],0x0,0x7d,this[_0x2cd2('fde','l^iV')](),_0x317e3b[_0x2cd2('fdf','#4SP')]);else if(_0x317e3b[_0x2cd2('fe0','3Y7y')](JHMW[_0x2cd2('19b','obzi')][_0x2cd2('53','%E7b')][0x4],0x0))this[_0x2cd2('ea8','*unX')](_0x317e3b[_0x2cd2('fe1','CFh^')],0x0,0x7d,this[_0x2cd2('f6b','8U%R')](),_0x317e3b[_0x2cd2('faa','8vsk')]);else{this[_0x2cd2('fe2','8vsk')](_0x317e3b[_0x2cd2('fe3','hS2h')],0x0,0x7d,this[_0x2cd2('fe4','R)LB')](),_0x317e3b[_0x2cd2('fe5','%E7b')]);var _0x30db0a=new Date(_0x317e3b[_0x2cd2('fe6','3Y7y')](JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('164','#4SP')][0x4],0x3e8));var _0x5069df=_0x30db0a[_0x2cd2('fe7','^)]V')]();this[_0x2cd2('fe8','YCbV')](_0x5069df,0x0,0xa5,this[_0x2cd2('fcc','8vsk')](),_0x317e3b[_0x2cd2('fe9','#fZb')]);}this[_0x2cd2('fea','c!9y')](_0x317e3b[_0x2cd2('feb','xWmq')]);this[_0x2cd2('f87','LX!4')](_0x317e3b[_0x2cd2('fec','wHIg')],0x0,0xeb,this[_0x2cd2('fed','hS2h')](),_0x317e3b[_0x2cd2('fee','vbKI')]);};Window_Ranking[_0x2cd2('786','gq!6')]=Object[_0x2cd2('fef','y046')](Window_Base[_0x2cd2('77c','ZboD')]);Window_Ranking[_0x2cd2('73f','g*Sm')][_0x2cd2('ff0','#4SP')]=Window_Ranking;Window_Ranking[_0x2cd2('904','y046')][_0x2cd2('d4e','c!9y')]=function(_0x4f9fe2,_0x1a6418,_0x46d76e,_0x5ad3ae){var _0x5adcae={'NAfZK':function(_0x35cbf3,_0x24b8b8){return _0x35cbf3(_0x24b8b8);},'dItAa':function(_0x4f9fe2,_0x1a6418){return _0x4f9fe2==_0x1a6418;},'SoEnP':function(_0x4f9fe2,_0x1a6418){return _0x4f9fe2!==_0x1a6418;},'Kdwvp':_0x2cd2('ff1','xWmq')};if(_0x5adcae[_0x2cd2('ff2','GDPD')](Utils[_0x2cd2('bd0','vB83')],'MZ')){if(_0x5adcae[_0x2cd2('ff3','wHIg')](_0x5adcae[_0x2cd2('ff4','dO1!')],_0x5adcae[_0x2cd2('ff5','X1Zu')])){var _0x267c29=_0x5adcae[_0x2cd2('ff6','UUq0')](Number,args[_0x2cd2('ff7','UUq0')]);JHMW[_0x2cd2('165','NeH6')][_0x2cd2('ff8','@rQL')](_0x267c29);}else{var _0x524bea={'x':_0x4f9fe2,'y':_0x1a6418,'width':_0x46d76e,'height':_0x5ad3ae};Window_Base[_0x2cd2('8b4','X1Zu')][_0x2cd2('77a','^)]V')][_0x2cd2('ff9','*unX')](this,_0x524bea);}}else Window_Base[_0x2cd2('a1d','W@80')][_0x2cd2('ffa','wHIg')][_0x2cd2('dd0','g*Sm')](this,_0x4f9fe2,_0x1a6418,_0x46d76e,_0x5ad3ae);this[_0x2cd2('ffb','c!9y')]=null;this[_0x2cd2('ffc','LX!4')]();this[_0x2cd2('ffd','hS2h')]();};Window_Ranking[_0x2cd2('ffe','LX!4')][_0x2cd2('fff','W@80')]=function(){this[_0x2cd2('646','^)]V')][_0x2cd2('1000',')%R)')]();this[_0x2cd2('1001','hS2h')]();};Window_Ranking[_0x2cd2('6f7','qqEP')][_0x2cd2('1002','g*Sm')]=function(_0x3ee0de){var _0x9de8de={'ugGDM':_0x2cd2('1003','lDFd'),'jBQMX':function(_0x28fba6,_0x3ee0de){return _0x28fba6-_0x3ee0de;},'trLWw':function(_0x4a9151,_0x3ee0de){return _0x4a9151+_0x3ee0de;},'TAlHG':function(_0x114a8c,_0x3ee0de){return _0x114a8c/_0x3ee0de;},'rItjU':function(_0x5efc67,_0x3ee0de){return _0x5efc67==_0x3ee0de;}};var _0x23c9e1=_0x9de8de[_0x2cd2('1004','g*Sm')][_0x2cd2('c0c','1L8W')]('|'),_0xe52464=0x0;while(!![]){switch(_0x23c9e1[_0xe52464++]){case'0':this[_0x2cd2('1005','*unX')][_0x2cd2('1006','Biy2')]=0x64;continue;case'1':var _0x41d60c=_0x9de8de[_0x2cd2('1007','gq!6')](_0x9de8de[_0x2cd2('1008','NeH6')](_0x3ee0de,_0x9de8de[_0x2cd2('1009','#iIl')](this[_0x2cd2('100a','HHLp')](),0x2)),0x1);continue;case'2':this[_0x2cd2('5e5','l^iV')][_0x2cd2('100b','2^*B')]=0xff;continue;case'3':if(_0x9de8de[_0x2cd2('100c','^)]V')](Utils[_0x2cd2('579','CFh^')],'MV'))var _0x2992b3=this[_0x2cd2('100d','*unX')]();else var _0x2992b3=ColorManager[_0x2cd2('100e','hS2h')]();continue;case'4':this[_0x2cd2('100f','Biy2')][_0x2cd2('1010','vnIt')](0x0,_0x41d60c,this[_0x2cd2('fed','hS2h')](),0x2,_0x2992b3);continue;}break;}};Window_Ranking[_0x2cd2('1011','l%aI')][_0x2cd2('1012','ZboD')]=function(){return 0x18;};Window_Ranking[_0x2cd2('757','8vsk')][_0x2cd2('1013','3psD')]=function(){return 0x1e;};Window_Ranking[_0x2cd2('6f7','qqEP')][_0x2cd2('1014','NeH6')]=function(){var _0x809d4={'xbMWF':_0x2cd2('1015','Biy2')};var _0x203876={'user':JHMW[_0x2cd2('23b','dO1!')][_0x2cd2('1016','obzi')],'game':JHMW[_0x2cd2('a3d','l^iV')][_0x2cd2('1017','P0Kn')],'cmd':_0x809d4[_0x2cd2('1018','R)LB')]};var _0x399fc2=JHMW[_0x2cd2('13b','hS2h')][_0x2cd2('1019','%E7b')](JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('101a','#4SP')]);JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('33a','4UjT')](_0x399fc2,_0x203876,![],this[_0x2cd2('101b','wHIg')]);};Window_Ranking[_0x2cd2('8a9','1L8W')][_0x2cd2('101c','vbKI')]=function(_0x5626d0){var _0x11b976={'OyZyL':_0x2cd2('101d','8vsk'),'qVfJt':function(_0x13ae80,_0x350efb){return _0x13ae80(_0x350efb);},'nFALb':function(_0x5345d7,_0x31a273){return _0x5345d7+_0x31a273;},'dcshG':function(_0x4090be,_0x207b71){return _0x4090be/_0x207b71;}};var _0x1a60dd=_0x11b976[_0x2cd2('101e','HHLp')][_0x2cd2('101f','g*Sm')]('|'),_0x53674c=0x0;while(!![]){switch(_0x1a60dd[_0x53674c++]){case'0':JHMW[_0x2cd2('247','YCbV')][_0x2cd2('1a5','%E7b')]=JHMW[_0x2cd2('a7','c!9y')][_0x2cd2('1020','y046')][_0x2cd2('1021','CFh^')](0x0,-0x1);continue;case'1':$gameVariables[_0x2cd2('1022','&)j!')](JHMW[_0x2cd2('123','vB83')][_0x2cd2('1023','c!9y')],_0x11b976[_0x2cd2('1024','1L8W')](parseInt,JHMW[_0x2cd2('61','HHLp')][_0x2cd2('1025','^)]V')][_0x11b976[_0x2cd2('1026','4UjT')](_0x1fc2de,0x2)]));continue;case'2':var _0x1fc2de=JHMW[_0x2cd2('247','YCbV')][_0x2cd2('1027','vnIt')][_0x2cd2('1028','dO1!')](JHMW[_0x2cd2('270','vB83')][_0x2cd2('1029','vnIt')]);continue;case'3':JHMW[_0x2cd2('d0','4[4f')][_0x2cd2('e7b','jpvP')]=_0x170220[_0x2cd2('b1f','P0Kn')](',');continue;case'4':var _0x170220=JHMW[_0x2cd2('0','l%aI')][_0x2cd2('223','CFh^')](JHMW[_0x2cd2('101','wHIg')][_0x2cd2('102a','gq!6')](_0x5626d0));continue;case'5':$gameVariables[_0x2cd2('102b','2^*B')](JHMW[_0x2cd2('d2','P0Kn')][_0x2cd2('102c','P0Kn')],Math[_0x2cd2('102d','3Y7y')](_0x11b976[_0x2cd2('102e','%E7b')](_0x11b976[_0x2cd2('102f','vB83')](_0x1fc2de,0x1),0x3)));continue;}break;}};Window_Ranking[_0x2cd2('8a9','1L8W')][_0x2cd2('f49','HHLp')]=function(){var _0x76a4cf={'HvDke':function(_0x5f50ad,_0x5886ab){return _0x5f50ad==_0x5886ab;},'yInkH':function(_0x178e89,_0x17ab95){return _0x178e89+_0x17ab95;},'Atqsw':function(_0x50cfca,_0x184497){return _0x50cfca*_0x184497;},'PjWsU':function(_0x4b8872,_0x84469e){return _0x4b8872+_0x84469e;},'ZIzwc':_0x2cd2('1030','xvh('),'BSGWE':function(_0x2dc851,_0x37cd47){return _0x2dc851/_0x37cd47;},'GPRro':function(_0x1eccf2,_0x26471d){return _0x1eccf2-_0x26471d;},'uvyhE':_0x2cd2('1031','3Y7y'),'TMaug':function(_0x33c5e8,_0x3b7188){return _0x33c5e8*_0x3b7188;},'hbAyv':function(_0x979e04,_0x292ff4){return _0x979e04<_0x292ff4;},'NLdJG':function(_0x596c67,_0xf6eba8){return _0x596c67!==_0xf6eba8;},'PaxgE':_0x2cd2('1032','dO1!'),'aiHho':_0x2cd2('1033','xWmq'),'NQWIj':_0x2cd2('1034','8vsk'),'mlmxy':function(_0x44105f,_0x275b7b){return _0x44105f==_0x275b7b;},'BAkgn':function(_0x523b75,_0x44ed16){return _0x523b75==_0x44ed16;},'dcmSi':function(_0x48fdd9,_0x190013){return _0x48fdd9+_0x190013;},'ZCucS':function(_0x2aed73,_0x107ac5){return _0x2aed73*_0x107ac5;},'OrAEo':function(_0x1a9432,_0x4ac66f){return _0x1a9432-_0x4ac66f;},'jhbaU':function(_0x23bf30,_0x5d33bc){return _0x23bf30*_0x5d33bc;},'ClhCs':_0x2cd2('1035','ghrv'),'YRatc':function(_0x597eef,_0x5dcfff){return _0x597eef==_0x5dcfff;},'AwPYi':function(_0x2a6c45,_0x17d142){return _0x2a6c45+_0x17d142;},'RlKTm':function(_0x36b8d2,_0x59a0a7){return _0x36b8d2*_0x59a0a7;},'xvSqP':function(_0x5939b8,_0x1e7660){return _0x5939b8*_0x1e7660;},'jqDFS':function(_0x163059,_0x5f5c13){return _0x163059-_0x5f5c13;},'JojGU':function(_0x4c6dee,_0x5619ab){return _0x4c6dee+_0x5619ab;},'oeBGV':function(_0x39ef41,_0x4061e7){return _0x39ef41*_0x4061e7;},'IeuGc':function(_0x4725c4,_0x3d7a7e){return _0x4725c4+_0x3d7a7e;},'anLhd':function(_0x2fcfad,_0xf40a20){return _0x2fcfad-_0xf40a20;},'gAOVu':function(_0x3eeacc,_0x4a1a09){return _0x3eeacc+_0x4a1a09;},'qaUwd':function(_0x147d29,_0x284339){return _0x147d29+_0x284339;},'UXeHo':function(_0x2adac2,_0x24b3d8){return _0x2adac2-_0x24b3d8;},'bIKqW':function(_0x35dd60,_0x3bec9c){return _0x35dd60*_0x3bec9c;},'UoTcc':function(_0x280f94,_0x39009f){return _0x280f94-_0x39009f;},'XHHZU':function(_0xa9375c,_0x4372e2){return _0xa9375c*_0x4372e2;},'UNHel':function(_0x596e4c,_0x1d529b){return _0x596e4c*_0x1d529b;},'JpQQk':_0x2cd2('1036','GDPD')};var _0x49514a=_0x76a4cf[_0x2cd2('1037','hS2h')](Utils[_0x2cd2('1038','l%aI')],'MV')?Graphics[_0x2cd2('1039','%E7b')]:$dataSystem[_0x2cd2('103a','X1Zu')][_0x2cd2('103b','NeH6')];var _0x4b0f21=Math[_0x2cd2('103c','HHLp')](_0x76a4cf[_0x2cd2('103d','#fZb')](_0x76a4cf[_0x2cd2('103e','P0Kn')](_0x49514a,0xa0),0x23));this[_0x2cd2('103f','HHLp')](this[_0x2cd2('1040','8U%R')]());this[_0x2cd2('1041','@rQL')]('排名',0x0,0x0,0x64,_0x76a4cf[_0x2cd2('1042','l%aI')]);this[_0x2cd2('1043','dO1!')]('玩家',0x82,0x0,0xc8,_0x76a4cf[_0x2cd2('1044','xvh(')]);this[_0x2cd2('1045','2^*B')]('称号',0x172,0x0,0xe6,_0x76a4cf[_0x2cd2('1046','*unX')]);this[_0x2cd2('1047','HHLp')](JHMW[_0x2cd2('dc','#4SP')][_0x2cd2('1048',')%R)')],0x28a,0x0,0x64,_0x76a4cf[_0x2cd2('1046','*unX')]);var _0x3c6ce8=Math[_0x2cd2('1049','xWmq')](0x19,this[_0x2cd2('f48','W@80')]());this[_0x2cd2('104a',')%R)')](_0x3c6ce8);this[_0x2cd2('104b','y046')]();for(var _0x2060bd=_0x76a4cf[_0x2cd2('104c','&)j!')](_0x76a4cf[_0x2cd2('104d','vB83')](JHMW[_0x2cd2('165','NeH6')][_0x2cd2('104e','X1Zu')],0x1),_0x4b0f21);_0x76a4cf[_0x2cd2('104f','4UjT')](_0x2060bd,_0x76a4cf[_0x2cd2('1050','g*Sm')](JHMW[_0x2cd2('fc','W@80')][_0x2cd2('1051','#fZb')],_0x4b0f21));++_0x2060bd){if(_0x76a4cf[_0x2cd2('1052',')%R)')](_0x76a4cf[_0x2cd2('1053','@rQL')],_0x76a4cf[_0x2cd2('1054','c!9y')])){if(_0x76a4cf[_0x2cd2('1055','y046')](JHMW[_0x2cd2('13b','hS2h')][_0x2cd2('1056','&)j!')][_0x76a4cf[_0x2cd2('1057','3Y7y')](_0x76a4cf[_0x2cd2('1058','YCbV')](_0x2060bd,0xb),0xb)],JHMW[_0x2cd2('380','CFh^')][_0x2cd2('1059','8vsk')])){var _0x19687f=_0x76a4cf[_0x2cd2('105a','gq!6')](_0x76a4cf[_0x2cd2('105b','ZboD')](JHMW[_0x2cd2('101','wHIg')][_0x2cd2('28f','ghrv')][_0x76a4cf[_0x2cd2('105c','gq!6')](_0x76a4cf[_0x2cd2('105d','1L8W')](_0x2060bd,0xb),0x1)],_0x76a4cf[_0x2cd2('105e','#fZb')]),JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('105f','ZboD')][_0x76a4cf[_0x2cd2('105b','ZboD')](_0x76a4cf[_0x2cd2('1060','jpvP')](_0x2060bd,0xb),0xa)]);Notification[_0x2cd2('1061','CFh^')](_0x19687f);}}else{if(_0x76a4cf[_0x2cd2('1062','xvh(')](_0x2060bd,0x0))this[_0x2cd2('fe2','8vsk')](_0x76a4cf[_0x2cd2('1063','*unX')],0x50,0x37,0x78,_0x76a4cf[_0x2cd2('1064','vbKI')]);if(_0x76a4cf[_0x2cd2('1065','vnIt')](_0x2060bd,0x1))this[_0x2cd2('680','1L8W')]('★★',0x50,0x5a,0x78,_0x76a4cf[_0x2cd2('1066',')%R)')]);if(_0x76a4cf[_0x2cd2('1067','g*Sm')](_0x2060bd,0x2))this[_0x2cd2('1047','HHLp')]('★',0x50,0x7d,0x78,_0x76a4cf[_0x2cd2('1068','4UjT')]);this[_0x2cd2('e94','#fZb')](_0x76a4cf[_0x2cd2('1069','P0Kn')](_0x2060bd,0x1),0x1e,_0x76a4cf[_0x2cd2('106a','GDPD')](_0x76a4cf[_0x2cd2('106b','8vsk')](_0x76a4cf[_0x2cd2('106c','YCbV')](_0x76a4cf[_0x2cd2('106d','[yfr')](_0x2060bd,_0x76a4cf[_0x2cd2('106e','qqEP')](_0x76a4cf[_0x2cd2('106f','4UjT')](JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('1070','dO1!')],0x1),_0x4b0f21)),0x1),0x23),0x14),0x78,_0x76a4cf[_0x2cd2('1071','#4SP')]);if(_0x76a4cf[_0x2cd2('1072','vbKI')](JHMW[_0x2cd2('380','CFh^')][_0x2cd2('1073','&)j!')][_0x76a4cf[_0x2cd2('1074','1L8W')](_0x2060bd,0x3)],JHMW[_0x2cd2('2d3','P0Kn')][_0x2cd2('3c9','ZboD')])){if(_0x76a4cf[_0x2cd2('1075','lDFd')](_0x76a4cf[_0x2cd2('1076','dO1!')],_0x76a4cf[_0x2cd2('1077','Biy2')])){return this[_0x2cd2('1078','vnIt')](text,0x0,this[_0x2cd2('1079','#fZb')][_0x2cd2('107a','ghrv')]);}else{if(_0x76a4cf[_0x2cd2('107b','2^*B')](Utils[_0x2cd2('107c','ghrv')],'MV'))this[_0x2cd2('107d','4[4f')](this[_0x2cd2('107e','CFh^')]());else this[_0x2cd2('107f','1L8W')](ColorManager[_0x2cd2('1080','ZboD')]());this[_0x2cd2('a15','qqEP')](JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('1a5','%E7b')][_0x76a4cf[_0x2cd2('1081','YCbV')](_0x2060bd,0x3)],0x82,_0x76a4cf[_0x2cd2('1082','#4SP')](_0x76a4cf[_0x2cd2('1083','jpvP')](_0x76a4cf[_0x2cd2('1084','3Y7y')](_0x76a4cf[_0x2cd2('1085','wHIg')](_0x2060bd,_0x76a4cf[_0x2cd2('1086','qqEP')](_0x76a4cf[_0x2cd2('1087','W@80')](JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('1088','vB83')],0x1),_0x4b0f21)),0x1),0x23),0x14),0xfa,_0x76a4cf[_0x2cd2('1089','R)LB')]);this[_0x2cd2('108a','GDPD')]();}}else this[_0x2cd2('108b','GDPD')](JHMW[_0x2cd2('fc','W@80')][_0x2cd2('1020','y046')][_0x76a4cf[_0x2cd2('108c','#4SP')](_0x2060bd,0x3)],0x82,_0x76a4cf[_0x2cd2('108d','#iIl')](_0x76a4cf[_0x2cd2('108e','qqEP')](_0x76a4cf[_0x2cd2('108f','gq!6')](_0x76a4cf[_0x2cd2('1090','%E7b')](_0x2060bd,_0x76a4cf[_0x2cd2('1091','CFh^')](_0x76a4cf[_0x2cd2('1092','X1Zu')](JHMW[_0x2cd2('d9','R)LB')][_0x2cd2('1093','l^iV')],0x1),_0x4b0f21)),0x1),0x23),0x14),0xfa,_0x76a4cf[_0x2cd2('1094','YCbV')]);this[_0x2cd2('1095','R)LB')](JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('1096','hS2h')][_0x76a4cf[_0x2cd2('1097','vnIt')](_0x76a4cf[_0x2cd2('1098','l^iV')](_0x2060bd,0x3),0x1)],0x172,_0x76a4cf[_0x2cd2('1099','2^*B')](_0x76a4cf[_0x2cd2('109a','hS2h')](_0x76a4cf[_0x2cd2('109b','g*Sm')](_0x76a4cf[_0x2cd2('109c','lDFd')](_0x2060bd,_0x76a4cf[_0x2cd2('109d','qqEP')](_0x76a4cf[_0x2cd2('109e','#fZb')](JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('109f','g*Sm')],0x1),_0x4b0f21)),0x1),0x23),0x14),0xe6,_0x76a4cf[_0x2cd2('10a0','#fZb')]);this[_0x2cd2('fd2','#4SP')](JHMW[_0x2cd2('15e','LX!4')][_0x2cd2('f7f','4UjT')][_0x76a4cf[_0x2cd2('10a1','xWmq')](_0x76a4cf[_0x2cd2('10a2','Biy2')](_0x2060bd,0x3),0x2)],0x244,_0x76a4cf[_0x2cd2('10a3','vbKI')](_0x76a4cf[_0x2cd2('10a4','X1Zu')](_0x76a4cf[_0x2cd2('10a5','CFh^')](_0x76a4cf[_0x2cd2('10a6','R)LB')](_0x2060bd,_0x76a4cf[_0x2cd2('10a7','vbKI')](_0x76a4cf[_0x2cd2('10a8','obzi')](JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('10a9','l%aI')],0x1),_0x4b0f21)),0x1),0x23),0x14),0x96,_0x76a4cf[_0x2cd2('10aa','X1Zu')]);}}this[_0x2cd2('10ab','y046')]=JHMW[_0x2cd2('162','y046')][_0x2cd2('10ac','3Y7y')];};Window_Ranking[_0x2cd2('7b7','CFh^')][_0x2cd2('40a','jpvP')]=function(){var _0x1f3be4={'kPLnY':function(_0x1d3998,_0x5d1fb2){return _0x1d3998!=_0x5d1fb2;}};Window_Base[_0x2cd2('85c','#4SP')][_0x2cd2('10ad','ZboD')][_0x2cd2('10ae','c!9y')](this);if(_0x1f3be4[_0x2cd2('10af','P0Kn')](this[_0x2cd2('10b0','CFh^')],JHMW[_0x2cd2('11b','xWmq')][_0x2cd2('10b1','wHIg')]))this[_0x2cd2('10b2','3psD')]();};Window_Action[_0x2cd2('cdd','xvh(')]=Object[_0x2cd2('10b3','R)LB')](Window_Command[_0x2cd2('7e5','^)]V')]);Window_Action[_0x2cd2('de0','#fZb')][_0x2cd2('10b4','g*Sm')]=Window_Action;Window_Action[_0x2cd2('b2e','NeH6')][_0x2cd2('10b5','[yfr')]=function(){var _0x48275a={'coCGt':function(_0x380854,_0x569a9b){return _0x380854==_0x569a9b;}};if(_0x48275a[_0x2cd2('10b6','4[4f')](Utils[_0x2cd2('10b7','#iIl')],'MV'))Window_Command[_0x2cd2('7c1','*unX')][_0x2cd2('10b8','W@80')][_0x2cd2('ecf','YCbV')](this,0x0,0x0);else{var _0x24e918=this[_0x2cd2('10b9','vB83')](0x3);var _0x50cff2={'x':0x0,'y':0x0,'width':0x258,'height':_0x24e918};Window_Command[_0x2cd2('10ba','&)j!')][_0x2cd2('10bb','l^iV')][_0x2cd2('d49','HHLp')](this,_0x50cff2);}this[_0x2cd2('c02','3psD')]();this[_0x2cd2('10bc','l^iV')]=0x0;};Window_Action[_0x2cd2('904','y046')][_0x2cd2('eca','xvh(')]=function(){var _0x1f85f7={'OlQau':_0x2cd2('10bd','hS2h'),'MAiTa':_0x2cd2('10be','^)]V')};this[_0x2cd2('10bf','qqEP')]('确定',_0x1f85f7[_0x2cd2('10c0','*unX')]);this[_0x2cd2('10c1','xvh(')]('返回',_0x1f85f7[_0x2cd2('10c2','Biy2')]);};Window_Action[_0x2cd2('6f7','qqEP')][_0x2cd2('10c3','HHLp')]=function(_0x319a09){var _0x1f8477={'DlmvD':function(_0x4423bf,_0x18f87e){return _0x4423bf==_0x18f87e;},'BFVeI':_0x2cd2('10c4','%E7b'),'SXsYO':function(_0x467c41,_0x490f20){return _0x467c41/_0x490f20;},'DSJkH':function(_0x53d448,_0x239ce4){return _0x53d448-_0x239ce4;},'xkvxs':function(_0x23dd8c,_0x5254f8){return _0x23dd8c+_0x5254f8;},'tYSmV':function(_0x220fde,_0x272c99){return _0x220fde*_0x272c99;},'ONxXq':function(_0xe11c43,_0x45a56a){return _0xe11c43+_0x45a56a;},'nAqWc':function(_0x305a5c,_0x33a1e2){return _0x305a5c*_0x33a1e2;},'zKWqw':function(_0x29245a,_0x3d1f9e){return _0x29245a-_0x3d1f9e;},'FLoHi':_0x2cd2('10c5','#fZb'),'qHGIE':function(_0xbcb8f4,_0x369398){return _0xbcb8f4/_0x369398;},'npzaW':function(_0xc85f8d,_0x4fb540){return _0xc85f8d-_0x4fb540;},'fWmII':function(_0x500f9e,_0xa39eb4){return _0x500f9e*_0xa39eb4;},'Vheen':_0x2cd2('10c6','hS2h')};this[_0x2cd2('b30','3Y7y')]=_0x319a09;if(_0x1f8477[_0x2cd2('10c7','wHIg')](Utils[_0x2cd2('10c8','4UjT')],'MV')){var _0x4c402c=_0x1f8477[_0x2cd2('10c9','vB83')][_0x2cd2('4ac','@rQL')]('|'),_0x2fd377=0x0;while(!![]){switch(_0x4c402c[_0x2fd377++]){case'0':this['y']=_0x1f8477[_0x2cd2('10ca','&)j!')](_0x1f8477[_0x2cd2('10cb','1L8W')](Graphics[_0x2cd2('10cc','#iIl')],this[_0x2cd2('931','3psD')]),0x2);continue;case'1':var _0x7577cb=_0x1f8477[_0x2cd2('10cd','#fZb')](this[_0x2cd2('10ce','#fZb')](this[_0x2cd2('10cf','qqEP')]),_0x1f8477[_0x2cd2('10d0','4UjT')](this[_0x2cd2('10d1','vnIt')](),0x2));continue;case'2':this[_0x2cd2('10d2','@rQL')]=_0x1f8477[_0x2cd2('10d3','*unX')](_0x7577cb,_0x1f8477[_0x2cd2('10d4','8vsk')](this[_0x2cd2('10d5','CFh^')](),0x2));continue;case'3':var _0x5956ec=this[_0x2cd2('10d6','R)LB')]();continue;case'4':this['x']=_0x1f8477[_0x2cd2('10d7','y046')](_0x1f8477[_0x2cd2('10d8','l%aI')](Graphics[_0x2cd2('10d9','xWmq')],this[_0x2cd2('b67','#4SP')]),0x2);continue;}break;}}else{var _0x12a7f3=_0x1f8477[_0x2cd2('10da','LX!4')][_0x2cd2('3b6','#iIl')]('|'),_0x4dd8ec=0x0;while(!![]){switch(_0x12a7f3[_0x4dd8ec++]){case'0':this['x']=_0x1f8477[_0x2cd2('10db','&)j!')](_0x1f8477[_0x2cd2('10dc','wHIg')]($dataSystem[_0x2cd2('10dd','CFh^')][_0x2cd2('10de','CFh^')],this[_0x2cd2('10df','4UjT')]),0x2);continue;case'1':var _0x7577cb=_0x1f8477[_0x2cd2('10e0','ZboD')](this[_0x2cd2('10e1','vbKI')](this[_0x2cd2('10e2','ZboD')]),_0x1f8477[_0x2cd2('10e3','y046')]($gameSystem[_0x2cd2('10e4','NeH6')](),0x2));continue;case'2':this['y']=_0x1f8477[_0x2cd2('10e5','3psD')](_0x1f8477[_0x2cd2('10e6','8vsk')]($dataSystem[_0x2cd2('10e7','#fZb')][_0x2cd2('10e8','HHLp')],this[_0x2cd2('10e9','&)j!')]),0x2);continue;case'3':this[_0x2cd2('52b','#iIl')]=_0x1f8477[_0x2cd2('10ea',')%R)')](_0x7577cb,_0x1f8477[_0x2cd2('10eb','LX!4')](this[_0x2cd2('10ec','%E7b')](),0x2));continue;case'4':var _0x5956ec=this[_0x2cd2('10ed','CFh^')]();continue;}break;}}this[_0x2cd2('86e','xvh(')]();this[_0x2cd2('10ee','obzi')](this[_0x2cd2('a8a','hS2h')],_0x5956ec,0x0,_0x1f8477[_0x2cd2('10ef','3Y7y')]);};Window_Action[_0x2cd2('8a9','1L8W')][_0x2cd2('10f0','3psD')]=function(_0x3e16ac){return this[_0x2cd2('10f1','R)LB')](_0x3e16ac,0x0,this[_0x2cd2('10f2','1L8W')][_0x2cd2('b7d','^)]V')]);};Window_Action[_0x2cd2('904','y046')][_0x2cd2('10f3','2^*B')]=function(){var _0x299a75={'TcIvF':_0x2cd2('10f4','#fZb')};return _0x299a75[_0x2cd2('10f5','8U%R')];};Window_Action[_0x2cd2('7ed','HHLp')][_0x2cd2('10f6','8vsk')]=function(){return this[_0x2cd2('10f7','g*Sm')](0x3);};Window_Action[_0x2cd2('dde','R)LB')][_0x2cd2('10f8','#iIl')]=function(_0xac8f3b){var _0x427c60=Window_Selectable[_0x2cd2('8ec','2^*B')][_0x2cd2('10f9','4UjT')][_0x2cd2('85e','NeH6')](this,_0xac8f3b);_0x427c60['y']+=this[_0x2cd2('100a','HHLp')]();return _0x427c60;};Window_TalkList[_0x2cd2('1011','l%aI')]=Object[_0x2cd2('10fa','hS2h')](Window_Base[_0x2cd2('6f7','qqEP')]);Window_TalkList[_0x2cd2('10fb','GDPD')][_0x2cd2('a82','LX!4')]=Window_TalkList;Window_TalkList[_0x2cd2('781','hS2h')][_0x2cd2('10fc','GDPD')]=function(){var _0x4fde76={'gqIIS':_0x2cd2('10fd','y046'),'lAEqw':_0x2cd2('4a9','NeH6'),'QHozb':function(_0x58610e,_0x499420){return _0x58610e+_0x499420;},'sXYoP':function(_0x34ce06,_0x5b3521){return _0x34ce06-_0x5b3521;},'pTDzR':function(_0x277c87,_0x2538db){return _0x277c87/_0x2538db;},'vayMJ':function(_0x17d624,_0x375571){return _0x17d624-_0x375571;},'zknMG':function(_0x146a66,_0x4d4691){return _0x146a66-_0x4d4691;},'HvOHs':function(_0x3a1677,_0x1b23f0){return _0x3a1677*_0x1b23f0;},'WXNai':function(_0x967422,_0x3a3cd5){return _0x967422(_0x3a3cd5);},'BEKVQ':function(_0x468dca,_0x39fce9){return _0x468dca==_0x39fce9;},'rocyb':function(_0x4a6d38,_0x3beba0){return _0x4a6d38!==_0x3beba0;},'rTDEI':_0x2cd2('10fe',')%R)'),'RcfcJ':_0x2cd2('10ff','xWmq'),'CEPFu':function(_0x574a56,_0x1542c1){return _0x574a56-_0x1542c1;},'dSdpb':function(_0x453d63,_0x33a5de){return _0x453d63(_0x33a5de);},'gtAPB':_0x2cd2('1100','gq!6'),'wMplq':_0x2cd2('1101','1L8W'),'ToAcZ':function(_0x3b722e,_0x1d4d2c){return _0x3b722e-_0x1d4d2c;}};var _0x18d349=this[_0x2cd2('1102','8U%R')]();var _0x1d17e8=this[_0x2cd2('1103','GDPD')]();var _0x5b482e=JHMW[_0x2cd2('de5','LX!4')][_0x2cd2('1104','l^iV')][_0x2cd2('3f3','dO1!')](',');var _0x3885ea=_0x4fde76[_0x2cd2('1105','qqEP')](parseInt,_0x5b482e[0x0]);if(_0x4fde76[_0x2cd2('1106','R)LB')](Utils[_0x2cd2('824','%E7b')],'MV')){if(_0x4fde76[_0x2cd2('1107','UUq0')](_0x4fde76[_0x2cd2('1108','LX!4')],_0x4fde76[_0x2cd2('1109','4[4f')])){var _0x5edef0=_0x4fde76[_0x2cd2('110a','xvh(')](parseInt,_0x5b482e[0x1]);Window_Base[_0x2cd2('110b','YCbV')][_0x2cd2('110c','%E7b')][_0x2cd2('d4f','lDFd')](this,_0x3885ea,_0x4fde76[_0x2cd2('110d','R)LB')](_0x5edef0,_0x1d17e8),_0x18d349,_0x1d17e8);}else{if(JHMW[_0x2cd2('1f9','[yfr')][_0x2cd2('fb5','1L8W')]&&!$gamePlayer[_0x2cd2('110e','[yfr')]()&&!SceneManager[_0x2cd2('110f','l%aI')]()){Graphics[_0x2cd2('1110','3psD')](_0x4fde76[_0x2cd2('1111','CFh^')],_0x4fde76[_0x2cd2('1112','LX!4')],_0x4fde76[_0x2cd2('1113','ZboD')](this['x'],0xa),_0x4fde76[_0x2cd2('1114','R)LB')](_0x4fde76[_0x2cd2('1115','1L8W')](this['y'],this[_0x2cd2('1116','HHLp')]()),0x23),_0x4fde76[_0x2cd2('1117','gq!6')](JHMW[_0x2cd2('11e','qqEP')][_0x2cd2('1118','ZboD')],0x19),0x12,0x10);Graphics[_0x2cd2('1119','2^*B')][_0x2cd2('111a','wHIg')]=Math[_0x2cd2('111b','hS2h')](_0x4fde76[_0x2cd2('111c','vbKI')](_0x4fde76[_0x2cd2('111d','LX!4')](_0x4fde76[_0x2cd2('111e','hS2h')](JHMW[_0x2cd2('9a','UUq0')][_0x2cd2('111f','vB83')],_0x4fde76[_0x2cd2('1120','R)LB')](JHMW[_0x2cd2('9a','UUq0')][_0x2cd2('a3e','NeH6')],0x2)),0x14),0xa));Graphics[_0x2cd2('fb7','vnIt')][_0x2cd2('1121','g*Sm')]='';}}}else{var _0x5edef0=_0x4fde76[_0x2cd2('1122','obzi')](_0x4fde76[_0x2cd2('1123','vB83')](parseInt,_0x5b482e[0x1]),_0x1d17e8);var _0x38889b={'x':_0x3885ea,'y':_0x5edef0,'width':_0x18d349,'height':_0x1d17e8};Window_Base[_0x2cd2('a80','P0Kn')][_0x2cd2('10fc','GDPD')][_0x2cd2('e92','8U%R')](this,_0x38889b);}this[_0x2cd2('1124','R)LB')]=JHMW[_0x2cd2('123','vB83')][_0x2cd2('1125','HHLp')];this[_0x2cd2('1126','8vsk')]();this[_0x2cd2('1127','UUq0')]();if(JHMW[_0x2cd2('de5','LX!4')][_0x2cd2('14c','R)LB')]){if(_0x4fde76[_0x2cd2('1128','qqEP')](_0x4fde76[_0x2cd2('1129','qqEP')],_0x4fde76[_0x2cd2('112a','3Y7y')])){Graphics[_0x2cd2('112b','*unX')](_0x4fde76[_0x2cd2('112c','@rQL')],_0x4fde76[_0x2cd2('1112','LX!4')],_0x4fde76[_0x2cd2('112d','LX!4')](_0x3885ea,0xa),_0x4fde76[_0x2cd2('112e','wHIg')](_0x5edef0,0x23),_0x4fde76[_0x2cd2('112f','hS2h')](JHMW[_0x2cd2('11e','qqEP')][_0x2cd2('1130','2^*B')],0x19),0x12,0x10);Graphics[_0x2cd2('4bb','ZboD')][_0x2cd2('1131','#iIl')]=Math[_0x2cd2('1132','dO1!')](_0x4fde76[_0x2cd2('1133','#iIl')](_0x4fde76[_0x2cd2('1134','UUq0')](_0x4fde76[_0x2cd2('1135','8U%R')](JHMW[_0x2cd2('195','gq!6')][_0x2cd2('1136','%E7b')],_0x4fde76[_0x2cd2('1137','dO1!')](JHMW[_0x2cd2('c80','Biy2')][_0x2cd2('1138','l^iV')],0x2)),0x14),0xa));Graphics[_0x2cd2('1139','^)]V')][_0x2cd2('113a','*unX')]='';}else{JHMW[_0x2cd2('eb','ghrv')][_0x2cd2('113b','hS2h')]();}}this[_0x2cd2('113c','8vsk')]=JHMW[_0x2cd2('d7','@rQL')][_0x2cd2('113d','obzi')];this[_0x2cd2('113e','W@80')]=$gameMessage[_0x2cd2('113f','LX!4')]();};Window_TalkList[_0x2cd2('77f','obzi')][_0x2cd2('1140','4[4f')]=function(){return 0xa;};Window_TalkList[_0x2cd2('84f','%E7b')][_0x2cd2('1141','R)LB')]=function(){return JHMW[_0x2cd2('12d','%E7b')][_0x2cd2('1142','jpvP')];};Window_TalkList[_0x2cd2('7ed','HHLp')][_0x2cd2('1143','@rQL')]=function(){switch(JHMW[_0x2cd2('56','@rQL')][_0x2cd2('1144','P0Kn')]){case 0x1:return 0xc8;break;case 0x2:return 0x12c;break;case 0x3:return 0x190;break;}};Window_TalkList[_0x2cd2('1011','l%aI')][_0x2cd2('86e','xvh(')]=function(){var _0x213fd1={'OYuQE':function(_0x52ad0a,_0x114459){return _0x52ad0a+_0x114459;},'UORFn':function(_0x1841c3,_0x509ec9){return _0x1841c3+_0x509ec9;},'bHEQt':function(_0x45af97,_0x3f2389){return _0x45af97+_0x3f2389;},'oNjNb':function(_0x3e9234,_0x3b2ac7){return _0x3e9234*_0x3b2ac7;},'RZwkv':function(_0x3be7e0,_0x303823){return _0x3be7e0+_0x303823;},'qnPWg':function(_0x292b7d,_0x53cab3){return _0x292b7d!==_0x53cab3;},'GrXDB':_0x2cd2('1145','*unX'),'MfSQG':_0x2cd2('1146','X1Zu')};if(JHMW[_0x2cd2('b42','NeH6')][_0x2cd2('1147','l^iV')]){this[_0x2cd2('1148',')%R)')][_0x2cd2('1149','jpvP')]=0xff;this[_0x2cd2('114a','@rQL')]=JHMW[_0x2cd2('fb4','4UjT')][_0x2cd2('114b','1L8W')];}else{if(_0x213fd1[_0x2cd2('114c','#iIl')](_0x213fd1[_0x2cd2('114d','4[4f')],_0x213fd1[_0x2cd2('114e','xWmq')])){var _0x406657=_0x213fd1[_0x2cd2('114f','vB83')](_0x213fd1[_0x2cd2('1150','W@80')](JHMW[_0x2cd2('165','NeH6')][_0x2cd2('299','LX!4')][_0x213fd1[_0x2cd2('1151','3Y7y')](_0x213fd1[_0x2cd2('1152','CFh^')](i,0xb),0x1)],'说：'),JHMW[_0x2cd2('165','NeH6')][_0x2cd2('876','vB83')][_0x213fd1[_0x2cd2('1153','dO1!')](_0x213fd1[_0x2cd2('1154','GDPD')](i,0xb),0xa)]);Notification[_0x2cd2('310','l^iV')](_0x406657);}else{Graphics[_0x2cd2('1a0','y046')](_0x213fd1[_0x2cd2('1155','%E7b')]);this[_0x2cd2('5e5','l^iV')][_0x2cd2('1156','P0Kn')]=0x0;this[_0x2cd2('1157','3Y7y')]=0x0;}}};Window_TalkList[_0x2cd2('7ed','HHLp')][_0x2cd2('1158','c!9y')]=function(_0x408795){var _0x3ebeae={'owpoF':function(_0xa49b1c,_0x408795){return _0xa49b1c==_0x408795;}};if(JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('1159','vnIt')]){this[_0x2cd2('115a','xWmq')][_0x2cd2('115b','YCbV')]=0x64;if(_0x3ebeae[_0x2cd2('115c','[yfr')](Utils[_0x2cd2('115d','jpvP')],'MV'))var _0xfa8cd8=this[_0x2cd2('115e',')%R)')]();else var _0xfa8cd8=ColorManager[_0x2cd2('115f','qqEP')]();this[_0x2cd2('f41','c!9y')][_0x2cd2('1160','^)]V')](0x0,_0x408795,this[_0x2cd2('1161','wHIg')](),0x2,_0xfa8cd8);this[_0x2cd2('1162','3Y7y')][_0x2cd2('1163','UUq0')]=0xff;}};Window_TalkList[_0x2cd2('84a','vB83')][_0x2cd2('1164','W@80')]=function(){return JHMW[_0x2cd2('c94','hS2h')][_0x2cd2('1165','hS2h')];};Window_TalkList[_0x2cd2('8a9','1L8W')][_0x2cd2('1166','8U%R')]=function(){var _0xbac1cf={'TFyrv':_0x2cd2('1167','xvh('),'HLarp':function(_0xac7247,_0x34a73d){return _0xac7247!==_0x34a73d;},'ooQVO':_0x2cd2('1168','*unX'),'riPTb':_0x2cd2('1169','8U%R'),'vzEOv':function(_0x38bfc6,_0xec0dd2){return _0x38bfc6+_0xec0dd2;},'iyjci':function(_0x37fc58,_0x2dc52e){return _0x37fc58-_0x2dc52e;},'XxibF':function(_0x56ad32,_0x58363d){return _0x56ad32-_0x58363d;},'fJkrR':function(_0xe9d602,_0x4f60c6){return _0xe9d602*_0x4f60c6;},'Ddnag':function(_0x5e09fe,_0x1ecbaa){return _0x5e09fe*_0x1ecbaa;},'qyiqG':function(_0x4c17fc,_0x365e3a){return _0x4c17fc==_0x365e3a;},'JUOuh':function(_0x58e2a5,_0x29bc6e){return _0x58e2a5+_0x29bc6e;},'cAhQC':_0x2cd2('f4a','xWmq'),'jCGYh':_0x2cd2('116a','^)]V'),'iyWRR':_0x2cd2('116b','jpvP'),'URGzG':function(_0x3b43fe,_0x3520ac){return _0x3b43fe>_0x3520ac;},'VCVgx':function(_0x125b4e,_0x5ba0d3){return _0x125b4e/_0x5ba0d3;},'OfcSc':function(_0x2ebaad,_0x14ea70){return _0x2ebaad<=_0x14ea70;},'txPGu':function(_0x4a64b1,_0x5841d1){return _0x4a64b1!==_0x5841d1;},'ZuDMV':_0x2cd2('116c','@rQL'),'cFXwD':function(_0x500fd8,_0x55c153){return _0x500fd8-_0x55c153;},'pDxJT':function(_0x3f090c,_0x4d279d){return _0x3f090c+_0x4d279d;},'uyNae':_0x2cd2('116d','4[4f'),'ZaEUO':_0x2cd2('116e','ZboD'),'XvIpq':function(_0x5163be,_0x432e55){return _0x5163be-_0x432e55;}};if(JHMW[_0x2cd2('dc','#4SP')][_0x2cd2('116f','ZboD')]){if(_0xbac1cf[_0x2cd2('1170','@rQL')](_0xbac1cf[_0x2cd2('1171','ZboD')],_0xbac1cf[_0x2cd2('1172','g*Sm')])){var _0x43bd40=JHMW[_0x2cd2('169','jpvP')][_0x2cd2('1173','qqEP')]?JHMW[_0x2cd2('247','YCbV')][_0x2cd2('1174','HHLp')]:JHMW[_0x2cd2('29','4UjT')][_0x2cd2('324','xWmq')];var _0x41f36a=_0xbac1cf[_0x2cd2('1175','YCbV')](this[_0x2cd2('1176','l%aI')](),this[_0x2cd2('1177','#iIl')]());var _0x943d70=_0xbac1cf[_0x2cd2('1178','#fZb')](_0xbac1cf[_0x2cd2('1179','vB83')](this[_0x2cd2('ee1','lDFd')],_0xbac1cf[_0x2cd2('117a','xvh(')](this[_0x2cd2('117b','^)]V')](),0x2)),_0xbac1cf[_0x2cd2('117c','8vsk')](this[_0x2cd2('117d','LX!4')](),0x3));if(_0xbac1cf[_0x2cd2('117e','l^iV')](Utils[_0x2cd2('dbd','2^*B')],'MZ'))this[_0x2cd2('117f','g*Sm')][_0x2cd2('1180','xWmq')]=this[_0x2cd2('67c','#iIl')]();this[_0x2cd2('1181','y046')](this[_0x2cd2('1182','obzi')]());this[_0x2cd2('1183','jpvP')](_0xbac1cf[_0x2cd2('1184','jpvP')](_0xbac1cf[_0x2cd2('1185','X1Zu')](_0xbac1cf[_0x2cd2('1186','vbKI')],_0x43bd40),_0xbac1cf[_0x2cd2('1187',')%R)')]),0x0,0x0,this[_0x2cd2('1188','4UjT')](),_0xbac1cf[_0x2cd2('1189','vB83')]);this[_0x2cd2('118a','y046')](_0x41f36a);this[_0x2cd2('118b','xWmq')]();if(_0xbac1cf[_0x2cd2('118c','W@80')](JHMW[_0x2cd2('125','lDFd')][_0x2cd2('681','&)j!')][_0x2cd2('118d','qqEP')],0x0)){var _0xf3ae3=Math[_0x2cd2('118e','wHIg')](_0xbac1cf[_0x2cd2('118f','xvh(')](_0xbac1cf[_0x2cd2('1190',')%R)')](_0x943d70,_0x41f36a),this[_0x2cd2('1191','obzi')]()));var _0x3cf43d=Math[_0x2cd2('1192','[yfr')](_0xf3ae3,JHMW[_0x2cd2('eb','ghrv')][_0x2cd2('1193','4[4f')][_0x2cd2('767','2^*B')]);for(var _0x333f14=0x1;_0xbac1cf[_0x2cd2('1194','hS2h')](_0x333f14,_0x3cf43d);++_0x333f14){if(_0xbac1cf[_0x2cd2('1195','y046')](_0xbac1cf[_0x2cd2('1196','4UjT')],_0xbac1cf[_0x2cd2('1197','&)j!')])){return _0xbac1cf[_0x2cd2('1198','8vsk')];}else{this[_0x2cd2('1199','obzi')](JHMW[_0x2cd2('61','HHLp')][_0x2cd2('19c','W@80')][_0xbac1cf[_0x2cd2('119a','xWmq')](JHMW[_0x2cd2('3cd','UUq0')][_0x2cd2('119b','R)LB')][_0x2cd2('7bc','CFh^')],_0x333f14)],0x0,_0xbac1cf[_0x2cd2('119c','^)]V')](0x4,_0xbac1cf[_0x2cd2('119d','obzi')](_0x333f14,this[_0x2cd2('119e','1L8W')]())),this[_0x2cd2('f93','LX!4')](),_0xbac1cf[_0x2cd2('119f','dO1!')]);}}}var _0x39079d=_0xbac1cf[_0x2cd2('11a0','c!9y')](JHMW[_0x2cd2('84','g*Sm')][_0x2cd2('11a1','3Y7y')][_0x2cd2('27d','wHIg')],0x1);this[_0x2cd2('11a2','4UjT')](_0x943d70);this[_0x2cd2('11a3','2^*B')](this[_0x2cd2('11a4','3psD')]());this[_0x2cd2('11a5','&)j!')](_0xbac1cf[_0x2cd2('11a6','vB83')](_0xbac1cf[_0x2cd2('11a7','vnIt')](_0xbac1cf[_0x2cd2('11a8','g*Sm')],_0x39079d),_0xbac1cf[_0x2cd2('11a9','LX!4')]),0x0,_0xbac1cf[_0x2cd2('11aa','Biy2')](_0x943d70,0x4),this[_0x2cd2('11ab','[yfr')](),_0xbac1cf[_0x2cd2('11ac','#4SP')]);}else{this[_0x2cd2('11ad','LX!4')]();JHMW[_0x2cd2('ad','3Y7y')][_0x2cd2('11ae','X1Zu')]=![];}}};Window_TalkList[_0x2cd2('747','UUq0')][_0x2cd2('11af','vB83')]=function(){var _0x1a5c45={'BctKT':_0x2cd2('11b0',')%R)'),'QwBqi':_0x2cd2('11b1','ZboD'),'FBLbw':function(_0x1098c8,_0x4e7adc){return _0x1098c8+_0x4e7adc;},'Ufned':function(_0x55d7f1,_0x9210ca){return _0x55d7f1+_0x9210ca;},'GFChU':_0x2cd2('11b2','hS2h'),'HTYgY':_0x2cd2('11b3','GDPD'),'unFbP':function(_0x50d575,_0x2de76b){return _0x50d575+_0x2de76b;},'XtXJa':function(_0x5abed4,_0xdc916a){return _0x5abed4==_0xdc916a;},'pdBOf':function(_0x453a41,_0x58c961){return _0x453a41-_0x58c961;},'xqmom':function(_0x5b3d7a,_0x216fe6){return _0x5b3d7a/_0x216fe6;},'qnDvi':function(_0x2c2714,_0x391020){return _0x2c2714-_0x391020;},'DVjRs':function(_0xbddd56,_0x1e3a0d){return _0xbddd56-_0x1e3a0d;},'EvONK':function(_0x18eb95,_0x8f8aac){return _0x18eb95-_0x8f8aac;},'nGlyh':function(_0x56c41a,_0x1d1304){return _0x56c41a/_0x1d1304;},'nYWvg':function(_0x4add42,_0x3ab559){return _0x4add42-_0x3ab559;},'blHzg':function(_0x1c8db1,_0x56a4c0){return _0x1c8db1!=_0x56a4c0;},'rdVTf':function(_0x3ca639,_0x5579){return _0x3ca639!==_0x5579;},'ONXLi':_0x2cd2('11b4','8U%R'),'GTPLs':_0x2cd2('11b5','hS2h'),'uLdsW':function(_0x141f84,_0x31c14f){return _0x141f84!==_0x31c14f;},'xwppV':_0x2cd2('11b6','HHLp'),'hIZBX':_0x2cd2('11b7','xWmq'),'LwoWd':function(_0x1c4596,_0x43f158){return _0x1c4596+_0x43f158;},'bJREt':function(_0x58e007,_0x5d791e){return _0x58e007-_0x5d791e;},'coALM':function(_0x1cc7c9,_0x434258){return _0x1cc7c9-_0x434258;},'SrDRH':function(_0x3a43e7,_0x1cf2e3){return _0x3a43e7*_0x1cf2e3;},'OgtwZ':function(_0x19b918,_0x1ad5eb){return _0x19b918-_0x1ad5eb;},'faTRZ':function(_0x55e1bd,_0x75d6ce){return _0x55e1bd-_0x75d6ce;},'DncJN':function(_0x41a411,_0x52bdc8){return _0x41a411*_0x52bdc8;}};Window_Base[_0x2cd2('7e5','^)]V')][_0x2cd2('11b8','&)j!')][_0x2cd2('11b9','ZboD')](this);if(JHMW[_0x2cd2('de','vbKI')][_0x2cd2('11ba','#fZb')]){this[_0x2cd2('2eb','8vsk')][_0x2cd2('11bb','GDPD')]();this[_0x2cd2('11bc','c!9y')]();this[_0x2cd2('11bd','*unX')]();JHMW[_0x2cd2('f4','xvh(')][_0x2cd2('11be','UUq0')]=![];}if(_0x1a5c45[_0x2cd2('11bf','1L8W')](this[_0x2cd2('11c0','ghrv')],JHMW[_0x2cd2('9a','UUq0')][_0x2cd2('11c1','dO1!')])){if(_0x1a5c45[_0x2cd2('11c2','vnIt')](_0x1a5c45[_0x2cd2('11c3',')%R)')],_0x1a5c45[_0x2cd2('11c4','[yfr')])){this[_0x2cd2('11c5','lDFd')][_0x2cd2('11c6','1L8W')]();this[_0x2cd2('11c7','@rQL')]();this[_0x2cd2('11c8',')%R)')]();if(JHMW[_0x2cd2('37d','3Y7y')][_0x2cd2('11c9','lDFd')]){if(_0x1a5c45[_0x2cd2('11ca','8U%R')](_0x1a5c45[_0x2cd2('11cb','^)]V')],_0x1a5c45[_0x2cd2('11cc','8vsk')])){var _0x23fd45=_0x1a5c45[_0x2cd2('11cd','8U%R')][_0x2cd2('227','qqEP')]('|'),_0x568d7c=0x0;while(!![]){switch(_0x23fd45[_0x568d7c++]){case'0':Graphics[_0x2cd2('11ce','vB83')][_0x2cd2('20a','#iIl')]='';continue;case'1':Graphics[_0x2cd2('11cf','obzi')][_0x2cd2('9f5','y046')]='';continue;case'2':JHMW[_0x2cd2('12a','&)j!')][_0x2cd2('11d0','R)LB')]=null;continue;case'3':JHMW[_0x2cd2('94','GDPD')][_0x2cd2('11d1','YCbV')]=null;continue;case'4':Graphics[_0x2cd2('11d2','HHLp')](_0x1a5c45[_0x2cd2('11d3',')%R)')],_0x1a5c45[_0x2cd2('11d4','#fZb')],_0x3c94d0,_0x1a5c45[_0x2cd2('11d5','3Y7y')](_0x1a5c45[_0x2cd2('11d6','qqEP')](JHMW[_0x2cd2('142','ZboD')][_0x2cd2('11d7','X1Zu')],_0x1fe982),0x82),0x96,0x19,0x18);continue;case'5':Graphics[_0x2cd2('d73','4UjT')](_0x1a5c45[_0x2cd2('11d8','@rQL')],_0x1a5c45[_0x2cd2('11d9','4UjT')],_0x3c94d0,_0x1a5c45[_0x2cd2('11da','wHIg')](_0x1a5c45[_0x2cd2('11db','xvh(')](JHMW[_0x2cd2('3db','8vsk')][_0x2cd2('d78','&)j!')],_0x1fe982),0x50),0x96,0x19,0x18);continue;case'6':Graphics[_0x2cd2('506','X1Zu')][_0x2cd2('11dc','ZboD')]=0xf;continue;case'7':Graphics[_0x2cd2('11dd','8vsk')][_0x2cd2('1131','#iIl')]=0xf;continue;case'8':if(_0x1a5c45[_0x2cd2('11de','g*Sm')](Utils[_0x2cd2('11df','gq!6')],'MV')){var _0x3c94d0=_0x1a5c45[_0x2cd2('11e0','g*Sm')](_0x1a5c45[_0x2cd2('11e1','R)LB')](Graphics[_0x2cd2('11e2','NeH6')],0x2),0x14);var _0x1fe982=0x0;}else{var _0x3c94d0=_0x1a5c45[_0x2cd2('11e3','1L8W')](_0x1a5c45[_0x2cd2('11e4','wHIg')]($dataSystem[_0x2cd2('c8c','xvh(')][_0x2cd2('11e5','y046')],0x2),0x14);var _0x1fe982=_0x1a5c45[_0x2cd2('11e6','hS2h')](_0x1a5c45[_0x2cd2('11e7','@rQL')]($dataSystem[_0x2cd2('e15','%E7b')][_0x2cd2('11e8','CFh^')],$dataSystem[_0x2cd2('11e9','3Y7y')][_0x2cd2('11ea','8U%R')]),0x2);}continue;}break;}}else{Graphics[_0x2cd2('11eb','y046')](_0x1a5c45[_0x2cd2('11ec','NeH6')],_0x1a5c45[_0x2cd2('11ed','HHLp')],_0x1a5c45[_0x2cd2('11ee','ghrv')](this['x'],0xa),_0x1a5c45[_0x2cd2('11ef','LX!4')](_0x1a5c45[_0x2cd2('11f0','#4SP')](this['y'],this[_0x2cd2('11f1','#iIl')]()),0x23),_0x1a5c45[_0x2cd2('11f2','^)]V')](JHMW[_0x2cd2('142','ZboD')][_0x2cd2('1142','jpvP')],0x19),0x12,0x10);Graphics[_0x2cd2('11f3','1L8W')][_0x2cd2('11f4',')%R)')]=Math[_0x2cd2('11f5','ZboD')](_0x1a5c45[_0x2cd2('11f6','P0Kn')](_0x1a5c45[_0x2cd2('11f7','xvh(')](_0x1a5c45[_0x2cd2('11f8','*unX')](JHMW[_0x2cd2('1f9','[yfr')][_0x2cd2('11f9',')%R)')],_0x1a5c45[_0x2cd2('11fa','3Y7y')](JHMW[_0x2cd2('313','l%aI')][_0x2cd2('11fb','lDFd')],0x2)),0x14),0xa));Graphics[_0x2cd2('11fc','[yfr')][_0x2cd2('11fd','l^iV')]='';}}this[_0x2cd2('11fe','xWmq')]=JHMW[_0x2cd2('f2','c!9y')][_0x2cd2('720','P0Kn')];}else{var _0x384a79=_0x1a5c45[_0x2cd2('11ff','hS2h')](Graphics[_0x2cd2('1200','4[4f')],0x48);var _0x260844=_0x1a5c45[_0x2cd2('1201','2^*B')](_0x1a5c45[_0x2cd2('1202','obzi')](Graphics[_0x2cd2('1203','#4SP')],0x320),0x2);}}if(_0x1a5c45[_0x2cd2('1204','vbKI')](this[_0x2cd2('1205','HHLp')],$gameMessage[_0x2cd2('1206','^)]V')]())){if($gameMessage[_0x2cd2('1207','l%aI')]())Graphics[_0x2cd2('1208','*unX')](_0x1a5c45[_0x2cd2('1209','l%aI')]);else{if(JHMW[_0x2cd2('3db','8vsk')][_0x2cd2('120a','2^*B')]&&!$gamePlayer[_0x2cd2('120b','hS2h')]()&&!SceneManager[_0x2cd2('120c',')%R)')]()){Graphics[_0x2cd2('120d','CFh^')](_0x1a5c45[_0x2cd2('120e','UUq0')],_0x1a5c45[_0x2cd2('120f','2^*B')],_0x1a5c45[_0x2cd2('1210','3psD')](this['x'],0xa),_0x1a5c45[_0x2cd2('1211','@rQL')](_0x1a5c45[_0x2cd2('1212','X1Zu')](this['y'],this[_0x2cd2('1213','dO1!')]()),0x23),_0x1a5c45[_0x2cd2('1214','wHIg')](JHMW[_0x2cd2('123','vB83')][_0x2cd2('935','1L8W')],0x19),0x12,0x10);Graphics[_0x2cd2('1215','X1Zu')][_0x2cd2('1216','3psD')]=Math[_0x2cd2('1217','X1Zu')](_0x1a5c45[_0x2cd2('1218','ghrv')](_0x1a5c45[_0x2cd2('1219','3Y7y')](_0x1a5c45[_0x2cd2('121a','vbKI')](JHMW[_0x2cd2('e1','*unX')][_0x2cd2('121b','y046')],_0x1a5c45[_0x2cd2('121c','y046')](JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('121d','&)j!')],0x2)),0x14),0xa));Graphics[_0x2cd2('b62','R)LB')][_0x2cd2('121e','xWmq')]='';}}this[_0x2cd2('121f','#fZb')]=$gameMessage[_0x2cd2('1220','wHIg')]();}};Window_RankCmd[_0x2cd2('10fb','GDPD')]=Object[_0x2cd2('ebf','&)j!')](Window_HorzCommand[_0x2cd2('84a','vB83')]);Window_RankCmd[_0x2cd2('8a9','1L8W')][_0x2cd2('1221','2^*B')]=Window_RankCmd;Window_RankCmd[_0x2cd2('7e5','^)]V')][_0x2cd2('1222','vbKI')]=function(_0x10ef6c,_0x44b434){var _0xb0c8f2={'fNAXg':function(_0x10ef6c,_0x44b434){return _0x10ef6c==_0x44b434;}};if(_0xb0c8f2[_0x2cd2('1223','LX!4')](Utils[_0x2cd2('1224','g*Sm')],'MV'))Window_HorzCommand[_0x2cd2('c06','vnIt')][_0x2cd2('1225','1L8W')][_0x2cd2('1226','^)]V')](this,_0x10ef6c,_0x44b434);else{var _0x183cc2=this[_0x2cd2('1227','GDPD')](0x1);var _0x3ecdd5={'x':_0x10ef6c,'y':_0x44b434,'width':0x320,'height':_0x183cc2};Window_HorzCommand[_0x2cd2('e68','lDFd')][_0x2cd2('1228','YCbV')][_0x2cd2('1229','dO1!')](this,_0x3ecdd5);}};Window_RankCmd[_0x2cd2('e68','lDFd')][_0x2cd2('122a','#4SP')]=function(){return 0x320;};Window_RankCmd[_0x2cd2('dde','R)LB')][_0x2cd2('122b','YCbV')]=function(){return 0x4;};Window_RankCmd[_0x2cd2('8a9','1L8W')][_0x2cd2('122c','hS2h')]=function(){var _0xae6277={'UfjUW':_0x2cd2('122d','#iIl'),'lGGvT':_0x2cd2('122e','P0Kn'),'TEvhv':_0x2cd2('122f','#fZb'),'lFeso':_0x2cd2('1230','2^*B'),'NhFsn':_0x2cd2('1231','LX!4'),'rafpO':_0x2cd2('1232','lDFd'),'cosAj':_0x2cd2('1233','UUq0')};this[_0x2cd2('cbf','W@80')]('首页',_0xae6277[_0x2cd2('1234','@rQL')]);this[_0x2cd2('e83','R)LB')](_0xae6277[_0x2cd2('1235','R)LB')],_0xae6277[_0x2cd2('1236','HHLp')]);this[_0x2cd2('68','hS2h')](_0xae6277[_0x2cd2('1237','hS2h')],_0xae6277[_0x2cd2('1238','l^iV')]);this[_0x2cd2('f16','*unX')](_0xae6277[_0x2cd2('1239','^)]V')],_0xae6277[_0x2cd2('123a','jpvP')]);};Sprite_Online[_0x2cd2('803','3psD')]=Utils[_0x2cd2('b84','^)]V')]=='MV'?Object[_0x2cd2('123b','gq!6')](Sprite_Base[_0x2cd2('10fb','GDPD')]):Object[_0x2cd2('dcb','obzi')](Sprite[_0x2cd2('a80','P0Kn')]);Sprite_Online[_0x2cd2('747','UUq0')][_0x2cd2('123c','vnIt')]=Sprite_Online;Sprite_Online[_0x2cd2('7b7','CFh^')][_0x2cd2('dcf','lDFd')]=function(){var _0xf999ba={'WBNal':_0x2cd2('123d','#iIl'),'SVzEk':_0x2cd2('123e','[yfr'),'BibOP':_0x2cd2('123f','P0Kn'),'EFFUj':_0x2cd2('1240','P0Kn'),'uFPej':_0x2cd2('1241','HHLp'),'GYnZe':_0x2cd2('1242','8U%R'),'HLHYH':function(_0x5d7f4c,_0x5c089e){return _0x5d7f4c<_0x5c089e;},'TEEVK':function(_0xb27bf1,_0xc1cd83){return _0xb27bf1===_0xc1cd83;},'HTiHq':_0x2cd2('1243','8vsk'),'wGbgO':_0x2cd2('1244','1L8W')};this[_0x2cd2('1245','P0Kn')]=new Sprite();this[_0x2cd2('1246','g*Sm')][_0x2cd2('1247','obzi')]=new Bitmap(0x12c,0x64);this[_0x2cd2('916','3psD')][_0x2cd2('1248','1L8W')][_0x2cd2('1249','vbKI')]=JHMW[_0x2cd2('123','vB83')][_0x2cd2('124a','vB83')];this[_0x2cd2('124b','NeH6')][_0x2cd2('124c','ZboD')][_0x2cd2('124d','*unX')]=JHMW[_0x2cd2('3db','8vsk')][_0x2cd2('124e','dO1!')];this[_0x2cd2('124f','4[4f')]();this[_0x2cd2('1250','#iIl')]=[];this[_0x2cd2('1251','xWmq')]=[];this[_0x2cd2('1252','#fZb')]=[];for(var _0x3e8589=0x0;_0xf999ba[_0x2cd2('1253','l%aI')](_0x3e8589,JHMW[_0x2cd2('1254','2^*B')][_0x2cd2('1255','3Y7y')]);_0x3e8589++){if(_0xf999ba[_0x2cd2('1256','*unX')](_0xf999ba[_0x2cd2('1257','xvh(')],_0xf999ba[_0x2cd2('1258','YCbV')])){this[_0x2cd2('1259','c!9y')](_0xf999ba[_0x2cd2('125a','vnIt')],0x50,0x41,this[_0x2cd2('1188','4UjT')](),_0xf999ba[_0x2cd2('125b','UUq0')]);this[_0x2cd2('f91','UUq0')](_0xf999ba[_0x2cd2('125c','ghrv')],0x50,0x6e,this[_0x2cd2('f40','HHLp')](),_0xf999ba[_0x2cd2('125d','CFh^')]);this[_0x2cd2('125e','l^iV')](_0xf999ba[_0x2cd2('125f','xWmq')],0x0,0xa0,this[_0x2cd2('1260','dO1!')](),_0xf999ba[_0x2cd2('1261','4[4f')]);this[_0x2cd2('1047','HHLp')](_0xf999ba[_0x2cd2('1262','HHLp')],0x0,0xc8,this[_0x2cd2('fa6','#4SP')](),_0xf999ba[_0x2cd2('1263','jpvP')]);}else{this[_0x2cd2('1264','gq!6')][_0x3e8589]=new Sprite();this[_0x2cd2('1264','gq!6')][_0x3e8589][_0x2cd2('1265','dO1!')]=new Bitmap(0x12c,0x64);this[_0x2cd2('1266','HHLp')][_0x3e8589][_0x2cd2('1267','l%aI')][_0x2cd2('1268','UUq0')]=JHMW[_0x2cd2('ff','CFh^')][_0x2cd2('1269','GDPD')];this[_0x2cd2('126a','#fZb')](_0x3e8589);}}this[_0x2cd2('126b','xWmq')]=JHMW[_0x2cd2('16e','ZboD')][_0x2cd2('126c','vB83')];};Sprite_Online[_0x2cd2('7c1','*unX')][_0x2cd2('126d','ghrv')]=function(){var _0x33bf10={'TwcpF':_0x2cd2('68d','NeH6'),'LYbQC':_0x2cd2('126e','^)]V'),'vliEC':function(_0x438a7d,_0x4d2c75){return _0x438a7d+_0x4d2c75;},'vdQsg':function(_0x362284,_0x7a7189){return _0x362284-_0x7a7189;},'OoNFB':function(_0x4dfddf,_0x386ad4){return _0x4dfddf+_0x386ad4;},'IxGyu':function(_0x1eb31a,_0x1bc70d){return _0x1eb31a/_0x1bc70d;},'stPpM':function(_0xbf8098,_0x3ee139){return _0xbf8098-_0x3ee139;},'wqsUE':function(_0x555b96,_0x20be48){return _0x555b96-_0x20be48;},'zDPdT':function(_0x523c10,_0x236657){return _0x523c10*_0x236657;},'wEdPu':function(_0x579867,_0x58e5fc){return _0x579867>_0x58e5fc;},'mfIwR':function(_0x1a0c74,_0x219e44){return _0x1a0c74!==_0x219e44;},'beLMG':_0x2cd2('126f','R)LB'),'OFYYZ':_0x2cd2('1270','vbKI'),'UZYkr':_0x2cd2('1271','[yfr'),'idabX':function(_0x25ad2c,_0x56032e){return _0x25ad2c+_0x56032e;},'QEtyS':function(_0x4b7c45,_0x34e2a4){return _0x4b7c45-_0x34e2a4;}};var _0x381ea0=$gameParty[_0x2cd2('1272','HHLp')]()?JHMW[_0x2cd2('1cc','#fZb')][_0x2cd2('1273','wHIg')]:'';var _0x1274d2=$gameParty[_0x2cd2('1274','&)j!')]()?$gameVariables[_0x2cd2('1275','3psD')](JHMW[_0x2cd2('c94','hS2h')][_0x2cd2('1276','%E7b')]):'';this[_0x2cd2('1277','vB83')][_0x2cd2('1278','#fZb')]=this[_0x2cd2('1245','P0Kn')][_0x2cd2('1279','&)j!')][_0x2cd2('127a','vnIt')](_0x381ea0);this[_0x2cd2('127b','y046')][_0x2cd2('127c','W@80')]=this[_0x2cd2('127d','LX!4')][_0x2cd2('127e','g*Sm')][_0x2cd2('127f','jpvP')](_0x1274d2);if(JHMW[_0x2cd2('ab','^)]V')][_0x2cd2('1280','ZboD')]&&!$dataMap[_0x2cd2('1281','qqEP')][_0x2cd2('87b','@rQL')]&&_0x33bf10[_0x2cd2('1282','GDPD')](JHMW[_0x2cd2('1f9','[yfr')][_0x2cd2('1283','8vsk')],0x0)){if(_0x33bf10[_0x2cd2('1284','y046')](_0x33bf10[_0x2cd2('1285','^)]V')],_0x33bf10[_0x2cd2('1286','R)LB')])){this[_0x2cd2('1287','X1Zu')][_0x2cd2('1288','Biy2')][_0x2cd2('f8c','^)]V')](_0x381ea0,0x30,_0x33bf10[_0x2cd2('1289','vB83')](0x5,JHMW[_0x2cd2('1254','2^*B')][_0x2cd2('128a','^)]V')]),this[_0x2cd2('1246','g*Sm')][_0x2cd2('128b','P0Kn')],0x18,_0x33bf10[_0x2cd2('128c','W@80')]);if($gameVariables[_0x2cd2('128d','l%aI')](JHMW[_0x2cd2('3db','8vsk')][_0x2cd2('128e','8U%R')]))this[_0x2cd2('128f','wHIg')][_0x2cd2('1290','CFh^')][_0x2cd2('fa4','Biy2')](_0x1274d2,_0x33bf10[_0x2cd2('1291','4UjT')](_0x33bf10[_0x2cd2('1292','8vsk')](_0x33bf10[_0x2cd2('1293','P0Kn')](this[_0x2cd2('1294','&)j!')][_0x2cd2('1295','HHLp')],this[_0x2cd2('1296','jpvP')][_0x2cd2('1297','ghrv')]),0x2),0x30),0x3,this[_0x2cd2('1298','gq!6')][_0x2cd2('127c','W@80')],0x18,_0x33bf10[_0x2cd2('1299','c!9y')]);}else{Graphics[_0x2cd2('129a','^)]V')](_0x33bf10[_0x2cd2('129b','4UjT')],_0x33bf10[_0x2cd2('129c','@rQL')],_0x33bf10[_0x2cd2('129d','4[4f')](this['x'],0xa),_0x33bf10[_0x2cd2('129e','4UjT')](_0x33bf10[_0x2cd2('129f','#4SP')](this['y'],this[_0x2cd2('12a0','2^*B')]()),0x23),_0x33bf10[_0x2cd2('12a1','jpvP')](JHMW[_0x2cd2('410','W@80')][_0x2cd2('12a2','8U%R')],0x19),0x12,0x10);Graphics[_0x2cd2('12a3','l%aI')][_0x2cd2('12a4','l^iV')]=Math[_0x2cd2('12a5','#fZb')](_0x33bf10[_0x2cd2('12a6','hS2h')](_0x33bf10[_0x2cd2('12a7','CFh^')](_0x33bf10[_0x2cd2('12a8','#4SP')](JHMW[_0x2cd2('940','HHLp')][_0x2cd2('12a9','3psD')],_0x33bf10[_0x2cd2('12aa','8U%R')](JHMW[_0x2cd2('461','4[4f')][_0x2cd2('12ab','g*Sm')],0x2)),0x14),0xa));Graphics[_0x2cd2('1139','^)]V')][_0x2cd2('c10','CFh^')]='';}}this[_0x2cd2('12ac','[yfr')]=_0x381ea0;this[_0x2cd2('12ad','P0Kn')]=_0x1274d2;};Sprite_Online[_0x2cd2('b2e','NeH6')][_0x2cd2('12ae','@rQL')]=function(_0x98b03d){var _0x22c668={'XssRx':_0x2cd2('12af','dO1!'),'DHTqr':function(_0x55f5d7,_0x3eb29f){return _0x55f5d7+_0x3eb29f;},'WlUUW':_0x2cd2('12b0','Biy2'),'ZiQsD':function(_0xcf6d6a,_0x8d4220){return _0xcf6d6a+_0x8d4220;},'QGneA':function(_0x311074,_0x43fb44){return _0x311074/_0x43fb44;},'twIHy':function(_0x5dab6a,_0x3d71aa){return _0x5dab6a-_0x3d71aa;}};var _0x25ad78=_0x22c668[_0x2cd2('12b1','l^iV')][_0x2cd2('b8','HHLp')]('|'),_0x43bbb5=0x0;while(!![]){switch(_0x25ad78[_0x43bbb5++]){case'0':this[_0x2cd2('12b2','ZboD')][_0x98b03d][_0x2cd2('8ae','y046')]=$gameMap[_0x2cd2('12b3','@rQL')]()[_0x98b03d][_0x2cd2('8af','hS2h')];continue;case'1':this[_0x2cd2('12b4','vbKI')][_0x98b03d]=this[_0x2cd2('12b5','W@80')][_0x98b03d][_0x2cd2('12b6','vB83')];continue;case'2':this[_0x2cd2('12b7',')%R)')][_0x98b03d][_0x2cd2('12b8','obzi')]=$gameMap[_0x2cd2('12b9','l^iV')]()[_0x98b03d][_0x2cd2('12ba','ZboD')];continue;case'3':if(JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('12bb','HHLp')]&&!$dataMap[_0x2cd2('12bc','Biy2')][_0x2cd2('12bd','[yfr')]){this[_0x2cd2('12be','Biy2')][_0x98b03d][_0x2cd2('12bf','3psD')][_0x2cd2('12c0','X1Zu')](this[_0x2cd2('12c1','3psD')][_0x98b03d][_0x2cd2('7a3','&)j!')],0x30,_0x22c668[_0x2cd2('12c2','^)]V')](0x5,JHMW[_0x2cd2('ee','&)j!')][_0x2cd2('12c3','#iIl')]),this[_0x2cd2('12c4','8vsk')][_0x98b03d][_0x2cd2('12c5','vbKI')],0x18,_0x22c668[_0x2cd2('12c6','l^iV')]);if(this[_0x2cd2('12c7','1L8W')][_0x98b03d][_0x2cd2('12b8','obzi')])this[_0x2cd2('12b5','W@80')][_0x98b03d][_0x2cd2('12c8','ghrv')][_0x2cd2('f8c','^)]V')](this[_0x2cd2('12c9','2^*B')][_0x98b03d][_0x2cd2('12ca','xvh(')],_0x22c668[_0x2cd2('12cb','dO1!')](_0x22c668[_0x2cd2('12cc','GDPD')](_0x22c668[_0x2cd2('12cd','lDFd')](this[_0x2cd2('913','#4SP')][_0x98b03d][_0x2cd2('12ce','l%aI')],this[_0x2cd2('12c1','3psD')][_0x98b03d][_0x2cd2('12cf','YCbV')]),0x2),0x30),0x3,this[_0x2cd2('12d0','4[4f')][_0x98b03d][_0x2cd2('12d1','vbKI')],0x18,_0x22c668[_0x2cd2('12d2','lDFd')]);}continue;case'4':this[_0x2cd2('12b7',')%R)')][_0x98b03d][_0x2cd2('12d3','HHLp')]=this[_0x2cd2('761','l^iV')][_0x98b03d][_0x2cd2('1265','dO1!')][_0x2cd2('12d4','8vsk')](this[_0x2cd2('12d5','UUq0')][_0x98b03d][_0x2cd2('12b8','obzi')]);continue;case'5':this[_0x2cd2('12d6','GDPD')][_0x98b03d][_0x2cd2('12d7','P0Kn')]=this[_0x2cd2('12d8','NeH6')][_0x98b03d][_0x2cd2('12d9','c!9y')][_0x2cd2('12da','hS2h')](this[_0x2cd2('12db','xWmq')][_0x98b03d][_0x2cd2('65b','W@80')]);continue;case'6':this[_0x2cd2('12dc','g*Sm')][_0x98b03d]=this[_0x2cd2('12dd','X1Zu')][_0x98b03d][_0x2cd2('12b8','obzi')];continue;}break;}};Sprite_Online[_0x2cd2('7ed','HHLp')][_0x2cd2('12de','^)]V')]=function(){var _0x3090ef={'hybxv':_0x2cd2('12df',')%R)'),'aJkVm':function(_0x1e49ab,_0x47382e){return _0x1e49ab!=_0x47382e;},'WfpmL':function(_0x488508,_0x33eda1){return _0x488508-_0x33eda1;},'fzBdV':function(_0x48583b,_0x113a36){return _0x48583b*_0x113a36;},'cqVXq':function(_0x2cd4e3,_0x258443){return _0x2cd4e3<_0x258443;},'dfdGK':_0x2cd2('12e0','YCbV'),'NjMEs':function(_0x4461b3,_0x1be645){return _0x4461b3-_0x1be645;},'kGvGf':function(_0x2986c2,_0x22b731){return _0x2986c2-_0x22b731;},'DImFG':function(_0xc29b96,_0x3ad079){return _0xc29b96/_0x3ad079;},'GThah':function(_0x39c9d9,_0x4e7677){return _0x39c9d9!=_0x4e7677;},'nbqBM':function(_0x1638bd,_0x57d5e6){return _0x1638bd!=_0x57d5e6;},'XQADL':function(_0x2f551b,_0x3f6eac){return _0x2f551b-_0x3f6eac;}};var _0x3b98df=_0x3090ef[_0x2cd2('12e1','2^*B')][_0x2cd2('12e2','8vsk')]('|'),_0x1db00d=0x0;while(!![]){switch(_0x3b98df[_0x1db00d++]){case'0':if($gameParty[_0x2cd2('12e3','xvh(')]()&&(_0x3090ef[_0x2cd2('12e4','[yfr')](this[_0x2cd2('12e5','gq!6')],JHMW[_0x2cd2('b4','#4SP')][_0x2cd2('12e6','hS2h')])||JHMW[_0x2cd2('97','obzi')][_0x2cd2('128e','8U%R')]&&_0x3090ef[_0x2cd2('12e7','l^iV')](this[_0x2cd2('12e8','c!9y')],$gameVariables[_0x2cd2('1121','g*Sm')](JHMW[_0x2cd2('11e','qqEP')][_0x2cd2('12e9','y046')])))){this[_0x2cd2('12ea','l%aI')][_0x2cd2('12eb','@rQL')][_0x2cd2('12ec','8vsk')]();this[_0x2cd2('12ed','dO1!')]();}continue;case'1':if(_0x3090ef[_0x2cd2('12ee','jpvP')](this[_0x2cd2('12ef','3psD')],JHMW[_0x2cd2('10f','#iIl')][_0x2cd2('12f0','GDPD')])){if(JHMW[_0x2cd2('1aa','gq!6')][_0x2cd2('12f1','NeH6')])this[_0x2cd2('12f2','GDPD')]();else this[_0x2cd2('12f3','dO1!')][_0x2cd2('1248','1L8W')][_0x2cd2('12f4','g*Sm')]();this[_0x2cd2('12f5','*unX')]=JHMW[_0x2cd2('d4','qqEP')][_0x2cd2('12f1','NeH6')];}continue;case'2':this[_0x2cd2('12f6','c!9y')]['y']=_0x3090ef[_0x2cd2('12f7','&)j!')](_0x3090ef[_0x2cd2('12f8','W@80')](_0x3090ef[_0x2cd2('12f9','xvh(')]($gamePlayer[_0x2cd2('12fa','%E7b')](),$gameScreen[_0x2cd2('12fb','&)j!')]()),$gameMap[_0x2cd2('12fc','vnIt')]()),JHMW[_0x2cd2('410','W@80')][_0x2cd2('12fd','gq!6')]);continue;case'3':for(var _0x252439=0x0;_0x3090ef[_0x2cd2('12fe','3Y7y')](_0x252439,JHMW[_0x2cd2('c94','hS2h')][_0x2cd2('12ff','YCbV')]);_0x252439++){var _0x472de3=_0x3090ef[_0x2cd2('1300','&)j!')][_0x2cd2('227','qqEP')]('|'),_0x5b1191=0x0;while(!![]){switch(_0x472de3[_0x5b1191++]){case'0':this[_0x2cd2('12d0','4[4f')][_0x252439]['x']=_0x3090ef[_0x2cd2('1301','@rQL')](_0x3090ef[_0x2cd2('1302','vbKI')](_0x1fc630,$gameMap[_0x2cd2('1303','#iIl')]()),_0x3090ef[_0x2cd2('1304','xWmq')](this[_0x2cd2('1305','xWmq')][_0x2cd2('1306','g*Sm')],0x2));continue;case'1':var _0x1fc630=_0x3090ef[_0x2cd2('1307','HHLp')]($gameMap[_0x2cd2('1308','4UjT')]()[_0x252439][_0x2cd2('1309','W@80')](),$gameScreen[_0x2cd2('12fb','&)j!')]());continue;case'2':if(_0x3090ef[_0x2cd2('130a','2^*B')](this[_0x2cd2('130b','Biy2')][_0x252439],$gameMap[_0x2cd2('130c','vbKI')]()[_0x252439][_0x2cd2('7fb','dO1!')])||_0x3090ef[_0x2cd2('130d','GDPD')](this[_0x2cd2('130e','2^*B')][_0x252439],$gameMap[_0x2cd2('130f','8U%R')]()[_0x252439][_0x2cd2('1310','LX!4')])){this[_0x2cd2('1311','c!9y')][_0x252439][_0x2cd2('1290','CFh^')][_0x2cd2('1312','W@80')]();this[_0x2cd2('1313','jpvP')](_0x252439);}continue;case'3':this[_0x2cd2('1314','lDFd')][_0x252439]['y']=_0x3090ef[_0x2cd2('1315','4UjT')](_0x3090ef[_0x2cd2('1316','#iIl')](_0xc5e3a1,$gameMap[_0x2cd2('1317','l%aI')]()),JHMW[_0x2cd2('177','3psD')][_0x2cd2('12fd','gq!6')]);continue;case'4':var _0xc5e3a1=_0x3090ef[_0x2cd2('1318','hS2h')]($gameMap[_0x2cd2('1319','3psD')]()[_0x252439][_0x2cd2('131a','@rQL')](),$gameScreen[_0x2cd2('131b','W@80')]());continue;}break;}}continue;case'4':if(!$gameParty[_0x2cd2('131c','*unX')]())this[_0x2cd2('131d','*unX')][_0x2cd2('131e','%E7b')][_0x2cd2('131f','l^iV')]();continue;case'5':this[_0x2cd2('1320','R)LB')]['x']=_0x3090ef[_0x2cd2('1321',')%R)')](_0x3090ef[_0x2cd2('1322','W@80')](_0x3090ef[_0x2cd2('1323','1L8W')]($gamePlayer[_0x2cd2('1324','hS2h')](),$gameScreen[_0x2cd2('1325','*unX')]()),$gameMap[_0x2cd2('1326','jpvP')]()),_0x3090ef[_0x2cd2('1327','P0Kn')](this[_0x2cd2('127b','y046')][_0x2cd2('1328','R)LB')],0x2));continue;}break;}};;_0xodZ='jsjiami.com.v6';